angular.module('interactions-module', ['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('interactionsCtrl', ['$scope', '$rootScope', '$http', 'ngDialog', '$window', '$timeout', 'uiGridConstants', 'gettextCatalog',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, uiGridConstants, gettextCatalog) {

        var selectedCredentialID = 0;
        var errorDialog = false;

        // Used to unbind the watched event on dialog close.
        var unbindHandler = null;

        $scope.initCtrl = function(dlg){
            switch (dlg.name){
                case 'add-edit-config-dialog':
                    processDialog(PHD, window);
                    break;
                default:
                    // No action.
                    break;
            }
        };

        function processDialog(PHD, window) {
            var $form = $("form[name=AddConfigurationForm]");
            $scope.$form = $form;

            $scope.config = {};
            $scope.credentials = {};

            var credentials = [];
            var credentialsSelect;
            var $credentialsSelect = $("#credential_id");

            $scope.credentialNamePattern = /^[ A-Za-z0-9_@./#&+-/!$%^*()=\\|:;'"<>,?`~{}\[\]]{0,64}$/;
            $scope.credentialNameErrorString = gettextCatalog.getString("Credential Name: Max 64 characters");

            var configForm = PHD.FormController($form, {
                serialize: function ($form) {
                    return $form.find("input").serialize();
                }
            });

            if ($scope.isEditConfig) {
                $scope.dialogTitle = gettextCatalog.getString("Edit ");
                $scope.config.url = $scope.selectedTool.config.url;
                $scope.config.company = $scope.selectedTool.config.company_id;
                $scope.selectedCredential = $scope.selectedTool.config.credentials_id;
            } else {
                $scope.dialogTitle = gettextCatalog.getString("Add ");
            }

            $.validator.addMethod("isPSAUrl", function(value, element) {
                var url = $("#id_url").val();

                if (url != '' && url != undefined && url.includes(".")) {
                    return true;
                } else {
                    return false;
                }
            }, "Invalid URL");

            configForm
                .validate({
                    rules: {
                        url: {
                            required: true,
                            isPSAUrl: true
                        },
                        company: {
                            required: true
                        },
                        display_name: {
                            required: true,
                            pattern: $scope.credentialNamePattern
                        },
                        user_name: {
                            required: true
                        }
                    },
                    messages: {
                        url: {
                            required: gettextCatalog.getString("URL is required"),
                            isPSAUrl: gettextCatalog.getString("Invalid URL")
                        },
                        company: {
                            required: gettextCatalog.getString("Company ID is required")
                        },
                        display_name: {
                            required: gettextCatalog.getString("Credential name is required"),
                            pattern: gettextCatalog.getString($scope.credentialNameErrorString)
                        },
                        user_name: {
                            required: gettextCatalog.getString("Username is required")
                        }
                    }
                });

            configForm.submit = function() {
                $scope.saveConfiguration();
            };

            function populateCredentials() {
                credentialsSelect = $credentialsSelect[0];
                if (credentialsSelect != undefined) {
                    credentialsSelect.options.length = 0;
                    var i = 0;
                    credentialsSelect.options[i] = new Option('New', 0);
                    if (credentials.length > 0) {
                        credentials.forEach(function (credential, index) {
                            if (credential.display !== null && credential.display !== undefined && credential.display.length > 0) {
                                credentialsSelect.options[index + (i + 1)] = new Option(credential.display, credential.credential_id);
                            }
                        })
                    }
                    // in case of edit, select the already associated credentials
                    if ($scope.isEditConfig && $scope.selectedTool.config.credentials_id != null) {
                        $credentialsSelect.val($scope.selectedTool.config.credentials_id);
                        selectedCredentialID = $credentialsSelect.val();
                        $(".modal-form-credential").hide();
                    }
                }
            }

            function showCredentialsFormWithBlankValues() {
                $(".modal-form-credential").show();
                $credentialsSelect.val(0);
                $("#id_credentials_displayName").val("");
                $("#id_credentials_username").val("");
                $("#id_credentials_password").val("");
                $("#id_domain").val("");
            }

            $credentialsSelect.on("change", function (event) {
                selectedCredentialID = $credentialsSelect.val();
                if (selectedCredentialID == 0) {
                    showCredentialsFormWithBlankValues();
                } else {
                    $(".modal-form-credential").hide();
                }
            });

            function initForm() {
                $form.find(".definition").popover({
                    appendTo: $form
                });

                PHD.Ajax
                    .get('/api/credentials/?sid=' + $scope.systemID)
                    .done(function (data) {
                    credentials = data.data;
                    credentials.forEach(function (credential, index) {
                        if(credential.display_name != undefined){
                            var display = credential.display_name;
                            if (credential.is_default) {
                                display += " (Default)";
                            }
                            credential.display = display;
                        }
                        else{
                            credentials.splice(credentials.indexOf(credential),1);
                        }
                    });
                    populateCredentials();
                });
            }
            initForm();

            $scope.saveConfiguration = function() {
                var base_url = "/api/psa/config/";
                var url = "";
                var obj = {};

                if ((!$scope.$form.valid())) {
                    return false;
                }
                obj.psa_tool_id = $scope.selectedToolID;
                obj.url = $("#id_url").val();
                obj.company_id = $("#id_company").val();
                if (noOfPSATools == 1) {
                    obj.is_default_tool = true;
                }

                if (selectedCredentialID == 0) {
                    var credentials = {};
                    credentials.display_name = $("#id_credentials_displayName").val();
                    credentials.username = $("#id_credentials_username").val();
                    credentials.password = $("#id_credentials_password").val();
                    credentials.domain = $("#id_domain").val();
                    credentials.is_default = $("#id_default").prop("checked");
                    obj.credentials = credentials;
                }
                else {
                    obj.credential_id = parseInt(selectedCredentialID);
                }
                obj = JSON.stringify(obj);
                savePSAConfigIndicator = PHD.showLoadingIndicator("body", true, "Saving...");

                if ($scope.isEditConfig) {
                    // pass in config ID in case of edit
                    url = base_url + $scope.selectedTool.config.config_id + '/?sid=' + $scope.systemID;
                    resp = PHD.Ajax.put(url, obj, null, handleError);
                } else {
                    url = base_url + "?sid=" + $scope.systemID;
                    resp = PHD.Ajax.post(url, obj, null, handleError);
                }

                resp.done(function(data) {
                    PHD.hideLoadingIndicator(savePSAConfigIndicator);
                    $scope.closeConfigurationDialog();
                    var result = data.result;
                    if (Array.isArray(result)) {
                        result = result[0];
                    }
                    if (result !== undefined) {
                        if (parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                            ngDialog.close("Info-Error-Dialog");
                            $(document).trigger("interactionsave", [$scope.selectedSystemID, data]);
                        } else if (parseInt(result.code) === AJAX_RESULT_ERROR) {
                            ngDialog.open({
                                dialogType: 'ERROR',
                                modelDialogId: 'Info-Error-Dialog',
                                dialogMessage: data,
                                overlay: true
                            });
                        } else {
                            $(document).trigger("interactionsave", [$scope.selectedSystemID, data]);
                            if (data.message !== undefined && data.message !== "") {
                                ngDialog.open({
                                    modelDialogId: 'Node-Info-Dialog',
                                    dialogType: 'Information',
                                    dialogMessage: data.message
                                })
                            }
                        }
                    }
                });
            };
        }

        var ticketsTemplate = '<div ng-repeat="(colRenderIndex, col) in colContainer.renderedColumns track by col.colDef.name" ' +
            'class="ui-grid-cell" ng-class="{ \'ui-grid-row-header-cell\': col.isRowHeader }" ui-grid-cell></div></div>';


        $scope.ticketsGridData = {
            multiSelect: false,
            selectionRowHeaderWidth: 35,
            rowHeight: 150,
            enableFullRowSelection : true,
            gridMenuShowHideColumns:false,
            enableColumnMenus:false,
            enableHorizontalScrollbar: 0,
            rowTemplate: ticketsTemplate,
            data: $scope.gridData
        };

        $scope.ticketsGridData.columnDefs = [
            {
                field: "id",
                displayName: "ID",
                sortable : true,
                type : "number",
                width: 75
            },
            {
                field: "time_sent",
                displayName: "Date",
                type : "date",
                width: 150,
                sort: {
                    direction: uiGridConstants.DESC,
                    priority: 0
                }
            },
            {
                field: "severity",
                displayName: "Severity",
                sortable : true,
                type : "string",
                width: 75
            },
            {
                field: "description",
                displayName: "Description",
                sortable : false,
                cellTemplate:'<div style="word-wrap: normal; white-space: pre-wrap">{{ row.entity.description }}</div>'
            },
            {
                field: "tool_name",
                displayName: "Tool",
                sortable : true,
                type : "string",
                width: 100
            }
        ];

        $scope.closeConfigurationDialog = function() {
            $scope.closeThisDialog();
            if ( _.isFunction(unbindHandler)) {
                unbindHandler();
            }
        };

        function handleError (jqXHR, textStatus, errorThrown) {
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var error = data.result[0].message;
                    console.log("The error is:" + error);
                    PHD.hideLoadingIndicator(savePSAConfigIndicator);
                    ngDialog.open({
                        dialogType:'Error',
                        modelDialogId:'ErrorDialog',
                        scope:$scope,
                        dialogMessage: error,
                        onConfirmOkButtonClick:'onConfirmError()'
                    });
                } else {
                    PHD.throwError(data);
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
            errorDialog = true;
        }

        $scope.onConfirmError = function () {
            ngDialog.close('ErrorDialog');
        };

        $scope.launchCredentialManager = function() {
            ngDialog.open({
                template: 'app/configure/credentials/credentials.html',
                scope: $scope,
                overlay:true,
                name: 'Credential-Manager',
                modelDialogId : 'credential-manager-dialog',
                ngDialogStyle:'width:700px; height:550px;',
                closeByDocument: false,
                closeByEscape: false,
                preCloseCallback: function (value) {
                    //processDialog(PHD,window);
                }
            });
        };

        // Initialize the controller, returning the function to turn off the listener.
        unbindHandler = $scope.$on('ngDialog.opened', function(event,obj) { $scope.initCtrl(obj) });

    }]);