angular.module('iscsi-chap-module', ['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            trapFocus: true,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('iscsiCHAPCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', function($scope, $rootScope, $http, ngDialog, $window, $timeout) {

        $scope.form = $();
        $scope.chap = {};
        $scope.chap.enabled = false;
        $scope.chap.username = null;
        $scope.chap.password = null;

        $scope.MAX_CHAP_LENGTH = 256;

        $scope.showCHAPtoolTip = function(){
            var form = angular.element("#iscsi-chap-form");
            form.find(".definition").popover({
                appendTo: form
            });
        };

        $scope.validateForm = function(form) {
            form.validate({
                rules: {
                    username: {
                        required: true,
                        minlength: 1,
                        maxlength: $scope.MAX_CHAP_LENGTH
                    },
                    chap_password: {
                        required: function(element) {
                            return $scope.chap.enabled;
                        },
                        minlength: 1,
                        maxlength: $scope.MAX_CHAP_LENGTH
                    },
                    chap_verifyPassword: {
                        required: function(element) {
                            return $scope.chap.enabled;
                        },
                        minlength: 1,
                        maxlength: $scope.MAX_CHAP_LENGTH,
                        equalTo: "#chap_password"
                    }
                },
                messages: {
                    username: {
                        required: gettext("A CHAP username is required")
                    },
                    chap_password: {
                        required: gettext("A CHAP password is required")
                    },
                    chap_verifyPassword: {
                        required: gettext("A CHAP confirm password is required"),
                        equalTo: gettext("Your CHAP password must be entered the same in both fields")
                    }
                }
            })
        };

        $scope.saveSettings = function(commandName) {
            if ($scope.form.valid()) {
                var url = "/api/storage/chap/?sid=" + selectedAppliance.id;
                var load = PHD.showLoadingIndicator("body", true);
                if ($scope.chap.enabled === false) {
                    $scope.chap.username = null;
                }
                var payload = $scope.chap;
                payload = JSON.stringify(payload);
                var resp = PHD.Ajax.put(url, payload, load, handleError);
                resp.done(function(data) {
                    $scope.closeThisDialog();
                });
            }
        };

        function handleError(jqXHR, textStatus, errorThrown) {
            if (jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var message = data.result[0].message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'chap-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.onConfirmError = function () {
            ngDialog.close('chap-error-dialog');
        };

        $scope.closeThisDialog = function() {
            // closing the wizard clears errors and state.
            $scope.PHD.currentMiniWizard.wizard("close");
        };

        $scope.getCHAPSettings = function () {
            var url = "/api/storage/chap/?sid=" + selectedAppliance.id;
            var load = PHD.showLoadingIndicator("body", true);
            var resp = PHD.Ajax.get(url, load, handleCHAPError);
            resp.done(function(data) {
                if (data.username) {
                    $scope.chap.username = data.username;
                    $scope.chap.enabled = true;
                    $timeout(function() {
                        $scope.$apply();
                    }, 100);
                }
            });
        };

        function handleCHAPError(jqXHR, textStatus, errorThrown) {
            if (jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var message = data.result[0].message;
                    // If CHAP is not enabled, don't show an error dialog, get the IQN
                    if (message.indexOf("CHAP not enabled.") != -1) {
                        // Get the IQN if not enabled, and populate the username field with it.
                        var url = "/api/storage/iqn";
                        var load = PHD.showLoadingIndicator("body", true);
                        var resp = PHD.Ajax.get(url, load, handleCHAPiqnError);
                        resp.done(function(data) {
                            if (data.iqn) {
                                $scope.chap.username = data.iqn;
                            }
                        });
                    } else {
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'chap-error-dialog',
                            scope: $scope,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    }
                } else {
                    PHD.ajaxError(jqXHR, textStatus, errorThrown);
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        function handleCHAPiqnError(jqXHR, textStatus, errorThrown) {
            if (jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var message = data.result[0].message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'chap-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                } else {
                    PHD.ajaxError(jqXHR, textStatus, errorThrown);
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.processDialog = function() {
            $scope.showCHAPtoolTip();

            $scope.form = angular.element("#iscsi-chap-form");
            $scope.validateForm($scope.form);

            // Load current CHAP settings.
            $scope.getCHAPSettings();
        };

        // use instead of ngDialog load event.
        angular.element(document).ready(function () {
            $scope.processDialog();
        });

    }]);