angular.module('networked-appliance-customers-module', ['ngDialog', 'ui.grid','ui.grid.resizeColumns','ui.grid.autoResize','ngGrid','ui.grid.selection'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('networkedApplianceCustomersCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants', 'gettextCatalog',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants, gettextCatalog) {

            $scope.customerDialog = null;
            $scope.locationDialog = null;
            $scope.selectedCustomer = null;
            $scope.savedCustomer = null;  // save customer to restore on cancel
            $scope.MAX_HOST_LENGTH = 64;
            $scope.refresh = false;
            $scope.editForm = null;

            $scope.saveSettings = function(commandName) {
                if ($scope.editForm != null && $scope.editForm.valid()) {
                    var url = "/portal/api/managers.php";
                    var load = PHD.showLoadingIndicator("body", true);
                    var payload = $scope.selectedCustomer;
                    if ($scope.isEdit && $scope.savedCustomer !== null) {
                        url += "?id=" + $scope.selectedCustomer['id'];
                    }
                    if (payload.manager !== undefined) {
                        payload.name = payload.manager;
                    }
                    payload = JSON.stringify(payload);
                    var method = $scope.isEdit ? 'PUT' : 'POST';
                    $http({
                        method: method,
                        url: url,
                        data: payload
                    }).success(function(data, status, headers) {
                        $scope.closeEditDialog();
                        $scope.reload();
                        PHD.hideLoadingIndicator(load);
                    }).error(function(data, status, headers) {
                        PHD.hideLoadingIndicator(load);
                        var message = data.message;
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'customer-error-dialog',
                            scope: $scope,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.onConfirmError = function() {
              ngDialog.close('customer-error-dialog');
            };

            $scope.closeEditDialog = function() {
                // clear any validation errors for the form before closing.
                $("#form-errors").hide();
                $scope.selectedCustomer = $scope.savedCustomer;
                // Must clear settings Dialog so it will open next time.
                ngDialog.close("edit-customer-dialog");
                // Reload as there may be new locations
                $scope.reload();
            };

            $scope.closeCustomersDialog = function() {
                // closing the wizard clears errors and state.
                $scope.closeThisDialog();
            };

            $scope.editCustomer = function(mode) {

                $scope.isEdit = mode;

                $scope.title = ($scope.isEdit ? gettextCatalog.getString("Edit") : gettextCatalog.getString("Add")) + gettextCatalog.getString(" Customer");

                if ($scope.isEdit) {
                    if ($scope.selectedCustomer == null) {
                        return;
                    }
                    $scope.savedCustomer = $scope.selectedCustomer;
                    $scope.selectedCustomer = jQuery.extend({}, $scope.selectedCustomer);
                } else {
                    $scope.savedCustomer = $scope.selectedCustomer;
                    $scope.selectedCustomer = {};
                }

                if ($scope.customerDialog == null) {
                    $scope.customerDialog = true;
                    ngDialog.open({
                        template: 'app/configure/networked-appliances/customers/customer-edit.html',
                        scope: $scope,
                        overlay:true,
                        modelDialogId: 'edit-customer-dialog',
                        name: 'edit-customer-dialog',
                        ngDialogStyle:'width:600px;height:400px',
                        closeByDocument: false,
                        closeByEscape: true,
                        preCloseCallback: function (value) {
                            setTimeout(function(){
                                // Clear errors and state.
                                $("#form-errors").hide();
                                $scope.customerDialog = null;
                                $scope.selectedCustomer = $scope.savedCustomer;
                                $scope.editForm = null;
                            }, 0);
                        }
                    });
                }
            };


            $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

            $scope.initCtrl = function(dlg){
                switch (dlg.name){
                    case 'edit-customer-dialog':
                        $scope.editForm = angular.element("#edit-customer-form");
                        break;
                    default:
                        // No action.
                        break;
                }
            };

            $scope.validateHostForm = function(form) {
                var IP_REGEXP = /\b(?:[0-9]{1,3}\.){3}[0-9]{1,3}\b/i;
                form.validate({
                    rules: {
                        name: {
                            required: true,
                            maxlength: $scope.MAX_HOST_LENGTH
                        },
                        ip: {
                            required: true,
                            pattern: IP_REGEXP
                        }
                    },
                    messages: {
                        name: {
                            required: gettextCatalog.getString("Hostname is required")
                        },
                        ip: {
                            required: gettextCatalog.getString("IP Address is required"),
                            pattern: gettextCatalog.getString("IP Address is not a valid address")
                        }
                    }
                })
            };

            $scope.confirmDeleteCustomer = function() {
                $scope.deleteTitle = gettextCatalog.getString('Confirm Customer Removal');
                $scope.confirmMessage1 = gettextCatalog.getString('Are you sure want to delete customer') + " " + $scope.selectedCustomer.name + '?';
                $scope.confirmMessage2 = gettextCatalog.getString('If you delete the customer, appliance information will not be available.');
                $scope.confirmMessage3 = gettextCatalog.getString('I understand the risks of removing a customer.');
                $scope.submitButtonText = 'Delete Customer';

                ngDialog.open({
                    template: 'app/common/confirmationDialog.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId: 'delete-customer-dialog',
                    ngDialogStyle:'width:600px;height:250px',
                    closeByDocument: false,
                    closeByEscape: true
                });
            };


            // Required name for common delete function.
            $scope.closeThisDialog = function() {
                ngDialog.close("delete-customer-dialog");
            };

            $scope.deleteAsset = function() {
                var url = "/portal/api/managers.php/?id=" + $scope.selectedCustomer.id;
                var load = PHD.showLoadingIndicator("body", true);
                $http({
                    method: 'DELETE',
                    url: url
                }).success(function(data, status, headers) {
                    PHD.hideLoadingIndicator(load);
                    $scope.reload();
                    $scope.closeThisDialog();
                }).error(function(data, status, headers) {
                    PHD.hideLoadingIndicator(load);
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'customer-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });
            };

            $scope.reload = function() {
                $scope.loadSettings();
            };

            $scope.customersGridData = {
                multiSelect: false,
                selectionRowHeaderWidth: 35,
                rowHeight: 30,
                enableFullRowSelection : true,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                onRegisterApi: function (gridApi) {
                    $scope.gridApi = gridApi;
                    gridApi.selection.on.rowSelectionChanged($scope, function(rows){
                        var selectedItems = gridApi.selection.getSelectedRows();
                        if (selectedItems.length > 0) {
                            $scope.selectedCustomer = selectedItems[0];
                        } else if (selectedItems.length == 0) {
                            $scope.selectedCustomer = null;
                        }
                    });
                },
                data: []
            };

            $scope.customersGridData.columnDefs = [
                {
                    field: "manager",
                    displayName: gettextCatalog.getString("Customer Name"),
                    sortable : true,
                    sortingType : "string",
                    sort: {
                        direction: uiGridConstants.ASC,
                        priority: 0
                    }
                },
                {
                    field: "locations_str",
                    displayName: gettextCatalog.getString("Locations"),
                    sortable : true,
                    sortingType : "string",
                    sort: {
                        direction: uiGridConstants.ASC,
                        priority: 1
                    }
                }
            ];

            $scope.loadSettings = function() {
                $http({
                    method: 'GET',
                    url: '/portal/api/managers.php/?level=locations'
                }).success(function(data, status, headers) {
                    $scope.selectedCustomer = null;
                    $scope.savedCustomer = null;
                    $scope.refresh = true;
                    $scope.customersGridData.data = data['managers'];
                    $timeout(function() {
                        $scope.refresh = false;
                    }, 0);

                }).error(function(data, status, headers) {
                    console.log('no device status');
                    $scope.customersGridData.data = [];
                });
            };

            // use instead of ngDialog load event.
            angular.element(document).ready(function () {
                $scope.loadSettings();
            });

        }])
    .controller('CustomerCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants', 'gettextCatalog',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants, gettextCatalog) {

            $scope.selectedLocation = null;
            $scope.savedLocation = null;
            $scope.editLocationForm = null;


            $scope.locationsGridData = {
                multiSelect: false,
                selectionRowHeaderWidth: 35,
                rowHeight: 30,
                enableFullRowSelection : true,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                onRegisterApi: function (gridApi) {
                    $scope.locationGridApi = gridApi;
                    gridApi.selection.on.rowSelectionChanged($scope, function(rows){
                        var selectedItems = gridApi.selection.getSelectedRows();
                        if (selectedItems.length > 0) {
                            $scope.selectedLocation = selectedItems[0];
                        } else if (selectedItems.length == 0) {
                            $scope.selectedLocation = null;
                        }
                    });
                },
                data: []
            };

            $scope.locationsGridData.columnDefs = [
                {
                    field: "location",
                    displayName: gettextCatalog.getString("Location"),
                    sortable : true,
                    sortingType : "string",
                    sort: {
                        direction: uiGridConstants.ASC,
                        priority: 0
                    }
                },
                {
                    field: "city",
                    displayName: gettextCatalog.getString("City"),
                    sortable : true
                },
                {
                    field: "state",
                    displayName: gettextCatalog.getString("State"),
                    sortable : true
                }
            ];

            if ($scope.selectedCustomer) {
                $scope.locationsGridData.data = $scope.selectedCustomer['locations'];
            }

            $scope.editLocation = function(mode) {

                $scope.isLocationEdit = mode;

                /*
                 * Check to see if the customer has been saved yet, and tell the user if not.
                 */
                if ($scope.selectedCustomer !== undefined && $scope.selectedCustomer['id'] === undefined) {
                    var message = gettextCatalog.getString("Please save the customer first, then add locations.");
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'customer-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                    return;
                }

                $scope.locationTitle = ($scope.isLocationEdit ? gettextCatalog.getString("Edit") : gettextCatalog.getString("Add")) + gettextCatalog.getString(" Location");

                if ($scope.isLocationEdit) {
                    if ($scope.selectedLocation == null) {
                        return;
                    }
                    $scope.savedLocation = $scope.selectedLocation;
                    $scope.selectedLocation = jQuery.extend({}, $scope.selectedLocation);
                } else {
                    $scope.savedLocation = $scope.selectedLocation;
                    $scope.selectedLocation = {};
                }

                if ($scope.locationDialog == null) {
                    $scope.locationDialog = true;
                    ngDialog.open({
                        template: 'app/configure/networked-appliances/customers/location-edit.html',
                        scope: $scope,
                        overlay:true,
                        modelDialogId: 'edit-location-dialog',
                        name: 'edit-location-dialog',
                        ngDialogStyle:'width:500px;height:400px',
                        closeByDocument: false,
                        closeByEscape: true,
                        preCloseCallback: function (value) {
                            setTimeout(function(){
                                // Clear errors and state.
                                $("#form-errors").hide();
                                $scope.locationDialog = null;
                                $scope.selectedLocation = $scope.savedLocation;
                                $scope.editLocationForm = null;
                            }, 0);
                        }
                    });
                }
            };

            $scope.reloadLocations = function() {
                $http({
                    method: 'GET',
                    url: '/portal/api/managers.php/?level=locations&id=' + $scope.selectedCustomer['id']
                }).success(function(data, status, headers) {
                    $scope.selectedLocation = null;
                    $scope.savedLocation = null;
                    $scope.refresh = true;
                    $scope.locationsGridData.data = data['managers'][0]['locations'];
                    $timeout(function() {
                        $scope.refresh = false;
                    }, 0);
                }).error(function(data, status, headers) {
                    console.log('no device status');
                    $scope.locationsGridData.data = [];
                });
            };



            $scope.confirmDeleteLocation = function() {
                $scope.deleteTitle = gettextCatalog.getString('Confirm Location Removal');
                $scope.confirmMessage1 = gettextCatalog.getString('Are you sure want to delete location') + ""  + $scope.selectedLocation.location + '?';
                $scope.confirmMessage2 = gettextCatalog.getString('Any distributed appliances associated with this location will be removed.');
                $scope.confirmMessage3 = gettextCatalog.getString('I understand the risks of removing a location.');
                $scope.submitButtonText = gettextCatalog.getString('Delete Location');

                ngDialog.open({
                    template: 'app/common/confirmationDialog.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId: 'delete-location-dialog',
                    ngDialogStyle:'width:600px;height:250px',
                    closeByDocument: false,
                    closeByEscape: true
                });
            };


            // Required name for common delete function.
            $scope.closeThisDialog = function() {
                ngDialog.close("delete-location-dialog");
            };

            $scope.deleteAsset = function() {
                var url = "/portal/api/locations.php/?location_id=" + $scope.selectedLocation['location_id'];
                var load = PHD.showLoadingIndicator("body", true);
                $http({
                    method: 'DELETE',
                    url: url
                }).success(function(data, status, headers) {
                    PHD.hideLoadingIndicator(load);
                    $scope.reloadLocations();
                    $scope.closeThisDialog();
                }).error(function(data, status, headers) {
                    PHD.hideLoadingIndicator(load);
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'customer-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });
            };

            $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

            $scope.initCtrl = function(dlg){
                switch (dlg.name){
                    case 'edit-location-dialog':
                        $scope.editLocationForm = angular.element("#edit-location-form");
                        break;
                    default:
                        // No action.
                        break;
                }
            };

        }])
    .controller('CustomerLocationCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants) {

            $scope.saveLocation = function() {
                if ($scope.editLocationForm != null && $scope.editLocationForm.valid()) {
                    var url = "/portal/api/locations.php";
                    var load = PHD.showLoadingIndicator("body", true);
                    var payload = $scope.selectedLocation;
                    if ($scope.isLocationEdit && $scope.savedLocation !== null) {
                        //payload.original_ip = $scope.savedCustomer.ip;
                        url += '?location_id=' + $scope.selectedLocation['location_id'];
                    } else {
                        payload.manager_id = $scope.selectedCustomer['id'];
                    }
                    if (payload.location !== undefined) {
                        payload.name = payload.location;
                    }
                    payload = JSON.stringify(payload);
                    var method = $scope.isLocationEdit ? 'PUT' : 'POST';
                    $http({
                        method: method,
                        url: url,
                        data: payload
                    }).success(function(data, status, headers) {
                        $scope.closeEditLocationDialog();
                        $scope.reloadLocations();
                        PHD.hideLoadingIndicator(load);
                    }).error(function(data, status, headers) {
                        PHD.hideLoadingIndicator(load);
                        var message = data.message;
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'customer-error-dialog',
                            scope: $scope,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.closeEditLocationDialog = function() {
                // clear any validation errors for the form before closing.
                $("#form-errors").hide();
                $scope.selectedLocation = $scope.savedLocation;
                // Must clear settings Dialog so it will open next time.
                ngDialog.close("edit-location-dialog");
            };

        }]);