angular.module('configNetworkedAppliances', ['ngDialog', 'angulartics', 'angulartics.google.analytics',
    'ui.grid', 'ui.grid.resizeColumns', 'ui.grid.autoResize', 'ui.grid.treeView', 'ui.grid.selection'])

    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('NetworkedAppliancesCtrl', ['$scope', '$rootScope', '$http', 'ngDialog', 'wizardService', '$analytics', '$timeout', 'gettextCatalog',
        function($scope, $rootScope, $http, ngDialog, wizardService, $analytics, $timeout, gettextCatalog) {

            var $globalScope = $("[ng-controller=GlobalCtrl]").scope();
            $scope.user.monitor = $globalScope.user.monitor;

            $scope.selectedAppliance = null;
            $scope.savedAppliance = null;  // save appliance to restore on cancel

            // Tunnel information.
            $scope.tunnel = 0;
            $scope.managerAddress = "";

            var DEM_KB_URL = KB_URL + "2030";
            var DEM_URL_TITLE = gettextCatalog.getString('Click for more information on DEM configuration');

            $scope.initCtrl = function () {
                processAppliances(PHD, window, $analytics);
            };

            $scope.loadDevices = function() {
                $http({
                    method: 'GET',
                    url: '/portal/api/managers.php/?level=devices'
                }).success(function(data, status, headers) {
                    $scope.deviceGridOptions.data = data['managers'];
                    // Set the height of the table once loaded.
                    $timeout(function() {
                        $scope.setTableHeight();
                    }, 0);
                }).error(function(data, status, headers) {
                    console.log('no device status');
                    $scope.deviceGridOptions.data = [];
                });
            };

            function processAppliances(PHD, window, $analytics) {

                var NetworkedAppliancesTab = function(index, $panel) {
                    this.index = index;
                    this.$panel = $panel;
                    this.$applianceArea = this.$panel.children("#appliances-list");
                    this.environmentList = {};
                    this.environments = {};
                    this.activeEnvironment = null;
                    this.isActive = false;
                    this.isBootstrapped = false;
                };

                NetworkedAppliancesTab.prototype.init = function() {
                    var self = this;
                    return this;
                };

                NetworkedAppliancesTab.prototype.activate = function() {
                    this.isActive = true;
                    $scope.loadDevices();
                    return this;
                };

                NetworkedAppliancesTab.prototype.deactivate = function() {
                    this.isActive = false;
                    return this;
                };

                NetworkedAppliancesTab.prototype.queueSelection = function (selected) {
                    var self = this;
                    var dbid = selected.id;
                    var sid = parseInt(selected.sid);
                    /*if (this.activeEnvironment) {
                        if (this.activeEnvironment.$applianceTable) {
                            this.activeEnvironment.$applianceTable.on("loadfinished", function () {
                                // Select the appliance from the Appliances list.
                                self.activeEnvironment.ApplianceTable.selectByID(sid);
                                if (usage !== 'archive') {
                                    // Select the backup storage by integer id.
                                    dbid = parseInt(dbid);
                                    if (self.activeEnvironment.$storageTable) {
                                        self.activeEnvironment.$storageTable.on("loadfinished", function () {
                                            setTimeout(function(){
                                                self.activeEnvironment.StorageTable.selectByID(dbid);
                                            }, 0);
                                        });
                                    }
                                } else {
                                    // Switch to tab 1, Backup Copy Targets, selected by string id.
                                    $(".appliance-details.sub-tabs-wrapper").tabs("option", "active", 1);
                                    if (self.activeEnvironment.$targetsTable) {
                                        self.activeEnvironment.$targetsTable.on("loadfinished", function () {
                                            setTimeout(function(){
                                                self.activeEnvironment.TargetsTable.selectByID(dbid);
                                            }, 0);
                                        });
                                    }
                                }
                            });
                        }
                    } */
                };

                PHD.NetworkedApplianceTab = NetworkedAppliancesTab;
            }

            var nameTemplate = '<div class="ui-grid-cell-contents" title="{{row.entity.hostname}}" ng-style="{ \'margin-left\': (row.treeLevel * 20 + 2) + \'px\' }"><i style="margin-right:4px" class="{{grid.appScope.getIcon(row.entity)}} icon"></i>{{row.entity.hostname}}</div>';

            $scope.handleRowSelection = function(row) {
                if (row.isSelected){
                    $scope.selectedAppliance = row.entity;
                } else {
                    $scope.selectedAppliance = null;
                }
            };

            $scope.deviceGridOptions = {
                enableSorting: true,
                enableFiltering: true,
                //showTreeExpandNoChildren: false,
               // treeRowHeaderBaseWidth: 20,
                enableRowSelection : false,
                enableRowHeaderSelection: true,
                selectionRowHeaderWidth: 28,
                rowHeight: 28,
                enableColumnResizing: true,
                enableHorizontalScrollbar: 0,
                multiSelect: false,
                columnDefs: [
                    {
                        name: 'manager',
                        displayName: gettextCatalog.getString('CUSTOMER'),
                        width: '20%',
                        cellTooltip: true
                    },
                    {
                        name: 'location',
                        displayName: gettextCatalog.getString('LOCATION'),
                        width: '20%',
                        cellTooltip: true
                    },
                    {
                        name: 'hostname',
                        displayName: gettextCatalog.getString('APPLIANCE HOSTNAME'),
                        width: '20%',
                        cellTooltip: true
                    },
                    {
                        name: 'uuid',
                        displayName: gettextCatalog.getString('ASSET TAG'),
                        width: '*',
                        cellTooltip: true
                    }
                ],
                data: [],
                onRegisterApi: function( gridApi ) {
                    $scope.gridApi = gridApi;
                    /*$scope.gridApi.treeBase.on.rowExpanded($scope, function(row) {
                        console.log('expand');
                    });*/
                    $scope.gridApi.selection.on.rowSelectionChanged($scope, $scope.handleRowSelection);
                    $scope.gridApi.selection.on.rowSelectionChangedBatch($scope, $scope.handleRowSelection);
                }
            };

            $scope.editAppliance = function(mode) {
                if (!$scope.user.monitor) {
                    $scope.isEdit = mode == 1;
                    if ($scope.isEdit) {
                        if ($scope.selectedAppliance == null) {
                            return;
                        }
                        $scope.savedAppliance = $scope.selectedAppliance;
                        $scope.selectedAppliance = jQuery.extend({}, $scope.selectedAppliance);
                    } else {
                        $scope.savedAppliance = $scope.selectedAppliance;
                        $scope.selectedAppliance = {};
                    }
                    $scope.applianceTitle = ($scope.isEdit ? gettextCatalog.getString("Edit") : gettextCatalog.getString("Add")) + gettextCatalog.getString(" Appliance");
                    $http({
                        method: 'GET',
                        url: '/portal/api/managers.php'
                    }).success(function (data, status, headers) {
                        $scope.managersList = data['managers'];
                        if ($scope.selectedAppliance !== undefined && $scope.selectedAppliance.manager_id !== undefined) {
                            $scope.onManagerChange($scope.selectedAppliance.manager_id);
                        }
                        ngDialog.open({
                            template: 'app/configure/networked-appliances/appliance-edit.html',
                            scope: $scope,
                            overlay: true,
                            modelDialogId: 'Device-Manager',
                            name: 'Device-Manager',
                            ngDialogStyle: 'width:700px; height:550px;',
                            closeByDocument: false,
                            closeByEscape: true,
                            preCloseCallback: function (value) {
                            }
                        });
                    }).error(function (data, status, headers) {
                        var message = data.message;
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'device-error-dialog',
                            scope: $scope,
                            helpArticleLink: DEM_KB_URL,
                            helpArticleTitle: DEM_URL_TITLE,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.onManagerChange = function(manager) {
                $http({
                    method: 'GET',
                    url: '/portal/api/managers.php?level=locations&id=' + manager
                }).success(function(data, status, headers) {
                    $scope.locationsList = [];
                    if (data['managers'] !== undefined && data['managers'].length > 0) {
                        var manager = data['managers'][0];
                        if (manager['locations'] !== undefined) {
                            $scope.locationsList = manager['locations'];
                        }
                    }
                }).error(function(data, status, headers) {
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'device-error-dialog',
                        scope: $scope,
                        helpArticleLink: DEM_KB_URL,
                        helpArticleTitle: DEM_URL_TITLE,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });
            };

            $scope.saveAppliance = function() {
                if ($scope.isEdit) {
                    $scope.saveAssociations();
                } else {
                    $scope.addAppliance();
                }
            };

            $scope.saveAssociations = function() {
                var url = "/portal/api/devices.php?id=" + $scope.selectedAppliance['device_id'];
                var load = PHD.showLoadingIndicator("body", true);
                var payload = {};
                payload.location_id = $scope.selectedAppliance.location_id;
                payload = JSON.stringify(payload);
                var method = 'PUT';
                $http({
                    method: method,
                    url: url,
                    data: payload
                }).success(function(data, status, headers) {
                    $scope.closeEditDialog();
                    $scope.loadDevices();
                    PHD.hideLoadingIndicator(load);
                }).error(function(data, status, headers) {
                    PHD.hideLoadingIndicator(load);
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'device-error-dialog',
                        scope: $scope,
                        helpArticleLink: DEM_KB_URL,
                        helpArticleTitle: DEM_URL_TITLE,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });
            };

            $scope.addAppliance = function() {
                if (!$scope.user.monitor) {
                    // set devtype to an even number when adding (to manage it by default)
                    var url = "/portal/api/devices.php?devtype=2";
                    var load = PHD.showLoadingIndicator("body", true);
                    var payload = $scope.selectedAppliance;
                    payload.asset_tag = payload.uuid;
                    payload = JSON.stringify(payload);
                    var method = 'POST';
                    $http({
                        method: method,
                        url: url,
                        data: payload
                    }).success(function (data, status, headers) {
                        $scope.closeEditDialog();
                        $scope.loadDevices();
                        PHD.hideLoadingIndicator(load);
                    }).error(function (data, status, headers) {
                        PHD.hideLoadingIndicator(load);
                        var message = data.message;
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'device-error-dialog',
                            scope: $scope,
                            helpArticleLink: DEM_KB_URL,
                            helpArticleTitle: DEM_URL_TITLE,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.onConfirmError = function() {
                ngDialog.close('device-error-dialog');
            };

            $scope.deleteAppliance = function() {
                if (!$scope.user.monitor) {
                    if ($scope.selectedAppliance) {
                        $scope.deleteTitle = gettextCatalog.getString('Confirm Appliance Removal');
                        $scope.confirmMessage1 = gettextCatalog.getString('Are you sure want to delete appliance ') + $scope.selectedAppliance.hostname + '?';
                        $scope.confirmMessage2 = gettextCatalog.getString('When an appliance is removed, its status is deleted and if open, the management connection is closed.');
                        $scope.confirmMessage3 = gettextCatalog.getString('I understand that appliance removal will delete status and close the management connection');
                        $scope.submitButtonText = gettextCatalog.getString('Remove Appliance');

                        ngDialog.open({
                            template: 'app/common/confirmationDialog.html',
                            scope: $scope,
                            overlay: true,
                            ngDialogStyle: 'width:600px;height:250px',
                            closeByDocument: false,
                            closeByEscape: true
                        });
                    } else {
                        ngDialog.open({
                            dialogType: 'Warning',
                            dialogMessage: gettextCatalog.getString('You must select one and only one appliance at a time for deletion.')
                        });
                    }
                }
            };

            /*
             * Delete the specified appliance, after checking to see if a tunnel is open.
             * If a tunnel is open, close it, then delete.
             */
            $scope.deleteAsset = function() {
                // close tunnel if open.
                var method = 'PUT';
                var url = "/portal/api/tunnel.php?id=";
                var load = PHD.showLoadingIndicator("body", true, gettextCatalog.getString("Closing any management connections to this appliance.  Please wait...."));
                url += $scope.selectedAppliance.device_id + "&cmd=close-tunnel&parameter=" + $scope.tunnel;
                var tunnelWasOpen = false;
                $http({
                    method: method,
                    url: url
                }).then(function(data, status, headers) {
                    PHD.hideLoadingIndicator(load);
                    $scope.tunnel = 0;
                    $scope.local_tunnel = 0;
                    $scope.managerAddress = "";
                    tunnelWasOpen = true;
                    $scope.doDelete(tunnelWasOpen);
                }, function(data, status, headers) {
                    PHD.hideLoadingIndicator(load);
                    $scope.doDelete(tunnelWasOpen);
                });
            };

            $scope.doDelete = function(tunnelWasOpen) {
                var url = '/portal/api/devices.php?id='+$scope.selectedAppliance.device_id;
                var load = PHD.showLoadingIndicator("body", true);
                var method = 'DELETE';

                $http({
                    method: method,
                    url : url
                }).success(function(status, headers){
                    $scope.loadDevices();
                    PHD.hideLoadingIndicator(load);
                    var message = $scope.selectedAppliance.hostname + gettextCatalog.getString(" removed successfully.");
                    if (tunnelWasOpen) {
                        message += gettextCatalog.getString("  A management connection was closed.  Please close any browser tab that may have been open to ") + $scope.selectedAppliance.hostname + ".";
                    }
                    ngDialog.open({
                        dialogType:'information',
                        dialogMessage: message
                    });
                    $scope.selectedAppliance = null;
                }).error(function(data, status, headers) {
                    PHD.hideLoadingIndicator(load);
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'device-error-dialog',
                        scope: $scope,
                        helpArticleLink: DEM_KB_URL,
                        helpArticleTitle: DEM_URL_TITLE,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });

            };

            $scope.manageAppliance = function() {
                if (!$scope.user.monitor) {
                    var method = 'PUT';
                    var url = "/portal/api/tunnel.php?id=";
                    var load = PHD.showLoadingIndicator("body", true, gettextCatalog.getString("Contacting distributed appliance.  Please wait...."));
                    url += $scope.selectedAppliance.device_id + "&cmd=check-tunnel";
                    $http({
                        method: method,
                        url: url
                    }).success(function (data, status, headers) {
                        PHD.hideLoadingIndicator(load);
                        $scope.tunnel = data['tunnel'];
                        $scope.managerAddress = data['host'];
                        if ($scope.tunnel == 0) {
                            // No tunnel is open.  Open one.
                            ngDialog.open({
                                template: 'app/configure/networked-appliances/confirm-manage-appliance.html',
                                scope: $scope,
                                overlay: true,
                                name: 'Manage-Appliance',
                                ngDialogStyle: 'width:500px; height:210px;',
                                closeByDocument: false,
                                closeByEscape: true,
                                preCloseCallback: function (value) {
                                }
                            });
                        } else {
                            // A tunnel is open.  Explain how to use or close it.
                            $scope.local_tunnel = data['localTunnel'] !== undefined ? data['localTunnel'] : $scope.tunnel + 1;
                            ngDialog.open({
                                template: 'app/configure/networked-appliances/tunnel-open-close.html',
                                scope: $scope,
                                overlay: true,
                                name: 'close-tunnel',
                                ngDialogStyle: 'width:500px; height:200px;',
                                closeByDocument: false,
                                closeByEscape: true
                            });
                        }
                    }).error(function (data, status, headers) {
                        PHD.hideLoadingIndicator(load);
                        var message = data.message;
                        if (message instanceof Array) {
                            message = message.toString();
                        }
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'device-error-dialog',
                            scope: $scope,
                            helpArticleLink: DEM_KB_URL,
                            helpArticleTitle: DEM_URL_TITLE,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.onConfirmCloseTunnel = function() {
                if ($scope.selectedAppliance) {
                    var method = 'PUT';
                    var url = "/portal/api/tunnel.php?id=";
                    url += $scope.selectedAppliance.device_id + "&cmd=close-tunnel&parameter=" + $scope.tunnel;
                    $http({
                        method: method,
                        url: url
                    }).success(function(data, status, headers) {
                        ngDialog.open({
                            dialogType: 'Information',
                            dialogMessage: gettextCatalog.getString("The management connection has been closed."),
                            preCloseCallback: function (value) {
                                $scope.tunnel = 0;
                                $scope.local_tunnel = 0;
                                $scope.managerAddress = "";
                            }
                        });
                    }).error(function(data, status, headers) {
                        var message = data.message;
                        if (message instanceof Array) {
                            message = message.toString();
                        }
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'device-error-dialog',
                            scope: $scope,
                            helpArticleLink: DEM_KB_URL,
                            helpArticleTitle: DEM_URL_TITLE,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.createManagementConnection = function() {
                if ($scope.selectedAppliance) {
                    var method = 'PUT';
                    var url = "/portal/api/tunnel.php?id=";
                    url += $scope.selectedAppliance.device_id + "&cmd=open-tunnel";
                    var load = PHD.showLoadingIndicator("body", true, gettextCatalog.getString("Opening connection to appliance...."));
                    $http({
                        method: method,
                        url: url
                    }).success(function(data, status, headers) {
                        PHD.hideLoadingIndicator(load);
                        $scope.tunnel = data['tunnel'];
                        $scope.local_tunnel = data['host_tunnel'];
                        $scope.managerAddress = data['host'];
                        ngDialog.open({
                            template: 'app/configure/networked-appliances/tunnel-open-success.html',
                            scope: $scope,
                            overlay:true,
                            name: 'Manage-Appliance',
                            ngDialogStyle:'width:500px; height:200px;',
                            closeByDocument: false,
                            closeByEscape: true
                        });
                    }).error(function(data, status, headers) {
                        PHD.hideLoadingIndicator(load);
                        var message = data.message;
                        if (message instanceof Array) {
                            message = message.toString();
                        }
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'device-error-dialog',
                            scope: $scope,
                            helpArticleLink: DEM_KB_URL,
                            helpArticleTitle: DEM_URL_TITLE,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.closeEditDialog = function() {
                $scope.selectedAppliance = $scope.savedAppliance;
                ngDialog.close('Device-Manager');
            };

            $scope.launchCustomerManager = function(followFunction) {
                if (!$scope.user.monitor) {
                    ngDialog.open({
                        template: 'app/configure/networked-appliances/customers/customers.html',
                        scope: $scope,
                        overlay: true,
                        name: 'Customer-Manager',
                        ngDialogStyle: 'width:700px; height:550px;',
                        closeByDocument: false,
                        closeByEscape: true,
                        preCloseCallback: function (value) {
                            if (followFunction != undefined) {
                                followFunction();
                            }
                        }
                    });
                }
            };

            $scope.launchPortalConfiguration = function(followFunction) {
                if (!$scope.user.monitor) {
                    ngDialog.open({
                        template: 'app/configure/networked-appliances/options/portal-settings.html',
                        scope: $scope,
                        overlay: true,
                        name: 'Settings-Manager',
                        ngDialogStyle: 'width:700px; height:550px;',
                        closeByDocument: false,
                        closeByEscape: true,
                        preCloseCallback: function (value) {
                            if (followFunction != undefined) {
                                followFunction();
                            }
                        }
                    });
                }
            };

            $scope.setTableHeight = function() {
                var containerHeight = angular.element(document.getElementById('networked-appliance-grid-section')).height();
                // leave some room between the table and the window bottom.
                var gridHeight = containerHeight - 32;
                angular.element(document.getElementById('networkedApplianceGrid')).css('height', gridHeight + 'px');
            };


        }]);