angular.module('networked-appliance-options-module', ['ngDialog', 'ui.grid','ui.grid.resizeColumns','ui.grid.autoResize','ngGrid','ui.grid.selection'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('PortalOptionsCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants', 'gettextCatalog',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants, gettextCatalog) {

            $scope.settingsDialog = null;
            $scope.selectedSetting = null;
            $scope.savedSetting = null;
            $scope.rowEntity = null;
            $scope.refresh = false;
            $scope.editForm = null;

            $scope.saveSettings = function(commandName) {
                var url = "/portal/api/settings.php";
                var load = PHD.showLoadingIndicator("body", true);
                var payload = { 'section': $scope.selectedSetting.section,
                                'field': $scope.selectedSetting.field,
                                'value': $scope.selectedSetting.value };
                payload = JSON.stringify(payload);

                var method = 'PUT';
                $http({
                    method: method,
                    url: url,
                    data: payload
                }).success(function(data, status, headers) {
                    $scope.closeEditDialog();
                    $scope.reload();
                    PHD.hideLoadingIndicator(load);
                }).error(function(data, status, headers) {
                    PHD.hideLoadingIndicator(load);
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'edit-settings-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });
            };

            $scope.onConfirmError = function () {
                ngDialog.close('edit-settings-error-dialog');
            };

            $scope.closeEditDialog = function() {
                // clear any validation errors for the form before closing.
                $("#form-errors").hide();
                $scope.selectedSetting = $scope.savedSetting;
                // Must clear settings Dialog so it will open next time.
                ngDialog.close("edit-settings-dialog");
            };

            $scope.closeSettingsDialog = function() {
                $scope.closeThisDialog();
            };

            $scope.editSetting = function(mode) {

                $scope.isEdit = mode;

                $scope.title = ($scope.isEdit ? gettextCatalog.getString("Edit") : gettextCatalog.getString("Add")) + "" + gettextCatalog.getString("Setting");

                if ($scope.isEdit) {
                    if ($scope.selectedSetting == null) {
                        return;
                    }
                    $scope.savedSetting = $scope.selectedSetting;
                    $scope.selectedSetting = jQuery.extend({}, $scope.selectedSetting);
                } else {
                    $scope.savedSetting = $scope.selectedSetting;
                    $scope.selectedSetting = {};
                }

                if ($scope.settingsDialog == null) {
                    $scope.settingsDialog = true;
                    ngDialog.open({
                        template: 'app/configure/networked-appliances/options/settings-edit.html',
                        scope: $scope,
                        overlay:true,
                        modelDialogId: 'edit-settings-dialog',
                        name: 'edit-settings-dialog',
                        ngDialogStyle:'width:600px;height:300px',
                        closeByDocument: false,
                        closeByEscape: true,
                        preCloseCallback: function (value) {
                            setTimeout(function(){
                                // Clear errors and state.
                                $("#form-errors").hide();
                                $scope.settingsDialog = null;
                                $scope.selectedSetting = $scope.savedSetting;
                                $scope.editForm = null;
                            }, 0);
                        }
                    });
                }
            };

            $scope.confirmDeleteSetting = function() {
                $scope.deleteTitle = gettextCatalog.getString('Confirm Setting Removal');
                $scope.confirmMessage1 = gettextCatalog.getString('Are you sure want to delete setting') + "" + $scope.selectedSetting.field + '?';
                $scope.confirmMessage2 = gettextCatalog.getString('If you delete the setting, the portal communications may be negatively affected.');
                $scope.confirmMessage3 = gettextCatalog.getString('I understand the risks of removing a setting.');
                $scope.submitButtonText = gettextCatalog.getString('Delete Setting');

                ngDialog.open({
                    template: 'app/common/confirmationDialog.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId: 'delete-setting-dialog',
                    ngDialogStyle:'width:600px;height:250px',
                    closeByDocument: false,
                    closeByEscape: true
                });
            };


            // Required name for common delete function.
            $scope.closeThisDialog = function() {
                ngDialog.close("delete-setting-dialog");
            };

            $scope.deleteAsset = function() {
                var method = 'DELETE';
                var url = "/portal/api/settings.php";
                var payload = { 'section': $scope.selectedSetting.section,
                    'field': $scope.selectedSetting.field,
                    'value': $scope.selectedSetting.value };
                payload = JSON.stringify(payload);
                $http({
                    method: method,
                    url: url,
                    data: payload
                }).success(function(data, status, headers) {
                    PHD.hideLoadingIndicator(load);
                    $scope.reload();
                    $scope.closeThisDialog();
                }).error(function(data, status, headers) {
                    PHD.hideLoadingIndicator(load);
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'edit-settings-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });
            };

            $scope.reload = function() {
                $scope.loadSettings();
            };

            $scope.settingsGridData = {
                multiSelect: false,
                selectionRowHeaderWidth: 35,
                rowHeight: 30,
                enableFullRowSelection : true,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                onRegisterApi: function (gridApi) {
                    $scope.gridApi = gridApi;
                    gridApi.selection.on.rowSelectionChanged($scope, function(rows){
                        var selectedItems = gridApi.selection.getSelectedRows();
                        if (selectedItems.length > 0) {
                            $scope.selectedSetting = selectedItems[0];
                        } else if (selectedItems.length == 0) {
                            $scope.selectedSetting = null;
                        }
                    });
                },
                data: []
            };

            $scope.settingsGridData.columnDefs = [
                {
                    field: "section",
                    displayName: gettextCatalog.getString("Area"),
                    sortable : true,
                    sortingType : "string",
                    sort: {
                        direction: uiGridConstants.ASC,
                        priority: 0
                    }
                },
                {
                    field: "field",
                    displayName: gettextCatalog.getString("Name"),
                    sortable : true,
                    sortingType : "string"
                },
                {
                    field: "value",
                    displayName: gettextCatalog.getString("Value"),
                    sortable : true,
                    sortingType : "string"
                }
            ];

            $scope.loadSettings = function() {
                $http({
                    method: 'GET',
                    url: '/portal/api/settings.php?type=flat'
                }).success(function(data, status, headers) {
                    $scope.selectedSetting = null;
                    $scope.savedSetting = null;
                    $scope.refresh = true;
                    $scope.settingsGridData.data = data['settings'];
                    $timeout(function() {
                        $scope.refresh = false;
                    }, 0);

                }).error(function(data, status, headers) {
                    console.log('no device status');
                    $scope.settingsGridData.data = [];
                });
            };

            // use instead of ngDialog load event.
            angular.element(document).ready(function () {
                $scope.loadSettings();
            });

            $scope.showSettingsTooltip = function(){
                var form = angular.element("#edit-settings-form");
                form.find(".definition").popover({
                    appendTo: form
                });
            };

            $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

            $scope.initCtrl = function(dlg){
                switch (dlg.name){
                    case 'edit-settings-dialog':
                        $scope.showSettingsTooltip();
                        break;
                    default:
                        // No action.
                        break;
                }
            };

        }]);