angular.module('network-module', ['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('networkCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', 'gettextCatalog',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, gettextCatalog) {

            $scope.showAdvanced = false;
            $scope.countdownSpinner = null;

            $scope.saveNetwork = function() {
                if ($scope.form !== undefined && $scope.form.valid()) {
                    var url = "/api/networks/" + $scope.network.name + "/?sid=" + $scope.selectedID
                    var load = PHD.showLoadingIndicator("body", true, "Updating Network.....");
                    var payload = $scope.network;
                    if (payload.auto_restore === true && $scope.countdownSpinner !== null) {
                        // The countdown spinner is in minutes; convert to the API countdown_time in seconds.
                        payload.countdown_time = $scope.countdownSpinner.val() * 60;
                    }
                    payload = JSON.stringify(payload);
                    var resp = PHD.Ajax.put(url, payload, load, handleError);
                    resp.done(function(data) {
                        $scope.closeNetworkDialog();
                        $(document).trigger("nicsave", [$scope.selectedID, data]);
                    });
                }
            };

            $scope.closeNetworkDialog = function() {
                PHD.currentDialog.wizard("close");
            };

            function handleError(jqXHR, textStatus, errorThrown) {
                if (jqXHR.status === 500) {
                    var data = jqXHR.responseJSON;
                    if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                        var message = data.result[0].message;
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'edit-nic-error-dialog',
                            scope: $scope,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    }
                } else {
                    PHD.ajaxError(jqXHR, textStatus, errorThrown);
                }
            }

            $scope.onConfirmError = function () {
                ngDialog.close('edit-nic-error-dialog');
            };

            $scope.onShowAdvanced = function() {
                var height = 580;
                if ($scope.showAdvanced) {
                    // Add advanced checkboxes, increase dialog height.
                    $scope.countdownSpinner = $("#nic-countdown");
                    $scope.countdownSpinner.spinner({min: 1, max: 120});
                    $scope.countdownSpinner.val(15);
                    height = 710;
                }
                PHD.currentDialog.wizard("option", "height", height);
            };

            $scope.onStopRestore = function() {
                if ($scope.network.stop_restore) {
                    ngDialog.open({
                        template: 'app/configure/networks/confirm-stop-restore.html',
                        scope: $scope,
                        overlay:true,
                        modelDialogId: 'stop-restore-dialog',
                        ngDialogStyle:'width:450px;height:180px',
                        closeByDocument: false,
                        closeByEscape: true
                    });
                }
            };

            $scope.stopNetworkRestore = function() {
                var url = "/api/networks/restore/?sid=" + $scope.selectedID;
                var load = PHD.showLoadingIndicator("body", true, "Stopping Network Settings Restore.....");
                var resp = PHD.Ajax.delete(url, null, load, handleError);
                resp.done(function(data) {
                    $scope.closeThisDialog();
                    $scope.closeNetworkDialog();
                    $(document).trigger("nicsave", [$scope.selectedID, data]);
                });
            };

            $scope.continueNetworkRestore = function() {
                if ($scope.network !== null && $scope.network !== undefined) {
                    $scope.network.stop_restore = false;
                    $("#nic-stop-restore").prop("checked", false);
                }
                $scope.closeThisDialog();
            };

            $scope.closeThisDialog = function() {
                ngDialog.close('stop-restore-dialog');
            };

            $scope.initForm= function() {
                $.validator.addMethod("ipv4", function(value, element, param) {
                    return this.optional(element) || /^(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)$/i.test(value);
                }, "Please enter a valid IP v4 address.");

                $scope.form = $("#nic-form");
                if ($scope.form !== undefined) {
                    $scope.form.validate({
                        errorLabelContainer: ".errorlist",
                        errorContainer: "#dialog-errors",
                        showErrors: function(errorMap, errorList) {
                            $("#dialog-errors").find(".summary")
                                .html(gettext("Correct the following errors:"));
                            this.defaultShowErrors();
                        },
                        rules: {
                            ip: {
                                required: true,
                                ipv4: true
                            },
                            gateway: {
                                required: false,
                                ipv4: true
                            },
                            netmask: {
                                required: true,
                                ipv4: true
                            },
                            dns1: {
                                required: true,
                                ipv4: true
                            }
                        },
                        messages: {
                            ip: {
                                required: gettextCatalog.getString("IP Address is required")
                            },
                            netmask: {
                                required: gettextCatalog.getString("Subnet is required")
                            },
                            dns1: {
                                required: gettextCatalog.getString("Preferred DNS is required")
                            }
                        }
                    });

                    $scope.showDNSSearchTooltip = function(){
                        $scope.form.find(".definition").popover({
                            appendTo: $scope.form,
                            position: {
                                my: "left-20 top",
                                at: "left-20 top+30"
                            }
                        });
                    };

                    // Put DNS tooltip in a timeout so that translations will load.
                    $timeout(function() {
                        $scope.showDNSSearchTooltip();
                    });

                }
            };

            // use instead of ngDialog load event.
            angular.element(document).ready(function () {
                $scope.initForm();
            });
        }]);
