angular.module('advanced-hosts-module', ['ngDialog', 'ui.grid','ui.grid.resizeColumns','ui.grid.autoResize','ngGrid','ui.grid.selection'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('hostsFileCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants', 'gettextCatalog',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants, gettextCatalog) {

            $scope.hostsDialog = null;
            $scope.selectedHost = null;
            $scope.savedHost = null;  // save host to restore on cancel
            $scope.MAX_HOST_LENGTH = 64;
            $scope.refresh = false;
            $scope.editForm = null;
            $scope.LOCALIP = '127.0.0.1';

            $scope.saveSettings = function(commandName) {
                if ($scope.editForm != null && $scope.editForm.valid()) {
                    var url = "/api/hosts/?sid=" + $scope.selectedID;
                    var load = PHD.showLoadingIndicator("body", true);
                    var payload = $scope.selectedHost;
                    if (payload.local !== undefined) {
                        delete payload.local;
                    }
                    if (payload.aliases_str !== null && payload.aliases_str !== undefined) {
                        if (payload.aliases_str === "") {
                            // delete alias properties
                            delete(payload.aliases_str);
                            delete(payload.aliases);
                        } else {
                            // trim any whitespace
                            payload.aliases_str = payload.aliases_str.replace(/\s/g, '');
                        }
                    }
                    if ($scope.isEdit && $scope.savedHost !== null) {
                        payload.original_ip = $scope.savedHost.ip;
                    }
                    payload = JSON.stringify(payload);
                    var resp = ($scope.isEdit) ? PHD.Ajax.put(url, payload, load, handleError) : PHD.Ajax.post(url, payload, load, handleError);
                    resp.done(function(data) {
                        $scope.closeEditDialog();
                        $scope.reloadHosts();
                    });
                }
            };

            function handleError(jqXHR, textStatus, errorThrown) {
                if (jqXHR.status === 500) {
                    var data = jqXHR.responseJSON;
                    if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                        var message = data.result[0].message;
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'edit-hosts-error-dialog',
                            scope: $scope,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    }
                } else {
                    PHD.ajaxError(jqXHR, textStatus, errorThrown);
                }
            }

            $scope.onConfirmError = function () {
                ngDialog.close('edit-hosts-error-dialog');
            };

            $scope.closeEditDialog = function() {
                // clear any validation errors for the form before closing.
                $("#form-errors").hide();
                $scope.selectedHost = $scope.savedHost;
                $scope.hostsDialog.wizard("close");
                // Must clear settings Dialog so it will open next time.
                $scope.hostsDialog = null;
            };

            $scope.closeHostsDialog = function() {
                // closing the wizard clears errors and state.
                $scope.PHD.currentMiniWizard.wizard("close");
            };

            $scope.editHost = function(mode) {

                $scope.isEdit = mode;

                /*
                 * Create a nested jQuery UI dialog for the host.  use jQueryUI because it tracks zIndex (and Edit Appliance dialog is jQuery UI).
                 */
                if ($scope.isEdit) {
                    if ($scope.selectedHost == null) {
                        return;
                    }
                    $scope.savedHost = $scope.selectedHost;
                    $scope.selectedHost = jQuery.extend({}, $scope.selectedHost);
                } else {
                    $scope.savedHost = $scope.selectedHost;
                    $scope.selectedHost = {};
                }

                if ($scope.hostsDialog == null) {
                    var options = {
                        title: ($scope.isEdit ? gettextCatalog.getString("Edit") : gettextCatalog.getString("Add")) + " " + gettextCatalog.getString("Host"),
                        helpArticle: PHD.App.getHelpLink("manage-hosts", true),
                        appendTo: "#dialogs",
                        width: 600,
                        height: 300,
                        beforeClose: function (event, ui) {
                            setTimeout(function(){
                                // Clear errors and state.
                                $("#form-errors").hide();
                                $scope.hostsDialog = null;
                                $scope.selectedHost = $scope.savedHost;
                            }, 0);
                        }
                    };
                    options = $.extend(PHD.dialogs.wizard, options);
                    var dialog = $("<div/>", { id: "dialog-edit-host", "class": "dialog-box" });
                    dialog.data("edit", false);
                    $("body").css("overflow", "hidden");
                    $scope.PHD.Ajax
                        .getHTML('app/configure/networks/hosts/host-edit.html')
                        .done(function(data) {
                            dialog.hide()
                                .appendTo("body")
                                .html(data)
                                .wizard(options);
                            var elements = angular.element(document.getElementById("advancedEditHostDialog"));
                            $compile(elements.contents())($scope);
                            $scope.$digest();
                            $scope.editForm = angular.element("#edit-host-form");
                            $scope.validateHostForm($scope.editForm);
                            $scope.hostsDialog = dialog;
                        });
                }
            };

            $scope.validateHostForm = function(form) {
                var IP_REGEXP = /\b(?:[0-9]{1,3}\.){3}[0-9]{1,3}\b/i;
                form.validate({
                    rules: {
                        name: {
                            required: true,
                            maxlength: $scope.MAX_HOST_LENGTH
                        },
                        ip: {
                            required: true,
                            pattern: IP_REGEXP
                        }
                    },
                    messages: {
                        name: {
                            required: gettextCatalog.getString("Hostname is required")
                        },
                        ip: {
                            required: gettextCatalog.getString("IP Address is required"),
                            pattern: gettextCatalog.getString("IP Address is not a valid address")
                        }
                    }
                })
            };

            $scope.confirmDeleteHost = function() {
                $scope.deleteTitle = gettextCatalog.getString('Confirm Host Removal');
                $scope.confirmMessage1 = gettextCatalog.getString('Are you sure want to delete this host?');
                $scope.confirmMessage2 = gettextCatalog.getString('If you delete the host for a registered asset, jobs will fail if its name cannot be resolved on the network.');
                $scope.confirmMessage3 = gettextCatalog.getString('I understand the risks of removing a host.');
                $scope.submitButtonText = gettextCatalog.getString('Delete Host');

                ngDialog.open({
                    template: 'app/common/confirmationDialog.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId: 'delete-host-dialog',
                    ngDialogStyle:'width:600px;height:250px',
                    closeByDocument: false,
                    closeByEscape: true
                });
            };


            // Required name for common delete function.
            $scope.closeThisDialog = function() {
                ngDialog.close("delete-host-dialog");
            };

            $scope.deleteAsset = function() {
                var url = "/api/hosts/";
                var sid ="/?sid=" + $scope.selectedID;
                var item = $scope.selectedHost.name;
                item = encodeURI(item);
                url += item + sid;
                var load = PHD.showLoadingIndicator("body", true);
                var resp = PHD.Ajax.delete(url, null, load, handleError);
                resp.done(function(data) {
                    $scope.reloadHosts();
                    $scope.closeThisDialog();
                });
            };

            $scope.reloadHosts = function() {
                var url = "/api/hosts/?sid=" + $scope.selectedID;
                var resp = PHD.Ajax.get(url);
                resp.done(function(data) {
                    $scope.selectedHost = null;
                    $scope.savedHost = null;
                    $scope.refresh = true;
                    $scope.gridData = data.data;
                    $scope.hostsGridData.data = $scope.gridData;
                    $timeout(function() {
                        $scope.refresh = false;
                    }, 0);
                });
            };

            $scope.hostsGridData = {
                multiSelect: false,
                selectionRowHeaderWidth: 35,
                rowHeight: 30,
                enableFullRowSelection : true,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                onRegisterApi: function (gridApi) {
                    $scope.gridApi = gridApi;
                    gridApi.selection.on.rowSelectionChanged($scope, function(rows){
                        var selectedItems = gridApi.selection.getSelectedRows();
                        if (selectedItems.length > 0) {
                            $scope.selectedHost = selectedItems[0];
                        } else if (selectedItems.length == 0) {
                            $scope.selectedHost = null;
                        }
                    });
                },
                data: []
            };

            $scope.hostsGridData.columnDefs = [
                {
                    field: "name",
                    displayName: gettextCatalog.getString("Hostname"),
                    sortable : true,
                    sortingType : "string",
                    sort: {
                        direction: uiGridConstants.ASC,
                        priority: 0
                    },
                    headerCellFilter: 'translate'
                },
                {
                    field: "ip",
                    displayName: gettextCatalog.getString("IP Address"),
                    sortable : true,
                    sortingType : "string",
                    headerCellFilter: 'translate'
                },
                {
                    field: "long_name",
                    displayName: gettextCatalog.getString("Qualified Name"),
                    sortable : true,
                    sortingType : "string",
                    headerCellFilter: 'translate'
                },
                {
                    field: "aliases_str",
                    displayName: gettextCatalog.getString("Aliases"),
                    sortable : false,
                    headerCellFilter: 'translate'
                }
            ];

            $scope.loadSettings = function() {
                $timeout(function() {
                    $scope.hostsGridData.data = $scope.gridData;
                });
            };

            // use instead of ngDialog load event.
            angular.element(document).ready(function () {
                $scope.loadSettings();
            });

        }]);