angular.module('advanced-ports-module', ['ngDialog', 'ui.grid','ui.grid.resizeColumns','ui.grid.autoResize','ngGrid','ui.grid.selection'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('portsSecurityCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants', 'gettextCatalog',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants, gettextCatalog) {

            $scope.refresh = false;
            $scope.osPassword = "";
            $scope.originalSecurity = "";
            $scope.osDialog = null;

            var DATAPORT = 1745;
            var MANAGER_PORT_DEFAULT = 11005;

            var SECURITY_LOW = 0;
            var SECURITY_MED = 1;
            var SECURITY_HIGH = 2;

            $scope.confirmChangePorts = function() {

                // If the same security level, just close and exit.
                if ($scope.security === $scope.originalSecurity) {
                    $scope.closeHostsDialog();
                    return;
                }

                if ($scope.osDialog === null) {
                    var options = {
                        title: gettextCatalog.getString("Enter OS Root Password"),
                        helpArticle: PHD.App.getHelpLink("ports-setting", true),
                        appendTo: "#dialogs",
                        width: $scope.security === "High" ? 500 : 400,
                        height: $scope.security === "High" ? 260 : 160,
                        beforeClose: function (event, ui) {
                            setTimeout(function(){
                                // Clear errors and state.
                                $("#form-errors").hide();
                                $scope.osDialog = null;
                            }, 0);
                        }
                    };
                    options = $.extend(PHD.dialogs.wizard, options);
                    var dialog = $("<div/>", { id: "dialog-ports-password", "class": "dialog-box" });
                    dialog.data("edit", false);
                    $("body").css("overflow", "hidden");
                    $scope.PHD.Ajax
                        .getHTML('app/configure/networks/ports/ports-edit.html')
                        .done(function(data) {
                            dialog.hide()
                                .appendTo("body")
                                .html(data)
                                .wizard(options);
                            var elements = angular.element(document.getElementById("osPasswordPortsDialog"));
                            $compile(elements.contents())($scope);
                            $scope.$digest();
                            $scope.editForm = angular.element("#edit-ports-form");
                            $scope.validatePassword($scope.editForm);
                            $scope.osDialog = dialog;
                        });
                }
            };

            $scope.saveSettings = function(commandName) {
                if ($scope.editForm !== null && $scope.editForm.valid()) {
                    var url = "/api/ports";
                    var load = PHD.showLoadingIndicator("body", true, gettextCatalog.getString("Updating Port Security Level....."));
                    var payload = {};
                    payload.security = $scope.security;
                    payload.password = $scope.password;
                    payload = JSON.stringify(payload);
                    var resp = PHD.Ajax.put(url, payload, load, handleError);
                    resp.done(function(data) {
                        $scope.closeEditDialog();
                        //$scope.setPortsGrid($scope.security);
                        //$scope.originalSecurity = $scope.security;
                        $scope.closeHostsDialog();
                    });
                }
            };

            function handleError(jqXHR, textStatus, errorThrown) {
                if (jqXHR.status === 500) {
                    var data = jqXHR.responseJSON;
                    if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                        var message = data.result[0].message;
                        if (message.indexOf('Invalid root password') !== -1) {
                            // clear the password field if invalid.
                            $scope.password = '';
                        }
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'edit-ports-error-dialog',
                            scope: $scope,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    }
                } else {
                    PHD.ajaxError(jqXHR, textStatus, errorThrown);
                }
            }

            $scope.onConfirmError = function () {
                ngDialog.close('edit-ports-error-dialog');
            };

            $scope.closeEditDialog = function() {
                // clear any validation errors for the form before closing.
                $("#form-errors").hide();
                $scope.osDialog.wizard("close");
                // Must clear settings Dialog so it will open next time.
                $scope.osDialog = null;
            };

            $scope.closeHostsDialog = function() {
                // clear any validation errors for the form before closing.
                $("#form-errors").hide();
                $scope.PHD.currentMiniWizard.wizard("close");
                // Must clear currentMiniWizard for the next time the dialog is loaded.
                $scope.PHD.currentMiniWizard = null;
            };

            $scope.validatePassword = function(form) {
                var IP_REGEXP = /\b(?:[0-9]{1,3}\.){3}[0-9]{1,3}\b/i;
                form.validate({
                    rules: {
                        password: {
                            required: true,
                            minlength: 1,
                            maxlength: MAX_PASSWORD_LENGTH
                        }
                    },
                    messages: {
                        required: {
                            required: gettext("The current OS root password is required")
                        }
                    }
                })
            };

            var portsTemplate='<div ng-class="{\'port-closed\':row.entity.open==false }">' +
                '<div ng-repeat="(colRenderIndex, col) in colContainer.renderedColumns track by col.colDef.name" ' +
                    'class="ui-grid-cell" ng-class="{ \'ui-grid-row-header-cell\': col.isRowHeader }" ui-grid-cell></div></div>';


            $scope.portsGridData = {
                multiSelect: false,
                selectionRowHeaderWidth: 35,
                rowHeight: 24,
                enableFullRowSelection : true,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                rowTemplate: portsTemplate,
                data: []
            };

            $scope.portsGridData.columnDefs = [
                {
                    field: "port",
                    displayName: gettext("Port"),
                    sortable : true,
                    type : "number",
                    sort: {
                        direction: uiGridConstants.ASC,
                        priority: 0
                    },
                    headerCellFilter: "translate"
                },
                {
                    field: "open",
                    displayName: gettext("Open"),
                    sortable : true,
                    type : "boolean",
                    headerCellFilter: "translate"
                },
                {
                    field: "service",
                    displayName: gettext("Service"),
                    sortable : true,
                    type : "string",
                    headerCellFilter: "translate"
                },
                {
                    field: "description",
                    displayName: gettext("Description"),
                    sortable : false,
                    headerCellFilter: "translate"
                }
            ];

            $scope.gridData = [
                {
                    port: 1,
                    open: true,
                    level: SECURITY_MED,
                    service: 'tcpmux',
                    description: gettextCatalog.getString('Vaulting setup')
                },
                {
                    port: 22,
                    open: true,
                    level: SECURITY_MED,
                    service: 'ssh',
                    description: gettextCatalog.getString('Secure shell')
                },
                {
                    port: 80,
                    open: true,
                    level: SECURITY_LOW,
                    service: 'http',
                    description: gettextCatalog.getString('HTTP web access')
                },
                {
                    port: 139,
                    open: true,
                    level: SECURITY_MED,
                    service: 'samba',
                    description: gettextCatalog.getString('Samba share')
                },
                {
                    port: 161,
                    open: true,
                    level: SECURITY_MED,
                    service: 'snmp',
                    description: gettextCatalog.getString('SNMP')
                },
                {
                    port: 443,
                    open: true,
                    level: SECURITY_MED,
                    service: 'https',
                    description: gettextCatalog.getString('Secure HTTP web access')
                },
                {
                    port: 445,
                    open: true,
                    level: SECURITY_MED,
                    service: 'cifs',
                    description: gettextCatalog.getString('CIFS')
                },
                {
                    port: 873,
                    open: true,
                    level: SECURITY_LOW,
                    service: 'rsync',
                    description: gettextCatalog.getString('Rsync')
                },
                {
                    port: 888,
                    open: true,
                    service: '3dm2',
                    level: SECURITY_LOW,
                    description: gettextCatalog.getString('3ware web access')
                },
                {
                    port: 1194,
                    open: true,
                    level: SECURITY_MED,
                    service: 'securetunnel',
                    description: gettextCatalog.getString('Secure Tunnel (OpenVPN)')
                },
                {
                    port: 1743,
                    open: true,
                    level: SECURITY_HIGH,
                    service: 'xinetd',
                    description: gettextCatalog.getString('Extended Internet daemon')
                },
                {
                    port: 1744,
                    open: true,
                    service: 'xinetd',
                    level: SECURITY_LOW,
                    description: gettextCatalog.getString('Extended Internet daemon')
                },
                {
                    port: 2049,
                    open: true,
                    level: SECURITY_LOW,
                    service: 'nfs',
                    description: gettextCatalog.getString('Network File System')
                },
                {
                    port: 3260,
                    open: true,
                    level: SECURITY_MED,
                    service: 'iSCSI',
                    description: gettextCatalog.getString('iSCSI')
                },
                {
                    port: 4970,
                    open: true,
                    level: SECURITY_MED,
                    service: 'postgres',
                    description: gettextCatalog.getString('Postgres database access')
                },
                {
                    port: 5432,
                    open: true,
                    level: SECURITY_MED,
                    service: 'postgres',
                    description: gettextCatalog.getString('Postgres database access')
                },
                {
                    port: 5801,
                    open: true,
                    level: SECURITY_LOW,
                    service: 'vnc(java)',
                    description: gettextCatalog.getString('VNC (Java) access')
                },
                {
                    port: 5900,
                    open: true,
                    level: SECURITY_LOW,
                    service: 'vnc',
                    description: gettextCatalog.getString('VNC access')
                },
                {
                    port: 5902,
                    open: true,
                    level: SECURITY_LOW,
                    service: 'vnc',
                    description: gettextCatalog.getString('VNC access')
                },
                {
                    port: 6001,
                    open: true,
                    level: SECURITY_LOW,
                    service: 'vnc-http',
                    description: gettextCatalog.getString('VNC HTTP web access')
                },
                {
                    port: 10000,
                    open: true,
                    level: SECURITY_MED,
                    service: 'ndmp',
                    description: gettextCatalog.getString('NDMP daemon control port')
                },
                {
                    port: 55404,
                    open: true,
                    level: SECURITY_HIGH,
                    service: 'elk',
                    description: gettextCatalog.getString('Telemetry Service')
                },
                {
                    port: 59200,
                    open: true,
                    level: SECURITY_HIGH,
                    service: 'elk',
                    description: gettextCatalog.getString('Telemetry Service')
                }
            ];

            $scope.onChangeSecurity = function() {
                $scope.setPortsGrid($scope.security, $scope.dataport_count);
            };

            $scope.setPortsGrid = function(securityLevel, dataport_count) {
                switch (securityLevel) {
                    case "None":
                    case "Low":
                        $scope.setOpen(SECURITY_LOW);
                        break;
                    case "Med":
                        $scope.setOpen(SECURITY_MED);
                        break;
                    case "High":
                        $scope.setOpen(SECURITY_HIGH);
                        break;
                }
            };

            $scope.setOpen = function(level) {
                for (var i = 0; i < $scope.portsGridData.data.length; i++) {
                    var port = $scope.portsGridData.data[i];
                    port.open = (port.level >= level);
                }
            };

            $scope.loadSettings = function() {
                $timeout(function() {
                    $scope.originalSecurity = $scope.security;
                    $scope.portsGridData.data = $scope.gridData;

                    $scope.portsMessage = gettextCatalog.getString("All other ports") + " ";
                    if ($scope.security === 'None') {
                        $scope.portsMessage += gettextCatalog.getString('including') + " " + gettextCatalog.getString("those listed above are") + " " + gettextCatalog.getString("open");
                    } else {
                        $scope.portsMessage += gettextCatalog.getString('except') + " " + gettextCatalog.getString("those listed above are") + " " + gettextCatalog.getString("closed");
                    }

                    // Add data ports based on how many are open.
                    $scope.dataPorts = $scope.dataport_count !== undefined ? $scope.dataport_count : 0;
                    var obj = {
                        open: true,
                        level: SECURITY_HIGH,
                        service: 'xinetd',
                        description: gettextCatalog.getString('Extended Internet daemon')
                    };
                    for (var i = 0; i < $scope.dataPorts; i++ ){
                        var dataportObj = jQuery.extend({}, obj);
                        dataportObj.port = DATAPORT + i;
                        $scope.portsGridData.data.push(dataportObj);
                    }

                    if ($rootScope.isNetworkManager) {
                        var portCount = $scope.managerport_count !== undefined ? $scope.managerport_count : 0;
                        var portStart = $scope.managerport_start !== undefined ? $scope.managerport_start : MANAGER_PORT_DEFAULT;
                        var managerObj = {
                            open: true,
                            level: SECURITY_LOW,
                            service: 'xg-portal',
                            description: gettextCatalog.getString('Distributed Enterprise Manager')
                        };
                        for (var j = 0; j < portCount; j++ ){
                            var managerPortObj = jQuery.extend({}, managerObj);
                            managerPortObj.port = portStart + j;
                            $scope.portsGridData.data.push(managerPortObj);
                        }
                        var amqpsObj = {
                            port: 5671,
                            open: true,
                            level: SECURITY_MED,
                            service: 'amqps',
                            description: gettextCatalog.getString('RabbitMQ AMQPS')
                        };
                        $scope.portsGridData.data.push(amqpsObj);
                        var amqpObj = {
                            port: 5672,
                            open: true,
                            level: SECURITY_LOW,
                            service: 'amqp',
                            description: gettextCatalog.getString('RabbitMQ AMQP')
                        };
                        $scope.portsGridData.data.push(amqpObj);
                    }
                    $scope.setPortsGrid($scope.security);
                });
            };

            // use instead of ngDialog load event.
            angular.element(document).ready(function () {
                $scope.loadSettings();
            });

        }]);