angular.module('advanced-snmp-module', ['ngDialog', 'ui.grid','ui.grid.resizeColumns','ui.grid.autoResize','ngGrid','ui.grid.selection'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .filter('true_or_false', function() {
        return function (text) {
            return (text) ? "true" : "false";
        }
    })
    .controller('SNMPCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants', 'DownloadService',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants, DownloadService) {

            $scope.editDialog = null;
            $scope.selectedItem = null;
            $scope.savedItem = null;  // save host to restore on cancel
            $scope.MAX_SNMP_LENGTH = 256;
            $scope.refresh = false;
            $scope.editForm = null;

            $scope.saveSettings = function(commandName) {
                if ($scope.editForm != null && $scope.editForm.valid()) {
                    var resp;
                    var url = "/api/traps/destinations";
                    var sid = "/?sid=" + $scope.selectedID;
                    var load = PHD.showLoadingIndicator("body", true);
                    // If there isn't a value for enabled, then set it to false.  UNIBP-11663 This probably is that the user has not clicked the checkbox, so wants it to be false.
                    if ($scope.isEdit == false && $scope.selectedItem.hasOwnProperty('enabled') == false) {
                        $scope.selectedItem.enabled = 0;
                    }
                    var payload = $scope.selectedItem;
                    payload = JSON.stringify(payload);
                    if ($scope.isEdit) {
                        url += "/" + $scope.selectedItem.id + sid;
                        resp = PHD.Ajax.put(url, payload, load, handleError)
                    } else {
                        url += sid;
                        resp = PHD.Ajax.post(url, payload, load, handleError);
                    }
                    resp.done(function(data) {
                        $scope.closeEditDialog();
                        $scope.reloadGrid();
                    });
                }
            };

            function handleError(jqXHR, textStatus, errorThrown) {
                if (jqXHR.status === 500) {
                    var data = jqXHR.responseJSON;
                    if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                        var message = data.result[0].message;
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'edit-snmp-error-dialog',
                            scope: $scope,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    }
                } else {
                    PHD.ajaxError(jqXHR, textStatus, errorThrown);
                }
            }

            $scope.onConfirmError = function () {
                ngDialog.close('edit-snmp-error-dialog');
            };

            $scope.closeEditDialog = function() {
                // closing the edit wizard clears errors and state.
                $scope.editDialog.wizard("close");
            };

            $scope.closeSNMPDialog = function() {
                // closing the wizard clears errors and state.
                $scope.PHD.currentMiniWizard.wizard("close");
            };

            $scope.editHost = function(mode) {

                $scope.isEdit = mode;

                /*
                 * Create a nested jQuery UI dialog for the host.  use jQueryUI because it tracks zIndex (and Edit Appliance dialog is jQuery UI).
                 */
                if ($scope.isEdit) {
                    if ($scope.selectedItem == null) {
                        return;
                    }
                    $scope.savedItem = $scope.selectedItem;
                    $scope.selectedItem = jQuery.extend({}, $scope.selectedItem);
                } else {
                    $scope.selectedItem = {};
                }

                if ($scope.editDialog == null) {
                    var options = {
                        title: ($scope.isEdit ? "Edit" : "Add") + " SNMP Destination",
                        helpArticle: PHD.App.getHelpLink("manage-snmp", true),
                        appendTo: "#dialogs",
                        width: 600,
                        height: 230,
                        beforeClose: function (event, ui) {
                            setTimeout(function(){
                                // Clear state.
                                $scope.editDialog = null;
                                $scope.selectedItem = $scope.savedItem;
                            }, 0);
                        }
                    };
                    options = $.extend(PHD.dialogs.wizard, options);
                    var dialog = $("<div/>", { id: "dialog-edit-snmp-host", "class": "dialog-box" });
                    dialog.data("edit", false);
                    $("body").css("overflow", "hidden");
                    $scope.PHD.Ajax
                        .getHTML('app/configure/notifications/snmp/snmp-dest-edit.html')
                        .done(function(data) {
                            dialog.hide()
                                .appendTo("body")
                                .html(data)
                                .wizard(options);
                            var elements = angular.element(document.getElementById("advancedEditSNMPDialog"));
                            $compile(elements.contents())($scope);
                            $scope.$digest();
                            $scope.editForm = angular.element("#edit-snmp-form");
                            $scope.validateForm($scope.editForm, $scope.isEdit);
                            $scope.editDialog = dialog;
                        });
                }
            };

            $scope.validateForm = function(form, isEdit) {
                var options = {
                    rules: {
                        destination: {
                            required: true,
                            maxlength: $scope.MAX_SNMP_LENGTH
                        },
                        community: {
                            required: isEdit == 1,
                            maxlength: $scope.MAX_SNMP_LENGTH
                        }
                    },
                    messages: {
                        destination: {
                            required: gettext("SNMP trap destination is required")
                        },
                        community: {
                            required: gettext("SNMP community name is required")
                        }
                    }
                };
                $rootScope.validateNestedDialog(form, options);
            };

            $scope.editAgent = function(mode) {

                $scope.savedAgent = $scope.agent;
                $scope.agent = jQuery.extend({}, $scope.agent);

                if ($scope.editDialog == null) {
                    var options = {
                        title: "Edit SNMP Agent",
                        helpArticle: PHD.App.getHelpLink("manage-snmp", true),
                        appendTo: "#dialogs",
                        width: 600,
                        height: 200,
                        beforeClose: function (event, ui) {
                            setTimeout(function(){
                                // Clear state.
                                $scope.editDialog = null;
                                $scope.agent = $scope.savedAgent;

                            }, 0);
                        }
                    };
                    options = $.extend(PHD.dialogs.wizard, options);
                    var dialog = $("<div/>", { id: "dialog-edit-agent", "class": "dialog-box" });
                    dialog.data("edit", false);
                    $("body").css("overflow", "hidden");
                    $scope.PHD.Ajax
                        .getHTML('app/configure/notifications/snmp/snmp-agent-edit.html')
                        .done(function(data) {
                            dialog.hide()
                                .appendTo("body")
                                .html(data)
                                .wizard(options);
                            var elements = angular.element(document.getElementById("advancedAgentSNMPDialog"));
                            $compile(elements.contents())($scope);
                            $scope.$digest();
                            $scope.editForm = angular.element("#edit-snmp-form");
                            $scope.validateAgentForm($scope.editForm);
                            $scope.editDialog = dialog;
                        });
                }
            };

            $scope.validateAgentForm = function(form) {
                var options = {
                    rules: {
                        community: {
                            required: true,
                            maxlength: $scope.MAX_SNMP_LENGTH
                        }
                    },
                    messages: {
                        community: {
                            required: gettext("SNMP community name is required")
                        }
                    }
                };
                $rootScope.validateNestedDialog(form, options);
            };

            $scope.confirmDelete = function() {
                $scope.deleteTitle = 'Confirm Trap Destination Removal';
                $scope.confirmMessage1 = 'Are you sure want to delete SNMP destination ' + $scope.selectedItem.destination + '?';
                $scope.confirmMessage2 = '';
                $scope.confirmMessage3 = 'I understand that traps will no longer be sent to this destination.';
                $scope.submitButtonText = 'Delete Destination';

                ngDialog.open({
                    template: 'app/common/confirmationDialog.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId: 'delete-dest-dialog',
                    ngDialogStyle:'width:600px;height:250px',
                    closeByDocument: false,
                    closeByEscape: true
                });
            };


            // Required name for common delete function.
            $scope.closeThisDialog = function() {
                ngDialog.close("delete-dest-dialog");
            };

            $scope.deleteAsset = function() {
                var url = "/api/traps/destinations/";
                var sid ="/?sid=" + $scope.selectedID;
                var item = $scope.selectedItem.id;
                item = encodeURI(item);
                url += item + sid;
                var load = PHD.showLoadingIndicator("body", true);
                var resp = PHD.Ajax.delete(url, null, load, handleError);
                resp.done(function(data) {
                    $scope.reloadGrid();
                    $scope.closeThisDialog();
                });
            };

            $scope.reloadGrid = function() {
                var url = "/api/traps/destinations/?sid=" + $scope.selectedID;
                var resp = PHD.Ajax.get(url);
                resp.done(function(data) {
                    $scope.selectedItem = null;
                    $scope.savedItem = null;
                    $scope.refresh = true;
                    $scope.gridData = data.data;
                    $scope.snmpGridData.data = $scope.gridData;
                    $timeout(function() {
                        $scope.refresh = false;
                    }, 0);
                });
            };

            $scope.snmpGridData = {
                multiSelect: false,
                selectionRowHeaderWidth: 35,
                rowHeight: 30,
                enableFullRowSelection : true,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                onRegisterApi: function (gridApi) {
                    $scope.gridApi = gridApi;
                    gridApi.selection.on.rowSelectionChanged($scope, function(rows){
                        var selectedItems = gridApi.selection.getSelectedRows();
                        if (selectedItems.length > 0) {
                            $scope.selectedItem = selectedItems[0];
                        } else if (selectedItems.length == 0) {
                            $scope.selectedItem = null;
                        }
                    });
                },
                data: []
            };

            var enabledTemplate = '<div class="ui-grid-cell-contents">{{ row.entity.enabled | true_or_false }}</div>';


            $scope.snmpGridData.columnDefs = [
                {
                    field: "id",
                    displayName: "",
                    visible: false,
                    sortable : true,
                    type : "number",
                    sort: {
                        direction: uiGridConstants.ASC,
                        priority: 0
                    }
                },
                {
                    field: "destination",
                    displayName: "Address",
                    sortable : true,
                    type : "string"
                },
                {
                    field: "community",
                    displayName: "Community",
                    sortable : true,
                    type : "string"
                },
                {
                    field: "enabled",
                    displayName: "Enabled",
                    cellTemplate: enabledTemplate,
                    sortable : true,
                    type : "boolean"
                }
            ];

            $scope.loadSettings = function() {
                $timeout(function() {
                    $scope.snmpGridData.data = $scope.gridData;
                });
            };

            $scope.loadAgent = function() {
                var url = "/api/snmpd/?sid=" + $scope.selectedID;
                var resp = PHD.Ajax.get(url);
                resp.done(function(data) {
                    $scope.agent = data.data;
                });
            };

            $scope.saveAgent = function() {
                if ($scope.editForm != null && $scope.editForm.valid()) {
                    var url = "/api/snmpd/?sid=" + $scope.selectedID;
                    var load = PHD.showLoadingIndicator("body", true);
                    var payload = $scope.agent;
                    payload = JSON.stringify(payload);
                    var resp = PHD.Ajax.put(url, payload, load, handleError);
                    resp.done(function(data) {
                        $scope.closeEditDialog();
                        $scope.loadAgent();
                    });
                }
            };

            $scope.sendTestTrap = function() {
                var url = "/api/traps/test/?sid=" + $scope.selectedID;
                var load = PHD.showLoadingIndicator("body", true, "Sending trap.....");
                var resp = PHD.Ajax.put(url, null, load, handleError);
                resp.done(function(data) {
                    ngDialog.open({
                        dialogType: 'Success',
                        dialogMessage: "An SNMP test trap has been successfully sent to all enabled destinations."
                    });
                });
            };

            $scope.downloadMIB = function() {
                var uri = '/snmp/unitrends.mib';
                var filename = 'unitrends.mib';
                DownloadService.saveAs(uri, filename);
            };

            // use instead of ngDialog load event.
            angular.element(document).ready(function () {
                $scope.loadSettings();
                $scope.loadAgent();
            });

        }]);