angular.module('os-password-module', ['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            trapFocus: true,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('osPasswordCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', function($scope, $rootScope, $http, ngDialog, $window, $timeout) {

        $scope.form = $();
        $scope.os = {};
        $scope.os.root_password = null;
        $scope.os.password = null;

        $scope.validateForm = function(form) {
            form.validate({
                rules: {
                    currentPassword: {
                        required: true,
                        minlength: 1,
                        maxlength: MAX_PASSWORD_LENGTH
                    },
                    newPassword: {
                        required: true,
                        minlength: 1,
                        maxlength: MAX_PASSWORD_LENGTH
                    },
                    newPasswordConfirm: {
                        required: true,
                        minlength: 1,
                        maxlength: MAX_PASSWORD_LENGTH,
                        equalTo: "#newPassword"
                    }
                },
                messages: {
                    currentPassword: {
                        required: gettext("The current OS root password is required")
                    },
                    newPassword: {
                        required: gettext("A new OS root password is required"),
                        maxlength: gettext("The new password must not be more than 64 characters")
                    },
                    newPasswordConfirm: {
                        required: gettext("The new OS root password confirmation is required"),
                        equalTo: gettext("Your new password must be entered the same in both fields")
                    }
                }
            })
        };

        $scope.saveSettings = function(commandName) {
            if ($scope.form.valid()) {
                var url = "/api/systems/os-password/";
                var load = PHD.showLoadingIndicator("body", true);
                var dimensions = 'width:650px; height:475px;';
                var payload = $scope.os;
                payload = JSON.stringify(payload);
                var resp = PHD.Ajax.put(url, payload, load, handleError);
                resp.done(function(data) {
                    $scope.closeThisDialog();
                });
            }
        };

        function handleError(jqXHR, textStatus, errorThrown) {
            if (jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var message = data.result[0].message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'os-password-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.onConfirmError = function () {
            ngDialog.close('os-password-dialog');
        };

        $scope.closeThisDialog = function() {
            // closing the wizard clears errors and state.
            $scope.PHD.currentMiniWizard.wizard("close");
        };

        $scope.processDialog = function() {
            //$scope.showADTooltip();

            $scope.form = angular.element("#os-password-advanced-form");
            $scope.validateForm($scope.form);
        };

        // use instead of ngDialog load event.
        angular.element(document).ready(function () {
            $scope.processDialog();
        });

    }]);