angular.module('configAssets', ['ngDialog', 'angulartics', 'angulartics.google.analytics', 'treeGrid',
        'ui.grid', 'ui.grid.resizeColumns', 'ui.grid.treeView', 'ui.grid.selection'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])

    .controller('AssetsCtrl', ['$scope', '$rootScope', '$http', '$timeout', 'ngDialog', '$analytics', 'gettextCatalog', 'AssetService', 'localStorageService', 'sortsvc',
        function($scope, $rootScope, $http, $timeout, ngDialog, $analytics, gettextCatalog, AssetService, localStorageService, sortsvc) {

        $scope.initCtrl = function () {
            $scope.Assets = {};
            processAssets(PHD, window);
        };

        $scope.COLUMN_COOKIE = 'p-assets';

        $scope.button_allowed = true;
        $scope.listViewFlag =false;
        $scope.tableView = true;
        $scope.updateAgentMessage='';
        $scope.getView = function(tabular) {
            var view = tabular || $scope.tableView;
            return (view  ===true ? gettext("Table") : gettext("List"));
        };

        $scope.toggleView = function() {
            $scope.selectedObj = null;
            PHD.appliance_sid = $rootScope.local.id;

            $scope.tableView = !$scope.tableView;

            if(!$scope.tableView){
                $scope.selectedAssets = $scope.protectedAssetsDataArr[0];
                if ($scope.selectedAssets !== undefined) {
                    $scope.selectedID = $scope.selectedAssets.id;
                } else {
                    $scope.selectedID = null;
                }
                $scope.selectProtectedAssetsFromList($scope.selectedID);
                $("#" + ($scope.selectedID)).css("background-color","#f5f5f5");
            }else{
                ($(".environment-assets").find(".assets-table")).load($scope.protectedAssetsDataArr);
                ($(".environment-assets").find(".assets-table")).resize();
                if ($scope.selectedArray.length >= 1) {
                    $scope.selectedObj = $scope.selectedArray[0];
                    PHD.appliance_sid = $scope.selectedObj.system_id;
                } else {
                    $scope.selectedObj = null;
                    $scope.selectedID = null;
                }
            }
        };

        $scope.display = ASSET_DISPLAY_FILTER_ALL;
        $scope.getDisplay = function(view) {
            return $scope.display;
        };

        $scope.launchRetentionManager = function() {
            var resp = PHD.Ajax.get("/api/retention/strategy/" + "?sid=" +  PHD.appliance_sid);
            resp.done(function(data){
                var result = data.data;
                var strategy = result.strategy;
                if (strategy == 'MinMax') {
                    ngDialog.open({
                        template: 'app/configure/retention/min-max.html',
                        overlay: true,
                        name: 'MinMaxRetention',
                        ngDialogStyle:'width:700px;',
                        closeByDocument: false,
                        closeByEscape: false
                    });
                } else {
                    ngDialog.open({
                        template: 'app/configure/retention/retentionpolicy.html',
                        scope: $scope,
                        overlay:true,
                        name: 'Retention-Policy',
                        ngDialogStyle:'width:300px; height:400px;',
                        // GFS TODO - enlarge dialog once we have weekly/monthly/yearly settings
                        //ngDialogStyle:'width:700px; height:400px;',
                        closeByDocument: false,
                        closeByEscape: false
                    });
                }
            });

        };

        $scope.getIcon = function(item) {
            return AssetService.getIconForTypeString(item);
        };

        var nameTemplate = '<div class="ui-grid-cell-contents" title="{{row.entity.name}}" '+
                                'ng-style="{ \'margin-left\': (row.treeLevel * 20 + 2) + \'px\' }">' +
                                '<i style="margin-right:4px" class="{{grid.appScope.getIcon(row.entity)}} icon"></i>{{row.entity.name}}' +
                            '</div>';

        $scope.tree_data = [];
        $scope.savedState = undefined;
        $scope.columnOrderRestored = false;

        $scope.treeGridOptions = {
            data: 'tree_data',
            enableSorting: true,
            enableFiltering: true,
            showTreeExpandNoChildren: false,
            treeRowHeaderBaseWidth: 20,
            enableFullRowSelection : false,
            enableRowHeaderSelection: true,
            selectionRowHeaderWidth: 28,
            rowHeight: 28,
            enableColumnResizing: true,
            enableHorizontalScrollbar: 0,
            multiSelect: true,
            enableGridMenu: true,
            gridMenuShowHideColumns: true,
            enableColumnMenus: true,
            enableSelectAll: false,
            enableExpandAll: true,
            gridMenuCustomItems: [{
                title: gettextCatalog.getString('Reset column defaults'),
                action: function($event) {
                    // clear the current grids, then re-draw as the original.
                    $scope.restoreGridState();
                },
                order: 99
            }],
            gridMenuTitleFilter: function(title) {
                return gettextCatalog.getString(title);
            },
            columnDefs: [
                {
                    name: 'name',
                    displayName: gettextCatalog.getString('NAME'),
                    width: '18%',
                    cellTemplate: nameTemplate
                },
                {
                    name: 'ip',
                    displayName: gettextCatalog.getString('ADDRESS'),
                    cellTooltip: true,
                    sortingAlgorithm: sortsvc.ipSortSeparate
                },
                {
                    name: 'description',
                    displayName: gettextCatalog.getString('DESCRIPTION'),
                    cellTooltip: true
                },
                {
                    name: 'credential_display',
                    displayName: gettextCatalog.getString('CREDENTIALS'),
                    cellTooltip: function(row, col) {
                        return (row.entity.credential_display === '(Unnamed)') ?
                            'Unnamed credentials are only for this asset and cannot be assigned to others.' :
                            row.entity.credential_display;
                    }
                },
                {
                    name: 'retention',
                    displayName: gettextCatalog.getString('RETENTION'),
                    cellTooltip: true
                },
                {
                    name: 'is_encrypted',
                    displayName: gettextCatalog.getString('ENCRYPTED'),
                    type: "boolean"
                },
                {
                    name: 'version',
                    displayName: gettextCatalog.getString('AGENT VERSION'),
                    cellTooltip: true
                },
                {
                    name: 'system_name',
                    displayName: gettextCatalog.getString('APPLIANCE'),
                    cellTooltip: true,
                    width: '*',
                    minWidth: 50
                },
                {
                    name: 'id',
                    displayName: gettextCatalog.getString('ID'),
                    cellTooltip: true,
                    visible: false
                },
                {
                    name: 'machine_type',
                    displayName: gettextCatalog.getString('MACHINE'),
                    visible: false,
                    cellFilter: 'nullTestFilter'
                }
            ],
            onRegisterApi: function (gridApi) {
                $scope.onRegisterGrid(gridApi);
            }
        };

        $scope.onRegisterGrid = function(gridApi) {
            $scope.gridApi = gridApi;
            /*$scope.gridApi.treeBase.on.rowExpanded($scope, function(row) {
                console.log('expand');
            });*/
            $scope.gridApi.core.on.rowsRendered($scope, function() {
                if (!$scope.stateExists()) {
                    $scope.saveGridState();
                    $scope.restorePersistedState($scope.gridApi);
                }
            });
            // configure handler for row selection changes
            $scope.gridApi.selection.on.rowSelectionChanged($scope, $scope.handleRowSelection);
            $scope.gridApi.selection.on.rowSelectionChangedBatch($scope, $scope.handleRowSelection);

            // configure handler to persist grid state changes
            gridApi.colMovable.on.columnPositionChanged($scope, $scope.persistGridState);
            gridApi.colResizable.on.columnSizeChanged($scope, $scope.persistGridState);
            gridApi.core.on.columnVisibilityChanged($scope, $scope.persistGridState);
            gridApi.core.on.filterChanged($scope, $scope.persistGridState);
            gridApi.core.on.sortChanged($scope, $scope.persistGridState);
        };

        /*
         * Returns true if a cached state exists for the tab, false if not.
         * Checks for the existence of columns because these may be delayed (returned after API returns).
         */
        $scope.stateExists = function() {
            return ($scope.savedState !== undefined && $scope.savedState.columns.length > 0);
        };

        /*
         * Use ui-grid state to save information in an array about the original grid: column order, width, filters, etc.
         */
        $scope.saveGridState = function() {
            $scope.savedState = $scope.gridApi.saveState.save();
        };

        /*
         * Save state of the grid in local storage: here the state is defined by the UI-grid.
         */
        $scope.persistGridState = function() {
            localStorageService.set($scope.COLUMN_COOKIE, $scope.gridApi.saveState.save());
        };

        /*
         * Restore the grid's state from the local storage for this tab, if it exists.
         */
        $scope.restorePersistedState = function(gridApi, tabIndex) {
            if (!$scope.columnOrderRestored) {
                var token = $scope.COLUMN_COOKIE;
                var assetState = localStorageService.get(token);
                if (assetState !== null && assetState !== undefined) {
                    gridApi.saveState.restore($scope, assetState);
                    $scope.columnOrderRestored = true;
                }
            }
        };

        /*
         * Restore this grid's state from the array (the original tabs), and delete the local storage.
         */
        $scope.restoreGridState = function() {
            $scope.gridApi.saveState.restore($scope, $scope.savedState);
            setTimeout(function () {
                var token = $scope.COLUMN_COOKIE;
                localStorageService.remove(token);
            }, 100);
            $scope.columnOrderRestored = false;
        };

        $scope.handleRowSelection = function(row) {
            $scope.selectedArray = $scope.gridApi.selection.getSelectedRows();
            if ($scope.selectedArray.length >= 1) {
                $scope.selectedObj = $scope.selectedArray[0];
                PHD.appliance_sid = $scope.selectedObj.system_id;
            } else if ($scope.selectedArray.length === 0) {
                $scope.selectedObj = null;
                $scope.selectedID = null;
                PHD.appliance_sid = $rootScope.local.id;
            }
            $scope.Assets.updateButtons();
        };

        $scope.updateTreeGrid = function(protectedAssets) {
            $scope.tree_data = [];
            writeoutNode(protectedAssets, 0, $scope.tree_data);
            $scope.$apply();
        };

        var writeoutNode = function( childArray, currentLevel, dataArray ){
            childArray.forEach( function( childNode ){
                if (childNode.children !== undefined) {
                    if (childNode.children.length >= 0 ){
                        childNode.$$treeLevel = currentLevel;
                    }
                    dataArray.push(childNode);
                    writeoutNode( childNode.children, currentLevel + 1, dataArray );
                } else {
                    childNode.$$treeLevel = currentLevel;
                    childNode.children = [];
                    dataArray.push(childNode);
                }
            });
        };

        $scope.selectedArray = [];

        $scope.displayAll = function() {
            $scope.display = ASSET_DISPLAY_FILTER_ALL;
            $scope.protectedAssetsDataArr = $scope.protectedAssetsAllDataArr;
            $timeout(function() {
                $scope.updateTreeGrid($scope.protectedAssetsDataArr);
            });
        };

        $scope.displayVirtual = function() {
            $scope.display = ASSET_DISPLAY_FILTER_VIRTUAL;

            var virtualassets = ($scope.protectedAssetsAllDataArr).filter(function(obj){
                var isHyperV = false;
                if(obj.children) {
                    for (var i = 0; i < obj.children.length; i++) {
                        if (obj.children[i].type === "Hyper-V") {
                            isHyperV = true;
                            break;
                        }
                    }
                }
                return obj.asset_type && (obj.asset_type != ASSET_TYPE_PHYSICAL
                    || (obj.asset_type === ASSET_TYPE_PHYSICAL && (obj.is_hyperv_cluster || isHyperV) ));
            });

            $scope.protectedAssetsDataArr = virtualassets;
            $timeout(function() {
                $scope.updateTreeGrid($scope.protectedAssetsDataArr);
            });
        };

        $scope.displayPhysical = function() {
            $scope.display = ASSET_DISPLAY_FILTER_PHYSICAL;
            var physicalassets = ($scope.protectedAssetsAllDataArr).filter(function(obj){
                return obj.asset_type && obj.asset_type === ASSET_TYPE_PHYSICAL;
            });

            $scope.protectedAssetsDataArr = physicalassets;
            $timeout(function() {
                $scope.updateTreeGrid($scope.protectedAssetsDataArr);
            });
        };

        $scope.launchCredentialManager = function(followFunction) {
            ngDialog.open({
                template: 'app/configure/credentials/credentials.html',
                scope: $scope,
                overlay:true,
                name: 'Credential-Manager',
                modelDialogId : 'credential-manager-dialog',
                ngDialogStyle:'width:700px; height:550px;',
                closeByDocument: false,
                closeByEscape: false,
                preCloseCallback: function (value) {
                    if ( followFunction !== undefined ) {
                        followFunction();
                    }
                }
            });
        };

        $scope.launchGlobalVMSettingsManager = function(followFunction) {
            ngDialog.open({
                template: 'app/configure/global-vm-settings/global-vm-settings.html',
                scope: $scope,
                overlay:true,
                name: 'Global-VM-Settings-Manager',
                id:'globalVMSettingsDialog',
                ngDialogStyle:'width:700px; height:490px;',
                closeByDocument: false,
                closeByEscape: true,
                preCloseCallback: function (value) {
                    if ( followFunction !== undefined ) {
                        followFunction();
                    }
                }
            });
        };

        $scope.launchAssetManager = function() {
            if(!$scope.button_allowed){                 //Set in updateButtons() function
                return false;
            }
            var assets;
            if($scope.tableView) {
                assets = $scope.selectedArray;
            } else {
                assets = [$scope.selectedAssets];
            }
            if (assets !== undefined) {
                $scope.copyEdit = false;
                if (assets.length > 1) {
                    console.log("selected > 1 asset");
                    ngDialog.open({
                        template: 'app/configure/environments/edit-multiple-assets.html',
                        scope: $scope,
                        overlay:true,
                        name: 'Edit-Assets',
                        data: assets,
                        ngDialogStyle:'width:600px;',
                        closeByDocument: false,
                        closeByEscape: true
                    });
                } else if (assets.length === 1) {
                    var asset = assets[0];
                    console.log("asset type:" + asset.asset_type);
                    var title;
                    if(asset.name.endsWith(NAS_POSTFIX) || asset.os_type == ASSET_TYPE_NDMP) {
                        title = "Edit NAS";
                        $scope.assetList=asset;
                        $analytics && $analytics.eventTrack(title, {  category: 'Configure', label: 'Protected Assets' });
                        ngDialog.open({
                            template: 'app/configure/environments/addEditNas.html',
                            scope: $scope,
                            overlay:true,
                            data:asset,
                            name: 'add-edit-nas-dialog',
                            modelDialogId:'add-edit-nas-dailog',
                            ngDialogStyle:'width:600px;',
                            ngDialogPostionStyle:'top:40px;',
                            preCloseCallback: function (value) {}

                        });
                        $scope.isEdit=true;
                        if(asset.os_type == ASSET_TYPE_NDMP) {
                            $scope.isNDMP= true;
                        } else {
                            $scope.isNDMP = false;
                        }
                        associatedAppliance = asset.system_id;
                        serverEdit = true;
                        //EditDisabled = true;
                    } else if(asset.asset_type === ASSET_TYPE_VMWARE_HOST ||
                        asset.asset_type === ASSET_TYPE_XEN ||
                        asset.asset_type === ASSET_TYPE_AHV){
                        ngDialog.open({
                            template: 'app/configure/environments/add-edit-hypervisor.html',
                            scope: $scope,
                            overlay:true,
                            data: assets,
                            name: 'Edit-Hypervisor',
                            ngDialogStyle: 'width:600px;',
                            ngDialogPostionStyle:'top:40px;',
                            closeByDocument: false,
                            closeByEscape: true
                        });


                    } else if (asset.asset_type == ASSET_TYPE_PHYSICAL) {
                        ngDialog.open({
                            template: 'app/configure/environments/EditAsset.html',
                            scope: $scope,
                            overlay:true,
                            name: 'Edit-Server',
                            data: asset,
                            ngDialogStyle:'width:600px;top:-5em;',
                            closeByDocument: false,
                            closeByEscape: true
                        });
                    } else {
                        console.log("minimal asset dialog for apps");
                        ngDialog.open({
                            template: 'app/configure/environments/edit-multiple-assets.html',
                            scope: $scope,
                            overlay:true,
                            name: 'Edit-Assets',
                            data: assets,
                            ngDialogStyle:'width:600px;',
                            ngDialogPostionStyle:'top:40px;',
                            closeByDocument: false,
                            closeByEscape: true
                        });
                    }
                }
            }
            return true;
        };

        $scope.installAgent = function() {
            var selected = $scope.Assets.selectedArray;
            var confirm = PHD.confirmDialog(
                "<p>"+ gettextCatalog.getString("Are you sure you want to push an agent to the selected assets?")+"</p>",
                { title: gettextCatalog.getString("Confirm Agent Push") }
            );

            confirm.done(function() {
                selected.forEach(function(asset, index) {
                    var url = "/api/clients/agent-push";
                    var data = [];
                    obj = {};
                    obj.id = asset.id;
                    obj.system_id = asset.system_id;
                    obj.address = asset.ip;
                    obj.credential_id = asset.credential_id;
                    data[0] = obj;
                    var clientArray = {};
                    clientArray.clients = data;
                    data = JSON.stringify(clientArray);
                    PHD.Ajax.put(url, data)
                        .done(function (data) {
                            ;
                        });
                });
            });

        };

        $scope.addServer = function() {
            ngDialog.open({
                template: 'app/configure/environments/AddAsset.html',
                scope: $scope,
                overlay:true,
                name: 'Add-Server',
                modelDialogId: 'addAssetDialogID',
                ngDialogPostionStyle:'top:60px;',
                ngDialogStyle: 'width:600px;',
                closeByDocument: false,
                closeByEscape: true
            });
            return true;
        };

        $scope.updateSelectedAgent = function (selected){
            var agentSelection = gettextCatalog.getString("Assets that support agent update must be selected at the top-most level, not their applications, databases, or virtual machines.");
            var noneSelectedMessage = gettextCatalog.getString("No agent-based assets were selected for update.") + " " + agentSelection;
            $scope.clientsList=[];
            var skipped = 0;
            if(selected == 0 ){
                if ($scope.selectedArray.length > 0){
                    $scope.updateAgentMessage = gettextCatalog.getString("Agent software will be updated on the selected asset(s).");
                    for (var i =0 ;i <$scope.selectedArray.length;i++){
                        var asset = $scope.selectedArray[i];
                        if (asset.asset_type == "physical") {
                            var obj={};
                            obj.client_id =$scope.selectedArray[i].id;
                            obj.system_id =$scope.selectedArray[i].system_id;
                            if (typeof($scope.selectedArray[i].credentials) != 'undefined' && $scope.selectedArray[i].credentials !== null){
                                obj.credential_id =$scope.selectedArray[i].credentials.credential_id;
                            }
                            $scope.clientsList.push(obj);
                        } else {
                            skipped++;
                        }
                    }
                    if ($scope.clientsList.length > 0) {
                        if (skipped) {
                            $scope.updateAgentMessage += "  " + gettextCatalog.getString("One or more selections of non agent-based assets were skipped.") + " " + agentSelection;
                        }
                        $scope.openDialogForAgentUpdate();
                    } else {
                        ngDialog.open({
                            dialogType: 'ERROR',
                            dialogMessage: noneSelectedMessage,
                            overlay:true
                        });
                    }
                } else {
                    ngDialog.open({
                        dialogType: 'ERROR',
                        dialogMessage: noneSelectedMessage,
                        overlay:true
                    });
                }

            } else if (selected == 1){
                 $scope.updateAgentMessage = gettextCatalog.getString("Agent software will be updated on all assets with an available agent update.");
                for (var i =0 ;i <$scope.protectedAssetsDataArr.length;i++){
                    asset = $scope.protectedAssetsDataArr[i];
                    if (asset.asset_type == "physical") {
                        var obj={};
                        obj.client_id =$scope.protectedAssetsDataArr[i].id;
                        obj.system_id =$scope.protectedAssetsDataArr[i].system_id;
                        if (typeof($scope.protectedAssetsDataArr[i].credentials) != 'undefined' && $scope.protectedAssetsDataArr[i].credentials !== null){
                            obj.credential_id =$scope.protectedAssetsDataArr[i].credentials.credential_id;
                        }
                        $scope.clientsList.push(obj);
                    }
                }
                if ($scope.clientsList.length > 0) {
                    $scope.openDialogForAgentUpdate();
                } else {
                    ngDialog.open({
                        dialogType: 'ERROR',
                        dialogMessage: noneSelectedMessage,
                        overlay:true
                    });
                }
            }else{
                $scope.updateAgentMessage = gettextCatalog.getString("Agent software will be updated on the selected asset.");
              //  $scope.openDialogForAgentUpdate();
                 console.log(selected)
            }
        };
        $scope.openDialogForAgentUpdate = function(){
            ngDialog.open({
                template: 'app/configure/protected-assets/agent-update.html',
                scope: $scope,
                modelDialogId:'agent-update-dialog',
                id:'agent-update'
            });

        };
        $scope.updateAgents = function(){
            ngDialog.close('agent-update-dialog');

            var dataObject = {
                'clients':$scope.clientsList
            }
            var load = PHD.showLoadingIndicator('body', true, "Agent update in progress.");
            $http({
                       url: '/api/clients/agent-push',
                       data: JSON.stringify(dataObject),
                       method: 'PUT'
                    }).success(function(data, status, headers){
                        $scope.errorListDisplay=[];
                        PHD.hideLoadingIndicator(load);
                        for (key in data){
                            if (typeof(data[key].message) !='undefined' && data[key].message != 'Latest available agent version is installed'){
                                var obj ={}
                                obj.client=data[key].client;
                                obj.message=data[key].message;
                                $scope.errorListDisplay.push(obj);
                            }

                        }
                        console.log($scope.errorListDisplay);
                        if($scope.errorListDisplay.length ==0){
                            var message = "Latest available agent version is installed";
                            ngDialog.open({
                                    dialogType: 'Success',
                                    dialogMessage: message
                            });

                        }else{
                            ngDialog.open({
                                template: 'app/configure/protected-assets/agent-update-messages.html',
                                scope: $scope,
                                modelDialogId:'agent-update-error-message',
                                ngDialogStyle: 'width:700px;',
                                id:'agent-update-error-message'
                            });
                        }


                    }).error(function(response){
                        console.log(response);
                         PHD.hideLoadingIndicator(load);
                        ngDialog.open({
                            dialogType: 'ERROR',
                            dialogMessage: data,
                            overlay:true
                        });

                    });

        }
		$scope.addHost = function() {
            ngDialog.open({
                template: 'app/configure/environments/add-edit-hypervisor.html',
                scope: $scope,
                overlay:true,
                name: 'Add-Hypervisor',
                modelDialogId:'addHostDialogID',
                ngDialogStyle: 'width:600px;',
                closeByDocument: false,
                closeByEscape: true
            });
            return true;
        };

        $scope.addNAS = function() {
            $scope.isEdit=false;
            ngDialog.open({
                template: 'app/configure/environments/addEditNas.html',
                scope: $scope,
                overlay:true,
                name:'add-edit-nas-dialog',
                modelDialogId:'add-edit-nas-dailog',
                ngDialogStyle:'width:600px;',
                preCloseCallback: function (value) {}
            });
            return true;
        };

        $scope.addCiscoUCSManager = function() {
            ngDialog.open({
                template: 'app/configure/environments/AddCiscoUCS.html',
                scope: $scope,
                overlay:true,
                name: 'Add-Cisco-UCS',
                modelDialogId:'addCiscoUCSDialogID',
                ngDialogPostionStyle:'top:60px;',
                ngDialogStyle: 'width:600px;',
                closeByDocument: false,
                closeByEscape: true
            });
            return true;
        };

        /*
         * Called once the user confirms deletion (selectedObj had to be set to get this far but we check anyway).
         */
        $scope.deleteAsset = function(){
            if ($scope.selectedObj) {
                var httpMethod = 'delete';
                var url = '/api/clients/'+$scope.selectedObj.id+'/?sid='+$scope.selectedObj.system_id;

                $http({
                    method: httpMethod,
                    url : url
                }).success(function(status, headers){
                    var newassets = $scope.protectedAssetsAllDataArr;
                    for (var i = 0; i < newassets.length; i++) {
                        if (newassets.id === $scope.selectedObj.id) {
                            delete newassets[i];
                        }
                    }
                    $scope.protectedAssetsDataArr = newassets;
                    $timeout(function() {
                        $scope.updateTreeGrid($scope.protectedAssetsDataArr);
                    });
                    $(document).trigger("assetChange");
                    ngDialog.open({
                        dialogType:'information',
                        dialogMessage: $scope.selectedObj.name + " " + gettextCatalog.getString("removed successfully.")
                    });
                }).error(function(response) {
                    ngDialog.open({
                        dialogType:'Error',
                        dialogMessage:response.result[0].message
                    });
                });
            }
        };

        $scope.deleteAssetConfirmationDialog = function() {
            if ($scope.selectedObj &&
                ($scope.tableView === false ||
                    ($scope.tableView === true && $scope.selectedArray.length === 1))) {
                var asset_type = $scope.selectedObj.asset_type;
                if (asset_type === ASSET_TYPE_PHYSICAL ||
                    asset_type === ASSET_TYPE_VMWARE_HOST ||
                    asset_type === ASSET_TYPE_XEN ||
                    asset_type === ASSET_TYPE_AHV) {

                    $scope.deleteTitle = gettextCatalog.getString('Confirm Asset Removal');
                    $scope.confirmMessage1 = gettextCatalog.getString('Are you sure want to delete asset') + ' ' + $scope.selectedObj.name + '?';
                    $scope.confirmMessage2 = gettextCatalog.getString('When an asset is removed, all of its backups are deleted (backup copies are not deleted).');
                    $scope.confirmMessage3 = gettextCatalog.getString('I understand that removing this asset will also delete all of its backups.');
                    $scope.submitButtonText = gettextCatalog.getString('Remove Asset and Delete backups');

                    ngDialog.open({
                        template: 'app/common/confirmationDialog.html',
                        scope: $scope,
                        overlay:true,
                        ngDialogStyle:'width:600px;height:250px',
                        closeByDocument: false,
                        closeByEscape: true
                    });
                } else {
                    ngDialog.open({
                        dialogType:'Warning',
                        dialogMessage: gettextCatalog.getString("Discovered assets (for example, a virtual machine or database) cannot be removed.")
                    });
                }
            } else {
                ngDialog.open({
                    dialogType:'Warning',
                    dialogMessage: gettextCatalog.getString('You must select one and only one asset at a time for deletion.')
                });
            }
        };

        function processAssets(PHD, window) {

            var Assets = function($el) {
                $scope.el = $el;
                this.$el = $el;
                this.dbid = this.$el.data("dbid");
                this.$header = $el.children(".environment-header");

                this.$buttons = $el.find(".button-bar a");
                $scope.buttons = this.$buttons;
                this.$options = this.$buttons.filter(".btn-environment-options");
                this.$el.find(".btn-group-display").flyout();
                this.$el.find(".btn-group-add-asset").flyout();
                this.$el.find(".btn-group-update-agent").flyout();

                this.appliancesInit = false;

                this.assets = [];
                this.$selectedAppliance = $();
                this.selectedID = null;
                $scope.selectedID = null;

                this.applianceProcess = null;
                this.detailProcess = null;

                this.timestamp = 0;
                this.refreshInterval = null;
                this.isActive = false;

                this.loadingIndicator = null;
            };

            Assets.prototype.init = function() {
                var self = this;

                this.initButtons();

                /*this.$applianceTable
                    .on("loadfinished updatefinished", function(event, newRows, DataTable) {
                        self.$applianceTable.scrollTable({"height": ".environment-assets"});
                        if(event.type === "loadfinished" && self.assets.length) {
                            //self.AssetsTable.selectByID(self.appliances[0].id);
                        }
                    }); */

                return this;
            };

            Assets.prototype.initButtons = function() {
                var self = this;
                return this;
            };

            Assets.prototype.updateButtons = function() {
                // Enable Edit button when 1 asset is selected.
                // Disable in case of: multiple assets are selected at level 1, Or Multiple assets at level 2 (enable if VMware VM's), Or multiple assets at multiple levels
                if($scope.selectedArray.length > 1){
                    var index, system_id = $scope.selectedArray[0].system_id, assetType = $scope.selectedArray[0].asset_type;
                    for (index = 0; index < $scope.selectedArray.length; ++index) {
                        if($scope.selectedArray[index].$$treeLevel === 0){
                            this.$buttons.filter(".settings-edit-assets").toggleButton();
                            $scope.button_allowed = false;
                            break;
                        }
                        else if(system_id !== $scope.selectedArray[index].system_id){
                            this.$buttons.filter(".settings-edit-assets").toggleButton();
                            $scope.button_allowed = false;
                            break;
                        }
                        else if($scope.selectedArray[index].$$treeLevel === 1 &&
                                ($scope.selectedArray[index].asset_type !== "VMware:VM" && $scope.selectedArray[index].asset_type !== "AHV:VM") ||
                                typeof ($scope.selectedArray[index].asset_type) === "undefined") {
                            // Allow multiple VMware and AHV VMS, at level 1 (host->item) be edited.
                            this.$buttons.filter(".settings-edit-assets").toggleButton();
                            $scope.button_allowed = false;
                            break;
                        }
                        else if($scope.selectedArray[index].$$treeLevel === 2 &&
                            (typeof ($scope.selectedArray[index].asset_type) === "undefined")){
                            // Allow multiple assets at level 2 (host->app->item) be edited.
                            this.$buttons.filter(".settings-edit-assets").toggleButton();
                            $scope.button_allowed = false;
                            break;
                        }
                        else{
                            this.$buttons.filter(".settings-edit-assets").toggleButton(true);
                            $scope.button_allowed = true;
                        }
                    }
                }
                else{
                    this.$buttons.filter(".settings-edit-assets").toggleButton(true);
                    $scope.button_allowed = true;
                }
                return this;
            };


            Assets.prototype.refresh = function(fn, updateDetails) {
                var self = this;
                if(this.isActive) {
                    fn = fn || this.updateAssets;
                    updateDetails = !(updateDetails === false);

                    this.applianceProcess = this.fetchAssets(fn);

                    $.when(this.applianceProcess).done(function() {
                        self.applianceProcess = null;
                        //if(!self.selectedID && self.appliances.length) self.AssetsTable.selectByID(self.appliances[0].state.id);
                        //if(!self.selectedID && self.assets.length) self.AssetsTable.selectByID(self.assets[0].id);
                        //self.setRefreshInterval();
                    });
                }
            };

            Assets.prototype.fetchAssets = function(fn) {
                if(!_.isFunction(fn)) {
                    fn = this.buildAssets;
                }

                this.loadingIndicator = PHD.showLoadingIndicator($("#config-assets"), false);
                return PHD.Ajax
                    .get("/api/assets/", this.loadingIndicator)
                    .done($.proxy(fn, this));
            };

            $scope.protectedAssetsDataArr = [];
            $scope.protectedAssetsAllDataArr = [];
            $scope.selectedAssets = [];
            Assets.prototype.buildAssets = function(data) {
                if(data) {
                    this.assets = data.data;
                    this.assets.forEach(function(asset, index) {
                       asset.description = (asset.os_type != undefined) ? asset.os_type : asset.asset_type;
                    });
                    $scope.selectedArray = [];
                    $scope.protectedAssetsAllDataArr = this.assets;
                    if($scope.display === ASSET_DISPLAY_FILTER_PHYSICAL){
                        $scope.displayPhysical();
                    } else if($scope.display === ASSET_DISPLAY_FILTER_VIRTUAL){
                        $scope.displayVirtual();
                    } else {
                        $scope.displayAll();
                    }

                    $scope.selectedID = null;
                }
                if(this.applianceProcess) this.applianceProcess.resolve();
            };

            Assets.prototype.updateAssets = function(data) {
                if(data) {
                    this.assets = data.data;
                    this.assets.forEach(function(asset, index) {
                        asset.description = (asset.os_type != undefined) ? asset.os_type : asset.asset_type;
                    });
                    this.timestamp = data.timestamp;
                    $scope.selectedArray = [];
                    $scope.protectedAssetsAllDataArr = this.assets;

                    // reset state of expandAll button on reload.
                    if ($scope.gridApi !== undefined) {
                        $scope.gridApi.grid.treeBase.expandAll = false;
                    }
                    if($scope.display === ASSET_DISPLAY_FILTER_PHYSICAL){
                        $scope.displayPhysical();
                    } else if($scope.display === ASSET_DISPLAY_FILTER_VIRTUAL){
                        $scope.displayVirtual();
                    } else {
                        $scope.displayAll();
                    }
                }
                if(this.applianceProcess) this.applianceProcess.resolve();
            };

            Assets.prototype.isReady = function() {
                return this.isActive && (this.applianceProcess === null || this.applianceProcess.state() === 'resolved') &&
                    (this.detailProcess === null || this.detailProcess.state() === 'resolved');
            };

            Assets.prototype.setButtonLock = function(locked) {
                this.$buttons.each(function() {
                    $(this).toggleButton(!locked);
                });
            };

            Assets.prototype.initAssets = function() {
                this.refresh(this.buildAssets);
                this.appliancesInit = true;
                return this;
            };

            Assets.prototype.kill = function() {
                var self = this,
                    url = "/api/clients/" + $scope.selectedObj.id + "/?sid=" + $scope.selectedObj.system_id;
                //UNIBP-1708 added loding indicator while removing
                var deleteProtectedAsset = PHD.showLoadingIndicator("body", true, "Removing...");
                var resp = PHD.Ajax.delete(url, null, deleteProtectedAsset);
                resp.done(function(data) {
                    $analytics && $analytics.eventTrack('Delete Asset', {  category: 'Configure', label: 'Configure' });
                    self.selectedID = null;
                    PHD.hideLoadingIndicator(deleteProtectedAsset);
                    self.refresh(self.fetchAssets);
                });
            };

            Assets.prototype.activate = function() {
                this.isActive = true;
                if(!this.appliancesInit) {
                    this.initAssets();
                } else {
                    this.refresh(this.updateAssets);
                }
                return this;
            };

            Assets.prototype.deactivate = function() {
                this.isActive = false;
                clearInterval(this.refreshInterval);
                return this;
            };

            var AssetsTab = function(index, $panel) {
                this.index = index;
                this.$panel = $panel;
                this.$assetsArea = this.$panel.children("#assets-list");
                this.$environments = null;
                this.environmentList = {};
                this.activeEnvironment = null;
                this.isActive = false;
                this.isBootstrapped = false;
            };

            AssetsTab.prototype.bootstrap = function(data) {
                this.$environments = this.$assetsArea.find(".environment");
                this.environmentList = new Assets(this.$environments).init();
                $scope.Assets = this.environmentList;
                this.activeEnvironment = this.environmentList; //this.environmentList.activate();
                this.isBootstrapped = true;
            };

            AssetsTab.prototype.init = function() {
                var self = this;
                $(document)
                    .on("assetChange", function(event, dbid) {
                        if ($scope.Assets !== undefined && $scope.Assets.appliancesInit !== undefined) {
                            if(!$scope.Assets.appliancesInit) {
                                $scope.Assets.initAssets();
                            } else {
                                $scope.Assets.refresh($scope.Assets.updateAssets);
                            }
                        }
                    });

                return this;
            };

            AssetsTab.prototype.activate = function() {
                this.isActive = true;
                if(!this.isBootstrapped) {
                    this.bootstrap();
                }
                if(this.activeEnvironment) {
                    this.activeEnvironment.activate();
                }
                return this;
            };

            AssetsTab.prototype.deactivate = function() {
                this.isActive = false;
                if(this.activeEnvironment) {
                    this.activeEnvironment.deactivate();
                }
                return this;
            };

            Assets.prototype.setSelectedAssets = function(selectedId){
                var self = this;

                $scope.listViewFlag = true;

                this.$selectedAppliance = $scope.el;
                this.selectedID = selectedId;
            };

            $scope.selectProtectedAssetsFromList = function(selectedId){
                if (selectedId !== undefined && selectedId !== null) {
                    var arrLength=($scope.protectedAssetsDataArr).length;

                    for (var i=0; i < arrLength; i++){
                        if ($scope.protectedAssetsDataArr[i].id===selectedId){
                            $scope.selectedAssets = $scope.protectedAssetsDataArr[i];
                            $scope.selectedObj = $scope.selectedAssets;
                            PHD.appliance_sid = $scope.selectedObj.system_id;
                            $("#" + selectedId).css("background-color","#f5f5f5");
                            $scope.selectedID = selectedId;
                        }else{
                            $("#" + $scope.protectedAssetsDataArr[i].id).css("background-color","#fff");
                        }
                    }
                    $scope.Assets.setSelectedAssets(selectedId);
                }
            };

            PHD.AssetsTab = AssetsTab;
        }

        // Initialize the controller.
        $scope.initCtrl();
    }]);