angular.module('replica-config-module', ['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            trapFocus: true,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('replicaConfigCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', function($scope, $rootScope, $http, ngDialog, $window, $timeout) {

        $scope.form = $();
        $scope.replicaConfig = {};
        $scope.replicaConfig.hypervisor_type = "VMware host"; //only one in 10.1.  Will add dropdown selection when more are supported
        $scope.replicaConfig.max_recovery_points = $rootScope.maxRecoveryPoints;

        $scope.showMaxSnapTooltip = function(){
            var form = angular.element("#replica-config-advanced-form");
            form.find(".definition").popover({
                appendTo: form
            });
        };
        $scope.saveSettings = function(commandName) {
            if($scope.form.valid()) {
                var url = "/api/replicas/max_recovery_points/?sid=" + selectedAppliance.id;
                var load = PHD.showLoadingIndicator("body", true);
                var dimensions = 'width:650px; height:475px;';
                var payload = $scope.replicaConfig;
                payload = JSON.stringify(payload);
                var resp = PHD.Ajax.put(url, payload, load, handleError);
                resp.done(function (data) {
                    $rootScope.maxRecoveryPoints = $scope.replicaConfig.max_recovery_points;
                    $scope.closeThisDialog();
                });
            }
        };

        function handleError(jqXHR, textStatus, errorThrown) {
            if (jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var message = data.result[0].message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'replica-config-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.validateForm = function(form) {
            form.validate({
                rules: {
                    max_recovery_points: {
                        required: true,
                        digits: true,
                        min: 1,
                        max: 28
                    }
                },
                messages: {
                    max_recovery_points: {
                        required: gettext("Maximum number of recovery points must be entered"),
                        min: gettext("A minimum of 1 recovery point is required"),
                        max: gettext("Maximum number of recovery points allowed is 28")
                    }
                }
            })
        };
        $scope.getMaxSnaps = function () {
            var url = "/api/replicas/max_recovery_points/?sid=" + selectedAppliance.id;
            var load = PHD.showLoadingIndicator("body", true);
            var resp = PHD.Ajax.get(url, load, handleMaxSnapError);
            resp.done(function(data) {
                if (data.recovery_points) {
                    $scope.replicaConfig.max_recovery_points = data.recovery_points[0].max_recovery_points;
                    $timeout(function() {
                        $scope.$apply();
                    }, 100);
                }
            });
        };

        function handleMaxSnapError(jqXHR, textStatus, errorThrown) {
            if (jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var message = data.result[0].message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'max-snap-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                } else {
                    PHD.ajaxError(jqXHR, textStatus, errorThrown);
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.onConfirmError = function () {
            ngDialog.close('replica-config-dialog');
        };

        $scope.closeThisDialog = function() {
            // closing the wizard clears errors and state.
            $scope.PHD.currentMiniWizard.wizard("close");
        };

        $scope.processDialog = function() {
            $scope.showMaxSnapTooltip();

            $scope.form = angular.element("#replica-config-advanced-form");
            $scope.validateForm($scope.form);

            $scope.getMaxSnaps();
        };

        // use instead of ngDialog load event.
        angular.element(document).ready(function () {
            $scope.processDialog();
        });

    }]);