angular.module('retention',['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])

    .controller('MinMaxRetentionCtrl', ['$scope', '$rootScope', '$http','ngDialog', function($scope, $rootScope, $http, ngDialog) {

        var Load;

        // Used to unbind the watched event on dialog close.
        var unbindHandler = null;

        $scope.initCtrl = function(dlg){
            switch (dlg.name) {
                case 'MinMaxRetention':
                    $scope.hold_days = {};
                    $scope.hold_days.value = 0;
                    $scope.hold_daysValueUpdate();
                    $scope.processMinmaxRetentionDialog($scope.ngDialogData);
                    break;
            }
        };

        // on load of the SLA policy, init the dialog.
        $scope.$on("SLA-Policy-Loaded", function(event, sla_policy) {
            $scope.ngDialogData = sla_policy;
            $scope.initCtrl({name:'MinMaxRetention'});
        });

        // Alternate initialization if not in a standalone ngDialog.
        $scope.initDialog = function(dlg, dialogData) {
            $scope.ngDialogData = dialogData;
            $scope.initCtrl(dlg);
        };

        $scope.minDaysSpinOptions = { "min": 0, "max": 9999, "step": 1};
        $scope.maxDaysSpinOptions = { "min": 0, "max": 9999, "step": 1};
        $scope.legalHoldSpinOptions = { "min": 0, "max": 9999, "step": 1};

        $scope.processMinmaxRetentionDialog = function(data) {
            var $form = $("#min_max_retention_form");
            $form.find(".definition").popover({
                appendTo: $form
            });

            $("#min_days").spinner($scope.minDaysSpinOptions);
            $("#max_days").spinner($scope.maxDaysSpinOptions);
            $("#hold_days").spinner($scope.legalHoldSpinOptions);
            $("#hold_days").on("change spin spinchange", $scope.hold_daysValueUpdate);

            if(data.length >1) {
                // local_system_id is set in case of copied assets and use that as the system ID
                if (data[0].local_system_id && data[0].local_system_id != null) {
                    $scope.system_id = data[0].local_system_id;
                    $scope.grandclient = true;
                } else {
                    $scope.system_id = data[0].system_id;
                    $scope.grandclient = false;
                }

                var flag1 = $scope.checkDuplicate(data, "retention_min");

                if (flag1 === false && data[0].min_max_policy.retention_min > 0) {
                    $("#id_use_minimum").prop("checked", true);
                    $("#min_days").val(data[0].min_max_policy.retention_min);
                }

                var flag2 = $scope.checkDuplicate(data, "retention_max");

                if (flag2 === false && data[0].min_max_policy.retention_max > 0) {
                    $("#id_use_maximum").prop("checked", true);
                    $("#max_days").val(data[0].min_max_policy.retention_max);
                }

                var flag3 = $scope.checkDuplicate(data, "legal_hold");
                // Hold forever is not an option in new satori UI as of 12/23/2015
                if (flag3 === false) {
                    if (data[0].min_max_policy.legal_hold == -1) {
                        //$("#hold_forever").prop("checked", true);
                        $scope.hold_days.value = 9999;              //If hold forever was set in previous versions we are updating it to 9999
                        $("#legal_hold").prop("checked",true);
                    }
                    else {
                        $("#hold_days").val(data[0].min_max_policy.legal_hold);
                        $scope.hold_days.value = data[0].min_max_policy.legal_hold;
                        if (data[0].min_max_policy.legal_hold > 0) {
                            $("#legal_hold").prop("checked",true);
                        }
                    }
                }
            }
            else{
                if(typeof (data.min_max_policy) === "undefined"){
                    // use local system ID in case of copied assets
                    if (data[0].local_system_id && data[0].local_system_id != null) {
                        $scope.system_id = data[0].local_system_id;
                        $scope.grandclient = true;
                    } else {
                        $scope.system_id = data[0].system_id;
                        $scope.grandclient = false;
                    }
                    data = data[0].min_max_policy;
                }
                else{
                    if (data.local_system_id && data.local_system_id != null) {
                        $scope.system_id = data.local_system_id;
                        $scope.grandclient = true;
                    } else {
                        $scope.system_id = data.system_id;
                        $scope.grandclient = false;
                    }
                    data = data.min_max_policy;
                }
                if (data.retention_min > 0) {
                    $("#id_use_minimum").prop("checked", true);
                }
                $("#min_days").val(data.retention_min);
                if (data.retention_max > 0) {
                    $("#id_use_maximum").prop("checked", true);
                }
                $("#max_days").val(data.retention_max);
                if(data.legal_hold === -1){
                    $scope.hold_days.value = 9999;
                    $scope.showForeverWarning = true;
                    //$("#hold_forever").prop("checked", true);
                    $("#legal_hold").prop("checked",true);
                } else {
                    $("#hold_days").val(data.legal_hold);
                    $scope.hold_days.value = data.legal_hold;
                    if (data.legal_hold > 0) {
                        $("#legal_hold").prop("checked",true);
                    }
                }
            }

            $scope.MinimumClicked();
            $scope.MaximumClicked();
            $scope.HoldForeverClicked();

        };

        $scope.hold_daysValueUpdate = function (data) {
            var $form = $("#min_max_retention_form");
            setTimeout(function() {
                if ($scope.showForeverWarning) {
                    $form.find("#hold_forever_warning").show();
                    $scope.showForeverWarning = false;
                } else {
                    if($("#hold_days").val() > 14){
                        $form.find("#hold_forever_warning").hide();
                        $form.find("#min_tooltip").show();
                    }
                    else{
                        $form.find("#hold_forever_warning").hide();
                        $form.find("#min_tooltip").hide();
                    }
                }
            }, 0);
        };

        $scope.checkDuplicate = function(data, type) {
            var index,flag = false;
            for (index = 0; index < data.length; ++index) {
                if(data[index].min_max_policy){
                    var min = data[0].min_max_policy[type];
                    if(data[index].min_max_policy[type] !== min){
                        flag = true;
                    }
                }
                else{
                    flag = true;
                }
            }
            return flag;
        };

        $scope.HoldForeverClicked = function() {
            if($("#hold_forever").prop("checked")){
                $scope.legal_hold = -1;
                $("#legal_hold").disable();
                $("#hold_days").disable();
                $("#hold_days").spinner("option", "disabled", true);
                $("#legal_hold").prop("checked",false);
            }
            else{
                $("#legal_hold").enable();
            }
            if( $("#legal_hold").prop("checked")){
                $("#hold_forever").disable();
                $("#hold_forever").prop("checked",false);
                $("#hold_days").enable();
                $("#hold_days").spinner("option", "disabled", false);
            }
            else{
                $("#hold_days").disable();
                $("#hold_days").spinner("option", "disabled", true);
                $("#hold_forever").enable();
            }

        };

        $scope.MinimumClicked = function() {
            if($("#id_use_minimum").prop("checked")){
                $("#min_days").enable();
                $("#min_days").spinner("option","disabled",false);
            } else {
                $("#min_days").disable();
                $("#min_days").spinner("option","disabled",true);
            }
        };

        $scope.MaximumClicked = function() {
            if($("#id_use_maximum").prop("checked")){
                $("#max_days").enable();
                $("#max_days").spinner("option","disabled",false);
            } else {
                $("#max_days").disable();
                $("#max_days").spinner("option","disabled",true);
            }
        };

        $scope.validateRetention = function(retentionInfo) {
            var msg = "",valid = true;
            var min = retentionInfo.retention_min;
            var max = retentionInfo.retention_max;
            var legal_hold = retentionInfo.legal_hold;
            if (min > 0 && max > 0 && legal_hold > 0 && min >= max && legal_hold >= max) {
                msg = '‘Delete backups after’ value must be greater than both ‘Keep backups’ and ‘Warn when less than’ values.';
                valid = false;
            }
            else if (max > 0 && legal_hold > 0 && legal_hold > max) {
                msg = '‘Keep backups’ value cannot exceed ‘Delete backups after’ value.';
                valid = false;
            }
            else if (min > 0 && max > 0 && min >= max) {
                msg = '’Delete backups after’ value must be greater than ‘Warn when less than’ value.';
                valid = false;
            }
            if(msg !== ""){
                ngDialog.open({
                    dialogType:'WARNING',
                    modelDialogId:'MinMaxErrorDialog',
                    scope:$scope,
                    dialogMessage: msg,
                    onConfirmOkButtonClick:'onConfirmError()'
                });
            }
            return valid;
        };

        $scope.AddMinMaxRetentionInfo = function() {
            var obj = {},retentionInfo = {},multipleRetentionInfo = {};
            if( $("#id_use_minimum").prop("checked")) {
                retentionInfo.retention_min = Number($("#min_days").val());
            } else {
                retentionInfo.retention_min = 0;
            }
            if( $("#id_use_maximum").prop("checked")) {
                retentionInfo.retention_max = Number($("#max_days").val());
            } else {
                retentionInfo.retention_max = 0;
            }
            if( $("#hold_forever").prop("checked")){
                retentionInfo.legal_hold = -1;
            } else if ($("#legal_hold").prop("checked")) {
                retentionInfo.legal_hold = Number($("#hold_days").val());
            } else {
                retentionInfo.legal_hold = 0;
            }

            if(!$scope.validateRetention(retentionInfo)){
                return false;
            }

            if($scope.ngDialogData.length >1){
                data = $scope.ngDialogData;
                data.forEach(function(asset, index) {
                    if(asset.min_max_policy){
                         var instance_id = asset.min_max_policy.instance_id;
                        multipleRetentionInfo.instance_id = instance_id;
                        multipleRetentionInfo.retention_min = retentionInfo.retention_min;
                        multipleRetentionInfo.retention_max = retentionInfo.retention_max;
                        multipleRetentionInfo.legal_hold = retentionInfo.legal_hold;
                        obj[index] = multipleRetentionInfo;
                    }
                    multipleRetentionInfo = {};
                })
            }
            else{
                if($scope.ngDialogData.type === "VMware"){
                    obj.client_id = $scope.ngDialogData.id;
                    obj.client_name = $scope.ngDialogData.name;
                    obj.type = $scope.ngDialogData.type;
                    obj.retentionInfo = retentionInfo;
                }
                else if($scope.ngDialogData.description === "Xen"){
                    if($scope.ngDialogData.children[0].type === "Xen"){
                        obj.client_id = $scope.ngDialogData.children[0].parent_id;
                        obj.id = $scope.ngDialogData.children[0].id;
                        obj.type = $scope.ngDialogData.description;
                        obj.retentionInfo = retentionInfo;
                    }
                }
                else if ($scope.ngDialogData.description === "AHV Host"){
                    obj.client_id = $scope.ngDialogData.id;
                    obj.client_name = $scope.ngDialogData.name;
                    obj.type = "AHV"; //$scope.ngDialogData.description;
                    obj.retentionInfo = retentionInfo;
                }
                else{
                    if(typeof ($scope.ngDialogData.min_max_policy) === "undefined"){
                        data = $scope.ngDialogData[0].min_max_policy;
                        if( typeof($scope.ngDialogData[0].type) !== "undefined" && ($scope.ngDialogData[0].type === "SQL Server" || $scope.ngDialogData[0].type === "Exchange" || $scope.ngDialogData[0].type === "Hyper-V"
                            || $scope.ngDialogData[0].type === "Oracle" || $scope.ngDialogData[0].type === "SharePoint" || $scope.ngDialogData[0].type === "UCS Service Profile" || $scope.ngDialogData[0].type === "NDMP Device")){
                            obj.client_id = $scope.ngDialogData[0].parent_id != undefined ? $scope.ngDialogData[0].parent_id : ($scope.ngDialogData[0].children !== undefined && $scope.ngDialogData[0].children.length > 0 ? $scope.ngDialogData[0].children[0].parent_id : -1);
                            obj.id = $scope.ngDialogData[0].id;
                            obj.type = $scope.ngDialogData[0].type;
                            obj.retentionInfo = retentionInfo;
                        }
                    }
                    else{
                        data = $scope.ngDialogData.min_max_policy;
                    }
                    $scope.instance_id = data.instance_id;
                    retentionInfo.instance_id = $scope.instance_id;
                    obj.retentionInfo = retentionInfo;
                }
            }

            var base_url = "/api/retention/min-max/";
            obj = JSON.stringify(obj);
            Load = PHD.showLoadingIndicator("body", true, "Saving...");
            var resp = PHD.Ajax.put(base_url + "?sid=" +  $scope.system_id + "&grandclient=" + $scope.grandclient, obj, null, handlePolicyError);

            resp.done(function(data) {
                PHD.hideLoadingIndicator(Load);
                var result = data.result[0];
                $scope.closeThisDialog();
                if ( _.isFunction(unbindHandler)) {
                    unbindHandler();
                }
                if(parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                    console.log("SUCCESS");
                    $(document).trigger("retentionChange");
                } else if (parseInt(result.code) === AJAX_RESULT_WARNING || parseInt(result.code) === AJAX_RESULT_PARTIAL_SUCCESS) {
                    if (parseInt(result.code) === AJAX_RESULT_PARTIAL_SUCCESS) {
                        // At least one asset was modified.
                        $(document).trigger("retentionChange");
                    }
                    ngDialog.open({
                        modelDialogId:'sla-retention-conflict',
                        name: 'sla-retention-conflict',
                        template: 'app/configure/retention/warning-in-sla-policy.html',
                        data: { policyAssets: result.in_policy },
                        overlay:true,
                        ngDialogStyle:"width:700px;height:320px;"
                    });
                } else if(parseInt(result.code) === AJAX_RESULT_ERROR) {
                    console.log("FAILURE");
                    ngDialog.open({
                        dialogType: 'ERROR',
                        dialogMessage: data,
                        overlay:true
                    });
                }
                //PHD.currentDialog.wizard("close");
            });
        };

        function handlePolicyError(jqXHR, textStatus, errorThrown) {
            PHD.hideLoadingIndicator(Load);
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var error = data.result[0].message;
                    console.log("The error is:" + error);
                    ngDialog.open({
                        dialogType:'Error',
                        modelDialogId:'MinMaxErrorDialog',
                        scope:$scope,
                        dialogMessage: error,
                        onConfirmOkButtonClick:'onConfirmError()'
                    });
                } else {
                    PHD.throwError(data);
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.onConfirmError = function () {
            ngDialog.close('MinMaxErrorDialog');
        };

        // Initialize the controller, returning the function to turn off the listener.
        unbindHandler = $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });
    }])
    .controller('RetentionSLAWarningCtrl', ['$scope', '$rootScope', 'ngDialog', function($scope, $rootScope, ngDialog) {

        $scope.retentionGridOptions = {
            enableSorting: true,
            enableFiltering: false,
            enableRowSelection: false,
            enableRowHeaderSelection: false,
            rowHeight: 28,
            enableColumnResizing: true,
            enableHorizontalScrollbar: 0,
            multiSelect: false,
            columnDefs: [
                {
                    name: 'name',
                    displayName: 'Asset',
                    cellTooltip: true
                },
                {
                    name: 'policy',
                    displayName: 'Policy',
                    cellTooltip: true
                }
            ],
            data: []
        };

        $scope.retentionGridOptions.data = $scope.ngDialogData.policyAssets;

        $scope.closeWarningDialog = function() {
            ngDialog.close('sla-retention-conflict');
        };

    }]);
