angular.module('retention',['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])

    .controller('RetentionCtrl', ['$scope', '$rootScope', '$http','ngDialog', function($scope, $rootScope, $http, ngDialog) {

        $scope.initCtrl = function(dlg){
            switch (dlg.name) {
                case 'Retention-Policy':
                    processDialog(PHD, window);
                    break;
                case 'Retention-Addition':
                    policyEdit = false;
                    $scope.processRetentionDialog();
                    break;
                case 'Retention-Edit':
                    policyEdit = true;
                    $scope.processEditDialog();
                    break;
            }
        };

        function handlePolicyError(jqXHR, textStatus, errorThrown) {
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var error = data.result[0].message;
                    console.log("The error is:" + error);
                    ngDialog.open({
                        dialogType:'Error',
                        modelDialogId:'GFSErrorDialog',
                        scope:$scope,
                        dialogMessage: error,
                        onConfirmOkButtonClick:'onConfirmError()'
                    });
                } else {
                    PHD.throwError(data);
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.onConfirmError = function () {
            ngDialog.close('GFSErrorDialog');
        };

        function processDialog(PHD, window){

            var $el = $("#retention-policy-dialog");
            var $retentionTable;
            var retentionSelect;
            var RetentionTable;
            var allRetention;
            var RetentionRowData = null;
            var base_url = "/api/retention/";
            var retentionEdit = false;



            function refreshRetentionList() {
                var retention = PHD.Ajax.get('/api/retention/policy/', null, handlePolicyError);
                allRetention = [];
                retention.done(function(data){

                    $scope.isPolicyGFS = true;
                    var retentionList = data.data;
                    retentionList.forEach(function(item, index) {
                     item.id = item.id;
                        if(item.is_default){
                            item.name = item.name + "(default)";
                        }
                     item.name = item.name;
                     item.days = item.days;
                     item.weeks = item.weeks;
                     item.months = item.months;
                     item.years = item.years;
                     });
                    allRetention = $.merge(allRetention, retentionList);
                    RetentionTable.load(allRetention);
                });
            }

            $retentionTable = $el.find(".retention-table");
            RetentionTable = PHD.DataTable($retentionTable, {});
            refreshRetentionList();

            $retentionTable.on("rowselect", "tr", function(event, dbid, $el, DataTable, DataRow) {
                RetentionRowData = DataRow.data;
                PolicyName = RetentionRowData.name;
                id = RetentionRowData.id;
                IsDefault = RetentionRowData.is_default;
                if(RetentionRowData.days === "Forever"){
                    Days = "";
                }
                else{
                    Days = RetentionRowData.days;
                }

                if(RetentionRowData.weeks === "Forever"){
                    Weeks = "";
                }
                else{
                    Weeks = RetentionRowData.weeks;
                }

                if(RetentionRowData.months === "Forever"){
                    Months = "";
                }
                else{
                    Months = RetentionRowData.months;
                }

                if(RetentionRowData.years === "Forever"){
                    Years = "";
                }
                else{
                    Years = RetentionRowData.years;
                }
            });
        }

        $scope.launchRetentionAddition = function(){

                ngDialog.open({
                    template: 'app/configure/retention/retention.html',
                    name: 'Retention-Addition',
                    scope: $scope,
                    overlay:true,
                    //ngDialogStyle:'width:700px; height:600px;',
                    // GFS TODO - enlarge dialog once we have weekly/monthly/yearly settings
                    ngDialogStyle:'width:700px; height:420px;',
                    closeByDocument: false,
                    closeByEscape: false,
                    showClose: false
                });

        };

        $scope.daySpinOptions = { "min": 0, "max": 32767, "step": 1};
        $scope.weekSpinOptions = { "min": 0, "max": 32767, "step": 1};
        $scope.monthSpinOptions = { "min": 0, "max": 32767, "step": 1};
        $scope.yearSpinOptions = { "min": 0, "max": 32767, "step": 1};

        $scope.processRetentionDialog = function() {
            $("#keep_days").spinner($scope.daySpinOptions);
            $("#keep_weeks").spinner($scope.weekSpinOptions);
            $("#keep_months").spinner($scope.monthSpinOptions);
            $("#keep_years").spinner($scope.yearSpinOptions);
            addSpinnerChange();
        };

        //Adds change event to spinners
        function addSpinnerChange() {
            var $keep_days = $("#keep_days");
            var $keep_weeks = $('#keep_weeks');
            var $keep_months = $('#keep_months');
            var $keep_years = $('#keep_years');

            $keep_days.on("change spin spinchange", onSpinnerChange);
            $keep_weeks.on("change spin spinchange", onSpinnerChange);
            $keep_months.on("change spin spinchange", onSpinnerChange);
            $keep_years.on("change spin spinchange", onSpinnerChange);
        }

        //Processes the values in spinners and sends them to a function to estimate the backups
        function onSpinnerChange() {
            if($("#days-forever").prop("checked")){
                daysToKeep = -1;
            } else {
                daysToKeep = Number($('#keep_days').val());
            }
            if ($('#weeks-forever').prop("checked")) {
                weeksToKeep = -1;
            } else {
                weeksToKeep = Number($('#keep_weeks').val());
            }
            if ($('#months-forever').prop("checked")) {
                monthsToKeep = -1;
            } else {
                monthsToKeep = Number($('#keep_months').val());
            }
            if ($('#years-forever').prop("checked")) {
                yearsToKeep = 1;
            } else {
                yearsToKeep = Number($('#keep_years').val());
            }
            backups = estimateBackups(daysToKeep, weeksToKeep, monthsToKeep, yearsToKeep);
            $('#estimate').text(backups);
        }

        //Function to estimate the number of backups based on the values entered
        function estimateBackups(days, weeks, months, years) {
            // we are estimating the number of backups that would be retained in 1 year
            if (days == -1 || days > 365) {
                days = 365;
            }
            if (weeks == -1 || weeks > 52) {
                weeks = 52;
            }
            if (months == -1 || months > 12) {
                months = 12;
            }
            var estimate = days;
            noOfWeeks = Math.floor(days/7);
            if (noOfWeeks > 0) {
                estimate += noOfWeeks;
            }
            if (weeks > noOfWeeks) {
                actualWeeklies = weeks - noOfWeeks;
                estimate += actualWeeklies;
            }
            noOfMonths = Math.floor(weeks/4);
            if (months > noOfMonths) {
                actualMonthlies = months - noOfMonths;
                estimate += actualMonthlies;
            }
            if (months < 12 && years > 0) {
                estimate += 1;
            }
            backupsEstimated = estimate.toString().concat(" Backups");
            return backupsEstimated;
        }

        // this can turn on and off spinners based on forever being checked or not.
        $scope.handleRetentionSpinner = function($el, elSpinner) {
            var checked = $el.is(":checked");
            elSpinner.spinner("option","disabled", checked);
        };

        $scope.processEditDialog = function () {
            // Initialize Spinners
            $scope.processRetentionDialog();

            //days = weeks = months = years = "";
            $("#retention_dialog_title").text("Edit Retention Policy");
            $("#save_policy").text("Save");
            $("#policy-name").val(PolicyName);
            if (IsDefault === true) {
                $('#default').prop("checked", true);
            } else {
                $('#default').prop("checked", false);
            }
            $("#keep_days").val(Days);
            if(Days === "")  {
                $("#keep_days").val("0");
                $("#days-forever").prop("checked", true);
                $("#keep_days").spinner("option", "disabled", true);
                $("#weeks-forever").disable();
                $("#keep_weeks").spinner("option", "disabled", true);
                $("#months-forever").disable();
                $("#keep_months").spinner("option", "disabled", true);
                $("#years-forever").disable();
                $("#keep_years").spinner("option", "disabled", true);
            }
            $("#keep_weeks").val(Weeks);
            if(Weeks === ""){
                $("#keep_weeks").val("0");
                $("#weeks-forever").prop("checked", true);
                $("#keep_weeks").spinner("disable");
                $("#months-forever").disable();
                $("#keep_months").spinner("disable");
                $("#years-forever").disable();
                $("#keep_years").spinner("disable");
            }
            $("#keep_months").val(Months);
            if(Months === "") {
                $("#keep_months").val("0");
                $("#months-forever").prop("checked", true);
                $("#keep_months").spinner("disable");
                $("#years-forever").disable();
                $("#keep_years").spinner("disable");
            }
            $("#keep_years").val(Years);
            if(Years === "") {
                $("#keep_years").val("0");
                $("#keep_years").spinner("disable");
                $("#years-forever").prop("checked", true);
            }
            if (Days === "") {
                daysToKeep = -1;
            } else {
                daysToKeep = Number(Days);
            }
            if (Weeks === "") {
                weeksToKeep = -1;
            } else {
                weeksToKeep = Number(Weeks);
            }
            if (Months === "") {
                monthsToKeep = -1;
            } else {
                monthsToKeep = Number(Months);
            }
            if (Years === "") {
                yearsToKeep = 1;
            } else {
                yearsToKeep = Number(Years);
            }
            estimatedBackups = estimateBackups(daysToKeep, weeksToKeep, monthsToKeep, yearsToKeep);
            $('#estimate').text(estimatedBackups);
        };

        $scope.launchRetentionEditor = function() {

                ngDialog.open({
                    template: 'app/configure/retention/retention.html',
                    name: 'Retention-Edit',
                    scope: $scope,
                    overlay:true,
                    //ngDialogStyle:'width:700px; height:600px;',
                    // GFS TODO - enlarge dialog once we have weekly/monthly/yearly settings
                    ngDialogStyle:'width:700px; height:420px;',
                    closeByDocument: false,
                    closeByEscape: false,
                    showClose: false
                });

        };

        $scope.ShowCalendar = function(){
            ngDialog.open({
                template: 'app/configure/retention/calendarview.html',
                scope: $scope,
                overlay:true,
                ngDialogStyle:'width:500px; height:500px;',
                closeByDocument: false,
                closeByEscape: false,
                showClose: false
            });
        };

        $scope.AddRetentionInfo = function () {
            var obj={};
            var policy_name = $("#policy-name").val();

            if (policy_name != "") {
                if($("#default").prop("checked")){
                    obj.is_default = true;
                }
                else{
                    obj.is_default = false;
                }

                if(policy_name.indexOf("(default)") > -1){
                    var name = PolicyName;
                    name = name.replace('(default)','');
                    obj.name = name;
                }
                else{
                    obj.name = policy_name;
                }

                obj.days = Number($("#keep_days").val());
                obj.weeks = Number($("#keep_weeks").val());
                obj.months = Number($("#keep_months").val());
                obj.years = Number($("#keep_years").val());

                if($("#days-forever").prop("checked")){
                    obj.days = -1;
                    obj.weeks = 0;
                    obj.months = 0;
                    obj.years = 0;
                }else if($("#weeks-forever").prop("checked")){
                    obj.weeks = -1;
                    obj.months = 0;
                    obj.years = 0;
                }else if($("#months-forever").prop("checked")){
                    obj.months = -1;
                    obj.years = 0;
                }else if($("#years-forever").prop("checked")){
                    obj.years = -1;
                }

                var base_url = "/api/retention/policy/";

                obj = JSON.stringify(obj);
                if(policyEdit){
                    base_url +=id + "/";
                    resp = PHD.Ajax.put(base_url + "?sid=" +  PHD.appliance_sid, obj, null, handlePolicyError);
                }
                else{
                    resp = PHD.Ajax.post(base_url + "?sid=" +  PHD.appliance_sid, obj, null, handlePolicyError);
                }

                resp.done(function(data) {
                    var result = data.result[0];
                    if(parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                        console.log("SUCCESS");
                        processDialog(PHD, window);
                        $scope.closeThisDialog();
                    } else if(parseInt(result.code) === AJAX_RESULT_ERROR) {
                        console.log("FAILURE");
                        ngDialog.open({
                            dialogType: 'ERROR',
                            dialogMessage: data,
                            overlay:true
                        });
                    }
                    //PHD.currentDialog.wizard("close");
                });
            }
            else{
                ngDialog.open({
                    dialogType:'Error',
                    modelDialogId:'GFSErrorDialog',
                    dialogMessage: 'Please enter policy name',
                    scope: $scope,
                    onConfirmOkButtonClick : 'onConfirmError()'
                });
            }
        };

        $scope.RetentionRemove = function() {
            if(IsDefault){
                var name = PolicyName;
                name = name.replace('(default)','');
                ngDialog.open({
                    dialogType: 'Confirmation',
                    dialogMessage: "Policy "+name+" is the default retention policy. Are you sure you want to delete the selected retention policy: "+name+"?",
                    scope: $scope,
                    overlay:true,
                    modelDialogId:'confirmation',
                    onConfirmOkButtonClick:'onConfirm()'
                });
            }
            else{
                ngDialog.open({
                    dialogType: 'Confirmation',
                    dialogMessage: "Are you sure you want to delete the selected retention policy: "+PolicyName+"?",
                    scope: $scope,
                    overlay:true,
                    modelDialogId:'confirmation',
                    onConfirmOkButtonClick:'onConfirm()'
                });
            }
        };

        $scope.onConfirm = function() {
            ngDialog.close('confirmation');
            var base_url = "/api/retention/policy/";
            base_url += id + "/" ;
            resp = PHD.Ajax.delete(base_url + "?sid=" +  PHD.appliance_sid);
             resp.done(function(data){
                 var result = data.result[0];
                 if(parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                        processDialog(PHD, window);
                     }else if(parseInt(result.code) === AJAX_RESULT_ERROR) {
                         ngDialog.open({
                         dialogType: 'ERROR',
                         dialogMessage: data,
                         overlay:true
                     });
                 }
             });
        };

        $scope.DaysForeverClicked = function() {
            $scope.handleRetentionSpinner($("#days-forever"), $("#keep_days"));
            if($("#days-forever").prop("checked")){
                console.log("Days forever enabled");
                $("#keep_days").spinner("disable");
                $("#keep_days").val("0");
                $("#weeks-forever").disable();
                $('#weeks-forever').prop("checked", false);
                $("#keep_weeks").spinner("disable");
                $("#keep_weeks").val("0");
                $("#months-forever").disable();
                $("#months-forever").prop("checked", false);
                $("#keep_months").spinner("disable");
                $("#keep_months").val("0");
                $("#years-forever").disable();
                $("#years-forever").prop("checked", false);
                $("#keep_years").spinner("disable");
                $("#keep_years").val("0");
            }
            else{
                console.log("Days forever disabled");
                $("#keep_days").spinner("enable");
                $("#weeks-forever").enable();
                $("#keep_weeks").enable();
                $("#keep_weeks").spinner("enable");
                $("#months-forever").enable();
                $("#keep_months").enable();
                $("#keep_months").spinner("enable");
                $("#years-forever").enable();
                $("#keep_years").enable();
                $("#keep_years").spinner("enable");
            }
            onSpinnerChange();
        };

        $scope.WeeksForeverClicked = function() {
            $scope.handleRetentionSpinner($("#weeks-forever"), $("#keep_weeks"));
            if ($("#weeks-forever").prop("checked")) {
                console.log("Weeks forever enabled");
                $("#keep_weeks").val("0");
                $("#keep_weeks").spinner("disable");
                $("#months-forever").disable();
                $('#months-forever').prop("checked", false);
                $("#keep_months").spinner("disable");
                $("#keep_months").val("0");
                $("#years-forever").disable();
                $("#years-forever").prop("checked", false);
                $("#keep_years").spinner("disable");
                $("#keep_years").val("0");
            } else {
                console.log("Weeks forever disabled");
                $("#keep_weeks").spinner("enable");
                $("#months-forever").enable();
                $("#keep_months").enable();
                $("#keep_months").spinner("enable");
                $("#years-forever").enable();
                $("#keep_years").enable();
                $("#keep_years").spinner("enable");
            }
            onSpinnerChange();
        };

        $scope.MonthsForeverClicked = function() {
            $scope.handleRetentionSpinner($("#months-forever"), $("#keep_months"));
            if ($("#months-forever").prop("checked")) {
                console.log("Months forever enabled");
                $("#keep_months").val("0");
                $("#keep_months").spinner("disable");
                $("#years-forever").disable();
                $("#years-forever").prop("checked", false);
                $("#keep_years").spinner("disable");
                $("#keep_years").val("0");
            } else {
                console.log("Months forever disabled");
                $("#keep_months").spinner("enable");
                $("#years-forever").enable();
                $("#keep_years").enable();
                $("#keep_years").spinner("enable");
            }
            onSpinnerChange();
        };

        $scope.YearsForeverClicked = function() {
            if($("#years-forever").prop("checked")){
                console.log("Years forever enabled");
                $("#keep_years").spinner("disable");
                $("#keep_years").val("0");
            }else{
                console.log("Years forever disabled");
                $("#keep_years").spinner("enable");
            }
            onSpinnerChange();
        };

        // Initialize the controller.
        $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });
    }]);

