angular.module('san-direct-module', ['ngDialog', 'ui.grid','ui.grid.resizeColumns','ui.grid.autoResize','ngGrid','ui.grid.selection'])
.config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
.controller('SANDirectCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants', 'inventorysvc',
    function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants, inventorysvc) {

        $scope.editDialog = null;
        $scope.selectedItem = null;
        $scope.savedItem = null;  // save item to restore on cancel
        $scope.refresh = false;

        $scope.editSANDirect = function (isEdit) {
            var dialogType = isEdit ? 'iSCSI-Edit-SAN-Direct' : 'iSCSI-SAN-Direct';
            var iSCSIData = isEdit ? $scope.selectedItem : {type: 'source'};

            // Clear this function so that the ngDialog elements can gain focus above a jQuery-ui dialog.
            $.ui.dialog.prototype._focusTabbable = function(){};

            ngDialog.open({
                template: 'app/configure/storage/iscsi/iscsi.html',
                modelDialogId: 'isci-storage-dialog',
                name: dialogType,
                scope: $scope,
                overlay:true,
                data: iSCSIData,
                ngDialogStyle:'width:500px; height:500px;',
                closeByEscape: false,
                preCloseCallback: function (value) {
                    // Since we don't know if the iSCSI dialog was cancelled or not, reload the grid each time.
                    $scope.reloadGrid();
                }
            });
        };

        $scope.confirmDelete = function() {
            var entry = $scope.selectedItem;
            var itemName = $scope.selectedItem !== undefined && $scope.selectedItem.name !== undefined ? $scope.selectedItem.name : undefined;
            if (itemName !== undefined) {
                $scope.deleteTitle = 'Confirm Connection Removal';
                $scope.confirmMessage1 = 'SAN Direct allows vProtect to more efficiently protect VMs on a SAN.';
                $scope.confirmMessage2 = 'Are you sure you want to remove the SAN Direct connection "' + itemName + '"?';
                $scope.confirmMessage3 = 'I understand that this connection will have to be re-created if deleted.';
                $scope.submitButtonText = 'Delete Connection';

                ngDialog.open({
                    template: 'app/common/confirmationDialog.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId: 'delete-scsi-dialog',
                    ngDialogStyle:'width:600px;height:250px',
                    closeByDocument: false,
                    closeByEscape: true
                });
            } else {
                var message = 'Cannot determine which SAN Direct entry is selected, please re-select and try again';
                ngDialog.open({
                    dialogType: 'Error',
                    modelDialogId: 'iscsi-error-dialog',
                    scope: $scope,
                    dialogMessage: message,
                    onConfirmOkButtonClick: 'onConfirmError()'
                });
            }
        };

        $scope.onConfirmError = function () {
            ngDialog.close('iscsi-error-dialog');
        };

        // Required name for common delete function.
        $scope.closeThisDialog = function() {
            ngDialog.close("delete-iscsi-dialog");
        };

        $scope.deleteAsset = function() {
            var url = "/api/storage/" + $scope.selectedItem.id + "/&sid=" + selectedAppliance.id;
            $http({ method: 'delete',
                url : url
            }).success(function(data, status, headers){
                $scope.selectedItem = null;
                $scope.savedItem = null;
                $scope.reloadGrid();
                $scope.closeThisDialog();
            }).error(function(response) {
                var message = response !== undefined && response.result instanceof Array ? response.result[0].message : 'Unknown Server Error';
                ngDialog.open({
                    dialogType:'Error',
                    modelDialogId: 'iscsi-error-dialog',
                    scope:$scope,
                    dialogMessage: message,
                    onConfirmOkButtonClick:'onConfirmError()'
                });
            });
        };

        $scope.reloadGrid = function() {
            var url = "/api/storage/?usage=source" + "&sid=" + selectedAppliance.id;
            $http({ method: 'get',
                url : url
            }).success(function(data, status, headers){
                $scope.selectedItem = null;
                $scope.savedItem = null;
                $scope.refresh = true;
                // For the grid, get the source storage that matches the type.
                $scope.gridData = _.filter(data.storage, function(item) {
                    return item.type == 'iscsi';
                });
                $scope.SANgridData.data = $scope.gridData;
                $timeout(function() {
                    $scope.refresh = false;
                }, 0);
            }).error(function(response) {
                var message = response !== undefined && response.result instanceof Array ? response.result[0].message : 'Unknown Server Error';
                ngDialog.open({
                    dialogType:'Error',
                    modelDialogId: 'iscsi-error-dialog',
                    scope:$scope,
                    dialogMessage: message,
                    onConfirmOkButtonClick:'onConfirmError()'
                });
            });
        };

        $scope.SANgridData = {
            multiSelect: false,
            selectionRowHeaderWidth: 30,
            rowHeight: 26,
            enableFullRowSelection : true,
            gridMenuShowHideColumns:false,
            enableColumnMenus:false,
            enableHorizontalScrollbar: 0,
            onRegisterApi: function (gridApi) {
                $scope.gridApi = gridApi;
                gridApi.selection.on.rowSelectionChanged($scope, function(rows){
                    var selectedItems = gridApi.selection.getSelectedRows();
                    if (selectedItems.length > 0) {
                        $scope.selectedItem = selectedItems[0];
                    } else if (selectedItems.length == 0) {
                        $scope.selectedItem = null;
                    }
                });
            },
            data: []
        };

        $scope.SANgridData.columnDefs = [
            {
                field: "name",
                displayName: "Name",
                type: "string"
            },
            {
                field: "properties.hostname",
                displayName: "Host",
                type: "string"
            },
            {
                field: "properties.port",
                displayName: "Port",
                type: "number"
            },
            {
                field: "properties.target",
                displayName: "Target",
                type: "string"
            },
            {
                field: "properties.lun",
                displayName: "LUN",
                type: "number"
            }
        ];

}]);
