angular.module('cifsStorage', ['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])

    .directive('dragMe', function() {
        return {
            restrict: 'A',
            link: function(scope, elem, attr, ctrl) {
                elem.draggable({
                    containment: '.ngdialog-content',
                    cursor: 'move'
                });
            }
        };
    })

    .controller('CifsStorageController', ['$scope', '$rootScope', '$http','ngDialog','gettextCatalog', function($scope, $rootScope, $http, ngDialog, gettextCatalog) {

        var StorageEdit = false;
        var AddStorage,isStatelessData;
        var cifsData = {};
        $scope.isEditstorage=false;

        // Used to unbind the watched event on dialog close.
        var unbindHandler = null;

        $scope.initCtrl = function(obj){

            $scope.sizes = [{code: 1, metric: 'GB'}, {code: 2, metric: 'TB'}];
            $scope.irsizes = [{code: 1, metric: 'GB'}];
            $("#stateless_requirement_text").hide();
            
            if(($scope.ngDialogData.type == "internal") && (typeof($scope.ngDialogData.irValue) !== "undefined")){
                $("#divInstantRecovery").show();
                $("#breakLine").show();
                $scope.isEditstorage = true;
            }else{
                $("#divInstantRecovery").hide();
                $("#breakLine").hide();
                $scope.isEditstorage = false;
            }

            var CIFSDialog = false;
            switch(obj.name){
                case 'CIFS-backup':
                    $scope.cifs = {};
                    $scope.type = "backup";
                    $scope.cifs.percent = 100;
                    $scope.pattern = /^([a-zA-Z0-9_-])*$/;
                    $scope.nameErrorString = gettextCatalog.getString("Name cannot contain special characters or space.");
                    $("#storage-allocation").hide();
                    isStatelessData = $scope.ngDialogData;
                    if(isStatelessData.isStateless){
                        $scope.cifs.name = "Internal";
                        $("#id-name").disable();
                        $scope.type = "stateless";
                        $scope.build_filesystem = 1;
                        $("#stateless_requirement_text").show();
                    }
                    CIFSDialog = true;
                    break;
                case 'CIFS-archive':
                    $scope.cifs = {};
                    $scope.type = "archive";
                    $("#title").text(gettextCatalog.getString('Add CIFS Backup Copy Target'));
                    $("#storage-allocation").hide();
                    $scope.pattern = /^([a-zA-Z0-9_-])*$/;
                    $scope.nameErrorString = gettextCatalog.getString("Name cannot contain special characters or space.");
                    CIFSDialog = true;
                    break;
                case 'CIFS-Edit-Backup':
                    $scope.cifs = {};
                    $scope.type = "backup";
                    $scope.pattern = /^([a-zA-Z0-9_-])*$/;
                    $scope.nameErrorString = gettextCatalog.getString("Name cannot contain special characters or space.");
                    StorageEdit = true;
                    cifsData = $scope.ngDialogData;
                    $scope.cifs.name = cifsData.name;
                    $("#title").text(gettextCatalog.getString('Edit CIFS storage'));
                    $("#id-name").disable();
                    $scope.cifs.host = cifsData.properties.hostname;
                    $scope.cifs.port = cifsData.properties.port;
                    $scope.cifs.share = cifsData.properties.share_name;
                    $scope.cifs.username = cifsData.properties.username;
                    $scope.cifs.password = cifsData.properties.password;
                    $scope.cifs.mb_available = cifsData.properties.mb_available;
                    gb_available = $scope.cifs.mb_available/1024;
                    $scope.cifs.max_size = cifsData.properties.max_size;
                    $scope.cifs.percent = Math.round(($scope.cifs.max_size/gb_available)*100);
                    $scope.cifs.sizeValue = cifsData.properties.max_size;
                    $scope.cifs.size = 1;
                    $scope.cifs.irValue = cifsData.irValue;
                    if( $scope.cifs.sizeValue > 1024){
                        $scope.cifs.sizeValue = $scope.cifs.sizeValue / 1024;
                        $scope.cifs.size = 2;
                    }
                    $scope.id = cifsData.id;
                    if(cifsData.type == "internal"){
                        $("#divUsePercent").hide();
                        $("#divActualValues").hide();
                        $("#breakLine").hide();
                        $scope.isEditstorage = true;
                    }
                    CIFSDialog = true;
                    break;
                case 'CIFS-Edit-Archive':
                    $scope.cifs = {};
                    $scope.type = "archive";
                    $scope.pattern = /^([a-zA-Z0-9_-])*$/;
                    $scope.nameErrorString = gettextCatalog.getString("Name cannot contain special characters or space.");
                    StorageEdit = true;
                    cifsData = $scope.ngDialogData;
                    $scope.cifs.name = cifsData.name;
                    $("#title").text(gettextCatalog.getString('Edit CIFS Backup Copy Target'));
                    $("#id-name").disable();
                    $("#storage-allocation").hide();
                    $scope.cifs.host = cifsData.properties.hostname;
                    $scope.cifs.port = cifsData.properties.port;
                    $scope.cifs.share = cifsData.properties.share_name;
                    $scope.cifs.username = cifsData.properties.username;
                    $scope.cifs.password = cifsData.properties.password;
                    $scope.id = cifsData.id;
                    CIFSDialog = true;
                    break;
            }

            if (CIFSDialog) {
                processDialog(PHD, window);
            }
        };

        function processDialog(PHD, window){
            var $port = $("#id_port"),
                $sizeValue = $("#size"),
                $instantRecoverySpinner = $('#instantRecoverySpinner'),
                portSpinnerOptions = {"min": 1, "max": 10000, "step": 1},
                sizeSpinnerOptions = {"min": 1, "max": 1024, "step": 1},
                instantRecoverySpinnerOptions = {"min": 1, "max": 10000, "step": 1};

            $scope.cifs.port = '445';
            $scope.cifs.choice = "useActualValues";

            var $form = $("form[name=CifsForm]");
            $scope.$form = $form;

            var applianceForm = PHD.FormController($form, {
                serialize: function ($form) {
                    return $form.find("input").serialize();
                }
            });
            
            $.validator.addMethod("maxSizeNameValidation", function(value, element) { 
                var maxNameSize = ($('#id-name').val()).length;

                if($scope.type == "backup"){
                    if(maxNameSize < 10){
                        return true;
                    }else{
                        return false;
                    }
                }else{
                    return true;
                }
            }, gettextCatalog.getString("Name can not exceed 9 characters"));
            
            $.validator.addMethod("maxIrSize", function(value, element) { 
                var ir= parseInt($("#instantRecoverySpinner").val().match(/\d+/), 10);
                var totalsize = parseInt(((cifsData.mb_size)/1024), 10);

                if(StorageEdit && (cifsData.type == "internal")){
                    if(ir <= totalsize){
                        return true;
                    }else{
                        return false;
                    }
                }else{
                    return true;
                }
            }, gettextCatalog.getString("IR size must not be greater than total storage size"));

            applianceForm
                .validate({
                    errorLabelContainer: "#dialog-error-list",
                    errorContainer: "#draggable-dialog-errors",
                    showErrors: function(errorMap, errorList) {
                        $("#draggable-dialog-errors").find(".summary")
                            .html('Correct the following errors: ' +
                                '<span class="highlight-identifier span-color-dark" style=" margin-left: 11px;float: right;" title="{{ \'Click to Drag\'|translate\' }}"><i class="icon-move" style="color:#d27477;"></i></span>');
                        this.defaultShowErrors();
                    },
                    rules: {
                        name: {
                            required: true,
                            pattern: $scope.pattern,
                            maxSizeNameValidation: true
                        },
                        host: {
                            required: true
                        },
                        share: {
                            required: true
                        },
                        ir: {
                            min: 0,
                            digits: true,
                            maxIrSize:true
                        }
                    },
                    messages: {
                        name: {
                            required: gettextCatalog.getString("Storage name is required"),
                            pattern:gettext($scope.nameErrorString),
                            maxSizeNameValidation:gettextCatalog.getString("Name cannot exceed 9 characters")
                        },
                        host: {
                            required: gettextCatalog.getString("Host is required")
                        },
                        share: {
                            required: gettextCatalog.getString("Share is required")
                        },
                        ir: {
                            min: gettextCatalog.getString("Please enter a positive Instant Recovery allocation"),
                            digits: gettextCatalog.getString("Instant Recovery allocation must be a number"),
                            maxIrSize: gettextCatalog.getString("IR size must not be greater than total storage size")
                        }
                    }
                });

            // Override default FormController submit.
            applianceForm.submit = function() {
                $scope.submitForm();
            };

            $port.spinner(portSpinnerOptions);
            $sizeValue.spinner(sizeSpinnerOptions);
            $instantRecoverySpinner.spinner(instantRecoverySpinnerOptions);
            $("#percent").disable();
            $("#percentValue").disable();
        }

        $scope.SizeChoiceClicked = function() {
            if($scope.cifs.choice == "usePercent"){
                $("#size").disable();
                $("#sizeMetric").disable();
                $("#percent").enable();
                $("#percentValue").enable();
                $("#size").spinner("disable");
            }
            else{
                $("#percent").disable();
                $("#percentValue").disable();
                $("#size").enable();
                $("#sizeMetric").enable();
                $("#size").spinner("enable");
            }
        };

        $scope.setDialogHeight = function(height){
                $(".ngdialog-content").height(height).css({
                    height: height + "px;"
                });
        };
        
        $scope.updateIrValue = function(){
            var ir= parseInt($("#instantRecoverySpinner").val().match(/\d+/), 10);
            var url_sid = (PHD.appliance_sid === null) ? "/" : "/?sid=" + PHD.appliance_sid;
            var url = "/api/storage/ir" + url_sid;
            var obj = {};
            obj.allowed = ir;
            obj = JSON.stringify(obj);
            PHD.Ajax
            .put(url,obj)
            .done(function(data) {
                PHD.hideLoadingIndicator(AddStorage);
                $scope.closeCIFSDialog();
            });
        }


        $scope.submitForm = function () {

            if ((!$scope.$form.valid())) {
                return false;
            }

            var obj = {};
            var properties = {};
            obj.type = 4;
            if(!StorageEdit){
                obj.name = $scope.cifs.name;
                obj.usage = $scope.type;
                properties.username = $scope.cifs.username;
                properties.password = $scope.cifs.password;
                if($scope.type === "stateless"){
                    obj.name = "_Stateless";
                    obj.build_filesystem = $scope.build_filesystem;
                }
            }
            else{
                obj.id = $scope.id;
                properties.username = $scope.cifs.username;
                properties.password = $scope.cifs.password;
                if(cifsData.type == "internal"){
                    obj.stateless_type = 4;
                    obj.type = 0;
                }
            }

            if($scope.type == "backup"){
                if(cifsData.stateless_type == null){
                    if($scope.cifs.choice == "usePercent" ){
                        if(!$scope.CifsForm.percent.$pristine || $scope.cifs.percent == 100){
                            percent_size = $scope.cifs.percent;
                            var availableGB = $scope.cifs.mb_available/1024;
                            properties.max_size = ((percent_size*availableGB)/100);
                        }
                        else{
                            console.log($scope.cifs.percent);
                            properties.max_size=0;
                        }
                    }
                    else{
                        size_entered = $("#size").val();
                        if(size_entered == 0 || size_entered ===''){
                            properties.max_size = 0;
                        }
                        else{
                            if($scope.cifs.size === 2){
                                properties.max_size = parseInt(size_entered *1024);
                            }
                            else{
                                properties.max_size = parseInt(size_entered);
                            }
                        }
                    }

                    if($rootScope.isXenUEB){
                        obj.max_concurrent_backups = 4;
                    }
                }
            }


            properties.port = $scope.cifs.port;
            properties.hostname = $scope.cifs.host;
            properties.protocol = "cifs";
            properties.share_name = $scope.cifs.share;

            obj.properties = properties;

            var base_url = "/api/storage/";
            $rootScope.disableTimers();
            obj = JSON.stringify(obj);
            if(StorageEdit){
                base_url +=$scope.id + "/";
                AddStorage = PHD.showLoadingIndicator("body", true, gettextCatalog.getString("Saving..."));
                if(cifsData.type == "internal"){
                   $scope.updateIrValue();
                }else{
                    resp = PHD.Ajax.put(base_url + "?sid=" +  PHD.appliance_sid, obj, null, handleStorageError);
                }
            }
            else{
                if($scope.type == "archive") {
                    AddStorage = PHD.showLoadingIndicator("body", true, gettextCatalog.getString("Adding and checking for existing Backup Copies.  Please wait..."));
                } else {
                    AddStorage = PHD.showLoadingIndicator("body", true, gettextCatalog.getString("Adding Storage..."));
                }
                resp = PHD.Ajax.post(base_url + "?sid=" +  PHD.appliance_sid, obj, null, handleStorageError);
            }

            if(cifsData.type !== "internal"){
                resp.done(function(data) {
                    PHD.hideLoadingIndicator(AddStorage);
                    $rootScope.enableTimers();
                    var result;

                    // When adding archive storage, see if sets need import, otherwise get the result.
                    if (!StorageEdit && $scope.type === "archive") {
                        result = data.storage;
                    } else {
                        result = data.result;
                        if (Array.isArray(result)) {
                            result = result[0];
                        }
                    }

                    if (result !== undefined) {
                        if(parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                            $scope.closeCIFSDialog();
                        } else if(parseInt(result.code) === AJAX_RESULT_ERROR) {
                            ngDialog.open({
                                dialogType: 'ERROR',
                                dialogMessage: data,
                                overlay:true
                            });
                        }
                        else{
                            if(!StorageEdit && $scope.type == "backup"){
                                ngDialog.open({
                                    dialogType:gettextCatalog.getString("Information"),
                                    dialogMessage:gettextCatalog.getString("Storage added successfully! To finish the process, use the Storage Allocation box to allocate the amount of the NAS to be used for backup."),
                                    modelDialogId:'ErrorDialog',
                                    scope: $scope,
                                    onConfirmOkButtonClick:'onConfirmError()'
                                });
                                $("#storage-allocation").show();
                                $scope.setDialogHeight(525);
                                StorageEdit = true;
                                $scope.id = result.id;
                                resp = PHD.Ajax.get(base_url + $scope.id + "/?sid=" +  PHD.appliance_sid, null, null, handleStorageError);
                                resp.done(function(data) {
                                    $scope.cifs.mb_available = data.storage.properties.mb_available;
                                });
                            }
                            else if($scope.type == "archive"){
                                var dialogType = gettextCatalog.getString("Information");
                                if(result['messages'] != undefined && Array.isArray(result['messages'])) {
                                    dialogType = "Warning";
                                    var message = '';
                                    for(var i=0; i<result['messages'].length;i++){
                                        message = message +"<br>"+result.messages[i];
                                    }
                                } else if (result['sets_needing_import'] == 0) {
                                    message = gettextCatalog.getString('No backup copies found needing import');
                                } else {
                                    message = gettextCatalog.getString('All backup copies imported successfully');
                                }

                                ngDialog.open({
                                    dialogType:dialogType,
                                    dialogMessage:message,
                                    modelDialogId:'InfoDialog',
                                    scope:$scope,
                                    onConfirmOkButtonClick:'onConfirmInfo()'
                                });
                            }
                        }
                    }
                    //PHD.currentDialog.wizard("close");
                });
            }
        };

        function handleStorageError(jqXHR, textStatus, errorThrown) {
            PHD.hideLoadingIndicator(AddStorage);
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var error = data.result[0].message;

                    if(error.indexOf(gettextCatalog.getString("Storage configuration failed - this volume contains existing data, are you sure you want to format it?")) > -1){               //Check if storage contains existing data, so we can show user option to recover.
                        var modifiedMessage = gettextCatalog.getString("The selected storage contains an existing filesystem.  If it contains Unitrends backup data, you can attempt to recover it.  Otherwise, you can erase the data or cancel and select different storage.")
                        ngDialog.open({
                            dialogType:gettextCatalog.getString('CONFIRMATION - STORAGE'),
                            modelDialogId:'ChooseRecoverOrErase',
                            scope:$scope,
                            dialogMessage: modifiedMessage,
                            tertiaryButton: true,
                            tertiaryButtonLabel: gettextCatalog.getString("Erase Data"),
                            onConfirmOkButtonClick:'onRecover()',
                            onTertiaryButtonClick:'OnErase()'
                        });
                    }
                    else{
                        $rootScope.enableTimers();
                        ngDialog.open({
                            dialogType:gettextCatalog.getString('Error'),
                            modelDialogId:'ErrorDialog',
                            scope:$scope,
                            dialogMessage: error,
                            onConfirmOkButtonClick:'onConfirmError()'
                        });
                    }
                } else {
                    $rootScope.enableTimers();
                    PHD.throwError(data);
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.onConfirmError = function () {
            ngDialog.close('ErrorDialog');
        };

        $scope.onConfirmInfo = function () {
            ngDialog.close('InfoDialog');
            $scope.closeCIFSDialog();
        };

        $scope.onRecover = function () {
            console.log("Recover the storage");
            $scope.build_filesystem = 0;
            ngDialog.close('ChooseRecoverOrErase');
            $scope.submitForm();
        };

        $scope.OnErase = function () {
            console.log("Erase the Data");
            $scope.build_filesystem = 2;
            ngDialog.close('ChooseRecoverOrErase');
            $scope.submitForm();
        };

        $scope.closeCIFSDialog = function() {
            if ( _.isFunction(unbindHandler)) {
                unbindHandler();
            }
            $scope.closeThisDialog();
        };

        unbindHandler = $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });
    }]);
