angular.module('iscsiStorage', ['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }]).directive('dragMe', function() {
        return {
            restrict: 'A',
            link: function(scope, elem, attr, ctrl) {
                elem.draggable({
                    containment: '.ngdialog-content',
                    cursor: 'move'
                });
            }
        };
    })
    .controller('iscsiStorageController', ['$scope', '$rootScope', '$http','ngDialog', function($scope, $rootScope, $http,ngDialog) {

        var StorageEdit = false;
        var AddStorage,isStatelessData;
        var $targetSelect = $("#id_target_list");
        var $lunSelect = $("#id_lun_list");

        // Used to unbind the watched event on dialog close.
        var unbindHandler = null;

        $scope.initCtrl = function(obj){
            $scope.irsizes = [{code: 1, metric: 'GB'}];
            $scope.isStorageAllocation = false;
            $scope.isEditstorage=false;
            $("#stateless_requirement_text").hide();
            if(($scope.ngDialogData.type == "internal") && typeof($scope.ngDialogData.irValue != undefined)){
                $scope.isStorageAllocation = true;
            }
            var processThisDialog = false;
            switch(obj.name){
                case 'iSCSI-backup':
                    $scope.iscsi = {};
                    $scope.iscsi.target = '';
                    $scope.type = "backup";
                    isStatelessData = $scope.ngDialogData;
                    if(isStatelessData.isStateless){
                        $scope.iscsi.name = "Internal";
                        $("#id-name").disable();
                        $scope.type = "stateless";
                        $scope.build_filesystem = 1;
                        $("#stateless_requirement_text").show();
                    }
                    processThisDialog = true;
                    break;
                case 'iSCSI-archive':
                    $scope.iscsi = {};
                    $scope.iscsi.target = '';
                    $scope.type = "archive";
                    $("#title").text('Add iSCSI Backup Copy Target');
                    processThisDialog = true;
                    break;
                case 'iSCSI-Edit-Backup':
                    $scope.iscsi = {};
                    iscsiData = $scope.ngDialogData;
                    $scope.type = "backup";
                    $scope.iscsi.name = iscsiData.name;
                    $("#title").text('Edit iSCSI Storage');
                    $("#id-name").disable();
                    $scope.iscsi.host = iscsiData.properties.hostname;
                    $scope.iscsi.port = iscsiData.properties.port;
                    $scope.iscsi.target = iscsiData.properties.target;
                    $scope.iscsi.lun = iscsiData.properties.lun;
                    $scope.id = iscsiData.id;
                    StorageEdit = true;
                    $scope.iscsi.size = 1;
                    $scope.iscsi.irValue = iscsiData.irValue;
                    if(iscsiData.type == "internal"){
                        $scope.setDialogHeight(600);
                        $scope.isEditstorage = true;
                    }
                    processThisDialog = true;
                    break;
                case 'iSCSI-Edit-Archive':
                    $scope.iscsi = {};
                    iscsiData = $scope.ngDialogData;
                    $scope.type = "archive";
                    $scope.iscsi.name = iscsiData.name;
                    $("#title").text('Edit iSCSI Backup Copy Target');
                    $("#id-name").disable();
                    $scope.iscsi.host = iscsiData.properties.hostname;
                    $scope.iscsi.port = iscsiData.properties.port;
                    $scope.iscsi.target = iscsiData.properties.target;
                    $scope.iscsi.lun = iscsiData.properties.lun;
                    $scope.id = iscsiData.id;
                    StorageEdit = true;
                    processThisDialog = true;
                    break;
                case 'iSCSI-SAN-Direct':
                    $scope.iscsi = {};
                    $scope.iscsi.target = '';
                    $scope.type = "source";
                    $("#title").text('Add iSCSI SAN Direct Connection');
                    processThisDialog = true;
                    break;
                case 'iSCSI-Edit-SAN-Direct':
                    $scope.iscsi = {};
                    iscsiData = $scope.ngDialogData;
                    $scope.type = "source";
                    $scope.iscsi.name = iscsiData.name;
                    $("#title").text('Edit iSCSI SAN Direct Connection');
                    $("#id-name").disable();
                    $scope.iscsi.host = iscsiData.properties.hostname;
                    $scope.iscsi.port = iscsiData.properties.port;
                    $scope.iscsi.target = iscsiData.properties.target;
                    $scope.iscsi.lun = iscsiData.properties.lun;
                    $scope.id = iscsiData.id;
                    StorageEdit = true;
                    processThisDialog = true;
                    break;
            }
            if (processThisDialog) {
                processDialog(PHD, window);
            }
        };

        function processDialog(PHD, window){
            var $port = $("#id_port"),
                $instantRecoverySpinner = $('#instantRecoverySpinner'),
                portSpinnerOptions = {"min": 1, "max": 10000, "step": 1},
                instantRecoverySpinnerOptions = {"min": 1, "max": 10000, "step": 1};

            $port.spinner(portSpinnerOptions);
            $instantRecoverySpinner.spinner(instantRecoverySpinnerOptions);
            if (!StorageEdit) {
                $scope.iscsi.port = 3260;
            }
            var $form = $("form[name=iscsiForm]");
            $scope.$form = $form;
            var applianceForm = PHD.FormController($form, {
                serialize: function ($form) {
                    return $form.find("input").serialize();
                }
            });
            $.validator.addMethod("maxSizeNameValidation", function(value, element) { 
                var maxNameSize = ($('#id-name').val()).length;

                if($scope.type == "backup"){
                    if(maxNameSize < 10){
                        return true;
                    }else{
                        return false;
                    }
                }else{
                    return true;
                }
            }, "Name can not exceed 9 characters");
            
            $.validator.addMethod("maxIrSize", function(value, element) { 
                var ir= parseInt($("#instantRecoverySpinner").val().match(/\d+/), 10);
                var totalsize = parseInt(((iscsiData.mb_size)/1024), 10);

                if(StorageEdit && (iscsiData.type == "internal")){
                    if(ir <= totalsize){
                        return true;
                    }else{
                        return false;
                    }
                }else{
                    return true;
                }
            }, "IR size must not be greater than total storage size");


            applianceForm
                .validate({errorLabelContainer: "#dialog-error-list",
                    errorContainer: "#draggable-dialog-errors",
                    showErrors: function(errorMap, errorList) {
                        $("#draggable-dialog-errors").find(".summary")
                            .html('Correct the following errors: <span class="highlight-identifier span-color-dark" style=" margin-left: 11px;float: right;" title="Click to Drag"><i class="icon-move" style="color:#d27477;"></i></span>');
                        this.defaultShowErrors();
                    },
                    rules: {
                        name: {
                            required: true,
                            maxSizeNameValidation: true
                        },
                        host: {
                            required: true
                        },
                        target: {
                            required: true
                        },
                        ir: {
                            min: 0,
                            digits: true,
                            maxIrSize:true
                        }
                    },
                    messages: {
                        name: {
                            required: gettext("Storage name is required"),
                            maxSizeNameValidation:gettext("Name cannot exceed 9 characters")
                        },
                        host: {
                            required: gettext("Host is required")
                        },
                        target: {
                            required: gettext("target is required")
                        },
                        ir: {
                            min: gettext("Please enter a positive Instant Recovery allocation"),
                            digits: gettext("Instant Recovery allocation must be a number"),
                            maxIrSize: gettext("IR size must not be greater than total storage size")
                        }
                    }
                });

            // Override default FormController submit.
            applianceForm.submit = function() {
                $scope.submitForm();
            };
        }
        
        $scope.setDialogHeight = function(height){
                $(".ngdialog-content").height(height).css({
                    height: height + "px;"
                });
        };
        
        $scope.updateIrValue = function(){
            var ir= parseInt($("#instantRecoverySpinner").val().match(/\d+/), 10);
            var url_sid = (PHD.appliance_sid === null) ? "/" : "/?sid=" + PHD.appliance_sid;
            var url = "/api/storage/ir" + url_sid;
            var obj = {};
            obj.allowed = ir;
            obj = JSON.stringify(obj);
            PHD.Ajax
            .put(url,obj)
            .done(function(data) {
                PHD.hideLoadingIndicator(AddStorage);
                processDialog(PHD, window);
                $scope.closeThisDialog();
            });
        };

        $scope.submitForm = function () {
            if ((!$scope.$form.valid())) {
                return false;
            }
            var obj = {};
            var properties = {};
            if(!StorageEdit){
                obj.name = $scope.iscsi.name;
                obj.usage = $scope.type;
                properties.username = $scope.iscsi.username;
                properties.password = $scope.iscsi.password;
                if($scope.type === "stateless"){
                    obj.name = "_Stateless";
                    obj.build_filesystem = $scope.build_filesystem;
                }
            }else{
                console.log("Editing");
            }

            obj.type = 1;
            properties.port = $scope.iscsi.port;
            properties.hostname = $scope.iscsi.host;
            properties.target = $scope.iscsi.target;
            properties.lun = parseInt($scope.iscsi.lun);
            obj.properties = properties;

            console.log(obj);

            var base_url = "/api/storage/";

            obj = JSON.stringify(obj);
            $rootScope.disableTimers();
            if(StorageEdit){
                console.log("PUT called");
                base_url +=$scope.id + "/";
                AddStorage = PHD.showLoadingIndicator("body", true, "Saving...");
                if(iscsiData.type == "internal"){
                   $scope.updateIrValue();
                }else{
                    resp = PHD.Ajax.put(base_url + "?sid=" +  PHD.appliance_sid, obj, null, handleStorageError);
                }
            }
            else{
                if($scope.type == "archive") {
                    AddStorage = PHD.showLoadingIndicator("body", true, "Adding and checking for existing Backup Copies.  Please wait...");
                } else {
                    AddStorage = PHD.showLoadingIndicator("body", true, "Adding Storage...");
                }
                resp = PHD.Ajax.post(base_url + "?sid=" +  PHD.appliance_sid, obj, null, handleStorageError);
            }

            resp.done(function(data) {
                PHD.hideLoadingIndicator(AddStorage);
                $rootScope.enableTimers();

                if($scope.type == "archive") {
                    var result = data.storage;
                    console.log("SUCCESS");
                    var dialogType = "Information";
                    if(result['messages'] != undefined && Array.isArray(result['messages'])) {
                        dialogType = "Warning";
                        var message = '';
                        for(var i=0; i<result['messages'].length;i++){
                            message = message +"<br>"+result.messages[i];
                        }
                    } else if (result['sets_needing_import'] == 0) {
                        message = 'No backup copies found needing import';
                    } else {
                        message = 'All backup copies imported successfully';
                    }

                    ngDialog.open({
                        dialogType:dialogType,
                        dialogMessage:message,
                        modelDialogId:'InfoDialog',
                        scope:$scope,
                        onConfirmOkButtonClick:'onConfirmInfo()'
                    });
                }
                else if($scope.type === "backup" && $rootScope.isXenUEB){
                    var getIdURL = "/api/storage/id/"+ $scope.iscsi.name +"/?sid=" + PHD.appliance_sid;
                    var resp = PHD.Ajax.get(getIdURL);
                    var newstorageObj = {};

                    resp.done(function(data) {
                        newstorageObj.max_concurrent_backups = 4;
                        newstorageObj = JSON.stringify(newstorageObj);
                        console.log(newstorageObj);
                        var url = "/api/storage/"+ data.id +"/?sid="+ PHD.appliance_sid;
                        PHD.Ajax
                            .put(url, newstorageObj)
                            .always(function() {
                                $scope.closeThisDialog();
                            });
                    });
                }
                else {
                    var result = data.result[0];
                    if(parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                        console.log("SUCCESS");
                        processDialog(PHD, window);
                        $scope.closeThisDialog();
                    } else if(parseInt(result.code) === AJAX_RESULT_ERROR) {
                        console.log("FAILURE");
                        ngDialog.open({
                            dialogType: 'ERROR',
                            dialogMessage: data,
                            overlay:true
                        });
                    }
                }

                //PHD.currentDialog.wizard("close");
            });

        };

        $scope.TargetChange = function() {
            console.log("Changes");
            var selectedtarget = $targetSelect.val();
            console.log(selectedtarget);
            $scope.iscsi.target = selectedtarget;
        };

        $scope.ScanTargets = function() {
            $targetSelect = $("#id_target_list");
            var scan_url = "/api/storage/targets/";
            var ScanIndicator = PHD.showLoadingIndicator("body", true, "Searching for Targets...");
            var resp = PHD.Ajax.get(scan_url + "?sid=" +  PHD.appliance_sid + "&host=" + $scope.iscsi.host + "&port=" + $scope.iscsi.port, ScanIndicator, handleStorageError);

            $("#backup-job-form-content").width(800);
            resp.done(function(data) {
                PHD.hideLoadingIndicator(ScanIndicator);
                var targets = data.targets;
                if(data.targets != null){
                    targetSelect = $targetSelect[0];
                    targetSelect.options.length = 0;
                    if (targets.length > 0) {
                        targets.forEach(function (target, index) {
                            targetSelect.options[index] = new Option(target);
                        })
                    }
                }
                else{
                    ngDialog.open({
                        dialogType:'Warning',
                        modelDialogId:'SelectCredential',
                        dialogMessage: 'No targets were found.'
                    });
                }
            });
        };

        $scope.LUNChange = function() {
            console.log("Changes");
            var selectedLUN = $lunSelect.val();
            console.log(selectedLUN);
            $scope.iscsi.lun = selectedLUN;
        };

        $scope.ScanLUNs = function() {
            $lunSelect = $("#id_lun_list");
            var scan_url = "/api/storage/targets/";
            if ($scope.iscsi.target === "") {
                ngDialog.open({
                    dialogType:'Error',
                    modelDialogId:'ErrorDialog',
                    scope:$scope,
                    dialogMessage: 'The target field cannot be empty when scanning for LUNs.',
                    onConfirmOkButtonClick:'onConfirmError()'
                });
                return;
            }
            var ScanIndicator = PHD.showLoadingIndicator("body", true, "Searching for LUNs...");
            var resp = PHD.Ajax.get(scan_url + $scope.iscsi.target +"/?sid=" +  PHD.appliance_sid + "&host="+ $scope.iscsi.host + "&port="+ $scope.iscsi.port, ScanIndicator, handleStorageError);

            resp.done(function(data) {
                PHD.hideLoadingIndicator(ScanIndicator);
                var luns = data.luns;
                if(data.luns != null){
                    lunSelect = $lunSelect[0];
                    lunSelect.options.length = 0;
                    if (luns.length > 0) {
                        luns.forEach(function (lun, index) {
                            lunSelect.options[index] = new Option(lun);
                        })
                    }
                }
                else{
                    ngDialog.open({
                        dialogType:'Warning',
                        modelDialogId:'SelectCredential',
                        dialogMessage: 'No targets were found.'
                    });
                }
            });
        };

        function handleStorageError(jqXHR, textStatus, errorThrown) {
            PHD.hideLoadingIndicator(AddStorage);
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var error = data.result[0].message;
                    console.log("The error is:" + error);

                    if(error.indexOf("Storage configuration failed - this volume contains existing data, are you sure you want to format it?") > -1){               //Check if storage contains existing data, so we can show user option to recover.
                        var modifiedMessage = "The selected storage contains an existing filesystem.  If it contains Unitrends backup data, you can attempt to recover it.  " +
                            "Otherwise, you can erase the data or cancel and select different storage.";
                        ngDialog.open({
                            dialogType:'CONFIRMATION - STORAGE',
                            modelDialogId:'ChooseRecoverOrErase',
                            scope:$scope,
                            dialogMessage: modifiedMessage,
                            tertiaryButton: true,
                            tertiaryButtonLabel: "Erase Data",
                            onConfirmOkButtonClick:'onRecover()',
                            onTertiaryButtonClick:'OnErase()'
                        });
                    }
                    else{
                        $rootScope.enableTimers();
                        ngDialog.open({
                            dialogType:'Error',
                            modelDialogId:'ErrorDialog',
                            scope:$scope,
                            dialogMessage: error,
                            onConfirmOkButtonClick:'onConfirmError()'
                        });
                    }
                } else {
                    $rootScope.enableTimers();
                    PHD.throwError(data);
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.onConfirmError = function () {
            ngDialog.close('ErrorDialog');
        };

        $scope.onConfirmInfo = function () {
            ngDialog.close('InfoDialog');
            processDialog(PHD, window);
            $scope.closeThisDialog();
        };

        $scope.onRecover = function () {
            console.log("Recover the storage");
            $scope.build_filesystem = 0;
            ngDialog.close('ChooseRecoverOrErase');
            $scope.submitForm();
        };

        $scope.OnErase = function () {
            console.log("Erase the Data");
            $scope.build_filesystem = 2;
            ngDialog.close('ChooseRecoverOrErase');
            $scope.submitForm();
        };

        $scope.closeThisDialog = function() {
            if ( _.isFunction(unbindHandler)) {
                unbindHandler();
            }
            ngDialog.close('isci-storage-dialog');
        };

        // Initialize the controller, returning the function to turn off the listener.
        unbindHandler = $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });
    }]);
