/*
 * Networked Appliance Controller
 */
angular.module('networked-appliances-module',['dynamicdashboardapp','ui.dashboard',
    'ui.grid', 'ui.grid.resizeColumns','ui.grid.autoResize', 'ui.grid.selection', 'ngDialog'])
    .filter('DecimalFormatter', function($filter) {
        return function(input) {
            return ((input === "" || isNaN(input)) ? "--" : input.toFixed(2));
        };
    })
    .controller('networkedAppliancesCtrl', function ($scope,$rootScope, $http, ngDialog, uiGridConstants, gettextCatalog){
        $scope.widgetName="Distributed Status";
        $scope.modalShown=false;
        $scope.viewLabel = "Protection View";
        $scope.statsShow = true;
        $scope.toggleModal=function(){
            $scope.modalShown=!$scope.modalShown;
        };

        $scope.deviceStatus = [];
        $scope.selectedApplianceName = "";
        $scope.GRID_TO_HEIGHT = 75;         // Pixels to subtract from tile height to get grid height.

        var DEM_KB_URL = KB_URL + "2030";
        var DEM_URL_TITLE = gettextCatalog.getString('Click for more information on DEM configuration');

        // Returns a status icon for the day, or "none" if on backups (or copies) are available.
        // Sets disabled to false so that the click function will not open a details dialog.
        $scope.showBackupStatus = function(element, day) {
            var obj = element[day];
            var dataDays = element[0]['updated_days'];
            var status = dataDays !== undefined && dataDays < 7 ? "unknown" : "none";
            if (obj.active !== undefined && obj.active.count > 0) {
                status = gettextCatalog.getString('active');
            }
            if (obj.successes !== undefined && obj.successes.count > 0) {
                status = gettextCatalog.getString('success');
            }
            if (obj.warnings !== undefined && obj.warnings.count > 0) {
                status = gettextCatalog.getString('warning');
            }
            if (obj.failures !== undefined && obj.failures.count > 0) {
                status = gettextCatalog.getString('error');
            }
            // See if the element should be clickable or not.
            // If the object property is disabled, the ng-click function in the cell template will not open a dialog.
            if (status !== gettextCatalog.getString("none")) {
                status += ' day-result'; // so that it is clickable.
            } else {
                obj.disabled = true;
            }
            return "icon-phd-" + status;
        };

        // Returns a formatted Day and date, e.g., Sunday, June 13, 2016.
        var dow_weekdays = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];
        function getDay(day, dow) {
            var offset = day - dow;
            if (dow < day) {
                offset -= 7;    // go to prior week.
            }
            var date = moment().day(dow + offset);
            return dow_weekdays[day] + ", " + date.format(DATE_DISPLAY_FORMAT);
        }

        $scope.showTitle = function(element, day) {
            var item = element[day];
            var tooltip = "";
            if (item !== undefined) {
                var statusDay = item['day'];
                tooltip = "Status for: " + getDay(statusDay, moment().day());
            }
            return tooltip;
        };

        $scope.statusGridData = {
            rowHeight: 30,
            enableGridMenu: true,
            enableHorizontalScrollbar: 0,
            data: []
        };

        $scope.statusGridData.columnDefs = [
            {
                field: "client_name",
                displayName: gettextCatalog.getString("Asset"),
                sortable : true,
                sortingType : "string",
                sort: {
                    direction: uiGridConstants.ASC,
                    priority: 0
                }
            },
            {
                field: "database_name",
                displayName: gettextCatalog.getString("Database Name"),
                sortable : true,
                visible: false
            },
            {
                field: "instance_name",
                displayName: gettextCatalog.getString("Instance Name"),
                sortable : true,
                visible: false
            },
            {
                field: "start_date",
                displayName: gettextCatalog.getString("Start Date"),
                sortable : true
            },
            {
                field: "end_date",
                displayName: gettextCatalog.getString("End Date"),
                sortable : true
            },
            {
                field: "elapsed_time",
                displayName: gettextCatalog.getString("Elapsed Time"),
                sortable : true
            },
            {
                field: "application_name",
                displayName: gettextCatalog.getString("Application"),
                sortable : true
            },
            {
                field: "type",
                displayName: gettextCatalog.getString("Type"),
                sortable : true
            },
            {
                field: "status",
                displayName: gettextCatalog.getString("Status"),
                sortable : true
            },
            {
                field: "complete",
                displayName: gettextCatalog.getString("Complete"),
                visible: false
            }
        ];

        $scope.showStatusDetails = function(device, which, dow) {
            var id = device['id'];
            var getCopies = which == 1;
            var url = '/portal/api/' + (getCopies ? 'backup-copies.php' : 'backups.php') + '/?id=' + id + '&dow=' + dow;
            $scope.selectedApplianceName = device['name'];
            $http({
                method: 'GET',
                url: url
            }).success(function(data, status, headers) {
                $scope.statusGridData.data = data['backups'];
                ngDialog.open({
                    template: 'app/dynamic-dashboard/networked-appliances/backup-status.html',
                    scope: $scope,
                    overlay:true,
                    name: 'networked-backup-status',
                    ngDialogStyle:'width:900px; height:465px;',
                    closeByDocument: false,
                    closeByEscape: true,
                    preCloseCallback: function (value) {
                    }
                });
            }).error(function(data, status, headers) {
                $scope.statusGridData.data = [];
                var message = gettextCatalog.getString("Error viewing status; please check manager logs for more information.");
                if (data !== undefined && data.message !== undefined) {
                    message = data.message;
                }
                ngDialog.open({
                    dialogType: 'Error',
                    modelDialogId: 'status-error',
                    scope: $scope,
                    dialogMessage: message,
                    onConfirmOkButtonClick: 'onConfirmError()'
                });
            });
        };

        $scope.onConfirmError = function () {
            ngDialog.close('status-error');
        };

        $scope.alertsGridData = {
            rowHeight: 30,
            enableGridMenu: true,
            enableHorizontalScrollbar: 0,
            data: []
        };

        //var resolvedTemplate = '<div class="ui-grid-cell-contents">{{ (row.entity.resolved == 1) | true_or_false }}</div>';

        $scope.alertsGridData.columnDefs = [
            {
                field: "created",
                displayName: gettextCatalog.getString("Created"),
                sortable : true,
                sortingType : "string",
                width: 150
            },
            {
                field: "id",
                visible: false,
                sortable : true,
                sort: {
                    direction: uiGridConstants.DESC,
                    priority: 0
                },
                sortingType : "number"
            },
            {
                field: "updated",
                displayName: gettextCatalog.getString("Updated"),
                sortable : true,
                visible: false,
                width: 150
            },
            {
                field: "message",
                displayName: gettextCatalog.getString("Message"),
                sortable : true,
                cellTooltip: true,
                width: '*'
            },
            /*{
                field: "resolved",
                displayName: "Resolved",
                sortable : false,
                visible: false,
                cellTemplate: resolvedTemplate

            },*/
            {
                field: "severity",
                displayName: gettextCatalog.getString("Severity"),
                sortable : true,
                width:100
            },
            {
                field: "source_name",
                displayName: gettextCatalog.getString("Source"),
                sortable : true,
                width:120
            }
        ];

        $scope.showAlertDetails = function(device) {
            var id = device['id'];
            $scope.selectedApplianceName = device['name'];
            $http({
                method: 'GET',
                url: '/portal/api/alerts.php/?id=' + id + '&open=true'
            }).success(function(data, status, headers) {
                $scope.alertsGridData.data = data['alerts'];
                ngDialog.open({
                    template: 'app/dynamic-dashboard/networked-appliances/alerts-status.html',
                    scope: $scope,
                    overlay:true,
                    name: 'networked-alerts-status',
                    ngDialogStyle:'width:900px; height:465px;',
                    closeByDocument: false,
                    closeByEscape: true,
                    preCloseCallback: function (value) {
                    }
                });
            }).error(function(data, status, headers) {
                $scope.alertsGridData.data = [];
                var message = gettextCatalog.getString("Error viewing alerts; please check manager logs for more information.");
                if (data !== undefined && data.message !== undefined) {
                    message = data.message;
                }
                ngDialog.open({
                    dialogType: 'Error',
                    modelDialogId: 'status-error',
                    scope: $scope,
                    dialogMessage: message,
                    onConfirmOkButtonClick: 'onConfirmError()'
                });
            });
        };

        /*
         * Called when the user clicks on an appliance, goes to the Configure page, Distributed Appliances tab.
         */
        $scope.selectAppliance = function(device) {
            var id = device['id'];
            var sid = "1";          // unused but must be set so the router will work.
            var e = $.Event('uinav.configure');
            $(document).trigger(e);
            location.href = "#/configure/" + id + '/' + 'network' + '/' + sid;
        };

        $scope.refreshAppliance = function(device) {
            var url = '/portal/api/status.php/?id=' + device['id'];
            var htmlID = '#id_refresh_' + device['id'];
            $refreshButton = $(htmlID);
            var load = PHD.showLoadingIndicator($refreshButton, false);
            $http({
                method: 'PUT',
                url: url
            }).success(function(data, status, headers) {
                PHD.hideLoadingIndicator(load);
                $rootScope.getNetworkedApplianceData();
            }).error(function(data, status, headers) {
                PHD.hideLoadingIndicator(load);
                var message = data.message;
                if (message instanceof Array) {
                    message = message.toString();
                }
                ngDialog.open({
                    dialogType: 'Error',
                    modelDialogId: 'status-error',
                    scope: $scope,
                    helpArticleLink: DEM_KB_URL,
                    helpArticleTitle: DEM_URL_TITLE,
                    dialogMessage: message,
                    onConfirmOkButtonClick: 'onConfirmError()'
                });
            });
        };

        // template for the weekly status report that produces 1 status icon per day based on backup status.
        var backupColoredStatusTemplate = '<div class="ui-grid-cell-contents"> ' +
            '<i ng-repeat="a in row.entity.backups track by $index" ' +
            'ng-click="grid.appScope.showStatusDetails(row.entity, 0, $index)" ' +
            'class="{{grid.appScope.showBackupStatus(row.entity.backups, $index)}} report-status-icon" ' +
            'title="{{grid.appScope.showTitle(row.entity.backups, $index)}}"></i>' +
            '</div>';

        var backupCopyColoredStatusTemplate = '<div class="ui-grid-cell-contents"> ' +
            '<i ng-repeat="a in row.entity.copies track by $index" ' +
            'ng-click="grid.appScope.showStatusDetails(row.entity, 1, $index)" ' +
            'class="{{grid.appScope.showBackupStatus(row.entity.copies, $index)}} report-status-icon" ' +
            'title="{{grid.appScope.showTitle(row.entity.copies, $index)}}"></i>' +
            '</div>';

        var alertsTemplate = '<div class="ui-grid-cell-contents"> ' +
            '<a href ng-click="grid.appScope.showAlertDetails(row.entity)">' +
            '<i class="icon-warningsign" style="padding-right:2px"></i>{{row.entity.open_alerts}} open ({{row.entity.open_critical}} critical)' +
            '</a></div>';

        /*var refreshStatusTemplate = '<div class="ui-grid-cell-contents"> ' +
            '{{row.entity.updated_date}}' +
            '<button class="btn-clear" style="margin-left:20px;" ng-click="grid.appScope.refreshAppliance(row.entity)" '+
            'title="Click to upload new data from {{row.entity.name}}."><i id="id_refresh_{{row.entity.id}}" class="icon-refreshalt"></i>' +
            '</button></div>';*/

        var refreshIconTemplate = '<div class="ui-grid-cell-contents"> ' +
            '<button class="btn-clear" ng-click="grid.appScope.refreshAppliance(row.entity)" '+
            'title="Click to refresh status for {{row.entity.name}}."><i id="id_refresh_{{row.entity.id}}" class="icon-refreshalt"></i>' +
            '</button></div>';

        var applianceTemplate = '<div class="ui-grid-cell-contents" title="Asset: {{row.entity.uuid}}; Click to go to the Configure page">' +
                                '<a href ng-click="grid.appScope.selectAppliance(row.entity)">{{row.entity.name}}</a></div>';

        $scope.deviceColumnDefs =    [
            {
                field: 'manager_name',
                displayName:gettextCatalog.getString('CUSTOMER')
            },
            {
                field: 'location_name',
                displayName:gettextCatalog.getString('LOCATION')
            },
            {
                field: 'name',
                displayName: gettextCatalog.getString('APPLIANCE'),
                cellTemplate: applianceTemplate
            },
            {
                field: "id",
                visible: false,
                sort: {
                    direction: uiGridConstants.ASC,
                    priority: 0
                },
                sortingType : "number"
            },
            {
                field: 'open_alerts',
                displayName: gettextCatalog.getString('ALERTS'),
                cellTemplate: alertsTemplate
            },
            {
                field: 'backups',
                displayName: gettextCatalog.getString('BACKUPS'),
                cellTemplate: backupColoredStatusTemplate
            },
            {
                field: 'copies',
                displayName: gettextCatalog.getString('HOT BACKUP COPIES'),
                cellTemplate: backupCopyColoredStatusTemplate
            },
            {
                field: 'updated_date',
                displayName: gettextCatalog.getString('LAST UPDATE')
            },
            {
                field: 'updated_date',
                displayName:'',
                cellTemplate: refreshIconTemplate,
                minWidth: 24,
                maxWidth: 24,
                width: 24
            }
        ];

        $scope.networkedApplianceOptions = {
            rowHeight: 30,
            enableRowSelection: false,
            gridMenuShowHideColumns:false,
            enableColumnMenus:false,
            enableHorizontalScrollbar: 0,
            columnDefs: $scope.deviceColumnDefs,
            data: 'deviceStatus'
        };

        $scope.protectColumnDefs =    [
            {
                field: 'manager_name',
                displayName:'CUSTOMER'
            },
            {
                field: 'location_name',
                displayName:'LOCATION'
            },
            {
                field: 'name',
                displayName:'APPLIANCE',
                cellTemplate: applianceTemplate
            },
            {
                field: "id",
                visible: false,
                sort: {
                    direction: uiGridConstants.ASC,
                    priority: 0
                },
                sortingType : "number"
            },
            {
                field: 'protected_assets',
                displayName:'PROTECTED ASSETS'
            },
            {
                field: 'gb_protected',
                displayName:'PROTECTED GB',
                cellFilter: "DecimalFormatter"
            },
            {
                field: 'replicated_assets',
                displayName:'REPLICATED ASSETS'
            },
            {
                field: 'updated_date',
                displayName:'LAST UPDATE'
            },
            {
                field: 'updated_date',
                displayName:'',
                cellTemplate: refreshIconTemplate,
                minWidth: 24,
                maxWidth:24,
                width: 24
            }
        ];

        $scope.networkedProtectOptions = {
            rowHeight: 30,
            enableRowSelection: false,
            gridMenuShowHideColumns:false,
            enableColumnMenus:false,
            enableHorizontalScrollbar: 0,
            columnDefs: $scope.protectColumnDefs,
            data: 'deviceStatus'
        };

        $scope.toggleView = function() {
            if ($scope.statsShow) {
                $scope.$broadcast('FLIP_EVENT_IN');
            } else {
                $scope.$broadcast('FLIP_EVENT_OUT');
            }
            $scope.statsShow = !$scope.statsShow;
            $scope.viewLabel = ($scope.statsShow === true ? "Protection View" : "Status view");
        };

        $scope.$on('dialogopenEvent', function (event, widget,index){
            if(widget.name === 'DISTRIBUTED STATUS'){
                $scope.toggleModal();
                $("[id='id_distributed statusRefresh']").val($rootScope.listOfTimer[9]/1000);
                event.defaultPrevented = true;
            }
        });
        /* Event listener for manual refresh */
        $scope.$on('manualRefreshEvent', function (event, widget,index){
            if(widget.name === 'DISTRIBUTED STATUS'){
                $rootScope.getNetworkedApplianceData();
                event.defaultPrevented = true;
            }
        });
        /* Event listener for tile resizing */
        $scope.$on('widgetResized', function (event, widget,index){
            if(widget.height) {
                $scope.height = widget.height;
                var gridHeight = widget.height - $scope.GRID_TO_HEIGHT;
                angular.element(document.getElementById('toruidDISTRIBUTED STATUS')).css('height', widget.height + 'px');
                $scope.setGridsHeight(gridHeight, 'px');
            }
            if (widget.width) {
                $scope.width = widget.width;
            }
        });

        /*
         * Set the height of the UI-grids based on the size of the container.
         */
        $scope.setGridsHeight = function(gridHeight, units) {
            angular.element(document.getElementById('networkedApplianceGrid')).css('height', gridHeight + units);
            angular.element(document.getElementById('networkedProtectGrid')).css('height', gridHeight + units);

            angular.element(document.getElementById('networkedApplianceFlipper')).css('height', gridHeight + units);
        };

        $rootScope.getNetworkedApplianceData = function(days){
            $http({
                method: 'GET',
                url: '/portal/api/status.php/?counts=1'
            }).success(function(data, status, headers) {
                var devices = data['devices'];
                for (var i = 0; i < devices.length; i++) {
                    var alerts = devices[i]['alerts'];
                    devices[i].open_alerts = alerts['open']['total'];
                    devices[i].open_critical = alerts['open']['critical'];
                    var backups, copies;
                    var allBackups = devices[i]['backups'];
                    if (allBackups.length > 0) {
                        backups = allBackups[0];
                    }
                    var allCopies = devices[i]['copies'];
                    if (allCopies.length > 0) {
                        copies = allCopies[0];
                    }
                    var updated = $scope.getUpdatedDate(backups, copies, alerts);
                    devices[i].updated_date = updated['updated'];
                    devices[i].timestamp = updated['timestamp'];
                    $scope.timeStamp = devices[i].timestamp;
                    devices[i].copy_status = 'none';
                    if (devices[i].assets !== undefined) {
                        var asset = devices[i].assets;
                        var protected_assets = 0;
                        if (asset['vms'] !== undefined) {
                            protected_assets += asset['vms'];
                        }
                        if (asset['servers'] !== undefined) {
                            protected_assets += asset['servers'];
                        }
                        if (asset['apps'] !== undefined) {
                            protected_assets += asset['apps'];
                        }
                        devices[i].protected_assets = protected_assets;
                        devices[i].gb_protected = asset['gb_protected'];
                        devices[i].replicated_assets = asset['all_replicated'];
                    }
                }
                $scope.deviceStatus = devices;
                var tileHeight = angular.element(document.getElementById('toruidDISTRIBUTED STATUS')).css('height');
                tileHeight = parseInt(tileHeight) - $scope.GRID_TO_HEIGHT;
                $scope.setGridsHeight(tileHeight, 'px');
            }).error(function(data, status, headers) {
                $scope.deviceStatus = [];
                var message = "Error viewing status; please check manager logs for more information.";
                if (data !== undefined && data.message !== undefined) {
                    message = data.message;
                }
                //Dialog here for error?
            });
        };

        $scope.getUpdatedDate = function(backups, copies, alerts) {
            var timestamp = null;
            var updated = "Never";
            if (backups !== undefined) {
                if (backups['timestamp'] !== null) {
                    updated = backups['updated_date'];
                    timestamp = backups['timestamp'];
                }
            }
            if (copies !== undefined) {
                if (copies['timestamp'] !== null && copies['timestamp'] > timestamp) {
                    updated = copies['updated_date'];
                    timestamp = copies['timestamp'];
                }
            }
            if (alerts !== undefined) {
                if (alerts['timestamp'] !== null && alerts['timestamp'] > timestamp) {
                    updated = alerts['updated_date'];
                    timestamp = alerts['timestamp'];
                }
            }
            return ({'updated': updated, 'timestamp': timestamp});
        };

        $(window).on("resize", function(event){

        });
        $scope.userPrefernce = function(){
            $rootScope.manualRefreshControl(0,$scope.widgetName);
        };
        if ($scope.user.loggedIn) {
            $scope.userPrefernce();
            $rootScope.getNetworkedApplianceData();
        }
        $(document)
            .on("login", stopAll)
            .on("logout", stopAll)
            .on("uinav.protect", stopAll)
            .on("uinav.recover", stopAll)
            .on("uinav.jobs", stopAll)
            .on("uinav.configure", stopAll)
            .on("uinav.reports", stopAll);

        function stopAll() {
            clearInterval($rootScope.listOfTimerFunction[9]);
            $(document).off("uinav.protect", stopAll);
            $(document).off("uinav.recover", stopAll);
            $(document).off("uinav.jobs", stopAll);
            $(document).off("uinav.configure", stopAll);
            $(document).off("uinav.reports", stopAll);
            $(document).off("login", stopAll);
            $(document).off("logout", stopAll);
        }

    });