angular.module("rdrWidgetModule",['dynamicdashboardapp','ui.dashboard'])
    .controller("rdrWidgetController", function($scope,$rootScope, $http, $timeout, gettextCatalog){

        $scope.refreshKey = "dash_recovery_assurance_refresh";
        $scope.widgetName = "Recovery Assurance";
        $scope.modalShown = false;
        $scope.reSizeGraph = true;

        $scope.chartColorArray = ['#7BBC3B', '#FF5C5C'];
        $scope.complianceChartLegendArray = [];

        $scope.success_percent = 100;
        $scope.failure_percent = MINPERCENT;

        // setting all values in dashboard widget to 0 initially
        $scope.compliance_ok = 0;
        $scope.rpo_failed = 0;
        $scope.rto_failed = 0;

        $scope.successful = 0;
        $scope.warning = 0;
        $scope.failed = 0;

        /* Event listener for resize Event */
        $(window).on("resize", function(event){
            if ($scope.reSizeGraph == true){
                var compliance_width = $('#compliance_chart').width();
                $('#compliance_chart').html("");
                var complianceChartDataArray = [$scope.success_percent, $scope.failure_percent];
                if ($('#compliance_chart').length) {
                    var title = "Compliance";
                    createpieChartRDR("compliance_chart", complianceChartDataArray, $scope.chartColorArray,
                        $scope.complianceChartLegendArray, compliance_width, title);
                }
            }
        });

        $scope.toggleModal = function() {
            $scope.modalShown = !$scope.modalShown;
        };

        /* Event listener for dialogopen Event */
        $scope.$on('dialogopenEvent', function (event, widget,index) {
            if(widget.name === 'RECOVERY ASSURANCE') {
                $scope.toggleModal();
                event.defaultPrevented = true;
                $("[id='id_recovery assuranceRefresh']").val($rootScope.listOfTimer[11]/1000);
            }
        });

        $rootScope.getRecoveryAssuranceData = function(days) {
            $http({
                method: 'GET',
                url: '/api/reports/rdr_summary/'
            }).success(function (data) {
                var compliance_width = $('#compliance_chart').width();
                $('#compliance_chart').html("");

                var compliance_data = data.compliance;

                // if there's no compliance data, ok_percent is null and the pie chart should be grayed out
                if (compliance_data.ok_percent == null) {
                    $scope.success_percent = MINPERCENT;
                    $scope.failure_percent = MINPERCENT;
                    $scope.complianceChartLegendArray = [
                        gettextCatalog.getString("Success") + " (0%)",
                        gettextCatalog.getString("Failure") + " (0%)"];
                    $scope.chartColorArray = ['#d3d3d3', '#d3d3d3'];
                } else {
                    $scope.compliance_ok = compliance_data.compliance_ok;
                    $scope.rpo_failed = compliance_data.rpo_failed;
                    $scope.rto_failed = compliance_data.rto_failed;

                    // setting MINPERCENT if either of the values is 0
                    $scope.success_percent = (compliance_data.ok_percent > 0) ? compliance_data.ok_percent : MINPERCENT;
                    $scope.failure_percent = (compliance_data.ok_percent === 100) ? MINPERCENT : (100 - compliance_data.ok_percent) % 1 == 0 ? 100 - compliance_data.ok_percent : (100 - compliance_data.ok_percent).toFixed(2);
                    $scope.complianceChartLegendArray = [
                        gettextCatalog.getString("Success") + " (" + compliance_data.ok_percent + "%)",
                        gettextCatalog.getString("Failure") + " (" + (100 - compliance_data.ok_percent) + "%)"
                    ];
                }
                var complianceChartDataArray = [$scope.success_percent, $scope.failure_percent];
                var title = "Compliance";

                if ($('#compliance_chart').length) {
                    createpieChartRDR("compliance_chart", complianceChartDataArray, $scope.chartColorArray,
                        $scope.complianceChartLegendArray, compliance_width, title);
                }

                var recovery_assurance_data = data.recovery_assurance;
                $scope.successful = recovery_assurance_data.successful;
                $scope.warning = recovery_assurance_data.warning;
                $scope.failed = recovery_assurance_data.failed;

            }).error(function (response) {

            });
        };

        /* Event listener for manual refresh */
        $scope.$on('manualRefreshEvent', function (event, widget,index) {
            if(widget.name === 'RECOVERY ASSURANCE') {
                $rootScope.getRecoveryAssuranceData();
                event.defaultPrevented = true;
            }
        });

        $scope.userPrefernce = function() {
            $scope.manualRefreshControl($rootScope.listOfTimer[11]/1000,$scope.widgetName);
        };

        $scope.cancelRefreshChange = function() {
            $rootScope.userPreferenceParentCancel($rootScope.listOfTimer[11]/1000, $scope.widgetName);
        };

        /* Event listener for leaving the widget */
        $scope.$on('leaveWidget', function (event, widget,index){
            if(widget.name === 'RECOVERY ASSURANCE'){
                if ($scope.modalShown) {
                    $scope.modalShown = false;
                    $timeout(function() {
                        $scope.$digest();
                    });
                }
            }
        });

        if ($scope.user.loggedIn) {
            $scope.userPrefernce();
            $rootScope.getRecoveryAssuranceData();
        }

        $(document)
            .on("login", stopAll)
            .on("logout", stopAll)
            .on("uinav.protect", stopAll)
            .on("uinav.recover", stopAll)
            .on("uinav.jobs", stopAll)
            .on("uinav.configure", stopAll)
            .on("uinav.reports", stopAll);

        function stopAll() {
            clearInterval($rootScope.listOfTimerFunction[11]);
            $scope.reSizeGraph = false;
            $(document).off("uinav.protect", stopAll);
            $(document).off("uinav.recover", stopAll);
            $(document).off("uinav.jobs", stopAll);
            $(document).off("uinav.configure", stopAll);
            $(document).off("uinav.reports", stopAll);
            $(document).off("login", stopAll);
            $(document).off("logout", stopAll);
        }

    });

function createpieChartRDR(holderId, dataPointsArray, colorsArray, legendArray, chart_width, title) {
    var r = Raphael(holderId,"100%","100%");
    var x_coordinate = (chart_width / 2) + 1;
    // changing just the x-coordinate;not the y-coordinate and radius as the height can't be changed
    var pie = r.piechart(x_coordinate, 60, 40, dataPointsArray, {minPercent:MINPERCENT, colors: colorsArray, legend: legendArray, legendpos: "south", matchColors: true});

    r.text(x_coordinate, 10, title).attr({ font: "1.5em sans-serif" });
    pie.hover(function () {
        this.sector.stop();
        this.sector.scale(1.1, 1.1, this.cx, this.cy);

        if (this.label) {
            this.label[0].stop();
            this.label[0].attr({ r: 7.5 });
            this.label[1].attr({ "font-weight": 600 });
        }
    }, function () {
        this.sector.animate({ transform: 's1 1 ' + this.cx + ' ' + this.cy }, 500, "bounce");

        if (this.label) {
            this.label[0].animate({ r: 5 }, 500, "bounce");
            this.label[1].attr({ "font-weight": 200 });
        }
    });
}


