helpApp.controller('feedbackCtrl', ['$scope', '$rootScope', '$http', '$route', '$window', 'helpFactory', 'ngDialog', 'gettextCatalog',
    function($scope, $rootScope, $http, $route, $window, helpFactory, ngDialog, gettextCatalog) {

        // Used to unbind the watched event on dialog close.
        var unbindHandler = null;

        $scope.initCtrl = function(obj){
            if (obj.name === 'feedback-form-dialog') {
                $scope.initDialog();
            }
        };

        // feedback to be sent, with name, mail, message.
        $scope.feedback = {};

        $scope.feedback.version = ''; // not sending now. $scope.ngDialogData.version;

        var feedbackForm;
        var $form;


        $scope.submitFeedback = function() {
            if ($form !== undefined && $form.valid()) {
                var url = "/api/feedback/";
                var feedbackObj = JSON.stringify($scope.feedback);
                var load = PHD.showLoadingIndicator('body', true, gettextCatalog.getString("Sending Feedback") + '...');

                $http ({
                    method: 'post',
                    url : url,
                    data: feedbackObj
                }).success(function(data) {
                    PHD.hideLoadingIndicator(load);
                    $scope.closeFeedbackDialog();
                }).error(function(response) {
                    PHD.hideLoadingIndicator(load);
                    var message = response.result !== undefined && response.result[0] !== undefined ? response.result[0].message : gettextCatalog.getString("Error sending feedback.  Please try again.");
                    ngDialog.open({
                        dialogType: 'error',
                        modelDialogId:'ErrorDialog',
                        dialogMessage: message,
                        scope:$scope,
                        onConfirmOkButtonClick:'onConfirmError()'
                    })
                })
            }
        };


        $scope.initDialog = function () {

            $form = $("#form-feedback");

            function initForm() {
                $form.find(".definition").popover({
                    appendTo: $form,
                    position: {
                        my: "right",
                        at: "right+10 center",
                        collision: "flip fit",
                        within: $form
                    }
                });

                feedbackForm = PHD.FormController($form, {})
                    .validate({
                        errorLabelContainer: ".feedback-dialog-error-list",
                        errorContainer: ".feedback-dialog-errors",
                        showErrors: function(errorMap, errorList) {
                            $(".feedback-dialog-errors").find(".summary")
                                .html(gettextCatalog.getString("Correct the following errors:"));
                            this.defaultShowErrors();
                        },
                        rules: {
                            name: {
                                required: true
                            },
                            email: {
                                required: true,
                                email: true
                            },
                            message: {
                                required: true
                            }
                        },
                        messages: {
                            name: {
                                required: gettextCatalog.getString("Your name is required.")
                            },
                            email: {
                                required: gettextCatalog.getString("Enter a valid email address."),
                                email: gettextCatalog.getString("Your email address must be in the form name@domain.com.")
                            },
                            message: {
                                required: gettextCatalog.getString("Please include your feedback.")
                            }
                        }
                    });

                // Override default FormController submit.
                feedbackForm.submit = function() {
                    $scope.submitFeedback();
                };
            }

            initForm();

        };

        $scope.closeFeedbackDialog = function() {
            if ( _.isFunction(unbindHandler)) {
                unbindHandler();
            }
            $scope.closeThisDialog();
        };

        $scope.onConfirmError = function() {
            ngDialog.close('ErrorDialog');
        };

        unbindHandler = $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });


    }]);
