var helpApp = angular.module('help', ['ngDialog']);

helpApp.factory('helpFactory', function($http) {
    return {
        async: function(url, extra_params, callback) {
            var params = {};
            params.ts = Date.now();
            if (extra_params != null) {
                params = $.extend(params, extra_params);
            }
            var promise = $http({
                url: url,
                method: "get",
                params: params
            }).success(function(data) {

            }).error(function(){
                window.PHD.throwError({"result": [{"message": gettext("Cannot load help information.")}]});
            });
            return promise;
        }
    };
});


helpApp.controller('HelpCtrl', ['$scope', '$rootScope', '$http', '$route', '$window', 'helpFactory', 'ngDialog', 'gettextCatalog',
    function($scope, $rootScope, $http, $route, $window, helpFactory, ngDialog, gettextCatalog) {

        $scope.getSupportURL = function() {
            return "http://www.unitrends.com/support";
        };

        $scope.getOnlineHelpURL = function() {
            var url = "https://unitrends-support.force.com/";
            var free_article = "UnitrendsFree/s/article/";
            if ($rootScope.isGoogle) {
                url += free_article + "000003715";
            } else {
                url += "UnitrendsBackup/s/";
            }
            return url;

        };

        $scope.getKnowledgeBaseURL = function() {
            return $rootScope.isCE ? "https://unitrends-support.force.com/UnitrendsFree" : "http://www.unitrends.com/support/knowledge-base";
        };

        $scope.tunnelNumber = 0;
        $scope.isTunnelOpen = false;

        $scope.changeTunnelState = function() {

            var obj = {name: "Support Tunnel"};
            obj = JSON.stringify(obj);
            var url = "/api/commands";
            var load = null;
            // For opening the tunnel, need to tell user to wait as it takes awhile.
            if (!$scope.isTunnelOpen) {
                var message =  gettextCatalog.getString("Opening Support Tunnel. Please Wait...");
                load = PHD.showLoadingIndicator("body", true, message);
            } else {
                var message = gettextCatalog.getString("Closing Support Tunnel. Please Wait...");
                load = PHD.showLoadingIndicator("body", true, message);
            }
            //this is the call to actually open/close the support tunnel
            var resp = PHD.Ajax.post(url, obj, load);

            resp.done(function (data) {
                $scope.isTunnelOpen = !$scope.isTunnelOpen;
                // A successful return will be a string with "Creating tunnel: <number>\n...
                var r = /\d+/;
                var message = data.data;
                $scope.tunnelNumber = message.match(r);

                if (!$scope.isTunnelOpen) {
                    if ( $scope.tunnelNumber > 1 ) {
                                    PHD.alert(gettextCatalog.getString("Support Tunnel ") + $scope.tunnelNumber + gettextCatalog.getString(" Successfully Closed."));
                                    $scope.tunnelNumber = 0;
                    } else {
                                    PHD.alert(gettextCatalog.getString("Support Tunnel Successfully Closed."));
                                    $scope.tunnelNumber = 0;
                    }
                } else {
                    if ( $scope.tunnelNumber > 1 ) {
                                    PHD.alert(gettextCatalog.getString("Support Tunnel ") + $scope.tunnelNumber + gettextCatalog.getString(" Successfully Opened."));
                    } else {
                                    PHD.alert(gettextCatalog.getString("Support Tunnel Successfully Opened."));
                    }
                }
            });
        };

        $scope.getTunnelStatus = function() {
            var obj = {name: gettextCatalog.getString("Active Support Tunnel")};
            obj = JSON.stringify(obj);
            var url = "/api/commands";
            var resp = PHD.Ajax.post(url, obj);

            resp.done(function (data) {
                var tunnel = parseInt(data.data);
                $scope.$apply(function(){
                    $scope.isTunnelOpen = tunnel > 0;
                    if ( $scope.tunnelNumber > 1 ) {
                        $scope.tunnelNumber = tunnel;
                    }
                });
            });
        };

        $scope.getFeedbackURL = function(){
            return FEEDBACK_URL + "/topic/0TO40000000PMPcGAO/Product Ideas";
        };

        $scope.getTourURL = function() {
            return "#";
        };
        $scope.startTour = function() {
            $scope.user.showTour = true;
            var loc = $window.location.href;
            // redirect to main page if not there already.
            if (!loc.match(/#\/$/)) {
                $window.location.href = "#";
            }
            $route.reload();
        };

        function displayAbout(data) {
            $scope.appliance = data.appliance;
            
            $scope.appliance.browser_info = bowser.ua;
            $scope.appliance.browser_name = bowser.chrome ? "Chrome" :
                                            bowser.msie ? "Internet Explorer" :
                                            bowser.msedge ? "Microsoft Edge" :
                                            bowser.firefox ? "Firefox" : gettextCatalog.getString("Other/Unknown");
            $scope.appliance.browser_resolution = screen.width + " x " + screen.height;
            
            ngDialog.open({
                template: 'app/help/about.html',
                scope: $scope,
                overlay:true,
                ngDialogStyle:'width: 850px'
            });
        }

        $scope.runAbout = function() {
            var load = PHD.showLoadingIndicator('body', true, gettextCatalog.getString('Loading information...'));
            helpFactory.async('/api/systems/details', {}).then(function(data) {
                PHD.hideLoadingIndicator(load);
                displayAbout(data.data);
            });
        };


        /*
         The Feedback Form
         */
        function displayFeedback(data) {
            var data = data.appliance;
            for (var i = 0; i < data.length; i++) {
                var obj = data[i];
                if (obj.local) {
                    if ($scope.dialogOpen === false || $scope.dialogOpen === undefined) {
                        $scope.dialogOpen = true;
                        ngDialog.open({
                            template: 'app/help/feedback.html',
                            scope: $scope,
                            overlay: true,
                            data: obj,
                            modelDialogId: 'feedback-form-dialog',
                            name: 'feedback-form-dialog',
                            ngDialogStyle: 'top:-80px;width:525px; height:580px;',
                            closeByDocument: false,
                            closeByEscape: true,
                            preCloseCallback: function (value) {
                                $scope.dialogOpen = false;
                            }
                        });
                        break;
                    }
                }
            }
        }

        /*
         Get the system info to display in the feedback email.
         */
        $scope.runFeedback = function() {
            helpFactory.async('/api/systems/', {}).then(function(data) {
                displayFeedback(data.data);
            });
        };
		$scope.registerAssetForSupport = function() {
			console.log("in registerAssetForSupport"+ PHD.appliance_sid);
			var info_url = "/api/license/request/?sid=" + PHD.appliance_sid;
			var load = PHD.showLoadingIndicator('body', true, gettextCatalog.getString('Gathering information for registration page...'));
			var resp = PHD.Ajax.get(info_url, load);
			resp.done(function(data) {
				var upgrade_link;
				if (data.request.link !== undefined) {
					upgrade_link = data.request.link;
				}
				var licenseObj = data.request.registration;
				upgrade_link.data = licenseObj;

				var upgradeForm = $('<form></form>');
				upgradeForm.attr('id', 'id_upgrade_form');
				upgradeForm.attr('action', upgrade_link);
				upgradeForm.attr('method', 'post');
				upgradeForm.attr('target', '_blank');
				for (name in licenseObj) {
					var licenseInput = $("<input>");
					licenseInput.attr('name', name);
					licenseInput.attr('value', licenseObj[name]);
					upgradeForm.append(licenseInput);
				}
				// append to body (required by some browsers.
				upgradeForm.appendTo('body').submit();
				setTimeout(function() {
					upgradeForm.remove();
				}, 2000);
			});
        };

        $scope.showReleaseNotes = function() {
            var version = PHD.Ajax.get("/api/systems/details");
            version.done(function(data) {
                $scope.version = data.appliance.version;
            });
            $scope.onDemand = true;

            ngDialog.open({
                template: 'app/release-notes/release-notes.html',
                scope: $scope,
                overlay:true,
                modelDialogId: 'release-notes-dialog',
                ngDialogStyle:'width:670px; height:325px;',
                closeByDocument: false,
                closeByEscape: true
            });
        };
    }]

);
