var backupStatusApp = angular.module('backup-status-module', []);

backupStatusApp.controller('backupStatusCtrl', ['$scope', '$rootScope', '$http', '$analytics', 'ngDialog', 'gettextCatalog',
    function($scope, $rootScope, $http, $analytics, ngDialog, gettextCatalog) {

        // Used to unbind the watched event on dialog close.
        var unbindHandler = null;

        $scope.dialogTitle = $scope.ngDialogData.title;

        function processDialog(PHD, window, $rootScope, $scope, $analytics) {

            var $jobLogTable = $("#job-log-table"),
                LogTable, url;
            var $form = $("#status-form");
            var statusForm;
            var allBackups;
            var backupCopyTargetHost;
            var backupToQueue = 0;
            var $replicateButton = $(".btn-replicate-backup");

            function initTable() {

                var sid = PHD.appliance_sid;
                var bid = backup_ids;
                var showCopies = copies;
                url = "/api/backups/?sid=" + sid + "&bid=" + bid + "&lang=" + $rootScope.userLanguageCode;

                if (showCopies === true) {
                    url += "&copies=1";
                }

                statusForm = PHD.FormController($form, {});

                LogTable = PHD.DataGrid($jobLogTable, {
                    dynamic: true,
                    tableScrollOptions: {"height": "article"},
                    select: false,
                    sortable: true,
                    sortOnLoad: true,
                    sortOnHeader: true,
                    cellClasses: [null, null, null, null, "full-text"],
                    classifiers: ['status']
                }).registerFormatters(cellFormatter);

                updateTable();

                function updateTable() {
                    PHD.Ajax
                        .get(url)
                        .done(function (data) {
                            $(".btn-delete-backup").hide();
                            $replicateButton.hide();
                            if (data.BackupStatus === undefined || data.BackupStatus.length === 0) {
                                var obj = {};
                                obj.id = 0;
                                obj.status = 'None';
                                obj.start_time = 'unknown';
                                obj.output = 'The backup(s) no longer exist on the appliance.';
                                data.BackupStatus = [obj];
                            } else {
                                for (var i = 0; i < data.BackupStatus.length; i++) {
                                    // Did any backups fail?
                                    var backupDetail = data.BackupStatus[i];
                                    if (backupDetail.status === 'Failed' && !$rootScope.configureDisabled) {
                                        $(".btn-delete-backup").show();
                                        break;
                                    }
                                }
                                // See if any are successful, look for the most recent full to add to the queue.
                                if (!showCopies && data.BackupStatus.length >= 1) {
                                    for (i = 0; i < data.BackupStatus.length; i++) {
                                        backupDetail = data.BackupStatus[i];
                                        if (backupDetail.type === 'Full' && backupDetail.status === 'Successful') {
                                            if (backupDetail.id > backupToQueue) {
                                                backupToQueue = backupDetail.id;
                                            }
                                        }
                                    }
                                    // If we found any backups to queue, check for targets.
                                    if (backupToQueue > 0) {
                                        getHotBackupCopyTargets();
                                    }
                                }
                            }
                            allBackups = data.BackupStatus;
                            LogTable.load(data.BackupStatus);
                        });
                }

                function getHotBackupCopyTargets() {
                    var url = '/api/replication/targets/?sid=' + PHD.appliance_sid;
                    console.log(url);
                    var resp = PHD.Ajax.get(url);
                    resp.done(function (data) {
                        if (data.targets !== undefined && data.targets.length > 0) {
                            var target = data.targets[0];
                            backupCopyTargetHost = target.host;
                            url = '/api/replication/is_in_queue/?backup_ids=' + backupToQueue + "&sid=" + PHD.appliance_sid;
                            console.log(url);
                            // See if the backup is in the Queue already
                            var queueResp = PHD.Ajax.get(url);
                            queueResp.done(function (data) {
                                if (data.QueueStatus !== undefined && data.QueueStatus.length === 1) {
                                    var queue = data.QueueStatus[0];
                                    if (queue.status === '') {
                                        $replicateButton.show();
                                        if (allBackups.length > 1) {
                                            $replicateButton.attr('title', 'Copy the most recent successful Full backup to the target');
                                        } else {
                                            $replicateButton.attr('title', 'Copy this backup to the target');
                                        }
                                    }
                                }
                            })
                        }
                    });
                }
            }

            $(".btn-delete-backup").on("click", function() {
                var failedBids = [];
                var failedBackups = [];
                var successCount = 0;
                var warningCount = 0;
                allBackups.forEach(function(obj, index){
                    if (obj.status == "Failed"){
                        failedBackups.push(obj);
                        failedBids.push(obj.id);
                    }
                    if (obj.status == "Successful"){
                        successCount++;
                    }
                    if (obj.status == "Warning"){
                        warningCount++;
                    }
                });
                if (failedBids.length !== 0){
                    data = failedBackups;
                    var message = gettextCatalog.getString("The following failed backups will be deleted:");
                    message = '<h4 class="text-ok">' + message + '</h4>';
                    message += backupTmpl(failedBackups);
                    confirm = PHD.confirmDialog(
                        message,
                        {
                            title: gettextCatalog.getString("Confirm Backup Deletion"),
                            open: function (event, ui) {
                                var $dialog = $(this);
                                $dialog.find("table.data-table").scrollTable({"height": 120, "resize": false});
                                $dialog.find(".ui-dialog-buttonset button:last-child").focus();
                                $dialog.dialog("option", "position", {my: "center", at: "center", of: window});
                                $dialog.find(".definition").popover({
                                    appendTo: $dialog,
                                    position: {
                                        my: "left-30 bottom",
                                        at: "right bottom+40",
                                        within: $dialog
                                    }
                                });
                            }
                        }
                    );
                    $('.dialog-confirm').css('border', '1px solid #ccc');
                    $('.dialog-confirm').css('zIndex', '10001');
                    $('.dialog-confirm').css('width', '700px');

                    confirm.done(function () {
                        var str_ids = failedBids.toString();
                        var sid = (PHD.appliance_sid === null) ? "" : "/?sid=" + PHD.appliance_sid;
                        var url = "/api/backups/" + str_ids + sid;
                        var resp = PHD.Ajax.delete(url);

                        resp.done(function(data) {
                            console.log("done with backup deletion, clearing ids: " + str_ids);
                            $scope.closeThisDialog();
                            $(document).trigger('failedJobDeleted', [failedBids, successCount, warningCount, PHD.appliance_sid]);

                        });
                    });

                }
            });

            $replicateButton.on("click", function() {
                var url = '/api/replication/add/?sid=' + PHD.appliance_sid;
                var payload = {'backup_id': backupToQueue, 'target': backupCopyTargetHost};
                payload = JSON.stringify(payload);
                var load = PHD.showLoadingIndicator($replicateButton, false);
                var resp = PHD.Ajax.post(url, payload, load);
                resp.done(function(data) {
                    var notice = {};
                    var message = gettext("A Hot Backup Copy Job has been started") + " " +
                        (allBackups.length > 1 ? gettext("for the most recent successful backup") : gettext("for this backup")) + ".";
                    notice.result = [ {'message': message}];
                    PHD.throwNotice(notice);
                    $scope.closeThisDialog();
                });
            });

            var cellFormatter = {
                output: function(value, row) {
                    var start = 'Start Date: ' + row.start_time + '<br/>';
                    var end = row.complete_time !== null && row.duration !== null  ? ('End Date: ' + row.complete_time + ' (Elapsed: ' + row.duration + ' seconds)' + '<br/>') : "";
                    return start + end + value.replace(/\n/g, "<br />");
                }
            };

            var backupTmpl = _.template(
                '<table class="backup-list user-backups data-table dashboard-table">' +
                '<thead><tr>' + '<th> </th>' +
                '<th>' + gettextCatalog.getString("Host/Server") + '</th>' +
                '<th>' + gettextCatalog.getString("Asset") + '</th>' +
                '<th>' + gettextCatalog.getString("Type") + '</th>' +
                '<th>' + gettextCatalog.getString('Status') + '</th>' +
                '<th>' + gettextCatalog.getString("Date") + '</th>' +
                '</tr></thead>' +
                '<tbody>' +
                '<% for(var i=0;i<data.length;i++) { %>' +
                '<% data[i].client_name = data[i].client_name ; %>' +
                '<% data[i].asset_name = data[i].asset_name; %>' +
                '<tr>' +
                '<td>' +
                '<% if(data[i].gfs_rp_type !== "") { %>' +
                '<div class="help-trigger"><i class="icon-infosign"></i></div>' +
                '<span class="definition"><translate>This backup was kept due to a retention policy.</translate></span>' +
                '<% } %>' + '</td>' +
                '<td><span class="truncates"><%= data[i].client_name %></span></td>' +
                '<td><%= data[i].asset_name %></td>' +
                '<td><%= data[i].type %></td>' +
                '<td><%= data[i].status %></td>' +
                '<td><%= data[i].start_time %></td>' +
                '</tr>' +
                '<% } %>' +
                '</tbody></table>'
            );

            /*
            *   DOM Ready Function
            */
            initTable();
        }

        $scope.closeThisDialog = function() {
            if ( _.isFunction(unbindHandler)) {
                unbindHandler();
            }
            ngDialog.close('backup-status-dialog');
        };

        $scope.initCtrl = function (dlg) {
            if (dlg.name === 'backup-status-dialog') {
                // Only run initialization for the main dialog.
                processDialog(PHD, window, $rootScope, $scope, $analytics);
            }
        };

        // Initialize the controller.
        unbindHandler = $scope.$on('ngDialog.opened', function (event, obj) {
            $scope.initCtrl(obj)
        });
    }
    ]);

backupStatusApp.controller('backupCopyStatusCtrl', ['$scope', '$rootScope', '$http', '$analytics', 'ngDialog',
    function($scope, $rootScope, $http, $analytics, ngDialog) {

        // Used to unbind the watched event on dialog close.
        var unbindHandler = null;

        $scope.dialogTitle = $scope.ngDialogData.title;

        function processDialog(PHD, window, $rootScope, $scope, $analytics) {

            var $jobLogTable = $("#backup-copy-table"),
                LogTable, url;
            var $form = $("#status-form");
            var statusForm;

            function initTable() {

                var sid = PHD.appliance_sid;
                var ids = copy_ids;
                if (ids !== null && ids[0] !== undefined && ids[0] === ',') {
                    ids = ids.slice(1);
                }
                var idJSON = {};
                if (!(ids instanceof String)) {
                    // convert to a string for common handling.  On-Demand Backup Copies will have an id,  schedules do not.
                    ids = String(ids);
                }
                var pieces = ids.split('.');
                if (pieces.length === 4) {
                    // If the ID is of the form x.y.'i'.count, we know we have sid.timestamp.i.count
                    // The status API will find return the backups that match the first set closest to this timestamp.
                    // Since archiving single threads, we should be able to rely on using hte first set.
                    sid = parseInt(pieces[0], 10);
                    url = "/api/archive/status/?sid=" + sid + "&lang=" + $rootScope.userLanguageCode;
                    idJSON.start_timestamp_archive_set = parseInt(pieces[1], 10);
                } else {
                    // If the ID is of the form xa,ya,za, break them apart call the API with the IDs to return.
                    url = "/api/archive/status/?sid=" + sid + "&lang=" + $rootScope.userLanguageCode;
                    idJSON.archive_ids = ids.split(',').map(function(id) {
                        return parseInt(id, 10);
                    });
                }
                idJSON = JSON.stringify(idJSON);

                statusForm = PHD.FormController($form, {});

                LogTable = PHD.DataGrid($jobLogTable, {
                    dynamic: true,
                    tableScrollOptions: {"height": "article"},
                    select: false,
                    sortable: true,
                    sortOnLoad: true,
                    sortOnHeader: true,
                    cellClasses: [null, null, 'long-text truncate', null, null, null, null, null, null, null],
                    classifiers: ['success']
                }).registerFormatters(cellFormatter);

                updateTable();

                function updateTable() {
                    PHD.Ajax
                        .post(url, idJSON)
                        .done(function(data) {
                            if (data.status === undefined || data.status.length == 0) {
                                var obj = {};
                                obj.instance_description = "The backup copy information is no longer available.";
                                data.status = [obj];
                            }
                            LogTable.load(data.status);
                        });
                }
            }

            var cellFormatter = {
                instance_description: function(value) {
                    var title = _.escape(value);
                    return '<span class="truncates item-name" title="' + title + '">' + value + '</span>';
                },
                success: function (value, row) {
                    var iconClass = value == undefined || value == '--' ? 'warning' : value === true ? "success" : "error";
                    return '<i class="icon-phd-'+ iconClass +' icon-large" title="' + iconClass + '"></i>';
                },
                encrypted: function (value, row) {
                    var isEncrypted = value;
                    if (isEncrypted !== '--') {
                        isEncrypted = (value === true) ? "Yes" : "No";
                    }
                    return(isEncrypted);
                },
                size: function (value, row) {
                    var gbSize = value;
                    if (gbSize !== '--') {
                        gbSize = (value/1024).toFixed(2);
                    }
                    return(gbSize);
                }
            };

            /*
            *   DOM Ready Function
            */
            initTable();
        }


        $scope.closeThisDialog = function() {
            if ( _.isFunction(unbindHandler)) {
                unbindHandler();
            }
            ngDialog.close('backup-copy-status-dialog');
        };

        $scope.initCtrl = function (dlg) {
            if (dlg.name === 'backup-copy-status-dialog') {
                // Only run initialization for the main dialog.
                processDialog(PHD, window, $rootScope, $scope, $analytics);
            }
        };

        // Initialize the controller.
        unbindHandler = $scope.$on('ngDialog.opened', function (event, obj) {
            $scope.initCtrl(obj)
        });
    }
    ]);
