var backupCopyApp = angular.module('backup-copy-module', ['download', 'angulartics', 'angulartics.google.analytics',
    'protect','ngDialog', 'ngTouch',
    'ui.grid.treeView','ui.grid.pinning','ui.grid','ui.grid.selection','ui.grid.autoResize','ui.grid.resizeColumns', 'ui.grid.expandable']);

backupCopyApp.controller('BackupCopyCtrl', ['$scope', '$rootScope', '$http',
    '$analytics','$protectService','ngDialog', '$timeout','$interval','uiGridTreeViewConstants','$q', 'gettextCatalog',
    function($scope, $rootScope, $http, $analytics, $protectService, ngDialog, $timeout, $interval, uiGridTreeViewConstants, $q, gettextCatalog) {

        angular.element(document).ready(function () {
            processBackupCopyDialog(PHD, window, $rootScope, $scope, $analytics);
        });

        function processBackupCopyDialog(PHD, window, $rootScope, $scope, $analytics) {
            var $form = $("#backup-job-form"), // The main <form> element
                BackupForm,
                $backupTreeContainer = $("#backup-tree-0"),

                // Inventory table management elements
                $inventoryTable = $("#job-inventory-table"),
                $inventoryCounter = $("#id_joborderoverride_set-TOTAL_FORMS"),
                inventoryRows = $("#id_joborderoverride_set-INITIAL_FORMS").val(),

                re = /__prefix__/gi,
                $inventoryRowTemplate = $("#inventory-row-template"),
                $inventoryRowEdit = $("#inventory-row-edit"),
                $selectedRow, $currentEditRow,

                inventory = {},

                EmailTable,

                recurrenceSpinner,
                $recurrenceCheckbox = $("#recurrence-checkbox"),
                $recurrenceInput = $("#id_recurrence"),
                $recurrencePeriod = $("#recurrence-period"),
                $recurrenceAmount = $("#recurrence-amount"),
                recurrenceAmount = $recurrenceAmount.val(),

                retentionSpinner,
                $retentionCheckbox = $("#retention-checkbox"),
                $retentionInput = $("#id_retention"),
                $retentionPeriod = $("#retention-period"),
                $retentionAmount = $("#retention-amount"),
                retentionAmount = $retentionAmount.val(),

                retentionOptions = {
                    years: {
                        max: 999,
                        min: 1,
                        step: 1
                    },
                    months: {
                        max: 999,
                        min: 1,
                        step: 1
                    },
                    weeks: {
                        max: 999,
                        min: 1,
                        step: 1
                    },
                    days: {
                        max: 999,
                        min: 1,
                        step: 1
                    }
                },

                $startTime = $("#cal_incr_start_time"),
                $endTime = $("#id_end_time"),
                $fullEndTimeToggle = $("#run-full-end-time-toggle"),
                $timeInputs = $(".time-input input"),

                backupStrategy = $("#id_backup_strategy option:selected").val(),
                scheduleType = parseInt($('[name="schedule_type"]:checked').val(), 10) || SCHEDULE_TYPE_SCHEDULED,
                TreeViewData,
                backupJobTreeview, backupJobTreeviewHeader, backupJobTreeviewSearch;

            // Info for adding/saving a job
            var system_id;
            var client_id;
            var instance_id;

            // info for inventory
            var num_clients = 0;
            var num_instances = 0;

            var inventoryRowDefaults = {
                name: "Unknown",
                app_aware: "Not Enabled"
            };

            var savedJobName = $("#id_name").val();

            var $customEditButton = $(".edit-custom-calendar");
            var $scheduleMode = $("#id-copy-mode");
            var isCustomJob = false;
            var calendarDescription;        // Will store the updated calendar description.


            var InventoryRow = function (objectID, rowCount, values) {
                if (this instanceof InventoryRow) {
                    this.ID = objectID;
                    this.rowCount = rowCount;
                    this.values = values || inventoryRowDefaults;
                    this.idRegex = /__prefix__/g;
                    this.sourceRegex = /joborderoverride_set-(__prefix__|\d{1,3}|__edit__)-/ig;
                } else {
                    return new InventoryRow(objectID, rowCount, values).init();
                }
            };

            var AdoptedInventoryRow = function ($el, rowCount) {
                var objectID = $el.data("dbid"),
                    values = {name: $el.find(".item-name").text().trim()},
                    ctrlRegex = /joborderoverride_set-(__prefix__|\d{1,3}|__edit__)-/ig;

                $el.find(".inputs").children().each(function () {
                    var $ctrl = $(this),
                        source = $ctrl.attr("name").replace(ctrlRegex, "");
                    values[source] = InventoryRow.prototype.extractValue.call(this, $ctrl, false);
                });

                return new InventoryRow(objectID, rowCount, values).adopt($el);
            }

            InventoryRow.prototype = {
                init: function () {
                    var self = this;
                    this.$el = null;
                    this.$inputs = null;
                    this.rendered = false;
                    $inventoryTable.on("backupjobeditall", function (event, $form) {
                        self.update($form);
                    });
                    return this.render();
                },

                getName: function () {
                    return this.values.name;
                },

                getValue: function (source) {
                    var $ctrl = $inventoryRowEdit.find("[name$=" + source + "]");
                    if ($ctrl.is("select")) {
                        return $ctrl.find("option[value='" + this.values[source] + "']").text();
                    } else if ($ctrl.is("input:checkbox")) {
                        return humanize($ctrl.prop("checked"));
                    } else {
                        return this.values[source];
                    }
                },

                render: function () {
                    var self = this,
                        html = $inventoryRowTemplate.html().replace(this.idRegex, this.rowCount);

                    this.$el = $("<tr>", {
                        "class": "inventory-row"
                    })
                        .attr("data-dbid", this.ID)
                        .html(html);

                    this.$el.find("td:not(.row-controls)")
                        .each(function () {
                            var $cell = $(this),
                                value = self.getValue($cell.data("source"));
                            $cell.html(value);
                        });

                    this.$el.find(".item-id input").val(this.ID);

                    this.$el.find("[data-source=name]").attr("title", this.values.name);

                    this.$el.find("button").data("row", this);
                    this.$inputs = this.$el.find(".inputs");

                    this.persistValues();

                    this.rendered = true;
                    return this;
                },

                insert: function ($container) {
                    if (this.$el && $container) {
                        $container.append(this.$el);
                    }
                    return this;
                },

                edit: function () {
                    var self = this.prepareEditDialog($inventoryRowEdit);

                    $inventoryRowEdit.dialog({
                        title: gettext("Edit settings for ") + self.getName(),
                        resizable: false,
                        draggable: false,
                        modal: true,
                        width: 340,
                        dialogClass: "wizard-edit wizard-backup-edit",
                        buttons: [
                            {
                                text: gettext("Save"),
                                click: function () {
                                    self.update($(this));
                                    $(this).dialog("close");
                                }
                            },
                            {
                                text: gettext("Cancel"),
                                click: function () {
                                    $(this).dialog("close");
                                }
                            }
                        ]
                    });
                },

                // Given a control (checkbox or select), extract a value optimized
                // for storage in JavaScript or for display
                extractValue: function ($ctrl, display) {
                    if ($ctrl.is("select")) {
                        return display ?
                            $ctrl.find("option:selected").text() :
                            $ctrl.find("option:selected").val();
                    } else if ($ctrl.is("input:checkbox")) {
                        return display ?
                            humanize($ctrl.prop("checked")) :
                            $ctrl.prop("checked");
                    } else {
                        return false;
                    }
                },

                update: function ($form) {
                    var self = this;
                    return this.saveValues($form.find("input, select"))
                        .persistValues();
                },
                // Saves values to self.values[] and in the html for display
                saveValues: function ($controls) {
                    var self = this;
                    $controls.each(function (index) {
                        var $ctrl = $(this),
                            source = $ctrl.attr("name").replace(self.sourceRegex, "");

                        self.values[source] = self.extractValue($ctrl, false);
                        self.$el
                            .find("[data-source=" + source + "]")
                            .html(self.extractValue($ctrl, true));
                    });
                    return this;
                },
                // Saves values to the row's hidden controls
                persistValues: function ($form, values) {
                    var values = (typeof values === 'undefined') ? this.values : values,
                        $form = (typeof $form === 'undefined') ? this.$inputs : $form;
                    $.each(values, function (key, value) {
                        var $ctrl = $form.find("[name$=" + key + "]");
                        if ($ctrl.is("select")) {
                            $ctrl.val(value).trigger("change");
                        } else if ($ctrl.is(":checkbox")) {
                            $ctrl.prop("checked", value);
                        }
                    });
                    return this;
                },

                prepareEditDialog: function ($form) {
                    return this.persistValues($form);
                },

                delete: function () {
                    this.$el
                        .addClass("deleted")
                        .find(".delete-box input")
                        .prop("checked", true);
                    return this;
                },

                undelete: function () {
                    this.$el
                        .removeClass("deleted")
                        .find(".delete-box input")
                        .prop("checked", false);
                    return this;
                },

                adopt: function ($row) {
                    var self = this;
                    $inventoryTable.on("backupjobeditall", function (event, $form) {
                        self.update($form);
                    });
                    this.$el = $row;
                    this.$inputs = $row.find(".inputs");
                    this.rendered = true;
                    return this;
                }
            };

            function humanize(value) {
                if (value === false) {
                    return gettext("No");
                }
                if (value === true) {
                    return gettext("Yes");
                }
                return value;
            }


            function treeViewCheckClickHandler(event, $node, simulated) {
                var dbId = $node.data("dbid"),
                    type = $node.data("node");

                if (!simulated) {
                    $form.validate().element($("#treeview-validator"));
                }

                if ($node.hasClass("expl-included")) {
                    var data_type = $node.data("type");
                    // UEB does not support containers, so we need to selected all instances when a container is selected.
                    // Support containers for both VMware and XenServer.
                    if (data_type == INVENTORY_ID_VMWARE_ESX_SERVER || data_type == INVENTORY_ID_VMWARE_FOLDER || data_type == INVENTORY_ID_VMWARE_RESOURCE_POOL ||
                        data_type == INVENTORY_ID_XEN_POOL_MASTER || data_type == INVENTORY_ID_XEN_SERVER_IN_A_POOL ||
                        data_type == INVENTORY_ID_EXCHANGE_APPLICATION || data_type == INVENTORY_ID_HYPER_V_SERVER || data_type == INVENTORY_ID_ORACLE_APPLICATION ||
                        data_type == INVENTORY_ID_SQL_SERVER || data_type == INVENTORY_ID_SQL_INSTANCE || data_type == INVENTORY_ID_NAVGROUP || data_type == INVENTORY_ID_NDMP_CLIENT ||
                        data_type == INVENTORY_ID_AHV_CLUSTER) {
                        // Add child objects from inventory when a container is checked
                        if ($node.find(".children").length) {
                            $node.find(".children .checkable").not(".expl-excluded").each(function (index) {
                                var add_type = $(this).data("type");
                                // UEB only handles VMs and templates, for either VMware or XenServer.
                                // ALso add group children, but not groups themselves.
                                if (add_type == INVENTORY_ID_VMWARE_VM || add_type == INVENTORY_ID_VMWARE_TEMPLATE ||
                                    add_type == INVENTORY_ID_XEN_VM || add_type == INVENTORY_ID_XEN_CUSTOM_TEMPLATE ||
                                    add_type == INVENTORY_ID_AHV_VM ||
                                    add_type == INVENTORY_ID_EXCHANGE_DATABASE || add_type == INVENTORY_ID_HYPER_V_VM ||
                                    add_type == INVENTORY_ID_ORACLE_DATABASE || add_type == INVENTORY_ID_SQL_DATABASE ||
                                    (data_type == INVENTORY_ID_NAVGROUP && add_type !== INVENTORY_ID_NAVGROUP) ||
                                    add_type == INVENTORY_ID_NDMP_VOLUME) {
                                    var alreadyPresent = checkForInstance($(this).data("dbid"));
                                    if (!alreadyPresent) {
                                        addInventoryRow($(this), $(this).data("dbid"));
                                        addInstance($(this), $(this).data("dbid"), add_type);
                                    }
                                }
                            });
                            $inventoryCounter.val($inventoryTable.find(".inventory-row").length);
                        }
                    } else {
                        // Add the row to inventory table
                        var alreadyPresent = checkForInstance(dbId);
                        if (!alreadyPresent) {
                            addInventoryRow($node, dbId);
                            // lmc
                            addInstance($node, dbId, data_type);
                            // keep track of the number of rows in inventory
                            $inventoryCounter.val($inventoryTable.find(".inventory-row").length);
                        }
                    }
                } else if ($node.hasClass("impl-included")) {
                    // Mark the row for deletion
                    removeInventoryRow(dbId);
                    // lmc
                    removeInstance($node, dbId);
                } else if ($node.hasClass("expl-excluded")) {
                    removeInventoryRow(dbId);
                    // lmc
                    removeInstance($node, dbId);
                    // Remove child objects from inventory when a container is unchecked
                    if ($node.find(".children").length && type !== "vm") {
                        $node.find(".children .checkable").not(".expl-included").each(function (index) {
                            removeInventoryRow($(this).data("dbid"));
                            // lmc - remove these from the list to protect.
                            removeInstance($(this), $(this).data("dbid"));
                        });
                    }
                }
                $inventoryTable.scrollTable({"height": ".modal-form-col", "resize": false});
            }

            function addInventoryRow($el, dbId) {
                // See if the object is already in the table and marked for deletion. If that is
                // the case, just unmark it and return true
                if ($inventoryTable.find('tr[data-dbid="' + dbId + '"]').length > 0) {
                    unDeleteInventoryRow(dbId);
                    return true;
                }

                // If the object we are adding has an already-included tree parent in inventory,
                // find and store it here so that its settings can be used
                var treeParentId = hasIncludedTreeParent($el, dbId),
                    defaults = inventoryRowDefaults;
                if (treeParentId !== false) {
                    defaults = inventory[treeParentId].values;
                }

                defaults = $.extend({}, defaults, {name: $el.children(".header").children(".item-name").text()});
                inventory[dbId] = InventoryRow(dbId, inventoryRows, defaults).insert($inventoryTable.children("tbody"));

                // Increment the inventory row counter
                inventoryRows++;
            }

            function addInstance($el, dbid, data_type) {
                var i = inventoryRows - 1;
                var rowText = "";
                if (isNaN(dbid)) {
                    console.log("In addInstance, dbid = " + dbid);
                    var a = dbid.split('_');
                    if (system_id === undefined) {
                        // Only set sid if system_id is not defined first.  It will be with pre-selected items.
                        system_id = a[0];
                        PHD.appliance_sid = system_id;
                    }
                    /*if (a.length == 2 || (a.length == 4 && a[3] == 1)) {
                     client_id = a[1];
                     rowText = '<input type="hidden" name="clients[' + num_clients++ + ']" value="' + client_id + '" id="id_inventory_' + i + '" data-itemtype="number" />';
                     console.log("adding client id " + client_id);
                     } else  */
                    if (a.length > 3) {
                        client_id = a[1];
                        instance_id = a[a.length - 1];
                        rowText = '<input type="hidden" name="instances[' + num_instances++ + ']" value="' + instance_id + '" id="id_inventory_' + i + '" data-itemtype="number" data-type="' + data_type + '" />';
                        console.log("adding instance id " + instance_id + " with client id " + client_id + " and type " + data_type);
                    }
                    if (rowText != "") {
                        $("#backup-job-selections").append(rowText);
                    }
                } else {
                    console.log("In addInstance, not adding as sid, dbid = ", dbid);
                    PHD.appliance_sid = dbid;
                }
            }

            function removeInventoryRow(dbId) {
                if (typeof inventory[dbId] !== 'undefined') {
                    inventory[dbId].delete();
                }
            }

            function removeInstance($el, dbid) {
                if (isNaN(dbid)) {
                    var a = dbid.split('_');
                    if (a.length > 0) {
                        instance_id = a[a.length - 1];
                    }
                    var selections = backupJobTreeview.$selections;
                    console.log("removing instance: dbid = " + dbid + " and instance id = " + instance_id);
                    var $node = selections.find('[value="' + instance_id + '"]');
                    if ($node.length > 0) {
                        console.log("Found: node length " + $node.length + ".  Removing instance " + instance_id);
                        $node.remove();
                    }
                }
            }

            function checkForInstance(dbid) {
                var found = false;
                if (isNaN(dbid)) {
                    var a = dbid.split('_');
                    if (a.length > 0) {
                        instance_id = a[a.length - 1];
                    }
                    var selections = backupJobTreeview.$selections;
                    console.log("check for instance: dbid = " + dbid + " and instance id = " + instance_id);
                    var $node = selections.find('[value="' + instance_id + '"]');
                    if ($node.length > 0) {
                        console.log("Found: node length " + $node.length + ". Instance " + instance_id + " already present.");
                        found = true;
                    }
                }
                return found;
            }

            function unDeleteInventoryRow(dbId) {
                inventory[dbId].undelete();
            }

            function enableEditClick(loopIndex) {
                var dbId = $(this).data("dbid");
                inventory[dbId] = AdoptedInventoryRow($(this), loopIndex);
            }

            // Recursive; walks up the TreeView and checks each ancenstor of the
            // original node for inclusion in the inventory table
            function hasIncludedTreeParent($el, dbId) {
                var uid = backupJobTreeview.universeID;

                // TODO: handle checking if the node is the 'universe' node
                // this may not always be 1, especially with multi-tenancy
                if (dbId == uid) {
                    return false;
                }

                var $tree = backupJobTreeview.$container,
                    $treeEl = $tree.find("[data-dbid=\"" + dbId + "\"]");

                var $elParent = getTreeParent($treeEl),
                    parentId = $elParent.data("dbid");

                if (parentId == uid) {
                    return $inventoryTable.find("[data-dbid=\"" + uid + "\"]").length > 0 ? uid : false;
                }

                if ($inventoryTable.find("[data-dbid=\"" + parentId + "\"]").length) {
                    return parentId;
                } else {
                    return hasIncludedTreeParent($elParent, parentId);
                }
            }

            function getTreeParent($el) {
                return $el.parent().closest(".checkable");
            }

            function handleRecurrenceState($el) {
                var checked = $el.is(":checked");
                recurrenceSpinner.spinner("option", "disabled", !checked);
                $recurrencePeriod.prop("disabled", !checked);
                $endTime.prop("disabled", !checked);
            }

            function setRecurrenceInputValue(enabled) {
                var multiplier = enabled ? $recurrencePeriod.val() : 0;
                $recurrenceInput.val($recurrenceAmount.val() * multiplier);
                if (!enabled) {
                    $recurrenceInput.prop("disabled", true);
                }
            }

            function handleRetentionState($el) {
                var checked = $el.is(":checked");
                retentionSpinner.spinner("option", "disabled", !checked);
                $retentionPeriod.prop("disabled", !checked);
            }

            function setRetentionInputValue(enabled) {
                var multiplier = enabled ? $retentionPeriod.val() : 0;
                $retentionInput.val($retentionAmount.val() * multiplier);
                if (!enabled) {
                    $retentionInput.prop("disabled", true);
                }
            }

            function handleScheduleTypeChange(val) {
                var mode = parseInt(val, 10);

                switch (mode) {
                    case SCHEDULE_TYPE_NOW:
                        $(".schedule-ctrl, .full-schedule-ctrl").hide();
                        $("#scheduling-mode").hide();
                        $(".custom-ctrl").hide();
                        $scheduleMode.val("standard");

                        // save current job name, change to "On Demand" if a now.
                        savedJobName = $("#id_name").val();
                        $("#id_name").val("On-Demand");
                        $("#id_name").prop("disabled", true);

                        break;
                    case SCHEDULE_TYPE_SCHEDULED:
                        $("#scheduling-mode").show();
                        if (isCustomJob) {
                            $scheduleMode.val("custom");
                            $(".schedule-ctrl").hide();
                            $(".custom-ctrl").show();
                            $("#calendar_description").val(getCalendarDescription());
                        } else {
                            $scheduleMode.val("standard");
                            $(".schedule-ctrl").show();
                            $(".custom-ctrl").hide();
                        }
                        //$(".recurrence-controls").hide();
                        if (backupStrategy == BACKUP_MODE_FULL_AND_INC) $(".full-schedule-ctrl").show();

                        // recall prior job name if saved.
                        $("#id_name").val(savedJobName);
                        $("#id_name").prop("disabled", false);

                        break;
                }
            }


            function initDateControls() {
                $(".datepicker input:text").datepicker({
                    showOn: "both",
                    buttonText: '<img src="assets/images/calendar.png" width="24" height="24">',
                    dateFormat: DATE_FORMAT,
                    minDate: 0,
                    yearRange: "c-1:c+2"
                }).attr("placeholder", DATE_DISPLAY_FORMAT);

                var recurrenceOptions = {
                    hours: {
                        max: 12,
                        min: 1,
                        step: 1
                    }
                    /* lmc - no minutes for archives
                     minutes: {
                     max: 30,
                     min: 15,
                     step: 15
                     }*/
                };

                /*if(recurrenceAmount >= 60) {
                 $recurrenceAmount.val(recurrenceAmount/60);
                 recurrenceSpinner = $("#recurrence-amount").spinner(recurrenceOptions.hours);
                 } else {
                 recurrenceSpinner = $("#recurrence-amount").spinner(recurrenceOptions.minutes);
                 }*/
                recurrenceSpinner = $("#recurrence-amount").spinner(recurrenceOptions.hours);

                recurrenceSpinner.on("spinchange", function () {
                    var val = parseFloat($(this).val()),
                        valid = $recurrencePeriod.val() === "60" ? [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12] : [15, 30];
                    if (!_.contains(valid, val)) {
                        val = Math.round(val);
                        if (!_.contains(valid, val)) {
                            $(this).spinner("value", valid[0]);
                        } else {
                            $(this).spinner("value", val);
                        }
                    }
                });

                $recurrencePeriod.on("change", function (event) {
                    var val = $(this).val();

                    switch (val) {
                        case "1":
                            recurrenceSpinner
                                .spinner("option", "min", 15)
                                .spinner("option", "max", 30)
                                .spinner("option", "step", 15)
                                .val(15);
                            break;

                        case "60":
                            recurrenceSpinner
                                .spinner("option", "min", 1)
                                .spinner("option", "max", 12)
                                .spinner("option", "step", 1)
                                .val(1);
                            break;
                    }
                });

                $endTime.timeEntry({
                    timeSteps: [1, 0, 0],
                    noSeparatorEntry: true,
                    spinnerImage: null,
                    ampmPrefix: " "
                });

                if (!jobEdit) {
                    $endTime.timeEntry('setTime', DEFAULT_DO_NOT_RUN_AFTER_TIME);
                }

                $recurrenceCheckbox.on("change", function (event) {
                    handleRecurrenceState($(this));
                }).trigger("change");

            }

            function initRetentionControls() {

                retentionSpinner = $("#retention-amount").spinner(retentionOptions.days);

                retentionSpinner.on("spinchange", function () {
                    var val = parseFloat($(this).val());
                    if (!isNaN(val) && val > 0) {
                        val = Math.round(val);
                        $(this).spinner("value", val);
                    } else {
                        $(this).spinner("value", 1);
                    }
                });

                $retentionPeriod.on("change", function (event) {

                    retentionSpinner
                        .spinner("option", "min", 1)
                        .spinner("option", "max", 999)
                        .spinner("option", "step", 1);

                });

                $retentionCheckbox.on("change", function (event) {
                    handleRetentionState($(this));
                }).trigger("change");
            }

            function initTimeControls() {
                var settings = {
                    noSeparatorEntry: true,
                    spinnerImage: null,
                    ampmPrefix: " "
                };

                $startTime.timeEntry(settings);
            }

            function initCustomCalendarControls() {

                $customEditButton.on("click", function (event) {
                    openCustomCalendar(event);
                });

                $scheduleMode.on("change", function (event) {
                    var val = $(this).val();
                    switch (val) {
                        case "standard":
                            isCustomJob = false;
                            $(".schedule-ctrl, .full-schedule-ctrl").show();
                            $(".custom-ctrl").hide();

                            // recall prior job name if saved.
                            $("#id_name").val(savedJobName);
                            $("#id_name").prop("disabled", false);
                            break;

                        case "custom":
                            isCustomJob = true;
                            $(".schedule-ctrl, .full-schedule-ctrl").hide();
                            $(".custom-ctrl").show();

                            $("#calendar_description").val(getCalendarDescription());

                            // recall prior job name if saved.
                            $("#id_name").val(savedJobName);
                            $("#id_name").prop("disabled", false);
                            break;
                    }
                });
            }

            function initDateRangeControls() {
                var $dateRange = $("#backups-date-range");
                $.get("app/common/date-range/date-range.html", function (dateRange_html) {
                    console.log($dateRange);
                    $dateRange.append(dateRange_html);

                    var elem = angular.element(document.querySelector('[ng-controller=GlobalCtrl]'));
                    var injector = elem.injector();
                    var $compile = injector.get('$compile');
                    var $scope = elem.scope();

                    var elements = angular.element(document.getElementById("date_range_component"));
                    $compile(elements.contents())($scope);
                    $scope.$digest();

                    elem = angular.element(document.querySelector('[ng-controller=DateRangeCtrl]'));
                    $scope = elem.scope();
                    if (jobOptions !== null) {
                        $scope.setDateRange(jobOptions.range_end, jobOptions.range_size);
                    } else {
                        $scope.setDateRange(0, 0);
                    }
                    $scope.$digest();
                });
            }

            var archive_targets_initialized = false;
            var storage_targets = [];
            var storageSelect = $("#id_storage")[0];
            var selectedTarget = null;
            var root_scope = $rootScope; //$("[ng-controller=GlobalCtrl]").scope();
            var is_replication_target = false;
            var is_changer = false;
            var is_tape = false;
            var changer_slot_length = 0;

            function initForm() {

                // Handle popovers for definitions.
                $form.find(".definition").popover({
                    appendTo: $form,
                    position: {
                        my: "left top",
                        at: "right bottom"
                    }
                });

                $("#id_slots").hide();
                // By default, append sets to the media.
                $("#overwrite").val(true);

                $('input:radio[name="schedule_type"]').on("change", function (event) {
                    scheduleType = parseInt($(this).val(), 10);
                    handleScheduleTypeChange(scheduleType);
                });
                handleScheduleTypeChange($('input:radio[name="schedule_type"]:checked').val());

                $.validator.addMethod("validate-future-date", function (value, ilelement) {
                        var date = moment(value + " " + $("#cal_incr_start_date").val() + " " + tz, DATE_DISPLAY_FORMAT + " hh:mm A Z").utc(),
                            now = moment().utc(),
                            isvalid = now.isBefore(date);

                        return isvalid || (scheduleType === SCHEDULE_TYPE_SCHEDULED);
                    },
                    gettext("Please select a future date and time")
                );

                $.validator.addMethod("validate-future-time", function (value, element) {
                        var date = moment($("#cal_incr_start_time").val() + " " + value + " " + tz, DATE_DISPLAY_FORMAT + " hh:mm A Z").utc(),
                            now = moment().utc(),
                            isvalid = now.isBefore(date);

                        return isvalid || (scheduleType === SCHEDULE_TYPE_SCHEDULED);
                    },
                    gettext("Please select a future date and time")
                );

                $('#id_overwrite').on("change", function(event) {
                    // If not a tape or changer, the storage threshold (div id_storage_threshold) will be vislble.
                    // When the user changes the overwrite option, enable/disable the storage/purge selections.
                    // If a tape or changer, no need to do anything, as the selections won't be visible anyway.
                    if (!is_changer && !is_tape) {
                        if ($('#overwrite').prop('checked')) {
                            $('#id_no_purge_backups').prop("checked", true);
                            $('#id_no_purge_backups').prop("disabled", true);
                            $('#id_purge_backups').prop("disabled", true);
                        } else {
                            $('#id_no_purge_backups').prop("disabled", false);
                            $('#id_purge_backups').prop("disabled", false);
                        }
                    }
                });

                $('#id_storage').on("change", function (event) {
                    selectedTarget = _.findWhere(storage_targets, {'target_id': !isNaN($('#id_storage').val()) ? parseInt($('#id_storage').val()) : $('#id_storage').val()});
                    is_replication_target = selectedTarget !== undefined && selectedTarget.type == "replication_target";
                    is_changer = selectedTarget !== undefined && selectedTarget.type == "changer";
                    is_tape = selectedTarget !== undefined && selectedTarget.type == "tape";
                    var $scheduled = $("#id_job_scheduled");

                    // Save the job name, unless it is On-Demand.
                    var checkJobName = $("#id_name").val();
                    if (checkJobName !== 'On-Demand') {
                        savedJobName = checkJobName;
                    }

                    if (is_replication_target) {
                        // Do not allow "on demand" replication job.
                        // Type 4 is a schedule job, id_job_scheduled.
                        $scheduled.prop("checked", true);
                        handleScheduleTypeChange($scheduled.val());
                        $(".schedule-ctrl").hide();
                        $(".custom-ctrl").hide();
                        $("#scheduling-mode").hide();
                        $("#backups-date-range").hide();
                        $("#scheduled_or_on_demand").hide();
                        $("#archive-mode-options").hide();
                        $("#replication-description").show();
                        $("#id_slots").hide();
                        // No test button for replication jobs
                        $("#id_test_button").hide();
                    } else {
                        if ($scheduled.is(":checked")) {
                            $("#scheduling-mode").show();
                            $("#backups-date-range").show();
                            if (isCustomJob) {
                                $scheduleMode.val("custom");
                                $(".schedule-ctrl").hide();
                                $(".custom-ctrl").show();
                                $("#calendar_description").val(getCalendarDescription());
                            } else {
                                $scheduleMode.val("standard");
                                $(".schedule-ctrl").show();
                                $(".custom-ctrl").hide();
                            }
                        }
                        if (is_changer || is_tape) {
                            //show the slots text box and/or the overwrite checkbox
                            if (is_changer) {
                                $("#id_slots").show();
                                var changer_info_url = '/api/archive/media/library/' + selectedTarget.name + '/';
                                var load = PHD.showLoadingIndicator('body', true, "Getting slots info for changer ....");
                                var resp = PHD.Ajax.get(changer_info_url, load);
                                resp.done(function (data) {
                                    PHD.hideLoadingIndicator(load);
                                    changer_slot_length = data.data.slots.length;
                                })
                            }
                            $('#id_no_purge_backups').prop("checked", true);
                            $('#id_storage_threshold').hide();
                            $('#id_overwrite').show();
                            $("#id_test_button").hide();
                        } else {
                            $("#id_slots").hide();
                            $('#id_storage_threshold').show();
                            $('#id_purge_backups').prop("checked", true);
                            // Show test button for archive (cold copy) jobs  if not tape/changer
                            $("#id_test_button").show();
                        }
                        $("#scheduled_or_on_demand").show();
                        $("#archive-mode-options").show();
                        $("#replication-description").hide();


                    }
                });

                BackupForm = PHD.StepForm($form, {
                    namespace: "backupform",
                    actions: {
                        "submit": function (data) {
                            setRecurrenceInputValue($recurrenceCheckbox.prop("checked"));
                            setRetentionInputValue($retentionCheckbox.prop("checked"));
                            //console.log('submit clicked, call save of schedule');
                            return this.$form.valid();
                        },
                        "test": function (data) {
                            return this.$form.valid();
                        }
                    },
                    serialize: function ($form) {
                        $timeInputs.timeEntry('option', 'show24Hours', true);
                        return $form.serialize();
                    },
                    isEditing: jobEdit
                });

                BackupForm
                    .validate({
                        onkeyup: function (element) {
                            var $el = $(element);
                            if ($el.is("input[type=text]")) {
                                this.element(element);
                            }
                            if ($el.is(".hasTimeEntry")) {
                                this.element($("#cal_incr_start_time")[0]);
                            }
                        },
                        onfocusout: function (element, event) {
                            var $el = $(element),
                                self = this;
                            if ($el.is("input[type=text]")) {
                                this.element(element);
                            }
                            if ($el.is(".hasTimeEntry")) {
                                this.element($("#cal_incr_start_time")[0]);
                            }
                        },
                        rules: {
                            name: {
                                required: true
                            },
                            storage: {
                                required: true
                            },
                            "calendar[0].start_date": {
                                required: true,
                                dateFmt: true,
                                "validate-future-date": true
                            },
                            "calendar[0].start_time": {
                                required: true,
                                time12h: true,
                                "validate-future-time": true
                            },
                            run_full_start_time: {
                                required: true,
                                time12h: true
                            },
                            end_time: {
                                required: function (element) {
                                    if ($recurrenceCheckbox.prop("checked")) {
                                        return $(element).val() == "";
                                    } else {
                                        return false;
                                    }
                                },
                                time12h: true
                            },
                            slots: {
                                required: true
                            }
                        },
                        messages: {
                            name: {
                                required: gettext("Job Name is required")
                            },
                            storage: {
                                required: gettext("Target name is required")
                            },
                            "calendar[0].start_date": {
                                required: gettext("Start Date is required")
                            },
                            "calendar[0].start_time": {
                                required: gettext("Start Time is required")
                            },
                            run_full_start_time: {
                                required: gettext("Full Schedule Start Time is required")
                            },
                            end_time: {
                                required: gettext("Please enter an end hour for recurrence")
                            },
                            slots: {
                                required: gettext("Target Slots are required")
                            }
                        }
                    })
                    .on("formcancel", function () {
                        PHD.currentDialog.wizard("close");
                    })
                    .on("formtest", function (event, data) {
                        var message = "Backup Copy Job size estimate not available.";
                        if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                            var result = data.result[0];
                            message = "The Job will be between " + PHD.util.formatMB(result.min) + " and " + PHD.util.formatMB(result.max) + " and most likely near " + PHD.util.formatMB(result.likely) + '.';
                            message += "<br/><br/>" + result.fit;
                        }
                        var options = {
                            dialogClass: "dialog-notice-border"
                        };
                        PHD.throwNotice({"result": [{"message": message}]}, options);
                    })
                    .on("formsubmit", function (event, data) {
                        PHD.currentDialog.wizard("close");

                        scheduleType = parseInt(scheduleType, 10);
                        if (scheduleType == SCHEDULE_TYPE_NOW) {
                            backupNowReturn = data;
                            if (backupNowReturn !== undefined && backupNowReturn.id !== undefined) {
                                // all good.
                            } else {
                                //PHD.throwNotice ?
                            }
                        } else {
                            if (data && data.result && data.result[0] && data.result[0].message !== undefined) {
                                // The structure returned on a job is the same as "now" status, so use the same dialog.
                                backupNowReturn = data.result;
                                PHD.wizard("app/joborders/backup/now_dialog.html", "backup-now", {
                                    title: gettext("Backup Copy Job Status"),
                                    dialogClass: "standalone",
                                    height: 290,
                                    helpArticle: PHD.App.getHelpLink("backup", true),
                                    close: function (event, ui) {
                                        // Include the wizard standard close code (from main.js)
                                        $("body")
                                            .removeClass("dialog-open")
                                            .css("overflow", "hidden");
                                        $(this).wizard("destroy").remove();

                                        var scope = angular.element(document).scope();
                                        if (scope.user.showTour) {
                                            window.location.href = "#";
                                        }
                                    }
                                });
                            } else {
                                // lmc - we checked HTTP status earlier, so if we get here, we know the call was successful.
                                if (window.location.href.indexOf("#/protect") !== -1) {
                                    console.log("we are in protect, not jobs");
                                    var options = {},
                                        tab = (parseInt(scheduleType, 10) === SCHEDULE_TYPE_NOW) ? "#0" : "#1";
                                    options["buttons"] = [
                                        {
                                            text: gettextCatalog.getString("View Jobs"),
                                            click: function () {
                                                window.location.href = "#/jobs/" + tab;
                                                $(this).dialog("destroy").remove();
                                            }
                                        }
                                    ];
                                    // check to see if there is a message to be displayed.
                                    if (data && data.result && data.result[0]) {
                                        data.result[0].message = gettextCatalog.getString("Nice work! You can check out your newly created job on the Jobs page.");
                                        PHD.throwNotice(data, options);
                                    }
                                }

                                //PHD.currentDialog.wizard("close");
                                var scope = angular.element(document).scope();
                                if (scope.user.showTour) {
                                    window.location.href = "#";
                                }
                            }
                        }
                    })
                    .on("stepformgoto", function (event, page) {
                        if (page === 1) {

                            if (!archive_targets_initialized) {
                                var sid = PHD.appliance_sid;
                                $.each(inventory, function (dbid, row) {
                                    console.log("dbid");
                                    console.log(dbid);
                                    var targets_url, encryption_url;
                                    if (useCopiedAssets == 1) {
                                        // On the copied assets tab, use the local system id instead of the listed sid.
                                        targets_url = "/api/backup-copy/connected_targets/"; ///api/archive/media/connected";
                                        encryption_url = "/api/encryption/?sid=" + root_scope.local.id;
                                    } else {
                                        var showArchiving = (!root_scope.showArchiving) ? "&showArchiving=false" : "";
                                        targets_url = "/api/backup-copy/connected_targets/?sid=" + sid + showArchiving;
                                        encryption_url = "/api/encryption/?sid=" + sid;
                                    }
                                    var load = PHD.showLoadingIndicator('body', true, "Getting list of available backup copy targets....");
                                    var errorMessage = null;
                                    var encryption_resp = PHD.Ajax.get(encryption_url);
                                    encryption_resp.done(function (data) {
                                        if ((data.data[0].state) !== "persist") {
                                            $("#id_encrypt").prop('disabled', true);
                                        } else {
                                            $("#id_encrypt").prop('disabled', false);
                                        }
                                    });
                                    var resp = PHD.Ajax.get(targets_url, load);
                                    resp.done(function (data) {
                                        storage_targets = data.data;
                                        storageSelect.options[0] = new Option("Please select target", "");
                                        if (storage_targets) {
                                            storage_targets.forEach(function (obj, index) {
                                                if (obj.target_id === -1) {
                                                    errorMessage = obj.name;
                                                } else {
                                                    storageSelect.options[index + 1] = new Option(obj.name, obj.target_id);
                                                }
                                            });
                                            if (jobEdit && jobOptions !== undefined && jobOptions.storage !== undefined) {
                                                console.log("set storage to " + jobOptions.storage);
                                                $("#id_storage").val(jobOptions.storage);
                                                var target = _.findWhere(storage_targets, {'name': jobOptions.storage});
                                                if (typeof(target) != 'undefined' && target.type == "changer") {
                                                    is_changer = true;
                                                    $("#id_slots").show();
                                                    var changer_info_url = '/api/archive/media/library/' + target.name + '/';
                                                    var load = PHD.showLoadingIndicator('body', true, "Getting slots info for changer ....");
                                                    var resp = PHD.Ajax.get(changer_info_url, load);
                                                    resp.done(function (data) {
                                                        PHD.hideLoadingIndicator(load);
                                                        changer_slot_length = data.data.slots.length;
                                                    });

                                                    $('#id_overwrite').show();
                                                    $('#id_no_purge_backups').prop("checked", true);
                                                    $('#id_storage_threshold').hide();
                                                    if (is_changer) {
                                                        $("#slots").val(jobOptions.slots);
                                                        $('#id_test_button').prop("disabled", true);
                                                    }
                                                } else {
                                                    $("#id_slots").hide();
                                                    $('#id_storage_threshold').show();
                                                }
                                            }
                                            if (errorMessage) {
                                                PHD.alert("Some Backup Copy Targets returned errors: " + errorMessage);
                                            }
                                        }
                                    });
                                    archive_targets_initialized = true;
                                    return false;
                                });
                            }
                        }
                    });

                BackupForm.test = function () {
                    console.log("testing backup copy job");
                    return BackupForm.submit(true);
                };

                function getBackupCopyDateRange() {
                    var elem = angular.element(document.querySelector('[ng-controller=DateRangeCtrl]'));
                    var $scope = elem.scope();
                    var range = {};

                    var dateRange = $scope.getDateRange();
                    var isValid = $scope.checkDateRange();
                    if (isValid) {
                        if (dateRange.startDate == null && dateRange.endDate == null) {
                            range.range_end = 0;
                            range.range_size = 0;
                        } else {
                            var end = moment(dateRange.endDate, "MM/DD/YYYY").endOf('day').unix();
                            var start = moment(dateRange.startDate, "MM/DD/YYYY").startOf('day').unix();
                            range.range_end = $scope.isCustomDate ? end : 0;
                            range.range_size = end - start;
                        }
                    }
                    return isValid ? range : false;
                }

                BackupForm.submit = function (test) {
                    if (is_changer) {
                        var test_string = $('#slots').val();
                        if (!isNaN(test_string)) {
                            if (parseInt(test_string) > changer_slot_length) {
                                window.PHD.throwError({"result": [{"message": gettext("Slots entered is greater than slots exists")}]});
                                return;
                            }
                        } else if (test_string.indexOf("-") > -1) {
                            var test_range_string = $('#slots').val().split("-");
                            if (test_range_string.length > 2) {
                                window.PHD.throwError({"result": [{"message": gettext("Please enter valid slots")}]});
                                return;
                            } else {
                                if (parseInt(test_range_string[0]) > changer_slot_length || parseInt(test_range_string[1]) > changer_slot_length) {
                                    window.PHD.throwError({"result": [{"message": gettext("Please enter valid slots")}]});
                                    return;
                                }
                            }
                        } else if (test_string.indexOf(",") > -1) {
                            var comma_flag = true;
                            var test_comma_string = $('#slots').val().split(",");
                            for (var i = 0; i < test_comma_string.length; i++) {
                                if (parseInt(test_comma_string[i]) > changer_slot_length) {
                                    comma_flag = false;
                                    break;
                                }
                            }
                            if (!comma_flag) {
                                window.PHD.throwError({"result": [{"message": gettext("Please enter valid slots")}]});
                                return;
                            }


                        } else if (isNaN(test_string)) {
                            if (test_string.toLowerCase() == 'all') {
                                //donothing
                            } else {
                                window.PHD.throwError({"result": [{"message": gettext("Please enter valid slots")}]});
                                return;
                            }
                        }
                    }

                    if (!is_replication_target) {
                        // get and check date range.
                        var dateRange = getBackupCopyDateRange();
                        if (dateRange === false) {
                            retryDialog("The date range is invalid.  Please ensure that the From: date is on or before the To: date.");
                            return;
                        }
                    }

                    // testing will be true if we are testing a job's size.
                    var testing = test || false;
                    var self = this;
                    var $now = $("#id_job_now");
                    var archive_now = $now.is(":checked");
                    var save_message = testing ? "Checking space needed for Backup Copy Job..." :
                        (archive_now ? "Starting On-Demand Backup Copy Job..." : "Saving Backup Copy Job...");

                    var load = PHD.showLoadingIndicator("body", true, save_message);

                    this.$buttons.blur().disable();

                    var submitData = form2js(this.$form.attr("id"), '.', false);
                    submitData.append = !($('#overwrite').prop('checked'));
                    // If overwrite, purge must be set to false.
                    if (submitData.append === false) {
                        submitData.purge = false;
                    }
                    var $editing_id = this.$form.find("#edit_object_id");
                    var sid = (PHD.appliance_sid === null || useCopiedAssets === 1) ? "?sid=" + root_scope.local.id : "?sid=" + PHD.appliance_sid;
                    var url = this.$form.attr("action");
                    var hasItems = false;

                    var backupTypes = [];
                    var selected = $('#id_modes :selected').val();
                    if (selected == "All") {
                        backupTypes = ["Full", "Differential", "Incremental", "Selective", "Bare Metal", "Transaction"];
                    } else {
                        backupTypes = ["Full"];
                    }

                    // retention not being unset if box is unchecked
                    if (!$("#retention-checkbox").prop('checked')) {
                        submitData["retention_days"] = 0;
                    }

                    // add the date range fields to submitData.
                    jQuery.extend(submitData, dateRange);

                    if (archive_now || testing) {
                        console.log("Archive Now or Test: submitData is " + JSON.stringify(submitData));
                        url = (useCopiedAssets === 1) ? "/api/archive" : "/api/archive/" + sid;
                        if (testing) {
                            url = (useCopiedAssets === 1) ? "/api/archive/check/" : "/api/archive/check/" + sid;
                        }
                        var archiveObj = {};
                        //archiveObj.system_id = parseInt(PHD.appliance_sid, 10);
                        // see if full or incremental is checked.
                        archiveObj.target = submitData.storage;
                        hasItems = !(submitData.instances === undefined && submitData.clients === undefined);
                        if (submitData.instances !== undefined) {
                            archiveObj.instances = submitData.instances;
                        }
                        if (submitData.clients !== undefined) {
                            archiveObj.clients = submitData.clients;
                        }
                        var options = {};
                        options.purge = submitData.purge;
                        options.email_report = submitData.email_report;
                        options.encrypt = submitData.encrypt;
                        options.retention_days = typeof submitData.retention_days == "string" ? Number(submitData.retention_days) : submitData.retention_days;
                        options.append = submitData.append;
                        archiveObj.options = options;

                        archiveObj.types = backupTypes;
                        //archiveObj.start_date = 0;
                        //archiveObj.end_date = 0;
                        archiveObj.range_end = submitData.range_end;
                        archiveObj.range_size = submitData.range_size;
                        if (is_changer) {
                            archiveObj.slots = submitData.slots;
                        }
                        if (hasItems) {
                            archiveObj = JSON.stringify(archiveObj);
                            console.log("Archive now with obj " + archiveObj);
                            PHD.Ajax
                                .put(url, archiveObj)
                                .done(function (data) {
                                    var formAction = testing ? "formtest" : "formsubmit";
                                    console.log(formAction);
                                    self.$form.trigger(formAction, [data, self.$form, self]);
                                })
                                .always(function () {
                                    self.$buttons.enable();
                                    PHD.hideLoadingIndicator(load);
                                });
                        } else {
                            self.$buttons.enable();
                            PHD.hideLoadingIndicator(load);
                            PHD.throwError({"result": [{"message": gettext("You must specify assets to copy.")}]});
                        }
                    } else {

                        if (is_replication_target) {
                            var obj = $.extend(true, {}, submitData);
                            submitData = {};
                            console.log("obj = " + JSON.stringify(obj));
                            submitData.instances = obj.instances;
                            submitData.target_id = selectedTarget.target_id;
                            submitData.name = obj.name;
                            submitData.type = "replication";
                            hasItems = !(submitData.instances === undefined);
                        } else {
                            // Check the schedule mode and pass in either the calendar (standard mode) or ical (custom).
                            if (isCustomJob) {
                                if (jobOptions !== null && jobOptions !== undefined && jobOptions.ical !== undefined) {
                                    submitData.ical = jobOptions.ical;
                                    delete submitData.calendar;
                                } else {
                                    self.$buttons.enable();
                                    PHD.hideLoadingIndicator(load);
                                    retryDialog("Custom calendars must contain at least one scheduled event.");
                                    return;
                                }
                            } else {
                                submitData.calendar.forEach(function (obj, index) {
                                    if (obj.backup_type == 'Archive') {
                                        var my_start_date = $("#cal_incr_start_date").val();
                                        obj.start_date = DateTimeHandler.getUSDateString($rootScope.userLanguageCode, my_start_date);
                                        var my_start = $("#cal_incr_start_time").val();
                                        obj.start_time = DateTimeHandler.convertTimeToAPI("00:00:00", my_start);
                                        if ($recurrenceCheckbox.prop('checked')) {
                                            obj.end_hour = DateTimeHandler.convertTimeToHour($endTime.val());
                                            console.log("Recurrence set, end hour is " + obj.end_hour);
                                        }
                                    }
                                });
                            }
                            submitData.types = backupTypes;
                            if (!is_changer && submitData.slots !== undefined) {
                                delete submitData.slots;
                            }
                            /*if(!is_changer && submitData.append !== undefined) {
                             delete submitData.append;
                             }*/
                            hasItems = !(submitData.instances === undefined && submitData.clients === undefined);
                        }

                        if (hasItems) {
                            var invalidAssetsString = checkForInvalidAssets();
                            if (invalidAssetsString == null) {
                                if (validateInventory()) {
                                    submitData = JSON.stringify(submitData);
                                    console.log('=== Submit form: object = ' + submitData);
                                    if ($editing_id.text() != "none") {
                                        var myID = $editing_id.text();
                                        url += myID + "/" + sid;
                                        console.log("IS Editing, url is " + url);
                                        PHD.Ajax
                                            .put(url, submitData)
                                            .done(function (data) {
                                                self.$form.trigger("formsubmit", [data, self.$form, self]);
                                                // trigger a refresh.
                                                $(document).trigger("editJob");
                                            })
                                            .always(function () {
                                                self.$buttons.enable();
                                                PHD.hideLoadingIndicator(load);
                                            });
                                    } else {
                                        url += sid;
                                        console.log("Is Adding, not Editing, url is " + url);
                                        PHD.Ajax
                                            .post(url, submitData)
                                            .done(function (data) {
                                                self.$form.trigger("formsubmit", [data, self.$form, self]);
                                                // trigger a refresh.
                                                $(document).trigger("editJob");
                                            })

                                            .always(function () {
                                                self.$buttons.enable();
                                                PHD.hideLoadingIndicator(load);
                                            });
                                    }
                                } else {
                                    self.$buttons.enable();
                                    PHD.hideLoadingIndicator(load);
                                    var editJob = jobEdit ? "  When editing a backup copy job, all assets must be on the same appliances." : "";
                                    PHD.throwError({
                                        "result": [{
                                            "message": gettext("You have selected incompatible assets for this backup job.") + "<br/><br/>" +
                                            gettext("All backups in the job must be of assets from the same appliance.") +
                                            gettext(editJob)
                                        }]
                                    });
                                }
                            } else {
                                self.$buttons.enable();
                                PHD.hideLoadingIndicator(load);
                                PHD.throwError({"result": [{"message": invalidAssetsString}]});
                            }
                        } else {
                            self.$buttons.enable();
                            PHD.hideLoadingIndicator(load);
                            PHD.throwError({"result": [{"message": gettext("You must specify assets to protect.")}]});
                        }
                    }

                    return this;
                };

                //initStorage();
            }


            // Start of code ported from backup job to handle custom calendaring
            // For edited jobs, jobOptions is equivalend to $scope.joborder.
            // For new jobs, jobOptions is null, so if the user saves calendar (saveCustomCalendar event)
            // we will create a jobOptions object with an ical property.
            var customOpen = false; // prevent double-click error.

            function openCustomCalendar(event) {
                event.stopPropagation(event);

                // Access the ngDialog module injected into the app.
                var elem = angular.element(document.querySelector('[ng-controller=GlobalCtrl]'));
                var injector = elem.injector();
                var ngDialog = injector.get('ngDialog');

                if (!customOpen) {
                    customOpen = true;
                    // To look like the backup job....
                    var $scope = {};
                    $scope.joborder = jobOptions !== null && jobOptions !== undefined ? jobOptions : {};
                    $scope.joborder.app_id = APP_ID_ARCHIVE;

                    // Clear this function so that the ngDialog elements can gain focus above a jQuery-ui dialog.
                    $.ui.dialog.prototype._focusTabbable = function () {
                    };
                    ngDialog.open({
                        template: 'app/protect/custom-schedule/custom.html',
                        scope: elem.scope(),
                        overlay: true,
                        data: {joborder: $scope.joborder, modes: ['Backup Copy'], calendarType: 'Backup Copy'},
                        modelDialogId: 'edit-custom-dialog',
                        name: 'edit-custom-dialog',
                        ngDialogStyle: 'width:900px;height:680px;',
                        preCloseCallback: function (value) {
                            customOpen = false;
                        }
                    });
                }
            }

            function hasCustomCalendar(job) {
                var isCustom = false;
                var calendar = job.calendar;
                if (calendar instanceof Array && calendar.length > 0) {
                    for (var i = 0; i < calendar.length; i++) {
                        var entry = calendar[i];
                        if (entry.custom === true) {
                            isCustom = true;
                            break;
                        }
                    }
                }
                return isCustom;
            }

            /*
             * If the global calendarDescription is not set, get the description from the job, otherwise, use the global one.
             */
            function getCalendarDescription() {
                var description;
                if (calendarDescription === undefined) {
                    // To look like the backup job....
                    var $scope = {};
                    $scope.joborder = jobOptions !== null && jobOptions !== undefined ? jobOptions : {};
                    description = ($scope.joborder.calendar_str instanceof Array) ? $scope.joborder.calendar_str.join('\r\n') : "No calendar exists";
                } else {
                    description = calendarDescription;
                }

                return (description);
            }

            /*
             * Sets the global calendar description.
             */
            function setCalendarDescription(description) {
                calendarDescription = description;
            }

            function retryDialog(message) {

                // Access the ngDialog module injected into the app.
                var elem = angular.element(document.querySelector('[ng-controller=GlobalCtrl]'));
                var injector = elem.injector();
                var ngDialog = injector.get('ngDialog');

                ngDialog.open({
                    dialogType: 'Retry',
                    dialogMessage: message
                });
            }

            // End of code ported from backup job to handle custom calendaring

            /*
             * Checks to see if any assets which can not be part of backup copy jobs in some situations are selected.
             * iSeries assets can be a part of a backup copy job to cold targets (archiving) but not a replication job, so we must catch it before save.
             */
            function checkForInvalidAssets() {
                var message = null;
                if (is_replication_target) {
                    $("#backup-job-selections").find("input").each(function () {
                        if ($(this).hasClass("deleted")) {
                            // skip deleted inventory items.
                        } else {
                            var type = $(this).data("type");
                            console.log(type);
                            if (type === INVENTORY_ID_CLIENT_I_SERIES) {
                                message = "iSeries assets cannot be included in Backup Copy Jobs to Hot Targets." +
                                    "<br/><br/>To configure an iSeries asset to copy to a Hot Target, please use the menu-based console interface.";
                                return message;
                            }
                        }
                    });
                }
                return message;
            }

            /*
             * Validate the inventory items to see that they match the scheduling rules.
             * Backup Copies can be mixed in a job, only can't be on different sids.
             */
            function validateInventory() {
                var valid_sid;

                // if on a target, all sid are the same.
                if (useCopiedAssets) {
                    return true;
                }

                // Assume valid at first.
                var compatible_sid = true;
                var compatible_client = true;
                var compatible_app = true;

                $inventoryTable.find(".inventory-row").each(function () {
                    console.log("dbid = " + $(this).data("dbid"));
                    if ($(this).hasClass("deleted")) {
                        // skip deleted inventory items.
                    } else {
                        var id = $(this).data("dbid");
                        // id components are "_" delimited, e.g.,
                        //
                        // system-id_client-id_app-id_instance-id
                        // system-id_client-id_app-id_host-guid_instance-id
                        // system-id_client-id
                        //
                        if (isNaN(id)) {
                            var a = id.split('_');
                            if (a.length > 1) {
                                // Ensure system id's are valid and compatible.
                                if (valid_sid === undefined) {
                                    valid_sid = a[0];
                                } else if (valid_sid != a[0]) {
                                    compatible_sid = false;
                                    return false;
                                }
                                if (jobEdit && valid_sid != PHD.appliance_sid) {
                                    compatible_sid = false;
                                    return false;
                                }
                            }
                        } else {
                            return false;
                        }
                    }
                });

                return (compatible_sid && compatible_client && compatible_app);
            }

            function initTreeView() {
                var load = PHD.showLoadingIndicator($backupTreeContainer);
                var scope = $("[ng-controller=GlobalCtrl]").scope();
                var sid = -1;
                if (jobEdit) {
                    var jobId = $("#edit_object_id").text();
                    var sidIndex = jobId.indexOf('.');
                    if (sidIndex !== -1) {
                        sid = parseInt(jobId.substring(sidIndex + 1));
                    }
                }

                var hypervisors = $("#backup-job-treeview-header").find("select").map(function () {
                        return parseInt($(this).val(), 10);
                    }).get(),
                    tvTask = PHD.TreeViewBuilder({
                        hypervisorView: hypervisors,
                        disks: 0,   // Do not show disks for Backup Copy (Archive) Job
                        isCopy: useCopiedAssets,
                        checkboxes: true,
                        loader: load,
                        uid: scope.user.id,    // user id for navigational groups (Protected Assets only, not Copied Assets).
                        sid: sid
                    });

                tvTask.done(function (tree) {
                    TreeViewData = tree.tree;
                    $backupTreeContainer.html(tree.html);

                    backupJobTreeview = PHD.TreeView(
                        $backupTreeContainer,
                        $("#backup-job-selections")
                    );

                    backupJobTreeviewHeader = PHD.TreeViewHeader(
                        $("#backup-job-treeview-header"),
                        backupJobTreeview,
                        "backupjob"
                    );
                    backupJobTreeviewSearch = PHD.TreeViewSearch(
                        $("#backup-job-treeview-find"),
                        backupJobTreeview,
                        backupJobTreeview.$container,
                        backupJobTreeview.$container.find(".tree-universe")
                    );

                    var initList = jobEdit;
                    // lmc - pre-select any items that were checked
                    if (preSelected != null) {
                        selectPreselected(backupJobTreeview.$container);
                        backupJobTreeview.$selections = $("#backup-job-selections");
                        initList = true;
                    }

                    // Attach event handler for Treeview checkbox clicks
                    backupJobTreeview.$container.on("checkclick", ".checkbox", treeViewCheckClickHandler);
                    backupJobTreeview.initSelections(initList);

                    PHD.hideLoadingIndicator(load);

                    initializeInventoryTable();
                });

            }

            function updateTreeView(hypervisors) {
                var load = PHD.showLoadingIndicator($backupTreeContainer);
                var scope = $("[ng-controller=GlobalCtrl]").scope();
                var tvTask = PHD.TreeViewBuilder({
                    hypervisorView: hypervisors,
                    disks: 0,   // Do not show disks for Backup Copy (Archive) Job
                    isCopy: useCopiedAssets,
                    checkboxes: true,
                    loader: load,
                    uid: scope.user.id     // user id for navigational groups (Protected Assets only, not Copied Assets).
                });

                tvTask.done(function (tree) {
                    TreeViewData = tree.tree;
                    $backupTreeContainer.html(tree.html);
                    PHD.hideLoadingIndicator(load);
                });
            }

            /*
             * If this is an edit/copy, initialize existing inventory rows.
             */
            function initializeInventoryTable() {
                if ($inventoryTable.find(".inventory-row").length) {
                    $inventoryTable.find(".inventory-row").each(enableEditClick);
                    PHD.currentDialog.one("wizardfocus", function () {
                        $inventoryTable.scrollTable({"height": ".modal-form-col", "resize": false});
                    });
                }
            }

            /*
             * Find pre-selected items, and add their nodes to the inventory on the rhs pane.
             */
            function selectPreselected($container) {
                //console.log("in selectPreselected, preselected = " + preSelected);
                var item_array = preSelected.split('&');
                var notfound_array = [];
                if (item_array.length > 0) {
                    //console.log('length is ' + item_array.length);
                    for (var i = 0; i < item_array.length; i++) {
                        var item = item_array[i];
                        var inventory = item.split('=');
                        if (inventory.length == 2) {
                            // inventory={id}
                            var id = inventory[1];
                            // split the id into pieces.
                            var a = id.split("_");
                            if (a.length > 1) {
                                system_id = a[0];
                                PHD.appliance_sid = system_id;
                                client_id = a[1];
                            }
                            if (a.length > 3) {
                                instance_id = a[a.length - 1];
                            } else {
                                instance_id = 0;
                            }
                            var found = false;
                            var $dbid_length = 0;
                            console.log("trying to find id " + id);
                            var $node = $container.find("[data-dbid=\"" + id + "\"]");
                            if ($node.html() == undefined) {
                                // try again, with our id pieces.
                                console.log("first search: undefined");
                                if (useCopiedAssets) {
                                    $node = $container.find("[data-dbid$=" + "_" + instance_id + "]");
                                } else {
                                    $node = $container.find("[data-dbid^=" + system_id + "_" + "][data-dbid$=" + "_" + instance_id + "]");
                                }
                                if ($node.length == 1) {
                                    found = true;
                                    $dbid_length = $node[0].dataset.dbid;
                                    $dbid_length = $dbid_length.split('_').length;
                                } else if ($node.length > 1) {
                                    var $nodes = $node;
                                    for (var j = 0; j < $nodes.length; j++) {
                                        var dbid = $nodes[j].dataset.dbid;
                                        var idLength = dbid.split('_');
                                        if (idLength.length >= 4) {
                                            $node = $($nodes[j]);
                                            $dbid_length = idLength;
                                            found = true;
                                        }
                                    }
                                }
                                if (found) {
                                    var dbid = $node.data("dbid");
                                    var data_type = $node.data("type");
                                    var rowText = '<input type="hidden" name="inventory" value="' + dbid + '" id="id_inventory_' + i + '" data-type="' + data_type + '" />';
                                    $("#backup-job-selections").append(rowText);
                                    //rowText = '<input type="hidden" name="instances[' + i + ']" value="' + instance_id + '" id="id_inventory_' + i + '" data-itemtype="number" />';
                                    //$("#backup-job-selections").append(rowText);
                                    //addInventoryRow($node, dbid);
                                    console.log("NODE was undefined: node dbid = " + $node.data("dbid") + " and row text is " + rowText);
                                    // Increment the inventory row counter
                                    inventoryRows++;
                                }
                            } else {
                                found = true;
                                var item_id = (instance_id > 0) ? instance_id : client_id;
                                var data_type = $node.data("type");
                                var rowText = '<input type="hidden" name="inventory" value="' + id + '" id="id_inventory_' + i + '" data-type="' + data_type + '" />';
                                $("#backup-job-selections").append(rowText);
                                console.log("NODE not undefined:  add id " + id + " and row text is " + rowText);
                                //addInventoryRow($node, id);
                            }
                            if (!found) {
                                // Add to the list of items not found.
                                notfound_array.push(item);
                            }
                        }
                    }
                } else {
                    PHD.throwNotice({"result": [{"message": gettext("No items are selected for protection in this joborder.")}]});
                }
                if (notfound_array.length > 0) {
                    var verb = notfound_array.length == 1 ? "is " : "are ";
                    PHD.throwNotice({
                        "result": [{
                            "message": gettext("There ") + verb + notfound_array.length +
                            gettext(" item(s) in this job not available in the inventory of possible items to protect.  ") +
                            gettext("A likely cause is that an item was deleted but inventory sync has not run yet.") + "<br/><br/>" +
                            gettext("To clear up this warning, go to the Options menu to re-run inventory sync, re-edit and save this job.")
                        }]
                    });
                }
            }

            function setJobOptions() {
                if (jobOptions != null) {
                    calendar = jobOptions.calendar;
                    console.log("calendar = " + JSON.stringify(calendar));
                    // part of details string? <!--<small>(The current time in US/Eastern is July 27, 2014, 10:41 a.m..)</small>-->

                    var hasFull = false;
                    for (var i = 0; i < calendar.length; i++) {
                        var cal = calendar[i];
                        $("#cal_incr_start_date").val(cal.start_date);
                        $("#cal_incr_schedule_run").val(cal.schedule_run);
                        var run_on = cal.run_on;
                        console.log("run_on:full " + JSON.stringify(run_on));
                        $("#cal_incr_run_on_sun").prop("checked", ($.inArray("0", run_on) != -1));
                        $("#cal_incr_run_on_mon").prop("checked", ($.inArray("1", run_on) != -1));
                        $("#cal_incr_run_on_tue").prop("checked", ($.inArray("2", run_on) != -1));
                        $("#cal_incr_run_on_wed").prop("checked", ($.inArray("3", run_on) != -1));
                        $("#cal_incr_run_on_thu").prop("checked", ($.inArray("4", run_on) != -1));
                        $("#cal_incr_run_on_fri").prop("checked", ($.inArray("5", run_on) != -1));
                        $("#cal_incr_run_on_sat").prop("checked", ($.inArray("6", run_on) != -1));
                        //$("#cal_incr_start_time").val(cal.start_time);
                        $("#cal_incr_start_time").timeEntry('setTime', DateTimeHandler.convertTimeFromAPI(cal.start_time));
                        // set default end time, may overwrite with hour value.
                        $endTime.timeEntry('setTime', DEFAULT_DO_NOT_RUN_AFTER_TIME);
                        if (cal.recurrence !== undefined) {
                            console.log("Edit: set recurrence: " + cal.recurrence);
                            $recurrenceAmount.val(cal.recurrence / $recurrencePeriod.val());
                            $recurrenceCheckbox.prop('checked', true);
                            handleRecurrenceState($recurrenceCheckbox);
                            if (cal.end_hour !== undefined) {
                                console.log("Edit: set end hour: " + cal.end_hour);
                                $endTime.timeEntry('setTime', DateTimeHandler.convertHourFromAPI(cal.end_hour));
                            }
                        }
                    }
                    if (jobOptions.email_report !== undefined) {
                        $("#id_schedule").prop("checked", jobOptions.email_report == true);
                    }
                    if (jobOptions.options.encrypt !== undefined) {
                        $("#id_encrypt").prop("checked", jobOptions.options.encrypt == true);
                    }
                    if (jobOptions.options.purge !== undefined) {
                        $("#id_purge_backups").prop("checked", jobOptions.options.purge == true);
                        $("#id_no_purge_backups").prop("checked", jobOptions.options.purge == false);
                    }
                    if (jobOptions.options.append !== undefined) {
                        $("#overwrite").prop("checked", jobOptions.options.append == false);
                    }
                    if (jobOptions.types !== undefined && jobOptions.types.length > 1) {
                        $("#id_modes").val("All");
                    }
                    if (jobOptions.options.retention_days !== undefined && jobOptions.options.retention_days !== 0) {
                        if (jobOptions.options.retention_days % 365 == 0) {
                            $retentionAmount.val(jobOptions.options.retention_days / 365);
                            retentionSpinner = $("#retention-amount").spinner(retentionOptions.years);
                            $retentionPeriod.val('365');
                        } else if (jobOptions.options.retention_days % 30 == 0) {
                            $retentionAmount.val(jobOptions.options.retention_days / 30);
                            retentionSpinner = $("#retention-amount").spinner(retentionOptions.months);
                            $retentionPeriod.val('30');
                        } else if (jobOptions.options.retention_days % 7 == 0) {
                            $retentionAmount.val(jobOptions.options.retention_days / 7);
                            retentionSpinner = $("#retention-amount").spinner(retentionOptions.weeks);
                            $retentionPeriod.val('7');
                        } else {
                            $retentionAmount.val(jobOptions.options.retention_days);
                            retentionSpinner = $("#retention-amount").spinner(retentionOptions.days);
                            $retentionPeriod.val('1');
                        }
                        $retentionCheckbox.prop('checked', true);
                        handleRetentionState($retentionCheckbox);
                    }

                    // Previously, waited until storage loads before setting it on edit.  But, if we do so, user cannot save on page 1.
                    // So, on edit, set storage and slots if set.  These will be updated once targets loaded on page 2.
                    if (jobOptions.storage !== undefined && jobOptions.storage != "") {
                        storageSelect.options[0] = new Option(jobOptions.storage, jobOptions.storage);
                        $("#id_storage").val(jobOptions.storage);
                    }
                    if (jobOptions.slots !== undefined && jobOptions.slots != "") {
                        $("#slots").val(jobOptions.slots);
                    }

                    isCustomJob = hasCustomCalendar(jobOptions);
                    if (isCustomJob) {
                        // Force this again to set proper view.
                        handleScheduleTypeChange(SCHEDULE_TYPE_SCHEDULED);
                    }

                    if (jobOptions.for_sla_policy !== false) {
                        $("#isForSLAPolicy").show();
                    }
                    console.log("Job Options");
                    console.log(jobOptions);
                } else {
                    // a new job
                    var start_date = moment().format(DATE_DISPLAY_FORMAT);
                    $("#cal_incr_start_date").val(start_date);
                }
            }

            function setReplicationJobOptions() {

                is_replication_target = true;

                $(".schedule-ctrl").hide();
                $(".custom-ctrl").hide();
                $("#scheduling-mode").hide();
                $("#backups-date-range").hide();
                $("#scheduled_or_on_demand").hide();
                $("#archive-mode-options").hide();
                $("#replication-description").show();
                // No test button for replication jobs
                $("#id_test_button").hide();


                if (jobOptions !== null) {
                    archive_targets_initialized = true;
                    storageSelect.options[0] = new Option(jobOptions.target, jobOptions.target_id);
                    selectedTarget = {'target_id': jobOptions.target_id};
                    $("#id_storage").val(jobOptions.target_id);

                    if (jobOptions.for_sla_policy !== false) {
                        $("#isForSLAPolicy").show();
                    }
                }
            }

            /*
             *   DOM Ready Function
             */
            $(function () {
                // Initialize form validation and event handlers
                initForm();

                // Initialize time controls
                initTimeControls();

                // Initialize date controls
                initDateControls();

                // Initialize retention controls
                initRetentionControls();

                // Load, build and initialize the Treeview
                initTreeView();

                // Initialize custom calendar controls
                initCustomCalendarControls();

                // Initialize date range
                initDateRangeControls();

                // Disable any form fields that are disabled by default
                $(".start-disabled").find("input, select").toggleDisabled();

                EmailTable = PHD.EmailTable($form);

                $inventoryTable.on("click", ".find-item", function () {
                    var id = $(this).closest("tr").data("dbid");
                    backupJobTreeviewSearch.goTo(id);
                });

                $inventoryTable.on("click", ".edit-row", function () {
                    var id = $(this).closest("tr").data("dbid"),
                        row = inventory[id];
                    row.edit();
                });

                initializeInventoryTable();

                // If this is an edit, disable "Now" radio button
                if (jobEdit) {
                    //$("#id_schedule_type_0").prop("disabled", true);
                    //$("#id_job_scheduled").prop("disabled", true);
                    //$("#id_job_now").prop("disabled", true);
                    $("#scheduled_or_on_demand").hide();
                }

                $inventoryTable.find("th .definition").popover({
                    appendTo: $form,
                    popoverClass: "backup-job-popover"
                });

                // Only set these options on edit
                if ((jobEdit && jobReplication) || !root_scope.showArchiving) {
                    setReplicationJobOptions();
                } else {
                    setJobOptions();
                }

                $(document)
                // Attach event handler to Hypervisor view select box changes
                    .on("hypervisorviewchange.backupjob", function (event, views) {
                        updateTreeView(views);
                        backupJobTreeview.clearSelections();
                        $inventoryTable.find(".inventory-row").each(function () {
                            inventory[$(this).data("dbid")].delete();
                        });
                    })
                    // Custom calendaring events.
                    .on("cancelCustomCalendar", function (event) {
                        handleScheduleTypeChange(SCHEDULE_TYPE_SCHEDULED);
                    })
                    .on("saveCustomCalendar", function (event, ical, description) {
                        if (jobOptions == null) {
                            // If a new backup copy job, create the options object for the ical.
                            jobOptions = {};
                        }
                        jobOptions.ical = ical;
                        setCalendarDescription(description);
                        $("#calendar_description").val(getCalendarDescription());
                    });
            });
        }

    }]);