angular.module('failoverJobModule',['ngDialog', 'ui.grid'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {
            }
        });
    }])

    .controller('failoverJobController', ['$scope', '$rootScope', '$http', '$analytics', 'ngDialog', '$timeout',
        function($scope, $rootScope, $http, $analytics, ngDialog) {

            $scope.mySelections = [];
            $scope.selectedItems = [];
            $scope.selectedInstancesAndRestorePoints = [];
            $scope.defaultSelectedInstancesAndRestorePoints = [];
            $scope.myExternalScope=$scope;

            $scope.cellSelectEditableTemplate = '<div style="padding-left:5px;padding-top:5px;"> ' +
                '<select style="width: 300px!important;" ng-class = "grid.appScope.defaultSelectedInstancesAndRestorePoints[rowRenderIndex].class" title="{{grid.appScope.defaultSelectedInstancesAndRestorePoints[rowRenderIndex].tooltip}}" ng-model = "backupSelected" ng-change="grid.appScope.backupOnChange(backupSelected)" >' +
                '<option ng-selected="{{currentBackup.name == grid.appScope.assetListToGetLatestBackups[rowRenderIndex].backup[0].name}}" ng-repeat="currentBackup in grid.appScope.assetListToGetLatestBackups[rowRenderIndex].backup" '+
                'ng-class = "currentBackup.certified == \'3\' ? \'greencolor-failover\' : currentBackup.certified == \'1\' ? \'redcolor\' : currentBackup.certified == \'4\' ? \'warningcolor\' : \'blackcolor\'" title="{{currentBackup.tooltip}}"> {{currentBackup.name}} </option>'+
                '</select>'+
                '</div>';
            $scope.failoverColumnDefs = [
                {field:'id',displayName:'',visible:false},
                {field:'assetName',displayName:'NAME',enableCellEdit:false,width:'40%'},
                {field:'defaultBackup',displayName:'BACKUP',name: 'COL_FIELD',cellTemplate:$scope.cellSelectEditableTemplate,width:'55%'}
            ];
            $scope.FailoverJobGridOptions={
                columnDefs: $scope.failoverColumnDefs,
                data:'assetListToGetLatestBackups',
                showSelectionCheckbox: true,
                selectWithCheckboxOnly: true,
                selectedItems: $scope.mySelections,
                rowHeight: 32,
                enableHorizontalScrollbar:0,
                enableVerticalScrollbar:2,
                resizable: true,
                enableColumnResize:true,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                onRegisterApi: function (gridApi) {
                    $scope.gridApi = gridApi;
                    gridApi.selection.on.rowSelectionChangedBatch($scope, function(rows){
                        $scope.selectedItems = gridApi.selection.getSelectedRows();
                        $scope.populateSelectedArray();
                    });
                    gridApi.selection.on.rowSelectionChanged($scope, function(rows){
                        $scope.selectedItems = gridApi.selection.getSelectedRows();
                        $scope.populateSelectedArray();
                    });
                }
            };

            $scope.populateSelectedArray = function () {
                $scope.selectedInstancesAndRestorePoints = [];
                if($scope.selectedItems.length > 0){
                    $scope.selectedItems.forEach(function(VM){
                        $scope.defaultSelectedInstancesAndRestorePoints.forEach(function(defaultSelectedVM){
                            if(VM.vm_id === defaultSelectedVM.vm_id){
                                var obj = {};
                                obj.vm_id = defaultSelectedVM.vm_id;
                                obj.restore_point_ref = defaultSelectedVM.restore_point_ref;
                                $scope.selectedInstancesAndRestorePoints.push(obj);
                            }
                        });
                    });
                }
                console.log($scope.selectedInstancesAndRestorePoints);
            };

            $scope.populateArray = function(){
                console.log($scope.assetListToGetLatestBackups);
                $scope.assetListToGetLatestBackups.forEach(function(VM, index){
                    var obj = {};
                    obj.vm_id = VM.vm_id;
                    if(VM.backup.length > 0){
                        obj.restore_point_ref = VM.backup[0].value;
                        obj.tooltip = VM.backup[0].tooltip;
                        obj.class = getBackupClass(VM.backup[0].certified);
                    }
                    $scope.defaultSelectedInstancesAndRestorePoints.push(obj);
                });
            };

            $scope.populateArray();

            function getBackupClass(certifiedStatus){
                switch(certifiedStatus){
                    case 1:
                        return "redcolor";
                        break;
                    case 3:
                        return "greencolor";
                        break;
                    case 4:
                        return "warningcolor";
                        break;
                }
            }

            $scope.backupOnChange = function(row) {
                console.log(row);
                row = row.trim();
                $scope.assetListToGetLatestBackups.forEach(function(VM, index){
                    VM.backup.forEach(function(backup){
                        if(backup.name === row){
                            $scope.defaultSelectedInstancesAndRestorePoints.forEach(function(defaultSelectedVM){
                                if(VM.vm_id === defaultSelectedVM.vm_id){
                                    defaultSelectedVM.restore_point_ref = backup.value;
                                    defaultSelectedVM.tooltip = backup.tooltip;
                                    defaultSelectedVM.class = getBackupClass(backup.certified);
                                }
                            });
                        }
                    });

                });
                $scope.populateSelectedArray();
                console.log($scope.selectedInstancesAndRestorePoints);
            };

            $scope.getRestorePointRef = function(backupName, backup){
                for(var i=0; i<backup.length; i++){
                    if(backupName == backup[i].name){
                        return backup[i].value;
                        break;
                    }
                }
            };

            $scope.closeThisDialog = function() {
                ngDialog.close('failover-dialog');
                var myscope = $scope.$parent;
                myscope.$destroy();
            };

            $scope.saveFailoverJob = function() {
                var obj = {};
                obj.failovers = $scope.selectedInstancesAndRestorePoints;
                if($scope.selectedInstancesAndRestorePoints.length === 0){
                    ngDialog.open({
                        dialogType:'Warning',
                        modelDialogId:'SelectBackupsWarning',
                        dialogMessage: "Select VM's to start the job.",
                        scope:$scope,
                        onConfirmOkButtonClick:'onConfirmWarning()'
                    });
                    return;
                }
                var dataObj = JSON.stringify(obj);
                console.log("dataObj "+dataObj);
                var mode = "";
                if($scope.Failover){
                    mode = "Failover";
                }
                else if($scope.Instantlab){
                    mode = "InstantLab";
                }
                var loadJobindicator = PHD.showLoadingIndicator("body", true, "Starting Job...");
                var jobOrderUrl = '/api/joborders/failover/'+$scope.selectedJobId +'/' + mode;

                $http({ method: 'put',
                    url : jobOrderUrl,
                    data: dataObj
                }).success(function(data, status, headers){
                    PHD.hideLoadingIndicator(loadJobindicator);
                    var message = mode + " Job submitted successfully! Once this job transitions to "+ mode + " status, select Commit in the Options drop-down to proceed. ";
                    ngDialog.open({
                        dialogType: 'Success',
                        dialogMessage: message,
                        ngDialogStyle:'width:450px;'
                    });
                }).error(function(response) {
                    var message = response.result[0].message;
                    $analytics && $analytics.eventTrack('Failure: ' + message, {  category: 'Failover', label: 'Protect' });
                    PHD.hideLoadingIndicator(loadJobindicator);
                    ngDialog.open({
                        dialogType:'Error',
                        modelDialogId:'FailoverJobPostErrorDialog',
                        scope:$scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick:'onConfirmError()'
                    });
                });
            };

            $scope.onConfirmError = function () {
                ngDialog.close('FailoverJobPostErrorDialog');
                $scope.jobErrorDialog = false;
            };

            $scope.onConfirmWarning = function () {
                ngDialog.close('SelectBackupsWarning');
            };

        }]);
