/**
 * Created by michelle on 2/27/17.
 */
var jobCalendarApp = angular.module('job-calendar', ['ui.calendar', 'ui.bootstrap', 'ngIcal']);


jobCalendarApp.controller('JobCalendarCtrl',
    function ($scope, $rootScope, $compile, $timeout, $http, uiCalendarConfig, ical) {

        $scope.events = [];
        $scope.eventSources = [];
        $scope.icalEvents = [];
        $scope.name = "";
        $scope.calendarItems = [];
        $scope.isEdit = false;
        $scope.view = {};
        $scope.tzOffset = 0;
        $scope.calendarName = 'jobsCalendar';
        $scope.calendarObj = {};
        $scope.fromiCal = true;     // perform initial load from iCal.
        $scope.jobCounter = 0;
        $scope.showDayWarning = false;

        /*
         Color palate to pick from for colorizing calendar items
         */
        var colorPalate = [

            '#76323F',
            '#663300',
            '#963126',
            '#266A2E',
            '#097054',
            '#587058',
            '#8D96A8',
            '#6D929B',
            '#004C70',
            '#006495',
            '#003366',
            '#525564',
            '#74828F',
            '#999999',
            '#9C8AA5',
            '#813772'

        ];


        /*
         * Called by the calendar when a day is clicked, selecting that date and going to Day mode.
         */
        $scope.selectDay = function (date, jsEvent, view) {

            var calendar = $scope.calendarName;
            $timeout(function () {
                if (uiCalendarConfig.calendars[calendar]) {
                    uiCalendarConfig.calendars[calendar].fullCalendar('gotoDate', date);
                    uiCalendarConfig.calendars[calendar].fullCalendar('changeView', 'agendaDay');
                }
            });

        };


        /* Render Tooltip */
        $scope.renderEvent = function (event, element, view) {
            element.prop('title', event.description);
            $compile(element)($scope);


        };


        /*
         * Called from the calendar to render the events.  Set the view start/end iCal, then loads the events from iCal, or updates existing event arrays.
         */
        $scope.renderView = function (view, element) {

            $scope.showDayWarning = view.type === 'agendaDay';
            var tzAdd = -$scope.tzOffset;
            $scope.view.startDate = moment(view.start).add(tzAdd, 'seconds').toDate();
            $scope.view.startiCal = new ical.Time.fromJSDate($scope.view.startDate);
            $scope.view.endDate = moment(view.end).add(tzAdd, 'seconds').toDate();
            $scope.view.endiCal = new ical.Time.fromJSDate($scope.view.endDate);

            $scope.eventSources.length = 0;
            $scope.jobCounter = 0;

            if ($scope.icalEvents.length > 0) {
                for (var key = 0; key < $scope.icalEvents.length; key++) {
                    $scope.renderEvents($scope.view, $scope.icalEvents[key]);
                }
                var events = $scope.icalEvents[key];
            }

        };

        /*
         * Calendar configuration object.
         * Note: Any functions here must be defined PRIOR to this object declaration.
         */

        $scope.jobCalendar = {
            config: {
                height: 650,
                width: 650,
                header: {
                    left: 'month,agendaDay,agendaWeek',
                    center: 'title',
                    right: 'today prev,next'
                },
                allDayDefault: false,
                defaultTimedEventDuration: '01:00:00',          // Show backups as 1 hour by default.
                forceEventDuration: true,
                nextDayThreshold: '00:00:00',
                eventLimit: true,
                navLinks: true,
                dayClick: $scope.selectDay,
                eventRender: $scope.renderEvent,
                viewRender: $scope.renderView,
                locale: $rootScope.userLanguageCode,
                // Angular UI calendar will set these from $locale if not set here.
                dayNames: $.datepicker._defaults.dayNames,
                dayNamesShort: $.datepicker._defaults.dayNamesShort,
                monthNames: $.datepicker._defaults.monthNames
            }
        };

        /*
         * Parses the job's iCal string into an array of icalEvents.
         *
         */
        $scope.createJobEvents = function () {

            var url = '/api/joborders';

            //get all of the job orders
            $http({
                method: 'get',
                url: url
            }).success(function (data, status, headers, response) {
                var job_schedule = data.data;

                if (job_schedule && job_schedule.length > 0) {
                    angular.forEach(job_schedule, function (value, key) {

                        if (value.enabled) {
                            var calendarObj = {}; //make an object to hold your results
                            calendarObj.include_ical = value.include_ical;
                            calendarObj.name = value.name;

                            $scope.calendarItems.push(calendarObj);

                            var ical_string = value.include_ical;
                            var jobName = value.name;

                            /*
                             parse out the jcal components.
                             If the ical string is defined, get jcal components and the name of the backup job
                             */
                            if (ical_string !== undefined) {
                                var jcal = ical.parse(ical_string);
                                var comp = new ical.Component(jcal);
                                $scope.icalEvents[key] = comp.getAllSubcomponents("vevent");
                                $scope.icalEvents[key].push({name: jobName});
                                $scope.renderEvents($scope.view, $scope.icalEvents[key]);

                            }
                        }

                    });
                }
            })

        };

        /*
         The event description will be displayed on hover
         */
        $scope.createEventDescription = function(event, addEdit) {
            var desc = event.title;
            var startTime = "";
            if (event.start !== undefined) {
                startTime = "at " + moment(event.start).format("hh:mm a");
            }
            desc += gettext(" Backup ") + startTime;
            if (event.rrule !== undefined) {
                var frequency = event.rrule.freq;
                if (frequency !== undefined) {
                    desc += gettext(", Recurring: ") + frequency.toLowerCase();
                }
                var interval = event.rrule.interval;
                if (interval !== undefined && interval != 1) {
                    desc += gettext(", Interval: ") + interval;
                }
                var repeat = event.rrule.parts;
                if (repeat !== undefined) {
                    if (repeat.BYDAY !== undefined) {
                        desc += gettext(", Day(s): ") + repeat.BYDAY.toString();
                    }
                    if (repeat.BYHOUR !== undefined) {
                        desc += gettext(", Hour(s): ") + repeat.BYHOUR.toString();
                    }
                    if (repeat.BYMINUTE !== undefined) {
                        desc += gettext(", Minute(s): ") + repeat.BYMINUTE.toString();
                    }
                    if (repeat.BYMONTHDAY !== undefined) {
                        desc += gettext(", Day of Month: ") + repeat.BYMONTHDAY.toString();
                    }
                }
            }

            return desc;
        };



        /*
         * Render the loaded and parsed iCal on the calendar.
         */
        $scope.renderEvents = function (view, icalEvents) {
            $scope.events = [];

            var vevents = icalEvents; //$scope.icalEvents;
            // get job name from last event.
            if (vevents === undefined || vevents.length <= 1) {
                return;
            }
            $scope.jobCounter++;
            var eventCount = 0;
            var jobName = icalEvents[vevents.length - 1].name;
            var eventsLength = vevents.length - 1;

            for (var i = 0; i < eventsLength; i++) {
                var vevent = vevents[i];
                var event = new ical.Event(vevent);

                //start/end on the time of the local machine
                var startDate = event.startDate.toJSDate();
                var endDate = event.endDate.toJSDate();
                var summary = event.summary;

                var originalEvent = {
                    id: eventCount++,
                    title: jobName,
                    start: startDate,
                    end: endDate,
                    stick: true,
                    color: $scope.getScheduleColor(),
                    repeated: false

                };
                var rules = vevent.getAllProperties('rrule');
                var dtstart = vevent.getFirstProperty('dtstart');
                var dtend = vevent.getFirstProperty('dtend');
                dtstart = dtstart.getFirstValue();
                dtend = dtend.getFirstValue();

                if (rules !== undefined && rules.length > 0) {
                    var newEvents = [];
                    var rrule = {};
                    for (var j = 0; j < rules.length; j++) {
                        var rule = rules[j].getFirstValue();
                        rrule = rule;

                        originalEvent.rrule = rrule;
                        originalEvent.description = $scope.createEventDescription(originalEvent, true);
                        newEvents.push(originalEvent);

                        if (typeof rule.iterator === "function") {

                            var iter = rule.iterator(dtstart);
                            for (var next = iter.next(); next; next = iter.next()) {

                                // skip if same date as dtstart.
                                // skip values outside the view range.
                                if (next.compare($scope.view.startiCal) < 0 || next.compare(dtstart) == 0) {
                                    continue;
                                }
                                if (next.compare($scope.view.endiCal) > 0) {
                                    break;
                                }
                                var unixTime = next.toUnixTime();
                                var hr = unixTime + 3600;
                                var milli = hr * 1000;
                                var start = next.toJSDate();
                                var event_end = new Date(+start + 3600000);
                                var eventObj = {
                                    id: eventCount++,
                                    title: jobName,
                                    start: start,
                                    end: event_end,
                                    // stick: true,
                                    color: $scope.getScheduleColor(),
                                    repeated: true,
                                    description: originalEvent.description,
                                    originalEvent: event

                                };

                                newEvents.push(eventObj);


                            }
                        }
                    }
                    Array.prototype.push.apply($scope.events, newEvents);

                } else {
                    originalEvent.description = $scope.createEventDescription(originalEvent, true);
                    $scope.events.push(originalEvent);
                }
            }

            $scope.eventSources.push($scope.events);

        };
        $scope.getScheduleColor = function() {

            if ($scope.jobCounter >= colorPalate.length) {
                $scope.jobCounter = 0;
            }
            return colorPalate[$scope.jobCounter];
        };

        $scope.initEvents = function () {

            var resp = PHD.Ajax.get("/api/date-time/?sid=" + $rootScope.local.id);
            /* Set appliance date time data */
            resp.done(function (data) {
                if (data !== undefined) {
                    var d = new Date();
                    $scope.tzOffset = data.tzOffset;
                }
            });

            $scope.createJobEvents();

        };

        $scope.rruleToiCal = function(rule) {
            var rrule = rule; //rule.freq;
            return rrule;
        };

        $scope.initJobsCalendarTab = function() {
            $scope.events.length = 0;
            $scope.eventSources.length = 0;
            $scope.jobCounter = 0;

            $scope.initEvents();
            var calendar = $scope.calendarName;
            $timeout(function () {
                if (uiCalendarConfig.calendars[calendar]) {
                    uiCalendarConfig.calendars[calendar].fullCalendar('render');
                }
            });
        };
    });