
function processJobsPage(PHD, window, $rootScope, $scope, $analytics, gettextCatalog, $interval, uiGridConstants, localStorageService) {

    var $activeJobCount = $("#active-job-count"),
        UserPreference, JobsTimer,
        tabs = [],
        initialTab = $.cookie("initial_tab_jobs") || 0,
        currentTabIndex = 0,
        DETAIL_TABLE_HEIGHT = 256,
        TOO_LARGE_FOR_SORT = 1000,
        MAX_JOB_ASSET_DETAILS = 50,
        TAB_BAR_HEIGHT = 42,
        slugs = ['active', 'manage', 'history', 'system', 'policies', 'calendar'],
        jobTypeIcon = [];

    var MIN_VIRTUAL_THRESHOLD = 30;

    $rootScope.isSelectedJobFileLevel  =false;
    $rootScope.isSelectedJobNAS = false;
    $scope.showCommitAndDiscardButtons = false;
    $scope.certifyJob = false;
    $scope.currentDCAJobIsFailover = false;

    jobTypeIcon["replication"] = "backup-copy";
    jobTypeIcon["restore"] = "recover";
    jobTypeIcon["instant recovery"] = "instant-recover";
    jobTypeIcon["backup"] = "backup";
    jobTypeIcon["archive"] = "backup-copy";
    jobTypeIcon['backup copy'] = "backup-copy";
    jobTypeIcon["certification"] = "certified-backup";

    function getJobIcon(jobType) {
        var icon = "unknown";
        var html;
        if (jobType !== null && jobType !== undefined) {
            jobType = jobType.toLowerCase();
            icon = jobTypeIcon[jobType];
        }
        switch (icon) {
            case "unknown":
                html = '<i class="icon-phd-unknown">' + '</i> ';
                break;
            case "backup-copy":
                html = '<i class="icon-uui-backup-copy">' + '</i> ';
                break;
            case "instant recovery":
                html = '<i class="icon-uui-instant-recover">' + '</i> ';
                break;
            case "certification":
                html = '<i class="icon-uui-certified-backup">' + '</i> ';
                break;
            case "backup":
            case "recover":
            case "restore":
            default:
                html = '<i class="icon-uui-' + jobTypeIcon[jobType] + '"></i> ';
                break;
        }
        return html;
    }

    function getJobMode(mode){
        var html = "";
        // don't put colon if undefined or archive as mode (since generic)
        if (mode !== undefined && mode != 'archive' && mode != 'Backup Copy' && mode != 'replication' && mode != 'Backup Copy (optimized)' ) {
           html = ": " + mode;
        }
        return html;
    }

    function getJobOperation(operation){
        var html = "";
        if (operation !== undefined) {
            html = ": " + operation;
        }
        return html;
    }

    function getJobType(type, row){
        var html = "";
        if (type !== undefined) {
            switch (type) {
                case "archive":
                case "backup copy":
                    html = "Backup Copy";
                    break;
                case "replication":
                    html = "Backup Copy (" + row.target +")";
                    break;
                case "backup":
                    html = "Backup";
                    break;
                case "certification":
                    html = "Data Copy Access";
                    break;
                default:
                    html = type;
                    break;
            }
        }
        return html;
    }

    $scope.getJobIcon = getJobIcon;
    $scope.getJobType = getJobType;
    $scope.getJobMode = getJobMode;
    $scope.getJobOperation = getJobOperation;
    $scope.getStatusIcon = getStatusIcon;

    $scope.getStatusString = function(row) {
        var statusText = row.status;
        switch (statusText) {                              //Added for DCA jobs to be consistent with Backup/Backup copy jobs
            case "Successful":
                statusText = "Success";
                row.status = "Success";                   // To avoid different sort values and css class
                break;
            case "Cancelled":
            case "Failed":
            case "Failure":
                statusText = "Error";
                break;
        }
        row.status = statusText;
        return statusText;
    };

    $scope.getSystemJobTypeIcon = function(row) {
        var icon = '</i>';
        var value = row.username;
        var icon_class = value === 'System' ? 'icon-managedhosting' :
            value.indexOf('bypass') !== -1 ? 'icon-logprograms' :  'icon-useralt';

        icon = '<i class="' + icon_class + ' icon" style="padding-right:4px">' + icon;
        return icon + value;
    };

    $scope.getStatusTitle = function(row) {
        return row.message !== undefined ? row.message :
                    (row.status !== undefined ? row.status : "");
    };

    $scope.getProgressBar = function(row, value) {
        var tooltip = (row.estimated_end !== null && row.estimated_end !== undefined) ? 'Est. Finish: ' + row.estimated_end : (value + '%');
        return '<div class="progress-bar" title="' + tooltip + '">' +
            '<div class="progress-meter" style="width:' + value + '%;"></div></div><span class="progress-count">'+ value + '%</span>';
    };
    
    function getJobTypeChar(jobID) {
        var jobIDPieces = String(jobID).split('.');
        return jobIDPieces[0].slice(-1);
    }

    $rootScope.JobsTimer.refreshControl = $("#jobs-page-preferences .timer-refresh");
    JobsTimer = $rootScope.JobsTimer;

    function setPageRefresh(interval) {
        refreshRate = interval;
        tabs.forEach(function (tab) {
            tab.setTimerInterval(refreshRate);
        });
    }

    function initTabs() {
        $(".ui-tabs-panel").each(function (index) {
            tabs[index] = new JobTab(index, $(this), slugs[index]).init();
        });

        tabs[currentTabIndex].activate();
    }

    $(document)
        .on("uinav.dashboard", stopAll)
        .on("uinav.protect", stopAll)
        .on("uinav.recover", stopAll)
        .on("uinav.configure", stopAll)
        .on("uinav.reports", stopAll)
        .on("userpreferencesave.jobspage", function (event, key, value) {
            setPageRefresh(value);
        })
        .on("timersetmanual.jobspage", function (event, $control) {
            $control.show();
        })
        .on("timerunsetmanual.jobspage", function (event, $control) {
            $control.hide();
        });

    function stopAll(event) {
        console.log("===>In Stop All: Jobs Tab, event " + event.namespace);
        JobsTimer.stop();
        for (var i = 0; i < tabs.length; i++) {
            tabs[i].isActive = false;
        }
        UserPreference.cancel();
        $interval.cancel($scope.stopJobDetails);
        $(document).off("uinav.dashboard", stopAll);
        $(document).off("uinav.protect", stopAll);
        $(document).off("uinav.recover", stopAll);
        $(document).off("uinav.configure", stopAll);
        $(document).off("uinav.reports", stopAll);
        $(document).off("userpreferencesave.jobspage");
        $(document).off("timertick.jobspage");
        $(document).off("timersetmanual.jobspage");
        $(document).off("timerunsetmanual.jobspage");
        $(document).off("editJob");
        $(document).off("limit-changed");
    }

    // lmc - move this up.
    var expandedItemsFormatter = {
        instances: function (value, row) {
            var nameArray = row.type === 'policy' ? getPolicyAssets(row) : getJobAssets(row);
            var names = nameArray.toString();
            var formattedNames = names.replace(/,/gi, "<br/>");
            return '<span class="truncate-long" title="' + names + '">' + formattedNames + '</span>';
        },
        rpo: function(value, row) {
            return '<span class="truncates item-name" title="' + value + '">' + getRPO(row) + '</span>';
        },
        rto: function(value, row) {
            return '<span class="truncates item-name" title="' + value + '">' + getRTO(row) + '</span>';
        },
        message: function(value, row) {
            return '<span class="truncates item-name" title="' + value + '">' + value + '</span>';
        },
        comment: function(value, row) {
            return '<span title="' + value + '">' + value + '</span>';
        }
    };

    function getRPO(row) {
        return (row.type === "certification") ? row.service_profile.rpo + " " + row.service_profile.rpo_frequency : "--";
    }

    function getRTO(row) {
        return (row.type === "certification") ? row.service_profile.rto + " " + row.service_profile.rto_frequency : "--";
    }

    function getJobAssets(row) {
        var assets = [];
        var param = {'max_size': MAX_JOB_ASSET_DETAILS, 'truncated': false};

        if(row.type === "certification"){
            for(var  i =0; i < row.certification_target.instances.length;i++) {
                var name = row.certification_target.instances[i].name;
                if (assets.length < param.max_size) {
                    assets.push(name);
                } else {
                    param.truncated = true;
                    break;
                }
            }
        }
        else{
            switch (row.app_id) {
                case APP_ID_VMWARE:
                    if (row.instances !== undefined) {
                        assets = _.isArray(row.instances) ? pluckWithLimit(row.instances, "secondary_name", param) : [row.instances];
                    }
                    break;
                case APP_ID_XEN:
                case APP_ID_AHV:
                    if(row.instances !== undefined) {
                        assets = assets = _.isArray(row.instances) ? pluckWithLimit(row.instances, "secondary_name", param) : [row.instances];
                    }
                    break;
                case APP_ID_FILE_LEVEL:
                    if (row.clients !== undefined && _.isArray(row.clients)) {
                        assets = pluckWithLimit(row.clients, "name", param);
                    }
                    break;
                default:
                    if (row.instances !== undefined) {
                        if (_.isArray(row.instances)) {
                            row.instances.forEach(function (obj, index) {
                                if (assets.length < MAX_JOB_ASSET_DETAILS) {
                                    var part1 = (obj.primary_name !== undefined) ? obj.primary_name : "";
                                    var part2 = (obj.secondary_name !== undefined) ?  obj.secondary_name : "";
                                    var name = (part1 !== "") ? part1 + ((part2 !== "") ? "/" + part2 : "") : ((part2 !== "") ? part2 : "");
                                    assets.push(name);
                                } else {
                                    param.truncated = true;
                                }
                            });
                        }
                    }
                    break;
            }
        }
        // Do not return all if too many to display.
        if (param.truncated) {
            assets.push('Edit Job to see all...');
        }
        return assets;
    }

    function pluckWithLimit(sourceArray, property, param) {
        var pluckedArray = [];
        for (var i = 0; i < sourceArray.length; i++) {
            if (pluckedArray.length < param.max_size) {
                pluckedArray.push(sourceArray[i][property]);
            } else {
                param.truncated = true;
                break;
            }
        }
        return pluckedArray;
    }

    function getPolicyAssets(row) {
        var assets = [];
        var param = {'max_size': MAX_JOB_ASSET_DETAILS, 'truncated': false};
        if (row.instances !== undefined) {
            assets = _.isArray(row.instances) ? pluckWithLimit(row.instances, "name", param) : [row.instances];
        }
        // Do not return all if too many to display.
        if (param.truncated) {
            assets.push('Edit Policy to see all...');
        }
        return assets;
    }

    var typeModeTemplate = '<div class="ui-grid-cell-contents"> ' +
        '<span class="job-type-display-grid" ' +
        'templ-compile-html="{{grid.appScope.getJobIcon(row.entity.type)}} {{grid.appScope.getJobType(row.entity.type, row.entity)}}' +
        '{{grid.appScope.getJobMode(row.entity.mode)}}{{grid.appScope.getJobOperation(row.entity.operation)}}"' + '>' +
        '</span>' +
        '</div>';

    var statusTemplate = '<div class="ui-grid-cell-contents"> ' +
        '<span class="job-status-display-grid" ' +
        'title="{{grid.appScope.getStatusTitle(row.entity)}}"' +
        'templ-compile-html="{{grid.appScope.getStatusIcon(row.entity)}} {{grid.appScope.getStatusString(row.entity)}}"' + '>' +
        '</span>' +
        '</div>';


    var progressTemplate = '<div class="ui-grid-cell-contents"> ' +
        '<span class="job-status-display-grid" ' +
        'templ-compile-html="{{grid.appScope.getProgressBar(row.entity, row.entity.percent_complete)}}"' +
        '</span>' +
        '</div>';

    var systemJobUsernameTemplate = '<div class="ui-grid-cell-contents"> ' +
        '<span class="job-type-display-grid" ' +
        'templ-compile-html="{{grid.appScope.getSystemJobTypeIcon(row.entity)}}"' +
        '</span>' +
        '</div>';

    var nameCol = {
        field: "name",
        headerCellFilter: 'translate',
        cellTooltip: true,
        displayName: gettextCatalog.getString("Name"),
        sortable : true
    };

    var jobNameCol = jQuery.extend(true, {}, nameCol);
    jobNameCol.displayName = gettextCatalog.getString("Job Name");

    var statusCol = {
        field: "status",
        headerCellFilter: 'translate',
        displayName: gettextCatalog.getString("Status"),
        cellTemplate: statusTemplate,
        sortable : true,
        filterCellFiltered: true,
        width: 115
    };

    var typeModeCol = {
        field: "type",
        headerCellFilter: 'translate',
        displayName: gettextCatalog.getString("Type/Mode"),
        cellTemplate: typeModeTemplate,
        sortable : true,
        filter: {
            condition:  function(term, value, row, col) {
                var type = $scope.getJobType(value, row.entity).toLocaleLowerCase();
                var mode = $scope.getJobMode(row.entity.mode).toLocaleLowerCase();
                var operation = $scope.getJobOperation(row.entity.operation).toLocaleLowerCase();
                var matchTerm = type + mode + operation;
                term = term.toLocaleLowerCase();
                return (matchTerm.match(term) !== null);
            }
        }
    };

    var typeCol = jQuery.extend(true, {}, typeModeCol);
    typeCol.displayName = gettextCatalog.getString("Type");
    typeCol.filter = {
        condition:  function(term, value, row, col) {
            var matchTerm = $scope.getJobType(value, row.entity).toLocaleLowerCase();
            term = term.toLocaleLowerCase();
            return (matchTerm.match(term) !== null);
        }
    };

    var startTimeCol = {
        field: "start_time",
        headerCellFilter: 'translate',
        displayName: gettextCatalog.getString("Started"),
        width: 165,
        sortable : true,
        /* sorting algorithm doesn't have row parameters
        sortingAlgorithm: function(a, b, rowA, rowB){
            if(rowA.entity.sort_start_time === rowB.entity.sort_start_time) return 0;
            if(rowA.entity.sort_start_time < rowB.entity.sort_start_time) return -1;
            return 1;
        }, */
        sort: {
            direction: 'desc',
            priority: 0
        },
        type: 'date'
    };

    var startDateCol = jQuery.extend(true, {}, startTimeCol);
    startDateCol.field = "start_date";

    var clientNameCol = {
        field: "client_name",
        headerCellFilter: 'translate',
        cellTooltip: true,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Host/Server"),
        sortable : true
    };

    var assetNameCol = {
        field: "asset_name",
        headerCellFilter: 'translate',
        cellTooltip: true,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Asset"),
        sortable : true
    };

    var systemNameCol = {
        field: "system_name",
        headerCellFilter: 'translate',
        cellTooltip: true,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Appliance"),
        sortable : true
    };

    var systemCol = jQuery.extend(true, {}, systemNameCol);
    systemCol.field = "system";
    systemCol.width = 175;

    var sort_ids = function (a, b) {
        if (a === '' || a === 'None' || a === 'N/A') {
            if (a === 'Never' && b === 'N/A'){
                return 1;
            }else{
                return -1;
            }
        }
        if (b === '' || b === 'None'|| b === 'N/A') {
            if (b === 'None' && a === 'N/A'){
                return -1;
            }else{
                return 1;
            }
        }
        return (a < b) ?
            -1 : (a > b) ?
                1 : 0;
    };

    var joborderIdCol = {
        field: "id",
        headerCellFilter: 'translate',
        cellTooltip: true,
        visible: false,
        displayName: gettextCatalog.getString("ID"),
        sortable : true,
        cellClass: 'ui-grid-numerical',
        width: 50
    };

    var idCol = jQuery.extend(true, {}, joborderIdCol);
    idCol.cellFilter = "idNumberFilter";
    idCol.filterCellFiltered = true;
    idCol.sortingAlgorithm = sort_ids;

    var notificationIdCol = jQuery.extend(true, {}, idCol);
    notificationIdCol.field = "notification_id";

    var sizeCol = {
        field: "size",
        headerCellFilter: 'translate',
        visible: false,
        displayName: gettextCatalog.getString("Size (MB)"),
        sortable : true,
        cellClass: 'ui-grid-numerical',
        width: 100,
        type: "number"
    };

    var filesCol = {
        field: "files",
        headerCellFilter: 'translate',
        visible: false,
        displayName: gettextCatalog.getString("Files"),
        sortable : true,
        cellClass: 'ui-grid-numerical',
        cellFilter: "nullTestFilter",
        width: 100,
        type: "number"
    };

    var percentCompleteCol = {
        field: "percent_complete",
        headerCellFilter: 'translate',
        visible: true,
        displayName: gettextCatalog.getString("Progress"),
        sortable : true,
        cellTemplate: progressTemplate,
        type: "number"
    };

    var sort_duration = function (a, b) {
        if (a === "" || b === null) {
            return 1;
        }
        if (b === "" || a === null) {
            return -1;
        }
        var aVals = a.split(":").map(function (val) {
                return parseInt(val, 10);
            }),
            bVals = b.split(":").map(function (val) {
                return parseInt(val, 10);
            });

        a = (aVals[0] * 60 * 60) + (aVals[1] * 60) + aVals[2];
        b = (bVals[0] * 60 * 60) + (bVals[1] * 60) + bVals[2];
        return a - b;
    };

    var durationCol = {
        field: "duration",
        headerCellFilter: 'translate',
        visible: true,
        displayName: gettextCatalog.getString("Duration"),
        sortable : true,
        sortingAlgorithm: sort_duration
    };

    var commentCol = {
        field: "comment",
        headerCellFilter: 'translate',
        visible: false,
        cellTooltip: true,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Comment"),
        sortable : true
    };

    var estimatedEndCol = {
        field: "estimated_end",
        headerCellFilter: 'translate',
        visible: false,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Est Finish"),
        sortable : true,
        type: "date"
    };

    var applicationCol = {
        field: "application",
        headerCellFilter: 'translate',
        visible: false,
        cellTooltip: true,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Application"),
        sortable : true
    };

    var applicationNameCol = jQuery.extend(true, {}, applicationCol);
    applicationNameCol.field = "application_name";

    var appNameCol = jQuery.extend(true, {}, applicationCol);
    appNameCol.field = "app_name";

    var scheduleCol = {
        field: "calendar_str",
        headerCellFilter: 'translate',
        visible: true,
        cellTooltip: true,
        cellFilter: "nullTestFilter|calendarFilter",
        displayName: gettextCatalog.getString("Schedule"),
        sortable : true
    };

    var sort_two_dates = function (a, b) {
        if (a === '' || a === 'Never' || a === 'N/A') {
            if (a === 'Never' && b === 'N/A'){
                return 1;
            }else{
                return -1;
            }
        }
        if (b === '' || b === 'Never'|| b === 'N/A') {
            if (b === 'Never' && a === 'N/A'){
                return -1;
            }else{
                return 1;
            }
        }
        var x, y;
        if (_.isArray(a)) {
            x = a.map(function (val) {
                return DateTimeHandler.getUSDateObject($rootScope.userLanguageCode, val);
            });
            x = x[0] < x[1] ? x[0] : x[1];
        } else {
            x = DateTimeHandler.getUSDateObject($rootScope.userLanguageCode, a);
        }
        if (_.isArray(b)) {
            y = b.map(function (val) {
                return DateTimeHandler.getUSDateObject($rootScope.userLanguageCode, val);
            });
            y = y[0] < y[1] ? y[0] : y[1];
        } else {
            y = DateTimeHandler.getUSDateObject($rootScope.userLanguageCode, b);
        }
        return (x < y) ?
            -1 : (x > y) ?
                1 : 0;
    };

    var lastRunCol = {
        field: "last_time",
        headerCellFilter: 'translate',
        visible: true,
        displayName: gettextCatalog.getString("Last Run"),
        sortable : true,
        sortingAlgorithm: sort_two_dates
    };

    var nextRunCol = {
        field: "next_time",
        headerCellFilter: 'translate',
        visible: true,
        cellFilter: "dateStringOrNA",
        displayName: gettextCatalog.getString("Next Run"),
        sortable : true,
        sortingAlgorithm: sort_two_dates
    };

    var rpoFormattedCol = {
        field: "rpo_formatted",
        headerCellFilter: 'translate',
        visible: true,
        displayName: gettextCatalog.getString("RPO"),
        sortable : true
    };

    var retentionCol = {
        field: "retention",
        headerCellFilter: 'translate',
        visible: true,
        displayName: gettextCatalog.getString("Retention"),
        sortable : true
    };

    var descriptionCol = {
        field: "name",
        headerCellFilter: 'translate',
        cellTooltip: true,
        visible: false,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Description"),
        sortable : true
    };

    var enabledCol = {
        field: "enabled",
        headerCellFilter: 'translate',
        displayName: gettextCatalog.getString("Enabled"),
        cellFilter: "true_or_false",
        sortable : true,
        type: "boolean",
        visible: false,
        filterCellFiltered: true
    };

    var categoryCol = {
        field: "category",
        headerCellFilter: 'translate',
        cellTooltip: true,
        visible: true,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Name"),
        sortable : true,
        width: 175
    };

    var sysusernameCol = {
        field: "username",
        headerCellFilter: 'translate',
        cellTooltip: true,
        visible: true,
        cellTemplate: systemJobUsernameTemplate,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Type"),
        sortable : true,
        width: 150
    };

    var eventTimeCol = {
        field: "event_time",
        headerCellFilter: 'translate',
        cellTooltip: true,
        visible: true,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Started"),
        sortable : true,
        width: 175
    };

    var messageCol = {
        field: "message",
        headerCellFilter: 'translate',
        cellTooltip: true,
        visible: true,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Message"),
        sortable : true,
        width: '*'
    };

    var replicationJobCol = {
        field: "replication",
        headerCellFilter: 'translate',
        visible: false,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Backup Copy (Hot)"),
        sortable : true
    };

    var archiveJobCol = {
        field: "archive",
        headerCellFilter: 'translate',
        visible: false,
        cellFilter: "nullTestFilter",
        displayName: gettextCatalog.getString("Backup Copy (Cold)"),
        sortable : true
    };

    $scope.jobActiveColumnDefs = [
        jobNameCol,
        assetNameCol,
        statusCol,
        typeModeCol,
        startDateCol,
        percentCompleteCol,
        durationCol,
        systemNameCol,
        commentCol,
        estimatedEndCol,
        applicationCol,
        idCol
    ];

    $scope.jobHistoryColumnDefs = [
        nameCol,
        jQuery.extend(true, {}, statusCol),
        jQuery.extend(true, {}, typeModeCol),
        startTimeCol,
        clientNameCol,
        jQuery.extend(true, {}, assetNameCol),
        jQuery.extend(true, {}, systemNameCol),
        appNameCol,
        sizeCol,
        filesCol,
        jQuery.extend(true, {}, idCol)
    ];

    $scope.jobManagerColumnDefs = [
        jQuery.extend(true, {}, nameCol),
        jQuery.extend(true, {}, statusCol),
        typeCol,
        scheduleCol,
        lastRunCol,
        nextRunCol,
        jQuery.extend(true, {}, systemNameCol),
        applicationNameCol,
        descriptionCol,
        enabledCol,
        joborderIdCol,
    ];

    $scope.systemJobsColumnDefs = [
        categoryCol,
        sysusernameCol,
        eventTimeCol,
        messageCol,
        systemCol,
        notificationIdCol
    ];

    $scope.slaPoliciesColumnDefs = [
        jQuery.extend(true, {}, nameCol),
        rpoFormattedCol,
        retentionCol,
        jQuery.extend(true, {}, systemCol),
        jQuery.extend(true, {}, applicationNameCol),
        jQuery.extend(true, {}, descriptionCol),
        replicationJobCol,
        archiveJobCol
    ];

    $scope.gridList = [0, 0, 0, 0, 0, 0];
    $scope.initialColumnOrder = [[], [], [], [], [], []];
    $scope.columnOrderRestored = false;
    $scope.placeHolder = gettext("There are no jobs to display");

    $scope.jobsGrid ={
        //columnDefs:
        //data: 'jobsList',
        rowHeight: 30,
        enableHorizontalScrollbar:0,
        //enableVerticalScrollbar:0,
        enableGridMenu: true,
        enableFiltering: true,
        resizable: true,
        multiSelect: false,
        //enableRowHashing: false,
        enableFullRowSelection : true,
        enableRowHeaderSelection: false,
        gridMenuShowHideColumns: true,
        enableColumnMenus: true,
        gridMenuCustomItems: [{
            title: gettextCatalog.getString('Reset column defaults'),
            action: function($event) {
                // clear the current grids, then re-draw as the original.
                $scope.restoreGridState();
            },
            order: 99
        }],
        gridMenuTitleFilter: function(title) {
            return gettextCatalog.getString(title);
        },
        onRegisterApi: function (gridApi) {
            $scope.onRegisterGrid(gridApi);
        }
    };

    $scope.savedState = [undefined, undefined, undefined, undefined, undefined, undefined];

    $scope.onRegisterGrid = function(gridApi) {
        var tabIndex = gridApi.grid.options.tabIndex;
        $scope.gridApi = gridApi;
        $scope.gridList[tabIndex] = gridApi;
        gridApi.core.on.rowsRendered($scope, function() {
            var tabIndex = $scope.gridApi.grid.options.tabIndex;
            if (!$scope.stateExists(tabIndex)) {
                $scope.saveGridState(tabIndex);
                $scope.restorePersistedState($scope.gridApi, tabIndex);
            }
        });
        // configure handler for row selection changes
        gridApi.selection.on.rowSelectionChanged($scope, $scope.onSelectionChanged);
        gridApi.selection.on.rowSelectionChangedBatch($scope, $scope.onSelectionChanged);
        
        // configure handler to persist grid state changes
        gridApi.colMovable.on.columnPositionChanged($scope, $scope.persistGridState);
        gridApi.colResizable.on.columnSizeChanged($scope, $scope.persistGridState);
        gridApi.core.on.columnVisibilityChanged($scope, $scope.persistGridState);
        gridApi.core.on.filterChanged($scope, $scope.persistGridState);
        gridApi.core.on.sortChanged($scope, $scope.persistGridState);
    };

    /*
     * Returns true if a cached state exists for the job tab, false if not.
     * Checks for the existence of columns because these may be delayed (returned after API returns).
     */
    $scope.stateExists = function(tabIndex) {
        return ($scope.savedState[tabIndex] !== undefined && $scope.savedState[tabIndex].columns.length > 0);
    };

    /*
     * Use ui-grid state to save information in an array about the original grid: column order, width, filters, etc.
     */
    $scope.saveGridState = function(tabIndex) {
        $scope.savedState[tabIndex] = $scope.gridApi.saveState.save();
    };

    /*
     * Save state of the grid in local storage: job-tab-N, where the state is defined by the UI-grid.
     */
    $scope.persistGridState = function() {
        var tabIndex = $scope.gridApi.grid.options.tabIndex;
        var token = 'job-tab-' + tabIndex;
        localStorageService.set(token, $scope.gridApi.saveState.save());
    };

    /*
     * Restore the grid's state from the local storage for this tab, if it exists.
     */
    $scope.restorePersistedState = function(gridApi, tabIndex) {
        if (!$scope.columnOrderRestored && gridHasColumns(gridApi)) {
            // The columns are present and need to be restored from saved state.
            var token = 'job-tab-' + tabIndex;
            var jobState = localStorageService.get(token);
            if (jobState !== null && jobState !== undefined) {
                gridApi.saveState.restore($scope, jobState);
                $scope.columnOrderRestored = true;
            }
        }
    };

    // Returns true if the grid exists and its columns are present, and false otherwise.
    // The ui-grid will first return the grid structure, with the column population delayed.
    var gridHasColumns = function(gridApi) {
        return gridApi !== null && gridApi.grid.columns !== undefined && gridApi.grid.columns.length > 0;
    };

    /*
     * Restore this grid's state from the array (the original tabs), and delete the local storage.
     */
    $scope.restoreGridState = function() {
        var tabIndex = $scope.gridApi.grid.options.tabIndex;
        $scope.gridApi.saveState.restore($scope, $scope.savedState[tabIndex]);
        setTimeout(function () {
            var token = 'job-tab-' + tabIndex;
            localStorageService.remove(token);
        }, 100);
        $scope.columnOrderRestored = false;
    };

    $scope.onSelectionChanged = function(rows) {
        var gridApi = $scope.gridApi;
        //console.log("selection in tab: " + $scope.gridApi.grid.options.tabIndex);
        var self = tabs[$scope.gridApi.grid.options.tabIndex];
        var selectedItems = gridApi.selection.getSelectedRows();
        if (selectedItems.length >= 1) {
            var sameRow = (selectedItems.length === 1 && self.selectedID === selectedItems[0].id);
            self.selectedID = selectedItems[0].id;
            self.rowData = selectedItems[0];
            self.$selected = selectedItems;
            var Ids = [];
            for (var i = 0; i < selectedItems.length; i++) {
                Ids.push( { 'id' : selectedItems[i].id, 'enabled' : selectedItems[i].enabled, 'type': selectedItems[i].type });
            }
            self.selectedIDs = Ids;
            if (selectedItems.length > 1) {
                $scope.showCommitAndDiscardButtons = false;
                $scope.certifyJob = false;
                setTimeout(function () {
                    $scope.$apply();
                }, 100);
                self.updateButtons();
            } else {
                self.handleRowSelect(self, self.selectedID, self.$selected, null, {data: self.rowData, sameRow: sameRow});
            }
        } else if (selectedItems.length === 0) {
            self.selectedID = null;
            self.rowData = null;
            self.$selected = [];
            self.selectedIDs = [];
            $scope.currentDCAJobDisabled = false;
            self.updateButtons();
        }
    };


    $scope.jobsActiveGrid = jQuery.extend(true, {}, $scope.jobsGrid);
    $scope.jobsActiveGrid.columnDefs = $scope.jobActiveColumnDefs;
    $scope.jobsActiveGrid.data = 'activeJobsList';
    $scope.jobsActiveGrid.tabIndex = JOB_TAB_ACTIVE;

    $scope.jobsManagerGrid = jQuery.extend(true, {}, $scope.jobsGrid);
    $scope.jobsManagerGrid.multiSelect = true;
    $scope.jobsManagerGrid.enableRowHeaderSelection = true;
    $scope.jobsManagerGrid.selectionRowHeaderWidth = 35;
    $scope.jobsManagerGrid.columnDefs = $scope.jobManagerColumnDefs;
    $scope.jobsManagerGrid.data = 'jobsManagerList';
    $scope.jobsManagerGrid.tabIndex = JOB_TAB_MANAGER;
    $scope.jobsManagerGrid.rowTemplate =         "<div ng-repeat=\"(colRenderIndex, col) in colContainer.renderedColumns track by col.uid\" " +
        "  ui-grid-one-bind-id-grid=\"rowRenderIndex + '-' + col.uid + '-cell'\" class=\"ui-grid-cell\" " +
        "  ng-class=\"{ 'ui-grid-row-header-cell': col.isRowHeader, 'ui-grid-row-selected': row.isSelected, 'job-enabled-false': row.entity.enabled == 0 }\" " +
        "  role=\"{{col.isRowHeader ? 'rowheader' : 'gridcell'}}\" ui-grid-cell></div>";

    $scope.jobsHistoryGrid = jQuery.extend(true, {}, $scope.jobsGrid);
    $scope.jobsHistoryGrid.columnDefs = $scope.jobHistoryColumnDefs;
    $scope.jobsHistoryGrid.data = 'jobsHistoryList';
    $scope.jobsHistoryGrid.tabIndex = JOB_TAB_HISTORY;

    $scope.jobsSystemGrid = jQuery.extend(true, {}, $scope.jobsGrid);
    $scope.jobsSystemGrid.columnDefs = $scope.systemJobsColumnDefs;
    $scope.jobsSystemGrid.data = 'systemJobsList';
    $scope.jobsSystemGrid.tabIndex = JOB_TAB_SYSTEM;
    $scope.jobsSystemGrid.enableFullRowSelection = false;

    $scope.slaPoliciesGrid = jQuery.extend(true, {}, $scope.jobsGrid);
    $scope.slaPoliciesGrid.columnDefs = $scope.slaPoliciesColumnDefs;
    $scope.slaPoliciesGrid.data = 'slaPoliciesList';
    $scope.slaPoliciesGrid.tabIndex = JOB_TAB_POLICIES;

    $scope.getRows = function(tabIndex) {
        var rows = null;
        switch (tabIndex) {
            case JOB_TAB_ACTIVE:
                rows = $scope.activeJobsList;
                break;
            case JOB_TAB_MANAGER:
                rows = $scope.jobsManagerList;
                break;
            case JOB_TAB_HISTORY:
                rows = $scope.jobsHistoryList;
                break;
            case JOB_TAB_SYSTEM:
                rows = $scope.systemJobsList;
                break;
            case JOB_TAB_POLICIES:
                rows = $scope.slaPoliciesList;
                break;
        }
        return rows;
    };

    $scope.selectGridRowsByIDs = function(ids, tabIndex) {
        var gridApi = $scope.gridList[tabIndex];
        gridApi.grid.modifyRows($scope.getRows(tabIndex)); //$scope.jobsList);
        setTimeout(function() {
            var rows = $scope.getRows(tabIndex);
            for (var i = 0; i < rows.length; i++) {
                var row = rows[i];
                for (var j = 0; j < ids.length; j++) {
                    if (row.id === ids[j].id) {
                        gridApi.selection.selectRow(row);
                        break;
                    }
                }
            }
        }, 0);
    };

    $scope.updateGridData = function(tabIndex, data) {
        var gridToShow = null;
        switch (tabIndex) {
            case JOB_TAB_ACTIVE:
                $scope.activeJobsList = data;
                gridToShow = $scope.jobsActiveGrid;
                break;
            case JOB_TAB_MANAGER:
                $scope.jobsManagerList = data;
                gridToShow = $scope.jobsManagerGrid;
                break;
            case JOB_TAB_HISTORY:
                $scope.jobsHistoryList = data;
                gridToShow = $scope.jobsHistoryGrid;
                break;
            case JOB_TAB_SYSTEM:
                $scope.systemJobsList = data;
                gridToShow = $scope.jobsSystemGrid;
                break;
            case JOB_TAB_POLICIES:
                $scope.slaPoliciesList = data;
                gridToShow = $scope.slaPoliciesGrid;
                break;
        }
        if (gridToShow && (currentTabIndex === tabIndex)) {
            if ($scope.gridList[tabIndex] !== 0) {
                $scope.gridApi = $scope.gridList[tabIndex];
            }
            //console.log("--->jobs update for tab: " + $scope.gridApi.grid.options.tabIndex);
            gridToShow.minRowsToShow = data.length;
            $scope.gridApi.grid.gridHeight = $scope.setGridHeight(tabIndex, !tabs[tabIndex].$inventory.is(":hidden"));
            var threshold = $scope.gridApi.grid.gridHeight / gridToShow.rowHeight;
            if (threshold > MIN_VIRTUAL_THRESHOLD) {
                gridToShow.virtualizationThreshold = threshold;
            } else {
                gridToShow.virtualizationThreshold = MIN_VIRTUAL_THRESHOLD;
            }
        } else {
            console.log("--->jobs data skipped, tab: " + currentTabIndex + ": not: "+ tabIndex);
        }
    };

    $scope.getAvailableVerticalSpace = function($element, fullHeightContainer) {
        var $parent = $element.parent(),
            $container = $element.closest(fullHeightContainer);

        if (!$container.length) {
            return $parent.outerHeight();
        } else {
            return $container.outerHeight();
        }
    };

    $scope.setGridHeight = function(tabIndex, detailsOpen) {
        tabs[tabIndex].fullHeight = $scope.getAvailableVerticalSpace(tabs[tabIndex].$buttons, ".full-height") - TAB_BAR_HEIGHT;
        var gridHeight = tabs[tabIndex].fullHeight;
        if (detailsOpen) {
            gridHeight -= tabs[tabIndex].inventoryTableHeight;
        }
        $scope.setGridElementHeight($('.ui-grid-full-height'), gridHeight);
        //console.log(gridHeight);
        return gridHeight;
    };

    $scope.setGridElementHeight = function($element, height) {
        $element.height(height);
    };

    var JobTab = function (index, panel, slug) {
        this.index = index;
        this.$panel = panel;
        this.$table = this.$panel.find(".active-table");
        this.$buttons = this.$panel.find(".button-bar");
        this.$inventory = this.$panel.find(".job-detail");
        this.$inventoryTable = this.$inventory.find("table");

        this.$dragHandle = this.$panel.find(".handle");
        this.namespace = "jobs" + slug;

        var placeHolder = gettext("There are no jobs to display");
        if ($rootScope.user.monitor) {
            if (this.index === JOB_TAB_MANAGER) {
                placeHolder = gettext("Monitor users cannot view these jobs");
            } else if (this.index === JOB_TAB_POLICIES) {
                placeHolder = gettext("Monitor users cannot view these policies");
            }
        } else if (this.index === JOB_TAB_POLICIES) {
            placeHolder = gettext("There are no policies to display");
        }
        this.placeHolder = placeHolder;

        // No need for the PHD.DataTable.
        this.dataTable = null;

        switch (this.index) {
            case JOB_TAB_ACTIVE:
                this.grid = $scope.jobsActiveGrid;
                break;
            case JOB_TAB_MANAGER:
                this.grid = $scope.jobsManagerGrid;
                break;
            case JOB_TAB_HISTORY:
                this.grid = $scope.jobsHistoryGrid;
                break;
            case JOB_TAB_SYSTEM:
                this.grid = $scope.jobsSystemGrid;
                break;
            case JOB_TAB_POLICIES:
                this.grid = $scope.slaPoliciesGrid;
                break;
        }

        // lmc - no hierarchy
        var inventoryTableDefaults = {
            select: false,
            sortable: true,
            cellClasses: this.index === JOB_TAB_ACTIVE ? ["numerical", null, "truncate-long", "long-text truncate-wide"] :
                                                        [null, null, "truncate-long", "long-text"]
        };
        this.JobInventory = PHD.DataTable(this.$inventoryTable, inventoryTableDefaults);
        this.JobInventory.registerFormatters(expandedItemsFormatter);

        this.$buttons.find(".jobcreate-btn-group").flyout();

        this.$selected = this.$table.find(".current-row");
        // when a single row is selected.
        this.selectedID = null;
        // when multiple rows are selected (checkbox)
        this.selectedIDs = [];
        this.rowData = null;

        this.isActive = false;
        this.timestamp = 0;
        this.inventoryTimestamp = 0;
        this.inventoryOpen = false;
        this.sort = null;

        //lmc - if Job manager, use joborders resource; otherwise use jobs resource.
        this.inventoryTableHeight = DETAIL_TABLE_HEIGHT;
        // sdmc - hard coded in language part of URL rather than differentiate between first additional URL parameter or not
        if (slug === 'manage') {
            this.url = "/api/joborders/" + "?lang=" + $rootScope.userLanguageCode;
        } else {
            if (slug === 'history'){
                this.url = "/api/jobs/history/?showFlat=true&sort=desc&limit=" + $rootScope.jobHistoryLimit + "&lang=" + $rootScope.userLanguageCode;
            } else if (slug === 'system') {
                this.url = "/api/jobs/system/?sort=desc&limit=" + $rootScope.systemJobHistoryLimit + "&lang=" + $rootScope.userLanguageCode;
            } else if (slug === 'policies') {
                this.url = "/api/sla-policies";
            } else if (slug === 'calendar') {
                this.url = "/api/joborders/nodata";       // TODO: jobscalendar also loads joborders, needed?  add any special joborders options for calendaring here.
            } else if (slug === 'active') {
                this.url = "/api/jobs/active" + "?lang=" + $rootScope.userLanguageCode;
            } else {
                this.url = "/api/jobs/" + slug;
            }
        }
    };

    JobTab.prototype.init = function () {
        var self = this;
        this.$inventory.hide();

        this.initButtons();

        $(document)
            .on("timertick.jobspage", $.proxy(this.handleTimerTick, this))
            .on("editJob", function(event) {
                // If Job manager, or SLA Policy Manager, update.  No need if on another tab as the tab activation will trigger a refresh.
                if (self.isActive && (self.index === JOB_TAB_MANAGER || self.index === JOB_TAB_POLICIES)) {
                    self.update();
                }
            })
            .on("limit-changed", function(event, limit_type, newValue) {
                if (self.index === limit_type) {
                    self.update();
                }
            });


        // Handle events thrown by DataTable
        this.$table
            .on("change", ".jobs-checkbox", $.proxy(this.handleCheckboxChange, this))
            .on("rowselect", "tr.clickable", $.proxy(this.handleRowSelect, this))
            .on("rowdelete", $.proxy(this.handleRowDelete, this))
            .on("updatefinished", $.proxy(this.handleRowAddition, this));

        this.$dragHandle.dragdelta({
            autoHide: true,
            disabled: true,
            min: 360,
            max: window.innerHeight - 90
        }).on("dragdeltastop", function (event, ui) {
            //console.log(ui.delta);
            self.adjustJobDetails(ui.delta);
        });

        // TODO - adjust table height if details are seen...
        this.$inventoryTable
            .on("afterscrolltable", $.proxy(this.handleJobDetailScroll, this));
        return this;
    };

    JobTab.prototype.handleTimerTick = function (event, timer, interval) {
        if (this.isActive) {
            this.update();
        }
    };

    JobTab.prototype.update = function (resetTable, resetTimer) {
        var self = this,
            reset = (this.timestamp === 0) || (resetTable === true),
            tableProcess, inventoryProcess;

        tableProcess = this.updateTable(reset);
        inventoryProcess = this.updateInventory(reset, this.selectedID);

        $.when(tableProcess, inventoryProcess).done(function () {
            JobsTimer.rerun();
            if (self.dataTable && elf.$table.is(":visible")) {
                self.$table.scrollTable();
            } else {

            }
        });
    };

    JobTab.prototype.deleteRow = function (jobID) {
        if (this.dataTable) {
            this.dataTable.deleteRow(jobID);
        }
    };

    JobTab.prototype.updateTable = function (reset) {
        var self = this,
            url = this.getUpdateUrl(reset),
            process = $.Deferred();

        var load = (reset) ? PHD.showLoadingIndicator(this.$panel) : null;
        var request = PHD.Ajax.get(url, load);
        $scope.loadingData = true;

        request.done(function (data) {
            $scope.loadingData = false;
            // No longer populating active job count, as we only get this when active jobs API is called.
            if (currentTabIndex === JOB_TAB_HISTORY) {
                $scope.limitTitle = gettext("Recent Jobs Limits");
                $scope.currentTabIndex = 2;
                if (data.limited === true) {
                    $rootScope.jobHistoryLimitHit = " " + gettextCatalog.getString("(limited to") + " " + $rootScope.jobHistoryLimit + " " + gettextCatalog.getString("Jobs)");
                    setTimeout(function () {
                        $scope.$apply();
                    }, 250);
                } else {
                    if ($rootScope.jobHistoryLimitHit !== "") {
                        $rootScope.jobHistoryLimitHit = "";
                        setTimeout(function () {
                            $scope.$apply();
                        }, 250);
                    }
                }
            } else if (currentTabIndex === JOB_TAB_SYSTEM) {
                $scope.limitTitle = gettext("Recent System Jobs Limits");
                $scope.currentTabIndex = 3;
                if (data.limited === true) {
                    $rootScope.systemJobHistoryLimitHit = " " + gettextCatalog.getString("(limited to") + " " + $rootScope.systemJobHistoryLimit + " " + gettextCatalog.getString("Jobs)");
                    setTimeout(function () {
                        $scope.$apply();
                    }, 250);
                } else {
                    if ($rootScope.systemJobHistoryLimitHit !== "") {
                        $rootScope.systemJobHistoryLimitHit = "";
                        setTimeout(function () {
                            $scope.$apply();
                        }, 250);
                    }
                }
            } else if(currentTabIndex === JOB_TAB_MANAGER){
                setTimeout(function () {
                    $scope.$apply();
                }, 100);
            }

            if (data.timestamp === self.timestamp) {
                self.timestamp += 1;
            } else {
                self.timestamp = data.timestamp;
            }

            if (reset) {
                if (self.dataTable) {
                    self.dataTable.load(data.data);
                    self.$table.columnResize();
                } else {
                    setTimeout(function () {
                        $scope.updateGridData(self.index, data.data);
                        $scope.$apply();
                    }, 0);
                }
            } else {
                if (self.dataTable) {
                    self.dataTable.update(data.data, true);
                } else {
                    setTimeout(function () {
                        $scope.updateGridData(self.index, data.data);
                        $scope.$apply();
                    }, 0);
                }
            }

            PHD.hideLoadingIndicator(load);

            self.selectRowByID();
            self.updateButtons();

            process.resolve();
            // old table code, dataTable is null now.
            // if a reset, we have sorted on the server side, so no sort is needed.
            // if an update, check length of data before deciding to sort.
            if (self.dataTable && data.data !== null && data.data !== undefined && data.data.length <= TOO_LARGE_FOR_SORT) {
                if(currentTabIndex === JOB_TAB_ACTIVE){
                    self.$table.sortTable();
                }else if(reset){
                    self.$table.sortTable();
                }
            }
         
        });
        
        return process;
    };

    JobTab.prototype.updateInventory = function (reset, jobID) {
        var process = $.Deferred();

        if (this.inventoryOpen && this.selectedID != null && this.selectedID == jobID) {
            var self = this;
            var data;
            if (this.index === JOB_TAB_HISTORY) {
                data = [];
                if (self.rowData.backups !== undefined) {
                    data = self.rowData.backups;
                    data.forEach(function(obj, index) {
                        //obj.system_name = self.rowData.system_name;
                        obj.server = (self.rowData.app_name == "VMware") ? obj.primary_name : self.rowData.client_name;
                        obj.instance = (self.rowData.app_name == "VMware") ? obj.secondary_name :
                            (self.rowData.app_name.indexOf("Hyper-V") != -1) ? obj.primary_name : "N/A";
                        obj.app_name = (self.rowData.app_name);
                    });
                    self.JobInventory.load(data);
                    self.$inventoryTable.scrollTable({height: self.inventoryTableHeight - 6});
                    self.$inventoryTable.sortTable();
                    self.$inventoryTable.columnResize();
                    self.updateButtons();
                }
                process.resolve();
            } else {
                var load = (reset) ? PHD.showLoadingIndicator(this.$inventory) : null;
                var url = this.getInventoryUrl();
                var request = PHD.Ajax.get(url, load, handleDetailsError);

                request.done(function (data) {
                    var found = false;
                    var jobType;
                    if (data.data) {
                        for (var i = 0; i < data.data.length; i++) {
                            var o = data.data[i];
                            if (o.id == self.selectedID) {
                                // copy relevant items from main row.
                                if (self.rowData != null) {
                                    var row = self.rowData;
                                    o.system_name = (row.system_name != undefined) ? row.system_name : "";
                                    if (o.description === undefined) {
                                        o.description = (row.description != undefined) ? row.description : row.name;
                                    }
                                    o.next_time = (typeof row.next_time != 'undefined') ? row.next_time : "";
                                    o.calendar_str = (typeof row.calendar_str != 'undefined') ? row.calendar_str : "";
                                    o.application = (row.application_name != undefined && row.application_name != "") ? row.application_name :
                                                    (row.application != undefined && row.application != "") ? row.application : null;
                                    jobType = row.type;
                                    switch (row.type) {
                                        case 'replication':
                                            o.client_name = row.target;
                                            if (o.instances !== undefined && o.instances[0].app_name !== undefined) {
                                                // Hot Backup Copy job manager
                                                o.application = o.instances[0].app_name;
                                            } else if (o.app_name !== undefined) {
                                                // Hot Backup Copy active job
                                                o.application = o.app_name;
                                            }
                                            break;
                                        case 'archive':
                                            o.application = o.application_name;
                                            break;
                                        default:
                                            if (o.client_name == undefined || o.client_name == null || o.client_name == "") {
                                                if (o.instances !== undefined && o.instances.length > 0) {
                                                    var first = o.instances[0];
                                                    if (first.client_name !== undefined) {
                                                        o.client_name = first.client_name;
                                                    } else {
                                                        o.client_name = first.primary_name;
                                                    }
                                                }
                                            }
                                            break;
                                    }
                                }
                                data.data = [o];
                                found = true;
                                break;
                            }
                        }
                    }
                    if (!found) {
                        data.data = [];
                    }
                    self.inventoryTimestamp = data.timestamp;
                    if (reset) {
                        self.JobInventory.load(data.data);
                    } else {
                        self.JobInventory.update(data.data);
                    }

                    self.$inventoryTable.scrollTable({height: self.inventoryTableHeight - 6});
                    self.updateButtons();
                    process.resolve();
                });
            }
        } else {
            process.resolve();
        }
        
        /*
         * If the details API fails, clear the inventory area so stale results are removed.
         * If the error is that the schedule that is not editable in Satori, display that in the bottom area rather than a popup.
         */
        function handleDetailsError(jqXHR, textStatus, errorThrown) {
            var data = [];
            var item = {};
            item.id = self.selectedID;
            var json = jqXHR.responseJSON;
            var results = (json !== undefined && json.result !== undefined) ? json.result : [];
            if (results.length > 0) {
                var response = results[0];
                if (response !== undefined && response.message !== undefined && response.message.indexOf('not editable in Satori') !== -1) {
                    item.name = "This job's details are only visible in the Legacy UI";
                } else {
                    PHD.ajaxError(jqXHR, textStatus, errorThrown);
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
            data = [item];
            self.JobInventory.load(data);
            self.$inventoryTable.scrollTable({height: self.inventoryTableHeight - 6});
            self.updateButtons();
        }

        return process;
    };

    JobTab.prototype.queueSelection = function (dbid) {
        var self = this;
        this.$table.one("loadfinished", function () {
            if (self.dataTable && self.dataTable.selectByID(dbid)) {
                self.$buttons.find(".btn-view-details").trigger("click");
            }
        });
    };

    JobTab.prototype.handleRowSelect = function (event, dbid, $row, DataTable, DataRow) {
        this.$selected = $row;
        if (DataRow !== undefined) {
            this.selectedID = dbid;
            this.rowData = DataRow.data;
            if(this.rowData.type == "certification") {
                if (DataRow.sameRow === false) {
                    // Close the Run Menu if a new selected job.
                    $scope.showRunMenu = false;
                }
                if(this.rowData.enabled === 1){
                    $scope.currentDCAJobDisabled = true;
                }
                else if(this.rowData.enabled === 0){
                    $scope.currentDCAJobDisabled = false;
                }
                $scope.certifyJob = true;
                this.$buttons.find(".jobrun-btn-group").flyout();
                this.$buttons.find(".failover-btn-group").flyout();
                $scope.selectedJobName = this.rowData.name;
                $scope.selectedJobId = this.selectedID;
                $scope.currentDCAJobIsFailover = false;
                $scope.showCommitAndDiscardButtons = false;
                if(this.rowData.status == 'InstantLab' || this.rowData.status == 'Failover') {
                    $scope.showCommitAndDiscardButtons = true;                          //Job with "InstantLab" or "Failover" status can only be committed or discarded
                    $scope.currentDCAJobIsFailover = true;
                }
            }
            else {
                $scope.certifyJob = false;
                $scope.showRunMenu = false;
            }

        } else {
            $scope.certifyJob = false;
            this.selectedID = null;
            this.rowData = null;
        }
        $rootScope.isSelectedDCAJob = false;
        $rootScope.isSelectedJobCopy = this.isSelectedJobCopy(dbid);
        if(!$rootScope.isSelectedJobCopy){
            $rootScope.isSelectedDCAJob = this.isSelectedJobDCA(dbid)
        }
        if (this.namespace === "jobspolicies") {
            $rootScope.selectedPolicyIdToEdit = dbid;
        } else {
            $rootScope.selectedJobIdToEdit = dbid;
        }

        $scope.selectedRowJobType = this.rowData.type;
        if (this.rowData != null && DataRow.data.application_name === "file-level"){
            if (this.rowData.is_nas !== undefined) {
                $rootScope.isSelectedJobNAS = this.rowData.is_nas;
                $rootScope.isSelectedJobFileLevel = !$rootScope.isSelectedJobNAS;
            } else {
                $rootScope.isSelectedJobFileLevel = true;
                $rootScope.isSelectedJobNAS = false;
            }
        }else{
            $rootScope.isSelectedJobFileLevel = false;
            $rootScope.isSelectedJobNAS = false;
        }
        //console.log("in handleRowSelect, id = " + dbid + " and rowData = " + JSON.stringify(this.rowData));
        this.updateButtons();
        if (this.inventoryOpen) {
            this.showJobDetails();
        }
        setTimeout(function () {
            $scope.$apply();
        }, 0);
    };
    
    JobTab.prototype.isSelectedJobCopy = function(jobID) {
        var jobType = getJobTypeChar(jobID);
        return jobType === 'a' || jobType === 'x';        
    };
    
    JobTab.prototype.isSelectedJobHotCopy = function(jobID) {
        var jobType = getJobTypeChar(jobID);
        return jobType === 'x';
    };

    JobTab.prototype.isSelectedJobDCA = function(jobID) {
        var jobType = getJobTypeChar(jobID);
        return jobType === 'c';
    };

    JobTab.prototype.handleRowDelete = function (event, dbid, DataTable) {
        if (this.selectedID === dbid) {
            this.$selected = $();
            this.selectedID = null;
            this.$inventory.find("table tbody tr").remove();
            this.hideJobDetails();
        }
    };

    JobTab.prototype.handleRowAddition = function (event, newRows, DataTable) {
        if (newRows) {
            if (DataTable.count() <= TOO_LARGE_FOR_SORT) {
                this.$table.sortTable();
            }
        }
    };

    JobTab.prototype.handleJobDetailScroll = function (event) {
        if (this.index === JOB_TAB_MANAGER) {
            if (this.rowData !== null && this.rowData.type === "certification") {
                $('#id_table_jobDetails td:nth-child(1), #id_table_jobDetails th:nth-child(1)').hide();
                $('#id_table_jobDetails td:nth-child(2), #id_table_jobDetails th:nth-child(2)').hide();
            } else {
                $('#id_table_jobDetails td:nth-child(3), #id_table_jobDetails th:nth-child(3)').hide();
                $('#id_table_jobDetails td:nth-child(4), #id_table_jobDetails th:nth-child(4)').hide();
            }
            $('#id_table_jobDetails thead tr th').each(function() {
                $(this).css('width', '410px');
                $(this).css('max-width', '410px');
                $(this).css('min-width', '200px');
                $(this).css('text-align', 'left');
            });
            $('#id_table_jobDetails tbody tr td').each(function() {
                $(this).css('width', '410px');
                $(this).css('max-width', '410px');
                $(this).css('min-width', '200px');
                $(this).css('text-align', 'left');
            });
        }
    };

    JobTab.prototype.handleCheckboxChange = function (event, $checkbox, checked) {
        var self = this;
        var dcaJobSelected = false;
        if (event.currentTarget.checked == false) {
            // If unchecking the box, ensure that this item is de-selected.
            this.deSelectRow();
        }
        this.selectedIDs = this.$table.find("tbody :checked").map(function () {
            var $row = $(this).closest("tr");
            // TODO this will need work for ui-grid.
            var enabled = $row.is(".job-enabled-true");
            var jobID = $(this).val();
            if (self.isSelectedJobHotCopy(jobID)) {
                // Hot Backup Copy Jobs cannot be selected for Edit.
                $(this).prop("checked", false);
            }
            else {
                if(self.isSelectedJobDCA(jobID)){
                    dcaJobSelected = true;
                }
                return {
                    "id": jobID,
                    "enabled": enabled
                };               
            }
        }).get();
        // If only one item is selected, select it in the DataTable as well.
        // This ensures that for 1 item, this.selectedID is the same as the selectedIDs element.
        if (this.selectedIDs.length === 1) {
            this.selectRowByID(this.selectedIDs[0].id);
        }
        else if( this.selectedIDs.length > 1){
            $scope.showCommitAndDiscardButtons = false;
            $scope.certifyJob = false;
            setTimeout(function () {
                $scope.$apply();
            }, 100);
        }
        this.updateButtons();
    };

    JobTab.prototype.selectRowByID = function (id) {
        id = id || this.selectedID;
        if (!id) {
            return false;
        }
        if (this.dataTable) {
            this.dataTable.selectByID(id);
        } else {
            $scope.selectGridRowsByIDs(this.selectedIDs, this.index);
        }
    };

    JobTab.prototype.deSelectRow = function () {
        this.$selected = $();
        this.selectedID = null;
        this.rowData = null;
        if (this.dataTable) {
            this.dataTable.deselect();
        }
    };

    JobTab.prototype.getUpdateUrl = function (reset) {
        var $column;
        var direction;
        if (this.namespace == "jobshistory"){
                $column = this.$table.find("th[data-source=start_time]");
                direction = ($column !== undefined && $column.data('sort-dir') !== undefined && $column.data('sort-dir') !== null) ? $column.data('sort-dir') : 'desc';
            this.url = "/api/jobs/history/?showFlat=true&sort=" + direction + "&limit=" + $rootScope.jobHistoryLimit + "&lang=" + $rootScope.userLanguageCode;
        } else if (this.namespace == "jobssystem"){
                $column = this.$table.find("th[data-source=event_time]");
                direction = ($column !== undefined && $column.data('sort-dir') !== undefined && $column.data('sort-dir') !== null) ? $column.data('sort-dir') : 'desc';
            this.url = "/api/jobs/system/?sort=" + direction + "&limit=" + $rootScope.systemJobHistoryLimit + "&lang=" + $rootScope.userLanguageCode;
        } else if (this.namespace == "jobspolicies") {
            this.url = "/api/sla-policies";
        } else if (this.namespace == "jobsmanage"){
            this.url = "/api/joborders/" + "?lang=" + $rootScope.userLanguageCode;
        } else if (this.namespace == "jobsactive"){
            this.url = "/api/jobs/active/" + "?lang=" + $rootScope.userLanguageCode;
        }
        return this.url;
    };

    JobTab.prototype.getInventoryUrl = function () {
        var url = "/api/jobs";
        switch (this.index) {
            case JOB_TAB_ACTIVE:
                url += "/active";
                break;
            case JOB_TAB_HISTORY:
                url += "/history/";
                break;
            case JOB_TAB_SYSTEM:
                url += "/system/";
                break;
            case JOB_TAB_MANAGER:
                url = "/api/joborders/" + this.selectedID;
                if (this.rowData !== null && this.rowData.sid !== undefined) {
                    url += "/?sid=" + this.rowData.sid;
                }
                break;
            case JOB_TAB_CALENDAR:
                //url = "/api/joborders/" + this.selectedID;
                // TODO: I think this can be a no-op as we won't have inventory drill down/details with the calendar that requires a separate API call.
                break;
            case JOB_TAB_POLICIES:
                url = "/api/sla-policies/" + this.selectedID;
                if (this.rowData !== null && this.rowData.sid !== undefined) {
                    url += "/?sid=" + this.rowData.sid;
                }
                break;
        }
        return url;
    };

    JobTab.prototype.initButtons = function () {
        var tab = this;
        this.$buttons.find("a").each(function (index) {
            var $but = $(this);
            PHD.buttonLogic($but, null, tab, null, null, {provider: $analytics, source: 'Jobs'}, null);
        });
    };

    JobTab.prototype.updateButtons = function ($button) {
        var $row = this.$selected,
            id = this.selectedID,
            rowData = this.rowData,
            ids = this.selectedIDs;
        tab = this;

        if ($button !== void 0) {
            PHD.buttonLogic($button, $row, tab, id, rowData, {provider: $analytics, source: 'Jobs'}, ids);
        } else {
            this.$buttons.find("a").each(function (index) {
                var $but = $(this);
                PHD.buttonLogic($but, $row, tab, id, rowData, {provider: $analytics, source: 'Jobs'}, ids);
            });
        }
    };

    JobTab.prototype.adjustJobDetails = function (delta) {
        this.inventoryTableHeight -= delta;
        if (!this.$inventory.is(":hidden")) {
            this.$inventoryTable.adjustTableHeight(this.inventoryTableHeight - 6);
            this.$table.offsetTableHeight(delta);
            $scope.gridApi.grid.gridHeight += delta;
            $scope.setGridElementHeight($('.ui-grid-full-height'), $scope.gridApi.grid.gridHeight);
            $scope.gridApi.grid.queueRefresh();
        }
    };

    JobTab.prototype.showJobDetails = function () {
        this.inventoryTimestamp = 0;
        if (this.$inventory.is(":hidden")) {
            if ($scope.gridApi.grid.gridHeight > 0) {
                $scope.gridApi.grid.gridHeight = $scope.setGridHeight(this.index, true);
                $scope.gridApi.grid.queueRefresh();
            }
            this.$table
                .adjustTableHeight(this.$table.getAvailableVerticalSpace(".full-height") - this.inventoryTableHeight);
            this.$inventory.show("slide", {direction: "down"});
            this.inventoryOpen = true;
            this.updateInventory(true, this.selectedID);
            console.log("in showJobDetails " + this.selectedID);
            this.$dragHandle.dragdelta("option", "disabled", false);
        } else {
            this.updateInventory(true, this.selectedID);
        }
        this.updateButtons(this.$panel.find(".btn-view-details"));
    };

    JobTab.prototype.hideJobDetails = function () {
        var self = this;
        this.$dragHandle.dragdelta("option", "disabled", true);
        this.$inventory.hide("slide", {direction: "down"}, function () {
            if (self.fullHeight !== undefined) {
                $scope.gridApi.grid.gridHeight = $scope.setGridHeight(self.index, false);
                $scope.gridApi.grid.queueRefresh();
            }
            self.$table
                .adjustTableHeight(self.$table.getAvailableVerticalSpace(".full-height"));

            self.inventoryOpen = false;
            self.updateButtons(self.$panel.find(".btn-view-details"));
        });
    };

    JobTab.prototype.setTimerInterval = function (interval) {
        JobsTimer.setInterval(refreshRate);
        if (this.$panel.is(":visible")) {
            JobsTimer.reset();
        }
    };

    JobTab.prototype.activate = function () {
        this.isActive = true;
        this.$dragHandle.dragdelta("option", "max", window.innerHeight - 90);
        if (this.namespace === 'jobscalendar') {
            var jobsCalendarScope = $("[ng-controller=JobCalendarCtrl]").scope();
            if (jobsCalendarScope !== undefined) {
                jobsCalendarScope.initJobsCalendarTab();
            }
        } else {
            var tabIndex = this.index;
            if ($scope.gridList[tabIndex] !== 0) {
                $scope.gridApi = $scope.gridList[tabIndex];
                $scope.columnOrderRestored = false;
            }
            //console.log("Activate tab " + tabIndex);
            this.update(false, true);
        }
        $scope.placeHolder = this.placeHolder;
    };

    JobTab.prototype.deactivate = function () {
        this.isActive = false;
        JobsTimer.stop();
        // If a modal dialog is shown, close it.
        $scope.closeModal();
    };

    JobTab.prototype.clearSelectedRows = function() {
        $scope.gridApi.selection.clearSelectedRows();
    };

    var lookupIcon = [];
    lookupIcon['on hold'] = "unknown";
    lookupIcon["queued"] = "";
    lookupIcon["connecting"] = "active";
    lookupIcon["active"] = "active";
    lookupIcon["running"] = "active";
    lookupIcon["db update"] = "active";
    lookupIcon["alreadyrunning"] = "warning";
    lookupIcon["warning"] = "";
    lookupIcon["success"] = "";
    lookupIcon["successful"] = "success";
    lookupIcon['error'] = "";
    lookupIcon["complete"] = "";
    lookupIcon["cancelled"] = "error";
    lookupIcon["failed"] = "error";
    lookupIcon["failure"] = "error";
    lookupIcon["process terminated"] = "error";
    lookupIcon["unknown"] = "unknown";
    lookupIcon["incomplete/cancelled"] = "canceled";
    lookupIcon["available"] = "";
    lookupIcon["idle"] = ""; // clear this from idle, as flag doesn't change properly when enabled/disabled.
    lookupIcon["failover"] = "failover";
    
    function getStatusIcon(row) {
        var iconStart =  '<i class="icon-phd-';
        var iconEnd = ' icon"></i>';
        var icon = iconStart;
        var status;
        if (row.status !== null && row.status !== undefined) {
            var raw_status = row.status.toLowerCase();
            status = lookupIcon[raw_status.toLowerCase()];
            if (status == 'active') {
                // This icon does not start with icon-phd
                iconStart = '<i class="icon-hourglass';
            } else if (status == '') {
                // If '', use the lowercase status.
                iconStart += raw_status;
            } else if (status == null && raw_status == 'instantlab') {
                iconStart = '<i class="icon-hv-vm';
            } else {
                // use the mapped/return status
                iconStart += status;
            }
        } else {
            iconStart += 'unknown';
        }
        return iconStart + iconEnd;
    }


    function handlePlugin() {
        var params = PHD.util.getParams();
        if (params) {
            if (params.hasOwnProperty("action")) {
                $(document).trigger("launchdialog", [params]);
            }

            if (params.hasOwnProperty("select")) {
                // lmc - add this to kill the other timers.
                var e = $.Event('uinav.protect');
                $(document).trigger(e);
                tabs[0].queueSelection(params.select);
            }
            // lmc - don't do this
            //window.history.replaceState({}, "", window.location.origin + window.location.pathname);
        }
    }

    function handleTabChange() {
        $(document).trigger("tabchange", ["jobs" + slugs[currentTabIndex]]);
        tabs[currentTabIndex].deSelectRow();
        $scope.showCommitAndDiscardButtons = false;
        $scope.certifyJob = false;
        tabs[currentTabIndex].updateButtons();
        tabs[currentTabIndex].hideJobDetails();
    }

    $scope.$on("languageChanged", function (event){
        // need to refresh any tab that's been loaded already (set all timers to 0)
        tabs[currentTabIndex].update(true);
        tabs.forEach(function (tab) {
            tab.timestamp = 0;
        });

    });

    /*
     *   DOM Ready Function
     */
    $(function () {
        // Quick and dirty way to change what tab opens by default via URL
        if (!isNaN(PHD.util.getHashInt())) {
            initialTab = PHD.util.getHashInt();
        }

        UserPreference = PHD.UserPreference($("#jobs-page-preferences .preference-trigger"), {
            name: gettext("Jobs Page"),
            namespace: "jobspage",
            key: "page_jobs_refresh",
            defaultValue: refreshRate
        });

        // Initialize the tabs
        $('.tabs-wrapper').tabs({
            active: initialTab,
            create: function (event, ui) {
                currentTabIndex = ui.tab.index();
                $scope.currentTabIndex = currentTabIndex;
                initTabs();
                handleTabChange();
            },
            activate: function (event, ui) {
                $(".loading").hide();
                currentTabIndex = ui.newTab.index();
                $.cookie("initial_tab_jobs", currentTabIndex, {expires: 365, path: '/'});
                tabs[ui.oldTab.index()].deactivate();
                tabs[currentTabIndex].activate();
                handleTabChange();
            }
        });

        // Handle actions from plugin
        handlePlugin();
    });

}
