
angular.module('updates',['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])

    .controller('UpdateCtrl', ['$scope', '$rootScope', '$http', 'AssetService', 'ngDialog', function($scope, $rootScope, $http, AssetService, ngDialog) {

        $scope.updateMessage = "";

        // Used to unbind the watched event on dialog close.
        var unbindHandler = null;

        $scope.initCtrl = function (dlg) {

            if (dlg.name !== 'updates-main-dialog') {
                // Only run initialization for the main dialog.
                return;
            }

            $scope.updates = $scope.ngDialogData;

            var $form = $("#form_updates"),
                $updateTable = $("#system-update-table"),
                updatesForm,
                UpdateTable;

                $form.find(".definition").popover({
                    appendTo: $form,
                    position: {
                        my: "left center",
                        at: "right+10 center",
                        collision: "flip fit",
                        within: $form
                    }
                });

                var cellFormatter = {
                    preCheck: function(value, row) {
                        var displayStatus = row.preCheck;
                        var iconClass = "";
                        var displayTitle = "";
                        var toolTip;
                        var html;
                        var iconColor = "green";
                        if (displayStatus.substring(0,2) === 'No') {
                            iconClass = "exclamationsign";
                            displayTitle = "";
                            iconColor = "red";
                            toolTip = displayStatus.substring(3);
                            toolTip = toolTip || displayTitle;
                            html =  '<i class="icon-'+iconClass + ' icon-large help-trigger" style="color:' + iconColor + ';margin-right:2px;"></i>' +
                                '<span  class = "definition" >' +toolTip + '</span>';
                        }
                        else if(displayStatus.substring(0,3) === "Yes" || displayStatus.substring(0,7) === "Message"){
                            if(row.count > 0){
                                html = " <input class=\"ngdialog-button\" type=\"submit\" value=\"&nbsp; Apply Update &nbsp;\"/>" +
                                   "<a id=\"viewDetails\" href='https://www.unitrends.com/wp-content/uploads/rs-ub-release-notes.pdf' target='_blank'> &nbsp; View Details </a>"
                            }
                            else{
                                iconClass = "okcircle";
                                toolTip = "This appliance is up to date";
                                html =  '<i class="icon-'+iconClass + ' icon-large help-trigger" style="color:' + iconColor + ';margin-right:2px;"></i>' +
                                '<span  class = "definition" >' +toolTip + '</span>';
                            }
                        }

                        return html;
                    }
                };

                UpdateTable = PHD.DataTable($updateTable, {
                    dynamic: false,
                    tableScrollOptions: {"height": "article"},
                    select: true,
                    sortable: true,
                    sortOnLoad: true,
                    sortOnHeader: true,
                    integerIDs: false,
                    cellClasses: [null, null, "full-text"],
                    classifiers: ['id']

                }).registerFormatters(cellFormatter);

                updatesForm = PHD.FormController($form, {});

                loadUpdates();

                function loadUpdates() {
                    var data = $scope.updates.data;
                    var updateSystems = [];
                    data.forEach(function(item,index) {
                        var obj = {};
                        var i, ver = "";
                        obj.count = item.count;
                        if (item.count > 0) {
                            obj.updates = gettext("Update Available") + "(" + item.size + ")";

                            for (var j = 0; j < item.count; j++) {
                                if (item.updates[j].name === "unitrends-rr") {
                                    obj.version = item.updates[j].version;
                                    break;
                                }
                            }
                        }
                        obj.sname = item.sname;
                        obj.id = item.sid;
                        obj.preCheck = item.preCheck;
                        obj.show_refresh_message = item.show_refresh_message;
                        if (obj.preCheck !== undefined && obj.preCheck.substring(0,7) === "Message") {
                            // Special message to display on update.
                            obj.updateMessage = obj.preCheck.substring(8) + '\n\nTo continue with the update, press Confirm.  To quit, press Cancel.';
                        } else {
                            obj.updateMessage = "";
                        }
                        updateSystems.push(obj);
                    });
                    UpdateTable.load(updateSystems);
                }

                $("#system-update-table").on("rowselect", "tr", function (event, dbid, $el, DataTable, DataRow) {
                    var UpdateSystem = DataRow.data;
                    $scope.id = UpdateSystem.id;
                    $scope.showRefreshMessage = (UpdateSystem.hasOwnProperty('show_refresh_message') && UpdateSystem.show_refresh_message === true);
                    $scope.updateMessage = UpdateSystem.updateMessage;
                });


                $("#system-update-table").on("click", "input", function(event,dbid, $el, DataTable, DataRow) {

                    if ($scope.updateMessage !== "") {
                        $scope.viewUpdateMessage();
                    } else {
                        $scope.runUpdates(false);
                    }
                    return this;
                });

                $form.find(".definition").popover({
                    appendTo: $form
                });

            };

            $scope.runUpdates = function(closeCheckDialog) {
                if (closeCheckDialog === true) {
                    ngDialog.close('update-or-cancel');
                }
                if ( $scope.showRefreshMessage === true ) {
                    $scope.load = PHD.showLoadingIndicator("body", true, "Appliance is updating, do not reboot. Please close and reopen the browser once the update is complete...");
                } else {
                    $scope.load = PHD.showLoadingIndicator("body", true, "Appliance is updating, do not reboot...");
                }

                var url_sid = ( $scope.id === null) ? "" : "/?sid=" +  $scope.id;
                var url = "/api/updates";
                url += url_sid;
                PHD.Ajax
                    .post(url, null, $scope.load, handleError)
                    .done(function (data) {
                    })
                    .always(function () {
                        PHD.hideLoadingIndicator($scope.load);
                    });
            };

            function handleError(jqXHR, textStatus, errorThrown) {
                PHD.hideLoadingIndicator($scope.load);
                if (jqXHR.status === 500) {
                    var data = jqXHR.responseJSON;
                    if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                        var message = data.result[0].message;
                        // remove pre tag from output.
                        message = message.replace("<pre>", "");
                        message = message.replace("</pre>", "");
                        $scope.outputTitle = "Update Results";
                        $scope.outputData = message;
                        $scope.onConfirmOkButtonClick = $scope.onConfirmOutput;
                        $scope.updatesOutput = true;
                        var dimensions = 'width:600px;height:350px;';
                        ngDialog.open({
                            template: 'app/common/command-output.html',
                            modelDialogId:'update-option-noticeDialog',
                            dialogMessage:message,
                            scope:$scope,
                            ngDialogStyle: dimensions,
                            onConfirmOkButtonClick: 'onConfirmOutput()'
                        });
                    } else {
                        ngDialog.open({
							dialogType:'Error',
							modelDialogId:'update-option-errorDialog',
							scope:$scope,
							dialogMessage: message,
							onConfirmOkButtonClick:'onConfirmError()'
                       });
                    }
                } else {
                    PHD.ajaxError(jqXHR, textStatus, errorThrown);
                }
            }

            $scope.viewUpdateMessage = function() {
                ngDialog.open({
                    dialogType:'CONFIRMATION',
                    ngDialogStyle:'width:500px',
                    modelDialogId:'update-or-cancel',
                    scope:$scope,
                    dialogMessage: $scope.updateMessage,
                    onConfirmOkButtonClick:'runUpdates(true)'
                });
            };

			$scope.onConfirmError = function () {
                 ngDialog.close('update-option-errorDialog');
            };

            $scope.onConfirmOutput = function () {
                ngDialog.close('update-option-noticeDialog');
                $scope.closeThisDialog();
            };

            $scope.closeThisDialog = function() {
                ngDialog.closeAll();
                if ( _.isFunction(unbindHandler)) {
                    unbindHandler();
                }
            };

            // Initialize the controller.
            unbindHandler = $scope.$on('ngDialog.opened', function (event, obj) {
                $scope.initCtrl(obj)
            });

    }]);


