angular.module('language-selection', ['ngDialog','tour-module'])
    .controller('LanguageCtrl', ['$scope', '$rootScope', '$http', 'Page', 'ngDialog', '$timeout', 'tourFactory', 'gettextCatalog', 'i18nService',
        function($scope, $rootScope, $http, Page, ngDialog, $timeout, tourFactory, gettextCatalog, i18nService) {

            /*
             * When the controller initializes, set the initial language set in the $rootScope.
             */
            $scope.initializeDefaultLanguage = function() {
                $scope.setUserLanguage($rootScope.userLanguageCode, true);
            };

            /*
             * Available application themes, the names are used for css styles.
             */
            $scope.availableLanguages = [
                { code: 'en', language: gettext('US English') },
                { code: 'fr', language: gettext('French') },
                { code: 'en-gb', language: gettext('Europe English')}
            ];

            $scope.selectLanguage = function() {
                ngDialog.open({
                    modelDialogId:'app-language',
                    name: 'app-language',
                    template: 'app/preferences/language.html',
                    scope:$scope,
                    overlay:true,
                    ngDialogStyle:"width:400px"
                });
            };

            $scope.setCurrentLanguage = function() {
                for (var i = 0; i < $scope.availableLanguages.length; i++) {
                    var language = $scope.availableLanguages[i];
                    language.selected = false;
                    if (language.code === $rootScope.userLanguageCode) {
                        language.selected = true;
                        $scope.selectedLanguage = language;
                    }
                }
            };

            $scope.pickLanguage = function(language) {
                $scope.selectedLanguage = language;
            };

            /*
             * Update the language in the UI (gettext) and in the preferences.
             */
            $scope.saveLanguage = function() {
                // Only update the DOM if the theme changed.
                if ($scope.selectedLanguage !== undefined && ($scope.selectedLanguage.code !== $rootScope.userLanguageCode)) {
                    // Update the UI.
                    $scope.setUserLanguage($scope.selectedLanguage.code);
                    // Persist preferences.
                    $scope.updateLanguagePreference($scope.selectedLanguage.code);
                }

                $scope.closeThisDialog();
            };

            /*
             * Calls the angular-gettext functions to set the language and load the strings JSON file.
             */
            $scope.setUserLanguage = function(code, force) {
                var forceChange = force || false;
                $scope.$evalAsync(function() {
                    if (forceChange || (code !== $rootScope.userLanguageCode)) {
                        $rootScope.userLanguageCode = code;
                        // set date format for datepicker
                        switch ($rootScope.userLanguageCode){
                            case "fr":
                                // If French, load json file
                                gettextCatalog.loadRemote("locale/" + code + "/LC_MESSAGES/" + code + ".json");
                                DATE_FORMAT = "dd/mm/yy";
                                DATE_DISPLAY_FORMAT = "DD/MM/YYYY";
                                $.datepicker.setDefaults($.datepicker.regional[code]);
                                $.extend($.fn.spectrum.defaults, $.spectrum.localization[code]);
                                i18nService.setCurrentLang(code);
                                break;
                            case "en-gb":
                                DATE_FORMAT = "dd/mm/yy";
                                DATE_DISPLAY_FORMAT = "DD/MM/YYYY";
                                $.datepicker.setDefaults($.datepicker.regional[code]);
                                $.extend($.fn.spectrum.defaults, $.spectrum.localization['en']);
                                i18nService.setCurrentLang('en');
                                break;
                            case "en":
                            case "en-us":
                            default:
                                DATE_FORMAT = "mm/dd/yy";
                                DATE_DISPLAY_FORMAT = "MM/DD/YYYY";
                                $.datepicker.setDefaults($.datepicker.regional['']);
                                $.extend($.fn.spectrum.defaults, $.spectrum.localization['en']);
                                i18nService.setCurrentLang('en');
                                break;
                        }
                        $rootScope.DATE_DISPLAY_FORMAT = DATE_DISPLAY_FORMAT;
                        $rootScope.DATE_FORMAT = DATE_FORMAT;
                        gettextCatalog.setCurrentLanguage(code);
                        moment.locale(code);
                        $rootScope.$broadcast('languageChanged');
                    }
                });
            };

            $scope.$on("update-language", function(event, code) {
                $scope.setUserLanguage(code);
            });

            $scope.updateLanguagePreference = function(code) {
                var prefURL = '/api/preferences/' ;
                var putData = {};
                putData['language'] = code;
                prefURL = prefURL + encodeURI($rootScope.user.name)+"/";
                tourFactory.sync('put', prefURL, $rootScope.local.id, putData).then(function(result){
                    console.log("success >> update language to " + code);
                });
            };

            $scope.closeThisDialog = function() {
                ngDialog.close();
            };

            $scope.initCtrl = function(dlg) {
                switch (dlg.name) {
                    case 'app-language':
                        $scope.setCurrentLanguage();
                        break;
                }
            };

            $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

        }]);