angular.module('user-preferences', ['ngDialog','tour-module'])
    .controller('themesCtrl', ['$scope', '$rootScope', '$http', 'Page', 'ngDialog', '$timeout', 'tourFactory', 'gettextCatalog',
    function($scope, $rootScope, $http, Page, ngDialog, $timeout, tourFactory, gettextCatalog) {


        /*
         * Available application themes, the names are used for css styles.
         */
        $scope.availableThemes = [];

        $scope.selectTheme = function() {
            ngDialog.open({
                modelDialogId:'app-theme',
                name: 'app-theme',
                template: 'app/preferences/user-theme.html',
                scope:$scope,
                overlay:true,
                ngDialogStyle:"width:425px"
            });
        };

        $scope.setCurrentTheme = function() {
            for (var i = 0; i < $scope.availableThemes.length; i++) {
                var theme = $scope.availableThemes[i];
                theme.selected = false;
                if (theme.name === $rootScope.userApplicationTheme) {
                    theme.selected = true;
                    $scope.selectedTheme = theme;
                }
            }
        };

        $scope.pickTheme = function(theme) {
            $scope.selectedTheme = theme;
        };

        /*
         * Update the theme by explicitly changing the theme class in the DOM.  We also change it using
         * ng-class in the HTML, but since that is in the dashboard template cache, it will not updated
         * until the next time we enter the dashboard from another subsystem.
         */
        $scope.saveTheme = function() {
            // Only update the DOM if the theme changed.
            if ($scope.selectedTheme !== undefined && ($scope.selectedTheme.name != $rootScope.userApplicationTheme)) {
                // Remove current theme class.
                $(document).find(".panel.highlighted-panel").removeClass($rootScope.userApplicationTheme);
                $(document).find(".widgetcube:nth-child(1)").removeClass($rootScope.userApplicationTheme);
                $(document).find(".widgetcube:nth-child(2)").removeClass($rootScope.userApplicationTheme);
                $(document).find(".widgetcube:nth-child(3)").removeClass($rootScope.userApplicationTheme);
                $(document).find(".narrow-tile-hexant").removeClass($rootScope.userApplicationTheme);
                $(document).find(".narrow-tile-triad").removeClass($rootScope.userApplicationTheme);
                $(document).find(".full-width-tile-hexant").removeClass($rootScope.userApplicationTheme);
                $(document).find(".feed-reader-box").removeClass($rootScope.userApplicationTheme);
                // Update theme and add classs.
                $rootScope.userApplicationTheme = $scope.selectedTheme.name;
                $(document).find(".panel.highlighted-panel").addClass($rootScope.userApplicationTheme);
                $(document).find(".widgetcube:nth-child(1)").addClass($rootScope.userApplicationTheme);
                $(document).find(".widgetcube:nth-child(2)").addClass($rootScope.userApplicationTheme);
                $(document).find(".widgetcube:nth-child(3)").addClass($rootScope.userApplicationTheme);
                $(document).find(".narrow-tile-hexant").addClass($rootScope.userApplicationTheme);
                $(document).find(".narrow-tile-triad").addClass($rootScope.userApplicationTheme);
                $(document).find(".full-width-tile-hexant").addClass($rootScope.userApplicationTheme);
                $(document).find(".feed-reader-box").addClass($rootScope.userApplicationTheme);

                // Persist preferences.
                $scope.updateThemePreference($rootScope.userApplicationTheme);
            }

            $scope.closeThisDialog();
        };

        $scope.updateThemePreference = function(theme) {
            var prefURL = '/api/preferences/' ;
            var putData = {};
            putData['application_theme'] = theme;
            prefURL = prefURL + encodeURI($rootScope.user.name)+"/";
            tourFactory.sync('put', prefURL, $rootScope.local.id, putData).then(function(result){

            });
        };

        $scope.closeThisDialog = function() {
            ngDialog.close();
        };

        $scope.initCtrl = function(dlg) {
            $scope.availableThemes = [
                  {name : 'theme-gray', display: gettextCatalog.getString('Gray shades, no dashboard highlights or colors')},
                  {name : 'theme-blue', display: gettextCatalog.getString('Blue slate dashboard highlights, colors')}
            ];
            switch (dlg.name) {
                case 'app-theme':
                    $scope.setCurrentTheme();
                    break;
            }
        };

        $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

    }]);