(function($, PHD, window, undefined) {
    
    /*
    *   Handles CRUD and events for widget- or page-based User Preferences
    */

    var defaults = {
        name: "PHD Virtual Backup",
        namespace: "phduserpreference",
        key: 0,
        defaultValue: 0,
        modal: false
    };

    var dialogOptions = {
        autoOpen: true,
        modal: false,
        show: { effect: "slide", direction: "up" },
        hide: { effect: "scale", percent: 0, origin: ["top", "right"] },
        dialogClass: "user-preference-dialog",
        resizable: false,
        draggable: false,
        width: 262,
        buttons: {}
    };

    var UserPreference = function UserPreference($trigger, options) {
        if(this instanceof UserPreference) {
            options = options || {};
            this.options = $.extend({}, defaults, options);

            this.$trigger = $trigger;
        } else {
            return new UserPreference($trigger, options).init();
        }
    };

    UserPreference.prototype = {
        init: function() {
            var self = this;

            this.editing = false;

            this.$dialog = $("<div>", {
                "id": "dialog-" + this.options.namespace,
                "class": "dialog-box user-preference"
            });

            this._dialogOptions = $.extend({}, dialogOptions);
            this._dialogOptions.modal = this.options.modal;
            this._dialogOptions.position ={
                my: "right top",
                at: "right top",
                of: this.$trigger.parent()
            };
            var elem = $("[ng-controller=GlobalCtrl]");
            this.scope = elem.scope();
            var injector = elem.injector();
            this.gettextCatalog = injector.get('gettextCatalog');

            this.username = this.scope.user.name;
            this._dialogOptions.title = this.gettextCatalog.getString(this.options.name) + " " + this.gettextCatalog.getString("Preferences");

            this.$trigger.on("click", function(event) {
                if (!self.editing) {
                    self.editing = true;
                    event.preventDefault();
                    event.stopPropagation();
                    self.edit();
                }
            });

            this.$dialog
                .on("click", ".btn-pref-save", $.proxy(this._save, this))
                .on("click", ".btn-pref-cancel", $.proxy(this._cancel, this));


            return this;
        },

        edit: function() {
            var self = this;

            PHD.Ajax.get("/api/preferences/" + encodeURI(self.username) + "/" + this.options.key + "/")
                .done(function(data) {
                    var preferences = data.data;
                    rate = preferences[0][self.options.key];
                    PHD.Ajax.getHTML("app/preferences/userpreference.html")
                        .done(function(html) {
                            angular.element(document).injector().invoke(function ($compile) {
                                html = $compile(html)(self.scope);
                                self.$dialog.html(html);
                                self.$dialog.find("#user-preference-name").text(self.gettextCatalog.getString(self.options.name));
                                self.$dialog.find("#refresh-name").text(self.options.refresh_name);
                                self.$dialog.find("#id_refresh").val(rate);
                                self.$dialog.find("#user-preference-rate").text(rate);
                                self.$dialog.insertAfter(self.$trigger).dialog(self._dialogOptions);
                            });
                        });
                });
        },

        _save: function() {
            var self = this,
                $form = this.$dialog.find("form"),
                key = this.options.key,
                values = $form.serializeArray(),
                value = _.findWhere(values, {"name": "refresh_rates"});

            var putData = {};
            putData[key] = value.value;

            PHD.Ajax.put('/api/preferences/' + encodeURI(self.username) + '/', JSON.stringify(putData))
                .done(function(data) {
		            var rate = parseInt(value.value) * 1000;
                    console.log("trigger namespace " + self.options.namespace + " and rate = ", rate);
                    $(document).trigger("userpreferencesave." + self.options.namespace, [key, rate]);
                    self._close();
                })
                .fail(function(data) {
                    $(document).trigger("userpreferencefail." + self.options.namespace, [key, value, data]);
                    self._close();
                });
        },

        _cancel: function() {
            this._close();
        },

        _close: function() {
            this.editing = false;
            this.$dialog.dialog("destroy").html("").detach();
        },

        cancel: function() {
            if (this.editing === true) {
                this._cancel();
            }
        }

    };

    PHD.UserPreference = UserPreference;

    var isInt = function(n) {
        return !isNaN(parseInt(n, 10));
    };

})(jQuery, window.PHD || (window.PHD = {}), window);
