angular.module('backupJobModule', ['ngDialog', 'angulartics', 'angulartics.google.analytics',
    'ui.grid','ui.grid.resizeColumns','ui.grid.autoResize','fileBrowserDialogApp','ngGrid','ui.bootstrap','inventory','checklist-model','ui.grid.selection', 'backup-now-dialog'])
.directive('angulardatePicker', function ($parse) {
    return function (scope, element, attrs, controller) {
        var ngModel = $parse(attrs.ngModel);
        var minDate = (attrs.minDate === 'none') ? null : 0;
        $(function(){
            element.datepicker({
                showOn: "both",
                buttonText: '<img src="assets/images/calendar.png" width="24" height="24">',
                dateFormat: DATE_FORMAT,
                minDate: minDate,
                onSelect:function (dateText, inst) {
                    scope.$apply(function(scope){
                        // Change binded variable
                        ngModel.assign(scope, dateText);
                    });
                }
            });
        });
    }
}).directive('angulartimeEntry', function ($rootScope, $parse) {
     return {
         restrict: 'A',
         scope: {ngModel: "="},
         link: function(scope,elm,attrs){
             elm.timeEntry({
                 noSeparatorEntry: true,
                 ampmNames: ['am', 'pm'],
                 ampmPrefix: " "
             });
             if (attrs.timetype === 'end_time') {
                 elm.timeEntry('option', 'timeSteps', [1, 0, 0]);
                 elm.timeEntry('option', 'beforeSetTime', lastMinuteOnly);

                 function lastMinuteOnly(oldTime, newTime, minTime, maxTime) {
                     newTime.setMinutes(59);
                     return newTime;
                 }
             } else if (attrs.timetype === 'begin_hour') {
                 elm.timeEntry('option', 'timeSteps', [1, 0, 0]);
             }
             scope.$watch('ngModel', function (newValue, oldValue) {
                 if (newValue !== undefined) {
                     // If setting end hour to a number, convert to a date.
                     // Use 59 minutes after the prior hour, e.g., 11:59 pm for hour 24.
                     if (!isNaN(newValue)) {
                         if (attrs.timetype === 'end_time') {
                             if (newValue > 0) {
                                 newValue--;
                             }
                             newValue = new Date(0, 0, 0, newValue, 59, 0);
                         } else if (attrs.timetype === 'begin_hour') {
                             newValue = new Date(0, 0, 0, newValue, 0, 0);
                         }
                     }
                     $(elm).timeEntry('setTime', newValue);
                 }
             })
         }
    }
}).directive('recurspinner', function ($parse) {
        return function (scope, element, attrs, controller) {
            var ngModel = $parse(attrs.ngModel);
            $(element).spinner({
                spin: function(event, ui) {
                    setTimeout(function() {
                        scope.$apply(function() {
                            ngModel.assign(scope, element.val());
                        });
                    }, 0);
                }
            });
        }
})
.config(['ngDialogProvider', function (ngDialogProvider) {
    ngDialogProvider.setDefaults({
        plain: false,
        closeByDocument: true,
        closeByEscape: true,
        appendTo: false,
        preCloseCallback: function (value) {
        }
    });
}])
.factory('sharedGridData', function(){
    return {
        tempArray:[]
        
    };
})
.service('includeExcludeLabelService', function () {
       var label = 'inclusion';
         return {
            getLabel: function () {
                return label;
            },
            setLabel: function(value) {
                label = value;
            }
        };
})

.controller('backupJobController', ['$scope', '$rootScope', '$http', 'ngDialog', 'wizardService', '$analytics',
    'fileBrowserDialogService','sharedGridData','$protectService','inventorysvc', 'gettextCatalog',
    function($scope, $rootScope, $http, ngDialog, wizardService, $analytics,fileBrowserDialogService,sharedGridData,$protectService,inventorysvc,gettextCatalog) {

        // Getting loadindicator reference (Declared in protect.js) and hiding it once Backup Job wizard is loaded
        var loadingIndicator = $scope.ngDialogData;
        if (loadingIndicator !== undefined) {
            PHD.hideLoadingIndicator(loadingIndicator);
        }

        $scope.jobName='Backup Job'; //default job name plaveholder
        $scope.backupJobNameModel='Backup Job';
        $scope.isCustomJob = false;
        $scope.joborder={};
        //init empty joboredr
        $scope.joborder.name = gettextCatalog.getString("Backup Job"); //TODO init to textfield from HTML
        $scope.joborder.type = "backup"; //this is always backup.
        $scope.joborder.verify = "none";
        $scope.joborder.app_id = "1"; //default assetType is server and hence app_id will be 1 i.e. file-level
        $scope.joborder.sid = "1"; // default sid = 1
        $scope.joborder.clients = [];
        $scope.joborder.instances = [];
        $scope.joborder.calendar = [];
        $scope.scheduleTypeSelection = {
                name: 'Scheduled'
        };
        $scope.errorField=false;
        $scope.startDateError=false;
        $scope.dayError=false;
        $scope.startDateOlderError = false;
        $scope.startTimeError=false;
        $scope.recuranceError=false;
        $scope.recurrenceBoundsError=false;
        $scope.recursTime=[{id:0,name:'Hour(s)'},{id:1,name:'Minutes'}];
        $scope.isScheduled = true;
        $scope.joborder.storage = null;
        $scope.PLACEHOLDER_BACKUP_TARGET = 'SelectBackupTarget';
        $scope.savedStorage = null;
		$scope.recursEveryError = false;
        $scope.startDateFormatError = false;
        
        $scope.isIncrementalOrDifferntial= false;
        
        $scope.joborder.email_report = true;
        $scope.joborder.failure_report = true;
        $scope.selectedTreeNode = [];
        $scope.selectedAsset =[];
        $scope.calendarDays=[
            {'name':gettextCatalog.getString('Sun'), 'value' : "0"},
            {'name':gettextCatalog.getString('Mon'), 'value' : "1"},
            {'name':gettextCatalog.getString('Tue'), 'value' : "2"},
            {'name':gettextCatalog.getString('Wed'), 'value' : "3"},
            {'name':gettextCatalog.getString('Thu'), 'value' : "4"},
            {'name':gettextCatalog.getString('Fri'), 'value' : "5"},
            {'name':gettextCatalog.getString('Sat'), 'value' : "6"}
        ];
        $scope.backupModes = [];
        $scope.backupModesForNow=[];
        $scope.backupModesForSchedule=[];
        $scope.backupStrategies=[];
        $scope.backup_types = {};
        $scope.backupModesforJob = {};
        $scope.scheduleForm = {};
        $scope.hideEdit=false;
        $scope.jobErrorDialog = false;
        $scope.isSingleClient = false;      // a single-client schedule from the Legacy UI, only possible on edit.
        $scope.isForSLAPolicy = false;      // a job created by an SLA Policy, recommended edit through the policy only.
        $scope.SLAPolicyName = "";
		
        $scope.daysFull=[{name:'SUN',id:'run_on_sun_full',val:"0"},{name:'MON',id:'run_on_mon_full',val:"1"},{name:'TUE',id:'run_on_tue_full',val:"2"},{name:'WED',id:'run_on_wed_full',val:"3"},{name:'THR',id:'run_on_thr_full',val:"4"},{name:'FRI',id:'run_on_fri_full',val:"5"},{name:'SAT',id:'run_on_sat_full',val:"6"}];
        $scope.daysIncr=[{name:'SUN',id:'run_on_sun_incr',val:"0"},{name:'MON',id:'run_on_mon_incr',val:"1"},{name:'TUE',id:'run_on_tue_incr',val:"2"},{name:'WED',id:'run_on_wed_incr',val:"3"},{name:'THR',id:'run_on_thr',val:"4"},{name:'FRI',id:'run_on_fri',val:"5"},{name:'SAT',id:'run_on_sat_incr',val:"6"}];

        var templateWithTooltip = '<div class="ui-grid-disable-selection" style="word-wrap:normal;font-size:12px;padding-top:5px;padding-left:5px" ' +
                                    'title="{{row.entity[col.field]}}">{{row.entity[col.field]}}</div>';
        $scope.inventoryColumnDefs = [
            {
                field:'id',
                displayName:'Id',
                visible:false
            },
            {
                field:'nodeId',
                displayName:'Node Id',
                visible:false
            },
            {
                field: 'name',
                displayName:gettext("Name"),
                headerCellFilter: 'translate',
                headerCellClass: 'ui-grid-not-bold',
                cellTooltip: true,
                enableRowHeaderSelection: false,
                cellTemplate: '<div style="font-size:12px;padding-top:5px;padding-left:5px;cursor:pointer" ' +
                                'title="{{row.entity[col.field]}}"> ' +
                                '<span ng-if="row.entity.marked_offline === true"> ' +
                                    '<img src="assets/images/delete.png" style="height:14px;cursor:pointer" ' +
                                    ' title="Click to remove the unavailable asset" ' +
                                    'ng-click="grid.appScope.checkForDelete(row.entity)">' +
                                '</span>' +
                                ' {{row.entity[col.field]}}' +
                                '</div>',
                width:'32%'
            },
            {
                field:'rules',
                displayName:gettext("Protection Rules"),
                cellTooltip: true,
                headerCellFilter: 'translate',
                headerCellClass: 'ui-grid-not-bold',
                cellTemplate: templateWithTooltip,
                width:'52%'
            },
            {
                field:'edit' ,
                displayName:'',
                cellTemplate: '<div ng-hide="grid.appScope.hideEdit" style="padding-left:25px;padding-top:5px;cursor:pointer" ' +
                                'ng-click="grid.appScope.openEditSettingServerDialog(row.entity,row.getProperty(col.field), col.field)">' +
                                '<img src="assets/images/edit_pencil.png" style="height:11.5px;cursor:pointer;"/>' +
                                '<span style="font-size:11px;padding-left:5px;font-weight:bold"><translate>Edit</translate></span>' +
                               '</div>',
                width:'18%'
            }
        ];
        $scope.inventoryGridOptions={
            columnDefs: $scope.inventoryColumnDefs,
            data:'inventoryData',
            rowHeight: 32,
            //virtualizationThreshold:200,
            enableHorizontalScrollbar:0,
            gridMenuShowHideColumns:false,
            enableColumnMenus:false,
            enableColumnResizing: true
        };
        $scope.assetType={};
        $scope.assetTypeList=[
            { name: "Agent-Based Assets", hypervisor:"1" ,id:INVENTORY_TYPE_FAMILY_CLIENT},
            { name: "VMware Assets", hypervisor:"3" ,id:INVENTORY_TYPE_FAMILY_VMWARE},
            { name: "Hyper-V Assets", hypervisor:"2" ,id:INVENTORY_TYPE_FAMILY_HYPER_V},
        ];
        if (!$rootScope.isKumo) {
            $scope.assetTypeList.push({ name: "NAS",    hypervisor:"1"    ,id:INVENTORY_TYPE_FAMILY_CLIENT_NAS});
        }

        if($rootScope.isXenUEB){
            $scope.assetTypeList.push({ name: "XenServer Assets", hypervisor:"4" ,id:INVENTORY_TYPE_FAMILY_XEN});
        }
        if(!($rootScope.isCE || $rootScope.isGoogle || $rootScope.isKumo)){

            $scope.assetTypeList.push({ name: "Exchange", hypervisor:"0" ,id:INVENTORY_TYPE_FAMILY_EXCHANGE});
            $scope.assetTypeList.push({ name: "Oracle", hypervisor:"0" ,id:INVENTORY_TYPE_FAMILY_ORACLE});
            $scope.assetTypeList.push({ name: "SQL", hypervisor:"0" ,id:INVENTORY_TYPE_FAMILY_SQL});
            $scope.assetTypeList.push({ name: "Sharepoint", hypervisor:"0" ,id:INVENTORY_TYPE_FAMILY_SHAREPOINT});
            $scope.assetTypeList.push({ name: "Cisco UCS", hypervisor:"0" ,id:INVENTORY_TYPE_FAMILY_CISCO_UCS});
            $scope.assetTypeList.push({ name: "NDMP", hypervisor:"0", id:INVENTORY_TYPE_FAMILY_NDMP});

            if ($rootScope.showAHV){
                $scope.assetTypeList.push({ name: "Nutanix AHV", hypervisor:"0", id:INVENTORY_TYPE_FAMILY_AHV});
            }

        }

        $scope.stretch_database = false;

        $scope.PM_ADJUST_EXTRA  = 13;
        $scope.PM_ADJUST = 12;
        $scope.showtoolTipForEmailVerification = function(){
            var form = angular.element("#test-job-form");
            form.find(".definition").popover({
                appendTo: form
            });
        };

        $scope.showVolumeGroupTooltip = function(){
            var element = angular.element("#ahv_volume_groups_tooltip");
            element.find(".definition").popover({
                appendTo: element
            });
        };


        $scope.setProtectionRule = $protectService.setProtectionRule;       // common function for protection rules.

        if($scope.isEdit){
            $http({
                method: 'get',
                url : '/api/joborders/'+$scope.jobId + "/?lang=" + $rootScope.userLanguageCode
            }).success(function(data, status, headers){
                $scope.joborder=data.data[0];
                document.getElementById("backupJobID").value=$scope.joborder.name;
                $scope.assetType.id= $scope.getAssetType($scope.joborder);
                $scope.isHypervisorHostSelected = $scope.isRegexJob($scope.joborder);
                $scope.hideEdit = checkHideEdit($scope.assetType.id);
                $scope.isNAS = $rootScope.isSelectedJobNAS;
                $scope.isSingleClient = ($scope.joborder.single_client === true);
                $scope.isForSLAPolicy = ($scope.joborder.for_sla_policy !== false && $scope.joborder.for_sla_policy !== undefined && $scope.joborder.for_sla_policy.name !== undefined);
                if ($scope.isForSLAPolicy) {
                    $scope.SLAPolicyName = $scope.joborder.for_sla_policy.name;
                }
                $scope.isCustomJob = $scope.hasCustomCalendar($scope.joborder);
                $scope.setRecurrenceModel();
                $scope.setEndHour();
                $protectService.loadTreeData($http,$scope,$rootScope,$scope.assetType.id,false,$scope.jobId,false);
                //$scope.checkNodesOnLoadEdit();
            }).error(function(response){
                ngDialog.open({
                    dialogType:'Error',
                    dialogMessage:response.result[0].message
                })
            });

        }else{
            $scope.assetType.id= parseInt($scope.selectedAssetType);
            setTimeout(function(){
                $scope.filterInventory();}, 1000);
        }

        /*
         * Examines the calendar entries for the job, returning true if an element
         * requiring the job to be handled as custom (not handled with the regular calendar object)
         * has been encountered.
         */
        $scope.hasCustomCalendar = function(job) {
            var isCustom = false;
            var calendar = job.calendar;
            if (calendar instanceof Array && calendar.length > 0) {
                for (var i = 0 ; i < calendar.length ; i++){
                    var entry = calendar[i];
                    if (entry.custom === true) {
                        isCustom = true;
                        break;
                    }
                }
            }
            return isCustom;
        };

        /*
         * If an end hour is specified, recurrences run through that hour (minute 59).
         */
        $scope.setEndHour = function(){
            for (var i = 0 ; i < $scope.joborder.calendar.length ; i++){
                $scope.setCalendarEntryEndHour(i);
            }
        };

        /*
         * Set a user-friendly end-hour in the "until" field.
         */
        $scope.setCalendarEntryEndHour = function(i) {
            var minute = "59";
            var ampm = " pm";
            if (($scope.joborder.calendar[i] !== undefined) && (!isNaN($scope.joborder.calendar[i].end_hour))) {
                // If the end hour is a number, convert to a string, e.g., hour 24 is 11:59 pm.
                var endHour = parseInt($scope.joborder.calendar[i].end_hour);
                if (endHour > 13){
                    endHour -= $scope.PM_ADJUST_EXTRA;
                    $scope.joborder.calendar[i].end_hour = endHour+":"+minute+ampm;
                } else {
                    ampm = (endHour == 13) ? " pm" : " am";
                    endHour--;
                    $scope.joborder.calendar[i].end_hour = endHour+":"+minute+ampm;
                }
            }
        };

        $scope.setRecurrenceModel = function(){
            for (var i =0 ; i <$scope.joborder.calendar.length ; i ++){
             if(typeof($scope.joborder.calendar[i].recurrence) !='undefined'){
                 $scope.joborder.calendar[i].recursCheck  =true;
                 if($scope.joborder.calendar[i].recurrence < 60){
                     $scope.joborder.calendar[i].recursTimeSelect=1;
                 }else{
                     $scope.joborder.calendar[i].recurrence = $scope.joborder.calendar[i].recurrence/60;
                     $scope.joborder.calendar[i].recursTimeSelect=0;
                 }
                 // Wrap with timeout so that recurs loads as a spinner.
                 (function(){const index = i;
                     setTimeout(function(){$scope.recurstimeModeChange(index,$scope.joborder.calendar[index].recursTimeSelect)},0,index);})();
             }else{
                 $scope.joborder.calendar[i].recursCheck = false;
                 // No recurrence, but initialize potential values in case the user wants recurrences.
                 $scope.joborder.calendar[i].recursTimeSelect = 0;
                 $scope.joborder.calendar[i].recurrence = 1;
                 $scope.joborder.calendar[i].end_hour = 24;
             }
            }
        };
        $scope.treeConfig = {
            core : {
                multiple : !$scope.isNAS, //TODO check if it needs to single select in case of NAS
                animation: true,
                error : function(error) {console.log('treeCtrl: error from js tree - ' + angular.toJson(error));},
                check_callback : true,
                worker : true
            },
            types : {
                default : {icon : 'icon-hv-datacenter icon'},
                star : {icon : 'icon-hv-host icon'},
                cloud : {icon : 'icon-hv-host icon'}
            },
            version : 1,
            plugins : ['types','checkbox','search']
        };
        
        $scope.setAssetType=function(){
            if($rootScope.isSelectedJobFileLevel){
                 $scope.assetType.id= $scope.assetTypeList[0].id;
            }else if($rootScope.isSelectedJobNAS){
                 $scope.assetType.id= $scope.assetTypeList[3].id;
            }else{
                 $scope.assetType.id= $scope.assetTypeList[3];
            }
        }

        $scope.filterInventory = function(){
            $('#jstreeDivId').jstree(true).uncheck_all();
            $scope.joborder={};
            //init empty joboredr
            $scope.joborder.name = "Backup Job"; //TODO init to textfield from HTML
            $scope.joborder.type = "backup"; //this is always backup.
            $scope.joborder.verify = "none";
            $scope.joborder.app_id = "1"; //default assetType is server and hence app_id will be 1 i.e. file-level
            $scope.joborder.sid = "1"; // default sid = 1
            $scope.joborder.clients = [];
            $scope.joborder.instances = [];
            $scope.joborder.calendar = [];
            $scope.joborder.email_report = true;
            $scope.joborder.failure_report = true;
            $scope.joborder.storage = null;
            $scope.savedStorage = null;
            $scope.isHypervisorHostSelected = false;
            $rootScope.clearRegex();
            $scope.clearSearch();


            $scope.hideEdit = checkHideEdit($scope.assetType.id);
            $scope.isNAS = false;
            if($scope.assetType.id == INVENTORY_TYPE_FAMILY_CLIENT_NAS){
                $scope.isNAS = true;
                // This does not work.  The selection moves to the other NAS, but the original
                // is not removed from the right-hand-side correctly.  There is logic at the
                // top of winzard.backupJobManageAsset.html to check for multiple NASes.
                //$('#jstreeDivId').jstree(true).settings.core.multiple = false;
            } else {
                $('#jstreeDivId').jstree(true).settings.core.multiple = true;
            }
            $rootScope.treeData=[];
            $scope.selectedAsset =[];
            $scope.inventoryData =[];
            $protectService.loadTreeData($http,$scope,$rootScope,$scope.assetType.id,false);
        };
        
        //
        // Based on the asset type, returns an indication of whether or not the edit button should be visible for the job.
        //
        function checkHideEdit(assetType) {
            return (assetType == INVENTORY_TYPE_FAMILY_HYPER_V || assetType == INVENTORY_TYPE_FAMILY_EXCHANGE || assetType == INVENTORY_TYPE_FAMILY_SQL ||
                    assetType == INVENTORY_TYPE_FAMILY_ORACLE || assetType == INVENTORY_TYPE_FAMILY_SHAREPOINT || assetType == INVENTORY_TYPE_FAMILY_CISCO_UCS ||
                    assetType == INVENTORY_TYPE_FAMILY_NDMP);
        }
        
        $scope.getAssetType = function(joborder){
            var app_id= joborder.app_id;

            if(app_id === APP_ID_FILE_LEVEL){
                if(joborder.clients[0].name.indexOf("-NAS-RRC")>-1){ //hack till backend fix it.
                    return INVENTORY_TYPE_FAMILY_CLIENT_NAS;
                }
                return INVENTORY_TYPE_FAMILY_CLIENT;
            } else if(app_id === APP_ID_VMWARE){
                return INVENTORY_TYPE_FAMILY_VMWARE;
            } else if(app_id === APP_ID_XEN){
                return INVENTORY_TYPE_FAMILY_XEN;
            } else if(app_id === APP_ID_HYPERV_2008_R2 || app_id === APP_ID_HYPERV_2012 || app_id === APP_ID_HYPERV_2016){
                return INVENTORY_TYPE_FAMILY_HYPER_V;
            } else if (app_id === APP_ID_EXCHANGE_2003 || app_id === APP_ID_EXCHANGE_2007 || app_id === APP_ID_EXCHANGE_2010 || app_id === APP_ID_EXCHANGE_2013 || app_id === APP_ID_EXCHANGE_2016) {
                return INVENTORY_TYPE_FAMILY_EXCHANGE;
            }else if (app_id === APP_ID_SQL_SERVER_2005 || app_id === APP_ID_SQL_SERVER_2008 || app_id === APP_ID_SQL_SERVER_2008_R2 
                || app_id === APP_ID_SQL_SERVER_2012 || app_id === APP_ID_SQL_SERVER_2014 || app_id === APP_ID_SQL_SERVER_2016 || app_id === APP_ID_SQL_SERVER_2017) {
                return INVENTORY_TYPE_FAMILY_SQL;
            }else if (app_id === APP_ID_SHAREPOINT_2007 || app_id === APP_ID_SHAREPOINT_2010 || app_id === APP_ID_SHAREPOINT_2013 || app_id === APP_ID_SHAREPOINT_2016) {
                return INVENTORY_TYPE_FAMILY_SHAREPOINT;
            }else if (app_id === APP_ID_ORACLE_11 || app_id === APP_ID_ORACLE_12 || app_id === APP_ID_ORACLE_10) {
                return INVENTORY_TYPE_FAMILY_ORACLE;
            } else if(app_id === APP_ID_CISCO_UCS_SERVICE_PROFILE){
                return INVENTORY_TYPE_FAMILY_CISCO_UCS;
            } else if (app_id === APP_ID_NDMP) {
                return INVENTORY_TYPE_FAMILY_NDMP;
            } else if(app_id === APP_ID_AHV) {
                return INVENTORY_TYPE_FAMILY_AHV;
            }
        };

        $scope.getClientOrInstance = function(joborder){
            var app_id= joborder.app_id;
            if(app_id === APP_ID_FILE_LEVEL){
                if(joborder.clients[0].name.indexOf("-NAS-RRC")>-1){ //hack till backend fix it.
                    return joborder.clients;
                }
                return joborder.clients;
            } else {
                /* for all other apps, return instances.
                    APP_ID_EXCHANGE_2003 = 2,
                    APP_ID_EXCHANGE_2007 = 3,
                    APP_ID_EXCHANGE_2010 = 4,
                    APP_ID_EXCHANGE_2013 = 5,
                    APP_ID_SQL_SERVER_2005 = 21,
                    APP_ID_SQL_SERVER_2008 = 22,
                    APP_ID_SQL_SERVER_2008_R2 = 23,
                    APP_ID_SQL_SERVER_2012 = 24,
                    APP_ID_SQL_SERVER_2014 = 25,
                    APP_ID_ARCHIVE = 30,
                    APP_ID_VMWARE = 40,
                    APP_ID_HYPERV_2008_R2 = 50,
                    APP_ID_HYPERV_2012 = 51,
                    APP_ID_ORACLE_11 = 100,
                    APP_ID_ORACLE_12 = 101,
                    APP_ID_ORACLE_10 = 102,
                    APP_ID_SHAREPOINT_2007 = 110,
                    APP_ID_SHAREPOINT_2010 = 111,
                    APP_ID_SHAREPOINT_2013 = 112; */
                return joborder.instances;
            }
        };

        $scope.uncheckNotEligibleNode=function(uncheckNodeId){
            var node_id='li#'+uncheckNodeId;
            $('#jstreeDivId').jstree(true).uncheck_node(node_id);
        }

        $scope.readyTree=function(event){
            if($scope.isEdit){
                 setTimeout(function(){
                   $scope.checkNodesOnLoadEdit();
                   $rootScope.loadRegex();
                }, 1000);
            } else {
                if($scope.selectedNodeList.length>0){
                    setTimeout(function(){
                       $scope.checkNodesOnPreSelect();
                       $rootScope.initRegex(false);
                    }, 1000);
                }
            }
        };

        $scope.checkNodesOnPreSelect=function(){
            $('#jstreeDivId').jstree(true).open_all();
            for(var i=0; i< $scope.selectedNodeList.length ; i++){
                var node_id='li#';
                node_id = node_id +$scope.selectedNodeList[i];
                $('#jstreeDivId').jstree(true).check_node(node_id);
            }
        };
        $scope.checkNodesOnLoadEdit=function(){
            var nodeIdArray =Object.keys($('#jstreeDivId').jstree(true)._model.data);
            // If the model data is not loaded yet, do not proceed.
            if (nodeIdArray.length <= 1) {
                return;
            }
            var arrayAssets = $scope.getClientOrInstance($scope.joborder);
            $('#jstreeDivId').jstree(true).open_all();
            if(arrayAssets !=undefined){
                for(var i=0; i< arrayAssets.length ; i++){
                    var node_id='li#';
                    for(var j=0 ;j<nodeIdArray.length;j++){
                        var foundInTree = false;
                        if (nodeMatches(nodeIdArray[j], arrayAssets[i], $scope.joborder)) {
                            foundInTree = true;
                            arrayAssets[i].nodeId = nodeIdArray[j];
                            if(arrayAssets[i].incl_list == undefined || arrayAssets[i].incl_list == null){
                                arrayAssets[i].incl_list = [];
                            }
                            if(arrayAssets[i].excl_list == undefined || arrayAssets[i].excl_list == null){
                                arrayAssets[i].excl_list = [];
                            }

                            $scope.selectedAsset[nodeIdArray[j]] = arrayAssets[i];
                            node_id = node_id+nodeIdArray[j];
                            $scope.before_command = arrayAssets[i].before_command;
                            $scope.after_command = arrayAssets[i].after_command;
                            break;
                        }
                    }
                    // If not found in tree, save as unavailable and create a selectedAsset entry.
                    // Set a flag so asset will only be added once for offline status.
                    if (!foundInTree && arrayAssets[i].marked_offline !== true) {
                        var nodeId = makeOfflineNode(arrayAssets[i].id);
                        arrayAssets[i].nodeId = nodeId;
                        arrayAssets[i].marked_offline = true;
                        arrayAssets[i].name += " (unavailable)";
                        arrayAssets[i].rules = $scope.setProtectionRule(arrayAssets[i].incl_list,arrayAssets[i].excl_list,arrayAssets[i].metanames);
                        if ($scope.inventoryData === undefined) {
                            $scope.inventoryData = [];
                        }
                        $scope.inventoryData.push(arrayAssets[i]);
                        $scope.selectedAsset[nodeId] = arrayAssets[i];
                    }
                    if ($scope.isHypervisorHostSelected) {
                        var obj = inventorysvc.nodeToObject(node_id);
                        $rootScope.joborderHypervisorID = obj.system_id + '_' + obj.client_id + '_' + obj.app_id + '_' + obj.host_guid;
                    }
                    node_id = inventorysvc.handleSpecialChars(node_id);
                    $('#jstreeDivId').jstree(true).check_node(node_id);
                }
                $('#jstreeDivId').jstree(true).close_all();
            }
        };

        // Create a node for the instance not found in the tree.
        function makeOfflineNode(instance_id) {
            var cid = ($scope.joborder.client_id !== undefined) ? $scope.joborder.client_id :
                      ($scope.joborder.clients !== undefined && $scope.joborder.clients.length > 0) ? $scope.joborder.clients[0].id : 0;

            return $scope.joborder.sid + '_' + cid + '_' + $scope.joborder.app_id + '_' + instance_id;
        }

        function nodeMatches(node, asset, joborder) {
            var match = false;
            var asset_node = joborder.sid + "_";
            if (parseInt(joborder.app_id) === APP_ID_FILE_LEVEL) {
                asset_node += asset.id + "_" + joborder.app_id + "_";
                match = node.indexOf(asset_node) === 0;
            } else {
                asset_node += joborder.client_id + "_" + joborder.app_id + "_";
                if (node.indexOf(asset_node) === 0) {
                    var parts = node.split('_');
                    match = isNaN(parts[parts.length - 1]) ? false : (asset.id === parseInt(parts[parts.length - 1]));
                }
            }
            return match;
        }


        $scope.deslectNodeFun = function(e,item){
            if (item.node.data !== undefined && item.node.data.type === INVENTORY_ID_VMWARE_ESX_SERVER) {
                $rootScope.joborderHypervisorID = item.node.id;
                $rootScope.clearRegex();
                $scope.$apply(function() {
                    $scope.isHypervisorHostSelected = false;
                });
            }
            if(item.node.data.type ===INVENTORY_ID_SQL_INSTANCE ||
                item.node.data.type ===INVENTORY_ID_HYPER_V_SERVER ||
                item.node.data.type === INVENTORY_ID_VMWARE_ESX_SERVER  ||
                item.node.data.type === INVENTORY_ID_VMWARE_RESOURCE_POOL ||
                item.node.data.type === INVENTORY_ID_AHV_CLUSTER){
                for(var k =0 ; k < item.node.children_d.length; k++){
                    if($scope.selectedAsset[item.node.children_d[k]] !=null || $scope.selectedAsset[item.node.children_d[k]] != undefined){
                            delete $scope.selectedAsset[item.node.children_d[k]];
                    }
                }
                $scope.inventoryData =[];
                var nodeIdArray =Object.keys($scope.selectedAsset);
                for(var j=0 ;j<nodeIdArray.length;j++){
                    var obj = $scope.selectedAsset[nodeIdArray[j]];
                    obj.rules = $scope.setProtectionRule(obj.incl_list,obj.excl_list,obj.metanames);
                    $scope.inventoryData.push(obj);
                }
                $(window).resize();

            } else {
                if(item.selected.length == 0){
                    $scope.joborder.clients =[];
                    $scope.selectedAsset =[];
                }
                if($scope.joborder.sid == inventorysvc.nodeToObject(item.node.id).system_id){
                    delete $scope.selectedAsset[item.node.id];
                }
                $scope.inventoryData =[];
                var nodeIdArray =Object.keys($scope.selectedAsset);
                for(var j=0 ;j<nodeIdArray.length;j++){
                    var obj = $scope.selectedAsset[nodeIdArray[j]];
                    obj.rules = $scope.setProtectionRule(obj.incl_list,obj.excl_list,obj.metanames);
                    $scope.inventoryData.push(obj);
                }
                $(window).resize();
            }
        };

        $scope.slectNodeFun = function(e,item){
            var osTypeId = 0;
            if (item.node.data !== undefined && item.node.data.os_type_id !== undefined) {
                osTypeId = item.node.data.os_type_id;
            }
            /*
             * Until iSeries is integrated into the system, protect it from being seleected or pre-selected in a backup job.
             * Rather than disabling, use the selection to display a helpful message to user on how to protect an iSeries asset.
             */
            if (item.node.data !== undefined && item.node.data.type === INVENTORY_ID_CLIENT_I_SERIES) {
                var message  = "An iSeries asset may not be included in a backup job in the web-based user interface.  To protect iSeries data, please use the menu-based console interface.";
                $scope.uncheckNotEligibleNode(item.node.id);

                if ($scope.jobErrorDialog == false) {
                    // Set the dialog flag so that multiple dialogs are not opened, as this function can be
                    // called multiple times when the user selects multiple nodes at once, i.e., on pre-selection.
                    $scope.jobErrorDialog = true;
                    ngDialog.open({
                        dialogType:'Error',
                        modelDialogId:'BackupPostErrorDialog',
                        scope:$scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick:'onConfirmError()'
                    });
                }
            }else if(item.node.data.type ===INVENTORY_ID_SQL_INSTANCE ||
                item.node.data.type === INVENTORY_ID_HYPER_V_SERVER ||
                item.node.data.type === INVENTORY_ID_VMWARE_ESX_SERVER  ||
                item.node.data.type === INVENTORY_ID_VMWARE_RESOURCE_POOL ||
                item.node.data.type === INVENTORY_ID_AHV_CLUSTER){
                /*if(item.node.data.expanded == false){*/
                $('#jstreeDivId').jstree(true).open_all(item.node);
                item.node.data.expanded =true;
                // retrieving the sid and app_id from the instance/server/resource pool selected
                $scope.joborder.sid = inventorysvc.nodeToObject(item.node.id).system_id;
                $scope.joborder.app_id = inventorysvc.nodeToObject(item.node.id).app_id;
                /*}*/
                if (item.node.data !== undefined && item.node.data.type === INVENTORY_ID_VMWARE_ESX_SERVER) {
                        $scope.isHypervisorHostSelected = true;
                        $rootScope.joborderHypervisorID = item.node.id;
                } 
                for(var k =0 ; k < item.node.children_d.length; k++){
                    var id = item.node.children_d[k]+'_anchor';
                    if($scope.selectedAsset[item.node.children_d[k]] == null || $scope.selectedAsset[item.node.children_d[k]] == undefined){
                        if (inventorysvc.isGroup(item.node.children_d[k]) ||
                            inventorysvc.isPoolOrVApp(item.node.children_d[k])) {
                            // If the child is a group, pool, or vApp, do not add to the selected list.
                            continue;
                        }
                        var $el = $(document.getElementById(id));
                        $scope.selectedAsset[item.node.children_d[k]] ={nodeId:item.node.children_d[k],name:$el.text(),id:inventorysvc.nodeToObject(item.node.children_d[k]).client_id,incl_list:[],excl_list:[]};
                    }
                }
                $scope.inventoryData =[];
                var nodeIdArray =Object.keys($scope.selectedAsset);
                for(var j=0 ;j<nodeIdArray.length;j++){
                    var obj = $scope.selectedAsset[nodeIdArray[j]];
                    obj.rules = $scope.setProtectionRule(obj.incl_list,obj.excl_list,obj.metanames);
                    $scope.inventoryData.push(obj);
                }
                $(window).resize();

            } else {
                if(item.selected.length>1){
                    if($scope.joborder.sid == inventorysvc.nodeToObject(item.node.id).system_id){
                        if($scope.selectedAsset[item.node.id] ==null || $scope.selectedAsset[item.node.id] == undefined){
                            $scope.selectedAsset[item.node.id] ={nodeId:item.node.id,name:item.node.text,id:inventorysvc.nodeToObject(item.node.id).client_id,incl_list:[],excl_list:[],os_type_id:osTypeId};
                        }
                    }else{
                        $scope.uncheckNotEligibleNode(item.node.id);
                    }
                }else{
                    $scope.joborder.sid = inventorysvc.nodeToObject(item.node.id).system_id;
                    $scope.joborder.app_id = inventorysvc.nodeToObject(item.node.id).app_id;
                    if (item.node.data !== undefined && item.node.data.type === INVENTORY_ID_VMWARE_ESX_SERVER) {
                        $scope.isHypervisorHostSelected = true;
                        $rootScope.joborderHypervisorID = item.node.id;
                    } else {
                        if($scope.selectedAsset[item.node.id] == undefined ||$scope.selectedAsset[item.node.id] ==null){
                            $scope.selectedAsset[item.node.id] ={nodeId:item.node.id,name:item.node.text,id:inventorysvc.nodeToObject(item.node.id).client_id,incl_list:[],excl_list:[],os_type_id:osTypeId};
                        }
                    }
                }

                $scope.inventoryData =[];
                var nodeIdArray =Object.keys($scope.selectedAsset);
                for(var j=0 ;j<nodeIdArray.length;j++){
                    var obj = $scope.selectedAsset[nodeIdArray[j]];
                    if (obj.rules === undefined) {
                        if (item.node.data.type === INVENTORY_ID_VMWARE_VM || item.node.data.type === INVENTORY_ID_XEN_VM || item.node.data.type === INVENTORY_ID_AHV_VM) {
                            $scope.getExcludedDatastore(nodeIdArray[j]);
                        }
                        obj.rules = $scope.setProtectionRule(obj.incl_list, obj.excl_list, obj.metanames);
                    }
                    if (obj.os_type_id === undefined) {
                        obj.os_type_id = osTypeId;
                    }
                    $scope.inventoryData.push(obj);
                }
                $(window).resize();
            }
        };

        $scope.hoverNode = function(e, item){
            // In case of SQL databases, display the recovery model when hovered over
            if (item.node.data.recovery_model !== undefined && item.node.data.recovery_model !== "") {
                var model = item.node.data.recovery_model;
                var hoverTip = (model === "S:") ? "VM is either offline or Integration Services are not installed." : ("model:" + model);
            }
            // In case of SQL clusters and SQL Availability Groups, display that when hovered over
            if (item.node.data.is_sql_cluster !== undefined && item.node.data.is_sql_cluster !== false) {
                var hoverTip = "SQL Cluster";
            }
            if (item.node.data.is_sql_alwayson !== undefined && item.node.data.is_sql_alwayson !== false) {
                var hoverTip = "SQL Availability Group";
            }
            if (hoverTip !== undefined && hoverTip !== "" && hoverTip !== null) {
                var $el = $(document.getElementById(item.node.id));
                $el.prop('title', hoverTip);
            }
        };

        $scope.refreshInventoryData = function(assets){
            var assets = assets || false;
            if (assets) {
                $scope.selectedAsset = assets;
            }
            $scope.inventoryData =[];
            var nodeIdArray =Object.keys($scope.selectedAsset);
            for(var j=0 ;j<nodeIdArray.length;j++){
                var obj = $scope.selectedAsset[nodeIdArray[j]];
                obj.rules = $scope.setProtectionRule(obj.incl_list,obj.excl_list,obj.metanames);
                $scope.inventoryData.push(obj);
            }
            $(window).resize();

        };

        $scope.setHypervisorHostValue = function(setting) {
            $scope.isHypervisorHostSelected = setting;
        };

        $scope.getCalendarDescription = function() {
            return ($scope.joborder.calendar_str instanceof Array) ? $scope.joborder.calendar_str.join('\r\n') : gettextCatalog.getString("No calendar exists");
        };

        $scope.applyModelChanges = function() {
            if($scope.isScheduled){
                if(typeof($scope.backupModesforJob.name) !='undefined'){
                    $scope.getBackupTypeForSelectedStrategies($scope.backupModesforJob.name);
                    if ($scope.backupModesforJob.name == 'Custom') {
                        $scope.isCustomJob = true;
                        $scope.calendar_description = $scope.getCalendarDescription();
                    } else {
                        $scope.isCustomJob = false;
                    }
                }
            }
        };

        $scope.searchIndex = 0;

        $scope.search = function(){
            setTimeout(function () {
                var v = $('#inventorySearchInputId').val();
                $('#jstreeDivId').jstree(true).search(v);
            }, 200);
        };

        $scope.goToSearchResult = function() {
            var el =  $('#jstreeDivId').jstree(true).get_node($scope.searchResults[$scope.searchIndex], true);
            if (el !== undefined) {
                el.children('.jstree-anchor').focus();
            }
            el.focus();
        };

        $scope.getSearchResults = function(event, obj) {
            setTimeout(function () {
                $scope.$apply(function() {
                    $scope.searchResults = obj.nodes;
                    $scope.searchIndex = 0;
                });
            });
        };

        $scope.next = function() {
            var len = $scope.searchResults.length;
            if ($scope.searchIndex + 1 > len - 1) {
                $scope.searchIndex = 0;
            } else {
                $scope.searchIndex++;
            }
            $scope.goToSearchResult();
        };

        $scope.previous = function() {
            var len = $scope.searchResults.length;
            if ($scope.searchIndex - 1 < 0) {
                $scope.searchIndex = len - 1;
            } else {
                $scope.searchIndex--;
            }
            $scope.goToSearchResult();
        };

        $scope.clearSearch = function() {
            $('#jstreeDivId').jstree(true).clear_search();
            $('#inventorySearchInputId').val("");
        };

        $scope.clearSearchResults = function() {
            $scope.searchIndex = 0;
            $scope.searchResults = undefined;
        };

        $scope.addNewInventory=function(item,id){
        }

        $scope.getProtectionRulesArray = function(inclusionData,exclusionData){
        }

        $scope.functiontofindIndexByKeyValue = function(arraytosearch, key, valuetosearch) {
            for (var i = 0; i < arraytosearch.length; i++) {
                if (arraytosearch[i][key] == valuetosearch) {
                  return i;
                 }
            }
                return null;
        };

        $scope.isRegexJob = function(job) {
            return (job !== undefined && job.regular_expressions !== undefined && job.regular_expressions.length > 0);
        };

        $scope.canShowFilter = function() {
            return ($scope.assetType.id == INVENTORY_TYPE_FAMILY_VMWARE);
        };

        /*
         * Returns true if the schedule supports auto-inclusion of new assets.
         */
        $scope.canAutoInclude = function(asset_type, singleClient) {
            var result = false;
            switch (asset_type) {
                // Removed SharePoint below - there is only 1 item (the farm)
                // Cisco UCS does not do auto-include either.
                case INVENTORY_TYPE_FAMILY_CLIENT:
                    if (!singleClient) {
                        // If a single-client schedule, do not show auto-include.
                        $scope.assetItems = "assets";
                        result = true;
                    }
                    break;
                case INVENTORY_TYPE_FAMILY_EXCHANGE:
                case INVENTORY_TYPE_FAMILY_ORACLE:
                //case INVENTORY_TYPE_FAMILY_SHAREPOINT:
                case INVENTORY_TYPE_FAMILY_SQL:
                    $scope.assetItems = "databases";
                    result = true;
                    break;
                case INVENTORY_TYPE_FAMILY_NDMP:
                    $scope.assetItems = "volumes";
                    result = true;
                    break;
                case INVENTORY_TYPE_FAMILY_HYPER_V:
                case INVENTORY_TYPE_FAMILY_VMWARE:
                case INVENTORY_TYPE_FAMILY_AHV:
                    $scope.assetItems = "VMs";
                    result = true;
                    break;
            }
            return result;
        };

/*************************************************************************
2ndstep
*/  
        $scope.validationForForm=function(){
            $scope.errorField=false;
            $scope.startDateError=false;
            $scope.dayError=false;
            $scope.startDateOlderError = false;
            $scope.startTimeError=false;
            $scope.recuranceError=false;
            $scope.recurrenceBoundsError = false;
			$scope.endHourError =false;
			$scope.recursEveryError = false;
            $scope.startDateFormatError = false;
			

            if ($scope.joborder.calendar === undefined || $scope.joborder.calendar.length == 0 ){
                return true;
            }else{
                for (var i =0 ; i < $scope.joborder.calendar.length ; i++){
                    if(!angular.isUndefined($scope.joborder.calendar[i].run_on)){
                        if ($scope.joborder.calendar[i].run_on.length <=0) {
                            $scope.errorField = true;
                            $scope.dayError = true;
                        }
                    }
                  if(typeof($scope.joborder.calendar[i].start_date) == 'undefined'){
                        $scope.errorField=true;
                        $scope.startDateError=true;
                    }
                 if(typeof($scope.joborder.calendar[i].start_time) == 'undefined'){
                        $scope.errorField=true;
                        $scope.startTimeError=true;
                   }
                 else{
                     var checkStartDateFormatError = $scope.validateStartDate($scope.joborder.calendar[i].start_date);
                     if(checkStartDateFormatError){
                         $scope.startDateFormatError = true;
                         $scope.errorField = true;
                     } else {
                         $scope.joborder.calendar[i].start_date =
                                DateTimeHandler.getUSDateString($rootScope.userLanguageCode, $scope.joborder.calendar[i].start_date);
                         if (!$scope.isEdit) {
                             var checkStartDate = $scope.validateStart($scope.joborder.calendar[i].start_date);
                             if (checkStartDate) {
                                 $scope.startDateOlderError = true;
                                 $scope.errorField = true;
                             }
                         }
                     }
                 }
				   if( $scope.joborder.calendar[i].recursCheck){
                      if(typeof($scope.joborder.calendar[i].recurrence )==='undefined'  || $scope.joborder.calendar[i].recurrence==null || $scope.joborder.calendar[i].recurrence==="" || $scope.joborder.calendar[i].recurrence==0 ){
                          $scope.errorField=true;
                          $scope.recuranceError=true;
                          $scope.recursEveryError=true;
                      } else {
                          var timerID = "#recurs_spinner_field" + i;
                          var isValid = $(timerID).spinner("isValid");
                          if (!isValid) {
                              $scope.errorField = true;
                              $scope.recurrenceBoundsError = true;
                          }
                      }
                  }
                  if( $scope.joborder.calendar[i].recursCheck){
                      if(typeof($scope.joborder.calendar[i].end_hour) == 'undefined') { //} || ($scope.joborder.calendar[i].end_hour< 0 && $scope.joborder.calendar[i].end_hour > 24) ){
                           $scope.errorField=true;
                           $scope.endHourError=true;
                      }
                      var start_hour = $scope.getHourFromString($scope.joborder.calendar[i].start_time);
                      var end_hour = $scope.getHourFromString($scope.joborder.calendar[i].end_hour);
                      if (start_hour >= end_hour) {
                          $scope.errorField=true;
                          $scope.endHourError=true;
                      }
                  }
                  
                  if($scope.errorField){
                      $scope.error_backupType =$scope.joborder.calendar[i].backup_type;
                        break;
                    }
                    
                }
                
                return !$scope.errorField;
            }
      
        }

        $scope.validateStartDate = function (date) {
            return DateTimeHandler.dateNotValid(date, $rootScope.userLanguageCode);
        };

        $scope.validateStart = function(date) {
            if (date === '') {
                return true;
            }
            var today = new Date();
            today.setHours(0);
            today.setMinutes(0);
            today.setSeconds(0);
            if (Date.parse(today) > Date.parse(date)) {
                return true;
            }
            return false;
        };

        /*
         * If the standard strategies are in use, make sure any extras are deleted.
         * If a custom calendar, the joborder.calendar will not exist.
         */
        $scope.removeUnwantedKeys = function(){
            if ($scope.joborder.calendar !== undefined) {
                for (var i =0 ; i < $scope.joborder.calendar.length ; i++){
                    if(!$scope.joborder.calendar[i].recursCheck){
                        delete $scope.joborder.calendar[i].recurrence;
                    }
                    else if ($scope.joborder.calendar[i].recursTimeSelect == 0 ){
                        $scope.joborder.calendar[i].recurrence =$scope.joborder.calendar[i].recurrence * 60;
                    }
                    $scope.joborder.calendar[i].end_hour = $scope.getHourFromString($scope.joborder.calendar[i].end_hour);
                    delete $scope.joborder.calendar[i].$$hashKey;
                    delete $scope.joborder.calendar[i].recursTimeSelect;
                    delete $scope.joborder.calendar[i].recursCheck;
                    if ($scope.joborder.calendar[i].begin_hour !== undefined) {
                        delete $scope.joborder.calendar[i].begin_hour;
                    }
                }
            }
        };

        $scope.getHourFromString =function(hourString){
            var hourStringArray = hourString.split(" ");
            var hourArray = hourStringArray[0].split(":");
            var Hour  = parseInt(hourArray[0]);
            var Minute = parseInt(hourArray[1]);
            if (hourStringArray[1] === "pm"){
                if (Hour < 12) {
                    Hour += $scope.PM_ADJUST;
                }
            } else {
                if (Hour == 12) {
                    Hour -= $scope.PM_ADJUST;
                }
            }
            if (Minute == 59) {
                Hour++;
            }
            return Hour;
        };

        $scope.onConfirmCredentialWarning = function() {
            ngDialog.close('sharepointCredentials');
        };
		
        $scope.nextWizard = function($event){
            if(wizardTitle === gettextCatalog.getString("Select Items")){
		        console.log($scope.selectedAsset);
                $scope.stretch_database = false;
                if (!$scope.validateAssets()) {
                    return;
                }
                var nodeIdArray =Object.keys($scope.selectedAsset);
                var flag = false;
                for(var j=0 ;j<nodeIdArray.length;j++){
                    var obj = $scope.selectedAsset[nodeIdArray[j]];
                    var idLength = obj.nodeId.split('_');
                    if((idLength[2] == APP_ID_SQL_SERVER_2016 || idLength[2] == APP_ID_SQL_SERVER_2017) && !flag){
                        flag = true;
                        $scope.stretch_database = true;
                    }
                    if((idLength[2] == APP_ID_SHAREPOINT_2016 ||
                        idLength[2] == APP_ID_SHAREPOINT_2013 ||
                        idLength[2] == APP_ID_SHAREPOINT_2010 ||
                        idLength[2] == APP_ID_SHAREPOINT_2007) && !flag){

                        var url = "/api/assets/" + idLength[1] + "/?sid=" + idLength[0];
                        var resp = PHD.Ajax.get(url);
                        resp.done(function(data) {
                            data = data.data[0];
                            var apps = data.children;
                            apps.forEach(function(app){
                                if(app.id == idLength[2]){
                                    console.log(app.children[0].num_app_servers,app.children[0].credentials,app.children[0].credentials.length);
                                    if(((app.children[0].num_app_servers > 1) || (app.children[0].num_app_servers == 1 && app.id == APP_ID_SHAREPOINT_2016))
                                        && app.children[0].credentials.length == 0){
                                        flag = true;
                                        console.log("WARNING");
                                        ngDialog.open({
                                            dialogType:'Warning',
                                            modelDialogId:'sharepointCredentials',
                                            scope:$scope,
                                            dialogMessage: gettextCatalog.getString('Credentials are required for selected SharePoint application backups. Please associate credentials to the application.'),
                                            onConfirmOkButtonClick:'onConfirmCredentialWarning()'
                                        });
                                    }
                                }
                            });
                        });
                    }
                }
                $rootScope.$broadcast("nextBtnClick");
                if($scope.isEdit == true){
                    $scope.isScheduled = true;
                    $scope.scheduleTypeSelection.name ='Scheduled';
                }
                $scope.getStorage();
                $scope.getBackupStrategies();
				$scope.showtoolTipForEmailVerification();


            }
            if(wizardTitle === gettextCatalog.getString("Define Job Settings")){
                if(document.getElementById("backupJobID").value === ""){
                    $scope.jobNameError=true;
                    return;
                }
                $scope.jobNameError=false;
                if (!$scope.validateBackupTarget()) {
                    return;
                }
                if($scope.validationForForm()){
                    $scope.saveBackupJobData(); //To save backupjob data.
                }else{
                    console.log('validation error');
                }
            }
        };

        $scope.validateBackupTarget = function() {
            var valid = true;
            if (typeof $scope.joborder.storage === 'object' && $scope.joborder.storage.value === $scope.PLACEHOLDER_BACKUP_TARGET){
                $scope.backuptarget=true;
                $scope.errorField=true;
                valid = false;
            } else {
                $scope.backuptarget = false;
                $scope.errorField = false;
            }
            return valid;
        };

        $scope.validateAssets = function() {
            var valid = true;
            var selectedCount = Object.keys($scope.selectedAsset).length;
            if (selectedCount == 0){
                // At least one asset must be selected.
                valid = false;
            } else if ($scope.isNAS && selectedCount > 1) {
                // If a NAS, only one should be selected.
                valid = false;
            }
            return valid;
        };

        $scope.validateAssetsAndSaveJobData = function() {
            if ($scope.validateAssets()) {
                $scope.handleDateFormats();
                $scope.saveBackupJobData();
            }
        };
        
		$scope.saveBackupJobData = function(){
			    $scope.joborder.name=document.getElementById("backupJobID").value;
                     if($scope.assetType.id ===INVENTORY_TYPE_FAMILY_CLIENT || $scope.assetType.id ===INVENTORY_TYPE_FAMILY_CLIENT_NAS){
                        $scope.joborder.clients =[];// $scope.j
                        var nodeIdArray =Object.keys($scope.selectedAsset);
                        for(var j=0 ;j<nodeIdArray.length;j++){
                            $scope.joborder.app_id = inventorysvc.nodeToObject($scope.selectedAsset[nodeIdArray[j]].nodeId).app_id;
                            $scope.selectedAsset[nodeIdArray[j]].id = parseInt($scope.selectedAsset[nodeIdArray[j]].id);
                            $scope.joborder.clients.push($scope.selectedAsset[nodeIdArray[j]]);
                            if($scope.isNAS || $scope.isSingleClient){
                                var client = $scope.joborder.clients[0];
                                $scope.joborder.client_id = client.id;
                                $scope.joborder.incl_list = client.incl_list;
                                $scope.joborder.excl_list = client.excl_list ;
                                $scope.joborder.before_command = client.before_command;
                                $scope.joborder.after_command = client.after_command;
                                delete $scope.joborder.clients;
                                delete $scope.joborder.instances;
                            }
                        }

                    }else{

                        $scope.joborder.instances =[];// $scope.j
                        var nodeIdArray =Object.keys($scope.selectedAsset);
                        for(var j=0 ;j<nodeIdArray.length;j++){
                            $scope.joborder.app_id = inventorysvc.nodeToObject($scope.selectedAsset[nodeIdArray[j]].nodeId).app_id;
                            $scope.selectedAsset[nodeIdArray[j]].id = parseInt(inventorysvc.nodeToObject($scope.selectedAsset[nodeIdArray[j]].nodeId).instance_id);
                            $scope.joborder.instances.push($scope.selectedAsset[nodeIdArray[j]]);
                        }

                    }
                    /*
                     * Save as iCal if custom job
                     */
                    if ($scope.isCustomJob) {
                        if ($scope.joborder.newiCal !== undefined) {
                            $scope.joborder.ical = $scope.joborder.newiCal;
                        }
                        if ($scope.joborder.ical !== undefined) {
                            delete $scope.joborder.calendar;
                            delete $scope.joborder.newiCal;
                        } else {
                            ngDialog.open({
                                dialogType:'Retry',
                                dialogMessage: gettextCatalog.getString('Custom calendars must contain at least one backup event.')
                            });
                            return;
                        }
                    } else {
                        if ($scope.joborder.ical !== undefined) {
                            $scope.joborder.newiCal = $scope.joborder.ical;
                        }
                        delete $scope.joborder.ical;
                    }
                    var jobOrderUrl='';
                    var dataObj ={};
                    var httpMethod='';
                    var meassageforLoad=(!$scope.isEdit)?gettextCatalog.getString("Creating Job"):gettextCatalog.getString("Updating Job");
                    var loadJobindicator = PHD.showLoadingIndicator("body", true, meassageforLoad);
                    // Get the joborder value - if an object, get its value.
                    $scope.savedStorage = $scope.joborder.storage;
                    $scope.joborder.storage = (typeof $scope.joborder.storage === 'object' && $scope.joborder.storage.value !== null) ?
                                                    $scope.joborder.storage.value : $scope.joborder.storage;
                    if($scope.isScheduled){
                        $scope.removeUnwantedKeys();
                        if($scope.isEdit == true){
                            httpMethod = 'put';
                            jobOrderUrl = '/api/joborders/'+$scope.joborder.id+'/?sid='+$scope.joborder.sid;

                        }else{
                            jobOrderUrl = '/api/joborders/?sid='+$scope.joborder.sid;
                            httpMethod = 'post';
                        }dataObj =JSON.stringify($scope.joborder)
                        console.log($scope.joborder);
                    }else{
                        console.log($scope.backupModesforJob.name);
                        if(typeof ($scope.joborder.calendar) != 'undefined'){
                            delete $scope.joborder.calendar;

                        }
                        $scope.joborder.backup_type=$scope.backupModesforJob.name;
                        console.log($scope.joborder);
                        jobOrderUrl = '/api/backups/?sid='+$scope.joborder.sid;
                        dataObj =JSON.stringify($scope.joborder)
                        httpMethod = 'put';
                    }

                    $http({ method: httpMethod,
                        url : jobOrderUrl,
                        data: dataObj
                    }).success(function(data, status, headers){
                        PHD.hideLoadingIndicator(loadJobindicator);
						
						if(data[0]){
						 ngDialog.open({
							dialogType: 'Error',
							modelDialogId: 'error-dialog',
							scope: $scope,
							dialogMessage: data[0].message
                         });
							
						}else{
                        if (!$scope.isScheduled) {
                            $analytics && $analytics.eventTrack('On Demand Backup Job', {  category: 'Backup', label: 'Protect' });
                            ngDialog.open({
                                modelDialogId:'backupNowDialog',
                                template: 'app/protect/backup-job/backupNowDialog.html',
                                scope:$scope,
                                data: data.data,
                                overlay:true,
                                ngDialogStyle:"width:700px;height:400px;"
                            });
                        } else {
                            if (!$scope.isEdit) {
                                var pageLabel = (window.location.href.indexOf("#/protect") !== -1) ? 'Protect' : 'Jobs';
                                $analytics && $analytics.eventTrack('Add Backup Job', {  category: 'Backup', label: pageLabel });
                                var message = gettextCatalog.getString("Job created successfully") + "!<br/><br/>" +
                                                gettextCatalog.getString("Use the Job Manager tab to view and manage your newly created job.");
                                console.log(data);
                                console.log(status);
                                console.log(headers);
                                
                                if (pageLabel === 'Jobs') {
                                    ngDialog.open({
                                        dialogType: 'Success',
                                        dialogMessage: message
                                    });
                                    $(document).trigger("editJob");
                                } else {
                                    ngDialog.open({
                                        dialogType: 'Success',
                                        scope:$scope,
                                        tertiaryButton: true,
                                        tertiaryButtonLabel: gettextCatalog.getString("View Jobs"),
                                        onTertiaryButtonClick: 'viewJobs()',
                                        dialogMessage: message
                                    });
                                }
                            } else {
                                $analytics && $analytics.eventTrack('Edit Backup Job', {  category: 'Backup', label: 'Jobs' });
                                ngDialog.close('backup-physical-dailog');
                                $(document).trigger("editJob");
                            }
                        }
					}
                    }).error(function(response) {
                        var message = response !== undefined && response.result instanceof Array ? response.result[0].message : 'Unknown Server Error';
                        $analytics && $analytics.eventTrack('Failure: ' + message, {  category: 'Backup', label: 'Protect' });
                        PHD.hideLoadingIndicator(loadJobindicator);
                        $scope.outputTitle = "Backup Job Error";
                        $scope.outputData = message;
                        $scope.onConfirmOkButtonClick = $scope.onConfirmError;
                        ngDialog.open({
                            template: 'app/common/scroll-output.html',
                            modelDialogId:'BackupPostErrorDialog',
                            dialogMessage:message,
                            scope:$scope,
                            ngDialogStyle: 'width:650px;',
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                        if ($scope.isScheduled) {
                            // re-set the elements deleted before the save.
                            if ($scope.joborder.calendar !== undefined) {
                                $scope.setRecurrenceModel();
                                $scope.setEndHour();
                            }
                            if ($scope.joborder.ical !== undefined) {
                                $scope.joborder.newiCal = $scope.joborder.ical;
                            } else if ($scope.joborder.newiCal !== undefined) {
                                $scope.joborder.ical = $scope.joborder.newiCal;
                            }
                        }
                        if ($scope.savedStorage !== undefined && $scope.savedStorage !== null) {
                            $scope.joborder.storage = $scope.savedStorage;
                        }
                    });
			
		};
        $scope.handleDateFormats = function() {
            if ($scope.joborder.calendar !== undefined){
                for (var i = 0; i < $scope.joborder.calendar.length; i++) {
                    $scope.joborder.calendar[i].start_date =
                        DateTimeHandler.getUSDateString($rootScope.userLanguageCode, $scope.joborder.calendar[i].start_date);
                }
            }
        };
        $scope.viewJobs = function () {
            ngDialog.closeAll();
            window.location.href = "#/jobs/#1";
        };
        $scope.onConfirmError = function () {
            ngDialog.close('BackupPostErrorDialog');
            $scope.jobErrorDialog = false;
            $scope.handleDateFormats(); // do this so that the date is swapped back for international formats
        };

         $scope.getBackupMode = function() {
            if($scope.scheduleTypeSelection.name == "Now") {
                var jobName=$scope.backupJobNameModel;
                $scope.jobOrderName=jobName;
                $scope.backupJobNameModel='On-Demand';
                $scope.jobNameError=false;    
                $scope.isScheduled = false;  
                $scope.joborder.calendar=[];
                $scope.backupModes=$scope.backupModesForNow;
                $scope.backupModesforJob.name= $scope.backupModes[0];
                $scope.isCustomJob = false;

            } else {
                if ($scope.backupJobNameModel == 'On-Demand'){
                    $scope.backupJobNameModel=(typeof($scope.jobOrderName) !='undefined')?$scope.jobOrderName:gettextCatalog.getString('Backup Job');
                }
                $scope.isScheduled = true;
                $scope.backupModes=$scope.backupModesForSchedule;

                var index =0;
                for ( var i =0 ; i < $scope.backupModes.length ; i++){
                    if ($scope.backupModes[i] == gettextCatalog.getString('Incremental Forever')){
                        index=i;
                        break;
                    }
                }

                $scope.backupModesforJob.name= $scope.backupModes[index];
                $scope.backupModesforJob.original = $scope.backupModesforJob.name;
                $scope.getBackupTypeForSelectedStrategies($scope.backupModes[index]);

            }
            

        };
        $scope.getBackupTypeForSelectedStrategies = function(backupStratiges){
            $scope.joborder.calendar=[];
            var dataArray=[];
            for (var i =0 ; i <$scope.backupStrategies.length ; i++ ){
                if(backupStratiges == $scope.backupStrategies[i].name){
                    dataArray =  $scope.backupStrategies[i].backup_types;
                }
            }
            for (j = 0 ; j < dataArray.length ; j++){
                var calendarObj={
                    start_date:new Date().toLocaleDateString($rootScope.userLanguageCode),
                    start_time:"03:00 AM",
                    backup_type:dataArray[j],
                    recursCheck:false,
                    recursTimeSelect:0,
                    recurrence:1,
                    schedule_run: 0,
                    end_hour:24
                };
                // Set up calendar frequencies based on type.
                switch (calendarObj.backup_type) {
                    case "Incremental":
                        if (dataArray.length == 1) {
                            calendarObj.run_on = ["0", "1", "2", "3", "4", "5", "6"];
                        } else {
                            calendarObj.run_on = ["1", "2", "3", "4", "5", "6"];
                        }
                        break;
                    case "Differential":
                    case "Transaction":
                        calendarObj.run_on = ["1", "2", "3", "4", "5", "6"];
                        break;
                    case "Full":
                    default:
                        calendarObj.run_on = ["0"];
                        break;
                }
                $scope.joborder.calendar.push(calendarObj);
                // wrap the setTimeout function so that we can pass in the index
                (function(){
                    const index = j;
                    setTimeout(function(){$scope.recurstimeModeChange(index,$scope.joborder.calendar[index].recursTimeSelect)},0,index);
                })();

                $scope.setCalendarEntryEndHour(j);
            }
        };

        $scope.customOpen = false;      // prevent double-click error.
        $scope.editCustomCalendar = function() {
            if (!$scope.customOpen) {
                $scope.customOpen = true;
                ngDialog.open({
                    template: 'app/protect/custom-schedule/custom.html',
                    scope: $scope,
                    overlay: true,
                    data: {joborder: $scope.joborder, modes: $scope.backupModesForNow, calendarType: 'Backups'},
                    modelDialogId: 'edit-custom-dialog',
                    name: 'edit-custom-dialog',
                    ngDialogStyle: 'width:900px;height:680px;',
                    preCloseCallback: function (value) {
                        $scope.customOpen = false;
                    }
                });
            }
        };
        $scope.setBackupModeForEdit = function(){
            var StrategiesObject={};
            for (var i=0 ; i < $scope.backupStrategies.length ; i++){
                var backupstrategyType="";
                for(var j=0 ; j < $scope.backupStrategies[i].backup_types.length ; j++){
                    backupstrategyType +=$scope.backupStrategies[i].backup_types[j];
                }
                StrategiesObject[backupstrategyType]=$scope.backupStrategies[i].name;
            }
            var backupType="";
            if ($scope.joborder.calendar !== undefined) {
                for ( var i = 0 ; i < $scope.joborder.calendar.length; i++){
                    backupType +=$scope.joborder.calendar[i].backup_type;
                }
            }
            var selectedBackupType=StrategiesObject[backupType];
            if (selectedBackupType === undefined || selectedBackupType == gettextCatalog.getString('Custom') || $scope.isCustomJob) {
                // if not found, or if there was a property on a calendar entry indicating it is custom, choose 'Custom'
                selectedBackupType = 'Custom';
                $scope.joborder.calendar = [];
                $scope.isCustomJob = true;
                $scope.calendar_description = $scope.getCalendarDescription();
            } else {
                $scope.isCustomJob = false;
            }
            $scope.backupModesforJob.name= $scope.backupModes[$scope.backupModes.indexOf(selectedBackupType)];
            $scope.backupModesforJob.original = $scope.backupModesforJob.name;
        };

        $scope.getBackupStrategies = function() {
            var ctr = 0;        // keeps track of number Win 10 or 2016 clients that've been selected
            for (var i = 0; i < $scope.inventoryData.length; i++) {
                if ($scope.inventoryData[i].os_type_id !== undefined && $scope.inventoryData[i].os_type_id != 0) {
                    if ($scope.inventoryData[i].os_type_id == 43 || $scope.inventoryData[i].os_type_id == 44) {
                        ctr++;
                    }
                }
            }
            var scheduleApi = "/api/backups/strategies/"+$scope.joborder.app_id+"/";
            // if all selected assets are either Win 10 or 2016, Bare Metal option shouldn't show; if atleast one isn't, show Bare Metal option
            if (ctr == $scope.inventoryData.length) {
                scheduleApi += "?noBareMetal=true";
            }
            if($scope.isNAS){
                scheduleApi += "?isNAS=true";
            }
            // If a Kumo, the only strategy is incremental forever backups.
            if ($rootScope.isKumo) {
                if (scheduleApi.indexOf('?') === -1) {
                    scheduleApi += "?onlyIncremental=true";
                } else {
                    scheduleApi += "&onlyIncremental=true";
                }
            }
            $http.get(scheduleApi)
                .then(function(response){
                        $scope.backup_types ={};
                        $scope.backupModesForSchedule = [];
                        $scope.backupModesForNow = response.data.on_demand;
                        var tempMap = new Object();
                        $scope.backupStrategies=response.data.strategies;
                        for(var i=0; i< response.data.strategies.length;i++) {    
                            $scope.backup_types[response.data.strategies[i].name] = response.data.strategies[i].backup_types;
                            $scope.backupModesForSchedule[i]=response.data.strategies[i].name;
                        }
                        if($scope.isScheduled){
                            $scope.backupModes=$scope.backupModesForSchedule;
                             if ($scope.isEdit){
                                 $scope.setBackupModeForEdit();
                                 
                             }else{
                                 $scope.backupModesforJob.name= $scope.backupModes[0];
                             }
                        }else {
                            $scope.backupModes=$scope.backupModesForNow;
                             $scope.backupModesforJob.name= $scope.backupModes[0];
                        }
                    if (!$scope.isEdit){
                        $scope.getBackupMode();
                    }
                },function(reason){
                    console.log(reason);
                });
            };
            $scope.recurstimeModeChange = function(index,timeSelect){
                var timmerId="#recurs_spinner_field"+index;
                if(timeSelect == 0){
                    $(timmerId).spinner({min: 1, max: 12});
                }else{
                    $(timmerId).spinner({min: 1, max: 60});
                }
            };
          $scope.getStorage = function(){
                var storageApi = "/api/storage/?sid="+$scope.joborder.sid+"&usage=backup";
                $scope.backupTarget ={};
                $scope.backupTarget.targets = [];

                $http.get(storageApi)
                    .then(function(response){
                        console.log(response.data.storage.length);
                        var defaultStorageIndex = 0;
                        var storageIndex = 0;
                        var offset = 0;
                        if (response.data.storage.length > 1){
                            $scope.backupTarget.targets.push({name:gettextCatalog.getString('Select Backup Target'),value: $scope.PLACEHOLDER_BACKUP_TARGET});
                            // Add 1 to the index as the targets array includes the "Selection" entry.
                            offset = 1;
                        }

                        for(var j=0;j<response.data.storage.length ;j++){
                            $scope.backupTarget.targets.push({name:response.data.storage[j].name,value:response.data.storage[j].name});
                            if (response.data.storage[j].is_default){
                                defaultStorageIndex = j + offset;
                            }
                            if($scope.joborder.storage == response.data.storage[j].name) {
                                storageIndex = j + offset;
                            }
                        }

                        if (!$scope.isEdit){
                            $scope.joborder.storage = $scope.backupTarget.targets[defaultStorageIndex];
                        } else if ($scope.joborder.storage === "") {
                            $scope.joborder.storage = $scope.backupTarget.targets[defaultStorageIndex];
                        } else {
                            $scope.joborder.storage = $scope.backupTarget.targets[storageIndex];
                        }

                    },function(reason){
                        console.log(reason);
                    });
            };

        $scope.getExcludedDatastore = function(selectedNodeId){
            
            var url ='/api/inventory/'+selectedNodeId +'/?disks=1';
                console.log(url);
            $http({
                method: 'GET',
                url: url
            }).success(function(data, status, headers) {
                console.log(data.inventory.nodes);
                var obj = $scope.selectedAsset[selectedNodeId];
                for(var i=0;i<data.inventory.nodes.length;i++){
                    if(data.inventory.nodes[i].is_excluded){
                      obj.excl_list.push(data.inventory.nodes[i].name); 
                    }
                }
                obj.rules = $scope.setProtectionRule(obj.incl_list,obj.excl_list,obj.metanames);

            }).error(function(data, status, headers) {
                console.log('api_inventory_error');
            });
            
        };

        $scope.checkForDelete = function(selectedAsset) {
            $scope.assetToDelete = null;
            if (selectedAsset !== undefined) {
                var assetName = selectedAsset.name;
                if (assetName.indexOf(' (unavailable)') !== -1) {

                    $scope.assetToDelete = selectedAsset;
                    var confirmText = gettextCatalog.getString("Do you want to delete this unavailable asset from the job?");

                    ngDialog.open({
                        dialogType: 'Confirmation',
                        dialogMessage: confirmText,
                        scope: $scope,
                        overlay:true,
                        modelDialogId:'delete-unavailable-asset',
                        onConfirmOkButtonClick:'onConfirmDeleteAsset()'
                    });
                }
            }
        };

        $scope.onConfirmDeleteAsset = function() {
            var asset = $scope.assetToDelete;
            var assetName = asset.name;
            for (var i = 0; i < $scope.inventoryData.length; i++) {
                if (assetName === $scope.inventoryData[i].name) {
                    $scope.inventoryData.splice(i, 1);
                }
            }
            delete $scope.selectedAsset[asset.nodeId];
            ngDialog.close('delete-unavailable-asset');
        };

       
        $scope.openEditSettingServerDialog = function(row,cell,column){

            $scope.selectedAssetId = row.id;
            $scope.selectedAssetName = row.name;
            $scope.selectedNodeId = row.nodeId;
            $scope.selectedAssetOS = row.os_type_id;
            if($scope.assetType.id === INVENTORY_TYPE_FAMILY_VMWARE ||
                $scope.assetType.id === INVENTORY_TYPE_FAMILY_XEN ||
                $scope.assetType.id === INVENTORY_TYPE_FAMILY_AHV){
                 ngDialog.open({
                            template: 'app/protect/backup-job/edit-vmware.html',
                            scope: $scope,
                            overlay:true,
                            modelDialogId:'edit-vmware-dailog',
                            ngDialogStyle:'width:600px;height:520px;',
                            preCloseCallback: function (value) {}
                });
            }
            if($scope.assetType.id === INVENTORY_TYPE_FAMILY_CLIENT_NAS ||
                $scope.assetType.id === INVENTORY_TYPE_FAMILY_CLIENT ){
                ngDialog.open({
                            template: 'app/protect/backup-job/edit-asset.html',
                            scope: $scope,
                            overlay:true,
                            modelDialogId:'edit-asset-dailog',
                            ngDialogStyle:'width:600px;height:520px;',
                            preCloseCallback: function (value) {}
                });
            }
      }

}])
.controller('editAssetController', ['$scope', '$rootScope', '$http', 'ngDialog', 'wizardService', '$analytics','fileBrowserDialogService','sharedGridData','$protectService','inventorysvc','dataToBeShared','includeExcludeLabelService','gettextCatalog',
    function($scope, $rootScope, $http, ngDialog, wizardService, $analytics,fileBrowserDialogService,sharedGridData,$protectService,inventorysvc,dataToBeShared,includeExcludeLabelService,gettextCatalog) {
        $scope.tab = 1;
        $scope.currentTab = 1;
        $scope.ADVANCED_TAB = 3;

        $scope.isWindowsAsset = $protectService.isWindowsAsset($scope.$parent.selectedAssetOS);
        $scope.isNixAsset = $protectService.isNixAsset($scope.$parent.selectedAssetOS);

        $scope.windowsOnlyTitle = gettextCatalog.getString('for Windows assets only');
        $scope.alwaysExcluded = $scope.isWindowsAsset ? ("(" + gettextCatalog.getString('always excluded') + ")") : "";
        $scope.nixOnlyTitle = gettextCatalog.getString('for Linux/Unix assets only');

        var index = $scope.$parent.selectedAssetName.indexOf("-NAS-RRC");
        if(index != undefined && index !=null && index > -1){
            $scope.isNAS = true;
        }

        $scope.inclusionGridOptions = {};
        $scope.inclusionColumnDefs =    [
            {   field:'delete',
                displayName:'',
                cellTemplate:'<div><img src="assets/images/delete.png" style="height:14px;cursor:pointer;margin:5px;" ng-click="grid.appScope.deleteRow(row.entity,1)"/></div>',
                width: 25
            },
            {
                field: 'name',
                displayName:gettextCatalog.getString('Name'),
                cellTemplate: '<div class="ngCellText" title="{{row.entity.name}}">{{row.entity.name}}</div>'
            }
        ];
        $scope.exclusionFileGridOptions = {};
        $scope.exclusionFileColumnDefs = [
            {
                field:'delete',
                displayName:'',
                cellTemplate:'<div><img src="assets/images/delete.png" style="height:14px;cursor:pointer;margin:5px;" ng-click="grid.appScope.deleteRow(row.entity,2)"/></div>',
                width: 25
            },
            {
                field: 'name',
                displayName:'Name',
                cellTemplate: '<div class="ngCellText" title="{{row.entity.name}}">{{row.entity.name}}</div>'
            }
        ];

        $scope.inclusionGridOptions = {
            rowHeight: 30,
            enableRowSelection: false,
            gridMenuShowHideColumns:false,
            enableColumnMenus:false,
            enableHorizontalScrollbar: 0,
            columnDefs: $scope.inclusionColumnDefs,
            data: 'inclusionData'
        };

        $scope.exclusionFileGridOptions = {
            rowHeight: 30,
            enableRowSelection: false,
            gridMenuShowHideColumns:false,
            enableColumnMenus:false,
            enableHorizontalScrollbar: 0,
            columnDefs: $scope.exclusionFileColumnDefs,
            data: 'exclusionData'
        };

        $scope.inclusionData = [];
        $scope.exclusionData = [];
        $scope.originalInclusionData = [];
        $scope.originalExclusionData = [];
        if ($scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].incl_list !== undefined) {
            for(var i =0;i<$scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].incl_list.length;i++){
                $scope.inclusionData.push({name:$scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].incl_list[i]});
                $scope.originalInclusionData.push({name:$scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].incl_list[i]});
            }
        }
        $scope.inclusionCheckboxModel = ($scope.originalInclusionData.length === 0);

        if ($scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].excl_list !== undefined) {
            for (var i = 0; i < $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].excl_list.length; i++) {
                $scope.exclusionData.push({name: $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].excl_list[i]});
                $scope.originalExclusionData.push({name: $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].excl_list[i]});
            }
        }
        
		if($scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].before_command != undefined){
			$scope.before_command = $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].before_command ;
		}else{
			$scope.before_command = ""; 
		}

		if($scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].after_command != undefined){
	       $scope.after_command = $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].after_command;
		}else{
			$scope.after_command = ""; 
		}

        $scope.showCurrentTab = function(currentTab){
            $scope.currentTab = currentTab;
            // Setup the warning popover for system state if the advanced tab.
            if ($scope.currentTab === $scope.ADVANCED_TAB) {
                $scope.showtoolTipForSystemState();
            }
            $(window).resize();
        };

        $scope.selectTab = function (setTab){
            $scope.tab = setTab;
        };
        $scope.isSelected = function(checkTab) {
            return $scope.tab === checkTab;
        };

        $scope.deleteRow = function(row,grid){
            if(grid == 1){
                var index = $scope.inclusionData.indexOf(row);
                $scope.inclusionData.splice(index, 1);
            }else{
                var index = $scope.exclusionData.indexOf(row);
                $scope.exclusionData.splice(index, 1); 
            }
        };

        $scope.openBrowseFileDialog  = function(tab){
            var dataObj={};
            var fileId = $scope.$parent.selectedAssetId;
            var dirName = "";
            var inclusionExclusionLabel ="";

            if(tab == 1){
                inclusionExclusionLabel = "include";
                includeExcludeLabelService.setLabel(inclusionExclusionLabel);
            }else{
                inclusionExclusionLabel = "exclude";
                includeExcludeLabelService.setLabel(inclusionExclusionLabel);
            }

            if($scope.isNAS){
                var index = $scope.$parent.selectedAssetName.indexOf("-NAS-RRC");
                dirName ="&dir="+"/mnt/nas/"+$scope.$parent.selectedAssetName.substring(0,index);
            }

            var url = '/api/clients/files/'+fileId+'/?sid='+inventorysvc.nodeToObject($scope.$parent.selectedNodeId).system_id+dirName;
            var fileBrowserTreeNodeUrl = '/api/clients/files/'+fileId+'/?sid='+inventorysvc.nodeToObject($scope.$parent.selectedNodeId).system_id+'&dir=';
            dataObj = {fileBrowserTreeNodeUrl:fileBrowserTreeNodeUrl,url:url};
            
            fileBrowserDialogService.getfileBrowserTreeData(dataObj); // to open andget data for file browser tree.
        };
        /*
         * Based on the advanced exclusions, build and return a metanames array.
         */
        $scope.buildMetanames = function() {
            var metanames = [];
            if ($scope.excludeNetmountsModel) {
                metanames.push('netmounts');
            }
            if ($scope.excludeSystemStateModel) {
                metanames.push('registry');
            }
            // Do not pass in tempfiles, as the flag makes no difference to the backup performed.
            // Tempfiles only applies to Windows assets, and they are always excluded.
            /*if ($scope.excludeTempFilesModel) {
                metanames.push('tempfiles');
            }*/
            if ($scope.excludeReadmountsModel) {
                metanames.push('readmounts');
            }
            if ($scope.excludeAllmountsModel) {
                metanames.push('allmounts');
            }
            return metanames;
        };
        $scope.savefileServerSettingData = function(){
             $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].incl_list =[];
             $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].excl_list =[];
            if($scope.inclusionData===undefined || $scope.inclusionData===null || $scope.inclusionData.length===0){
                $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].incl_list =[];
            } else {
                for(var i =0;i<$scope.inclusionData.length;i++){
                    $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].incl_list.push($scope.inclusionData[i].name);
                }
            }
            if($scope.exclusionData===undefined || $scope.exclusionData===null || $scope.exclusionData.length===0){
                $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].excl_list =[];
            } else {
                for(var i =0;i<$scope.exclusionData.length;i++){
                    $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].excl_list.push($scope.exclusionData[i].name);
                }
            }
			$scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].before_command = $scope.before_command;
			$scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].after_command =  $scope.after_command;
            $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].metanames = $scope.buildMetanames();

            $scope.$parent.$parent.refreshInventoryData();
            unbind();
            ngDialog.close('edit-asset-dailog');
        }
        $scope.closeEditServerSettingDialog =function(){
            unbind();
                if($scope.isInclusionExclusionChanged() || $scope.haveAdvancedSettingsChanged()) {
                  ngDialog.open({
                     template: 'app/protect/backup-job/confirm-backup-job.html',
                     scope: $scope,
                     overlay: true,
                     modelDialogId: 'confirm-edit-backup',
                     showHelpArticleButton: true,
                     ngDialogStyle: 'width:46em; height:15em;  top: 70px;left: 27px;right: 27px;'
                 });
               } else {
                ngDialog.close('edit-asset-dailog');
            }
        }
        
         $scope.isInclusionExclusionChanged = function(){
            $scope.isInclusionModified = false;
            if(angular.equals($scope.inclusionData, $scope.originalInclusionData)!==true || angular.equals($scope.exclusionData,$scope.originalExclusionData)!==true ){
                $scope.isInclusionModified = true;
            }else{
                $scope.isInclusionModified = false;
            }
            return $scope.isInclusionModified;
        }

        $scope.haveAdvancedSettingsChanged = function(){
            var stateChanged = false;
            if ($scope.excludeSystemStateModel !== $scope.originalSystemStateModel ||
                    $scope.excludeTempFilesModel !== $scope.originalTempFilesModel ||
                    $scope.excludeAllmountsModel !== $scope.originalAllmountsModel ||
                    $scope.excludeReadmountsModel !== $scope.originalReadmountsModel ||
                    $scope.excludeNetmountsModel !== $scope.originalNetmountsModel) {
                stateChanged = true;
            }
            return stateChanged;
        };

        $scope.closeEditBackupDialogs = function(){
            ngDialog.close('edit-asset-dailog');
            ngDialog.close('confirm-edit-backup');
        }
        $scope.cancelEditBackup = function(){
            ngDialog.close('confirm-edit-backup'); 
        }

        var unbind= $scope.$watch(function(){
            return dataToBeShared.objectList;
        }, function(newValue,oldValue){
            if (newValue === "___no_data_2___"){
                // Special token that indicates that there is no data, so clear the list and unbind.
                dataToBeShared.objectList = undefined;
                unbind();
            } else {
                if (!angular.isUndefined(newValue) && (angular.isUndefined(oldValue) || newValue.name != oldValue.name)) {
                    console.log(newValue);
                    $scope.addBrowseItem(newValue);
                    dataToBeShared.objectList = undefined;
                }
            }
        });

        $scope.addIncludeText = function(){
           var obj = {name:$scope.includeText};
            var uniqueFlag = containsObject(obj,$scope.inclusionData);
            if(uniqueFlag != true && $scope.includeText != undefined && $scope.includeText !== ""){
                $scope.inclusionData.push(obj);
            }
            $scope.includeText="";
        }
         
        $scope.addExcludeText = function(){
            var obj = {name:$scope.excludeText};
            var uniqueFlag = containsObject(obj,$scope.exclusionData);
            if(uniqueFlag != true && $scope.excludeText != undefined && $scope.excludeText !== ""){
                $scope.exclusionData.push(obj);
            }
            $scope.excludeText="";
        }
        function containsObject(obj, list) {
            if(!angular.isUndefined(list)){
                var i;
                for (i = 0; i < list.length; i++) {
                    if (list[i].name === obj.name) {
                        return true;
                    }
                }
            }
            return false;
        }

        $scope.addBrowseItem = function(listArray){
            for(var i=0;i<listArray.length;i++){
                var dirName = "";
                if($scope.isNAS){
                    var index = $scope.$parent.selectedAssetName.indexOf("-NAS-RRC");
                    if(index != undefined && index !=null && index > -1) {
                        var name = listArray[i].name;
                        // Don't add /mnt/nas if already there.
                        if (name.indexOf("@@@:/mnt/nas/") === -1) {
                            dirName = "/mnt/nas/" + $scope.$parent.selectedAssetName.substring(0, index) + "/";
                        }
                    }
                }
                listArray[i].name = dirName+listArray[i].name;
                var unique = true;
                if($scope.currentTab == 1) {
                    for (var j=0; j<$scope.inclusionData.length; j++){
                        if ($scope.inclusionData[j].name === listArray[i].name){
                            unique = false;
                            break;
                        }
                    }
                    if (unique) {
                        $scope.inclusionData.push(listArray[i]);
                    }
                }else{
                    for (j=0; j<$scope.exclusionData.length; j++){
                        if ($scope.exclusionData[j].name === listArray[i].name){
                            unique = false;
                            break;
                        }
                    }
                    if (unique){
                        $scope.exclusionData.push(listArray[i]);
                    }
                }
            }
      
            $(window).resize();
        }

		$scope.addPresets = function(tab){
        	if(tab == 1){
				$scope.inclusionData = [];
		    	for(var i=0;i< $scope.originalInclusionData.length;i++){
				  $scope.inclusionData.push( $scope.originalInclusionData[i]);
				}
			}else{
				$scope.exclusionData =[];
				for(var j=0;j<$scope.originalExclusionData.length;j++){
				  $scope.exclusionData.push($scope.originalExclusionData[j]);
				}
		    }
		};

        $scope.excludeStateWarningMessage = gettextCatalog.getString('Only select this option for protecting data volumes.') + " " +
            gettextCatalog.getString('Recovery of this backup will not restore server metadata,') + " " +
                gettextCatalog.getString('and these backups cannot be used to create Windows Replicas.');

        $scope.showExcludeStateWarning = function() {
            if ($scope.excludeSystemStateModel) {
                ngDialog.open({
                    dialogType:'warning',
                    dialogMessage: $scope.excludeStateWarningMessage,
                    modelDialogId: 'exclude-state-dialog',
                    scope: $scope,
                    onConfirmOkButtonClick:'onConfirmStateWarning()'
                });
            }
        };

        $scope.onConfirmStateWarning = function() {
            ngDialog.close('exclude-state-dialog');
        };

        $scope.isNameExcluded = function(selectedAsset, metaname) {
            var itemExcluded = false;
            if (selectedAsset !== undefined && selectedAsset.metanames !== undefined && Array.isArray(selectedAsset.metanames)) {
                var metanames = selectedAsset.metanames;
                for (var i = 0; i < metanames.length; i++) {
                    if (metanames[i] === metaname) {
                        itemExcluded = true;
                        break;
                    }
                }
            }
            return itemExcluded;
        };

        $scope.excludeSystemStateModel = $scope.isNameExcluded($scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId], 'registry');
        $scope.originalSystemStateModel = $scope.excludeSystemStateModel;

        // Always set exclude tempfiles model if a Windows asset.
        $scope.excludeTempFilesModel = $scope.isWindowsAsset; //$scope.isNameExcluded($scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId], 'tempfiles');
        $scope.originalTempFilesModel = $scope.excludeTempFilesModel;

        $scope.excludeReadmountsModel = $scope.isNameExcluded($scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId], 'readmounts');
        $scope.originalReadmountsModel = $scope.excludeReadmountsModel;

        $scope.excludeNetmountsModel = $scope.isNameExcluded($scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId], 'netmounts');
        $scope.originalNetmountsModel = $scope.excludeNetmountsModel;

        $scope.excludeAllmountsModel = $scope.isNameExcluded($scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId], 'allmounts');
        $scope.originalAllmountsModel = $scope.excludeAllmountsModel;


        $scope.showtoolTipForSystemState = function(){
            var element = angular.element("#exclude-state-checkbox");
            element.find(".definition").popover({
                appendTo: element
            });
        };

    }]).controller('editVmWareController', ['$scope', '$rootScope', '$http', 'ngDialog', 'wizardService', '$analytics','fileBrowserDialogService','sharedGridData','$protectService','inventorysvc','dataToBeShared',
    function($scope, $rootScope, $http, ngDialog, wizardService, $analytics,fileBrowserDialogService,sharedGridData,$protectService,inventorysvc,dataToBeShared) {
        $scope.tab =1;
        $scope.excluded_datastore = $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].excl_list;
        $scope.showCheckError=false;
        console.log('editVmWareController');
        $scope.gridOptions={
                    multiSelect: true,
                    showHeader: false,
                    enableFullRowSelection : true,
                    gridMenuShowHideColumns:false,
                    enableHorizontalScrollbar: 0,
                    columnDefs: [
                        { field: 'name', showHeader: false, displayName:"", cellTooltip: true },
                        { field: 'id', visible:false , showHeader: false }
                    ]
                };
    
        //$scope.gridOptions.data=[{'name':'Abc','id':'1'}];
         $scope.selectTab = function (setTab){
            $scope.tab = setTab;
        };
        $scope.isSelected = function(checkTab) {
            return $scope.tab === checkTab;
        };

        var selectedNodeId = $scope.$parent.selectedNodeId;
        console.log(selectedNodeId);
        var url ='/api/inventory/'+selectedNodeId +'/?disks=1';
         console.log(url);
         $http({
            method: 'GET',
            url: url
        }).success(function(data, status, headers) {
             console.log(data.inventory.nodes);
          $scope.gridOptions.data=data.inventory.nodes;
           
        }).error(function(data, status, headers) {
            console.log('api_inventory_error');
        });
        
         $scope.gridOptions.onRegisterApi = function(gridApi){
                    //set gridApi on scope
                    $scope.gridApi = gridApi;
                    gridApi.selection.on.rowSelectionChanged($scope,function(row){
                        if (row.isSelected){
                            $scope.showCheckError=false;
                        }
                    });

                };
         $scope.gridOptions.isRowSelectable =function(row){
                    for(var i=0; i<$scope.excluded_datastore.length;i++){
                        if($scope.excluded_datastore[i] === row.entity.name){
                            return true;
                        }
                    }
                    row.setSelected(true);
                    return true;
                };


        $scope.saveVmWareSettingData = function(){
            $scope.selectedDataStores = $scope.gridApi.selection.getSelectedRows();
            //console.log($scope.selectedDataStores);
            if($scope.selectedDataStores.length > 0){
                $scope.showCheckError=false;
            }else{
                $scope.showCheckError=true;
            }
            
            if(!$scope.showCheckError){
                var dataStoreGridData = $scope.gridOptions.data;
                var filterExcludeDataStore = dataStoreGridData.filter(function(obj) { return $scope.gridApi.selection.getSelectedRows().indexOf(obj) == -1; })
                //console.log(filterExcludeVolumeArray);
                $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].excl_list =[];
                for(var i=0;i<filterExcludeDataStore.length;i++){
                    $scope.excluded_datastore.push(filterExcludeDataStore[i].name);
                    $scope.$parent.$parent.selectedAsset[$scope.$parent.selectedNodeId].excl_list.push(filterExcludeDataStore[i].name);
                }
                $scope.$parent.$parent.refreshInventoryData();
                ngDialog.close('edit-vmware-dailog');
            }
        }
        

        $scope.closeEditServerSettingDialog =function(){
            $scope.closeThisDialog();
        }

}]).controller('filterVMwareController', ['$scope', '$rootScope', '$http', 'ngDialog', 'wizardService', '$analytics',
    'fileBrowserDialogService','sharedGridData','$protectService','inventorysvc','dataToBeShared', 'gettextCatalog',
    function($scope, $rootScope, $http, ngDialog, wizardService, $analytics,
             fileBrowserDialogService,sharedGridData,$protectService,inventorysvc,dataToBeShared, gettextCatalog) {

        $scope.containers = [
            { name: 'Select Container', id: -1},
            { name: 'Datacenters',      id: 4 },
            { name: 'ESX Clusters',     id: 5 },
            { name: 'ESX Servers',      id: 6 },
            { name: 'Datastores',       id: 7 },
            { name: 'Resource Pools',   id: 8 },
            { name: 'vApps',            id: 9 },
            { name: 'Folders',          id: 10 },
            { name: 'VM DisplayName',   id: 11 }
        ];

        $scope.actions = [
            { name: 'Select Action',    id: -1},
            { name: 'Equal',            id: 0 },
            { name: 'Not Equal',        id: 1 },
            { name: 'Contains',         id: 2 },
            { name: 'Not Contains',     id: 3 },
            { name: 'Starts with',      id: 4 },
            { name: 'Ends With',        id: 5 }
        ];

        $scope.default_regex = {
            container: $scope.containers[0],
            action: $scope.actions[0],
            filter: '',
            isEdit: false
        };
        $scope.regexList = [];
        $scope.ROW_HEIGHT = 28;
        $scope.MAX_REGEX_COUNT = 10;

        $scope.addRegex = function(regex) {
            if ($scope.isHypervisorHostSelected) {
                if ($scope.regexList.length == 0 ||
                    ($scope.regexList.length < $scope.MAX_REGEX_COUNT && validRegex($scope.regexList[$scope.regexList.length - 1]))) {
                    var regex = regex || null;
                    if (regex) {
                        $scope.regexList.push(regex);
                    } else {
                        $scope.regexList.push(jQuery.extend(true, {}, $scope.default_regex));
                        $scope.regexList[$scope.regexList.length - 1].container = $scope.containers[0];
                        $scope.regexList[$scope.regexList.length - 1].action = $scope.actions[0];
                    }

                    var $tree = $("#jstreeDivId");
                    var $controls = $("#protect-tree-controls");
                    var $footer = $("#protect-treeview-footer-dialog");
                    $tree.height($tree.height() - $scope.ROW_HEIGHT);
                    $controls.height($controls.height() + $scope.ROW_HEIGHT);
                    $footer.height($footer.height() + $scope.ROW_HEIGHT);
                }
            } else {
                ngDialog.open({
                    dialogType: 'Information',
                    onConfirmOkButtonClick: 'closeThisDialog()',
                    dialogMessage: gettextCatalog.getString('A VMware host must be selected when using filters.')
                });
            }
        };

        /*
         * Removes either the regex specified as the index, or the last regex from the list if no index is given.
         */
        $scope.removeRegex = function(index) {
            var $tree = $("#jstreeDivId");
            var $controls = $("#protect-tree-controls");
            var $footer = $("#protect-treeview-footer-dialog");

            if (index !== undefined && index < $scope.regexList.length) {
                $scope.regexList.splice(index, 1);
            } else {
                $scope.regexList.pop();
            }
            console.log($footer);
            $tree.height($tree.height() + $scope.ROW_HEIGHT);
            $controls.height($controls.height() - $scope.ROW_HEIGHT);
            $footer.height($footer.height() - $scope.ROW_HEIGHT);
            if ($scope.regexList.length === 0) {
                $scope.joborder.instances =[];
                $scope.joborder.regular_expressions = {};
                $scope.joborder.include_new = false;
                $('#jstreeDivId').jstree(true).uncheck_all();
                $scope.$parent.$parent.setHypervisorHostValue(false);
                $scope.$parent.$parent.refreshInventoryData([]);
            } else {
                var nonEmptyRegex = $scope.getRegex(false);
                if (nonEmptyRegex.length > 0) {
                    // If there are non-empty remaining regexes, apply them after deletion.
                    $scope.applyRegex(nonEmptyRegex);
                }
            }
        };

        $rootScope.clearRegex = function() {
            var item_count = $scope.regexList.length;
            for (var i = 0; i < item_count; i++) {
                $scope.removeRegex();
            }
        };

        /*
         * Apply the regex list, either processing the list passed in as a parameter or
         * the current list defined at the $scope.
         */
        $scope.applyRegex = function(regexToApply) {
            if ($scope.isHypervisorHostSelected) {
                var node = $rootScope.joborderHypervisorID;
                var nodeObj = inventorysvc.nodeToObject(node);
                var sidString = (nodeObj.system_id !== null) ? "?sid=" + nodeObj.system_id : "";
                var url = '/api/hypervisor/regex/' + sidString;
                var regex = regexToApply || $scope.getRegex(true);
                console.log('regex');
                console.log(regex);
                var validObj = isValidRegexList(regex);
                if (validObj.valid) {
                    var obj = {};
                    obj.host = node;
                    obj.regex = regex;
                    var json_obj = JSON.stringify(obj);
                    var load = PHD.showLoadingIndicator('body', true, 'Searching for VMs that meet the filter criteria....');
                    var resp = PHD.Ajax.put(url, json_obj, load, handleError);
                    resp.done(function(data) {
                        console.log(data.data);
                        $scope.selectedAsset = [];
                        for (var i = 0; i < data.data.length; i++) {
                            var vm = data.data[i];
                            var item = {};
                            item.nodeId = $rootScope.joborderHypervisorID + '_' + vm.instance_id;
                            item.name = vm.name;
                            item.id = vm.instance_id;
                            item.incl_list = [];
                            item.excl_list = [];
                            $scope.selectedAsset[item.nodeId] = item;
                        }
                        $scope.$parent.$parent.refreshInventoryData($scope.selectedAsset);
                        $scope.joborder.regular_expressions = regex;
                        $scope.joborder.include_new = true;
                        // Add another if less than the limit.
                        if ($scope.regexList.length < $scope.MAX_REGEX_COUNT) {
                            $scope.addRegex();
                        }
                    });
                } else {
                    ngDialog.open({
                        dialogType:'Error',
                        modelDialogId:'regex-dialog-not-valid',
                        scope:$scope,
                        dialogMessage: validObj.message,
                        onConfirmOkButtonClick:'closeOnConfirmError()'
                    });
                }
            } else {
                ngDialog.open({
                    dialogType: 'Information',
                    onConfirmOkButtonClick: 'closeThisDialog()',
                    dialogMessage: gettextCatalog.getString('A VMware host must be selected when using filters.')
                });
            }
        };
		$scope.closeOnConfirmError = function () {
            ngDialog.close('regex-dialog-not-valid');
        };

        /*
         * Checks the validity of the regex list.
         */
        function isValidRegexList(regex) {
            var obj = {valid: true, message: ''};
            for (var i = 0; i < regex.length; i++) {
                var item = regex[i];
                obj = isValidRegex(item);
                if (obj.valid === false) {
                    break;
                }
            }
            return obj;
        }

        /*
         * Checks the validity of the regex and returns an object with a boolean validity check and a message.
         */
        function isValidRegex(item) {
            var obj = {valid: true, message: ''};
            if (item.container_id < 0) {
                obj.message = gettextCatalog.getString("Please select a container name.");
                obj.valid = false;
            } else if (item.action_id < 0) {
                obj.message = gettextCatalog.getString("Please select a filter action.");
                obj.valid = false;
            } else if (item.filter.replace(/\s/g, "").length <= 0) {
                obj.message = gettextCatalog.getString("Please enter filter text.");
                obj.valid = false;
            }
            return obj;
        }

        /*
         * Checks the validity of the regex and returns Boolean true (valid) or false (not valid).
         */
        function validRegex(item) {
            var obj = isValidRegex(item);
            return obj.valid;
        }

        function handleError(jqXHR, textStatus, errorThrown) {
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var error = data.result[0].message;
                    console.log("The error is:" + error);
                    //PHD.hideLoadingIndicator(AddStorage);
                    ngDialog.open({
                        dialogType:'retry',
                        modelDialogId:'regex-dialog',
                        scope:$scope,
                        dialogMessage: error
                    });
                } else {
                    PHD.throwError(data);
                }
            } else {
                PHD.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        /*
         * Get the current list of regexes.  If returnAll is false, don't return
         * the uninitialized regex (the one that appears after the prior list is applied.
         */
        $scope.getRegex = function(returnAll) {
            var regex = [];
            for (var i = 0; i < $scope.regexList.length; i++) {
                var item = $scope.regexList[i];
                var obj = {};
                var skipEmpty = returnAll === false;
                if (skipEmpty && item.container.id < 0 && item.action.id < 0) {
                    // If returnAll is false, skip "empty" regexes.
                    continue;
                }
                obj.container_id = item.container.id;
                obj.action_id = item.action.id;
                obj.filter = item.filter;
                regex.push(obj);
            }
            return regex;
        };

        $rootScope.initRegex = function(isEdit) {
            $scope.regexList = [];
            var $tree = $("#jstreeDivId");
            var $controls = $("#protect-tree-controls");
            var $footer = $("#protect-treeview-footer-dialog");
            if (isEdit) {
                $tree.height(450);
                $controls.height(30);
                $footer.height(30);
            } else {
                $tree.height(400);
                $controls.height(30);
                $footer.height(30);
            }
        };

        $rootScope.loadRegex = function() {
            $rootScope.initRegex(true);
            if ($scope.joborder.regular_expressions !== undefined) {
                for (var i = 0; i < $scope.joborder.regular_expressions.length; i++) {
                    var short_regex = $scope.joborder.regular_expressions[i];
                    var container = _.findWhere($scope.containers, {"id": short_regex.id});
                    var action =  _.findWhere($scope.actions, {"id": short_regex.action});
                    var regex = {};
                    regex.container = container;
                    regex.action = action;
                    regex.filter = short_regex.filter;
                    // so that it will be disabled, part of schedule.
                    regex.isEdit = true;
                    console.log(regex);
                    $scope.addRegex(regex);
                }
            }
        };
}]);