/*
* calendarDemoApp - 0.9.0
*
* Unitrends iCal format:
* string(221) "BEGIN:VCALENDAR^M
 BEGIN:VEVENT^M
 SUMMARY:VMware Incremental^M
 DESCRIPTION:^M
 DTSTART:20160412T030000^M
 DTEND:20160412T040000^M
 RRULE:FREQ=WEEKLY;BYDAY=SU,MO,TU,WE,TH,FR,SA;BYHOUR=3,4,5,6,7,8,9,10,11^M
 END:VEVENT^M
 END:VCALENDAR^M

 string(323) "BEGIN:VCALENDAR^M
 BEGIN:VEVENT^M
 SUMMARY:SQL Full^M
 DESCRIPTION:^M
 DTSTART:20160601T030000^M
 DTEND:20160601T040000^M
 RRULE:FREQ=WEEKLY;BYDAY=SU^M
 END:VEVENT^M
 BEGIN:VEVENT^M
 SUMMARY:SQL Differential^M
 DESCRIPTION:^M
 DTSTART:20160601T030000^M
 DTEND:20160601T040000^M
 RRULE:FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR,SA^M
 END:VEVENT^M
 END:VCALENDAR^M

 *
 */
var calendarDemoApp = angular.module('custom_calendar', ['ui.calendar', 'ui.bootstrap', 'ngDialog', 'ngIcal']);

calendarDemoApp.directive('updating', function () {
    return {
        restrict: 'E',
        replace:true,
        template: '<span style="position:absolute;display:block;width:inherit;top:300px;left:500px"><p style="font-size:1.6em">Updating...</p><span class="loading" style="padding-left:20px"><i class="icon-restart icon-spin"></i></span></span>',
        link: function (scope, element, attr) {
            scope.$watch('updating', function (val) {
                if (val)
                    $(element).show();
                else
                    $(element).hide();
            });
        }
    }
});

calendarDemoApp.controller('CalendarCtrl',
    function($scope, $rootScope, $compile, $timeout, uiCalendarConfig, ngDialog, ical) {

        // Used to unbind the watched event on dialog close.
        var unbindHandler = null;

        $scope.joborder = $scope.ngDialogData.joborder;
        $scope.backupModes = $scope.ngDialogData.modes;
        $scope.calendarType = $scope.ngDialogData.calendarType;
        $scope.events = [];
        $scope.eventSources = [];
        $scope.icalEvents = [];
        $scope.isEdit = false;
        $scope.view = {};
        $scope.tzOffset = 0;
        $scope.calendarName = 'backupsCalendar';
        $scope.CALENDAR_MODIFIED_STRING = "** Please save the job to save calendar changes. **";

        $scope.editOpen = false;    // Prevents error on double-click of event.
        $scope.fromiCal = true;     // perform initial load from iCal.

        $scope.updating = false;    // set true to show load indicator

        /*
         * Called when the user clicks on an event.  If a repeated event, find the original to edit.
         */
        $scope.handleEventClick = function(event, jsEvent, view){
            $scope.isEdit = true;
            if (event.repeated) {
                event = event.originalEvent;
            }
            $scope.editEvent(event);
        };

        $scope.editEvent = function(event) {
            if (!$scope.editOpen) {
                $scope.editOpen = true;
                ngDialog.open({
                    template: 'app/protect/custom-schedule/event-edit.html',
                    scope: $scope,
                    data: {isEdit: $scope.isEdit, event: event},
                    overlay:true,
                    modelDialogId:'edit-event-dialog',
                    name:'edit-event-dialog',
                    ngDialogStyle:'width:320px',
                    preCloseCallback: function (value) {
                        $scope.editOpen = false;
                    }
                });
            }
        };

        /*
         * Called when the user drops a new event on the calendar from an external source.
         * Handles appliance's timezone offset for the default start time.
         */
        $scope.alertOnReceive = function(event) {
            $scope.isEdit = false;
            var tzAdd = -$scope.tzOffset;
            var start = event.start;
            start = moment(start).add(tzAdd, 'seconds');

            var eid = 'new_' + Math.random().toString();
            var newEvent = {
                id: eid,
                title: event.title,
                start: start.toDate(),
                //end: end.toDate(),
                color: event.color
            };
            $scope.droppedEvent = event;
            $scope.editEvent(newEvent);
        };

        /*
         * Called from the child controller when the user saves an event.
         * Updates the event arrays that are rendered (events, eventSources).
         */
        $scope.saveEvent = function(calEvent, isEdit, leaveDialogOpen) {
            var open = leaveDialogOpen || false;
            if (!open) {
                $scope.closeEditDialog();
            }
            $scope.updating = true;
            if (isEdit) {
                for (var i = 0; i < $scope.events.length; i++) {
                    var event = $scope.events[i];
                    if (event.id == calEvent.id) {
                        // Find the event and replace with the modified one.
                        $scope.events[i] = calEvent;
                        break;
                    }
                }
            } else {
                $scope.addEvent(calEvent);
            }
            $scope.eventSources.length = 0;
            $timeout(function() {
                $scope.updateEvents($scope.view);
            }, 0);
        };

        /* add custom event*/
        $scope.addEvent = function(event) {
            $scope.events.push(event);
        };
        /* remove event */
        $scope.remove = function(index) {
            $scope.events.splice(index,1);
        };
        /*
         * Called from the child/edit controller when the user deletes a backup.
         */
        $scope.removeEvents = function(id, calendar, deleteEvent) {
            if (calendar === undefined) {
                calendar = $scope.calendarName;
            }
            if (deleteEvent) {
                // Start from the end so as we remove repeated events the indices will be correct.
                for (var i = $scope.events.length - 1; i >= 0; i--) {
                    var event = $scope.events[i];
                    if (event.id == id ||
                        (event.originalEvent !== undefined && event.originalEvent.id == id)) {
                        $scope.remove(event.id);
                    }
                }
            }
            $timeout(function() {
                if(uiCalendarConfig.calendars[calendar]){
                    uiCalendarConfig.calendars[calendar].fullCalendar('removeEvents', id);
                }
            });
        };
        /* Render Tooltip */
        $scope.renderEvent = function( event, element, view ) {
            element.prop('title', event.description);
            $compile(element)($scope);
        };

        /*
         * Called from the calendar to render the events.  Set the view start/end iCal, then loads the events from iCal, or updates existing event arrays.
         */
        $scope.renderView = function(view, element) {
            $scope.view.startDate = moment(view.start).toDate();
            $scope.view.startiCal = new ical.Time.fromJSDate($scope.view.startDate);
            $scope.view.endDate = moment(view.end).toDate();
            $scope.view.endiCal = new ical.Time.fromJSDate($scope.view.endDate);

            $scope.eventSources.length = 0;
            if ($scope.fromiCal) {
                $scope.fromiCal = false;
                $scope.renderEvents($scope.view);
            } else {
                // Only show updating after ical load.
                $scope.updating = true;
                $timeout(function() {
                    $scope.updateEvents($scope.view);
                }, 0);
            }
        };

        /*
         * Called by the calendar to see if the event may be dropped at the target (dropinfo).
         */
        $scope.checkAllow = function(dropInfo, event) {
            var now = moment().add(-1, 'days').startOf('day');
            var valid = false;
            if (dropInfo.start.isSameOrAfter(now)) {
                valid = true;
            }
            return valid;
        };

        /*
         * Called by the calendar when an event is moved within it.
         */
        $scope.handleMove = function(event, delta, revertFunc, jsEvent, ui, view) {
            if (event.repeated) {
                // Don't let a repeated event move.
                revertFunc();
            } else {
                var tzAdd = -$scope.tzOffset;
                var start = event.start;
                start = moment(start).add(tzAdd, 'seconds');
                event.start = start.toDate();
                var end = moment(start).add(1, 'hours');
                event.end = end.toDate();
                //event.end = moment(start).add(1, 'hours');
                $scope.saveEvent(event, true, true);
            }
        };

        /*
         * Called by the calendar when a day is clicked, selecting that date and going to Day mode.
         */
        $scope.selectDay = function(date, jsEvent, view) {
            var calendar = $scope.calendarName;
            $timeout(function() {
                if(uiCalendarConfig.calendars[calendar]){
                    uiCalendarConfig.calendars[calendar].fullCalendar('gotoDate', date);
                    uiCalendarConfig.calendars[calendar].fullCalendar('changeView', 'basicDay');

                }
            });
        };

        $scope.afterRender = function() {
            if ($scope.eventSources.length > 0) {
                $scope.updating = false;
            }
        };

        /*
         * Calendar configuration object.
         * Note: Any functions here must be defined PRIOR to this object declaration.
         */
        $scope.uiConfig = {
            calendar:{
                height: 550,
                width: 650,
                droppable: true, // this allows things to be dropped onto the calendar
                header:{
                    left: 'title',
                    center: 'month basicDay',
                    right: 'today prev,next'
                },
                allDayDefault: false,
                defaultTimedEventDuration: '01:00:00',          // Show backups as 1 hour by default.
                forceEventDuration: true,
                nextDayThreshold: '00:00:00',
                eventStartEditable: true,
                eventLimit: true,
                navLinks: true,
                dayClick: $scope.selectDay,
                eventAllow: $scope.checkAllow,
                eventDrop: $scope.handleMove,
                eventClick: $scope.handleEventClick,
                eventReceive: $scope.alertOnReceive,
                eventRender: $scope.renderEvent,
                viewRender: $scope.renderView,
                eventAfterAllRender: $scope.afterRender
            }
        };

        /*
         * Parses the job's iCal string into an array of icalEvents.
         */
        $scope.createJobEvents = function() {
            $scope.events = [];
            // if previously saved, newiCal will be defined.
            // if a schedule exists, iCal will be defined.
            var iCal = $scope.joborder.newiCal !== undefined ? $scope.joborder.newiCal : $scope.joborder.ical;
            if (iCal !== undefined) {
                var jcal = ical.parse(iCal);
                var comp = new ical.Component(jcal);
                $scope.icalEvents = comp.getAllSubcomponents("vevent");
            }
        };


        /*
         * To handle monthly day of week recurrences on the calendar, we prepend with 1, i.e., 1SU, 1MO.
         * This will need to be removed before saving, using $scope.removeFirst.
         */
        $scope.addFirst = function(rrule) {
            if (rrule.parts !== undefined) {
                if (rrule.parts.BYDAY !== undefined) {
                    var dayArray = rrule.parts.BYDAY;
                    var day = dayArray[0];
                    if (day.charAt(0) != '1') {
                        day = "1" + day;
                    }
                    rrule.parts.BYDAY = [day];
                }
            }
        };

        $scope.createEventDescription = function(event, addEdit) {
            var desc = event.title;
            var addEditMessage = addEdit;
            var startTime = "";
            if (event.start !== undefined) {
                startTime = "at " + moment(event.start).format("hh:mm a");
            }
            desc += " Backup " + startTime;
            if (event.rrule !== undefined) {
                var frequency = event.rrule.freq;
                if (frequency !== undefined) {
                    desc += ", Recurring: " + frequency.toLowerCase();
                }
                var interval = event.rrule.interval;
                if (interval !== undefined && interval != 1) {
                    desc += ", Interval: " + interval;
                }
                var repeat = event.rrule.parts;
                if (repeat !== undefined) {
                    if (repeat.BYDAY !== undefined) {
                        desc += ", Day(s): " + repeat.BYDAY.toString();
                    }
                    if (repeat.BYHOUR !== undefined) {
                        desc += ", Hour(s): " + repeat.BYHOUR.toString();
                    }
                    if (repeat.BYMINUTE !== undefined) {
                        desc += ", Minute(s): " + repeat.BYMINUTE.toString();
                    }
                    if (repeat.BYMONTHDAY !== undefined) {
                        desc += ", Day of Month: " + repeat.BYMONTHDAY.toString();
                    }
                }
            }
            if (addEditMessage) {
                desc += "\n\nClick to Edit";
            }
            return desc;
        };

        /*
         * Render the loaded and parsed iCal on the calendar.
         */
        $scope.renderEvents = function(view) {
            var vevents = $scope.icalEvents;
            if (vevents.length == 0) {
                // timing - need to understand.
                // load if not loaded.
                $scope.initEvents();
                vevents = $scope.icalEvents;
                if (vevents.length === 0) {
                    return;
                }
            }
            var ec = 0;
            for (var i = 0; i < vevents.length; i++) {
                var vevent = vevents[i];
                var event = new ical.Event(vevent);
                var backupType = $scope.mapType(event.summary);
                var startDate = event.startDate.toJSDate();
                var endDate = event.endDate.toJSDate();
                console.log(event.summary);
                var originalEvent = {
                    id: ec++,
                    title: backupType,
                    start: startDate,
                    end: endDate,
                    color: $scope.mapColor(backupType),
                    repeated: false
                };
                var rules = vevent.getAllProperties('rrule');
                var dtstart = vevent.getFirstProperty('dtstart');
                dtstart = dtstart.getFirstValue();
                if (rules !== undefined && rules.length > 0) {
                    var newEvents = [];
                    var rrule = {};
                    for (var j = 0; j < rules.length; j++) {
                        var rule = rules[j].getFirstValue();
                        rrule = rule;
                        // fixup bad rule
                        if (rrule.freq == 'MONTHLY') {
                            $scope.addFirst(rrule);
                        }
                        originalEvent.rrule = rrule;
                        originalEvent.description = $scope.createEventDescription(originalEvent, true);
                        newEvents.push(originalEvent);
                        if ( _.isFunction(rule.iterator)) {
                            var iter = rule.iterator(dtstart);
                            for (var next = iter.next(); next; next = iter.next()) {
                                // skip if same date as dtstart.
                                // skip values outside the view range.
                                if (next.compare(view.startiCal) < 0 || next.compare(dtstart) == 0){
                                    continue;
                                }
                                if (next.compare(view.endiCal) > 0) {
                                    break;
                                }
                                var unixTime = next.toUnixTime();
                                var hr = unixTime + 3600;
                                var milli = hr * 1000;
                                var start = next.toJSDate();
                                var event_end = new Date(+start + 3600000);
                                var eventObj = {
                                    id: ec++,
                                    title: backupType,
                                    start: start,
                                    end: event_end,
                                    color: $scope.mapColor(backupType),
                                    repeated: true,
                                    description: originalEvent.description,
                                    originalEvent: originalEvent
                                };
                                newEvents.push(eventObj);
                            }
                        }
                    }
                    Array.prototype.push.apply($scope.events, newEvents);
                } else {
                    originalEvent.description = $scope.createEventDescription(originalEvent, true);
                    $scope.events.push(originalEvent);
                }
            }
            $scope.eventSources.push($scope.events);
        };

        $scope.updateEvents = function(view) {

            var newEvents = [];
            var ec = 0;
            for (var i = 0; i < $scope.events.length; i++) {
                var event = $scope.events[i];
                if (event.repeated) {
                    continue;
                }
                event.id = ec++;
                event.description = $scope.createEventDescription(event, true);
                newEvents.push(event);
                if (event.rrule !== undefined) {
                    var rule = $scope.rruleToiCal(event.rrule);
                    // fixup bad rule
                    if (rule.freq == 'MONTHLY') {
                        $scope.addFirst(rule);
                    }
                    var startDate = $.fullCalendar.moment(event.start).toDate();
                    var dtstart = new ical.Time.fromJSDate(startDate);
                    var backupType = event.title;
                    if ( _.isFunction(rule.iterator)) {
                        var iter = rule.iterator(dtstart);
                        for (var next = iter.next(); next; next = iter.next()) {
                            // skip first repeat as it is same as original.
                            // skip values outside the view range.
                            if (next.compare(view.startiCal) < 0 || next.compare(dtstart) == 0){
                                continue;
                            }
                            if (next.compare(view.endiCal) > 0) {
                                break;
                            }
                            var unixTime = next.toUnixTime();
                            var hr = unixTime + 3600;
                            var milli = hr * 1000;
                            var start = next.toJSDate();
                            //var event_end = new Date(start.getYear(), start.getMonth(), start.getDate(), start.getHours() + 1, start.getMinutes(), 0);
                            var event_end = new Date(+start + 3600000);
                            //var event_end = new Date(milli);
                            var eventObj = {
                                id: ec++,
                                title: backupType,
                                start: start,
                                end: event_end,
                                color: $scope.mapColor(backupType),
                                repeated: true,
                                description: event.description,
                                originalEvent: event
                            };
                            //$scope.events.push(eventObj);
                            newEvents.push(eventObj);
                        }
                    }
                }
            }
            $scope.events = newEvents;
            $scope.eventSources.push($scope.events);
        };

        /*
         * Map the backup type from iCal into a generic, not application-specific, type.
         */
        $scope.mapType = function(type) {
            var mapped = type;
            var map = [ 'Full', 'Differential', 'Incremental', 'Transaction', 'Selective' ];
            if (type == 'Master') {
                mapped = 'Full';
            } else if (type == 'BareMetal') {
                mapped = 'Bare Metal';
            } else if (type == 'Archive') {
                mapped = 'Backup Copy';
            } else {
                for (var i = 0; i < map.length; i++) {
                    var tryMatch = map[i];
                    if (type.indexOf(tryMatch) !== -1) {
                        mapped = tryMatch;
                        break;
                    }
                }
            }
            return mapped;
        };

        /*
         * Map the backup type to a color.
         */
        $scope.mapColor = function(type) {
            var color = "#000000";
            switch (type) {
                case 'Full':
                case 'Backup Copy':
                    color = '#083548';
                    break;
                case 'Differential':
                    color = '#006400';
                    break;
                case 'Incremental':
                    color = '#19A347';
                    break;
                case 'Transaction':
                    color = '#91765d';
                    break;
                case 'Selective':
                    color = '#91765d';
                    break;
                case 'Bare Metal':
                    color = '#9a9a9a';
                    break;
            }
            return color;
        };

        $scope.initEvents = function() {

            var resp = PHD.Ajax.get("/api/date-time/?sid=" + $rootScope.local.id);
            /* Set appliance date time data */
            resp.done(function (data) {
                if (data !== undefined) {
                    var d = new Date();
                    $scope.tzOffset = data.tzOffset;
                }
            });

            $scope.createJobEvents();

        };

        $scope.selectedItem = [];
        $scope.savedItem = null;
        $scope.droppedEvent = null;
        $scope.closeEditDialog = function(saved) {
            // not saving, redraw
            if (!saved) {
                if ($scope.droppedEvent && $scope.droppedEvent.id !== undefined) {
                    $scope.removeEvents($scope.droppedEvent.id, $scope.calendarName, false);
                    $scope.droppedEvent = null;
                }
            }
            ngDialog.close('edit-event-dialog');
        };

        $scope.closeCalendarDialog = function() {
            if ( _.isFunction(unbindHandler)) {
                unbindHandler();
            }
            $scope.events.splice(0, $scope.events.length);
            $scope.eventSources.splice(0, $scope.eventSources.length);
            $scope.icalEvents = [];
            ngDialog.close('edit-custom-dialog');
        };

        $scope.makeModesDraggable = function() {
            $('#external-events .fc-event').each(function () {

                // store data so the calendar knows to render an event upon drop
                $(this).data('event', {
                    id: Math.random(),
                    title: $.trim($(this).text()), // use the element's text as the event title
                    stick: false, // add to $scope.events, so we can delete on cancel.
                    repeated: false,
                    start: "03:00",
                    duration: "01:00:00",
                    color: $scope.mapColor($.trim($(this).text()))
                });

                // make the event draggable using jQuery UI
                $(this).draggable({
                    zIndex: 4000,
                    revert: true,      // will cause the event to go back to its
                    revertDuration: 0  //  original position after the drag
                });

            });
        };

        $scope.showBackupsTooltip = function(){
            var form = angular.element("#ui-calendar-form");
            form.find(".definition").popover({
                appendTo: form
            });
        };

        // Initialize the controller, returning the function to turn off the listener.
        unbindHandler = $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

        $scope.initCtrl = function(dlg){
            switch (dlg.name){
                case 'edit-custom-dialog':
                     // move up the dialog(the default ngDialog top is 25%).
                    $("#edit-custom-dialog").css('top', '15%');
                    // set modes
                    if ($scope.joborder.app_id == APP_ID_FILE_LEVEL) {
                        // Remove selective, Add BareMetal
                        for (var index = 0; index < $scope.backupModes.length; index++) {
                            if ($scope.backupModes[index] == 'Selective') {
                                $scope.backupModes.splice(index, 1);
                            }
                        }
                    }
                    $scope.showBackupsTooltip();
                    $timeout(function() {
                        $scope.makeModesDraggable();
                    });
                    // callback of render view should call this.
                    //$scope.initEvents();
                    break;
                default:
                    // No action.
                    break;
            }
        };

        $scope.cancelDialog = function(){
            if ($scope.$parent !== undefined && $scope.$parent.$parent !== undefined && $scope.$parent.$parent.backupModesforJob !== undefined) {
                $scope.$parent.$parent.backupModesforJob.name = $scope.$parent.$parent.backupModesforJob.original;
                $scope.$parent.$parent.applyModelChanges();
            } else {
                $(document).trigger("cancelCustomCalendar");
            }
            $scope.closeCalendarDialog();
        };

        $scope.createCalendarDescription = function() {
            var events = $scope.eventSources[0];
            var description = "";
            var arr = [];
            if ((events !== undefined) && (events instanceof Array) && (events.length > 0)) {
                for (var i = 0; i < events.length; i++) {
                    var event = events[i];
                    if (event.repeated) {
                        // skip repeated events.
                        continue;
                    }
                    arr.push($scope.createEventDescription(event, false));
                }
            }
            return arr.toString();
        };

        $scope.saveAndClose = function() {
            var iCal = $scope.eventsToiCal();
            var description;
            if (iCal !== "") {
                description = $scope.createCalendarDescription() + "\n\n" + $scope.CALENDAR_MODIFIED_STRING;
                if ($scope.$parent !== undefined && $scope.$parent.$parent !== undefined && $scope.$parent.$parent.joborder !== undefined) {
                    $scope.$parent.$parent.joborder.newiCal = iCal;
                    $scope.$parent.$parent.calendar_description = description;
                } else {
                    $(document).trigger("saveCustomCalendar", [iCal, description]);
                }
                $scope.closeCalendarDialog();
            } else {
                ngDialog.open({
                    dialogType:'Retry',
                    dialogMessage: 'Calendars must contain at least one backup event.'
                });
            }
        };

        /*
         * To handle monthly day of week recurrences on the calendar, we prepend with 1, i.e., 1SU, 1MO.
         * in the function $scope.addFirst.
         *
         * Before saving the calendar, we must convert back to SU, MO so that the core API will handle it.
         */
        $scope.removeFirst = function (rrule) {
            if (rrule.parts.BYDAY !== undefined) {
                var dayArray = rrule.parts.BYDAY;
                if (dayArray.length > 0) {
                    var day = dayArray[0];
                    if (day.charAt(0) == '1') {
                        day = day.slice(1);
                        dayArray[0] = day;
                    }
                }
                rrule.parts.BYDAY = dayArray;
            }
            /* clean up this event, rrule for monthly by day */
        };

        $scope.eventsToiCal = function() {
            var events = $scope.eventSources[0];
            var iCal = "";
            if ((events !== undefined) && (events instanceof Array) && (events.length > 0)) {
                iCal = "BEGIN:VCALENDAR";
                var sep = "\r\n";
                for (var i = 0; i < events.length; i++) {
                    var event = events[i];
                    if (event.repeated) {
                        // skip repeated events.
                        continue;
                    }
                    iCal += sep + "BEGIN:VEVENT";
                    iCal += sep + "DESCRIPTION:";
                    iCal += sep + "SUMMARY:" + $scope.getCoreBackupType(event.title, parseInt($scope.joborder.app_id));
                    var eStart = event.start;
                    var eEnd = event.end;
                    if (!(eStart instanceof Date)) {
                        eStart = moment(eStart).toDate();
                    }
                    if (!(eEnd instanceof Date)) {
                        eEnd = moment(eEnd).toDate();
                    }
                    var start = new ical.Time.fromJSDate(eStart);
                    var end = new ical.Time.fromJSDate(eEnd);
                    var startString = start.toString();
                    startString = startString.replace(/-/g, '');
                    startString = startString.replace(/:/g, '');
                    var endString = end.toString();
                    endString = endString.replace(/-/g, '');
                    endString = endString.replace(/:/g, '');
                    iCal += sep + "DTSTART:" + startString;
                    iCal += sep + "DTEND:" + endString;
                    if (event.rrule) {
                        if (event.rrule.freq == 'MONTHLY') {
                            $scope.removeFirst(event.rrule);
                        }
                        iCal += sep + "RRULE:" + $scope.rruleToiCal(event.rrule);
                    }
                    iCal += sep + "END:VEVENT";
                }
                iCal += sep + "END:VCALENDAR";
            }
            return iCal;
        };

        $scope.rruleToiCal = function(rule) {
            var rrule = rule; //rule.freq;
            return rrule;
        };

        $scope.getModeClass = function(mode) {
            mode = mode.split(' ').join('');
            return mode.toLowerCase();
        };


        $scope.getCoreBackupType = function(type, app_id) {
            var appType = type;
            if(app_id === APP_ID_FILE_LEVEL){
                if (type === 'Full') {
                    appType = "Master";
                } else if (type === 'Bare Metal') {
                    appType = 'BareMetal';
                }
            } else if(app_id === APP_ID_VMWARE){
                appType = "VMware " + type;
            } else if(app_id === APP_ID_XEN){
                appType = "Xen " + type;
            } else if(app_id === APP_ID_AHV){
                appType = "AHV " + type;
            } else if(app_id === APP_ID_HYPERV_2008_R2 || app_id === APP_ID_HYPERV_2012 || app_id === APP_ID_HYPERV_2016){
                appType = "Hyper-V " + type;
            } else if (app_id === APP_ID_EXCHANGE_2003 || app_id === APP_ID_EXCHANGE_2007 || app_id === APP_ID_EXCHANGE_2010
                || app_id === APP_ID_EXCHANGE_2013 || app_id === APP_ID_EXCHANGE_2016) {
                appType = "Exchange " + type;
            } else if (app_id === APP_ID_SQL_SERVER_2005 || app_id === APP_ID_SQL_SERVER_2008 || app_id === APP_ID_SQL_SERVER_2008_R2
                || app_id === APP_ID_SQL_SERVER_2012 || app_id === APP_ID_SQL_SERVER_2014 || app_id === APP_ID_SQL_SERVER_2016 || app_id === APP_ID_SQL_SERVER_2017) {
                appType = "SQL " + type;
            } else if (app_id === APP_ID_SHAREPOINT_2007 || app_id === APP_ID_SHAREPOINT_2010 || app_id === APP_ID_SHAREPOINT_2013 || app_id === APP_ID_SHAREPOINT_2016) {
                appType = "SharePoint " + type;
            } else if (app_id === APP_ID_ORACLE_11 || app_id === APP_ID_ORACLE_12 || app_id === APP_ID_ORACLE_10) {
                appType = "Oracle " + type;
            } else if(app_id === APP_ID_CISCO_UCS_SERVICE_PROFILE){
                appType = "UCS Service Profile " + type;
            } else if (app_id === APP_ID_NDMP) {
                appType = "NDMP " + type;
            } else if (app_id === APP_ID_ARCHIVE) {
                appType = "Archive";
            }
            return appType;
        };
    });

calendarDemoApp.controller('CalendarEditCtrl', [ '$scope', '$compile', '$timeout', 'ngDialog', 'ical', 'gettextCatalog',
    function($scope, $compile, $timeout, ngDialog, ical, gettextCatalog) {

        $scope.isEdit = $scope.ngDialogData.isEdit;
        $scope.calEvent = $scope.ngDialogData.event;

        $scope.recursType = [
            { id: 'None', name: 'None', iCal: '', value: 0},
            { id: 'Hourly', name: 'Hourly', iCal: 'DAILY', value: 1},
            { id: 'Daily', name: 'Daily', iCal: 'DAILY', value: 2},
            { id: 'Weekly', name: 'Weekly', iCal: 'WEEKLY', value: 3},
            { id: 'Monthly', name: 'Monthly', iCal: 'MONTHLY', value: 4}
        ];

        $scope.weekDays = [
            { id: 'Sunday', name: 'Sunday', abbrev: 'SUN', iCal: 'SU', value: 'SU'},
            { id: 'Monday', name: 'Monday', abbrev: 'MON', iCal: 'MO', value: 'MO'},
            { id: 'Tuesday', name: 'Tuesday', abbrev: 'TUE', iCal: 'TU', value: 'TU'},
            { id: 'Wednesday', name: 'Wednesday', abbrev: 'WED', iCal: 'WE', value: 'WE'},
            { id: 'Thursday', name: 'Thursday', abbrev: 'THU', iCal: 'TH', value: 'TH'},
            { id: 'Friday', name: 'Friday', abbrev: 'FRI', iCal: 'FR', value: 'FR'},
            { id: 'Saturday', name: 'Saturday', abbrev: 'SAT', iCal: 'SA', value: 'SA'}
        ];

        $scope.dayHours = [
            { name: '12 am',  value: 0 },
            { name: '1 am ',  value: 1 },
            { name: '2 am ',  value: 2 },
            { name: '3 am ',  value: 3 },
            { name: '4 am ',  value: 4 },
            { name: '5 am ',  value: 5 },
            { name: '6 am ',  value: 6 },
            { name: '7 am ',  value: 7 },
            { name: '8 am ',  value: 8 },
            { name: '9 am ',  value: 9 },
            { name: '10 am',  value: 10 },
            { name: '11 am',  value: 11 },
            { name: '12 pm',  value: 12 },
            { name: '1 pm ',  value: 13 },
            { name: '2 pm ',  value: 14 },
            { name: '3 pm ',  value: 15 },
            { name: '4 pm ',  value: 16 },
            { name: '5 pm ',  value: 17 },
            { name: '6 pm ',  value: 18 },
            { name: '7 pm ',  value: 19 },
            { name: '8 pm ',  value: 20 },
            { name: '9 pm ',  value: 21 },
            { name: '10 pm',  value: 22 },
            { name: '11 pm',  value: 23 }
        ];

        $scope.dayGroups = [$scope.dayHours.slice(0, 8), $scope.dayHours.slice(8, 16), $scope.dayHours.slice(16, 24)];

        $scope.eventEditTitle = gettextCatalog.getString(($scope.isEdit ? "Edit" : "Add") + ($scope.calendarType == "Backup Copy" ? " Backup Copy" : " Backup"));

        $scope.getRecurrence = function(recurrence, rrule, name) {
            var obj = {};
            var frequency, hasHours = false;
            if (rrule !== undefined) {
                if (rrule.freq !== undefined) {
                    frequency = rrule.freq;
                }
                if (rrule.parts !== undefined) {
                    if (rrule.parts.BYHOUR !== undefined) {
                        hasHours = true;
                    }
                }
            }

            // Handle weekly in a special way, as it is both weekly and hourly.
            if (frequency == 'DAILY') {
                obj = hasHours ? $scope.recursType[1] : $scope.recursType[2];
            } else {
                for (var i = 0; i < $scope.recursType.length; i++) {
                    var recur = $scope.recursType[i];
                    if (frequency == recur.iCal) {
                        obj = $scope.recursType[i];
                        break;
                    } else if (name !== undefined && recur.name == name) {
                        obj = $scope.recursType[i];
                        break;
                    }
                }
            }

            // Save existing recurrence settings, otherwise start new.
            if (recurrence !== undefined) {
                recurrence = (jQuery.extend({}, recurrence, obj));
            } else {
                recurrence = (jQuery.extend({}, obj));
            }
            return recurrence;
        };


        $scope.setWeeklyInterval = function(interval) {
            $scope.recurrence.interval = interval;
        };

        $scope.recursCheck = $scope.calEvent.rrule !== undefined && $scope.calEvent.rrule.freq !== undefined;
        if ($scope.recursCheck) {
            $scope.recurrence = $scope.getRecurrence($scope.recurrence, $scope.calEvent.rrule);
            if ($scope.recurrence.name === 'Weekly') {
                var interval = $scope.calEvent.rrule.interval !== undefined ? $scope.calEvent.rrule.interval : 1;
                $scope.setWeeklyInterval(interval);
            }
            if ($scope.calEvent.rrule.parts !== undefined) {
                if ($scope.recurrence.name === 'Monthly') {
                    $scope.recurrence.monthlyType = 'dow';
                    $scope.recurrence.monthDay = 1;
                    $scope.recurrence.byDay = ['SU'];
                }
                if ($scope.calEvent.rrule.parts.BYMONTHDAY !== undefined) {
                    var monthArray = $scope.calEvent.rrule.parts.BYMONTHDAY;
                    if (monthArray[0] == -1) {
                        $scope.recurrence.monthlyType = 'last';
                    } else {
                        $scope.recurrence.monthlyType = 'day';
                        $scope.recurrence.monthDay = monthArray[0];
                    }
                }
                if ($scope.calEvent.rrule.parts.BYDAY !== undefined) {
                    var dayArray = $scope.calEvent.rrule.parts.BYDAY;
                    if ($scope.recurrence.monthlyType == 'dow') {
                        var monthDay = dayArray[0];
                        if (monthDay.charAt(0) == '1') {
                            // If we are moving month-to-month, remove the 1 in front of the day,
                            // i.e, 1FR, 1SA, to get the day.
                            monthDay = monthDay.slice(1);
                        }
                        $scope.recurrence.byDay = [monthDay];
                    } else {
                        $scope.recurrence.byDay = dayArray;
                    }
                    if ($scope.recurrence.name === 'Daily') {
                        // The only daily types are every day, weekend days (Sat, Sun)
                        // or weekdays (Mon-fri).  Based on the length of the day array, we can determine which it is.
                        if (dayArray.length == 7) {
                            $scope.recurrence.dailyType = 'every';
                        } else if (dayArray.length == 2) {
                            $scope.recurrence.dailyType = 'weekend';
                        } else {
                            // default, handles also users who custom set with iCal.
                            $scope.recurrence.dailyType = 'weekday';
                        }
                    }
                }
                if ($scope.calEvent.rrule.parts.BYHOUR !== undefined) {
                    $scope.recurrence.byHour = $scope.calEvent.rrule.parts.BYHOUR;
                    if ($scope.recurrence.byHour.length > 0){
                        if ($scope.recurrence.byHour.length === 24) {
                            $scope.recurrence.hourlyType = 'every';
                        } else {
                            $scope.recurrence.hourlyType = 'selected';
                        }
                    }
                    // set recurrence.
                    $scope.recurrence.name = 'Hourly';
                }
                if ($scope.calEvent.rrule.parts.BYMINUTE !== undefined) {
                    var minutes = $scope.calEvent.rrule.parts.BYMINUTE;
                    if (minutes.length > 1) {
                        $scope.recurrence.isIntraHour = true;
                        // First value in the array is the minute of the backup time start
                        // Second value is the increment higher than the first.
                        // e.g., 0, 15, 30, 45, the increment is 15.  For 0, 20, 40, it is 20.
                        $scope.recurrence.minutes = minutes[1] - minutes[0];
                    }
                    // set recurrence.
                    $scope.recurrence.name = 'Hourly';
                }
            }
        } else {
            // Initialize to no recurrences.
            $scope.recurrence = $scope.getRecurrence(undefined, undefined, "None");
        }

        /*
         * Called when type changes, initialize recurrence to None unless Incremental.
         * If incremental, set recurrence to every day.
         */
        $scope.onBackupTypeChange = function(type) {
            if (type == 'Incremental') {
                $scope.recurstimeModeChange('Weekly');
                $scope.recursCheck = true;
            } else {
                $scope.recurrence = $scope.getRecurrence(undefined, undefined, "None");
                $scope.recursCheck = false;
            }
        };

        $scope.recurstimeModeChange = function(mode){
            console.log("mode change:" + mode);
            $scope.recurrence = $scope.getRecurrence($scope.recurrence, undefined, mode);
            if (mode == 'Weekly') {
                if ($scope.recurrence.interval === undefined) {
                    $scope.setWeeklyInterval(1);
                }
                if ($scope.recurrence.byDay === undefined) {
                    $scope.recurrence.byDay = ['SU'];
                }
            } else if (mode == 'Daily') {
                $scope.recurrence.byDay = _.pluck($scope.weekDays, 'value');
                $scope.recurrence.dailyType = 'every';
            } else if (mode == 'Hourly') {
                if ($scope.recurrence.hourlyType === undefined) {
                    $scope.recurrence.hourlyType = 'selected';
                    $scope.recurrence.byHour = [6, 12, 18];
                }
                if ($scope.recurrence.byDay === undefined) {
                    $scope.recurrence.byDay = ['SU'];
                }
                if ($scope.recurrence.minutes === undefined) {
                    $scope.recurrence.minutes = 15;
                }
            } else if (mode == 'Monthly') {
                if ($scope.recurrence.monthlyType === undefined) {
                    $scope.recurrence.monthlyType = 'day';
                    $scope.recurrence.monthDay = 1;
                    $scope.recurrence.byDay = ['SU'];
                }
            }
        };

        var day = $scope.calEvent.start;
        $scope.calEvent.start_date = moment(day).format(DATE_DISPLAY_FORMAT);
        $scope.calEvent.start_time = moment(day).format("hh:mm a");

        $scope.checkRepeatRule = function(rule) {
            var error = false;
            if (rule.freq == 'DAILY') {
                if (rule.parts.BYDAY === undefined) {
                    error = "You must select at least one day";
                }
            }
            return error;
        };

        $scope.saveEvent = function() {
            var error = false;
            var m = $.fullCalendar.moment($scope.calEvent.start_date + ' ' + $scope.calEvent.start_time, DATE_DISPLAY_FORMAT + ' hh:mm a');
            $scope.calEvent.start = m;
            $scope.calEvent.end = $.fullCalendar.moment(m).add(1, 'hours');
            $scope.calEvent.color = $scope.mapColor($scope.calEvent.title);
            var rrule = {};
            if ($scope.recurrence.name === 'None') {
                delete $scope.calEvent.rrule;
            } else {
                var rrule = {};
                rrule.freq = $scope.recurrence.iCal;
                rrule.parts = {};
                if ($scope.recurrence.name === 'Weekly') {
                    if ($scope.recurrence.interval !== undefined) {
                        rrule.interval = parseInt($scope.recurrence.interval);
                    }
                }
                if ($scope.recurrence.name === 'Monthly') {
                    if ($scope.recurrence.monthlyType !== undefined) {
                        if ($scope.recurrence.monthlyType == 'last') {
                            rrule.parts.BYMONTHDAY = [-1];
                            delete $scope.recurrence.byDay;
                        } else if ($scope.recurrence.monthlyType == 'day') {
                            rrule.parts.BYMONTHDAY = [$scope.recurrence.monthDay];
                            delete $scope.recurrence.byDay;
                        }
                    }
                }
                if ($scope.recurrence.byDay !== undefined && $scope.recurrence.byDay.length > 0) {
                    if ($scope.recurrence.name === 'Daily') {
                        if ($scope.recurrence.dailyType == 'every') {
                            rrule.parts.BYDAY = _.pluck($scope.weekDays, 'value');
                        } else if ($scope.recurrence.dailyType == 'weekday') {
                            rrule.parts.BYDAY = ['MO', 'TU', 'WE', 'TH', 'FR'];
                        } else if ($scope.recurrence.dailyType == 'weekend') {
                            rrule.parts.BYDAY = ['SU', 'SA'];
                        }
                    } else {
                        rrule.parts.BYDAY = $scope.recurrence.byDay;
                    }
                }
                if ($scope.recurrence.name === 'Hourly') {
                    if ($scope.recurrence.hourlyType == 'every') {
                        $scope.recurrence.byHour = _.pluck($scope.dayHours, 'value');
                    }
                    if ($scope.recurrence.byHour !== undefined && $scope.recurrence.byHour.length > 0) {
                        rrule.parts.BYHOUR = $scope.recurrence.byHour;
                    }
                    if ($scope.recurrence.isIntraHour && $scope.recurrence.minutes > 0) {
                        var minutes = [];
                        var increment = parseInt($scope.recurrence.minutes);
                        for (var i = 0; i < 60; i += increment) {
                            minutes.push(i);
                        }
                        rrule.parts.BYMINUTE = minutes;
                    }
                }
                error = $scope.checkRepeatRule(rrule);
                $scope.calEvent.rrule = new ical.Recur(rrule, rrule.freq);
            }
            if (error === false) {
                console.log('save event');
                $scope.$parent.$parent.saveEvent($scope.calEvent, $scope.isEdit, false);
            } else {
                // If the combination is not valid, flag the user and let them retry.
                ngDialog.open({
                    dialogType:'Retry',
                    dialogMessage:error
                });
            }
        };

        $scope.confirmDelete = function() {
            var itemName = $scope.calEvent.title;
            var eStartDate = moment($scope.calEvent.start).format("MM/DD/YYYY");
            var eStartTime = moment($scope.calEvent.start).format("hh:mm a");
            if (itemName !== undefined) {
                $scope.deleteTitle = 'Confirm Backup Removal';
                $scope.confirmMessage1 = 'Are you sure you want to remove the ' + itemName + ' backup that starts on ' +
                                        eStartDate + ' at ' + eStartTime + '?';
                $scope.confirmMessage2 = '';
                $scope.confirmMessage3 = 'I understand that backups will have to be re-added if deleted.';
                $scope.submitButtonText = 'Delete Backup';

                ngDialog.open({
                    template: 'app/common/confirmationDialog.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId: 'delete-backup-dialog',
                    ngDialogStyle:'width:600px;height:200px',
                    closeByDocument: false,
                    closeByEscape: true
                });
            }
        };

        $scope.deleteAsset = function() {
            $scope.$parent.$parent.removeEvents($scope.calEvent.id, undefined, true);
            ngDialog.close('delete-backup-dialog');
            ngDialog.close('edit-event-dialog');
        };

    }
    ]);