angular.module('groups-module', ['ngDialog', 'ui.grid','ui.grid.resizeColumns','ui.grid.autoResize','ngGrid','ui.grid.selection'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('GroupsCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants', 'inventorysvc', 'gettextCatalog',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants, inventorysvc, gettextCatalog) {

            $scope.editDialog = null;
            $scope.selectedItem = null;
            $scope.savedItem = null;  // save item to restore on cancel
            $scope.refresh = false;
            $scope.default = {};

            $scope.saveSettings = function() {
                var groups = $scope.groupsGridData.data;
                $scope.saveGroups(groups, 'edit');
            };

            function convertToDecimal(colorString) {
                var value = -1;
                if (colorString !== "-1") {
                    // get rid of the '#' in front before converting to decimal.
                    value = parseInt(colorString.substring(1), 16);
                }
                return value;
            }
            
            function compareColors(colorString1, colorString2) {
                return (convertToDecimal(colorString1) == convertToDecimal(colorString2));
            }
            
            $scope.saveGroups = function(groups, op) {
                var url = "/api/inventory/nav-groups";
                var operation = op || null;
                var load = PHD.showLoadingIndicator("body", true);
                var groupObj = { 'group': groups };
                groupObj.defaultColor = convertToDecimal($scope.default.color);
                groupObj.defaultUsers = $scope.default.users;
                var navgroup = { 'navgroup': groupObj };
                var payload = JSON.stringify(navgroup);
                var resp = PHD.Ajax.put(url, payload, load, handleError);
                resp.done(function() {
                    if (operation == 'edit') {
                        $scope.closeEditDialog(true);
                    } else if (operation == 'delete') {
                        $scope.closeThisDialog();
                    }
                    $scope.reloadGrid();
                });
            };

            function handleError(jqXHR, textStatus, errorThrown) {
                if (jqXHR.status === 500) {
                    var data = jqXHR.responseJSON;
                    if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                        var message = data.result[0].message;
                        ngDialog.open({
                            dialogType: gettextCatalog.getString('Error'),
                            modelDialogId: 'edit-groups-error-dialog',
                            scope: $scope,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    }
                } else {
                    PHD.ajaxError(jqXHR, textStatus, errorThrown);
                }
            }

            $scope.onConfirmError = function () {
                ngDialog.close('edit-groups-error-dialog');
            };

            $scope.closeEditDialog = function(saved) {
                if (!saved) {
                    $timeout(function() {
                        $scope.selectedItem = angular.copy($scope.savedItem);
                    }, 0);
                }
                ngDialog.close('edit-group-dialog');
            };

            $scope.closeGroupsDialog = function() {
                // clear any validation errors for the form before closing.
                $("#form-errors").hide();
                ngDialog.close('groups-manager-dialog');
            };

            $scope.editGroup = function(mode) {
                $scope.isEdit = mode;
                var siblings = [];
                var children = [];
                var users = [];

                if ($scope.isEdit) {
                    if ($scope.selectedItem == null) {
                        return;
                    }
                    $scope.groupsEditTitle = gettextCatalog.getString('Edit Group');
                    // Save the original to restore on cancel.
                    $scope.savedItem = $scope.selectedItem;
                    $scope.selectedItem = angular.copy($scope.selectedItem);
                    var selectedID = inventorysvc.getGroupID($scope.selectedItem);
                    siblings = inventorysvc.getSiblings(selectedID, $rootScope.treeDataCache);
                    children = inventorysvc.getChildren(selectedID, $rootScope.treeDataCache);
                } else {
                    var inv_root = $rootScope.local.id;
                    if ($scope.selectedItem === null) {
                        // If no item is selected, build a top-level group for the local appliance.
                        if ($scope.preSelectedInventoryID === undefined || $scope.preSelectedInventoryID === null || !canBeGrouped($scope.preSelectedInventoryID)) {
                            $scope.groupsEditTitle = gettextCatalog.getString('Add Top-Level Group');
                            $scope.selectedItem = inventorysvc.createItem(undefined, '', inv_root, $scope.default.color);
                            siblings = inventorysvc.getSiblings(inv_root, $rootScope.treeDataCache);
                        } else {
                            // If an item is selected, and we can build a group here, do so.
                            // Otherwise, build a top-level group for the selected appliance.
                            siblings = inventorysvc.getChildren($scope.preSelectedInventoryID, $rootScope.treeDataCache);
                            if (siblings === undefined || siblings.length == 0) {
                                var node = inventorysvc.nodeToObject($scope.preSelectedInventoryID);
                                var appliance_id = node.system_id !== undefined ? node.system_id : inv_root;
                                $scope.selectedItem = inventorysvc.createItem(undefined, '', appliance_id, $scope.default.color);
                                $scope.groupsEditTitle = gettextCatalog.getString('Add Top-Level Group');
                                siblings = inventorysvc.getSiblings(appliance_id, $rootScope.treeDataCache);
                            } else {
                                var parentID = inventorysvc.convertInvID($scope.preSelectedInventoryID);
                                var itemName = inventorysvc.getNodeName($scope.preSelectedInventoryID, $rootScope.treeDataCache);
                                $scope.selectedItem = inventorysvc.createItem(undefined, '', parentID, $scope.default.color);
                                $scope.groupsEditTitle = gettextCatalog.getString('Add Group For Item ') + itemName;
                            }
                        }
                    } else {
                        $scope.savedItem = $scope.selectedItem;
                        // Initializea new group based on previous group.
                        var groupName = inventorysvc.getGroupName($scope.selectedItem);
                        var groupID = inventorysvc.getGroupID($scope.selectedItem);
                        $scope.groupsEditTitle = gettextCatalog.getString('Add Inner Group');
                        $scope.selectedItem = inventorysvc.createItem(undefined, groupName + '-inner', groupID, $scope.default.color);
                        siblings = inventorysvc.getChildren($scope.preSelectedInventoryID, $rootScope.treeDataCache);
                    }
                }
                $scope.isDefaultColor = compareColors($scope.default.color, inventorysvc.getGroupColor($scope.selectedItem));
                
                if ($scope.editDialog == null) {
                    ngDialog.open({
                        template: 'app/protect/groups/groups-edit.html',
                        scope: $scope,
                        data: {'users': users, 'siblings': siblings, 'children': children},
                        overlay:true,
                        modelDialogId:'edit-group-dialog',
                        name: 'edit-group-dialog',
                        ngDialogStyle:'width:600px;height:550px;',
                        preCloseCallback: function (value) {
                            // clear so dialog will open next time.
                            $scope.editDialog = null;
                        },
                        closeByEscape: false
                    });
                    $scope.editDialog = true;
                }
            };

            $scope.confirmDelete = function() {
                var itemName = inventorysvc.getGroupName($scope.selectedItem);
                if (itemName !== undefined) {
                    $scope.deleteTitle = gettextCatalog.getString('Confirm Group Removal');
                    $scope.confirmMessage1 = gettextCatalog.getString('Are you sure you want to remove the selected group?');
                    $scope.confirmMessage2 = gettextCatalog.getString('');
                    $scope.confirmMessage3 = gettextCatalog.getString('I understand that groups will have to be re-created once deleted.');
                    $scope.submitButtonText = gettextCatalog.getString('Delete Group');

                    ngDialog.open({
                        template: 'app/common/confirmationDialog.html',
                        scope: $scope,
                        overlay:true,
                        modelDialogId: 'delete-group-dialog',
                        ngDialogStyle:'width:600px;height:250px',
                        closeByDocument: false,
                        closeByEscape: true
                    });
                } else {
                    var message = gettextCatalog.getString('Cannot determine which item is selected, please re-select and try again');
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'edit-groups-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                }
            };

            // Required name for common delete function.
            $scope.closeThisDialog = function() {
                ngDialog.close("delete-group-dialog");
            };

            $scope.deleteAsset = function() {
                var itemID = inventorysvc.getGroupID($scope.selectedItem);
                if (itemID !== undefined) {
                    var groups = $scope.groupsGridData.data;
                    var groupParentID;
                    var removed = false;
                    var childrenToMove = [];
                    for (var i = 0; i < groups.length; i++) {
                        var groupID = inventorysvc.getGroupID(groups[i]);
                        if (groupID === itemID) {
                            // Found it.  Get its Parent ID.
                            groupParentID = inventorysvc.getGroupParent(groups[i]);
                            var parentIsGroup = inventorysvc.isGroup(groupParentID);
                            if (parentIsGroup) {
                                // The parent ID is a group. Move members to parent group.
                                childrenToMove = groups[i]['childIDs'] !== undefined ? groups[i]['childIDs']['child'] : null;
                            }
                            // Delete it from the groups.
                            groups.splice(i, 1);
                            // and remove it as a child from any groups.
                            inventorysvc.removeFromGroup({id:groupID, isExact:true}, groups);
                            removed = true;
                            break;
                        }
                    }
                    // Find any group that has as its parent the group being deleted.
                    // Set parent to be group's parent.
                    if (groupParentID !== undefined && removed === true) {
                        for (i = 0; i < groups.length; i++) {
                            var parentID = inventorysvc.getGroupParent(groups[i]);
                            if (itemID === parentID) {
                                inventorysvc.setGroupParent(groups[i], groupParentID);
                            }
                        }
                    }
                    // Find the group that is the parent of the group being deleted.
                    // Move the deleted group's children to the parent group.
                    if (parentIsGroup && childrenToMove.length > 0) {
                        for (i = 0; i < groups.length; i++) {
                            groupID = inventorysvc.getGroupID(groups[i]);
                            if (groupID === groupParentID) {
                                var new_children = groups[i]['childIDs'] && groups[i]['childIDs']['child'] !== undefined ? groups[i]['childIDs']['child'] : null;
                                if (new_children !== null) {
                                    new_children = new_children.concat(childrenToMove);
                                } else {
                                    new_children = childrenToMove;
                                }
                                groups[i]['childIDs'] = {'child': new_children};
                            }
                        }
                    }
                }
                $scope.saveGroups(groups, 'delete');
            };

            $scope.reloadGrid = function() {
                var url = "/api/inventory/nav-groups";
                var resp = PHD.Ajax.get(url);
                resp.done(function(data) {
                    $scope.selectedItem = null;
                    $scope.savedItem = null;
                    $scope.refresh = true;
                    $scope.gridData = data.navgroup !== undefined && data.navgroup.group !== undefined ? data.navgroup.group : [];
                    // A single group will be an object, so wrap with an array in this case.
                    if (!($scope.gridData instanceof Array)) {
                        $scope.gridData = [$scope.gridData];
                    }
                    $scope.groupsGridData.data = $scope.gridData;
                    $timeout(function() {
                        $scope.refresh = false;
                    }, 0);
                    // Reload gropus in protect tree as they have changed.
                    var e = $.Event('changeNavGroups');
                    $(document).trigger(e);
                });
            };

            $scope.groupsGridData = {
                multiSelect: false,
                selectionRowHeaderWidth: 35,
                rowHeight: 30,
                enableFullRowSelection : true,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                onRegisterApi: function (gridApi) {
                    $scope.gridApi = gridApi;
                    gridApi.selection.on.rowSelectionChanged($scope, function(rows){
                        var selectedItems = gridApi.selection.getSelectedRows();
                        if (selectedItems.length > 0) {
                            $scope.selectedItem = selectedItems[0];
                        } else if (selectedItems.length == 0) {
                            $scope.selectedItem = null;
                        }
                    });
                },
                data: []
            };

            $scope.groupsGridData.columnDefs = [
                {
                    field: "@attributes['name']",
                    headerCellFilter: "translate",
                    displayName: gettextCatalog.getString("Group Name"),
                    type: "string",
                    sortable : true,
                    sort: {
                        direction: uiGridConstants.ASC,
                        priority: 0
                    }
                }
            ];

            $scope.loadSettings = function() {
                $timeout(function() {
                    $scope.gridData = $scope.ngDialogData !== undefined && $scope.ngDialogData.navgroup !== undefined && $scope.ngDialogData.navgroup.group !== undefined ? $scope.ngDialogData.navgroup.group : [];
                    if ($scope.ngDialogData !== undefined && $scope.ngDialogData.navgroup !== undefined) {
                        $scope.default.color = $scope.ngDialogData.navgroup.defaultColor;
                        if ($scope.default.color !== "-1") {
                            // convert from decimal string to color string (6 chars)
                            var color = parseInt($scope.default.color);
                            color = color.toString(16);
                            while (color.length < 6) {
                                color = "0" + color;
                            }
                            color = "#" + color;
                            $scope.default.color = color;
                        }
                        $scope.default.users = $scope.ngDialogData.navgroup.defaultUsers;
                    } else {
                        $scope.default.color = "-1";
                        $scope.default.users = [];
                    }
                    // A single group will be an object, so wrap with an array in this case.
                    if (!($scope.gridData instanceof Array)) {
                        $scope.gridData = [$scope.gridData];
                    }
                    $scope.groupsGridData.data = $scope.gridData;

                    // If a group was selected in the tree, pre-selected it here.
                    if ($scope.ngDialogData !== undefined && $scope.ngDialogData.preSelected !== undefined) {
                        $scope.preSelectedInventoryID = $scope.ngDialogData.preSelected;
                        var preSelectedID = $scope.preSelectedInventoryID;
                        if (preSelectedID !== undefined) {
                            $scope.gridApi.grid.modifyRows($scope.groupsGridData.data);
                            $scope.groupsGridData.data.forEach(function (row) {
                                if (inventorysvc.getGroupID(row) == preSelectedID) {
                                    $scope.gridApi.selection.selectRow(row);
                                }
                            });
                        }
                    }
                });
            };
            
            //
            // Return true if the node can be grouped, false otherwise.
            //
            function canBeGrouped(nodeID) {
                var canGroup = true;
                if (nodeID !== undefined) {
                    var node = inventorysvc.nodeToObject(nodeID);
                    if (node.app_id == APP_ID_FILE_LEVEL) {
                        canGroup = false;
                    }
                }
                return canGroup;
            }

            // use instead of ngDialog load event.
            angular.element(document).ready(function () {
                $scope.loadSettings();
            });

        }])
    .filter('icon_of', ['$protectService',
        function($protectService) {
        return function (type) {
            if (type instanceof Array) {
                type = type[0];
            }
            return $protectService.getIconForType(type);
        }
    }])
    .controller('GroupsEditCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants', 'inventorysvc', 'gettextCatalog',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants, inventorysvc, gettextCatalog) {

            $scope.tabSelected = 1;
            $scope.currentTab = 1;

            $scope.spectrumOptions = {
                "showInitial": true,
                "preferredFormat": "hex"
            };

            $scope.editForm = null;
            $scope.addToGroup = [];
            $scope.removeFromGroup = [];

            $scope.saveSettings = function() {
                if ($scope.editForm != null && $scope.editForm.valid()) {
                    if ($scope.isDefaultColor) {
                        $scope.default.color = inventorysvc.getGroupColor($scope.selectedItem);
                    }
                    if ($scope.updateDefaultUsers) {
                        $scope.default.users = inventorysvc.getUsersString($scope.selectedUsers);
                    }
                    // update groups Grid based on changees.
                    // addGroups($scope.addToGroup)
                    // removeChildren ($scope.removeFromGroups);
                    var itemID = inventorysvc.getGroupID($scope.selectedItem);
                    if (itemID === undefined) {
                        itemID = inventorysvc.createUUID();
                        inventorysvc.setGroupID($scope.selectedItem, itemID);
                        $scope.groupsGridData.data.push($scope.selectedItem);
                        // Also update the parent to add this new group as a child.
                        var parentID = inventorysvc.getGroupParent($scope.selectedItem);
                        inventorysvc.addChildToGroup(parentID, itemID, $scope.groupsGridData.data);
                    }
                    if (itemID !== undefined) {
                        var groups = $scope.groupsGridData.data;
                        for (var i = 0; i < groups.length; i++) {
                            var groupID = inventorysvc.getGroupID(groups[i]);
                            if (groupID === itemID) {
                                inventorysvc.setGroupName(groups[i], inventorysvc.getGroupName($scope.selectedItem));
                                inventorysvc.setGroupColor(groups[i], inventorysvc.getGroupColor($scope.selectedItem));
                                inventorysvc.setGroupUsers(groups[i], $scope.selectedUsers);
                                // set children for groups[i].
                                var children = groups[i]['childIDs'];
                                var childIDs = [];
                                if (children !== undefined) {
                                    if (children instanceof Array && children.length == 0) {
                                        childIDs = [];
                                    } else {
                                        childIDs = children['child'];
                                    }
                                }
                                if (!(childIDs instanceof Array)) {
                                    childIDs = [childIDs];
                                }
                                var new_children = [];
                                for (var j = 0; j < childIDs.length; j++) {
                                    var thisChild = childIDs[j];
                                    var found = false;
                                    for (var rem = 0; rem < $scope.removeFromGroup.length && !found; rem++) {
                                        var remove = $scope.removeFromGroup[rem];
                                        var removeID = inventorysvc.convertInvID(remove.id, true);
                                        if (removeID == thisChild) {
                                            found = true;
                                            // If a group is removed as a child, set its parent to be the parent of this group.
                                            if (inventorysvc.isGroup(removeID, '.')) {
                                                var movedGroup = inventorysvc.getGroup(removeID, groups);
                                                var parentID = inventorysvc.getGroupParent(groups[i]);
                                                inventorysvc.setGroupParent(movedGroup, parentID);
                                            }
                                        }
                                    }
                                    if (!found) {
                                        new_children.push(thisChild);
                                    }
                                }
                                for (j = 0; j < $scope.addToGroup.length; j++) {
                                    var invObj = $scope.addToGroup[j];
                                    var childID = inventorysvc.convertInvID(invObj.id, true);
                                    // remove from current group (if in one).
                                    inventorysvc.removeFromGroup({id:childID, isExact:true}, groups);
                                    new_children.push(childID);
                                    // If a group is added as a child, set its parent to this group.
                                    if (inventorysvc.isGroup(childID, '.')) {
                                        var movedGroup = inventorysvc.getGroup(childID, groups);
                                        inventorysvc.setGroupParent(movedGroup, itemID);
                                    }
                                }
                                if (new_children.length > 1) {
                                    groups[i]['childIDs'] = {'child': new_children};
                                } else if (new_children.length == 1) {
                                    groups[i]['childIDs'] = {'child': new_children[0]};
                                } else {
                                    groups[i]['childIDs'] = [];
                                }
                                break;
                            }
                        }
                    }
                    $scope.$parent.saveSettings();
                }
            };

            $scope.showCurrentTab = function(currentTab){
                $scope.currentTab = currentTab;
                $(window).resize();
            };

            var iconNameTemplate = '<div class="ui-grid-cell-contents"><i class="{{ row.entity.type | icon_of }} icon"></i>&nbsp;&nbsp;{{ row.entity.name }}</div>';

            $scope.notInGridData = {
                multiSelect: true,
                selectionRowHeaderWidth: 35,
                rowHeight: 30,
                enableFullRowSelection : true,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                onRegisterApi: function (gridApi) {
                    $scope.notInGridApi = gridApi;
                    gridApi.selection.on.rowSelectionChanged($scope, function(row){
                        $scope.addToGroup = gridApi.selection.getSelectedRows();
                    });
                    gridApi.selection.on.rowSelectionChangedBatch($scope, function (rows) {
                        $scope.addToGroup = gridApi.selection.getSelectedRows();
                    });
                },

                data: []
            };

            $scope.notInGridData.columnDefs = [
                {
                    field: "name",
                    cellTemplate: iconNameTemplate,
                    headerCellFilter: "translate",
                    displayName: gettextCatalog.getString("Name"),
                    sortable : true,
                    type : "string"
                }
            ];

            $scope.inGridData = jQuery.extend({}, $scope.notInGridData);
            $scope.inGridData.onRegisterApi = function(gridApi) {
                $scope.inGridApi = gridApi;
                gridApi.selection.on.rowSelectionChanged($scope, function(row){
                    $scope.removeFromGroup = gridApi.selection.getSelectedRows();
                });
                gridApi.selection.on.rowSelectionChangedBatch($scope, function (rows) {
                    $scope.removeFromGroup = gridApi.selection.getSelectedRows();
                });
            };

            $scope.notInGridData.data = $scope.ngDialogData.siblings;
            $scope.inGridData.data = $scope.ngDialogData.children;

            $scope.userGridData = {
                multiSelect: true,
                selectionRowHeaderWidth: 35,
                rowHeight: 30,
                enableFullRowSelection : true,
                enableSelectAll: false,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                onRegisterApi: function (gridApi) {
                    $scope.userGridApi = gridApi;
                    gridApi.selection.on.rowSelectionChanged($scope, function(rows){
                        $scope.userGridData.data.forEach(function (row, index) {
                            if (row.superuser == true || row.administrator == true) {
                                $scope.userGridApi.selection.selectRow(row);
                            }
                        });
                        var selectedItems = gridApi.selection.getSelectedRows();
                        $scope.selectedUsers = selectedItems;
                    });
                },
                data: [],
                columnDefs: [
                    {
                        field: "name",
                        headerCellFilter: "translate",
                        displayName: gettextCatalog.getString("User name"),
                        type : "string",
                        sortable : true,
                        sort: {
                            direction: uiGridConstants.ASC,
                            priority: 0
                        }
                    }
                ]
            };

            $scope.loadUsers = function() {
                var url = "/api/users";
                var resp = PHD.Ajax.get(url);
                resp.done(function(data) {
                    // Get the appliance user list.
                    var userList = data.data;
                    if (!$scope.isEdit) {
                        // On a new group, if we have default users. assign them to the group.{
                        inventorysvc.setGroupUsers($scope.selectedItem, $scope.default.users);
                    }
                    var groupUsers = inventorysvc.getGroupUsers($scope.selectedItem);
                    // Loop through group users and add nnn-numerics as AD users.
                    for ( i = 0; i < groupUsers.length; i++) {
                        if (isNaN(groupUsers[i])) {
                            userList.push({'id': groupUsers[i], 'name': groupUsers[i]});
                        }
                    }
                    $scope.refresh = true;
                    $scope.userGridData.data = userList;
                    $scope.userGridApi.grid.modifyRows($scope.userGridData.data);
                    $timeout(function() {
                        $scope.refresh = false;
                        $timeout(function() {
                            $scope.userGridData.data.forEach(function (row, index) {
                                // In addition, admins and superusers can edit all.
                                if (row.superuser == true || row.administrator == true || inventorysvc.userInGroup(row.id, $scope.selectedItem)) {
                                    $scope.userGridApi.selection.selectRow(row);
                                }
                            });
                        }, 0);
                    }, 0);
                });
            };

            $scope.validateForm = function(form) {
                groupNameValidator();
                form.validate({
                    errorLabelContainer: "#dialog-error-list",
                    errorContainer: "#dialog-errors",
                    showErrors: function(errorMap, errorList) {
                        $("#dialog-errors").find(".summary")
                            .html(gettextCatalog.getString("Correct the following errors:"));
                        this.defaultShowErrors();
                    },
                    rules: {
                        name: {
                            required: true,
                            maxlength: 64,
                            isGroupNameValid: true
                        }
                    },
                    messages: {
                        name: {
                            required: gettextCatalog.getString("Group name is required")
                        }
                    }
                })
            };

            function groupNameValidator(){
                //var re =  /^[a-zA-Z_0-9@!#\$^%&*()+=-[]\\\';,\.\/\{\}\|\":<>\? ]+$/i;
                $.validator.addMethod("isGroupNameValid", function(value, element) {
                    return (value.indexOf('<') == -1);
                    //return value.match(re);
                }, "'<' is not allowed");
            }
            
            $scope.addActiveDirectoryUser = function() {
                ngDialog.open({
                    template: 'app/protect/groups/groups-add-ad-user.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId: 'add-ad-user-dialog',
                    name: 'add-ad-user-dialog',
                    ngDialogStyle:'width:350px;height:180px',
                    closeByDocument: false,
                    closeByEscape: true
                });                
            };
            
            $scope.adUser = {};
            $scope.saveADuser = function() {
                if ($scope.userForm != undefined && $scope.userForm.valid()) {
                    $scope.adUser.id = $scope.adUser.name;
                    $scope.userGridData.data.push($scope.adUser);
                    $timeout(function() {
                        var lastIndex = $scope.userGridData.data.length - 1;
                        $scope.userGridApi.grid.modifyRows($scope.userGridData.data);
                        $scope.userGridApi.selection.selectRow($scope.userGridData.data[lastIndex]);
                    }, 10);
                    $scope.closeADDialog();
                }
            };
            
            $scope.closeADDialog = function() {
                ngDialog.close('add-ad-user-dialog');
                $scope.adUser = {};
            };

            $scope.validateADForm = function(form) {
                ADuserValidator();
                noDuplicatesValidator();
                var options = {
                    rules: {
                        adname: {
                            required: true,
                            isADNameValid: true,
                            isADNameUnique: true
                        }
                    },
                    messages: {
                        adname: {
                            required: gettextCatalog.getString('Username is required')
                        }
                    }
                };
                $rootScope.validateNestedDialog(form, options);
            };

            $scope.showADUserToolTip = function(){
                var form = $scope.userForm;
                if (form !== undefined) {
                    form.find(".definition").popover({
                        appendTo: form
                    });                   
                }
            };

            function ADuserValidator(){
                $.validator.addMethod("isADNameValid", function(value, element) {
                    return (value.indexOf('@') == -1 && value.indexOf(' ') == -1);
                }, gettextCatalog.getString("Invalid username"));
            }

            function noDuplicatesValidator(){
                $.validator.addMethod("isADNameUnique", function(value, element) {
                    var valid = true;
                    for (var i = 0; i < $scope.userGridData.data.length; i++) {
                        var item = $scope.userGridData.data[i];
                        if (isNaN(item.id) && (item.name === $scope.adUser.name)) {
                            valid = false;
                            break;
                        }
                    }
                    return valid;
                }, gettextCatalog.getString("AD username has already been added"));
            }

            $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

            $scope.initCtrl = function(dlg){
                switch (dlg.name){
                    // When putting an ngDialog above a UI Dialog, we need to adjust the vertical alignment (the default ngDialog top is 25%).
                    case 'edit-group-dialog':
                        $scope.editForm = angular.element("#edit-groups-form");
                        $scope.loadUsers();
                        $scope.validateForm($scope.editForm);
                        break;
                    case 'add-ad-user-dialog':
                        $scope.userForm = angular.element("#ad-user-form");
                        $scope.validateADForm($scope.userForm);
                        $scope.showADUserToolTip();
                        break;
                    default:
                        // No action.
                        break;
                }
            };

        }]);