angular.module('sla-policy-module', ['ngDialog', 'angulartics', 'angulartics.google.analytics','ui.grid','ui.grid.resizeColumns','ui.grid.autoResize',
    'ui.bootstrap','inventory','checklist-model','ui.grid.selection'])
.controller('slaPolicyCtrl', ['$scope', '$rootScope', '$http', 'ngDialog', 'wizardService', '$analytics','fileBrowserDialogService','sharedGridData','$protectService','inventorysvc','gettextCatalog',
    function($scope, $rootScope, $http, ngDialog, wizardService, $analytics,fileBrowserDialogService,sharedGridData,$protectService,inventorysvc,gettextCatalog) {
        $scope.policyName=gettextCatalog.getString('SLA Policy'); //default job name plaveholder
        $scope.optionalDescription=gettextCatalog.getString('Description (optional)');
        $scope.policyNameModel=gettextCatalog.getString('SLA Policy');
        $scope.isCustomJob = false;

        // Creates an empty Policy object with a default name, app and system id.
        function createEmptyPolicy() {
            var policy={};
            policy.name = gettextCatalog.getString("SLA Policy");
            policy.type = "policy"; //this is always policy.
            policy.description = "";
            policy.app_id = APP_ID_FILE_LEVEL; //default assetType is the server, so a file-level (1) app type.
            policy.sid = $rootScope.local.id;  // local system by default.
            policy.min_max_policy = {
                'retention_min': 0,
                'retention_max': 0,
                'legal_hold': 0
            };
            policy.instances = [];
            policy.calendar = [];
            policy.replication = false;
            policy.archive = false;
            policy.archive_retention_days = 0;
            policy.archive_encryption = false;
            return policy;
        }

        $scope.policy = createEmptyPolicy();

        $scope.errorField=false;
        $scope.startDateError=false;
        $scope.dayError=false;
        $scope.startDateOlderError = false;
        $scope.startTimeError=false;
        $scope.recuranceError=false;
        $scope.recurrenceBoundsError = false;
        $scope.recursTime=[{id:0,name:gettextCatalog.getString('Hour(s)')},{id:1,name:gettextCatalog.getString('Minutes')}];
        $scope.isScheduled = true;
		$scope.recursEveryError = false;
        $scope.startDateFormatError = false;

        $scope.selectedTreeNode = [];
        $scope.selectedAsset =[];
        $scope.backupModes = [];
        $scope.backupModesForNow=[];
        $scope.backupModesForSchedule=[];
        $scope.backupStrategies=[];
        $scope.backup_types = {};
        $scope.backupModesforJob = {};
        $scope.scheduleForm = {};
        $scope.hideEdit=false;
        $scope.jobErrorDialog = false;

        // Don't show warnings unless these started as true.
        $scope.showHotBackupCopyWarning = false;
        $scope.showBackupCopyWarning = false;

        // initialize archival and replication capability to true.
        $scope.canColdBackupCopy = true;
        $scope.canHotBackupCopy = true;

        var templateWithTooltip = '<div class="ui-grid-disable-selection" style="word-wrap:normal;font-size:12px;padding-top:5px;padding-left:5px" ' +
                                    'title="{{row.entity[col.field]}}">{{row.entity[col.field]}}</div>';
        $scope.inventoryColumnDefs = [
            {
                field:'id',
                displayName:'Id',
                visible:false
            },
            {
                field:'nodeId',
                displayName:'Node Id',
                visible:false
            },
            {
                field: 'name',
                displayName:gettext("Name"),
                headerCellFilter: 'translate',
                headerCellClass: 'ui-grid-not-bold',
                cellTooltip: true,
                enableRowHeaderSelection: false,
                width:'35%'
            },
            {
                field:'rules',
                displayName:gettext("Protection Rules"),
                cellTooltip: true,
                headerCellFilter: 'translate',
                headerCellClass: 'ui-grid-not-bold',
                cellTemplate: templateWithTooltip,
                width:'47%'
            },
            {
                field:'edit',
                displayName:'',
                cellTemplate: '<div ng-hide="grid.appScope.hideEdit" style="padding-left:25px;padding-top:5px;cursor:pointer" ' +
                'ng-click="grid.appScope.openEditSettingServerDialog(row.entity,row.getProperty(col.field), col.field)">' +
                '<img src="assets/images/edit_pencil.png" style="height:11.5px;cursor:pointer;"/>' +
                '<span style="font-size:11px;padding-left:5px;font-weight:bold"><translate>Edit</translate></span>' +
                '</div>',
                width:'18%'
            }
        ];
        $scope.inventoryGridOptions = {
            columnDefs: $scope.inventoryColumnDefs,
            data:'inventoryData',
            rowHeight: 32,
            //virtualizationThreshold:200,
            enableHorizontalScrollbar:0,
            gridMenuShowHideColumns:false,
            enableColumnMenus:false,
            enableColumnResizing: true
        };
        $scope.assetType={};
        // Policies are only support for Agents, VMware, and Hyper-V.
        $scope.assetTypeList=[
            { name: gettextCatalog.getString("Agent-Based Assets"), hypervisor:"1" ,id:INVENTORY_TYPE_FAMILY_CLIENT},
            { name: gettextCatalog.getString("VMware Assets"), hypervisor:"3" ,id:INVENTORY_TYPE_FAMILY_VMWARE},
            { name: gettextCatalog.getString("Hyper-V Assets"), hypervisor:"2" ,id:INVENTORY_TYPE_FAMILY_HYPER_V},
            { name: gettextCatalog.getString("AHV Assets"), hypervisor:"4", id:INVENTORY_TYPE_FAMILY_AHV}
           /* { name: "NAS",    hypervisor:"1"    ,id:INVENTORY_TYPE_FAMILY_CLIENT_NAS} */
        ];

        $scope.PM_ADJUST_EXTRA  = 13;
        $scope.PM_ADJUST = 12;
        $scope.showtoolTipForEmailVerification = function(){
            var form = angular.element("#test-job-form");
            form.find(".definition").popover({
                appendTo: form
            });
        };

        $scope.setProtectionRule = $protectService.setProtectionRule;       // common function for protection rules.

        if($scope.isEdit){
            $http({
                method: 'get',
                url : '/api/sla-policies/'+$scope.policyId  + "/?lang=" + $rootScope.userLanguageCode
            }).success(function(data, status, headers){
                $scope.policy=data.data[0];

                // Don't show warnings unless these started as true.
                $scope.showHotBackupCopyWarning = $scope.policy.replication;
                $scope.showBackupCopyWarning = $scope.policy.archive;

                setTimeout(function(){
                    // Send a message to the retention controllers.
                    $rootScope.$broadcast("SLA-Policy-Loaded", $scope.policy);
                    $rootScope.$broadcast("Retention-Loaded", $scope.policy.archive_retention_days);
                }, 1000);
                document.getElementById("policyId").value = $scope.policy.name;
                $scope.assetType.id= $scope.getAssetType($scope.policy);
                $scope.hideEdit = checkHideEdit($scope.assetType.id);
                $scope.setRecurrenceModel();
                $scope.setEndHour();
                $protectService.loadTreeData($http,$scope,$rootScope,$scope.assetType.id,false);
            }).error(function(response){
                ngDialog.open({
                    dialogType:'Error',
                    dialogMessage:response.result[0].message
                })
            });
        }else{
            $scope.assetType.id= parseInt($scope.selectedAssetType);
            setTimeout(function(){
                $scope.filterInventory();
                // Send a message to the retention controller.
                $rootScope.$broadcast("SLA-Policy-Loaded", $scope.policy);
                $rootScope.$broadcast("Retention-Loaded", $scope.policy.archive_retention_days);
            }, 1000);
        }

        /*
         * If an end hour is specified, recurrences run through that hour (minute 59).
         */
        $scope.setEndHour = function(){
            for (var i = 0 ; i < $scope.policy.calendar.length ; i++){
                $scope.setCalendarEntryBeginHour(i);
                $scope.setCalendarEntryEndHour(i);
            }
        };

        $scope.setCalendarEntryEndHour = function(i) {
            var minute = "59";
            var ampm = " pm";
            if ($scope.policy.calendar[i] !== undefined && !isNaN($scope.policy.calendar[i].end_hour)) {
                var endHour = parseInt($scope.policy.calendar[i].end_hour);
                if (endHour > 13){
                    // use PM_ADJUST_EXTRA as we add 59 minutes for display.
                    endHour -= $scope.PM_ADJUST_EXTRA;
                    $scope.policy.calendar[i].end_hour = endHour+":"+minute+ampm;
                } else {
                    ampm = (endHour == 13) ? " pm" : " am";
                    endHour--;
                    $scope.policy.calendar[i].end_hour = endHour+":"+minute+ampm;
                }
            }
        };

        $scope.setCalendarEntryBeginHour = function(i) {
            var minute = "0";
            var ampm = " pm";
            var adjustEnd = false;
            if ($scope.policy.calendar[i] !== undefined && !isNaN($scope.policy.calendar[i].begin_hour)) {
                var endHour = parseInt($scope.policy.calendar[i].begin_hour);
                if (endHour > 12){
                    // If not adjusting the end, use PM_ADJUST as no minutes are added.
                    endHour -= $scope.PM_ADJUST;
                    $scope.policy.calendar[i].begin_hour = endHour+":"+minute+ampm;
                } else {
                    ampm = (endHour == 12) ? " pm" : " am";
                    if (adjustEnd) {
                        endHour--;
                    }
                    $scope.policy.calendar[i].begin_hour = endHour+":"+minute+ampm;
                }
            }
        };

        $scope.setRecurrenceModel = function(){
            for (var i =0 ; i <$scope.policy.calendar.length ; i ++){
             if(typeof($scope.policy.calendar[i].recurrence) !='undefined'){
                 $scope.policy.calendar[i].recursCheck  =true;
                 if($scope.policy.calendar[i].recurrence < 60){
                     $scope.policy.calendar[i].recursTimeSelect=1;
                 }else{
                     $scope.policy.calendar[i].recurrence = $scope.policy.calendar[i].recurrence/60;
                     $scope.policy.calendar[i].recursTimeSelect=0;
                 }
                 // Wrap with timeout so that recurs loads as a spinner.
                 (function(){const index = i;
                     setTimeout(function(){
                         $scope.recurstimeModeChange("",$scope.policy.calendar[index].recursTimeSelect)},0,index);
                 })();
             }else{
                 $scope.policy.calendar[i].recursCheck = true;
                 // For policy, always set recurrence.
                 $scope.policy.calendar[i].recursTimeSelect = 0;
                 $scope.policy.calendar[i].recurrence = 12;
                 $scope.policy.calendar[i].end_hour = 24;
             }
            }
        };
        $scope.treeConfig = {
            core : {
                multiple : !$scope.isNAS, //TODO check if it needs to single select in case of NAS
                animation: true,
                error : function(error) {console.log('treeCtrl: error from js tree - ' + angular.toJson(error));},
                check_callback : true,
                worker : true
            },
            types : {
                default : {icon : 'icon-hv-datacenter icon'},
                star : {icon : 'icon-hv-host icon'},
                cloud : {icon : 'icon-hv-host icon'}
            },
            version : 1,
            plugins : ['types','checkbox','search']
        };

        $scope.filterInventory = function(){
            $('#jstreeDivId').jstree(true).uncheck_all();
            $scope.policy = createEmptyPolicy();
            $scope.clearSearch();


            $scope.hideEdit = checkHideEdit($scope.assetType.id);
            $scope.isNAS = false;
            if($scope.assetType.id == INVENTORY_TYPE_FAMILY_CLIENT_NAS){
                $scope.isNAS = true;
                // This does not work.  The selection moves to the other NAS, but the original
                // is not removed from the right-hand-side correctly.  There is logic at the
                // top of winzard.backupJobManageAsset.html to check for multiple NASes.
                //$('#jstreeDivId').jstree(true).settings.core.multiple = false;
            } else {
                $('#jstreeDivId').jstree(true).settings.core.multiple = true;
            }
            $rootScope.treeData=[];
            $scope.selectedAsset =[];
            $scope.inventoryData =[];
            $protectService.loadTreeData($http,$scope,$rootScope,$scope.assetType.id,false);
        };
        
        //
        // Based on the asset type, returns an indication of whether or not the edit button should be visible for the job.
        //
        function checkHideEdit(assetType) {
            return (assetType == INVENTORY_TYPE_FAMILY_HYPER_V || assetType == INVENTORY_TYPE_FAMILY_EXCHANGE || assetType == INVENTORY_TYPE_FAMILY_SQL ||
                    assetType == INVENTORY_TYPE_FAMILY_ORACLE || assetType == INVENTORY_TYPE_FAMILY_SHAREPOINT || assetType == INVENTORY_TYPE_FAMILY_CISCO_UCS ||
                    assetType == INVENTORY_TYPE_FAMILY_NDMP);
        }
        
        $scope.getAssetType = function(joborder){
            var app_id= joborder.app_id;

            if(app_id === APP_ID_FILE_LEVEL){
                if(joborder.instances[0].name.indexOf("-NAS-RRC")>-1){ //hack till backend fix it.
                    return INVENTORY_TYPE_FAMILY_CLIENT_NAS;
                }
                return INVENTORY_TYPE_FAMILY_CLIENT;
            } else if(app_id === APP_ID_VMWARE){
                return INVENTORY_TYPE_FAMILY_VMWARE;
            } else if(app_id === APP_ID_XEN){
                return INVENTORY_TYPE_FAMILY_XEN;
            } else if(app_id === APP_ID_AHV){
                return INVENTORY_TYPE_FAMILY_AHV;
            } else if(app_id === APP_ID_HYPERV_2008_R2 || app_id === APP_ID_HYPERV_2012 || app_id === APP_ID_HYPERV_2016){
                return INVENTORY_TYPE_FAMILY_HYPER_V;
            } else if (app_id === APP_ID_EXCHANGE_2003 || app_id === APP_ID_EXCHANGE_2007 || app_id === APP_ID_EXCHANGE_2010 || app_id === APP_ID_EXCHANGE_2013 || app_id === APP_ID_EXCHANGE_2016) {
                return INVENTORY_TYPE_FAMILY_EXCHANGE;
            }else if (app_id === APP_ID_SQL_SERVER_2005 || app_id === APP_ID_SQL_SERVER_2008 || app_id === APP_ID_SQL_SERVER_2008_R2 
                || app_id === APP_ID_SQL_SERVER_2012 || app_id === APP_ID_SQL_SERVER_2014 || app_id === APP_ID_SQL_SERVER_2016 || app_id === APP_ID_SQL_SERVER_2017) {
                return INVENTORY_TYPE_FAMILY_SQL;
            }else if (app_id === APP_ID_SHAREPOINT_2007 || app_id === APP_ID_SHAREPOINT_2010 || app_id === APP_ID_SHAREPOINT_2013 || app_id === APP_ID_SHAREPOINT_2016) {
                return INVENTORY_TYPE_FAMILY_SHAREPOINT;
            }else if (app_id === APP_ID_ORACLE_11 || app_id === APP_ID_ORACLE_12 || app_id === APP_ID_ORACLE_10) {
                return INVENTORY_TYPE_FAMILY_ORACLE;
            } else if(app_id === APP_ID_CISCO_UCS_SERVICE_PROFILE){
                return INVENTORY_TYPE_FAMILY_CISCO_UCS;
            } else if (app_id === APP_ID_NDMP) {
                return INVENTORY_TYPE_FAMILY_NDMP;
            }
        }

        // sla-policies are by instance only.
        $scope.getClientOrInstance = function(policy){
            return policy.instances;
        };

        $scope.uncheckNotEligibleNode=function(uncheckNodeId){
            var node_id='li#'+uncheckNodeId;
            $('#jstreeDivId').jstree(true).uncheck_node(node_id);
        };

        $scope.readyTree=function(event){
            if($scope.isEdit){
                 setTimeout(function(){
                   $scope.checkNodesOnLoadEdit();
                }, 1000);
            } else {
                if($scope.selectedNodeList.length>0){
                    setTimeout(function(){
                        $scope.checkNodesOnPreSelect();
                    }, 1000);
                }
            }
        };

        $scope.checkNodesOnPreSelect=function(){
            $('#jstreeDivId').jstree(true).open_all();
            for(var i=0; i< $scope.selectedNodeList.length ; i++){
                var node_id='li#';
                node_id = node_id +$scope.selectedNodeList[i];
                $('#jstreeDivId').jstree(true).check_node(node_id);
            }
        };

        $scope.checkNodesOnLoadEdit=function(){
            var nodeIdArray =Object.keys($('#jstreeDivId').jstree(true)._model.data);
            // If the model data is not loaded yet, do not proceed.
            if (nodeIdArray.length <= 1) {
                return;
            }
            var arrayAssets = $scope.getClientOrInstance($scope.policy);
            $('#jstreeDivId').jstree(true).open_all();
            if(arrayAssets !=undefined){
                for(var i=0; i< arrayAssets.length ; i++){
                    var node_id='li#';
                    for(var j=0 ;j<nodeIdArray.length;j++){
                        if (nodeMatches(nodeIdArray[j], arrayAssets[i], $scope.policy)) {
                            arrayAssets[i].nodeId = nodeIdArray[j];
                            if(arrayAssets[i].incl_list == undefined || arrayAssets[i].incl_list == null){
                                arrayAssets[i].incl_list = [];
                            }
                            if(arrayAssets[i].excl_list == undefined || arrayAssets[i].excl_list == null){
                                arrayAssets[i].excl_list = [];
                            }

                            $scope.selectedAsset[nodeIdArray[j]] = arrayAssets[i];
                            node_id = node_id+nodeIdArray[j];
                            $scope.before_command = arrayAssets[i].before_command;
                            $scope.after_command = arrayAssets[i].after_command;
                            break;
                        }
                    }
                    node_id = inventorysvc.handleSpecialChars(node_id);
                    $('#jstreeDivId').jstree(true).check_node(node_id);
                }
                $('#jstreeDivId').jstree(true).close_all();
            }
        };

        function nodeMatches(node, asset, policy) {
            var match = false;
            var asset_node = policy.sid + "_";
            if (node.indexOf(asset_node) === 0) {
                // sid matches, instance_id should be unique for htis system.
                var parts = node.split('_');
                // Must be a number, must match the instnace ID.
                match = isNaN(parts[parts.length - 1]) ? false : (asset.id === parseInt(parts[parts.length - 1]));
            }
            return match;
        }


        $scope.deslectNodeFun = function(e,item){
            if (item.node.data !== undefined && item.node.data.type === INVENTORY_ID_VMWARE_ESX_SERVER) {
                $rootScope.joborderHypervisorID = item.node.id;
                $rootScope.clearRegex();
                $scope.$apply(function() {
                    $scope.isHypervisorHostSelected = false;
                });
            }
            if(item.node.data.type ===INVENTORY_ID_SQL_INSTANCE || item.node.data.type ===INVENTORY_ID_HYPER_V_SERVER || item.node.data.type  ==INVENTORY_ID_VMWARE_ESX_SERVER  || item.node.data.type  ==INVENTORY_ID_VMWARE_RESOURCE_POOL){
                for(var k =0 ; k < item.node.children_d.length; k++){
                    if($scope.selectedAsset[item.node.children_d[k]] !=null || $scope.selectedAsset[item.node.children_d[k]] != undefined){
                            delete $scope.selectedAsset[item.node.children_d[k]];
                    }
                }
                $scope.inventoryData =[];
                var nodeIdArray =Object.keys($scope.selectedAsset);
                for(var j=0 ;j<nodeIdArray.length;j++){
                    var obj = $scope.selectedAsset[nodeIdArray[j]];
                    if(item.node.data.type === INVENTORY_ID_VMWARE_VM || item.node.data.type ===INVENTORY_ID_XEN_VM || item.node.data.type === INVENTORY_ID_AHV_VM){
                        $scope.getExcludedDatastore(nodeIdArray[j]);
                    }
                    obj.rules = $scope.setProtectionRule(obj.incl_list,obj.excl_list,obj.metanames);
                    $scope.inventoryData.push(obj);
                }
                $(window).resize();

            } else {
                if(item.selected.length == 0){
                    $scope.policy.clients =[];
                    $scope.selectedAsset =[];
                }
                if($scope.policy.sid == inventorysvc.nodeToObject(item.node.id).system_id){
                    delete $scope.selectedAsset[item.node.id];
                }
                $scope.inventoryData =[];
                var nodeIdArray =Object.keys($scope.selectedAsset);
                for(var j=0 ;j<nodeIdArray.length;j++){
                    var obj = $scope.selectedAsset[nodeIdArray[j]];
                    obj.rules = $scope.setProtectionRule(obj.incl_list,obj.excl_list,obj.metanames);
                    $scope.inventoryData.push(obj);
                }
                $(window).resize();
            }
        };

        $scope.slectNodeFun = function(e,item){
            var osTypeId = 0;
            if (item.node.data !== undefined && item.node.data.os_type_id !== undefined) {
                osTypeId = item.node.data.os_type_id;
            }
            /*
             * Until iSeries is integrated into the system, protect it from being seleected or pre-selected in a backup job.
             * Rather than disabling, use the selection to display a helpful message to user on how to protect an iSeries asset.
             */
            if (item.node.data !== undefined && item.node.data.type === INVENTORY_ID_CLIENT_I_SERIES) {
                var message  = "An iSeries asset may not be included in a backup job in the web-based user interface.  To protect iSeries data, please use the menu-based console interface.";
                $scope.uncheckNotEligibleNode(item.node.id);

                if ($scope.jobErrorDialog == false) {
                    // Set the dialog flag so that multiple dialogs are not opened, as this function can be
                    // called multiple times when the user selects multiple nodes at once, i.e., on pre-selection.
                    $scope.jobErrorDialog = true;
                    ngDialog.open({
                        dialogType:'Error',
                        modelDialogId:'BackupPostErrorDialog',
                        scope:$scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick:'onConfirmError()'
                    });
                }
            }else if(item.node.data.type ===INVENTORY_ID_SQL_INSTANCE || item.node.data.type  ==INVENTORY_ID_HYPER_V_SERVER || item.node.data.type  ==INVENTORY_ID_VMWARE_ESX_SERVER  || item.node.data.type  ==INVENTORY_ID_VMWARE_RESOURCE_POOL ){
                /*if(item.node.data.expanded == false){*/
                $('#jstreeDivId').jstree(true).open_all(item.node);
                item.node.data.expanded =true;
                // retrieving the sid and app_id from the instance/server/resource pool selected
                $scope.policy.sid = inventorysvc.nodeToObject(item.node.id).system_id;
                $scope.policy.app_id = inventorysvc.nodeToObject(item.node.id).app_id;
                /*}*/
                if (item.node.data !== undefined && item.node.data.type === INVENTORY_ID_VMWARE_ESX_SERVER) {
                        $scope.isHypervisorHostSelected = true;
                        $rootScope.joborderHypervisorID = item.node.id;
                } 
                for(var k =0 ; k < item.node.children_d.length; k++){
                    var id = item.node.children_d[k]+'_anchor';
                    if($scope.selectedAsset[item.node.children_d[k]] == null || $scope.selectedAsset[item.node.children_d[k]] == undefined){
                        if (inventorysvc.isGroup(item.node.children_d[k]) ||
                            inventorysvc.isPoolOrVApp(item.node.children_d[k])) {
                            // If the child is a group, pool, or vApp, do not add to the selected list.
                            continue;
                        }
                        var $el = $(document.getElementById(id));
                        $scope.selectedAsset[item.node.children_d[k]] ={nodeId:item.node.children_d[k],name:$el.text(),id:inventorysvc.nodeToObject(item.node.children_d[k]).client_id,incl_list:[],excl_list:[]};
                    }
                }
                $scope.inventoryData =[];
                var nodeIdArray =Object.keys($scope.selectedAsset);
                for(var j=0 ;j<nodeIdArray.length;j++){
                    var obj = $scope.selectedAsset[nodeIdArray[j]];
                    if(item.node.data.type === INVENTORY_ID_VMWARE_VM || item.node.data.type ===INVENTORY_ID_XEN_VM || item.node.data.type === INVENTORY_ID_AHV_VM){
                        $scope.getExcludedDatastore(nodeIdArray[j]);
                    }
                    obj.rules = $scope.setProtectionRule(obj.incl_list,obj.excl_list,obj.metanames);
                    $scope.inventoryData.push(obj);
                }
                $(window).resize();

            } else {
                if(item.selected.length>1){
                    if($scope.policy.sid == inventorysvc.nodeToObject(item.node.id).system_id){
                        if($scope.selectedAsset[item.node.id] ==null || $scope.selectedAsset[item.node.id] == undefined){
                            $scope.selectedAsset[item.node.id] ={nodeId:item.node.id,name:item.node.text,id:inventorysvc.nodeToObject(item.node.id).client_id,incl_list:[],excl_list:[],os_type_id:osTypeId};
                        }
                    }else{
                        $scope.uncheckNotEligibleNode(item.node.id);
                    }
                }else{
                    $scope.policy.sid = inventorysvc.nodeToObject(item.node.id).system_id;
                    $scope.policy.app_id = inventorysvc.nodeToObject(item.node.id).app_id;
                    if (item.node.data !== undefined && item.node.data.type === INVENTORY_ID_VMWARE_ESX_SERVER) {
                        $scope.isHypervisorHostSelected = true;
                        $rootScope.joborderHypervisorID = item.node.id;
                    } else {
                        if($scope.selectedAsset[item.node.id] == undefined ||$scope.selectedAsset[item.node.id] ==null){
                            $scope.selectedAsset[item.node.id] ={nodeId:item.node.id,name:item.node.text,id:inventorysvc.nodeToObject(item.node.id).client_id,incl_list:[],excl_list:[],os_type_id:osTypeId};
                        }
                    }
                }

                $scope.inventoryData =[];
                var nodeIdArray =Object.keys($scope.selectedAsset);
                for(var j=0 ;j<nodeIdArray.length;j++){
                    var obj = $scope.selectedAsset[nodeIdArray[j]];
                    if(item.node.data.type === INVENTORY_ID_VMWARE_VM || item.node.data.type ===INVENTORY_ID_XEN_VM || item.node.data.type === INVENTORY_ID_AHV_VM){
                        $scope.getExcludedDatastore(nodeIdArray[j]);
                    }
                    obj.rules = $scope.setProtectionRule(obj.incl_list,obj.excl_list,obj.metanames);
                    $scope.inventoryData.push(obj);
                }
                $(window).resize();
            }
        };

        $scope.hoverNode = function(e, item){
            // In case of SQL databases, display the recovery model when hovered over
            if (item.node.data.recovery_model !== undefined && item.node.data.recovery_model !== "") {
                var hoverTip = "model: " + item.node.data.recovery_model;
            }
            // In case of SQL clusters and SQL Availability Groups, display that when hovered over
            if (item.node.data.is_sql_cluster !== undefined && item.node.data.is_sql_cluster !== false) {
                var hoverTip = "SQL Cluster";
            }
            if (item.node.data.is_sql_alwayson !== undefined && item.node.data.is_sql_alwayson !== false) {
                var hoverTip = "SQL Availability Group";
            }
            if (hoverTip !== undefined && hoverTip !== "" && hoverTip !== null) {
                var $el = $(document.getElementById(item.node.id));
                $el.prop('title', hoverTip);
            }
        };

        $scope.refreshInventoryData = function(assets){
            var assets = assets || false;
            if (assets) {
                $scope.selectedAsset = assets;
            }
            $scope.inventoryData =[];
            var nodeIdArray =Object.keys($scope.selectedAsset);
            for(var j=0 ;j<nodeIdArray.length;j++){
                var obj = $scope.selectedAsset[nodeIdArray[j]];
                obj.rules = $scope.setProtectionRule(obj.incl_list,obj.excl_list,obj.metanames);
                $scope.inventoryData.push(obj);
            }
            $(window).resize();

        };

        $scope.searchIndex = 0;

        $scope.search = function(){
            setTimeout(function () {
                var v = $('#inventorySearchInputId').val();
                $('#jstreeDivId').jstree(true).search(v);
            }, 200);
        };

        $scope.goToSearchResult = function() {
            var el =  $('#jstreeDivId').jstree(true).get_node($scope.searchResults[$scope.searchIndex], true);
            if (el !== undefined) {
                el.children('.jstree-anchor').focus();
            }
            el.focus();
        };

        $scope.getSearchResults = function(event, obj) {
            setTimeout(function () {
                $scope.$apply(function() {
                    $scope.searchResults = obj.nodes;
                    $scope.searchIndex = 0;
                });
            });
        };

        $scope.next = function() {
            var len = $scope.searchResults.length;
            if ($scope.searchIndex + 1 > len - 1) {
                $scope.searchIndex = 0;
            } else {
                $scope.searchIndex++;
            }
            $scope.goToSearchResult();
        };

        $scope.previous = function() {
            var len = $scope.searchResults.length;
            if ($scope.searchIndex - 1 < 0) {
                $scope.searchIndex = len - 1;
            } else {
                $scope.searchIndex--;
            }
            $scope.goToSearchResult();
        };

        $scope.clearSearch = function() {
            $('#jstreeDivId').jstree(true).clear_search();
            $('#inventorySearchInputId').val("");
        };

        $scope.clearSearchResults = function() {
            $scope.searchIndex = 0;
            $scope.searchResults = undefined;
        };

        $scope.addNewInventory=function(item,id){
        };

        $scope.getProtectionRulesArray = function(inclusionData,exclusionData){
        };

        $scope.functiontofindIndexByKeyValue = function(arraytosearch, key, valuetosearch) {
            for (var i = 0; i < arraytosearch.length; i++) {
                if (arraytosearch[i][key] == valuetosearch) {
                  return i;
                 }
            }
                return null;
        };

        $scope.isRegexJob = function(job) {
            return (job !== undefined && job.regular_expressions !== undefined && job.regular_expressions.length > 0);
        };

        $scope.canShowFilter = function() {
            return ($scope.assetType.id == INVENTORY_TYPE_FAMILY_VMWARE);
        };

/*************************************************************************
2ndstep
*/  
        $scope.validationForForm=function(){
            $scope.errorField=false;
            $scope.startDateError=false;
            $scope.dayError=false;
            $scope.startDateOlderError = false;
            $scope.startTimeError=false;
            $scope.recuranceError=false;
            $scope.recurrenceBoundsError = false;
			$scope.endHourError =false;
			$scope.recursEveryError = false;
            $scope.startDateFormatError = false;
			

            if ($scope.policy.calendar === undefined || $scope.policy.calendar.length == 0 ){
                return true;
            }else{
                for (var i =0 ; i < $scope.policy.calendar.length ; i++){
                  if(typeof($scope.policy.calendar[i].start_date) == 'undefined'){
                        $scope.errorField=true;
                        $scope.startDateError=true;
                    }
                 if(typeof($scope.policy.calendar[i].start_time) == 'undefined'){
                        $scope.errorField=true;
                        $scope.startTimeError=true;
                   }
                 else{
                     var checkStartDateFormatError = $scope.validateStartDate($scope.policy.calendar[i].start_date);
                     if(checkStartDateFormatError){
                         $scope.startDateFormatError = true;
                         $scope.errorField = true;
                     } else {
                         $scope.policy.calendar[i].start_date =
                             DateTimeHandler.getUSDateString($rootScope.userLanguageCode, $scope.policy.calendar[i].start_date);
                         if (!$scope.isEdit) {
                             var checkStartDate = $scope.validateStart($scope.policy.calendar[i].start_date);
                             if (checkStartDate) {
                                 $scope.startDateOlderError = true;
                                 $scope.errorField = true;
                             }
                         }
                     }
                 }
				   if( $scope.policy.calendar[i].recursCheck){
                      if(typeof($scope.policy.calendar[i].recurrence )==='undefined'  || $scope.policy.calendar[i].recurrence==null || $scope.policy.calendar[i].recurrence==="" || $scope.policy.calendar[i].recurrence==0 ){
                          $scope.errorField=true;
                          $scope.recuranceError=true;
                          $scope.recursEveryError=true;
                      } else {
                          var timerID = "#recurs_spinner_field";
                          var isValid = $(timerID).spinner("isValid");
                          if (!isValid) {
                              $scope.errorField=true;
                              $scope.recurrenceBoundsError=true;
                          }
                      }
                  }
                  if( $scope.policy.calendar[i].recursCheck){
                      if(typeof($scope.policy.calendar[i].end_hour) == 'undefined') { //} || ($scope.policy.calendar[i].end_hour< 0 && $scope.policy.calendar[i].end_hour > 24) ){
                           $scope.errorField=true;
                           $scope.endHourError=true;
                      }
                      var start_hour = $scope.getHourFromString($scope.policy.calendar[i].begin_hour);
                      var end_hour = $scope.getHourFromString($scope.policy.calendar[i].end_hour);
                      if (start_hour >= end_hour) {
                          $scope.errorField=true;
                          $scope.endHourError=true;
                      }
                  }
                  
                  if($scope.errorField){
                      $scope.error_backupType =$scope.policy.calendar[i].backup_type;
                        break;
                    }
                    
                }
                
                return !$scope.errorField;
            }
      
        };

        $scope.validateStartDate = function (date) {
            return DateTimeHandler.dateNotValid(date, $rootScope.userLanguageCode);
        };

        $scope.validateStart = function(date) {
            if (date === '') {
                return true;
            }
            var today = new Date();
            today.setHours(0);
            today.setMinutes(0);
            today.setSeconds(0);
            if (Date.parse(today) > Date.parse(date)) {
                return true;
            }
            return false;
        };

        /*
         * If the standard strategies are in use, make sure any extras are deleted.
         * If a custom calendar, the policy.calendar will not exist.
         */
        $scope.removeUnwantedKeys = function(){
            if ($scope.policy.calendar !== undefined) {
                for (var i =0 ; i < $scope.policy.calendar.length ; i++){
                    if(!$scope.policy.calendar[i].recursCheck){
                        delete $scope.policy.calendar[i].recurrence;
                    }
                    else if ($scope.policy.calendar[i].recursTimeSelect == 0 ){
                        $scope.policy.calendar[i].recurrence =$scope.policy.calendar[i].recurrence * 60;
                    }
                    $scope.policy.calendar[i].begin_hour = $scope.getHourFromString($scope.policy.calendar[i].begin_hour);
                    $scope.policy.calendar[i].end_hour = $scope.getHourFromString($scope.policy.calendar[i].end_hour);
                    delete $scope.policy.calendar[i].$$hashKey;
                    delete $scope.policy.calendar[i].recursTimeSelect;
                    delete $scope.policy.calendar[i].recursCheck;
                }
            }
        };

        $scope.getHourFromString = function(hourString){
            var Hour = 0;
            if (hourString !== undefined) {
                var hourStringArray = hourString.split(" ");
                var hourArray = hourStringArray[0].split(":");
                Hour  = parseInt(hourArray[0]);
                var Minute = parseInt(hourArray[1]);
                if (hourStringArray[1] === "pm"){
                    if (Hour < 12) {
                        Hour += $scope.PM_ADJUST;
                    }
                } else {
                    if (Hour == 12) {
                        Hour -= $scope.PM_ADJUST;
                    }
                }
                if (Minute == 59) {
                    Hour++;
                }
            }
            return Hour;
        };

        $scope.nextWizard = function($event){
            if(wizardTitle === gettextCatalog.getString("Select Items")){
		        console.log($scope.selectedAsset);
                if (!$scope.validateAssets()) {
                    return;
                }
                $rootScope.$broadcast("nextBtnClick");
                if($scope.isEdit === true){
                    $scope.isScheduled = true;
                }
                $scope.getColdBackupCopyTargets();
                $scope.getHotBackupCopyTargets();
                $scope.getEncryptionStatus();
                $scope.getBackupStrategies();
				$scope.showtoolTipForEmailVerification();
            }
            if(wizardTitle === gettextCatalog.getString("Define Policy Options")){
                if(document.getElementById("policyId").value === ""){
                    $scope.jobNameError=true;
                    return;
                }
                $scope.backuptarget=false;
                $scope.errorField=false;
                $scope.jobNameError=false;
                $scope.retentionError=false;

                $scope.validateOptionsAndSavePolicy();

                $rootScope.$broadcast("nextBtnClick");
            }
        };

        /*
         * This function validates the form, options for retention and saves the policy if valicated.
         * Side effect: if not validated, sets error flags:  errorField and retentionError.
         * errorField is true on any error, and retentionError is the specific error set.
         */
        $scope.validateOptionsAndSavePolicy = function() {
            if($scope.validationForForm()) {
                if ($rootScope.showRetention) {
                    var retentionInfo = $scope.getRetention();
                    if ($scope.validateRetention(retentionInfo)) {
                        $scope.policy.min_max_policy = retentionInfo;
                        $scope.policy.archive_retention_days = getColdBackupCopyRetention();
                        $scope.savePolicy();
                    } else {
                        $scope.errorField = true;
                        $scope.retentionError = true;
                        console.log('validation error');
                    }
                } else {
                    // remove any retention setting information.
                    delete($scope.policy.min_max_policy);
                    delete($scope.policy.archive_retention_days);
                    $scope.savePolicy();
                }
            } else {
                console.log('validation error');
            }
        };

        function getColdBackupCopyRetention() {
            var elem = angular.element(document.querySelector('[ng-controller=RetentionPeriodCtrl]'));
            var $scope = elem.scope();
            var retention = 0;
            if ($scope !== undefined) {
                retention = $scope.getRetentionDays();
            }
            return retention;
        }

        $scope.validateAssets = function() {
            var valid = true;
            var selectedCount = Object.keys($scope.selectedAsset).length;
            if (selectedCount === 0){
                // At least one asset must be selected.
                valid = false;
            } else if ($scope.isNAS && selectedCount > 1) {
                // If a NAS, only one should be selected.
                valid = false;
            }
            return valid;
        };

        $scope.getRetention = function() {
            var retentionInfo = {};
            if( $("#id_use_minimum").prop("checked")) {
                retentionInfo.retention_min = Number($("#min_days").val());
            } else {
                retentionInfo.retention_min = 0;
            }
            if( $("#id_use_maximum").prop("checked")) {
                retentionInfo.retention_max = Number($("#max_days").val());
            } else {
                retentionInfo.retention_max = 0;
            }
            if( $("#hold_forever").prop("checked")){
                retentionInfo.legal_hold = -1;
            } else if ($("#legal_hold").prop("checked")) {
                retentionInfo.legal_hold = Number($("#hold_days").val());
            } else {
                retentionInfo.legal_hold = 0;
            }
            return retentionInfo;
        };

        $scope.validateRetention = function(retentionInfo) {
            var msg = "",valid = true;
            var min = retentionInfo.retention_min;
            var max = retentionInfo.retention_max;
            var legal_hold = retentionInfo.legal_hold;
            if ($("#legal_hold").prop("checked")){
                if (legal_hold > 9999 || legal_hold < -1){
                    msg = '"Keep backups" value must be between 0 and 9999';
                    valid = false;
                }
            }
            if ($("#id_use_minimum").prop("checked")){
                if (min > 9999 || min < -1){
                    msg = '"Warn when less than" value must be between 0 and 9999';
                    valid = false;
                }
            }
            if ($("#id_use_maximum").prop("checked")){
                if (max > 9999 || max < -1){
                    msg = '"Delete backups after" value must be between 0 and 9999';
                    valid = false;
                }
            }
            if (min > 0 && max > 0 && legal_hold > 0 && min >= max && legal_hold >= max) {
                msg = '‘Delete backups after’ value must be greater than both ‘Keep backups’ and ‘Warn when less than’ values.';
                valid = false;
            }
            else if (max > 0 && legal_hold > 0 && legal_hold > max) {
                msg = '‘Keep backups’ value cannot exceed ‘Delete backups after’ value.';
                valid = false;
            }
            else if (min > 0 && max > 0 && min >= max) {
                msg = '’Delete backups after’ value must be greater than ‘Warn when less than’ value.';
                valid = false;
            }
            $scope.retention_error_message = msg;
            return valid;
        };

        $scope.validateAssetsAndSaveJobData = function() {
            if ($scope.validateAssets()) {
                $scope.validateOptionsAndSavePolicy();
            }
        };

        $scope.deleteBackupCopyWarning = function() {
            if ($scope.policy.archive === false && $scope.showBackupCopyWarning) {
                $scope.showBackupCopyWarning = false;
                $scope.showDeleteWarning("Cold Backup Copy");
            }
        };

        $scope.deleteHotBackupCopyWarning = function() {
            if ($scope.policy.replication === false && $scope.showHotBackupCopyWarning) {
                $scope.showHotBackupCopyWarning = false;
                $scope.showDeleteWarning("Hot Backup Copy");
            }
        };

        $scope.showDeleteWarning = function(jobType) {
            var message = "If you Save with this unchecked, then the previously-created " + jobType + " job for this policy will be removed.";
            ngDialog.open({
                dialogType:'Warning',
                modelDialogId:'Jobs-Warning',
                scope:$scope,
                dialogMessage: message,
                onConfirmOkButtonClick:'onConfirmWarning()'
            });
        };

        $scope.onConfirmWarning = function () {
            ngDialog.close('Jobs-Warning');
        };

        $scope.savePolicy = function(){
            $scope.policy.name=document.getElementById("policyId").value;
            $scope.policy.instances =[];
            var nodeIdArray =Object.keys($scope.selectedAsset);
            for(var j=0 ;j<nodeIdArray.length;j++){
                $scope.policy.app_id = inventorysvc.nodeToObject($scope.selectedAsset[nodeIdArray[j]].nodeId).app_id;
                $scope.selectedAsset[nodeIdArray[j]].id = parseInt(inventorysvc.nodeToObject($scope.selectedAsset[nodeIdArray[j]].nodeId).instance_id);
                $scope.policy.instances.push($scope.selectedAsset[nodeIdArray[j]]);
            }
            var jobOrderUrl='';
            var dataObj ={};
            var httpMethod='';
            var meassageforLoad=(!$scope.isEdit)?"Creating Policy":"Updating Policy";
            var loadJobindicator = PHD.showLoadingIndicator("body", true, meassageforLoad);
            // Get the joborder value - if an object, get its value.

            $scope.removeUnwantedKeys();
            if($scope.isEdit == true){
                httpMethod = 'put';
                jobOrderUrl = '/api/sla-policies/'+$scope.policy.id+'/?sid='+$scope.policy.sid;
            }else{
                jobOrderUrl = '/api/sla-policies/?sid='+$scope.policy.sid;
                httpMethod = 'post';
            }
            dataObj =JSON.stringify($scope.policy);
            console.log($scope.policy);


            $http({ method: httpMethod,
                url : jobOrderUrl,
                data: dataObj
            }).success(function(data, status, headers){
                PHD.hideLoadingIndicator(loadJobindicator);

                if (data[0]){
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'error-dialog',
                        scope: $scope,
                        dialogMessage: data[0].message
                    });
                } else {
                    var result = data.result;
                    var policyTable = [];
                    var obj;
                    if (result !== undefined) {
                        if (result['backup_job'] !== undefined) {
                            obj = {'status': 'success', 'name' : result['backup_job'], 'message': 'Job created', 'type': 'Backup'};
                            policyTable.push(obj);
                        }
                        if (result['replication_job'] !== undefined) {
                            obj = {'status': 'success', 'name' : result['replication_job'], 'message': 'Job created','type': 'Hot Backup Copy'};
                            policyTable.push(obj);
                        }
                        if (result['archive_job'] !== undefined) {
                            obj = {'status': 'success', 'name' : result['archive_job'], 'message': 'Job created', 'type': 'Cold Backup Copy'};
                            policyTable.push(obj);
                        }
                        if (result['archive_job_error'] !== undefined) {
                            obj = { 'status': 'warning', 'name': 'N/A', 'message': result['archive_job_error'], 'type': 'Cold Backup Copy'};
                            policyTable.push(obj);
                        }
                        if (result['replication_job_error'] !== undefined) {
                            obj = { 'status': 'warning', 'name': 'N/A', 'message': result['replication_job_error'], 'type': 'Hot Backup Copy'};
                            policyTable.push(obj);
                        }
                        if (result['retention_error'] !== undefined) {
                            obj = { 'status': 'warning', 'name': 'N/A', 'message': result['retention_error'], 'type': 'Retention'};
                            policyTable.push(obj);
                        }
                    }
                    if (policyTable.length > 0) {
                        ngDialog.open({
                            modelDialogId:'sla-success-dialog',
                            name: 'sla-success-dialog',
                            template: 'app/protect/sla-policy/sla-policy-success.html',
                            scope:$scope,
                            data: { policyTable: policyTable, isEdit : $scope.isEdit },
                            overlay:true,
                            ngDialogStyle:"width:700px;height:320px;"
                        });
                    } else {
                        $analytics && $analytics.eventTrack('Edit SLA Policy', {  category: 'Backup', label: 'Jobs' });
                        ngDialog.close('backup-physical-dailog');
                        $(document).trigger("editJob");
                    }
                }
            }).error(function(response) {
                var message = response !== undefined && response.result instanceof Array ? response.result[0].message : 'Unknown Server Error';
                $analytics && $analytics.eventTrack('Failure: ' + message, {  category: 'Backup', label: 'Protect' });
                PHD.hideLoadingIndicator(loadJobindicator);
                $scope.outputTitle = "SLA Policy Error";
                $scope.outputData = message;
                $scope.onConfirmOkButtonClick = $scope.onConfirmError;
                ngDialog.open({
                    template: 'app/common/scroll-output.html',
                    modelDialogId:'BackupPostErrorDialog',
                    dialogMessage:message,
                    scope:$scope,
                    ngDialogStyle: 'width:650px;',
                    onConfirmOkButtonClick: 'onConfirmError()'
                });
                if ($scope.isScheduled) {
                    // re-set the elements deleted before the save.
                    if ($scope.policy.calendar !== undefined) {
                        $scope.setRecurrenceModel();
                        $scope.setEndHour();
                    }
                }
            });

        };
        $scope.onConfirmError = function () {
            ngDialog.close('BackupPostErrorDialog');
            $scope.jobErrorDialog = false;
        };
        $scope.getBackupTypeForSelectedStrategies = function(){
            $scope.policy.calendar = [];
            var calendarObj={
                start_date:new Date().toLocaleDateString($rootScope.userLanguageCode),
                start_time:"03:00 AM",
                recursTimeSelect:0,
                recursCheck:1,
                recurrence:12,
                schedule_run: 0,
                begin_hour: 0,
                end_hour:24
            };
            $scope.policy.calendar.push(calendarObj);
            $scope.recurstimeModeChange("",0);
            $scope.setCalendarEntryBeginHour(0);
            $scope.setCalendarEntryEndHour(0);
        };

        $scope.setBackupModeForEdit = function(){
            var StrategiesObject={};
            for (var i=0 ; i < $scope.backupStrategies.length ; i++){
                var backupstrategyType="";
                for(var j=0 ; j < $scope.backupStrategies[i].backup_types.length ; j++){
                    backupstrategyType +=$scope.backupStrategies[i].backup_types[j];
                }
                StrategiesObject[backupstrategyType]=$scope.backupStrategies[i].name;
            }
            var backupType="";
            if ($scope.policy.calendar !== undefined) {
                for ( var i = 0 ; i < $scope.policy.calendar.length; i++){
                    backupType +=$scope.policy.calendar[i].backup_type;
                }
            }
            var selectedBackupType=StrategiesObject[backupType];
            if (selectedBackupType === undefined || selectedBackupType == 'Custom' || $scope.isCustomJob) {
                // if not found, or if there was a property on a calendar entry indicating it is custom, choose 'Custom'
                selectedBackupType = 'Custom';
                $scope.policy.calendar = [];
                $scope.isCustomJob = true;
                //$scope.calendar_description = $scope.getCalendarDescription();
            } else {
                $scope.isCustomJob = false;
            }
            $scope.backupModesforJob.name= $scope.backupModes[$scope.backupModes.indexOf(selectedBackupType)];
            $scope.backupModesforJob.original = $scope.backupModesforJob.name;
        };

        $scope.getBackupStrategies = function() {
            if ($scope.isEdit) {
                ;
            } else {
                $scope.getBackupTypeForSelectedStrategies();
            }
        };

        $scope.recurstimeModeChange = function(index,timeSelect){
            var timmerId="#recurs_spinner_field"+index;
            if(timeSelect == 0){
                $(timmerId).spinner({min: 1, max: 24});
            }else{
                $(timmerId).spinner({min: 1, max: 60});
            }
        };

        $scope.getColdBackupCopyTargets = function(){
            var url ='/api/storage/?sid=' + $scope.policy.sid + '&usage=archive';
            console.log(url);
            $http({
                method: 'GET',
                url: url
            }).success(function(data, status, headers) {
                console.log(data.storage);
                $scope.canColdBackupCopy = data.storage !== undefined && data.storage.length > 0;
                if ($scope.canColdBackupCopy) {
                    $scope.coldTargets = data.storage;
                } else {
                    $scope.coldTargets = [];
                }
            }).error(function(response) {
                console.log('do not know if storage for archiving is configured');
            });
        };

        $scope.getHotBackupCopyTargets = function(){
            var url ='/api/replication/targets/?sid=' + $scope.policy.sid;
            console.log(url);
            $http({
                method: 'GET',
                url: url
            }).success(function(data, status, headers) {
                console.log(data.targets);
                $scope.canHotBackupCopy = data.targets !== undefined && data.targets.length > 0;
                if ($scope.canHotBackupCopy) {
                    $scope.hotTargets = data.targets;
                } else {
                    $scope.hotTargets = [];
                }
            }).error(function(response) {
                console.log('do not know if replicating');
            });
        };

        $scope.getEncryptionStatus = function() {
            $scope.encryptionIsEnabled = false;
            var url ='/api/encryption/?sid=' + $scope.policy.sid;
            console.log(url);
            $http({
                method: 'GET',
                url: url
            }).success(function(data, status, headers) {
                if (data.data !== undefined && data.data[0] !== undefined) {
                    var state = data.data[0].state;
                    $scope.encryptionIsEnabled = (state === 'persist' || state === 'on');
                } else {
                    console.log('do not know if encryption enabled');
                }
            }).error(function(response) {
                console.log('do not know if encryption enabled');
            });
        };

        $scope.getExcludedDatastore = function(selectedNodeId){
            
            var url ='/api/inventory/'+selectedNodeId +'/?disks=1';
                console.log(url);
            $http({
                method: 'GET',
                url: url
            }).success(function(data, status, headers) {
                console.log(data.inventory.nodes);
                var obj = $scope.selectedAsset[selectedNodeId];
                for(var i=0;i<data.inventory.nodes.length;i++){
                    if(data.inventory.nodes[i].is_excluded){
                      obj.excl_list.push(data.inventory.nodes[i].name); 
                    }
                }
                obj.rules = $scope.setProtectionRule(obj.incl_list,obj.excl_list,obj.metanames);

            }).error(function(response) {
                console.log('api_inventory_error');
            });
            
        };
        
       
        $scope.openEditSettingServerDialog = function(row,cell,column){

            $scope.selectedAssetId = row.id;
            $scope.selectedAssetName = row.name;
            $scope.selectedNodeId = row.nodeId;
            $scope.selectedAssetOS = row.os_type_id;
            if($scope.assetType.id === INVENTORY_TYPE_FAMILY_VMWARE ||
                $scope.assetType.id === INVENTORY_TYPE_FAMILY_XEN ||
                $scope.assetType.id === INVENTORY_TYPE_FAMILY_AHV){
                 ngDialog.open({
                            template: 'app/protect/backup-job/edit-vmware.html',
                            scope: $scope,
                            overlay:true,
                            modelDialogId:'edit-vmware-dailog',
                            ngDialogStyle:'width:600px;height:520px;',
                            preCloseCallback: function (value) {}
                });
            }
            if($scope.assetType.id === INVENTORY_TYPE_FAMILY_CLIENT_NAS ||
                $scope.assetType.id === INVENTORY_TYPE_FAMILY_CLIENT ){
                var obj = inventorysvc.nodeToObject(row.nodeId);
                $scope.selectedAssetId = obj.client_id;
                ngDialog.open({
                            template: 'app/protect/backup-job/edit-asset.html',
                            scope: $scope,
                            overlay:true,
                            modelDialogId:'edit-asset-dailog',
                            ngDialogStyle:'width:600px;height:520px;',
                            preCloseCallback: function (value) {}
                });
            }
        };

        $scope.configureBackupCopy = function() {
            $scope.closeAllDialog();
            window.location.href = "#/configure";
        }

}])
.filter('icon_of', function() {
    return function (status) {
        return 'icon-phd-' + status;
    }
})
.controller('slaPolicyStatusCtrl', ['$scope', 'ngDialog','$analytics',
    function($scope, ngDialog, $analytics) {

        $scope.isEdit = false;
        var pageLabel = (window.location.href.indexOf("#/protect") !== -1) ? 'Protect' : 'Jobs';
        if (pageLabel == 'Jobs') {
            $(document).trigger("editJob");
            $scope.onJobsPage = true;
        } else {
            $scope.onJobsPage = false;
        }

        var statusTemplate = '<div class="ui-grid-cell-contents"><i class="{{ row.entity.status | icon_of }} icon"></i></div>';

        $scope.policyGridOptions = {
            enableSorting: true,
            enableFiltering: false,
            enableRowSelection: false,
            enableRowHeaderSelection: false,
            rowHeight: 28,
            enableColumnResizing: true,
            enableHorizontalScrollbar: 0,
            multiSelect: false,
            columnDefs: [
                {
                    name: 'status',
                    displayName: '',
                    cellTemplate: statusTemplate,
                    width: 24,
                    enableColumnMenu: false
                },
                {
                    name: 'name',
                    displayName: 'Job Name',
                    cellTooltip: true,
                    width: 230
                },
                {
                    name: 'type',
                    displayName: 'Job Type',
                    cellTooltip: true,
                    width: 110
                },
                {
                    name: 'message',
                    displayName: 'Message',
                    cellTooltip: true,
                    width: '*'
                }
            ],
            data: []
        };

        $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

        $scope.initCtrl = function(dlg){
            switch (dlg.name){
                // When putting an ngDialog above a UI Dialog, we need to adjust the vertical alignment (the default ngDialog top is 25%).
                case 'sla-success-dialog':
                    $scope.policyGridOptions.data = $scope.ngDialogData.policyTable;
                    $scope.isEdit = $scope.ngDialogData.isEdit;
                    $analytics && $analytics.eventTrack('Add SLA Policy', {  category: 'Backup', label: pageLabel });
                    $scope.showJobsTooltip();
                    break;
                default:
                    // No action.
                    break;
            }
        };

        $scope.showJobsTooltip = function(){
            var form = angular.element("#sla-success-form");
            form.find(".definition").popover({
                appendTo: form
            });
        };

        $scope.closeAllDialog = function() {
          ngDialog.closeAll();
        };

        $scope.viewPolicies = function () {
            $scope.closeAllDialog();
            window.location.href = "#/jobs/#4";
        };
    }])
.controller('slaPolicyDeleteCtrl', ['$scope', 'ngDialog', '$http', '$analytics',
    function($scope, ngDialog, $http, $analytics) {

        var policyId = $scope.ngDialogData.policyId;
        var selectedPolicy = $scope.ngDialogData.selectedPolicy;
        var loadSelector = $scope.ngDialogData.loadSelector;
        var analyticsSource = $scope.ngDialogData.analyticsSource;
        var policyJobTab = $scope.ngDialogData.tab;

        var sid = selectedPolicy.sid;

        $scope.deleteAll = true;


        $scope.deletePolicy = function () {
            var purgeAll = $scope.deleteAll;
            var url = '/api/sla-policies/' + policyId + '/?sid=' + sid;
            var load = PHD.showLoadingIndicator(loadSelector);
            var method = 'DELETE';
            var payload = {'purge_all': purgeAll };
            payload = JSON.stringify(payload);
            console.log(url);
            $http({
                method: method,
                url: url,
                data: payload
            }).success(function (data, status, headers) {
                PHD.hideLoadingIndicator(load);
                $analytics && $analytics.eventTrack('Delete Policy', {  category: 'Jobs', label: analyticsSource });
                ngDialog.close('delete-sla-dialog');
                policyJobTab.deleteRow(policyId);
                policyJobTab.update();
            }).error(function (response) {
                PHD.hideLoadingIndicator(load);
                var message = response !== undefined && response.result instanceof Array ? response.result[0].message : 'Unknown Server Error';
                ngDialog.open({
                    dialogType: 'Error',
                    modelDialogId:'delete-policy-error',
                    scope: $scope,
                    dialogMessage: message,
                    onConfirmOkButtonClick:'onConfirmError()'
                })
            });
        };

        $scope.onConfirmError = function () {
            ngDialog.close('delete-policy-error');
        };

        $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

        $scope.initCtrl = function(dlg){
            switch (dlg.name){
                // When putting an ngDialog above a UI Dialog, we need to adjust the vertical alignment (the default ngDialog top is 25%).
                case 'delete-sla-dialog':
                    $('.btn-cancel').focus();
                    break;
                default:
                    // No action.
                    break;
            }
        };
    }
]);