recover.controller('ImportBackupCopyCtrl', ['$scope', '$rootScope', '$http', '$analytics', 'ngDialog', 'gettextCatalog',
    function($scope, $rootScope, $http, $analytics, ngDialog, gettextCatalog) {

        // Used to unbind the watched event on dialog close.
        var unbindHandler = null;

        var selections = $scope.ngDialogData.selections;
        var instance_id = $scope.ngDialogData.instance_id;

        var loadArchivesErrorFlag = false;

        $scope.import_from_appliance = false;

        var $form, ImportForm;
        var $jobLogTable, LogTable;
        var ids_to_import = null;

        $scope.processImportDialog = function() {

            function initForm() {

                $form = $("#backup-copy-recovery-form");

                $jobLogTable = $("#backup-copy-table");

                var sid = selections.system_id;
                console.log("selections = " + JSON.stringify(selections));
                var bids = selections.backups;
                var instance = instance_id;
                var clientID = selections.client_id;

                var load;

                LogTable = PHD.DataGrid($jobLogTable, {
                    dynamic: true,
                    tableScrollOptions: {"height": 450},        /* used height to fit within fixed-height dialog */
                    select: false,
                    sortable: true,
                    sortOnLoad: true,
                    sortOnHeader: true,
                    cellClasses: [null, null, 'long-text truncate', null, null, null, null, null, null, null],
                    classifiers: ['success']
                }).registerFormatters(cellFormatter);

                if (selections.remote) {
                    $scope.import_from_appliance = true;
                    loadRemoteBackups();
                } else {
                    $scope.import_from_appliance = false;
                    loadArchives();
                }

                function loadArchives(gettext) {
                    loadArchivesErrorFlag = false;
                    var related_url = "/api/archive/related/" + bids[0] + "/?sid=" + sid + "&instance=" + instance_id + "&clientID=" + clientID;
                    load = PHD.showLoadingIndicator($("#backup-copy-recovery-dialog"));
                    var resp = PHD.Ajax.get(related_url, load, handleLoadError);
                    resp.done(function (data) {
                        var status_url = "/api/archive/status/?sid=" + sid + "&lang=" + $rootScope.userLanguageCode;
                        ids_to_import = data.ids;
                        if (ids_to_import !== undefined) {
                            var idJSON = {};
                            idJSON.archive_ids = ids_to_import.split(',').map(function (id) {
                                return parseInt(id, 10);
                            });
                            idJSON = JSON.stringify(idJSON);
                            PHD.Ajax
                                .post(status_url, idJSON)
                                .done(function (data) {
                                    if (data.status === undefined || data.status.length === 0) {
                                        var obj = {};
                                        obj.id = 0;
                                        obj.success = 'none';
                                        obj.backup_date = 'unknown';
                                        obj.instance_description = gettextCatalog.getString('The backup copies no longer exist on the appliance.');
                                        data.status = [obj];
                                    }
                                    LogTable.load(data.status);
                                });
                        } else {
                            var obj = {};
                            obj.id = 0;
                            obj.success = 'none';
                            obj.backup_date = 'unknown';
                            obj.instance_description = gettextCatalog.getString('The backup copies no longer exist on the appliance.');
                            data.status = [obj];
                            LogTable.load(data.status);
                        }
                    });
                }


                function handleLoadError(jqXHR, textStatus, errorThrown) {
                    PHD.hideLoadingIndicator(load);
                    handleError();
                    if (jqXHR.status === 500) {
                        loadArchivesErrorFlag = true;
                    } else {
                        loadArchivesErrorFlag = false;
                    }
                }


                function loadRemoteBackups(gettext) {
                    // It is a Hot Backup Copy
                    var related_url = "/api/restore/target-group/?backup_id=" + bids[0] + "&lang=" + $rootScope.userLanguageCode;
                    ids_to_import = bids[0];
                    var resp = PHD.Ajax.get(related_url);
                    resp.done(function (data) {
                        console.log(data.data);
                        if (data.data !== undefined) {
                            LogTable.load(data.data);
                        } else {
                            var obj = {};
                            obj.id = 0;
                            obj.success = 'none';
                            obj.backup_date = 'unknown';
                            obj.instance_description = gettextCatalog.getString('The backup copies no longer exist on the appliance.');
                            data.status = [obj];
                            LogTable.load(data.status);
                        }
                    });
                }

                ImportForm = PHD.FormController($form);

                ImportForm
                    .on("formsubmit", function (event, data) {
                        var message = "<p>" + gettextCatalog.getString("Importing of Backup Copies successfully started.") + "<br/><br/>" +
                            gettextCatalog.getString("After all recovery jobs have completed, please return to the Recover page to recover the imported backups.") + "</p>";
                        if (window.location.href.indexOf("#/jobs") !== -1) {
                            ngDialog.open({
                                dialogType: 'Success',
                                scope:$scope,
                                dialogMessage: message,
                                onConfirmOkButtonClick:'closeThisDialog();closeImportDialog()'
                            });
                        } else {
                            ngDialog.open({
                                dialogType: 'Success',
                                scope:$scope,
                                tertiaryButton: true,
                                tertiaryButtonLabel: gettextCatalog.getString("View Jobs"),
                                onConfirmOkButtonClick:'closeThisDialog();closeImportDialog()',
                                onTertiaryButtonClick: 'viewJobs()',
                                dialogMessage: message
                            });
                        }
                    });

                ImportForm.submit = function () {
                    var self = this,
                        load = PHD.showLoadingIndicator("body", true, gettextCatalog.getString("Starting Import..."));

                    if ($scope.import_from_appliance) {
                        reverseReplicateCopies(self, load);
                    } else {
                        importCopies(self, load);
                    }

                    return this;
                };
            }

            function reverseReplicateCopies(self, load) {
                var restoreObj = {};
                restoreObj.backup_id = ids_to_import;
                restoreObj.target = $rootScope.local.name;
                restoreObj = JSON.stringify(restoreObj);
                console.log('=== Submit form: object = ' + restoreObj);
                var url = "/api/replication/target-queue/";
                PHD.Ajax
                    .post(url, restoreObj, null, handleError)
                    .done(function (data) {
                        self.$form.trigger("formsubmit", [data, self.$form, self]);
                    })

                    .always(function () {
                        self.$buttons.enable();
                        PHD.hideLoadingIndicator(load);
                    });
            }

            function importCopies(self, load) {
                PHD.appliance_sid = selections.system_id;
                var restoreObj = {};
                restoreObj.archive_ids = ids_to_import;
                restoreObj.association = "dr";              // associates with an instance if it exists, creates one if it does not.
                restoreObj.check_media = true;
                restoreObj = JSON.stringify(restoreObj);

                var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
                var url = "/api/restore/archive/full/" + sid;
                console.log("POST url is " + url);
                PHD.Ajax
                    .post(url, restoreObj, load, handleError)
                    .done(function (data) {
                        self.$buttons.enable();
                        self.$form.trigger("formsubmit", [data, self.$form, self]);
                    })

                    .always(function () {
                        PHD.hideLoadingIndicator(load);
                    });
            }

            function handleError(jqXHR, textStatus, errorThrown) {
                if (jqXHR.status === 500) {
                    var data = jqXHR.responseJSON;
                    if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                        var errorText = data.result[0].message;
                        ngDialog.open({
                            dialogType:'Error',
                            dialogMessage: errorText,
                            scope:$scope,
                            onConfirmOkButtonClick:'closeThisDialog();closeImportDialog()'
                        })
                    } else {
                        PHD.throwError(data);
                    }
                } else {
                    PHD.ajaxError(jqXHR, textStatus, errorThrown);
                }
            }

            var cellFormatter = {
                instance_description: function (value) {
                    var title = _.escape(value);
                    return '<span class="truncates item-name" title="' + title + '">' + value + '</span>';
                },
                success: function (value, row) {
                    var iconClass = value == undefined ? 'warning' : value === true ? "success" : "error";
                    return '<i class="icon-phd-' + iconClass + ' icon-large" title="' + iconClass + '"></i>';
                    //return '<span class="storage-status">' + statusIcon + '</span>';
                },
                encrypted: function (value, row) {
                    return (value === true) ? "Yes" : "No";
                },
                size: function (value, row) {
                    return (value / 1024).toFixed(2);
                }
            };

            /*
            *   DOM Ready Function
            */
            initForm();

        };

        $scope.viewJobs = function () {
            $scope.closeImportDialog();
            ngDialog.closeAll();
            window.location.href = "#/jobs/#0";
        };

        $scope.closeImportDialog = function() {
            if ( _.isFunction(unbindHandler)) {
                unbindHandler();
            }
            ngDialog.close('import-backup-copy-dialog');
        };

        // Initialize the controller, returning the function to turn off the listener.
        unbindHandler = $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

        $scope.initCtrl = function(dlg){
            switch (dlg.name){
                // When putting an ngDialog above a UI Dialog, we need to adjust the vertical alignment (the default ngDialog top is 25%).
                case 'import-backup-copy-dialog':
                    $scope.processImportDialog();
                    break;
                default:
                    // No action.
                    break;
            }
        };
    }
]);
