function processBackupFilter(PHD, window, ngDialog, $rootScope, $scope, $http, localStorageService, gettextCatalog) {

        /*
         *   BackupFilter defines behavior for the controls used to filter backups
         *   from the backup catalog
         */

        var BackupFilter = function BackupFilter($form, catalog) {
            if(this instanceof BackupFilter) {
                this.$form = $form;
                this.catalog = catalog;
                this.data = catalog.tree;
                this.trueForm = this.$form.is("form");
            } else {
                return new BackupFilter($form, catalog).init();
            }
        };

        var errorTooltipOptions = {
            disabled: true,
            position: {
                my: "right-10 center",
                at: "left center"
            },
            tooltipClass: "error-tooltip",
            track: false
        };

        BackupFilter.prototype = {
            init: function() {
                var self = this;

                this.$name = this.$form.find(".filter-name").data("fid", "name");
                this.$environments = this.$form.find(".filter-environment").data("fid", "env");
                this.$hosts = this.$form.find(".filter-host").data("fid", "host");
                this.$applications = this.$form.find(".filter-application").data("fid", "app_type");
                this.$types = this.$form.find(".filter-type").data("fid", "type");
                this.$recent = this.$form.find(".filter-most-recent").data("fid", "recent");
                this.$pinned = this.$form.find(".filter-pinned").data("fid", "pin");
                this.$filters = this.$form.find(".filter-list").data("fid", "list");

                this.$form.find(".definition").popover({
                    appendTo: this.$form
                });

                this.$success = this.$form.find(".filter-success").data("fid", "status");
                this.$backupType = this.$form.find(".filter-backup-type").data("fid", "backup-type");

                this.initFromToFilters();

                // lmc - initialize from date same as to date.
                this.$fromDate.val(moment().format(DATE_DISPLAY_FORMAT));
                // lmc - initialize to now.
                this.$toDate.val(moment().format(DATE_DISPLAY_FORMAT));

                this.$submit = this.$form.find(".filter-submit").on("click", $.proxy(this.submitFilter, this));

                this.$clear = this.$form.find(".filter-clear").on("click", $.proxy(this.clearFilter, this));

                this.$addFilter = this.$form.find(".filter-add").on("click", $.proxy(this.addFilter, this));

                this.$saveFilter = this.$form.find(".filter-save").on("click", $.proxy(this.saveFilter, this, { isEdit: true }));

                this.$deleteFilter = this.$form.find(".filter-delete").on("click", $.proxy(this.deleteFilter, this));

                this.$selectFilter = this.$form.find(".filter-list").on("change", $.proxy(this.changeFilter, this));

                this.$form.on("keydown", "input, select", function(e) {
                    e.stopPropagation();
                    if(e.keyCode == $.ui.keyCode.ENTER) {
                        self.submitFilter();
                    }
                    if(e.keyCode == $.ui.keyCode.ESCAPE) {
                        self.clearFilter();
                    }
                });

                this.$recent.on("change", $.proxy(this.handleRecent, this));

                this.$toDate.closest(".control-group")
                    .tooltip(errorTooltipOptions);

                this.$fromDate.closest(".control-group")
                    .tooltip(errorTooltipOptions);


                this.startLoadedDate = this.$fromDate.val();
                this.endLoadedDate = this.$toDate.val();

                this.loadedLevel = this.getLoadLevel();

                this.loadedSID = "&sid=" + ($rootScope.local !== undefined ? $rootScope.local.id : "1");

                this.defaultFilterName = ''; // default filter name.
                this.defaultRange = 0;      // default date range.

                // lmc - allow submit, only disallow while loading more data, i.e., needMoreData returned true.
                this.submitAllowed = true;

                this.showPinned = true;     // show pinned backups

                // prevent stack recursion overflow.
                this.retries = 0;
                return this;
            },

            handleRecent: function() {
                var checked = this.$recent.prop("checked");
                this.$toDate.prop("disabled", checked);
                this.$fromDate.prop("disabled", checked);
            },

            refresh: function(data) {
                this.data = data;
                return this.collectOptions();
            },

            collectOptions: function() {
                this.updateEnvironmentsDropdown(this.$environments, this.catalog.getEnvironments(), $scope.catalogFilter.applianceName);
                this.resetDropdown(this.$hosts, this.catalog.getHosts());
                this.resetDropdown(this.$applications, this.catalog.getApplications());
                this.resetDropdown(this.$types, this.catalog.getTypes());
                this.updateFilterDropdown(this.$filters, $rootScope.catalogFilters, $scope.catalogFilter.name);
                this.updateHoldCheckbox(this.showPinned);
                return this;
            },

            serializeForm: function() {
                return this.$form.find("input, select").serialize();
            },

            submitFilter: function() {
                var fns = [],
                    datesValid = this.validateDates();

                console.log('submitFilter');
                if(!datesValid) return;

                var environment = this.$environments.val();
                if (environment && environment !== undefined) {
                    $scope.catalogFilter.applianceName = this.$environments.val();
                }

                if (this.submitAllowed && this.needMoreData()) {
                    this.$form.trigger('needMoreData');
                    this.submitAllowed = false;
                    return true;
                } else {
                    this.submitAllowed = true;
                }

                console.log('submitFilter: after needMoreData');

                fns.push(function(node) {
                    if(node.parent && node.parent.filtered.length) {
                        node.filtered.push("filtered-parent");
                        return true;
                    }
                    return false;
                });

                if (this.$backupType.val()) {
                    var keep = this.$backupType.val();
                    fns.push(function(node) {
                        if (node.type === 'backup' && node.replicated) {
                            if (keep.indexOf('replicated') === -1) {
                                node.filtered.push("filtered-backup-type");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            if ((node.type === 'backup' || node.type === 'archive' || node.type === 'imported') && keep.indexOf(node.type) === -1) {
                                node.filtered.push("filtered-backup-type");
                                return true;
                            } else {
                                return false;
                            }
                        }
                    });
                }

                if(this.$pinned.is(":checked")) {
                    fns.push(function(node) {
                        if((node.type === 'backup' || node.type === 'archive' || node.type === 'imported') && !node.legal_hold) {
                            node.filtered.push("filtered-pinned");
                            return true;
                        } else {
                            return false;
                        }
                    });
                }

                if(this.$success.is(":checked")) {
                    fns.push(function(node) {
                        if ((node.type === 'backup' || node.type === 'archive' || node.type === 'imported') && (node.status == 'Failed' || node.status == 'Warning')) {
                            node.filtered.push("filtered-successes");
                            return true;
                        } else {
                            return false;
                        }
                    });
                }

                if(this.$recent.is(":checked")) {
                    fns.push(function(node) {
                        if(node.type === 'backup' && !node.last) {
                            node.filtered.push("filtered-recent");
                            return true;
                        } else {
                            return false;
                        }
                    });
                }

                if(this.$name.val() !== undefined && this.$name.val() !== "") {
                    var nameQuery = this.$name.val().toLowerCase();
                    fns.push(function(node) {
                        if(node.type === 'vm' || node.type === 'compute_resource') {
                            var subject = node.name.toLowerCase();
                            if(subject.indexOf(nameQuery) === -1) {
                                node.filtered.push("filtered-name");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    });
                }

                if (this.$environments.val() && this.$environments.val() !== "") {
                    var envQuery = this.$environments.val();
                    fns.push(function(node) {
                        if(node.type === 'vm' || node.type === 'compute_resource') {
                            var subject = node.system_name;
                            if(envQuery !== 'All' && subject !== envQuery) {
                                node.filtered.push("filtered-env");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    });
                }

                if (this.$hosts.val() !== undefined && this.$hosts.val() !== "") {
                    var hostQuery = this.$hosts.val();
                    fns.push(function(node) {
                        if(node.type === 'vm' || node.type === 'compute_resource') {
                            var subject = node.host ? node.host : "";
                            if(subject !== hostQuery) {
                                node.filtered.push("filtered-host");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    });
                }

                if (this.$applications.val() !== undefined && this.$applications.val() !== "") {
                    var applicationQuery = this.$applications.val();
                    fns.push(function(node) {
                        if (node.type === 'backup' || node.type === 'archive' || node.type === 'imported') {
                            var subject = node.app_name ? node.app_name : "";
                            if(subject !== applicationQuery) {
                                node.filtered.push("filtered-application");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    });
                }

                if (this.$types.val() !== undefined && this.$types.val() !== "") {
                    var typeQuery = this.$types.val();
                    fns.push(function(node) {
                        if (node.type === 'backup' || node.type === 'archive' || node.type === 'imported') {
                            var subject = node.backup_mode;
                            if (subject !== typeQuery) {
                                node.filtered.push("filtered-type");
                                    return true;
                                } else {
                                    return false;
                                }
                        } else {
                            return false;
                        }
                    });
                }

                if (this.$fromDate.val() !== undefined  && this.$fromDate.val() !== "" && !this.$fromDate.prop("disabled")) {
                    var fromQuery = moment(this.$fromDate.val(), DATE_DISPLAY_FORMAT);
                    fns.push(function(node) {
                        if(node.type === 'backup' || node.type === 'archive' || node.type === 'imported') {
                            var subject = moment(node.start_date, DATE_DISPLAY_FORMAT);
                            if(subject.isBefore(fromQuery)) {
                                node.filtered.push("filtered-datefrom");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    });
                }

                if (this.$toDate.val() !== undefined && this.$toDate.val() !== "" && !this.$toDate.prop("disabled")) {
                    var toQuery = moment(this.$toDate.val(), DATE_DISPLAY_FORMAT);
                    //3.14.15:sab:UNIBBP-1859.
                    toQuery.add(1, 'days').subtract(1, 'seconds');
                    fns.push(function(node) {
                        if(node.type === 'backup' || node.type === 'archive' || node.type === 'imported') {
                            var subject = moment(node.start_date, DATE_DISPLAY_FORMAT);
                            if(subject.isAfter(toQuery)) {
                                node.filtered.push("filtered-dateto");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    });
                }

                this.runFilter(this.data, fns);
                this.catalog.filter();
            },

            runFilter: function(nodes, fns) {
                var self = this,
                    all = true;

                if(fns.length) {
                    nodes.forEach(function(node) {
                        var filtered = false;
                        node.filtered = [];
                        fns.forEach(function(fn) {
                            if(fn(node)) filtered = true;
                        });
                        if(node.children) {
                            if(self.runFilter(node.children, fns)) {
                                node.filtered.push("filtered-children");
                                filtered = true;
                            }
                        }
                        if(!filtered) {
                            all = false;
                        }
                    });
                    return all;
                }
                return false;
            },

            clearFilter: function() {
                this.$form.find("[type=text]").val("");
                this.$form.find("[type=checkbox]").prop("checked", false);
                this.$success.prop("checked", true);
                this.$form.find("select").prop("selectedIndex", 0);
                this.$backupType.find("option[value='backup']").prop("selected", true);
                this.$backupType.find("option[value='imported']").prop("selected", true);
                this.$backupType.find("option[value='archive']").prop("selected", false);
                this.$backupType.find("option[value='replicated']").prop("selected", false);
                this.clearDates();
                this.submitFilter();
            },

            addFilter: function(event) {
                this.saveFilter({ isEdit: false }, event);
            },

            /*
             * Validate and save the new filter options, then persist the filters as a preference.
             */
            saveFilter: function(attr, event) {
                var isEdit = attr.isEdit;

                var to = this.$toDate.val();
                var from = this.$fromDate.val();

                var toRange = moment(to, DATE_DISPLAY_FORMAT);
                var fromRange = moment(from, DATE_DISPLAY_FORMAT);

                var range = toRange.diff(fromRange, 'days');
                if (range < 0) {
                    ngDialog.open({
                        dialogType: 'error',
                        dialogMessage: gettextCatalog.getString("'From Date' must be before 'To Date'")
                    })
                } else {
                    $scope.filterRange = range;
                    if (isEdit) {
                        // Get and save the current name as original, on edit.
                        $scope.catalogFilter.name = this.$selectFilter.val();
                        $scope.catalogFilter.originalName = $scope.catalogFilter.name;
                        $scope.catalogFilter.dialogTitle = gettext("Edit Filter");  /* using gettext as translate filter is applied in the dialog */
                    } else {
                        // Create a new unique filter name.
                        $scope.catalogFilter.name = 'New Filter-' + (new Date().getTime() % 1000).toString();
                        $scope.catalogFilter.originalName = null;
                        $scope.catalogFilter.dialogTitle = gettext("Add Filter");  /* using gettext as translate filter is applied in the dialog */
                    }
                    var filters = $rootScope.catalogFilters;
                    var filter = this.findFilter(filters, $scope.catalogFilter.name);
                    $scope.catalogFilter.isDefault = filter !== null ? filter.is_default : false;
                    $scope.catalogFilter.typesString = this.buildTypesString();
                    $scope.catalogFilter.appliance = (this.$environments.val() === $rootScope.local.name) ? 'Load Local Only' : 'Load All';
                    $scope.catalogFilter.range = $scope.filterRange + 1;
                    ngDialog.open({
                        template: 'app/recover/catalog_filters/filter_save.html',
                        name: 'catalog-filter-save',
                        scope: $scope,
                        overlay:true,
                        ngDialogStyle:'width:390px;',
                        modelDialogId : 'catalog-filter-save-dialog',
                        closeByDocument: false,
                        closeByEscape: true,
                        showClose: false
                    });
                }
            },

            /*
             * Given the types array, build a string with the names, mapping names as needed.
             */
            buildTypesString: function() {
                var types = this.$backupType.val();
                var typesString = "";
                var sep = "";
                for (var i = 0; i < types.length; i++) {
                    var type = types[i];
                    if (type === 'archive') {
                        type = 'cold copy';
                    } else if (type === 'replicated') {
                        type = 'hot copy';
                    }
                    typesString += sep + type;
                    sep = ",";
                }
                return typesString;
            },

            /*
             * Given an array of filters, find the filter with the given name.
             */
            findFilter: function(filters, findFilterName) {
                var filter = null;
                for (var i = 0; i < filters.length; i++) {
                    if (filters[i].name === findFilterName) {
                        filter = filters[i];
                        break;
                    }
                }
                return filter;
            },

            /*
             * Given an array of filters, find the default one.
             */
            findDefaultFilter: function(filters) {
                var filter = null;
                if (filters !== undefined) {
                    for (var i = 0; i < filters.length; i++) {
                        if (filters[i].is_default === true) {
                            filter = filters[i];
                            break;
                        }
                    }
                }
                return filter;
            },

            /*
             * Called when closing the filter save dialog, to get final save info for saving.
             */
            finishFilterSave: function() {
                var newName = ($scope.catalogFilter.name).trim();
                if (newName !== "") {
                    var options = {'range': $scope.filterRange, 'types' : this.$backupType.val()};
                    var applianceName = this.$environments.val();
                    if (applianceName === $rootScope.local.name) {
                        options.appliance = this.$environments.val();
                    }
                    var filters = $rootScope.catalogFilters;
                    var filter = this.findFilter(filters, $scope.catalogFilter.originalName);
                    if (filter !== null) {
                        filter.options = options;
                        filter.name = newName;
                        if ($scope.catalogFilter.isDefault === true) {
                            var currentDefault = this.findDefaultFilter(filters);
                            if (currentDefault !== null) {
                                currentDefault.is_default = false;
                            }
                            filter.is_default = true;
                            this.setDefaultFilterName(filter.name);
                        } else {
                            filter.is_default = false;
                        }
                    } else {
                        // a new filter, make sure name the new name is unique.
                        filter = this.findFilter(filters, newName);
                        if (filter !== null) {
                            ngDialog.open({
                                dialogType: 'retry',
                                dialogMessage: gettextCatalog.getString("A filter named") + ' ' + newName + ' ' + gettextCatalog.getString('already exists') + '.'
                            });
                            // Don't close the dialog, don't persist the filters.
                            return;
                        } else {
                            if ($scope.catalogFilter.isDefault === true) {
                                currentDefault = this.findDefaultFilter(filters);
                                if (currentDefault !== null) {
                                    // Changing default from another filter.
                                    currentDefault.is_default = false;
                                }
                            }
                            var newFilter = { 'name': newName, 'is_default': $scope.catalogFilter.isDefault, 'options': options };
                            filters.push(newFilter);
                        }
                    }
                    ngDialog.close('catalog-filter-save-dialog');
                    this.persistFilters(filters);
                } else {
                    ngDialog.open({
                        dialogType: 'retry',
                        dialogMessage: gettextCatalog.getString("Filters must have non-empty names.")
                    })
                }
            },

            persistFilters: function(filters) {
                var self = this;
                var putData = {};
                var prefURL = '/api/preferences/';
                putData['catalog_filter'] = JSON.stringify(filters);
                prefURL = prefURL + encodeURI($rootScope.user.name)+"/";
                $http ({
                    method: 'put',
                    data: JSON.stringify(putData),
                    url : prefURL
                }).success(function(data){
                    $rootScope.catalogFilters = filters;
                    self.updateFilterDropdown(self.$filters, $rootScope.catalogFilters, $scope.catalogFilter.name);
                    localStorageService.set('catalogFilters', $rootScope.catalogFilters);
                }).error(function(response) {
                    ngDialog.open({
                        dialogType: 'error',
                        dialogMessage: gettextCatalog.getString("Error saving filters; please try again.")
                    })
                });
            },

            deleteFilter: function() {
                var filters = $rootScope.catalogFilters;
                var selectedFilter = this.$selectFilter.val();
                var filter = this.findFilter(filters, selectedFilter);
                if (filter !== null) {
                    if (filter.is_default === true) {
                        ngDialog.open({
                            dialogType: 'error',
                            dialogMessage: gettextCatalog.getString("You cannot delete the default filter.")
                        });
                    } else {
                        ngDialog.open({
                            template: 'app/recover/catalog_filters/filter_delete.html',
                            name: 'catalog-filter-delete',
                            scope: $scope,
                            overlay:true,
                            ngDialogStyle:'width:400px;height:130px',
                            modelDialogId : 'catalog-filter-delete-dialog',
                            closeByDocument: false,
                            closeByEscape: true,
                            showClose: false
                        });
                    }
                } else {
                    ngDialog.open({
                        dialogType: 'error',
                        dialogMessage: gettextCatalog.getString("Cannot find the filter to delete.  Please refresh the page.")
                    });
                }
            },

            finishFilterDelete: function() {
                var filters = $rootScope.catalogFilters;
                var selectedFilter = this.$selectFilter.val();
                var found = false;
                for (var i = 0; i < filters.length; i++) {
                    var filter = filters[i];
                    if (filter.name === selectedFilter) {
                        filters.splice(i, 1);
                        found = true;
                        break;
                    }
                }
                ngDialog.close('catalog-filter-delete-dialog');
                if (found) {
                    // set name to undefined so will switch to the default.
                    $scope.catalogFilter.name = undefined;
                    $scope.catalogFilter.applianceName = undefined;
                    this.persistFilters(filters);
                }
            },

            /*
             * When a filter is selected, update the affected fields, and run it.
             */
            changeFilter: function() {
                var filters = $rootScope.catalogFilters;
                $scope.catalogFilter.name = this.$selectFilter.val();
                var selectedFilter = $scope.catalogFilter.name;
                if (filters !== undefined) {
                    for (var i = 0; i < filters.length; i++) {
                        if (selectedFilter === filters[i].name) {
                            var options = filters[i].options;
                            if (options.asset !== undefined) {
                                this.$name.val(options.asset);
                            }
                            if (options.appliance !== undefined) {
                                $scope.catalogFilter.applianceName = options.appliance;
                                this.$environments.val(options.appliance);
                            } else {
                                this.$environments.val('All');
                            }
                            if (options.host !== undefined) {
                                this.$hosts.val(options.host);
                            }
                            if (options.application !== undefined) {
                                this.$applications.val(options.application);
                            }
                            if (options.types !== undefined) {
                                this.$backupType.val(options.types);
                            }
                            if (options.range !== undefined) {
                                var dateRange = parseInt(options.range);
                                this.$fromDate.val(moment().subtract(dateRange, 'd').format(DATE_DISPLAY_FORMAT));
                                // lmc - initialize to now.
                                this.$toDate.val(moment().format(DATE_DISPLAY_FORMAT));
                            }
                        }
                    }
                    this.submitFilter();
                }
            },

            getRetries: function() {
                return this.retries;
            },

            setRetries: function(value) {
                this.retries = value;
            },

            clearDates:  function(){
                this.$toDate
                    .removeClass("error")
                    .closest(".control-group")
                    .attr("title", "")
                    .tooltip("option", "disabled", true)
                    .removeClass("error");
                this.$fromDate
                    .removeClass("error")
                    .closest(".control-group")
                    .attr("title", "")
                    .tooltip("option", "disabled", true)
                    .removeClass("error");

                // lmc - initialize to today.
                this.$fromDate.val(moment().format(DATE_DISPLAY_FORMAT));
                // lmc - initialize to now.
                this.$toDate.val(moment().format(DATE_DISPLAY_FORMAT));
            },

            /*
             * Initialize the date pickers for the catalog from/to filters.
             */
            initFromToFilters: function() {
                this.$fromDate = this.$form.find(".filter-date-from").datepicker({
                    showOn: "both",
                    buttonText: '<img src="assets/images/calendar.png" width="24" height="24">',
                    dateFormat: DATE_FORMAT
                }).data("fid", "from").attr("placeholder", DATE_DISPLAY_FORMAT);

                this.$toDate = this.$form.find(".filter-date-to").datepicker({
                    showOn: "both",
                    buttonText: '<img src="assets/images/calendar.png" width="24" height="24">',
                    dateFormat: DATE_FORMAT
                }).data("fid", "to").attr("placeholder", DATE_DISPLAY_FORMAT);
            },

            /*
             * Re-create the date pickers, reset dates to the default, ensure the API is called, and pull data.
             * This is used when the user changes their language setting.
             */
            resetDateFilter: function() {
                // Destroy the prior filters
                $(".filter-date-from").datepicker('destroy');
                $(".filter-date-to").datepicker('destroy');

                // Re-create and clear the dates.
                this.initFromToFilters();
                this.clearDates();

                // Reset loadedSID so that new data will load.
                this.loadedSID = DUMMY_URL_SID;
                this.changeFilter();
            },

            getStartDate: function() {
                var from = this.$fromDate.val();
                var mFrom = null;
                if (from !== "") {
                    mFrom = moment(from, DATE_DISPLAY_FORMAT).format(DATE_DISPLAY_FORMAT);
                }
                return mFrom;
            },

            getEndDate: function() {
                var to = this.$toDate.val();
                var mTo = null;
                if (to !== "") {
                    mTo = moment(to, DATE_DISPLAY_FORMAT).format(DATE_DISPLAY_FORMAT);
                }
                return mTo;
            },

            validDateRange: function() {
                return true;
            },

            setDataRange: function(startDate, endDate) {
                this.startLoadedDate = startDate;
                this.endLoadedDate = endDate;
            },

            getLoadLevel: function() {
                var types = this.$backupType.val();
                var level = BACKUP_CATALOG_LOAD_BACKUPS;
                if (types.indexOf('archive') !== -1) {
                    if (types.indexOf('replicated') !== -1) {
                        level = BACKUP_CATALOG_LOAD_BACKUPS_COLD_HOT_COPIES;
                    } else {
                        level = BACKUP_CATALOG_LOAD_BACKUPS_COLD_COPIES;
                    }
                } else if (types.indexOf('replicated') !== -1) {
                    level = BACKUP_CATALOG_LOAD_BACKUPS_COLD_HOT_COPIES;
                }
                return level;
            },

            setLoadLevel: function(level) {
                this.loadedLevel = level;
            },

            getSystemID: function() {
                var sid = DUMMY_URL_SID;   /* initialize to dummy value, no data loaded */
                var applianceName = this.$environments.val();
                if (applianceName !== null && applianceName !== "") {
                    if (applianceName !== $rootScope.local.name) {
                        // Could be 'All' or any appliance, load all systems.
                        sid = "";
                    } else {
                        // load the local only.
                        sid = "&sid=" + $rootScope.local.id;
                    }
                }
                return sid;
            },

            setSystemID: function(system) {
                this.loadedSID = system;
            },

            /*
             * Returns true if the requested date range is outside the boundaries of what we have loaded in the catalog, false otherwise.
             */
            needMoreData: function() {
                var to = this.$toDate.val();
                var from = this.$fromDate.val();

                var toRange = moment(to, DATE_DISPLAY_FORMAT);
                var fromRange = moment(from, DATE_DISPLAY_FORMAT);

                var toLoaded = moment(this.endLoadedDate, DATE_DISPLAY_FORMAT);
                var fromLoaded = moment(this.startLoadedDate, DATE_DISPLAY_FORMAT);

                var applianceName = this.$environments.val();

                if ((fromRange.diff(fromLoaded) < 0) || (toRange.diff(toLoaded) > 0)){
                    return true;
                } else if (this.loadedLevel < this.getLoadLevel()) {
                    return true;
                } else if (applianceName !== $rootScope.local.name && this.loadedSID !== "" ||
                            applianceName === $rootScope.local.name && this.loadedSID === DUMMY_URL_SID) {
                    // See how much appliance data is loaded.
                    // If local name, load if not loaded.  If not local name, load if not all appliances loaded.
                    return true;
                }

                return false;
            },

            resetDropdown: function($select, values, value) {
                var allValue = value || "";
                $select.html('<option value="' + allValue + '" selected="selected">'+gettext("All")+'</option>');
                if(values && _.isArray(values)) {
                    var options = values.map(function(value) {
                        return value === "" ? null : '<option value="' + value + '">' + value + '</option>';
                    }).join("");
                    $select.append($(options));
                }
                return this;
            },

            updateEnvironmentsDropdown: function($select, appliances, selectedAppliance) {
                if (selectedAppliance !== undefined) {
                    $select.html('<option value="All" selected="selected">'+gettext("All")+'</option>');
                    if(appliances !== undefined && appliances.length > 0) {
                        for (var i = 0; i < appliances.length; i++) {
                            var option = '<option value="' + appliances[i] + '"';
                            if (appliances[i] === selectedAppliance) {
                                option += ' selected="selected"';
                            }
                            option += '>' + appliances[i] + '</option>';
                            $select.append(option);
                        }
                    } else {
                        option = '<option value="' + selectedAppliance + '" selected="selected">' + selectedAppliance + '</option>';
                        $select.append(option);
                    }
                } else {
                    if (appliances !== undefined) {
                        this.resetDropdown($select, appliances, "All");
                    }
                }
                return this;
            },

            setDefaultFilterName: function(name) {
                this.defaultFilterName = name;
            },

            getDefaultFilterName: function() {
                return this.defaultFilterName;
            },

            resetFilterDropdown: function($select, filters) {
                var foundNewDefault = false;
                $select.html('');
                if(filters !== undefined && _.isArray(filters)) {
                    for (var i = 0; i < filters.length; i++) {
                        var option = '<option value="' + filters[i].name + '"';
                        if (filters[i].is_default === true || (i === filters.length - 1 && foundNewDefault === false)) {
                            option += ' selected="selected"';
                            if (filters[i].name !== this.defaultFilterName) {
                                foundNewDefault = true;
                                filters[i].is_default = true;
                            }
                        }
                        option += '>' + filters[i].name + '</option>';
                        $select.append(option);
                    }
                }
                if (foundNewDefault) {
                    this.changeFilter();
                }
                return this;
            },

            updateFilterDropdown: function($select, filters, selectedFilter) {
                if (selectedFilter !== undefined) {
                    $select.html('');
                    if(filters !== undefined && _.isArray(filters)) {
                        for (var i = 0; i < filters.length; i++) {
                            var option = '<option value="' + filters[i].name + '"';
                            if (filters[i].name === selectedFilter) {
                                option += ' selected="selected"';
                            }
                            option += '>' + filters[i].name + '</option>';
                            $select.append(option);
                        }
                    }
                } else {
                    this.resetFilterDropdown($select, filters);
                }
                return this;
            },

            setShowPinned: function(showPinned) {
                if (showPinned === true || showPinned === false) {
                    this.showPinned = showPinned;
                }
            },

            updateHoldCheckbox: function(showPinned) {
                if (showPinned === false) {
                    this.$pinned.prop('disabled', true);
                    this.$pinned.prop('title', 'There were too many backups loaded to return "Held" status.\nPlease use the backup browser to see which backups are on-hold.');
                } else {
                    this.$pinned.prop('disabled', false);
                    this.$pinned.prop('title', '');
                }
            },

            buildDropdown: function($select, items) {
                items.forEach(function(item) {
                    $select.append($("<option/>", { value: item.id, html: item.name }));
                });
                return this;
            },

            validateDates: function() {
                var to = this.$toDate.val(),
                    from = this.$fromDate.val(),
                    mTo, mFrom, valid = true;

                if(to) {
                    if(PHD.util.isValidDate(to)) {
                        mTo = moment(to, DATE_DISPLAY_FORMAT);
                        this.$toDate
                            .removeClass("error")
                            .closest(".control-group")
                            .attr("title", "")
                            .tooltip("option", "disabled", true)
                            .removeClass("error");
                    } else {
                        this.$toDate
                            .addClass("error")
                            .closest(".control-group")
                            .attr("title", "Enter a valid date")
                            .tooltip("option", "disabled", false)
                            .addClass("error");
                        valid = false;
                    }
                }

                if(from) {
                    if(PHD.util.isValidDate(from)) {
                        mFrom = moment(from, DATE_DISPLAY_FORMAT);
                        this.$fromDate
                            .removeClass("error")
                            .closest(".control-group")
                            .attr("title", "")
                            .tooltip("option", "disabled", true)
                            .removeClass("error");
                    } else {
                        this.$fromDate
                            .addClass("error")
                            .closest(".control-group")
                            .attr("title", "Enter a valid date")
                            .tooltip("option", "disabled", false)
                            .addClass("error");
                        valid = false;
                    }
                }

                if(valid && mTo && mFrom) this.swapDates(from, to, mFrom, mTo);

                return valid;
            },

            swapDates: function(from, to, mFrom, mTo) {
                if(mTo.isBefore(mFrom)) {
                    /*
                     this.$toDate.val(from);
                     this.$fromDate.val(to);
                     */

                    this.$toDate.add(this.$fromDate)
                        .addClass("error")
                        .closest(".control-group")
                        .attr("title", "'From Date' must be before 'To Date'")
                        .tooltip("option", "disabled", false)
                        .addClass("error");
                }
            }
        }

        PHD.BackupFilter = BackupFilter;
}
