var fileLevelRecoveryModule = angular.module('fileLevelRecoveryModule', ['ngDialog', 'angulartics', 'angulartics.google.analytics','ui.grid','ui.grid.selection']);

fileLevelRecoveryModule.config(['ngDialogProvider', function (ngDialogProvider) {
    ngDialogProvider.setDefaults({
        plain: false,
        closeByDocument: true,
        closeByEscape: true,
        appendTo: false
    });
}]);

fileLevelRecoveryModule.controller('fileLevelRecoveryCtrl', ['$scope', '$rootScope', '$http', '$timeout', 'ngDialog', 'wizardService', '$analytics', 'rolesSvc', 'gettextCatalog',
    function($scope, $rootScope, $http, $timeout, ngDialog, wizardService, $analytics, rolesSvc, gettextCatalog) {

        var filePath="";
        var totalListItems = 0;
        $scope.flr_flag = "";
        $scope.flr_target = "";
        $scope.filesOnly = false;
        $scope.fileSizeMap = [];
        $scope.fileDirMap = [];
        $scope.backupSize = 0;
        $scope.MAX_FILES_TO_LOAD = 200;
        $scope.nextValue = "Next";
        $scope.synthCapable = selectedBackup.synth_capable ? (selectedBackup.remote === 'true'? "target-synthesized-files" : "synthesized-files") :
                                                             (selectedBackup.remote === 'true' ? 'target-files' : "files");
        $scope.FileLevelRecoveryOptionForm = $("#flr-file-option-form");
        $scope.isPhysicalServerRecovery = (selectedBackup.instance_id === undefined) ? true:false;
        $scope.system_id = selectedBackup.replicated == "true" ? selectedBackup.local_system_id : selectedBackup.sid;
        $scope.browseOnly = false;  // block restore but allow browse - was iSeries, but it is now blocked on the recover page.
        $scope.selectAssetMessage = (selectedBackup.replicated == 'true') ?
                gettextCatalog.getString("Select an asset on the target appliance.") :
                gettextCatalog.getString("Select an asset or default to the original.");

        // NDMP variables
        $scope.ndmpRestoreTarget = {};
        $scope.ndmpRestoreTarget.asset = "";
        $scope.ndmpRestoreTarget.assetName = "";
        $scope.ndmpRestoreTarget.directory = "";
        $scope.ndmpTargets = [];
        $scope.ndmpTargetVolumes = [];
        $scope.ndmpRestoreTargetVolume = {};
        $scope.ndmpRestoreTargetVolume.instance_id = "";
        $scope.ndmpRestoreTargetVolume.assetName = "";

        $scope.initializeController = function () {
            if((selectedBackup.backup_id != undefined) && (selectedBackup.backup_id != null) && (selectedBackup.backup_id != "")){
                $scope.flr_flag = "Backup";
                if (selectedBackup.remote === "true") {
                    $scope.nextValue = "Save";
                    $scope.flr_flag = "HotBackupCopy";
                    $scope.backupSize = selectedBackup.backup_size;
                    //$scope.filesOnly = true;
                }
                $scope.initFileBrowserTree();
            }else if((selectedBackup.archive_id != undefined) && (selectedBackup.archive_id != null) && (selectedBackup.archive_id != "")){
                $scope.flr_flag = "BackupCopy";
                $scope.initFileBrowserTreeFLRBackupCopy();
            }
        };

        $scope.initializeRecoveryOptionTab = function(){
            $scope.getRestoreBackupTarget();
            var suffixArray = rolesSvc.getSuffixArray($rootScope.user.role_recover_options);
            for (var j = 0; j < suffixArray.length; j++) {
                $("#id_add").val(suffixArray[j]);
                $scope.addExcludePatternRow(true);
            }
        };

        $scope.initializeNDMPRecoveryOptionTab = function() {
            $scope.getNDMPRestoreBackupTarget();
        };

        function validateFileBrowser(){
            $.validator.addMethod("isValidateFileBrowser", function(value, element) {
                    var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
                    return listbox.length > 0;
                },
                "Select files to recover"
            );
        }

        /**********************************Wizard next steps.*********************/
        $scope.nextStepWizard = function($event) {
            if ($scope.browseOnly) {
                var message = selectedBackup.client_os + " backups may be browsed but not recovered using the web-based user interface.  " +
                              " To recover an " + selectedBackup.client_os + " backup, use the menu-based console interface.";
                ngDialog.open({
                    dialogType: 'Error',
                    modelDialogId: 'browseOnlyError',
                    scope: $scope,
                    dialogMessage: message,
                    onConfirmOkButtonClick: 'onConfirmError()'
                });
            } else {
                if ($scope.flr_flag === "Backup") {
                    if (wizardTitle === gettextCatalog.getString("File Browser")) {//1st step for backup client
                        var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
                        if (validateFileBrowserTab()) {
                            $rootScope.$broadcast("nextBtnClick");
                        }
                    } else if (wizardTitle === gettextCatalog.getString("Recovery Options")) { //2nd step
                        $scope.postFileLevelRecoveryData(); //call to the POST API
                    }
                } else if ($scope.flr_flag === "BackupCopy") {
                    if (wizardTitle === gettextCatalog.getString("File Browser")) {//1st step for backup copy client
                        var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
                        if (validateFileBrowserTab()) {
                            $rootScope.$broadcast("nextBtnClick");
                        }
                    } else if (wizardTitle === gettextCatalog.getString("Backup Copy Options")) { //2nd step
                        $scope.postFileLevelBackupCopyRecoveryData(); //call to the POST API
                    }
                } else if ($scope.flr_flag === "HotBackupCopy") {
                    if (wizardTitle === gettextCatalog.getString("File Browser")) {//1st step for backup client
                        var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
                        if(validateFileBrowserTab()){
                            $scope.postDataForHotBackupCopyFLR();
                        }
                    }
                }
            }
        };

        $scope.onConfirmError = function () {
            ngDialog.close('browseOnlyError');
            ngDialog.close('DownloadErrorDialog');
        };

        /**************************************************************************/

        /* START: File browser tab for FLR-Bckup */
        $scope.initFileBrowserTree = function(){
            var parentId = " ";
            var backupSelectionObject =null;

            $('#fileBrowserTreeID').jstree({
                'plugins' : ['types','dnd'],
                'core' : {
                    'data' : {
                        'url' : function( node ){
                            var last = node.original !== undefined && node.original.last !== undefined ? node.original.last : "";
                            $scope.priorChildren = [];
                            var addLast = "";
                            if (last !== "") {
                                addLast = "&last=" + encodeURIComponent(last);
                                node.id = node.parent;
                                var docID = document.getElementById(node.parent);
                                var parent = $('#fileBrowserTreeID').jstree('get_json', docID);
                                if (parent !== undefined && parent.children instanceof Array) {
                                    // Create object array of these children, not including the parent itself.
                                    for (var i = 0; i < parent.children.length; i++) {
                                        var item = parent.children[i];
                                        if (item.id != node.id) {
                                            var obj = { 'id':item.id,
                                                'text': item.text,
                                                'icon': item.icon,
                                                'state': item.state,
                                                'children': item.children };
                                            $scope.priorChildren.push(obj);
                                        }
                                    }
                                }
                            }
                            $scope.nodeId=node.id;
                            parentId = encodeURIComponent(node.id);
                            var filesUrl = "/api/backups/" + $scope.synthCapable + "/" + selectedBackup.backup_id;
                            var addSID = "/?sid=" + $scope.system_id;
                            var addDir = "&dir=" + parentId;
                            var addCount = "&count=" + $scope.MAX_FILES_TO_LOAD;
                            if( node.id == "#" ){
                                filesUrl += addSID + addCount + addLast;
                            } else {
                                filesUrl += addSID + addDir + addCount + addLast;
                            }
                            return filesUrl;
                        },
                        'dataFilter' : function (data) {
                            var jsonData=JSON.parse(data);
                            jsonData = jsonData.data;

                            var treeArray = $scope.priorChildren !== undefined ? $scope.priorChildren : [];
                            var treeObj={};
                            if(jsonData !== null && jsonData !== undefined){
                                var lengthOfJsonData = jsonData.length;
                                for( i =0 ; i<lengthOfJsonData ; i++ ){
                                    treeObj = $scope.createTreeObj(jsonData[i], true);
                                    treeArray.push(treeObj);
                                }
                            }
                            var newdata1= JSON.stringify(treeArray);
                            return newdata1;
                        }
                    },
                    'multiple' : true,
                    'animation': true,
                    'error' : function(error) {console.log('treeCtrl: error from js tree - ' + angular.toJson(error));},
                    'check_callback' : true,
                    'worker' : true,
                    'strings': {
                        'nodes': 'files/folders'
                    }
                }

            });

            $('#fileBrowserTreeID').on('select_node.jstree',function(e,item) {
                var i, j, r = [];
                $scope.filePathArray = [];
                for(i = 0, j = item.selected.length; i < j; i++) {
                    filePath = item.selected[i];
                    if (item.node !== undefined && item.node.original !== undefined &&
                        item.node.original.last !== undefined && (item.node.original.last != "")) {
                        // This node has "last" set, it is special.  Get more files and leave (do not select it)
                        $('#fileBrowserTreeID').jstree('load_node', item.node);
                        break;
                    }
                    $scope.filePathArray.push(filePath);
                    if (item.node !== undefined && item.node.original !== undefined && item.node.original.id !== undefined && !isNaN(item.node.original.size)) {
                        $scope.fileSizeMap[item.node.original.id] = item.node.original.size;
                        var icon = item.node.original.icon;
                        if (icon !== undefined) {
                            // A file is a directory if we see a folder icon.
                            $scope.fileDirMap[item.node.original.id] = (icon.indexOf('icon-hv-folder') !== -1);
                        }
                    } else {
                        $scope.fileSizeMap[filePath] = 0;
                        $scope.fileDirMap[filePath] = false;
                    }
                }
            });
        };

        $scope.createTreeObj = function(jsonItem, isBackup) {
            var index = jsonItem.name.indexOf("/");
            var nodeName = jsonItem.name;
            var nodeId = jsonItem.id;
            var fileSize = jsonItem.size;
            var addTrailingSlash = isBackup || false;
            if (index != undefined && index !=null && index == -1 && (jsonItem.type === "directory" || jsonItem.type === "volume")){
                nodeName = jsonItem.name + "/";
                if (addTrailingSlash) {
                    nodeId = jsonItem.id + "/";
                }
            }
            jsonItem.isBranch = true;
            var last = jsonItem.last !== undefined ? jsonItem.last : "";
            return { "id":nodeId,
                "text":nodeName,
                "size":fileSize,
                icon: (last !== "") ? "icon-plussign" :
                    ((jsonItem.type === "directory" ||  jsonItem.type === "volume")? "icon-hv-folder icon" : "icon-file icon"),
                state: { opened: false},
                last: last,
                "children": (jsonItem.type === "directory" || jsonItem.type === "volume")};
        };


        $scope.addItem = function(){
            console.log($scope.filePathArray);
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            for(var i=0; i<$scope.filePathArray.length;i++){
                addOption(listbox, $scope.filePathArray[i]);
            }
        };

        function addOption(selectbox,node){
            console.log(node);
            var optn = document.createElement("OPTION");
            optn.text = node;
            optn.value = node;

            var exists = false;
            $('#listboxId option').each(function(){
                if (this.value == node) {
                    exists = true;
                }
            });

            if(!exists && !angular.isUndefined(node) && node !== null){
                selectbox.options.add(optn);
                totalListItems = selectbox.options.length;
                document.getElementById("totalListItemId").innerHTML= gettextCatalog.getString("File Selected:")+totalListItems;
            }
        }

        $scope.removeItem=function(){
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;

            $(listbox.options).each(function(index) {
                var elem = $(this)[0];
                if (elem.selected) {
                    $(elem).remove();
                }
            });

            totalListItems = listbox.options.length;
            document.getElementById("totalListItemId").innerHTML= gettextCatalog.getString("File Selected:")+totalListItems;
        };

        $scope.removeAllItem=function(){
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            listbox.innerHTML = "";
            totalListItems = listbox.options.length;
            document.getElementById("totalListItemId").innerHTML= gettextCatalog.getString("File Selected:")+totalListItems;
        };

        function validateFileBrowserTab(){
            validateFileBrowser();
            $("#flr-file-browser-form").validate({
                errorLabelContainer: ".errorlist",
                errorContainer: "#dialog-errors",
                showErrors: function(errorMap, errorList) {
                    $("#dialog-errors").find(".summary")
                        .html(gettext("Correct the following errors:"));
                    this.defaultShowErrors();
                },
                rules: {
                    inventoryListbox:{
                        isValidateFileBrowser:true
                    }
                },
                messages: {
                    isValidateFileBrowser:"Select files to recover"
                }
            });
            if((!$('#flr-file-browser-form').valid())) {
                return false;
            }else{
                return true;
            }
        }


        /* END: File browser tab */

        /* START: Recovery Options Tab */

        $scope.getRestoreBackupTarget=function(){
            var defaulatSelectedClientId = 1;
            $('#id_asset').empty();
            var form = angular.element("#flr-file-option-form");
            form.find(".definition").popover({
                appendTo: form
            });
            $('#id_selectAssetMessage').text($scope.selectAssetMessage);
            var resp = PHD.Ajax.get("/api/restore/targets/?backup_id=" + selectedBackup.backup_id + "&sid=" + $scope.system_id);
            resp.done(function(data) {
                console.log(data);
                data.targets.sort(function(x,y){
                    return x.client_name.localeCompare(y.client_name);
                });
                defaulatSelectedClientId = parseInt(selectedBackup.client_id, 10);
                console.log(defaulatSelectedClientId);
                $.each(data.targets,function(i,target){
                    $('#id_asset').append("<option type="+target.client_os+" value="+target.client_id+">"+target.client_name+"</option>");
                });
                $('#id_asset').val(defaulatSelectedClientId);
            })
        };

        $scope.openFLRFileBrowserDialog = function(){
            console.log("in it");
            $scope.flr_target = $('#id_asset').val();
            $scope.selectedBackup = selectedBackup;
            var dataObj={};
            var sid = (selectedBackup.replicated == 'true') ? selectedBackup.local_system_id : selectedBackup.sid;

            dataObj = {
                singleFileBrowserTreeNodeUrl:"/api/clients/files/" + $scope.flr_target + "/?sid=" + sid + "&dir=",
                url:"/api/clients/files/" + $scope.flr_target + "/?sid=" + sid + ""
            };

            $scope.openSingleFileBrowserDialog(dataObj);
        };

        $scope.$on("singleFilesBrowseSelected", function(event, filePath) {
            $('#id_directory').val(filePath);
        });

        $scope.openSingleFileBrowserDialog = function(dataObject){
            $rootScope.singleFileBrowserTreeNodeUrl = dataObject.singleFileBrowserTreeNodeUrl;
            var load = PHD.showLoadingIndicator("body", true, "Loading...");
            $http({
                method: 'get',
                url : dataObject.url
            }).success(function(data, status, headers){
                var jsonData = data.data;
                var treeObj = {};
                var treeArray = [];
                var childDummayDataArray = [];
                var finalTreeDataArray = [];

                for(var i=0;i<jsonData.length;i++){
                    var nodeName = jsonData[i].name;
                    var nodeId = jsonData[i].id;
                    var lastChar = nodeName.substr(nodeName.length - 1);
                    if (lastChar != undefined && lastChar != "/") {
                        nodeName = jsonData[i].name+"/";
                        nodeId = jsonData[i].id+"/";
                    }

                    treeObj = {id:nodeId,parent:"#",text:nodeName,icon:(jsonData[i].isBranch == true)? "icon-hv-folder icon" : "icon-file icon",state: { opened: false},data:{expanded:false}};
                    treeArray.push(treeObj);

                    var id = nodeId+"...";
                    if(jsonData[i].isBranch == true){
                        treeObj={id:id,parent:nodeId,text:"...",state: { opened: false}};
                        childDummayDataArray.push(treeObj);
                    }
                }
                finalTreeDataArray = treeArray.concat(childDummayDataArray);
                $rootScope.originalData = finalTreeDataArray;

                PHD.hideLoadingIndicator(load);
                ngDialog.open({
                    template: 'app/recover/recoveryOptions/single-file-browser.html',
                    scope: $rootScope,
                    overlay:true,
                    modelDialogId:'single-file-browser-setting-dailog',
                    ngDialogStyle:'width:800px;height:600px;',
                    preCloseCallback: function (value) {}
                });
            }).error(function(response) {
                PHD.hideLoadingIndicator(load);
                ngDialog.open({
                    dialogType:'Retry',
                    dialogMessage:response.result[0].message
                });
            });
        };

        $scope.getNDMPRestoreBackupTarget = function() {
            var defaultSelectedClientId = 1;
            $('#id_host').empty();
            var resp = PHD.Ajax.get("/api/restore/targets/?backup_id=" + selectedBackup.backup_id + "&sid=" + $scope.system_id);
            resp.done(function(data) {
                console.log(data);
                data.targets.sort(function(x,y){
                    return x.client_name.localeCompare(y.client_name);
                });
                var restore_targets = data.targets;
                defaultSelectedClientId = parseInt(selectedBackup.client_id, 10);
                console.log(defaultSelectedClientId);
                angular.forEach(restore_targets, function (value, key) {
                    var locationTempObj = {};
                    locationTempObj.value = value.client_id;
                    locationTempObj.name = value.client_name;
                    $scope.ndmpTargets.push(locationTempObj);
                });
                //$('#id_host').val(defaultSelectedClientId);
            })
        };

        $scope.ndmpTargetChange = function (ndmp_target_client_id) {
            if (ndmp_target_client_id !== null) {
                $scope.ndmpRestoreTarget.assetName = $.grep($scope.ndmpTargets, function (asset) {
                    return asset.value === ndmp_target_client_id;
                })[0].name;
            } else {
                $scope.ndmpRestoreTarget.assetName = "";
            }

            var url = "/api/restore/targets/?backup_id=" + selectedBackup.backup_id + "&ndmp_target_client_id=" + ndmp_target_client_id + "&replicated=";
            url += selectedBackup.replicated == 'true' ? "1&sid=" + selectedBackup.local_system_id : "0&sid=" + selectedBackup.sid;
            load = PHD.showLoadingIndicator("body", true, "Loading Target Volumes...");
            var resp = PHD.Ajax.get(url);

            resp.done(function(data) {
                PHD.hideLoadingIndicator(load);
                var restore_target_volumes = data.targets;
                if (restore_target_volumes && restore_target_volumes.length > 0) {
                    $scope.ndmpTargetVolumes = [];
                    angular.forEach(restore_target_volumes, function (value, key) {
                        var volumeTempObj = {};
                        volumeTempObj.value = value.instance_id;
                        volumeTempObj.name = value.name;
                        $scope.ndmpTargetVolumes.push(volumeTempObj);
                    });
                    //$scope.ndmpRestoreTarget.asset = $scope.ndmpTargets[0].value;
                    //$scope.ndmpRestoreTargetVolume.instance_id = $scope.ndmpTargetVolumes[0].value;
                    //$scope.ndmpRestoreTarget.assetName = $scope.ndmpTargets[0].name;

                }
            });
        };

        $scope.ndmpTargetVolumeChange = function () {
            if ($scope.ndmpRestoreTargetVolume.instance_id !== null) {
                $scope.ndmpRestoreTargetVolume.assetName = $.grep($scope.ndmpTargetVolumes, function (asset) {
                    return asset.value === $scope.ndmpRestoreTargetVolume.instance_id;
                })[0].name;
            } else {
                $scope.ndmpRestoreTargetVolume.assetName = "";
            }
        };


        // For add exclude patern.
        $scope.addExcludePatternRow = function(noRemoveButton) {
            var noRemove = noRemoveButton || false;
            var table = $("#exclusionTable")[0];
            var excludePattern = $("#id_add").val();

            if(excludePattern == ""){
                return;
            }
            var rowCount = table.rows.length;
            var row = table.insertRow(rowCount);
            var cell1 = row.insertCell(0);
            cell1.innerHTML= excludePattern;

            if (!noRemove) {
                var cell2 = row.insertCell(1);
                cell2.innerHTML = '<span class="btn-remove-recipient"><i title="Remove" onclick="deleteRow(this)" class="icon-removesign"></i></span>';
            }
            $("#id_add").val("");
        };

        $scope.postFileLevelRecoveryData = function() {
            var load = PHD.showLoadingIndicator("body", true, "Starting File Level Recovery...");
            // var backups = selectionObj.backups;
            var bids = selectedBackup.backup_id;
            PHD.appliance_sid = $scope.system_id;
            var restoreObj = {};
            restoreObj.backup_id = bids;

            $scope.isPhysicalServerRecovery = (selectedBackup.instance_id === undefined) ? true:false;

            // this needs to be done only in case of File level recovery
            restoreObj.client_id = $('#id_asset').val();
            if ( $scope.is_cisco_ucs_restore !== true && $scope.is_NDMP_restore !== true) {
                var target = {};
                target.flat = !$('#id_preserve_der').is(":checked");
                target.non_destructive = !$('#id_overwrt_file').is(":checked");
                target.newer = $('#id_newer_file_restore').is(":checked");
                target.today = $('#id_file_dates').is(":checked");
                target.unix = $('#idunix_conver').is(":checked");
                target.directory = $('#id_directory').val();
                restoreObj.target = target;
                restoreObj.client_id = $('#id_asset').val();
                restoreObj.before_cmd = $('#id_pre_restore').val();
                restoreObj.after_cmd = $('#id_post_restore').val();
            }
            if($scope.synthCapable === "synthesized-files") {
                restoreObj.synthesis = true;
            } else {
                restoreObj.synthesis = false;
            }
            if($scope.is_NDMP_restore === true) {
                restoreObj.target = {};
                restoreObj.target.volume = "";
                restoreObj.target.directory = "";
                restoreObj.client_id = $scope.ndmpRestoreTarget.asset;
                restoreObj.target.volume = $scope.ndmpRestoreTargetVolume.assetName;
                restoreObj.target.directory = $scope.ndmpRestoreTarget.directory;
                restoreObj.backup_id = (selectedBackup.backup_id);
            }
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            var includeListArray = [];
            var excludeListArray = [];

            for(var i=0; i< listbox.length; i++){
                includeListArray.push(listbox[i].value);
            }
            console.log(includeListArray);
            restoreObj.includes = includeListArray;

            if ( $scope.is_cisco_ucs_restore !== true && $scope.is_NDMP_restore !== true) {
                var data = document.getElementById("exclusionTable").rows;
                for (var j = 0; j < data.length; j++) {
                    var excludeList = document.getElementById("exclusionTable").rows[j].cells[0].innerHTML;
                    excludeList = excludeList.replace(/[\r\n]/g, "").trim();
                    if (excludeList != '\n' && excludeList != '\r' && excludeList != '\t' && excludeList != "") {
                        if (excludeList.length != 0) {
                            excludeListArray.push(excludeList);
                        }
                    }
                }
                restoreObj.excludes = excludeListArray;
            }
            restoreObj = JSON.stringify(restoreObj);

            var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
            //var url = "/api/restore/files/" + sid;
            url = "/api/restore/full/" + sid;

            $http ({
                method: 'post',
                url : url,
                data: restoreObj
            }).success(function(data){
                /*data = {};
                 PHD.hideLoadingIndicator(load);

                 ngDialog.open({
                 dialogType:'Information',
                 scope:$scope,
                 dialogMessage:"File level recovery job successfully started."
                 });*/
                PHD.hideLoadingIndicator(load);
                data = {};
                data.result = [{'message': 'File level recovery job successfully started.'}];
                var options = {};
                if (window.location.href.indexOf("#/protect") !== -1 || window.location.href.indexOf("#/jobs") !== -1) {
                    options.buttons = [
                        {
                            text: gettext("View FLR"),
                            click: function () {
                                window.location.href = "#/recover/#1";
                                $(this).dialog("destroy").remove();
                            }
                        }
                    ];
                }

                PHD.throwNotice(data, options);

                ngDialog.close('create-filelevel-recovery-dialog');

            }).error(function(response) {
                PHD.hideLoadingIndicator(load);
                ngDialog.open({
                    dialogType:'retry',
                    dialogMessage:response.result[0].message
                })
            });
        };
        /* END: Recovery Options Tab */


        /* START: Call POST api for file level backup copy recovery data */
        $scope.postFileLevelBackupCopyRecoveryData = function(){
            var load = PHD.showLoadingIndicator("body", true, "Starting File Level Recovery...");
            PHD.appliance_sid = $scope.system_id;
            var backupCopyRestoreObj = {};
            backupCopyRestoreObj.archive_ids = selectedBackup.archive_id; //selectedBackup.archive_ids;
            backupCopyRestoreObj.association = "dr"; //association  association
            //backupCopyRestoreObj.instance_id = "";
            //backupCopyRestoreObj.instance_id = selectedBackup.instance_id;  // TO DO: when instance_id corrected


            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            var includes_list = [];
            for(var i=0; i< listbox.length; i++){
                includes_list[i] = listbox[i].value;
            }
            backupCopyRestoreObj.inclusions = includes_list;
            backupCopyRestoreObj.exclusions = [];
            backupCopyRestoreObj.restore_backup = true;

            backupCopyRestoreObj = JSON.stringify(backupCopyRestoreObj);

            var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
            url = "/api/restore/archive/files/" + sid;


            $http ({
                method: 'post',
                url : url,
                data: backupCopyRestoreObj
            }).success(function(data){
                PHD.hideLoadingIndicator(load);
                data = {};
                data.result = [{'message': 'File level recovery job successfully started.'}];
                var options = {};
                if (window.location.href.indexOf("#/protect") !== -1 || window.location.href.indexOf("#/jobs") !== -1) {
                    options.buttons = [
                        {
                            text: gettext("View FLR"),
                            click: function () {
                                window.location.href = "#/recover/#1";
                                $(this).dialog("destroy").remove();
                            }
                        }
                    ];
                }

                PHD.throwNotice(data, options);
                ngDialog.close('create-filelevel-recovery-dialog');

            }).error(function(response) {
                PHD.hideLoadingIndicator(load);
                ngDialog.open({
                    dialogType:'retry',
                    dialogMessage:response.result[0].message
                })
            });
        };


        /* END: Call POST api for file level backup copy recovery data  */

        /* START: File browser tab for FLR-Bckup copy - GET Api */
        $scope.initFileBrowserTreeFLRBackupCopy = function(){
            var parentId = " ";
            var file_id = selectedBackup.archive_id;     //selectedBackup.backup_id

            $('#fileBrowserTreeID').jstree({
                'plugins' : ['types','dnd'],
                'core' : {
                    'data' : {
                        'url' : function( node ){
                            var last = node.original !== undefined && node.original.last !== undefined ? node.original.last : "";
                            $scope.priorChildren = [];
                            var addLast = "";
                            if (last !== "") {
                                addLast = "&last=" + encodeURIComponent(last);
                                node.id = node.parent;
                                var docID = document.getElementById(node.parent);
                                var parent = $('#fileBrowserTreeID').jstree('get_json', docID);
                                if (parent !== undefined && parent.children instanceof Array) {
                                    // Create object array of these children, not including the parent itself.
                                    for (var i = 0; i < parent.children.length; i++) {
                                        var item = parent.children[i];
                                        if (item.id != node.id) {
                                            var obj = { 'id':item.id,
                                                'text': item.text,
                                                'icon': item.icon,
                                                'state': item.state,
                                                'children': item.children };
                                            $scope.priorChildren.push(obj);
                                        }
                                    }
                                }
                            }
                            parentId = encodeURIComponent(node.id);
                            var filesUrl = "/api/archive/files/" + file_id;
                            var addSID = "/?sid=" + $scope.system_id;
                            var addDir = "&dir=" + parentId;
                            var addCount = "&count=" + $scope.MAX_FILES_TO_LOAD;
                            if( node.id == "#" ){
                                filesUrl += addSID + addCount + addLast;
                            } else {
                                filesUrl += addSID + addDir + addCount + addLast;
                            }
                            return filesUrl;
                        },
                        'dataFilter' : function (data) {
                            var jsonData=JSON.parse(data);
                            jsonData = jsonData.data;

                            var treeArray = $scope.priorChildren !== undefined ? $scope.priorChildren : [];
                            var treeObj={};
                            if(jsonData !== null && jsonData !== undefined){
                                var lengthOfJsonData = jsonData.length;
                                for( i =0 ; i<lengthOfJsonData ; i++ ){
                                    treeObj = $scope.createTreeObj(jsonData[i], false);
                                    treeArray.push(treeObj);
                                }
                            }
                            var newdata1= JSON.stringify(treeArray);
                            return newdata1;
                        }
                    },
                    'multiple' : true,
                    'animation': true,
                    'error' : function(error) {console.log('treeCtrl: error from js tree - ' + angular.toJson(error));},
                    'check_callback' : true,
                    'worker' : true
                }
            });

            $('#fileBrowserTreeID').on('select_node.jstree',function(e,item) {
                var i, j, r = [];
                $scope.filePathArray = [];
                for(i = 0, j = item.selected.length; i < j; i++) {
                    if (item.node !== undefined && item.node.original !== undefined &&
                        item.node.original.last !== undefined && (item.node.original.last != "")) {
                        // This node has "last" set, it is special.  Get more files and leave (do not select it)
                        $('#fileBrowserTreeID').jstree('load_node', item.node);
                        break;
                    }
                    filePath = item.selected[i];
                    $scope.filePathArray.push(filePath);
                }
            });
        };

        $(document).on('dnd_move.vakata', function (e, data) {
            var t = $(data.event.target);
            if(!t.closest('.jstree').length) {
                if(t.closest('.validate-filebrowser').length) {
                    if (isAPlusSign($(data.element.children))) {
                        return;
                    }
                    if ($scope.filesOnly) {
                        if (isAFolder($(data.element.children))) {
                            return;
                        }
                    }
                    data.helper.find('.jstree-icon').removeClass('jstree-er').addClass('jstree-ok');
                }
                else {
                    data.helper.find('.jstree-icon').removeClass('jstree-ok').addClass('jstree-er');
                }
            }

        }).on('dnd_stop.vakata', function (e, data) {
            console.log(data.data.nodes);
            if (isAPlusSign($(data.element.children))) {
                return;
            }
            if ($scope.filesOnly) {
                if (isAFolder($(data.element.children))) {
                    return;
                }
            }
            var t = $(data.event.target);
            if(!t.closest('.jstree').length) {
                if(t.closest('.validate-filebrowser').length) {
                    console.log(data.data.nodes);
                    addDragNodesInList(data.data.nodes);
                    $(data.element).clone().appendTo(t.closest('.validate-filebrowser'));
                }
            }
        });

        function isAFolder($node) {
            var folder = false;
            if ($node.length > 0) {
                var child = $node[0];
                var $child = $(child);
                folder = $child.hasClass('icon-hv-folder');
                if (folder) {
                    console.log("is a folder.....");
                }
            }
            return folder;
        }

        function isAPlusSign($node) {
            var match = false;
            if ($node.length > 0) {
                var child = $node[0];
                var $child = $(child);
                match = $child.hasClass('icon-plussign');
                if (match) {
                    console.log("is an adder (plus).....");
                }
            }
            return match;
        }

		function addDragNodesInList(textArray){
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            for(var i=0;i<textArray.length;i++){
                if(textArray[i] != null){
                    addOption(listbox,textArray[i]);
                }
	         }
        }

        $scope.postDataForHotBackupCopyFLR = function() {
            var load = PHD.showLoadingIndicator("body", true, "Starting File Level Recovery on the Target...");
            var backup_id = selectedBackup.backup_id;
            var restoreObj = {};
            restoreObj.backup_id = backup_id;
            var size_KB = 0;
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            var includes_list = [];

            var dirList = [];
            var MAX_DIRS_TO_DISPLAY = 10;
            for(var i=0; i< listbox.length; i++){
                includes_list.push(listbox[i].value);
                var size = $scope.fileSizeMap[listbox[i].value];
                if (size !== undefined && !isNaN(size)) {
                    size_KB += size;
                }
                var dir = $scope.fileDirMap[listbox[i].value];
                if (dir === true) {
                    dirList.push(listbox[i].value);
                }
            }
            if (dirList.length > 0) {
                restoreObj.dir_list = dirList;
                restoreObj.backup_size = $scope.backupSize;
            }
            // UNIBP-12843 commenting out the code to restrict folder download
            // if (dirList.length > 0) {
            //     if (dirList.length > MAX_DIRS_TO_DISPLAY) {
            //         var additional = dirList.length - MAX_DIRS_TO_DISPLAY;
            //         dirList.length = MAX_DIRS_TO_DISPLAY;
            //         dirList.push('... and ' + additional + ' more (too many to display)');
            //     }
            //     PHD.hideLoadingIndicator(load);
            //     var message = 'Volumes or directories cannot be selected for recovery from the target.\n' +
            //         'Click Retry, then remove these selections from the right-hand-side:\n\n' + dirList.join('\n');
            //     ngDialog.open({
            //         dialogType: 'retry',
            //         dialogMessage: message
            //     });
            //     // Get out, we can't restore...
            //     return;
            // }
            restoreObj.includes = includes_list;
            restoreObj.excludes = [];
            restoreObj.before_cmd = '';
            restoreObj.after_cmd = '';
            var target = {};
            target.flat = false;
            target.non_destructive = false;
            target.newer = false;
            target.today = false;
            target.unix = true;
            restoreObj.target = target;
            restoreObj.client_id = 1; // get system client id.
            if ($scope.synthCapable === "target-synthesized-files") {
                restoreObj.synthesis = true;
            } else {
                restoreObj.synthesis = false;
            }
            restoreObj.size_KB = size_KB;
            var url = "/api/restore/target-full/?sid="+ 1;
            $http ({
                method: 'post',
                url : url,
                data: restoreObj
            }).success(function(data){
                PHD.hideLoadingIndicator(load);
                var jobID = data.id;
                ngDialog.open({
                    template: 'app/recover/filerecovery/searchFLR/viewTargetFileRecovery.html',
                    scope: $scope,
                    overlay:true,
                    name: 'MonitorFileRecovery',
                    data: {jobID: jobID, includes: includes_list},
                    modelDialogId:'flr-view-jobs-target-dialog',
                    ngDialogStyle:'width:64em; height:22em; left: 80px;'
                });

            }).error(function(response) {
                PHD.hideLoadingIndicator(load);
                ngDialog.open({
                    dialogType:'retry',
                    dialogMessage:response.result[0].message
                })
            });
        };

    }]);

fileLevelRecoveryModule.controller('fileLevelRecoveryTargetCtrl', ['$scope', '$rootScope', '$http', '$timeout', 'ngDialog', 'wizardService', '$analytics', 'DownloadService',
    function($scope, $rootScope, $http, $timeout, ngDialog, wizardService, $analytics, DownloadService) {

        console.log("in flrTargetCtrl");
        $scope.restoreStatusBase = gettext("Restoring Files") + "....";
        $scope.restoreStatusString = $scope.restoreStatusBase;
        $scope.jobID = $scope.ngDialogData.jobID;
        $scope.fileList = $scope.ngDialogData.includes;

        // Used to unbind the watched event on dialog close.
        var unbindHandler = null;

        $scope.initCtrl = function(dlg) {
            switch (dlg.name) {
                case 'MonitorFileRecovery':
                    $scope.monitorRestoreJob($scope.jobID, false);
                    break;
            }
        };

        $scope.closeFLRStatusDialog = function() {
            ngDialog.close('window-flr-view-jobs-dialog');
            ngDialog.close('flr-search-dialog');
            if ( _.isFunction(unbindHandler)) {
                unbindHandler();
            }
        };

        $scope.monitorRestoreJob = function(jobID, indicator){
            $scope.isJobComplete = false;
            var url = "/api/jobs/" + $scope.jobID + ".target";
            //var load = indicator == false ? PHD.showLoadingIndicator($("#target_restore_status"), false) : indicator;
            $http({
                method: 'get',
                url   : url
            }).success(function(data, status, headers) {
                var jobInfo = data.data;
                if (jobInfo === null) {
                    $scope.restoreStatusString = gettext("Error");
                    $scope.isJobComplete = true;
                    //PHD.hideLoadingIndicator(load);
                    ngDialog.open({
                        dialogType:'error',
                        dialogMessage: "Error monitoring remote file recovery job."
                    })
                } else {
                    if (jobInfo !== undefined) {
                        $scope.restoreStatusString = $scope.restoreStatusBase + gettext("Status") + ": " + jobInfo.status + ".";
                    }
                    var lookup = PHD.StateLookup.job_status[jobInfo.status];
                    if (lookup.indexOf('job-status-finished') === -1) {
                        $timeout(function() {
                            $scope.monitorRestoreJob($scope.jobID, load);
                        }, 3000);
                    } else {
                        //PHD.hideLoadingIndicator(load);
                        if (jobInfo.status === 'Successful') {
                            $scope.restoreStatusString = $scope.restoreStatusBase + "Status: Downloading.";
                            $scope.downloadFiles();
                            $scope.isJobComplete = true;
                        } else {
                            $scope.restoreStatusString = gettext("Error recovering files.  Please try again.");
                            $scope.isJobComplete = true;
                        }
                    }
                }
            }).error(function(response) {
                $scope.restoreStatusString = gettext("Error");
                $scope.isJobComplete = true;
                //PHD.hideLoadingIndicator(load);
                ngDialog.open({
                    dialogType:'error',
                    dialogMessage:response.result[0].message
                })
            });
        };

        $scope.downloadFiles = function() {
            var fileList = [];
            for(var i = 0; i < $scope.fileList.length; i++) {
                var fileName = $scope.fileList[i];
                if (fileName.indexOf("@@@:") !== -1) {
                    var afterAt = 4; // length of '@@@:'
                    fileName = fileName.substring(afterAt);
                }
                // remove end slash if present
                if (fileName[fileName.length - 1] === '/') {
                    fileName = fileName.slice(0, -1);
                }
                console.log(fileName);
                fileList.push(fileName);
            }

            $scope.system_id = 1;

            console.log("download files");
            var baseURL = "/api/restore/target-download-files";
            var fileObj = {};
            fileObj.filenames = fileList;
            fileObj.targetDir = true;       // create the target directory
            fileObj.deleteDir = true;       // delete directory after zip creation complete.
            fileObj.id = $scope.jobID;      // passing job ID to create a unique directory where the files will be downloaded
            fileObj = JSON.stringify(fileObj);
            $http({
                method: 'post',
                url: baseURL + '/?sid=' + $scope.system_id,
                data: fileObj,
                responseType: "arraybuffer"
            }).success(function(data, status, headers){
                console.log("success");
                headers = headers();
                var contentType = headers['content-type'];
                // See if we need to display information or download a file.
                if (contentType.indexOf('application/json') !== -1) {
                    // convert response array buffer to JSON to get information.
                    var response = String.fromCharCode.apply(null, new Uint8Array(data));
                    response = JSON.parse(response);
                    $scope.URLlink = response.link;
                    if ($scope.URLlink !== undefined) {
                        var urlZip = response.urlzip;
                        if (urlZip !== undefined) {
                            var fullPath = $scope.URLlink + urlZip;
                            var baseName = urlZip.substring(urlZip.lastIndexOf("/") + 1);
                            DownloadService.saveAs(fullPath, baseName);
                            $scope.downloadFullPath = fullPath;
                            var $progressDialog = angular.element(document.getElementById("flr-view-jobs-target-dialog"));
                            if ($progressDialog.length > 0) {
                                // The progress dialog is open, therefore we can show the download dialog.
                                // Otherwise, the scope is removed and the download dialog will not function.
                                ngDialog.open({
                                    template: 'app/recover/download-information.html',
                                    scope: $scope,
                                    overlay:true,
                                    modelDialogId:'DownloadLinkDialog',
                                    ngDialogStyle:'width:440px;height:180px;'
                                });
                            }
                        } else {
                            ngDialog.open({
                                dialogType:'Error',
                                modelDialogId:'DownloadErrorDialog',
                                scope:$scope,
                                dialogMessage: 'The files have been placed in the downloads location but the link is unavailable.',
                                onConfirmOkButtonClick:'onConfirmError()'
                            });
                        }
                    } else {
                        ngDialog.open({
                            dialogType:'Error',
                            modelDialogId:'DownloadErrorDialog',
                            scope:$scope,
                            dialogMessage: 'The files have been placed in the downloads location but the link is unavailable.',
                            onConfirmOkButtonClick:'onConfirmError()'
                        });
                    }
                } else {
                    var filename = "Unitrends-Restore.zip";
                    console.log(headers);
                    var blob = new Blob([data], {type: contentType});
                    DownloadService.downloadFile(blob, filename);
                    ngDialog.closeAll();
                }
            }).error(function(response){
                // convert response array buffer to JSON to get error message.
                response = String.fromCharCode.apply(null, new Uint8Array(response));
                response = JSON.parse(response);
                ngDialog.open({
                    dialogType:'Error',
					modelDialogId:'DownloadBrowserErrorDialog',
                    dialogMessage: response.result[0].message,
					onConfirmOkButtonClick:'onConfirmBrowserErrorDialog()'
                });
                ngDialog.close('create-filelevel-recovery-dialog');
            });
        };
		
		$scope.onConfirmBrowserErrorDialog = function() {
            ngDialog.close('DownloadBrowserErrorDialog');
         };

        // Initialize the controller, returning the function to turn off the listener.
        unbindHandler = $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

        $scope.closeDialog = function() {
            ngDialog.close("flr-view-jobs-target-dialog");
        };

    }]);

/* function for delete row from exclusion list in recovery option tab */
function deleteRow(r){
    var i = r.parentNode.parentNode.parentNode.rowIndex;
    document.getElementById("exclusionTable").deleteRow(i);
}
