(function($, PHD, window, undefined) {

    var $form = $("#file-recovery-form"),
        FLRForm,
        $backupCatalog = $("#file-recovery-backup-table"),
        $backupFilter = $("#file-recovery-backup-filter"),
        backupCatalog, backupFilter,
        backupCatalogView = BACKUP_CATALOG_VIEW_VM;
    var lastSelectedBackup = null;
    var filePath="";
    var totalListItems = 0;
    var isPhysicalServerRecovery = false;
    var synthCapable = "files";
    

        //$recoveryType = $("[name=recovery_type]"),
        //$credentialType = $("[name=credential_type]"),
        //currentRecoveryType = FLR_TYPE_CIFS,
        //currentCredentialType = CREDENTIAL_TYPE_NONE,
        //$credentials = $("#file-recovery-credentials");

    var options = {};
    options[BACKUP_CATALOG_VIEW_VM] = {
        unique: "backup",
        0: { multiselect: false, childselect: "first", expandonselect: true },
        1: { multiselect: false, childselect: "all", expandonselect: true },
        2: { multiselect: true, childselect: false }
    };
    options[BACKUP_CATALOG_VIEW_DATE] = {
        unique: "backup",
        0: { multiselect: false, childselect: "first", expandonselect: true },
        1: { multiselect: false, childselect: "first", expandonselect: true },
        2: { multiselect: true, childselect: false }
       /*
        lmc - merged 3rd and 4th level since UEB does not have disk FLR
        2: { multiselect: false, childselect: "all", expandonselect: true },
        3: { multiselect: true, childselect: false }
        */
    };
    options[BACKUP_CATALOG_VIEW_BDS] = {
        unique: "backup",
        0: { multiselect: false, childselect: "first", expandonselect: true },
        1: { multiselect: false, childselect: "first", expandonselect: true },
        2: { multiselect: false, childselect: "all", expandonselect: true },
        3: { multiselect: true, childselect: false }
    };
    options[BACKUP_CATALOG_VIEW_APPLIANCE] = {
        unique: "backup",
        0: { multiselect: false, childselect: "first", expandonselect: true },
        1: { multiselect: false, childselect: "first", expandonselect: true },
        2: { multiselect: false, childselect: "all", expandonselect: true },
        3: { multiselect: true, childselect: false }
    };

    function initBackupsTable() {
        backupCatalog = PHD.BackupCatalog($backupCatalog, options[backupCatalogView]);
        backupFilter = PHD.BackupFilter($backupFilter, backupCatalog);

        backupCatalog.$table.on("treeselect", ".row-select", function() {
            selections = backupCatalog.getSelectionObject();
            updateSelections();
        });
        updateBackups();

    }

    function updateSelections() {
        if (selections.backups.length>0) {
            //var sid = $node.attr('system-id');
            var resp = PHD.Ajax.get("/api/backups/?sid=" + (selectedBackup.replicated == 'true' ? selectedBackup.local_system_id : selections.system_ids[0]) + "&bid=" + selections.backups[0]);
            resp.done(function(data) {
                isPhysicalServerRecovery = data.BackupStatus[0].instance_id === undefined ? true:false;
                synthCapable = data.BackupStatus[0].synth_capable?"synthesized-files":"files";
                if(isPhysicalServerRecovery) {
                    enableFileRecoveryWizard();
                } else {
                    disableFileRecoveryWizard();
                }
            });
        }
    }

    function updateBackups() {
        var load = PHD.showLoadingIndicator($backupCatalog),
            task = PHD.BackupCatalogBuilder({
                "view": backupCatalogView, 
                "vms": {
                    "show": true,
                    "checkboxes": false
                },
                "disks": {
                    "show": true,
                    "checkboxes": false
                },
                "checked": selections,
                "loader": load,
                "load_level": BACKUP_CATALOG_LOAD_BACKUPS
            });

        task
            .done(function(data) {
                buildCatalog(data);
            })
            .always(function() {
                PHD.hideLoadingIndicator(load);
            });
    }

    function buildCatalog(data) {
        backupCatalog.load(data);
        backupCatalog.refresh(options[backupCatalogView]);
        backupFilter.refresh(data.tree).submitFilter();
        if ($backupCatalog.find("input[name=backup]:checked").length > 0) {
            selections = backupCatalog.getSelectionObject();
            updateSelections();
        }
    }


    function initForm() {
        //$recoveryType.eq(1).prop('disabled', true);
        $form = $("#file-recovery-form");
        $.validator.addMethod("validate-catalog", function(value, element) {
                return $backupCatalog.find("input[name=backup]:checked").length > 0;
            },
            "Select at least one Backup"
        );

        $.validator.addMethod("validate-filebrowser", function(value, element) {
                var listbox = document.fileRecoveryForm.inventoryListbox;

                return listbox.length > 0;
            },
            "Select files to recover"
        );


        $.validator.addMethod("validate-directory", function(value, element) {
                if(isPhysicalServerRecovery){
                    return ($('#id_directory').val() !=null && $('#id_directory').val() !=undefined && $('#id_directory').val() !="");
                }
                return true;
            },
            "Directory is mandatory"
        );

        $.validator.addMethod("validate-path", function(value, element) {
                if(isPhysicalServerRecovery){

                    var element = $('#id_asset').find('option:selected'); 
                    var osType = element.attr("type"); 
                    var regex= /^[A-Za-z]:\\/;

                    if(osType.lastIndexOf("Linux") >-1 || osType.lastIndexOf("Mac") >-1){
                          if ( $('#id_directory').val().indexOf("\\") == 0 ){
                            return true;
                        } else {
                            return false;
                        }

                    } else{
                            return regex.test($('#id_directory').val());
                    }

                    
                }
                return true;
            },
            "Provide valid directory path"
        );


        FLRForm = PHD.StepForm($form, {
            namespace: "flrform",
            serialize: function($form) {
               // $startTime.timeEntry('option', 'show24Hours', true);
                return $form.serialize() + "&" + backupCatalog.getSelections();
            }
        });

        FLRForm
            .validate({
                rules: {
                    confirm_password: {
                        equalTo: "#id_password"
                    },
                    cifs_password: {
                        minlength: 12,
                        maxlength: 16
                    },
                    cifs_confirm_password: {
                        equalTo: "#id_cifs_password"
                    }
                },
                messages: {
                    confirm_password: {
                        equalTo: gettext("Your password must be the same in both fields")
                    },
                    cifs_password: {
                        minlength: gettext("Your password must be 12 characters long")
                    },
                    cifs_confirm_password: {
                        equalTo: gettext("Your password must be the same in both fields")
                    }
                }
            })
            .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                data = {};
                data.result = [{'message': 'File level recovery job successfully started.'}];
                var options = {};
                if (window.location.href.indexOf("#/protect") !== -1 || window.location.href.indexOf("#/jobs") !== -1) {
                    console.log("options ", options);
                    options.buttons = [
                        {
                            text: gettext("View FLR"),
                            click: function () {
                                window.location.href = "#/recover/#1";
                                $(this).dialog("destroy").remove();
                            }
                        }
                    ];
                }

                PHD.throwNotice(data, options);

                PHD.currentDialog.wizard("close");
            })

            .on("stepformgoto", function(event, page) {
                if(page == 1){
                    if(isPhysicalServerRecovery){
                        initFileBrowserTree();       
                    }    
                }else if(page == 2){
				    // initTarget();           
                }

            });

        FLRForm.submit = function() {
			console.log("in submit function:::");
            // lmc - TODO - ned a good way to customize the standard submit.
            var self = this,
                load = PHD.showLoadingIndicator("body", true, "Starting File Level Recovery...");

            this.$buttons.blur().disable();

            //var submitData = form2js(this.$form.attr("id"), '.', false);

            var selectionObj = backupCatalog.getSelectionObject();
            console.log("selected object = " + JSON.stringify(selectionObj));

            var backups = selectionObj.backups;
            var bids = backups[0];
            var system_id = selectionObj.system_id;
            PHD.appliance_sid = system_id;

            var restoreObj = {};
            restoreObj.backup_id = bids;
            // this needs to be done only in case of File level recovery
            if(isPhysicalServerRecovery){
                var target ={};
                target.flat = !$('#id_preserve_der').is(":checked");
                target.non_destructive = !$('#id_overwrt_file').is(":checked");
                target.newer = $('#id_newer_file_restore').is(":checked");
                target.today = $('#id_file_dates').is(":checked");
                target.unix = $('#idunix_conver').is(":checked");
                target.directory = $('#id_directory').val();
                restoreObj.target = target;
                restoreObj.client_id = $('#id_asset').val();
                restoreObj.before_cmd =$('#id_pre_restore').val();
                restoreObj.after_cmd = $('#id_post_restore').val();
                if(synthCapable === "synthesized-files") {
                    restoreObj.synthesis = true;    
                } else {
                    restoreObj.synthesis = false;    
                }
                var listbox = document.fileRecoveryForm.inventoryListbox;
                var includes_list ="";
				var exclude_list="";
				var excludeListArray = [];
				
                for(var i=0; i< listbox.length; i++){
                    if(i!=0){
                        includes_list = includes_list +","+listbox[i].text;
						console.log(includes_list);
                    }else{
                        includes_list = listbox[i].text;
                    }
					
                }
                restoreObj.includes = includes_list;
				
				var data = document.getElementById("exclusionTable").rows;
				for(var j=0;j<data.length;j++){
					var excludeList = document.getElementById("exclusionTable").rows[j].cells[0].innerHTML;
					    excludeList = excludeList.replace(/[\r\n]/g, "").trim();
				 	if (excludeList != '\n' && excludeList != '\r' && excludeList != '\t' && excludeList != ""){
						 if(excludeList.length != 0){
						 excludeListArray.push(excludeList);
						}
					  }
				   }
					exclude_list = excludeListArray.toString();
					console.log(exclude_list);
					restoreObj.excludes = exclude_list;
            }
            restoreObj = JSON.stringify(restoreObj);
            console.log('=== Submit form: object = ' + restoreObj);

            var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + selectedBackup.replicated == 'true' ? selectedBackup.local_system_id : PHD.appliance_sid;
            var url = "/api/restore/files/" + sid;
            if(isPhysicalServerRecovery){
                url = "/api/restore/full/" + sid;
            }
            console.log("POST url is " + url);
            PHD.Ajax
                .post(url, restoreObj)
                .done(function(data) {
                    self.$form.trigger("formsubmit", [data, self.$form, self]);
                })

                .always(function() {
                    self.$buttons.enable();
                    PHD.hideLoadingIndicator(load);
                });
            
            return this;
        };
    }

    function initFilters() {
        $backupCatalog.on("click",".btn-dateview",  function(event) {
            event.preventDefault();
            backupCatalogView = BACKUP_CATALOG_VIEW_DATE;
            updateBackups();
        });

        $backupCatalog.on("click", ".btn-vmview", function(event) {
            event.preventDefault();
            backupCatalogView = BACKUP_CATALOG_VIEW_VM;
            updateBackups();
        });

        $backupCatalog.on("click", ".btn-bdsview", function(event) {
            event.preventDefault();
            backupCatalogView = BACKUP_CATALOG_VIEW_BDS;
            updateBackups();
        });
        
        $backupCatalog.on("afterfilter backupcatalogload", function() {
            $backupCatalog.scrollTable({"height": "article", "resize": false});
        });
    }

/*    $('#addselectedNodeID').click(function(){
		console.log("click::");
            var listBox = document.fileRecoveryForm.inventoryListbox;
		console.log(listBox);
            addOption(listBox, filePath, filePath);
    });
    
    function addOption(selectbox,text,value ){
		console.log("****"+text);
        var optn = document.createElement("OPTION");
        optn.text = text;
        optn.value = value;
      if(!$("#listboxId option[value='"+value+"']").length > 0 && typeof(value) != "undefined"){
        selectbox.options.add(optn);
         totalListItems = selectbox.options.length;
         document.getElementById("totalListItemId").innerHTML= "File Selected:"+totalListItems;
          
      }
    }
    
    $('#removeSelectedNodeID').click(function removeItem(){
    var listbox = document.fileRecoveryForm.inventoryListbox;
      for(var i = 0; i < listbox.options.length; i++) {
         //if the option is selected, delete the option
         if(listbox.options[i].selected == true) {
           try {listbox.remove(i, null);} catch(error) {
                  listbox.remove(i);
            }
          }
       }
         totalListItems = listbox.options.length;
        document.getElementById("totalListItemId").innerHTML= "File Selected:"+totalListItems;
     });

     $('#removeAllSelectedNodeID').click(function removeAllItem(){
         removeAllSelectedNodeItem();
    });

    function removeAllSelectedNodeItem(){
        var listbox = document.fileRecoveryForm.inventoryListbox;
         listbox.innerHTML = ""
         totalListItems = listbox.options.length;
        document.getElementById("totalListItemId").innerHTML= "File Selected:"+totalListItems;
    } */

    /*
    *   DOM Ready Function
    */
    $(function() {
        
        //initOptions();

        initFilters();
        initEnableFileRecoveryWizard();
        var load = PHD.showLoadingIndicator("body", true, "Preparing File-Level Recovery Wizard......");

        setTimeout(function(){
            initBackupsTable();
            initForm();
            PHD.hideLoadingIndicator(load);
        }, 2000); 

        

    });

function initEnableFileRecoveryWizard(){
    $el = $form.find("#breadcrumbul");
    $el.find('li:not(:first)').remove();

    $.get("app/recover/flr/filerecoveryphysicalserverbreadcrums.html", function(filerecoverybreadcrumhtml) {
            $el = $form.find("#selectBackupBreadcrum");
            $el.after(filerecoverybreadcrumhtml);
    });
    $.get("app/recover/flr/filerecoverysteps.html", function(filerecoverystepshtml) {
            $el = $form.find("#file-recovery-steps");
            console.log($el);
            $el.empty().append(filerecoverystepshtml);
    });

}

function enableFileRecoveryWizard(){
    $form.find("#file-recovery-steps2 *").find('*').attr('disabled', false);
    //console.log($el);
    $form.find("#file-recovery-steps3 *").find('*').attr('disabled', false);
    //Enable 'Next' and disable 'Save' if backup is of a physical server
    $form.find("#id_next_btn").attr('disabled', false);
    $form.find("#id_save_btn").attr('disabled', true);
}

function disableFileRecoveryWizard(){
//    $el = $form.find("#breadcrumbul");
//    $el.find('li:not(:first)').remove();

    $form.find("#file-recovery-steps2 *").find('*').attr('disabled', true);
    //console.log($el);
    $form.find("#file-recovery-steps3 *").find('*').attr('disabled', true);
    // Allow user to 'Save' immediately if a non file-level backup.
    $form.find("#id_next_btn").attr('disabled', true);
    $form.find("#id_save_btn").attr('disabled', false);
    // It looks a little strange to remove the breadcrumbs.
    //$form.find("#fileBrowseBreadcrum").hide();
    //$form.find("#recoveryOptionBreadcrum").hide();
    
}

function initTarget(){
    $('#id_asset').empty();
    var resp = PHD.Ajax.get("/api/restore/targets/?backup_id="+lastSelectedBackup);
    resp.done(function(data) {
        console.log(data);
        $.each(data.targets,function(i,target){
            $('#id_asset').append("<option type="+target.client_os+" value="+target.client_id+">"+target.client_name+"</option>");
        });
        var selectionObject = backupCatalog.getSelectionObject();
        $('#id_asset').val(selectionObject.client_id);

    });
}
/****************************************************************Step2 Jstree code for file browser***********************************/
function initFileBrowserTree(){
    //with Lazy loading.	
    var parentId = " ";
    var backupSelectionObject =null;
    if(lastSelectedBackup == null ){
        lastSelectedBackup = backupCatalog.getSelectionObject().backups[0];
        initTarget();
    }
    if(backupCatalog.getSelectionObject().backups.length >0){
        if(lastSelectedBackup != backupCatalog.getSelectionObject().backups[0]){
            lastSelectedBackup = backupCatalog.getSelectionObject().backups[0];
            $('#fileBrowserTreeID').jstree("destroy");
            filePath="";
            totalListItems = 0;
          //  removeAllSelectedNodeItem();
            initTarget();
        }
    }
    $('#fileBrowserTreeID').jstree({
    	'core' : {
    	'data' : {
    		"url" : function( node ){
                backupSelectionObject = backupCatalog.getSelectionObject();
                console.log(backupSelectionObject);
    			console.log(node.id);
    			parentId = node.id;
        		if(backupSelectionObject.backups.length>0){
                    if( node.id == "#" ){
            			 return "/api/backups/"+synthCapable+"/"+backupSelectionObject.backups[0]+"/";
            			} else {
            			return "/api/backups/"+synthCapable+"/"+backupSelectionObject.backups[0]+"/?&dir="+parentId+"";
            		 }
                } else {

                }
    		},
    		'dataFilter' : function (data) {
				//console.log(data);
    			var jsonData=JSON.parse(data);
    			jsonData = jsonData.data;
    		
    			var treeArray = []; 
    			var treeObj={};
    			if(jsonData !== null && jsonData !== undefined){
    				for( i =0 ; i<jsonData.length ; i++ ){

    				   var index = jsonData[i].name.indexOf("/");
    							var nodeName = jsonData[i].name;
    							var nodeId = jsonData[i].id;
    							if(index != undefined && index !=null && index == -1){
    								nodeName = jsonData[i].name;
    								nodeId = jsonData[i].id+"/"; 
    							}

    				console.log(parentId);		
                    jsonData[i].isBranch = true;
    				treeObj = {"id":nodeId,"text":nodeName,icon:(jsonData[i].type === "directory" || jsonData[i].type === "volume")? "icon-hv-folder icon" : "icon-file icon",state: { opened: false},"children":(jsonData[i].type === "directory" || jsonData[i].type === "volume")? true: false};

    				treeArray.push(treeObj);
    			   }
    			}
    			console.log(treeArray);
    			var newdata1= JSON.stringify(treeArray);
    			console.log(newdata1);
    			return newdata1;
    		 }
       },
       'multiple' : false,
       'animation': true,
       'error' : function(error) {console.log('treeCtrl: error from js tree - ' + angular.toJson(error));},
       'check_callback' : true,
       'worker' : true
     }
    });
    
    $('#fileBrowserTreeID').on('select_node.jstree',function(e,item) {
    var i, j, r = [];
    for(i = 0, j = item.selected.length; i < j; i++) {
    var path = $('#fileBrowserTreeID').jstree(true).get_path(item.selected[i],"");
        path = path +"";
        filePath = path.replace(/,/g,"/");
        console.log(filePath);
     }
    });


}
	


})(jQuery, window.PHD || (window.PHD = {}), window);
