var flrBrowseModule = angular.module('flrBrowseModule', ['ngDialog', 'angulartics', 'angulartics.google.analytics', 'fileBrowserDialogApp', 'ui.grid','ui.grid.selection']);

flrBrowseModule.config(['ngDialogProvider', function (ngDialogProvider) {
    ngDialogProvider.setDefaults({
        plain: false,
        closeByDocument: true,
        closeByEscape: true,
        appendTo: false
    });
}]);

flrBrowseModule.controller('flrBrowseCtrl', ['$scope', '$rootScope', '$http', '$timeout', 'ngDialog', 'wizardService', '$analytics', 'DownloadService', 'gettextCatalog',
    function($scope, $rootScope, $http, $timeout, ngDialog, wizardService, $analytics, DownloadService, gettextCatalog) {

        var totalListItems = 0;
        $scope.flrObject = $scope.ngDialogData;
        $scope.isRemote = $scope.flrObject.remote !== undefined ? $scope.flrObject.remote : false;
        $scope.system_id = $scope.flrObject.remote ? 1 : $scope.flrObject.system_id;
        $scope.flr_flag = $scope.isRemote ? "Target FLR" : "Local FLR";
		var isParentNodeSelected = false;
        $scope.MAX_FILES_TO_LOAD = 200;
        /*
         * Get the system client ID, used to browse FLR files, then initialize the browser tree.
         */
        $scope.initializeController = function () {
            var url = $scope.isRemote ? "/api/clients/target-system" : "/api/clients/system";
            $http({
                method: 'get',
                url: url + '/?sid=' + $scope.system_id
            }).success(function(data, status, headers){
                if (data.system_client !== undefined) {
                    $scope.systemClientID = data.system_client.id;
                    $timeout(function() {
                        $scope.initFileBrowserTree();
                    });
                } else {
                    ngDialog.open({
                        dialogType:gettextCatalog.getString('Error'),
                        dialogMessage:gettextCatalog.getString('Cannot get system information required for browsing.')
                    })
                }
            }).error(function(response){
                ngDialog.open({
                    dialogType:gettextCatalog.getString('Error'),
                    dialogMessage:response.result[0].message
                })
            });
        };

        function validateFileBrowser(){
            $.validator.addMethod("isValidateFileBrowser", function(value, element) {
                    var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
                    return listbox.length > 0;
                },
                "Select files to recover"
            );
        }

        /* START: File browser tab for FLR-Bckup */
        $scope.initFileBrowserTree = function(){
			
            var parentId = " ";
            var backupSelectionObject =null;
			isJstreeBrowserDialog = true;
		     $('#fileBrowserTreeID').jstree({
                'plugins' : ['types','dnd'],
                'core' : {
                    'data' : {
                        'url' : function(node){
                            var last = node.original !== undefined && node.original.last !== undefined ? node.original.last : "";
                            $scope.priorChildren = [];
                            var addLast = "";
                            if (last !== "") {
                                addLast = "&last=" + encodeURIComponent(last);
                                node.id = node.parent;
                                var docID = document.getElementById(node.parent);
                                var parent = $('#fileBrowserTreeID').jstree('get_json', docID);
                                if (parent !== undefined && parent.children instanceof Array) {
                                    // Create object array of these children, not including the parent itself.
                                    for (var i = 0; i < parent.children.length; i++) {
                                        var item = parent.children[i];
                                        if (item.id != node.id) {
                                            var obj = { 'id':item.id,
                                                'text': item.text,
                                                'icon': item.icon,
                                                'state': item.state,
                                                'children': item.children };
                                            $scope.priorChildren.push(obj);
                                        }
                                    }
                                }
                            }
                            parentId = encodeURIComponent(node.id);
                            $scope.nodeId=node.id;
                            console.log(node);
                            var baseURL = $scope.isRemote ? "/api/clients/target-files/" : "/api/clients/files/";
                            var addSID = "/?sid=" + $scope.system_id;
                            var addDir = "&dir=" + $scope.flrObject.restore_path;
                            var addCount = "&count=" + $scope.MAX_FILES_TO_LOAD;
                            if( node.id == "#" ){
                                // The root path comes from the FLR status REST API.
								isParentNodeSelected = true;
                                return baseURL + $scope.systemClientID + addSID + addDir + addCount + addLast;
                            } else {
							    isParentNodeSelected = false;
							    addDir = "&dir=" + parentId;
                                return baseURL + $scope.systemClientID + addSID + addDir + addCount + addLast;
                            }
                        },
                        'dataFilter' : function (data) {
							isJstreeBrowserDialog = false;
                            var jsonData=JSON.parse(data);
                            jsonData = jsonData.data;

                            var treeArray = $scope.priorChildren !== undefined ? $scope.priorChildren : [];
                            var treeObj={};
                            if(jsonData !== null && jsonData !== undefined){
                                var lengthOfJsonData =jsonData.length;

                                for( i =0 ; i<lengthOfJsonData ; i++ ){
                                    treeObj = $scope.createTreeObj(jsonData[i]);
                                    treeArray.push(treeObj);
                                }
                            }
                            var newdata1= JSON.stringify(treeArray);
                            return newdata1;
                        }
                    },
                    'multiple' : true,
                    'animation': true,
                    'error' : function(error) {
					    
						var messageData = JSON.parse(error.data).xhr.responseText;
						var errorMessage = JSON.parse(messageData);
					    ngDialog.open({
							dialogType:'Error',
							modelDialogId:'fileBrowserErrorDialog',
							scope:$scope,
							dialogMessage: errorMessage.result[0].message,
							onConfirmOkButtonClick:'onConfirmErrorDialog()',
							closeByEscape: false
                       });
					
					 },
                    'check_callback' : true,
                    'worker' : true
                }

            });
            
            $('#fileBrowserTreeID').on('select_node.jstree',function(e,item) {
                var i, j, r = [];
                $scope.filePathArray = [];
                for(i = 0, j = item.selected.length; i < j; i++) {
                    if (item.node !== undefined && item.node.original !== undefined &&
                        item.node.original.last !== undefined && (item.node.original.last != "")) {
                        // This node has "last" set, it is special.  Get more files and leave (do not select it)
                        $('#fileBrowserTreeID').jstree('load_node', item.node);
                        break;
                    }
                    var path = $('#fileBrowserTreeID').jstree(true).get_path(item.selected[i],"");
                    path = path +"";
                    var filePath = path.replace(/,/g,"");
                    $scope.filePathArray.push(filePath);
                }
            });

            $scope.createTreeObj = function(jsonItem) {
                var nodeName = jsonItem.name;
                var nodeId = jsonItem.id;
                var lastChar = nodeName.substr(nodeName.length - 1);
                if (lastChar != undefined && lastChar != "/") {
                    nodeName = jsonItem.name + "/";
                    nodeId = jsonItem.id + "/";
                }
                jsonItem.isBranch = true;
                var last = jsonItem.last !== undefined ? jsonItem.last : "";
                return { "id":nodeId,
                    "text":nodeName,
                    icon: (last !== "") ? "icon-plussign" :
                        ((jsonItem.type === "directory" ||  jsonItem.type === "volume")? "icon-hv-folder icon" : "icon-file icon"),
                    state: { opened: false},
                    last: last,
                    "children": (jsonItem.type === "directory" || jsonItem.type === "volume")};
            };
        };

		$scope.onConfirmStateWarning = function() {
            ngDialog.close('Information-recover-child-dialog');
        };
		$scope.onConfirmErrorDialog = function () {
			console.log("in confirm:::"+isJstreeBrowserDialog);
		    isJstreeBrowserDialog = false;
			ngDialog.close('fileBrowserErrorDialog');
			if(isParentNodeSelected == true){
			  ngDialog.close('flr-browse-dialog');
			}
			
        };
		
        $scope.addItem = function(){
            console.log($scope.filePathArray);
            if ($scope.filePathArray === undefined) {
                $scope.filePathArray = [];
            }
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            for(var i=0; i<$scope.filePathArray.length;i++){
                addOption(listbox, $scope.filePathArray[i]);
            }
        };

        function addOption(selectbox,node){
            console.log(node);
            var optn = document.createElement("OPTION");
            optn.text = node;
            optn.value = node;

            if(!$("#listboxId option[value='"+node+"']").length > 0 && !angular.isUndefined(node) && node !== null){
                selectbox.options.add(optn);
                totalListItems = selectbox.options.length;
                document.getElementById("totalListItemId").innerHTML= gettextCatalog.getString("File Selected:")+totalListItems;
            }
        }

        $scope.removeItem=function(){
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;

            $(listbox.options).each(function(index) {
                var elem = $(this)[0];
                if (elem.selected) {
                    $(elem).remove();
                }
            });

            totalListItems = listbox.options.length;
            document.getElementById("totalListItemId").innerHTML= gettextCatalog.getString("File Selected:")+totalListItems;
        };

        $scope.removeAllItem=function(){
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            listbox.innerHTML = "";
            totalListItems = listbox.options.length;
            document.getElementById("totalListItemId").innerHTML= gettextCatalog.getString("File Selected:")+totalListItems;
        };

        function validateFileBrowserTab(){
            validateFileBrowser();
            $("#flr-file-browser-form").validate({
                errorLabelContainer: ".errorlist",
                errorContainer: "#dialog-errors",
                showErrors: function(errorMap, errorList) {
                    $("#dialog-errors").find(".summary")
                        .html(gettext("Correct the following errors:"));
                    this.defaultShowErrors();
                },
                rules: {
                    inventoryListbox:{
                        isValidateFileBrowser:true
                    }
                },
                messages: {
                    isValidateFileBrowser:"Select files to recover"
                }
            });
            if((!$('#flr-file-browser-form').valid())) {
                return false;
            }else{
                return true;
            }
        }
		
        $scope.closeFileBrowserDialog = function(){
			 isJstreeBrowserDialog = false;
			 ngDialog.close('flr-browse-dialog');
		}
        $scope.downloadSelectedFiles = function() {
			isJstreeBrowserDialog = false;
            if (validateFileBrowserTab()) {
                var message = gettextCatalog.getString("Do you wish to download the selected files?");
                ngDialog.open({
                    dialogType: gettextCatalog.getString('CONFIRMATION'),
                    modelDialogId: 'ChooseDownloadFLR',
                    scope: $scope,
                    dialogMessage: message,
                    onConfirmOkButtonClick: 'onDownload()'
                });
            }
        };

        $scope.onDownload = function() {
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            var fileList = [];
            var restore_path = $scope.flrObject.restore_path;
            for(var i = 0; i < listbox.options.length; i++) {
                var fileName = listbox.options[i].value;
                var delim = "";

                if ((fileName.indexOf("@@@:/backups/_flr/") === -1) && (fileName.indexOf("@@@:/backups/rae") === -1) && fileName.indexOf("@@@:/backups/backups_restore_share/") === -1) {
                    // if restore path doesn't end with '/', append it
                    if (restore_path[restore_path.length - 1] != '/') {
                        delim = "/";
                    } else {
                        delim = "";
                    }
                    // Add restore path if full path not present.
                    fileName = restore_path + delim + fileName;
                } else {
                    // Strip the '@@@:' if full path is present.
                    var afterAt = 4; // length of '@@@:'
                    fileName = fileName.substring(afterAt);
                }
                // remove end slash if present
                if (fileName[fileName.length - 1] === '/') {
                    fileName = fileName.slice(0, -1);
                }
                console.log(fileName);
                fileList.push(fileName);
            }

            ngDialog.close('ChooseDownloadFLR');
            listbox.innerHTML = "";
            console.log("download files");
            var delimiter = "";
            if (restore_path[restore_path.length - 1] != '/') {
                delimiter = "/";
            } else {
                delimiter = "";
            }
            ngDialog.open({
                dialogType: 'Information',
                modelDialogId: 'downloadInfoDialog',
                scope: $scope,
                dialogMessage: "Files are being prepared for download. It might take some time for the download to start.",
                onConfirmOkButtonClick: 'onConfirmInformation()'
            });
            var baseURL = $scope.isRemote ? "/api/restore/target-download-files" : "/api/restore/download-files";
            var fileObj = {};
            fileObj.filenames = fileList;
            fileObj.restore_path = restore_path + delimiter;
            fileObj = JSON.stringify(fileObj);
            $http({
                method: 'post',
                url: baseURL + '/?sid=' + $scope.system_id,
                data: fileObj,
                responseType: "arraybuffer"
            }).success(function(data, status, headers){
                console.log("success");
                headers = headers();
                var contentType = headers['content-type'];
                // See if we need to display information or download a file.
                if (contentType.indexOf('application/json') !== -1) {
                    // convert response array buffer to JSON to get information.
                    var response = String.fromCharCode.apply(null, new Uint8Array(data));
                    response = JSON.parse(response);
                    $scope.URLlink = response.link;
                    if ($scope.URLlink !== undefined) {
                        ngDialog.close('downloadInfoDialog');
                        var urlZip = response.urlzip;
                        if (urlZip !== undefined) {
                            var fullPath = $scope.URLlink + urlZip;
                            var baseName = urlZip.substring(urlZip.lastIndexOf("/") + 1);
                            DownloadService.saveAs(fullPath, baseName);
                            $scope.downloadFullPath = fullPath;
                            ngDialog.open({
                                template: 'app/recover/download-information.html',
                                scope: $scope,
                                overlay:true,
                                modelDialogId:'DownloadLinkDialog',
                                ngDialogStyle:'width:440px;height:180px;'
                            });
                        } else {
                            ngDialog.open({
                                dialogType:'Error',
                                modelDialogId:'DownloadErrorDialog',
                                scope:$scope,
                                dialogMessage: 'The files have been placed in the downloads location but the link is unavailable.',
                                onConfirmOkButtonClick:'onConfirmError()'
                            });
                        }
                    } else {
                        ngDialog.open({
                            dialogType:'Error',
                            modelDialogId:'DownloadErrorDialog',
                            scope:$scope,
                            dialogMessage: 'The files have been placed in the downloads location but the link is unavailable.',
                            onConfirmOkButtonClick:"onConfirmError()"
                        });
                    }
                } else {
                    var filename = "Unitrends-Restore.zip";
                    console.log(headers);
                    var blob = new Blob([data], {type: contentType});
                    DownloadService.downloadFile(blob, filename);
                    ngDialog.closeAll();
                }
            }).error(function(response){
                // convert response array buffer to JSON to get error message.
                response = String.fromCharCode.apply(null, new Uint8Array(response));
                response = JSON.parse(response);
                ngDialog.open({
                    dialogType:'Error',
                    modelDialogId:'DownloadErrorDialog',
                    scope:$scope,
                    dialogMessage: response.result[0].message,
                    onConfirmOkButtonClick:'onConfirmError()'
                })
            });
        };

        $scope.onConfirmInformation = function() {
            ngDialog.closeAll();
        };

        $scope.onConfirmError = function() {
            ngDialog.close('DownloadErrorDialog');
        };

        $(document).on('dnd_move.vakata', function (e, data) {
            var t = $(data.event.target);
            if (isAPlusSign($(data.element.children))) {
                return;
            }
            if(!t.closest('.jstree').length) {
                if(t.closest('.validate-filebrowser').length) {
                    data.helper.find('.jstree-icon').removeClass('jstree-er').addClass('jstree-ok');
                }
                else {
                    data.helper.find('.jstree-icon').removeClass('jstree-ok').addClass('jstree-er');
                }
            }

        }).on('dnd_stop.vakata', function (e, data) {
            console.log(data.data.nodes);
            if (isAPlusSign($(data.element.children))) {
                return;
            }
            var t = $(data.event.target);
            if(!t.closest('.jstree').length) {
                if(t.closest('.validate-filebrowser').length) {
                    console.log(data.data.nodes);
                    addDragNodesInList(data.data.nodes);
                    $(data.element).clone().appendTo(t.closest('.validate-filebrowser'));
                }
            }
        });

        function isAPlusSign($node) {
            var match = false;
            if ($node.length > 0) {
                var child = $node[0];
                var $child = $(child);
                match = $child.hasClass('icon-plussign');
                if (match) {
                    console.log("is an adder (plus).....");
                }
            }
            return match;
        }

        function addDragNodesInList(textArray){
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            for(var i=0;i<textArray.length;i++){
                if(textArray[i] != null){
                    addOption(listbox,textArray[i]);
                }
            }
        }

    }]);

