var irModule = angular.module('flrModule', ['ngDialog', 'angulartics', 'angulartics.google.analytics','ui.grid.selection']);

irModule.config(['ngDialogProvider', function (ngDialogProvider) {
    ngDialogProvider.setDefaults({
        plain: false,
        closeByDocument: true,
        closeByEscape: true,
        appendTo: false,
        preCloseCallback: function (value) {

        }
    });
}]);

irModule.factory('irFactory', function($http, ngDialog) {
    return {
        getIRDetail: function(method, url, sid,dataObj, type, hypervisor) {
            var params = {};
            if (sid != 0) {
                params.sid = sid;
                if(type !== "" ){
                    params.type = type;
                }
            }
            var promise = $http({
                url: url,
                data: dataObj,
                method: method,
                params: params
            }).success(function(data, status, headers){

            }).error(function(response){
                ngDialog.open({
                    dialogType:'error',
                    dialogMessage:response.result[0].message
                });
            });
            return promise;
        }
    };
});

irModule.controller('flrController', ['$scope', '$rootScope', '$http', 'ngDialog', 'wizardService', '$analytics', 'irFactory', function($scope, $rootScope, $http, ngDialog, wizardService, $analytics, irFactory ) {

    $scope.vfcDetails = {
        virtualClientMemory:0,
        virtualClientProcessor:0,
        number_of_recovery_points:0
    };
    $scope.isSelecetedTypeVMware = hypervisorType === HYPERVISOR_VMWARE_HOST;
    $scope.isNewBackups = false;
    $scope.isNewBackupsDisabled = false;

    $scope.isNewBackupsHidden = liveTime !== "";

    $scope.vfcOptionsGridDataArray = [];
    $scope.vfccolumnDefintion = [];
    $scope.wirVfcObject = {
        tabs:[]
    };
    $scope.dataGrids = [];
    $scope.dataGridOption = {};
    $scope.getGridData = [];
    $scope.gridList = [];

    $scope.hasApplications = false;

    $scope.maxRecoveryPoints = $rootScope.maxRecoveryPoints;

    var virtualStateURL = "";
    if (jobType === 'replica'){
        virtualStateURL = "/api/replicas/" + replicaID + "/state/";
    } else {
        virtualStateURL = "/api/virtual_clients/" + virtualId + "/state/";
    }
    irFactory.getIRDetail('get',virtualStateURL,sid,"","","").then(function(data){
        $scope.isNewBackupsDisabled = data.data.disabled;
        if($scope.isNewBackupsDisabled){
            $("#newBackupCheckId").removeAttr("disabled ");
        }else if(!$scope.isNewBackupsDisabled){
            $("#newBackupCheckId").removeAttr("disabled");
            $scope.isNewBackups = true;
        }

    });// End

    var virtualDetailsURL = "";

    if (jobType === 'replica'){
        virtualDetailsURL = "/api/replicas/" + replicaID +"/details/";
    } else {
        virtualDetailsURL = "/api/virtual_clients/"+virtualId+"/details/";
    }

    irFactory.getIRDetail('get',virtualDetailsURL,sid,"",virtualType,"").then(function(data){

        if (jobType === "replica"){
            $scope.vfcDetails.virtualClientMemory = data.data.details.memory;
            $scope.vfcDetails.virtualClientProcessor = data.data.details.processors;
            $scope.vfcDetails.virtualClientSnaps = data.data.details.number_of_snaps;
        } else {
            $scope.vfcDetails.virtualClientMemory = data.data.virtual_client.memory;
            $scope.vfcDetails.virtualClientProcessor = data.data.virtual_client.processors;
            $scope.vfcDetails.auditVerify = data.data.virtual_client.audit_verify;
        }
        var min_memory = 1,
            max_memory = 6275072,
            memory_step = 1,
            min_processors = 1,
            min_snap = 1,
            max_snap = 1,
            snap_step = 1,
            max_processors = jobType === 'replica' ? 128 : 99;


        if (hypervisorType === HYPERVISOR_VMWARE_HOST) {
            min_memory = 1024;
            memory_step = 4;

            $("#ir-edit-processor").spinner({min: min_processors, max: max_processors});
            $("#ir-edit-memory").spinner({min: min_memory, max: max_memory, step: memory_step});
            if (jobType === 'replica'){
                min_snap = 1;
                max_snap = $scope.maxRecoveryPoints;
                snap_step = 1;

                $("#ir-edit-recovery-points").spinner({min: min_snap, max: max_snap, step: snap_step});
            }
            $scope.min_processors = min_processors;
            $scope.max_processors =max_processors;
            $scope.min_memory=min_memory;
            $scope.max_memory=max_memory;
            $scope.min_snap = min_snap;
            $scope.max_snap = max_snap;

        } else if (hypervisorType === HYPERVISOR_HYPERV_HOST) {
            min_memory = 1024;
            memory_step = 2;
            $("#ir-edit-processor").spinner({min: min_processors, max: max_processors});
            $("#ir-edit-memory").spinner({min: min_memory, max: max_memory, step: memory_step});
            $scope.min_processors = min_processors;
            $scope.max_processors =max_processors;
            $scope.min_memory=min_memory;
            $scope.max_memory=max_memory;

        } else if (hypervisorType === HYPERVISOR_UNITRENDS_APPLIANCE) {
            max_processors = 4;
            max_memory = 8192;
            var params = {};
            params.sid = sid;
            $http({
                url: "/api/systems/details/",
                method: "get",
                params: params
            }).success(function (data) {
                max_processors = data.appliance.processors;
                var str_memory = data.appliance.memory.toString();
                str_memory.replace(" GB", "");
                max_memory = parseInt(str_memory, 10) * 1000;
                $("#ir-edit-processor").spinner({min: min_processors, max: max_processors});
                $("#ir-edit-memory").spinner({min: min_memory, max: max_memory, step: memory_step});
                $scope.min_processors = min_processors;
                $scope.max_processors =max_processors;
                $scope.min_memory=min_memory;
                $scope.max_memory=max_memory;

            })
        }

        $scope.min_processors = min_processors;
        $scope.max_processors =max_processors;
        $scope.min_memory=min_memory;
        $scope.max_memory=max_memory;

        if (jobType !== 'replica') {
            $scope.getApplicationData(data.data.virtual_client.client_id);
        }

    });// End

    $scope.getApplicationData = function(client_id) {
        var tabParams = {};
        tabParams.sid = sid;
        tabParams.client_id = client_id;

        $http({
            url: "/api/applications/",
            method: "get",
            params: tabParams
        }).success(function(data, status, headers) {
            var applicationData = data.data[0].applications;
            if(typeof(data.data[0].applications)!= "undefined") {
                for(var i=0; i<data.data[0].applications.length; i++) {
                    if(data.data[0].applications[i].id === APP_ID_EXCHANGE_2003
                        ||data.data[0].applications[i].id === APP_ID_EXCHANGE_2007
                        ||data.data[0].applications[i].id === APP_ID_EXCHANGE_2010
                        ||data.data[0].applications[i].id === APP_ID_EXCHANGE_2013
                        ||data.data[0].applications[i].id === APP_ID_EXCHANGE_2016
                        ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2005
                        ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2008
                        ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2008_R2
                        ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2012
                        ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2014
                        ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2016
                        ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2017) {
                        $scope.wirVfcObject.tabs.push(data.data[0].applications[i].name);
                        $scope.vfcOptionsGridDataArray.push(data.data[0].applications[i].children);
                        $scope.hasApplications = true;

                        if(data.data[0].applications[i].id === APP_ID_SQL_SERVER_2005
                            ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2008
                            ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2008_R2
                            ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2012
                            ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2014
                            ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2016
                            ||data.data[0].applications[i].id === APP_ID_SQL_SERVER_2017) {
                            $scope.vfccolumnDefintion.push([{ field: 'name',displayName: 'Instance', showHeader: false,cellFilter:'getInstance'},{ field: 'name',displayName: 'Database', showHeader: false,cellFilter:'getDatabase'}]);
                        } else {
                            $scope.vfccolumnDefintion.push([{ field: 'name',displayName: 'Database', showHeader: false}]);
                        }

                    }
                }
            }
            $scope.createGrid();
        }).error(function(response) {
            PHD.hideLoadingIndicator(load);
            ngDialog.open({
                dialogType:'retry',
                dialogMessage:response.result[0].message
            });
        });
    };

    $scope.createGrid = function(){
        for (var i = 0; i < $scope.vfcOptionsGridDataArray.length; i++) {
            $scope.dataGridOption = {
                columnDefs: $scope.vfccolumnDefintion[i],
                data: $scope.vfcOptionsGridDataArray[i],
                onRegisterApi: function(gridApi) {
                    //set gridApi on scope
                    $scope.gridApi = gridApi;
                    $scope.gridList.push(gridApi);
                    gridApi.selection.on.rowSelectionChanged($scope, function(row){
                        if(row.entity.is_system_db != undefined){
                            if((!row.isSelected) && row.entity.is_system_db){
                                row.setSelected(true);
                            }
                        }
                    });
                },
                isRowSelectable: function(row) {
                    if(row.entity.is_system_db != undefined) {
                        if(row.entity.is_system_db) {
                            row.setSelected(true);
                        }
                    }
                    if(row.entity.wvf != undefined && row.entity.wvf) {
                        row.setSelected(true);
                    }
                    return true;
                }
            };
            $scope.dataGrids.push($scope.dataGridOption);
        }
    };

    $scope.newBackupChange = function(isNewBackupChanged){
        $scope.isNewBackups = isNewBackupChanged;
    };

    $scope.startStopBackups = function(){
        $scope.urlStartStop = "";
        if ($scope.isNewBackups){
            $scope.urlStartStop = jobType === 'replica' ? "/api/replicas/restores/start/" + replicaID + "/" : "/api/virtual_clients/restores/start/"+virtualId+"/";
        } else if(!$scope.isNewBackups){
            $scope.urlStartStop = jobType === 'replica' ? "/api/replicas/restores/stop/" + replicaID + "/" : "/api/virtual_clients/restores/stop/"+virtualId+"/";
        }
        irFactory.getIRDetail('put',$scope.urlStartStop,sid,"","","").then(function(data){
        });
    };

    $scope.virtualClientDetails = {client:{}};
    var parameters={};

    $scope.tab = 0;
    $scope.currentTab = 0;

    $scope.replicaDetails = {replica:{}};

    $scope.showCurrentTab = function(currentTab){
        $scope.currentTab = currentTab;
        $scope.tab = currentTab
        $(window).resize();
    };

    $scope.isSelected = function(checkTab) {
        return $scope.tab === checkTab;
    };

    $scope.saveVirtualClientDetails = function(){
        var processor = $( "#ir-edit-processor" ).val();
        var memory = $("#ir-edit-memory").val();
        var recovery_points = $("#ir-edit-recovery-points").val();

        $scope.vfcDetails.virtualClientProcessor = processor;
        $scope.vfcDetails.virtualClientMemory = memory;
        $scope.vfcDetails.virtualClientSnaps = recovery_points;
        $scope.processorValidFlag = false;
        $scope.memoryValidFlag = false;
        $scope.checkMemoryStepFlag = false;
        $scope.maxSnapsValidFlag = false;

        $scope.memoryStep = 1;

        if (hypervisorType === HYPERVISOR_VMWARE_HOST) {
            $scope.memoryStep = 4;
            $scope.recoveryPointStep = 1;
        } else if (hypervisorType === HYPERVISOR_HYPERV_HOST) {
            $scope.memoryStep = 2;
        }

        if (processor < $scope.min_processors || processor > $scope.max_processors) {
            $scope.processorValidFlag = true;
        }
        if (memory < $scope.min_memory || memory > $scope.max_memory) {
            $scope.memoryValidFlag = true;
        }
        if (memory % $scope.memoryStep !== 0) {
            $scope.checkMemoryStepFlag = true;
        }

        if (recovery_points && recovery_points < 1 || recovery_points > $scope.maxRecoveryPoints){
            $scope.maxSnapsValidFlag = true;
        }

        if ($scope.processorValidFlag || $scope.memoryValidFlag || $scope.checkMemoryStepFlag  || $scope.maxSnapsValidFlag) {
            return;
        }

        if (jobType === 'replica'){
            $scope.replicaDetails.replica.processors = $scope.vfcDetails.virtualClientProcessor;
            $scope.replicaDetails.replica.memory = $scope.vfcDetails.virtualClientMemory;
            $scope.replicaDetails.replica.number_of_snaps = $scope.vfcDetails.virtualClientSnaps;
        } else {
            $scope.virtualClientDetails.client.processors = $scope.vfcDetails.virtualClientProcessor;
            $scope.virtualClientDetails.client.memory = $scope.vfcDetails.virtualClientMemory;
            $scope.virtualClientDetails.client.audit_verify = $scope.vfcDetails.auditVerify;
        }
        var databaseArray = [];
        for(var i = 0; i < $scope.gridList.length; i++){
            var selectedDB = eval($scope.gridList[i]).selection.getSelectedRows();
            if (selectedDB != undefined || selectedDB != null) {
                for (var j = 0; j < selectedDB.length; j++) {
                    if (selectedDB[j].is_system_db != undefined) { // As per businesslogic that do not include system DB
                        if (selectedDB[j].is_system_db) {
                            continue;
                        }
                    }
                    databaseArray.push(selectedDB[j].id);
                }
            }
        }

        if (databaseArray.length > 0) {
            $scope.virtualClientDetails.client.instances = databaseArray.toString();
        }

        var detailsPayload = "";
        var clientDetailURL = "";
        if (jobType === 'replica'){
            clientDetailURL = "/api/replicas/" + replicaID + "/";
            detailsPayload = $scope.replicaDetails;
        } else {
            clientDetailURL = "/api/virtual_clients/" + virtualId + "/";
            detailsPayload = $scope.virtualClientDetails;
        }
        parameters.sid = sid;
        $http({
            method: 'PUT',
            params:parameters,
            url :clientDetailURL,
            data: JSON.stringify(detailsPayload)
        }).success(function(data, status, headers){
            $scope.startStopBackups();
            ngDialog.open({
                dialogType:'Information',
                dialogMessage:"Successfully Updated"
            })
        }).error(function(response) {
            ngDialog.open({
                dialogType:'retry',
                modelDialogId:'irEditDialog',
                dialogMessage:response.result[0].message
            })
        });
    }
}]);//end of controller