var recover = angular.module('recover', ['angulartics', 'angulartics.google.analytics','ngDialog','flrModule']);

/*
 * Controller for the Recover page.
*/

recover.controller('RecoverCtrl', ['$scope', '$rootScope', '$window', '$http', '$timeout', 'Page', '$analytics', 'ngDialog',
        'wizardService', '$compile', 'gettextCatalog', '$protectService', 'localStorageService',
    function($scope, $rootScope, $window, $http,$timeout, Page, $analytics, ngDialog,
             wizardService, $compile, gettextCatalog, $protectService, localStorageService) {

		Page.setTitle('Recover');
        $scope.flr_flag = "BackupCopy";
        ngDialogService = ngDialog;

        var filterPageProcessed = false;

        $scope.filterLoaded = function() {
            var PHD = window.PHD;

            // clean up filter.
            var $footer = $("#recovery-backup-footer");
            var replace = $footer.html().replace(/filter_namespace/g, "recover");
            $footer.html(replace);

            // wrap existing jQuery in functions and run after the filter (ng-include) is loaded.
            processBackupFilter(PHD, window, ngDialog, $rootScope, $scope, $http, localStorageService, gettextCatalog);

            filterPageProcessed = true;

            $scope.setCatalogView();
        };

            // called when the last ng-include on the page is loaded.
        // The main recover page functions must load after the filter.
            $scope.loadRecoverPage = function() {

            if (filterPageProcessed) {
                processRecoverPage(PHD, window, $rootScope, $analytics, $scope, gettextCatalog);
            } else {
                // add a timeout for filter processing.
                $timeout(function() {
                    processRecoverPage(PHD, window, $rootScope, $analytics, $scope, gettextCatalog);
                }, 500);
            }
            };

		    /*******************On click of Window Instant Recovery button open dialog with job wizard*****************/
			$scope.openWindowRecoverDialog = function(){
				if (!$('#winIRBtn').hasClass("disabled")) {
                    ngDialog.open({
                        template: 'app/recover/wir/windowRecoveryWizard.html',
                        scope: $scope,
                        overlay: true,
                        modelDialogId: 'window-instant-recover-dialog',
                        ngDialogStyle: 'width:82em; height:60em;  top: -150px;left: 80px;'
                    });
                }
		    };
            /*On click of Show Details open dialog*/
            $scope.openSearchFLRDialog = function() {
                if (!$('#searchFilesBtn').hasClass("disabled")) {
                    console.log('openSearchFLRDialog');
                    ngDialog.open({
                        template: 'app/recover/filerecovery/searchFLR/searchFLR.html',
                        scope: $scope,
                        overlay: true,
                        modelDialogId: 'flr-search-dialog',
                        ngDialogPostionStyle: 'top:40px;',
                        ngDialogStyle: 'width:900px;height:700px;'
                    });
                }
               
            };
            /* On click of Recover button open dialog */
            $scope.openRecoverDialog = function() {
		        isIRJob = false;
                if (!$('#restoreBtn').hasClass("disabled")) {
                    if ((app_type != undefined) && (app_type === APPLICATION_TYPE_NAME_UCS_SERVICE_PROFILE)) {
                        // UNIBP- For Cisco UCS, we are really doing an FLR, but we are labeling the button as Recover.
                        $scope.flr_flag = "Backup";
                        $scope.is_cisco_ucs_restore = true;
                        ngDialog.open({
                            template: 'app/recover/filerecovery/fileLevelRecoveryJobWizard.html',
                            scope: $scope,
                            overlay: true,
                            modelDialogId: 'create-filelevel-recovery-dialog',
                            ngDialogStyle: 'width:120em; height:70em;  top: -150px;left: 80px;'
                        });
                    } else {
                        ngDialog.open({
                            template: 'app/recover/recoveryWizard.html',
                            scope: $scope,
                            overlay: true,
                            modelDialogId: 'recover-dialog',
                            ngDialogStyle: (app_type === APPLICATION_TYPE_NAME_SHAREPOINT || app_type === APPLICATION_TYPE_NAME_ORACLE) ? 'width:100em;top: -140px;left: 80px;' : 'width:120em;top: -150px;left: 80px;'
                        });
                    }
                }
            };

            /* On click of VM Instant Recovery button open dialog */
            $scope.openIRDialog = function() {
                isIRJob = true;
                if (!$('#irBtn').hasClass("disabled") && !$('#vmIRBtn').hasClass("disabled")) {
                    ngDialog.open({
                        template: 'app/recover/recoveryWizard.html',
                        scope: $scope,
                        overlay: true,
                        modelDialogId: 'recover-dialog',
                        ngDialogStyle: 'width:120em; height:65em;  top: -150px;left: 80px;'
                    });
                }
            };

            $scope.openVMReplicaDialog = function(){
                $protectService.openVMReplicaDialog($http, $scope, $rootScope, ngDialog);
            };

            $scope.openFileBrowserDialog = function() {
                if (!$("#flrBrowse").hasClass("disabled")) {
                    console.log('openFileBrowserDialog');
                    console.log(selectedFLR);

                    $analytics && $analytics.eventTrack('Browse FLR', {category: 'Recover', label: 'Recover'});

                    // if selected FLR doesn't have CIFS, present a message that it's an unsupported file system
                    if (!selectedFLR.cifs) {
                        if (selectedFLR.remote == "true") {
                            ngDialog.open({
                                dialogType: 'notice',
                                dialogMessage: 'File Level Recovery cannot be completed because we are unable to browse files within this recovery point. ' +
                                    'This may be the result of limited access to the file system within the recovery point.\n\n ' +
                                    'Return to the Recover -> Backup Catalog interface and select the “Import to Source” button to restore the entire recovery point.',
                                ngDialogStyle:'width: 575px;'
                            })
                        } else {
                            ngDialog.open({
                                dialogType: 'notice',
                                dialogMessage: 'File Level Recovery cannot be completed because we are unable to browse files within this recovery point. ' +
                                    'This may be the result of limited access to the file system within the recovery point. \n\n' +
                                    'Select the “Show Details” button for instructions on completing the File Level Recovery operation using the iSCSI interface to access data.',
                                ngDialogStyle:'width: 575px;'
                            })
                        }
                    } else {
                        if ($scope.fileBrowserDialogOpen === false || $scope.fileBrowserDialogOpen === undefined) {
                            $scope.fileBrowserDialogOpen = true;

                            var url = "/api/jobs/active/flr/" + selectedFLR.id + "/?sid=" + selectedFLR.system_id;
                            $http({
                                method: 'get',
                                url: url
                            }).success(function (data) {
                                if (data && data.data !== undefined) {
                                    var obj = data.data;
                                    if (obj.restore_path !== undefined && obj.restore_path !== null) {
                                        console.log(obj);
                                        ngDialog.open({
                                            template: 'app/recover/flr/flrBrowser.html',
                                            scope: $scope,
                                            overlay: true,
                                            data: obj,
                                            modelDialogId: 'flr-browse-dialog',
                                            ngDialogPostionStyle: 'top:10%;',
                                            ngDialogStyle: 'width:1100px;height:62em;',
                                            closeByEscape: false,
                                            preCloseCallback: function (value) {
                                                $scope.fileBrowserDialogOpen = false;
                                            }
                                        });
                                    } else {
                                        $scope.fileBrowserDialogOpen = false;
                                    }
                                } else {
                                    $scope.fileBrowserDialogOpen = false;
                                    ngDialog.open({
                                        dialogType: 'error',
                                        dialogMessage: 'There is no restore path available for browsing.  Please use iSCSI or the Samba share to restore files.'
                                    })
                                }
                            }).error(function (response) {
                                $scope.fileBrowserDialogOpen = false;
                                ngDialog.open({
                                    dialogType: 'retry',
                                    dialogMessage: response.result[0].message
                                })
                            });
                        }
                    }
                }
            };
        
        // Edit WIR
        $scope.openEditDialog = function(){
            $scope.vmName = vmName;
            if (!$('#editReplicaBtn').hasClass("disabled") ) {
                if (jobType === "replica"){
                    ngDialog.open({
                        template: 'app/recover/ir/replica-edit-dialog/replica-edit-dialog.html',
                        scope: $scope,
                        overlay: true,
                        modelDialogId: 'replicaEditDialog',
                        ngDialogStyle: 'min-width: 46em; top: -90px;left: 27px;'
                    });
                } else {
                    ngDialog.open({
                        template: 'app/recover/ir/ir-edit-dialog/ir-edit-dialog.html',
                        scope: $scope,
                        overlay: true,
                        modelDialogId: 'irEditDialog',
                        ngDialogStyle: 'min-width: 46em; top: -90px;left: 27px;'
                    });
                }
            }
        };

        $scope.handleReplicaLive = function(){
            if(!$('#replica_btn_live').hasClass("disabled")) {
                if (goLive) {
                    ngDialog.open({
                        dialogType: 'Confirmation',
                        dialogMessage: "Once you go live, any future backups will not be restored to this replica, as it will be marked invalid. Are you sure you wish to continue? ",
                        scope: this,
                        overlay: true,
                        modelDialogId: 'goLiveApiConfirmationDialog',
                        onConfirmOkButtonClick: 'goLiveApiCall()'
                    });
                } else {
                    var url = jobType === "wir" ? "/api/virtual_clients/live/stop/" + virtual_id + "/?sid=" + sid : "/api/replicas/live/stop/" + replicaID + "/?sid=" + sid;
                    $http ({
                        method: 'put',
                        url : url
                    }).success(function(data){
                        goLive = true;

                    }).error(function(response) {
                        ngDialog.open({
                            dialogType: 'error',
                            dialogMessage: response.result[0].message
                        })
                    });
                }
            }
        };

        $scope.goLiveApiCall=function(){
            var url = jobType === "wir" ? "/api/virtual_clients/live/start/" + virtual_id + "/?sid=" + sid : "/api/replicas/live/start/" + replicaID +"/?sid=" + sid;
            var payload = {};
            payload.recovery_point = recovery_point;
            $http ({
                method: 'put',
                data: JSON.stringify(payload),
                url : url
            }).success(function(data){
                goLive = false;
                ngDialog.close('goLiveApiConfirmationDialog');
                $scope.resetButtons();
           }).error(function(response) {
                    ngDialog.open({
                        dialogType: 'error',
                        dialogMessage: response.result[0].message
                    })
            });
        };

        $scope.handleReplicaAudit = function(){
            if(!$('#replica_btn_audit').hasClass("disabled")) {
                if (goAudit) {
                    ngDialog.open({
                        dialogType: 'Confirmation',
                        dialogMessage: "Are you sure you want to go into audit mode? ",
                        scope: this,
                        overlay: true,
                        modelDialogId: 'auditApiConfirmationDialog',
                        onConfirmOkButtonClick: 'auditApiCall()'
                    });
                } else {
                    var url = jobType === "wir" ? "/api/virtual_clients/audit/stop/" + virtual_id + "/?sid=" + sid : "/api/replicas/audit/stop/" + replicaID + "/?sid=" + sid;
                    $http ({
                        method: 'put',
                        url : url
                    }).success(function(data){
                        goLive = true;

                        $("#replica_btn_audit").html('<i class="icon-binoculars icon-large"></i> ' + gettext('Audit'));
                        $scope.resetButtons();

                    }).error(function(response) {
                        ngDialog.open({
                            dialogType: 'error',
                            dialogMessage: response.result[0].message
                        })
                    });
                }
            }
        };

        $scope.auditApiCall=function(){
            var url = jobType === "wir" ? "/api/virtual_clients/audit/start/" + virtual_id + "/?sid=" + sid : "/api/replicas/audit/start/" + replicaID +"/?sid=" + sid;
            var payload = {};
            payload.recovery_point = recovery_point;
            $http ({
                method: 'put',
                data: JSON.stringify(payload),
                url : url
            }).success(function(data){
                goAudit = false;
                ngDialog.close('auditApiConfirmationDialog');
                $scope.resetButtons();
                $('#replica_btn_audit').addClass("disabled")
            }).error(function(response) {
                ngDialog.open({
                    dialogType: 'error',
                    dialogMessage: response.result[0].message
                })
            });
        };

        $scope.resetButtons = function(){
            $('#replica_btn_audit').addClass("disabled");
            $('#replica_btn_live').addClass("disabled");
            $('#tearDownReplica').addClass("disabled");
            $('#editReplicaBtn').addClass("disabled");
            $('#tearDownBtnID').addClass("disabled");
            $("#replica_btn_audit").html('<i class="icon-rawaccesslogs icon-large"></i> ' + gettext('Audit'));
            $("#replica_btn_live").html('<i class="icon-binoculars icon-large"></i> ' + gettext('Go Live'));
            $("#replica_btn_live").prop("title", "");
            $("#replica_btn_audit").prop("title", "");

        }

        /* On click of 'Recover Files' button to open Create file-level recovery job dialog */
        $scope.openFileLevelRecoveryJobDialogClick = function() {
            $scope.is_cisco_ucs_restore = false;
            $scope.is_NDMP_restore = false;
             if((selectedBackup.backup_id != undefined) && (selectedBackup.backup_id != null) && (selectedBackup.backup_id != "")){
                 $scope.flr_flag = "Backup";
                 $scope.FLRBackup();
             }else if((selectedBackup.archive_id != undefined) && (selectedBackup.archive_id != null) && (selectedBackup.archive_id != "")){
                $scope.flr_flag = "BackupCopy";
                $scope.FLRBackupCopy();
            }
        };
    
        /* Call function if FLR for backup */
        $scope.FLRBackup=function(){
            if (!$('#restoreFilesBtn').hasClass("disabled")) {
                if ((app_type != undefined) && (app_type === APPLICATION_TYPE_NAME_PHYSICAL_SERVER || app_type === APPLICATION_TYPE_NAME_NDMP_DEVICE)) {
                    if(app_type === APPLICATION_TYPE_NAME_NDMP_DEVICE) {
                        $scope.is_NDMP_restore = true;
                    }
                    ngDialog.open({
                        template: 'app/recover/filerecovery/fileLevelRecoveryJobWizard.html',
                        scope: $scope,
                        overlay: true,
                        modelDialogId: 'create-filelevel-recovery-dialog',
                        ngDialogStyle: 'width:120em; height:70em;  top: -150px;left: 80px;'
                    });
                } else {
                    ngDialog.open({
                        dialogType: gettextCatalog.getString('Confirmation'),
                        dialogMessage: gettextCatalog.getString("Do you want to create the file recovery object?"),
                        scope: this,
                        overlay: true,
                        modelDialogId: 'fileRecoveryPostApiConfirmationDialog',
                        onConfirmOkButtonClick: 'fileRecoveryPostApiCall()'
                    });
                }
            }
        };
    
        $scope.FLRBackupCopy=function(){
            if (!$('#restoreFilesBtn').hasClass("disabled")) {
                 ngDialog.open({
                    template: 'app/recover/filerecovery/fileLevelRecoveryJobWizard.html',
                    scope: $scope,
                    overlay: true,
                    modelDialogId: 'create-filelevel-recovery-dialog-backupcopy',
                    ngDialogStyle: 'width:120em; height:70em;  top: -150px;left: 80px;'
                });

                /*if ((app_type != undefined) && (app_type === APPLICATION_TYPE_NAME_PHYSICAL_SERVER)) {
                    ngDialog.open({
                        template: 'app/recover/filerecovery/fileLevelRecoveryJobWizard.html',
                        scope: $scope,
                        overlay: true,
                        modelDialogId: 'create-filelevel-recovery-dialog-backupcopy',
                        ngDialogStyle: 'width:120em; height:70em;  top: -150px;left: 80px;'
                    });
                } else {
                    ngDialog.open({
                        dialogType: 'Confirmation',
                        dialogMessage: "Do you want to create the file recovery object?",
                        scope: this,
                        overlay: true,
                        modelDialogId: 'fileRecoveryPostApiConfirmationDialog-backupcopy',
                        onConfirmOkButtonClick: 'fileRecoveryPostApiCall()'
                    });
                }*/
            }
        };
    
    
        $scope.fileRecoveryPostApiCall=function(){
            var loadMessage = gettextCatalog.getString("Starting File Level Recovery" ) + ((selectedBackup.remote === 'true') ? " from the Target" : "") + "...";
            var load = PHD.showLoadingIndicator("body", true, loadMessage);
            var restoreObj= {};
            var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + ((selectedBackup.replicated == 'true') ? selectedBackup.local_system_id : selectedBackup.sid);
            // Handle remote replicated restores
            if (selectedBackup.remote === 'true') {
                restoreObj.sourceID = selectedBackup.sid;
                restoreObj.remote = true;
            }
            var url = "/api/restore/files/" + sid;

            restoreObj.backup_id = selectedBackup.backup_id;
            restoreObj = JSON.stringify(restoreObj);

            $http ({
                method: 'post',
                url : url,
                data: restoreObj
            }).success(function(data){
                PHD.hideLoadingIndicator(load);
                data = {};
                data.result = [{'message': gettextCatalog.getString('File level recovery job successfully started.')}];
                var options = {};

                //mmb - it is no longer an option to create a flr from protect or jobs. Removed references to protect page and jobs page
                options.buttons = [
                        {
                            text: gettextCatalog.getString("View FLR"),
                            click: function () {
                                window.location.href = "#/recover/#1";
                                $(this).dialog("destroy").remove();
                            }
                        }
                ];

                PHD.throwNotice(data, options);
                ngDialog.close('fileRecoveryPostApiConfirmationDialog');
                
            }).error(function(response) {
                PHD.hideLoadingIndicator(load);
                if ((response.result[0].message.indexOf("Maximum number of allowable recovery jobs have been reached.") !== -1) ||
                    (response.result[0].message.indexOf("Maximum number of recovery jobs for the appliance") !== -1)) {
                    // in case of max number of FLRs on the target or max number per source have been reached, present an error dialog
                    ngDialog.open({
                        dialogType: 'error',
                        dialogMessage: response.result[0].message
                    })
                } else {
                    ngDialog.open({
                        dialogType: 'retry',
                        dialogMessage: response.result[0].message
                    })
                }
            });
        };

        /*********************************Delete Replica row **************************************/

        $scope.openReplicaDeleteDialog = function () {
            if (!$('tearDownReplica').hasClass("disabled")) {
                if (jobType === "replica") {
                    ngDialog.open({
                        template: 'app/recover/ir/ir-delete-dialog/vmware-replica-delete-dialog.html',
                        scope: $scope,
                        overlay: true,
                        modelDialogId: 'deleteReplicaRow',
                        showHelpArticleButton: true,
                        ngDialogStyle: 'width:50em; height:34em;  top: -90px;left: 27px;right: 27px;'
                    });
                } else if (jobType === 'wir') {
                    if (hypervisorType === "Unitrends appliance" || wirMode === "halted") {
                        ngDialog.open({
                            template: 'app/recover/ir/ir-delete-dialog/wir-delete-haltedmode-dialog.html',
                            scope: $scope,
                            overlay: true,
                            modelDialogId: 'deleteIRConfirmation',
                            ngDialogStyle: 'width:45em; height:18em;  top: -90px;left: 27px;right: 27px;'
                        });
                    } else {
                        $scope.wirDeleteRadioSelection = "2";
                        ngDialog.open({
                            template: 'app/recover/ir/ir-delete-dialog/wir-delete-dialog.html',
                            scope: $scope,
                            overlay: true,
                            modelDialogId: 'deleteReplicaRow',
                            showHelpArticleButton: true,
                            ngDialogStyle: 'width:50em; height:34em;  top: -90px;left: 27px;right: 27px;'
                        });
                    }
                }
            }
        };

		/*********************************Delete WIR row **************************************/
        $scope.openIRDeleteDialog = function () {
            if (!$('#tearDownBtnId').hasClass("disabled")) {
                if (jobType !== "wir" && jobType !== "replica") {
                    ngDialog.open({
                        template: 'app/recover/ir/ir-delete-dialog/ir-delete-dialog.html',
                        scope: $scope,
                        overlay: true,
                        modelDialogId: 'deleteExistinIRConfirmation',
                        ngDialogStyle: 'width:44em; height:20em;  top: -90px;left: 27px;right: 27px;'
                    });
                }
            }

        };
 
		$scope.onConfirmWIRDelete = function(deleteFromHypervisorFlag) {
			
		    var irdeleteRadioVal =  $("input[name='irDeleteRadioBtn']:checked").val();
         	var url = "/api/virtual_clients/"+virtual_id+"/";
			var params = {};
			var deleteObj={};
            if(jobType != "wir"){ 
                     params.sid = sid;
				     deleteObj.type= jobType;
     	    }else{
                 if(hypervisorType == HYPERVISOR_UNITRENDS_APPLIANCE || wirMode == MODE_HALTED ){
                     params.sid =sid;
                     deleteObj.type= jobType;
                     deleteObj.deleteFromHypervisor= deleteFromHypervisorFlag;  
                 }else{
                     if(irdeleteRadioVal == 1){
                       deleteObj.deleteFromHypervisor= 1;    
                         
                     }else{
                       deleteObj.deleteFromHypervisor= 0;
                     }
                     params.sid =sid;
                     deleteObj.type= jobType;
                     
                 }
            }
            ngDialog.close('deleteIRConfirmation');
            ngDialog.close('deleteIRRow');
			
            $http({
                method:'delete',
                url:url,
				data:JSON.stringify(deleteObj),
                params:params
            }).success(function(data, status, headers){
                $timeout(function() {
                    
                    $rootScope.RecoverTimer.stop();
                    $(document).off("timertick.recoverpage");
                    delete $(document);

                    var eventsTab = angular.element(document.getElementById("recover-ir"));
                    var injector = eventsTab.injector();
                    var route = injector.get('$route');
                    route.reload();
                }, 500);

		    }).error(function(response) {
                 ngDialog.open({
                 dialogType:'retry',
                 modelDialogId:'replication-dailog',
                 dialogMessage:response.result[0].message
                })
            });

		};

        $scope.onConfirmVMwareReplicaDelete = function(deleteFromHypervisorFlag) {

            var irdeleteRadioVal =  $("input[name='replicaDeleteRadioBtn']:checked").val();
            var url = "/api/replicas/"+replicaID+"/?sid=" + sid;
            var deleteObj={};
                deleteObj.type= jobType;
                  if(irdeleteRadioVal == 1){
                        deleteObj.deleteFromHypervisor= 1;

                    }else{
                        deleteObj.deleteFromHypervisor= 0;
                    }
                    deleteObj.type= jobType;

            ngDialog.close('deleteVMwareReplicaConfirmation');
            ngDialog.close('deleteIRRow');

            $http({
                method:'delete',
                url:url,
                data:JSON.stringify(deleteObj),
            }).success(function(data, status, headers){
                $timeout(function() {
                    $rootScope.RecoverTimer.stop();
                    $(document).off("timertick.recoverpage");
                    delete $(document);

                    var eventsTab = angular.element(document.getElementById("recover-ir"));
                    var injector = eventsTab.injector();
                    var route = injector.get('$route');
                    route.reload();
                }, 500);

            }).error(function(response) {
                ngDialog.open({
                    dialogType:'retry',
                    modelDialogId:'replication-dailog',
                    dialogMessage:response.result[0].message
                })
            });

        };

        $scope.onChangeCatalogFilter = function() {
            $rootScope.showCatalogFilter = !$rootScope.showCatalogFilter;
            $scope.setCatalogView();
        };

        $scope.setCatalogView = function() {
            var treebleWidth = ($rootScope.showCatalogFilter) ? '75%' : '100%';
            $("#recover-backup-treeble").width(treebleWidth);
            $("#recover-backup-table").scrollTable({"height": "article"});
        };

        $scope.openDRTestDialog = function() {
            //dr test dailog goes here
            $scope.drCheck = false;

            if (!$('#drTestBtn').hasClass("disabled")) {
                console.debug("clicked test");
                $analytics && $analytics.eventTrack('DR Test', {category: 'Recover', label: 'DR Test'});
                var url = "/api/drtest";

                $http({
                    method: 'get',
                    url: url
                }).success(function (data) {
                    // if (data && data.data === true) {
                    if (data) {
                        $scope.drCheck = data.data;
                        $scope.input = {};
                        ngDialog.open({
                            template: 'app/recover/drtest/drtest.html',
                            overlay: true,
                            scope: $scope,
                            modelDialogId: 'drTestDialog',
                            ngDialogStyle: 'width:600px;height:auto;padding-bottom:50px',
                            closeByEscape: true
                        });
                    } else {
                        ngDialog.open({
                            dialogType: 'notice',
                            dialogMessage: 'You are not replicating to the unitrends cloud.'
                        })
                    }

                }).error(function (response) {
                    ngDialog.open({
                        dialogType: 'retry',
                        dialogMessage: response.result[0].message
                    })
                });
            }
        };

           $scope.sendDRTestForm = function(){
            console.debug('in send request function');
            $scope.drForm = angular.element("#drTestDialogForm");

            var params = {};
               params.name = $scope.input.username;
               params.company = $scope.input.company;
               params.email = $scope.input.email;
               params.phone = $scope.input.phone;



               if ($scope.validateDRTestForm($scope.drForm)) {
                   var sendDRTestRequest = PHD.showLoadingIndicator('body', true, "Sending Test DR request to CloudOps.....");
                   var url = "/api/drtest/send_drtest_request";
                   $http({
                       method: 'post',
                       url: url,
                       data: JSON.stringify(params)
                   }).success(function (data, status, headers) {
                       PHD.hideLoadingIndicator(sendDRTestRequest);
                       $analytics && $analytics.eventTrack('Send DR Test Form', {
                           category: 'Recover',
                           label: 'Test DR'
                       });
                       ngDialog.open({
                           dialogType: 'notice',
                           dialogMessage: 'Test Disaster Recovery request successfully sent to Unitrends CloudOps.'
                       })
                   }).error(function (response) {
                       PHD.hideLoadingIndicator(sendDRTestRequest);
                       var message = response.result[0].message;
                       $analytics && $analytics.eventTrack('Failure: ' + message, {
                           category: 'Recover',
                           label: 'Test DR'
                       });
                       ngDialog.open({
                           dialogType: 'error',
                           dialogMessage: 'Unable to send request.  Is mail configured?'
                       })
                   });
               }
        };

        $scope.isValidPhoneNumber = function(str){
            var PHONE_REGEXP = /^\(?([0-9]{3})\)?[-. ]?([0-9]{3})[-. ]?([0-9]{4})$/i;
            return PHONE_REGEXP.test(str);
        };

        $scope.validateDRTestForm = function(form) {
            var invalidPhone = false;
            var invalidPhoneText = "Enter a valid phone number";
            console.debug("validating dr form");
            if(!$scope.isValidPhoneNumber ($scope.input.phone)){
                console.log("invalid phone #");//Display error
                invalidPhone = true;
            }

            $scope.drForm.validate({
                errorLabelContainer: ".datetime-dialog-error-list",
                errorContainer: ".datetime-dialog-errors",
                showErrors: function (errorMap, errorList) {
                    $(".datetime-dialog-errors").find(".summary")
                        .html(gettextCatalog.getString("Correct the following errors:"));
                    this.defaultShowErrors();
                },
                rules: {
                    username: {
                        required: true
                    },
                    company: {
                        required: true
                    },
                    email: {
                        required: true,
                        email: true
                    },
                    phone: {
                        required: true,
                        digits: true

                    }
                },
                messages: {
                    username: {
                        required: gettext("Name is required")
                    },
                    company: {
                        required: gettext("Company is required")
                    },
                    email: {
                        required: gettext("Email is required")
                    },
                    phone: {
                        required: gettext("Phone is required"),
                        digits: gettext(invalidPhone ? invalidPhoneText : "")

                    }
                }
            });
            if ((!$('#drTestDialogForm').valid())) {
                return false;
            } else {
                return true;
            }
        };

        $scope.$on("languageChanged", function (event){
            // On language change, reset the backup catalog date filters to the default date range.
            if ($scope.recoveryBackupFilter !== undefined) {
                $scope.recoveryBackupFilter.resetDateFilter();
            }
        });

        $scope.catalogFilter = {};
        $scope.saveCatalogFilter = function() {
            // On filter save call the catalog filter to finish the save.
            if ($scope.recoveryBackupFilter !== undefined) {
                $scope.recoveryBackupFilter.finishFilterSave();
            }
        };

        $scope.deleteCatalogFilter = function() {
            // On filter deletion call the catalog filter to finish the deletion.
            if ($scope.recoveryBackupFilter !== undefined) {
                $scope.recoveryBackupFilter.finishFilterDelete();
            }
        }

        /*
         * We only need this if we ever support recovery page preferences.
         */
        /*$scope.showHideRecoverPreferences = function() {

            if ($scope.preferencesDialog == null) {
                var $preferenceButton = $("#recover-page-preferences");
                var options = {
                    autoOpen: true,
                    modal: false,
                    show: { effect: "slide", direction: "up" },
                    hide: { effect: "scale", percent: 0, origin: ["top", "right"] },
                    dialogClass: "user-preference-dialog",
                    resizable: false,
                    draggable: false,
                    width: 262,
                    title: "Recover Page Preferences",
                    helpArticle: PHD.App.getHelpLink("recover", true),
                    buttons: {}
                };

                options.position = {
                    my: "right top",
                    at: "right top",
                    of: $preferenceButton
                };
                options = $.extend({}, options);

                var dialog = $("<div/>", {id: "dialog-edit-setting", "class": "dialog-box"});
                dialog.data("edit", false);

                PHD.Ajax
                    .getHTML('app/recover/preferences.html')
                    .done(function (data) {
                        dialog.html(data);
                        dialog.insertAfter($preferenceButton).dialog(options);
                        var elements = angular.element(document.getElementById("recoveryPreferencesDialog"));
                        $compile(elements.contents())($scope);
                        $scope.$digest();
                        $scope.preferencesDialog = dialog;
                    });
            }
        };

        $scope.closePreferencesDialog = function() {
            if ($scope.preferencesDialog != null) {
                $scope.preferencesDialog.dialog("destroy").html("").detach();
                $scope.preferencesDialog = null;
            }
        };

        $scope.savePreferences = function() {
            $scope.showCatalogFilter = !$scope.collapseCatalogFilter;
            var treebleWidth = ($scope.showCatalogFilter) ? '75%' : '100%';
            $("#recover-backup-treeble").width(treebleWidth);
            $("#recover-backup-table").scrollTable({"height": "article"});
            $scope.closePreferencesDialog();
        };
        */

	}]
);

