angular.module('backup-details-module', ['ngDialog', 'ui.grid',
                                          'ui.grid.resizeColumns','ui.grid.autoResize', 'ui.grid.exporter', 'ui.grid.selection'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('backupDetailsCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants', 'uiGridExporterConstants', 'gettextCatalog',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants, uiGridExporterConstants, gettextCatalog) {

            $scope.item = $scope.ngDialogData.item;
            $scope.system_id = $scope.ngDialogData.applianceId;
            $scope.showCopies = $scope.ngDialogData.showCopies;
            $scope.showArchives = $scope.ngDialogData.showArchives;
            $scope.showRawOutput = !$scope.showArchives;
            $scope.title = $scope.ngDialogData.title + gettextCatalog.getString(' Report Entry');
            $scope.skip = 0;
            $scope.showingFiles = false;
            $scope.totalFiles = $scope.item.files;
            $scope.PAGE = 5000;                 // maximum number of items that can be "PAGED" in at once.
            $scope.PAGE_FLOAT = 5000.0;         // needed for rounding filecount to nearest "PAGE".

            $scope.closeDialog = function() {
                ngDialog.close('item-details-dialog');
            };

            $scope.reportGridData = {
                multiSelect: false,
                rowHeight: 30,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                data: []
            };

            $scope.reportGridData.columnDefs = [
                {
                    field: "category",
                    headerCellFilter: "translate",
                    displayName: gettextCatalog.getString("Category"),
                    sortable : true,
                    sortingType : "string",
                    sort: {
                        direction: uiGridConstants.ASC,
                        priority: 0
                    }
                },
                {
                    field: "entry",
                    headerCellFilter: "translate",
                    displayName: gettextCatalog.getString("Entry"),
                    sortable : true,
                    sortingType : "string"
                }
            ];

            $scope.loadDetails = function() {
                $scope.gridData = [];
                // Create a grid with each of the item's properties and values.
                for (var prop in $scope.item) {
                    if ($scope.item.hasOwnProperty(prop)) {
                        if (prop.indexOf('$$') == -1) {
                            // Skip any '$$' property as these are auto-generated by the UI-grid.
                            var obj = { 'category' : prop, 'entry': $scope.item[prop]};
                            $scope.gridData.push(obj);
                        }
                    }
                }

                var url;
                var resp;
                if ($scope.showArchives) {
                    url = "/api/archive/status/?sid=" + $scope.system_id;
                    var idJSON = {};
                    idJSON.archive_ids = [ $scope.item.archive_id ];
                    idJSON = JSON.stringify(idJSON);
                    resp = PHD.Ajax.post(url, idJSON);
                } else {
                    url = "/api/backups/?sid=" + $scope.system_id + "&bid=" + $scope.item.id;
                    if ($scope.showCopies) {
                        url += "&copies=1";
                    }
                    resp = PHD.Ajax.get(url);
                }
                resp.done(function(data) {
                    var item = null;
                    if (data.BackupStatus !== undefined && data.BackupStatus.length > 0) {
                        item = data.BackupStatus[0];
                    } else if (data.status !== undefined && data.status.length > 0) {
                        item = data.status[0];
                    }
                    for (var prop in item) {
                        if ($scope.item.hasOwnProperty(prop)) {
                            // If already present, skip it.
                            continue;
                        }
                        if (item.hasOwnProperty(prop)) {
                            if (prop !== 'output' && prop.indexOf('$$') == -1) {
                                // Skip any '$$' property as these are auto-generated by the UI-grid.
                                // Skip output, will be handled in a special way, below.
                                var obj = { 'category' : prop, 'entry':item[prop]};
                                $scope.gridData.push(obj);
                            }
                        }
                    }
                    if (item.output !== undefined) {
                        var output = item.output;
                        // Process to remove any extra HTML before the Backup Copy Message.
                        $scope.item.output = output.replace("<br/>", "\n");
                    }

                    $timeout(function() {
                        $scope.reportGridData.data = $scope.gridData;
                    });
                });
            };

            // use instead of ngDialog load event.
            angular.element(document).ready(function () {
                $scope.loadDetails();
            });

            $scope.fileListData = {
                multiSelect: false,
                rowHeight: 30,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                onRegisterApi: function(gridApi) {
                    $scope.fileListGridApi = gridApi;
                },
                data: []
            };

            $scope.fileListData.columnDefs = [
                {
                    field: "size",
                    headerCellFilter: "translate",
                    displayName: gettextCatalog.getString("Size(KB)"),
                    sortable : true,
                    sortingType : "number",
                    width: 80
                },
                {
                    field: "name",
                    headerCellFilter: "translate",
                    displayName: gettextCatalog.getString("File Name"),
                    sortable : true,
                    cellTooltip: true,
                    sortingType : "string",
                    width:"*"
                }
            ];

            $scope.viewBackupDetails = function() {
                if ($scope.showArchives) {
                    ngDialog.open({
                        modelDialogId: 'file-details-error-dialog',
                        dialogType: 'Error',
                        dialogMessage: gettextCatalog.getString('To view files in a backup copy, go to the Recover page, select the backup copy you want to browse, then click Recover Files.'),
                        scope: $scope,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                } else if ($scope.totalFiles === undefined || $scope.totalFiles === 0) {
                    ngDialog.open({
                        modelDialogId: 'file-details-error-dialog',
                        dialogType: 'Error',
                        dialogMessage: gettextCatalog.getString('Details can only be viewed on backups with a file list.'),
                        scope: $scope,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                } else {
                    $scope.showMore = false;
                    $scope.refresh = true;
                    $scope.showCount = ($scope.totalFiles - $scope.PAGE > $scope.skip) ? $scope.skip + $scope.PAGE : $scope.totalFiles;
                    $scope.rangeText = ($scope.skip + 1) + " to " + $scope.showCount;
                    $scope.footerDetail = gettextCatalog.getString("Showing Files " + $scope.rangeText);
                    var url = "/api/backups/file-list/?sid=" + $scope.system_id + "&bid=" + $scope.item.id + "&skip=" + $scope.skip;
                    PHD.Ajax
                        .get(url)
                        .done(function(data) {
                            var filesArray = data.files;
                            if (filesArray !== undefined && filesArray.length > 0) {
                                var last = filesArray[filesArray.length - 1];
                                if (last.more !== undefined && last.more == 1) {
                                    filesArray.pop();
                                    $scope.showMore = true;
                                }
                                if (!$scope.showingFiles) {
                                    ngDialog.open({
                                        template: 'app/reports/details/backup-files.html',
                                        scope: $scope,
                                        overlay: true,
                                        modelDialogId: 'file-details-dialog',
                                        ngDialogStyle: 'width:800px;height:600px',
                                        preCloseCallback: function (value) {
                                            $scope.showingFiles = false;
                                        }
                                    });
                                    $scope.showingFiles = true;
                                }

                                $timeout(function () {
                                    $scope.fileListData.data = filesArray;
                                    $scope.refresh = false;
                                });
                            } else if (filesArray !== undefined && filesArray.length === 0) {
                                ngDialog.open({
                                    modelDialogId: 'file-details-end-dialog',
                                    dialogType: 'Information',
                                    dialogMessage: gettextCatalog.getString('At end but no files seen.  Click Okay to retry.'),
                                    scope: $scope,
                                    onConfirmOkButtonClick: 'onConfirmEndDialog()'
                                });
                            } else {
                                ngDialog.open({
                                    modelDialogId: 'file-details-error-dialog',
                                    dialogType: 'Error',
                                    dialogMessage: gettextCatalog.getString('Error getting file list.'),
                                    scope: $scope,
                                    onConfirmOkButtonClick: 'onConfirmError()'
                                });
                            }
                        });
                }
            };

            $scope.viewFirst = function(){
                $scope.skip = 0;
                $scope.viewBackupDetails();
            };

            $scope.viewNext = function(){
                $scope.skip += $scope.PAGE;
                $scope.viewBackupDetails();
            };

            $scope.viewPrevious = function(){
                $scope.skip -= $scope.PAGE;
                $scope.viewBackupDetails();
            };

            $scope.viewLast = function(){
                if ($scope.totalFiles > $scope.PAGE) {
                    $scope.skip = Math.floor($scope.totalFiles / $scope.PAGE_FLOAT) * $scope.PAGE;
                }
                $scope.viewBackupDetails();
            };

            $scope.exportFiles = function() {
                /*$scope.fileListData.exporterCsvFilename = 'BackupFiles: ' + $scope.rangeText + '.csv';
                var exporterApi =  $scope.fileListGridApi.exporter;
                if (exporterApi !== undefined) {
                    exporterApi.csvExport(uiGridExporterConstants.ALL, uiGridExporterConstants.VISIBLE);
                } */
                ngDialog.open({
                    template: 'app/reports/details/export-files.html',
                    scope: $scope,
                    data: {'item': $scope.item, 'system_id': $scope.system_id },
                    overlay:true,
                    modelDialogId: 'export-files-dialog',
                    name: 'export-files-dialog',
                    ngDialogStyle: 'width:500px;height:250px',
                    preCloseCallback: function (value) {

                    }
                });
            };

            $scope.closeFilesDialog = function() {
                ngDialog.close("file-details-dialog");
                $scope.skip = 0;
            };

            $scope.onConfirmError = function() {
                ngDialog.close("file-details-error-dialog");
            };

            $scope.onConfirmEndDialog = function() {
                ngDialog.close("file-details-end-dialog");
                $scope.viewPrevious();
            };

            /*
             *  If a Cold backup copy, there is no Raw output available.  So, hide that area and
             *  increase the height of the cateogry grid.
             */
            $scope.getGridStyle = function() {
                var style = "overflow:hidden;margin-bottom:0;font-size:12px;";
                style += $scope.showArchives ? "height:510px": "height:280px";
                return style;
            };

        }])
    .controller('exportFileListCtrl', ['$scope', '$rootScope', '$http','ngDialog', 'DownloadService',
        function($scope, $rootScope, $http, ngDialog, DownloadService) {

            var MAX_FILES = 100000;
            var maxOffset = 0;

            $scope.item = $scope.ngDialogData.item;
            $scope.system_id = $scope.ngDialogData.system_id;

            $scope.fileCount = $scope.item.files;
            $scope.fileOffset = 0;
            $scope.showPartial = $scope.fileCount > MAX_FILES;
            if ($scope.showPartial) {
                maxOffset = (parseInt($scope.fileCount) / MAX_FILES) * MAX_FILES;
            }

            // Used to unbind the watched event on dialog close.
            var unbindHandler = null;

            $scope.initCtrl = function(dlg){
                switch (dlg.name){
                    case 'export-files-dialog':
                        var offsetId="#spinner_offset";
                        $(offsetId).spinner({"min": 0, "max": maxOffset, "step": MAX_FILES});
                        break;
                    default:
                        // No action.
                        break;
                }
            };

            $scope.exportFileList = function() {
                var url = "/api/backups/file-list/?sid=" + $scope.system_id + "&bid=" + $scope.item.id + "&skip=" + $scope.fileOffset + "&max=" + MAX_FILES + "&format=csv";
                var load = PHD.showLoadingIndicator("body", true, 'Retrieving file list.  Please wait....');
                $http({
                    method: 'get',
                    url : url
                }).success(function(data, status, headers){
                    PHD.hideLoadingIndicator(load);
                    // This should be encapsulated in a reusable function so that other areas (outside reports) can export to CSV or PDF.
                    headers = headers();
                    var contentType = headers['content-type'];
                    if (contentType.indexOf('text/csv') > -1) {
                        var filename = 'BackupFiles.csv';
                        var blob = new Blob([data], {type: contentType});
                        DownloadService.downloadFile(blob, filename);
                    }
                    ngDialog.close('export-files-dialog');
                }).error(function(response) {
                    PHD.hideLoadingIndicator(load);
                    ngDialog.open({
                        dialogType:'Retry',
                        dialogMessage: gettextCatalog.getString('Error exporting file list')
                    });
                });
            };

            $scope.closeDialog = function() {
                if ( _.isFunction(unbindHandler)) {
                    unbindHandler();
                }
                ngDialog.close('export-files-dialog');
            };

            // Initialize the controller.
            unbindHandler = $scope.$on('ngDialog.opened', function(event,obj) { $scope.initCtrl(obj) });

        }]);