
var distributedReports = angular.module('distributed-reports-module', []);

distributedReports.service("distributedReportsSvc", function() {

    /*
     * Creates a line graph of protected capacity over time for one or more appliances.
     */
    this.plotDistributedCapacity = function(resultData, start_date_val, end_date_val) {

        var data = resultData['data'];

        data.forEach(function (d) {
            d.Date = new Date(d.date * 1000);
        });

        var dataNest = d3.nest()
            .key(function (d) {
                return d.device_id;
            })
            .entries(data);

        var margin = {top: 30, right: 100, bottom: 60, left: 60},
            width = 1000,
            height = 600;

        var x = d3.scaleTime()
            .range([0, width - margin.left - margin.right]);

        var y = d3.scaleLinear()
            .range([height - margin.top - margin.bottom, 0]);

        var xAxis = d3.axisBottom()
            .scale(x)
            .ticks(d3.timeDay, 1)
            .tickFormat(d3.timeFormat('%m/%d/%y'))
            .tickPadding(8);

        var yAxis = d3.axisLeft()
            .scale(y)
            .tickPadding(8);

        var svg = d3.select('#flot_holder').append('svg')
            .attr('class', 'chart')
            .attr('width', width)
            .attr('height', height)
            .append('g')
            .attr('transform', 'translate(' + margin.left + ', ' + margin.top + ')');

        var valueline = d3.line()
            .defined(function(d) { return d; })
            .x(function (d) {
                return x(d.Date);
            })
            .y(function (d) {
                return y(d.gb_used);
            });

        x.domain(d3.extent(data, function (d) {
            return d.Date;
        }));
        y.domain([0, d3.max(data, function (d) {
            return d.gb_used;
        })]);

        // Create x-axis, with date labels, rotated
        svg.append('g')
            .attr('class', 'x d3-axis')
            .attr('transform', 'translate(0, ' + (height - margin.top - margin.bottom) + ')')
            .call(xAxis)
            .selectAll("text")
            .style("text-anchor", "end")
            .attr("dx", "-.8em")
            .attr("dy", ".15em")
            .attr("transform", function (d) {
                return "rotate(-65)"
            });

        // Create y-axis
        svg.append('g')
            .attr('class', 'y d3-axis')
            .call(yAxis);

        // Add y-axis title near the left margin
        svg.append("text")
            .attr("text-anchor", "middle")
            // text is drawn off the screen top left, move down and out and rotate
            .attr("transform", "translate(-" + (margin.left - 16) + "," + ((height - margin.top - margin.bottom) / 2) + ")rotate(-90)")
            .text("Size (GB)");

        // Color map for line color.
        var colors = ['#4286f4', '#3bbf0f', '#f45f42', '#371a66', '#701036', '#0a114c', '#bfa10f'];
        var colorOf = function (id) {
            var color = 'black';
            for (var i = 0; i < dataNest.length; i++) {
                var key = dataNest[i].key;
                if (key == id) {
                    color = colors[i % colors.length];
                    break;
                }
            }
            return color;
        };

        dataNest.forEach(function (d, i) {
            // Create each line
            svg.append("path")
                .attr('class', 'line')
                .attr('d', valueline(d.values))
                .attr('stroke-width', 2)
                .style("stroke", colorOf(d.key))
                .attr('fill', 'none');

            // Add a line label at the end.
            svg.append("text")
                .attr("transform", "translate(" + x(d.values[0].Date) + "," + y(d.values[0].gb_used) + ")")
                .attr("x", 3)
                .attr("dy", "0.35em")
                .style("font", "10px sans-serif")
                .text(d.values[0].name);
        });
    }
});