angular.module('report-details-module', ['ngDialog', 'ui.grid',
                                          'ui.grid.resizeColumns','ui.grid.autoResize', 'ui.grid.exporter', 'ui.grid.selection'])
.config(['ngDialogProvider', function (ngDialogProvider) {
    ngDialogProvider.setDefaults({
        plain: false,
        showClose: true,
        closeByDocument: true,
        closeByEscape: true,
        appendTo: false,
        preCloseCallback: function (value) {

        }
    });
}])
.controller('reportDetailsCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants', 'uiGridExporterConstants',
    function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants, uiGridExporterConstants) {

        $scope.title = $scope.ngDialogData.title + ' Report Entry';
        $scope.showLargeField = false;
        $scope.showIncludes = false;
        $scope.showExcludes = false;
        $scope.includesCount = $scope.excludesCount = 0;
        $scope.largeFieldName = $scope.ngDialogData.specialField;

        var details = $scope.ngDialogData.details;
        var row = $scope.ngDialogData.row;

        for (var prop in row) {
            if (row.hasOwnProperty(prop)) {
                if (prop != 'details') {
                    details[prop] = row[prop];
                }
            }
        }
        $scope.item = details;

        $scope.closeDialog = function() {
            ngDialog.close('item-details-dialog');
        };

        $scope.reportGridData = {
            multiSelect: false,
            rowHeight: 30,
            gridMenuShowHideColumns:false,
            enableColumnMenus:false,
            enableHorizontalScrollbar: 0,
            data: []
        };

        $scope.reportGridData.columnDefs = [
            {
                field: "category",
                displayName: "Category",
                sortable : true,
                sortingType : "string",
                sort: {
                    direction: uiGridConstants.ASC,
                    priority: 0
                },
                cellClass: 'capitalize'
            },
            {
                field: "entry",
                displayName: "Entry",
                sortable : true,
                sortingType : "string",
                cellTooltip: true
            }
        ];

        function prettyPrint(prop) {
            var prettyProp;
            if (prop == 'calendar_str') {
                prettyProp = "Backup Job Frequency";
            } else {
                var a = prop.split('_');
                prettyProp = a.join(' ');
            }
            return prettyProp;
        }

        $scope.loadDetails = function() {
            $scope.gridData = [];
            // Create a grid with each of the item's properties and values.
            var field, obj;
            for (var prop in $scope.item) {
                if ($scope.item.hasOwnProperty(prop)) {
                    if (prop.indexOf('$$') == -1) {
                        // Skip any '$$' property as these are auto-generated by the UI-grid.
                        if (prop.indexOf($scope.largeFieldName) != -1) {
                            $scope.largeFieldName = prettyPrint(prop);
                            field = $scope.item[prop];
                            field = field.join('\n');
                            $scope.largeField = field;
                            $scope.showLargeField = true;
                        } else {
                            var entry = $scope.item[prop];
                            if (entry instanceof Array) {
                                var entryLength = entry.length;
                                if (entry.length > 0) {
                                    if (prop.indexOf('includes') != -1) {
                                        entry = entry.join('\n');
                                        $scope.includesCount = entryLength;
                                        $scope.includes = entry;
                                        $scope.showIncludes = true;
                                    } else if (prop.indexOf('excludes') != -1) {
                                        entry = entry.join('\n');
                                        $scope.excludesCount = entryLength;
                                        $scope.excludes = entry;
                                        $scope.showExcludes = true;
                                    } else {
                                        entry = entry.toString();
                                    }
                                } else {
                                    entry = "None";
                                    obj = { 'category' : prettyPrint(prop), 'entry': entry};
                                    $scope.gridData.push(obj);
                                }
                            } else {
                                obj = { 'category' : prettyPrint(prop), 'entry': entry};
                                $scope.gridData.push(obj);
                            }
                        }
                    }
                }
            }

            $timeout(function() {
                $scope.reportGridData.data = $scope.gridData;
            });
        };

        /*
         *  Vary the main window size depending on what is displayed.
         */
        $scope.getGridStyle = function() {
            var style = "overflow:hidden;margin-bottom:8px;";
            var gridHeight = 350;
            if ($scope.showIncludes) {
                gridHeight -= 90;
            }
            if ($scope.showExcludes) {
                gridHeight -= 90;
            }
            style += "height:" + gridHeight + "px";
            return style;
        };

        // use instead of ngDialog load event.
        angular.element(document).ready(function () {
            $scope.loadDetails();
        });


    }]);