(function($) {

    $.fn.filterTable = function() {
        return this.each(function() {
            var $table = $(this).addClass("filter-table"),
                $thead = $table.find("thead"),
                $tbody = $table.find("tbody"),
                $tfoot = $(this).find("tfoot"),
                $columns = $thead.children("tr:last-child").children("th"),
                numcols = $columns.length,
                widths = $thead.find("tr:last-child th").map(function(){return $(this).width();}).get(),
                totalRows = $tbody.children("tr").length;

            $tfoot.find(".filter").each(function(index){$(this).data("fid", "filtered-" + index);});

            $tfoot.find("input.filter").on("keyup", function() {
                var $el = $(this),
                    fid = $el.data("fid"),
                    $cell = $el.closest("th"),
                    val = $el.val().toLowerCase(),
                    idx = $cell.index();

                $table.trigger("beforecolumnfilter");

                if(val === "") {
                    $tbody.find("tr").removeClass(fid);
                } else {
                    $tbody.find("tr").each(function() {
                        var $cell = $(this).children("td").eq(idx),
                            search = $cell.text().toLowerCase();

                        if(search.indexOf(val) === -1) {
                            $(this).addClass(fid);
                        } else {
                            $(this).removeClass(fid);
                        }
                    });

                    if($table.data("scrollTable")) {
                        $table.lockRows($table.data("widths"));
                    }
                }

                $table.trigger("aftercolumnfilter");

            });

            $tfoot.find("select.filter").on("change", function() {
                var $el = $(this),
                    fid = $el.data("fid"),
                    $cell = $(this).closest("th"),
                    $sel = $(this).find(":selected"),
                    isRegex = $el.hasClass("filter-rx"),
                    isCalc = $el.hasClass("filter-calc");

                var val = isRegex || isCalc ? $sel.val() : $sel.val().toLowerCase();
                var idx = $cell.index();
                var bool = $sel.is(".option-check");

                $table.trigger("beforecolumnfilter");
                if(bool) val = Boolean(parseInt(val, 10));

                if(val === "all") {
                    $tbody.find("tr").removeClass(fid);
                } else {
                    $tbody.find("tr").each(function() {
                        var $cell = $(this).children("td").eq(idx),
                            $items = $cell.find(".item"),
                            search;

                        if(bool) {
                            $(this).toggleClass(fid, !($cell.find(":checkbox").prop("checked") === val));
                            return;
                        }

                        if($items.length) {
                            search = $items.map(function(idx,v){return $(v).text();}).get().join("").toLowerCase();
                        } else {
                            search = $cell.text().toLowerCase();
                        }

                        if(isRegex) {
                            val = new RegExp(val);
                            $(this).toggleClass(fid, !val.test(search));
                            return;
                        }
                        
                        if((search.indexOf(val) === -1 && $items.length) || (search.indexOf(val) !== 0 && !$items.length)) {
                            $(this).addClass(fid);
                        } else {
                            $(this).removeClass(fid);
                        }

                        if (isCalc) {
                            var match = $sel.val();
                            var removeMatches = false;
                            if (match === 'data-any') {
                                // Find where there are any successes, warnings, or failures.
                                var $anySuccess = $cell.find("[data-successes][data-successes!=\"0\"]");
                                var $anyFail = $cell.find("[data-failures][data-failures!=\"0\"]");
                                var $anyWarn = $cell.find("[data-warnings][data-warnings!=\"0\"]");
                                if ($anySuccess.length > 0 || $anyFail.length > 0 || $anyWarn.lengh > 0) {
                                    $(this).removeClass(fid);
                                }
                            } else {
                                if (match === 'data-no-successes') {
                                    // Find where there ARE successes and remove them, reversing the logic.
                                    match = 'data-successes';
                                    removeMatches = true;
                                }
                                var $match = $cell.find("[" + match + "][" + match + '!=\"0\"]');
                                var matches = $match.length;
                                if (matches > 0) {
                                    if (!removeMatches) {
                                        if (match === 'data-successes') {
                                            // lmc - successes are special, meaning that if found, warnings and failures cannot be.
                                            var $failures = $cell.find("[data-failures][data-failures!=\"0\"]");
                                            var $warnings = $cell.find("[data-warnings][data-warnings!=\"0\"]");
                                            if ($failures.length === 0 && $warnings.length === 0) {
                                                $(this).removeClass(fid);
                                            }
                                        } else {
                                            $(this).removeClass(fid);
                                        }
                                    }
                                } else {
                                    if (removeMatches) {
                                        $(this).removeClass(fid);
                                    }
                                }
                            }
                        }
                    });

                    if($table.data("scrollTable")) {
                        $table.lockRows($table.data("widths"));
                    }
                }

                $table.trigger("aftercolumnfilter");
            });

            $table.data("filterTable", true);
        });
    };

})(jQuery);