/*  PHD Virtual Flyout Menu
    Extends/Unstupids jQueryUI Menu widget
*/

(function($) {

    $.widget("phd.flyout", {
        version: "1.10.3",
        defaultElement: "<ul>",
        delay: 300,
        options: {
            links: "a",
            menus: "ul",
            positions: [
                {
                    my: "left top",
                    at: "left bottom"
                },
                {
                    my: "left top",
                    at: "right top"
                }
            ],
            role: "menu",

            // callbacks
            blur: null,
            focus: null,
            select: null
        },

        _create: function() {
            this.activeMenu = this.element;
            // flag used to prevent firing of the click handler
            // as the event bubbles up through nested menus
            this.mouseHandled = false;

            this.element
                .uniqueId()
                .addClass("flyout-menu-root")
                .attr({
                    role: this.options.role
                });

            this._on({
                "click .flyout-menu-item": function(event) {
                    if(!$(event.target).is(".disabled")) {
                        this.toggle( event );   
                    }                    
                }
            });

            this.refresh();

            // Clicks outside of a menu collapse any open menus
            this._on(this.document, {
                click: function(event) {
                    if(!$(event.target).closest(".flyout-menu-root").length) {
                        this.collapseAll(event);
                    }
                }
            });
        },

        refresh: function() {
            var self = this;
            // Initialize nested menus
            this.$menus = this.element.find(this.options.menus).filter(":not(.flyout-menu)")
                .addClass("flyout-menu")
                .attr({
                    role: this.options.role,
                    "aria-hidden": "true",
                    "aria-expanded": "false"
                })
                .each(function() {
                    var $menu = $(this),
                        $link = $menu.prev(self.options.links),
                        depth = $menu.parentsUntil(self.element).filter(self.options.menus).length;

                    $link.attr("aria-haspopup", "true").addClass("flyout-menu-item")
                        .attr("data-depth", depth);
                    $menu.attr("aria-labelledby", $link.attr("id"))
                        .attr("data-depth", depth);
                });
        },

        toggle: function(event) {
            var $link = $(event.target).closest(".flyout-menu-item"),
                $menu = $link.next(".flyout-menu"),
                depth = $link.data("depth"),
                position = this._getPosition(depth);

            // hide any sibling menus
            this.$menus.filter("[data-depth=" + depth + "]").not($menu)
                .removeClass("flyout-menu-expanded");

            position = $.extend({}, position, {"of": $link});
            $menu.addClass("flyout-menu-expanded").position(position);
        },

        collapseAll: function( event, all ) {
            this.$menus.removeClass("flyout-menu-expanded");
        },

        _getPosition: function(depth) {
            if(depth > this.options.positions.length - 1) {
                return this.options.positions[this.options.positions.length - 1];
            }
            return this.options.positions[depth];
        },

        _destroy: function() {
            // Destroy (sub)menus
            this.element
                .removeAttr("aria-activedescendant")
                .removeClass("flyout-menu-root")
                .find(".flyout-menu").addBack()
                    .removeClass("flyout-menu flyout-menu-expanded")
                    .removeAttr("role")
                    .removeAttr("aria-labelledby")
                    .removeAttr("aria-expanded")
                    .removeAttr("aria-hidden")
                    .removeAttr("aria-disabled")
                    .removeUniqueId()
                    .show();

            // Destroy menu items
            this.element.find(".flyout-menu-item")
                .removeClass("flyout-menu-item")
                .removeAttr("role")
                .removeAttr("aria-disabled")
                .removeAttr("aria-haspopup");
        },

    });

})(jQuery);
