/*!
 * jQuery UI Selectmenu @VERSION
 * http://jqueryui.com
 *
 * Copyright 2012 jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Selectmenu
 *
 * Depends:
 *  jquery.ui.core.js
 *  jquery.ui.widget.js
 */
(function( $, undefined ) {

$.widget( "phd.selectmenu", {
    version: "@VERSION",
    defaultElement: "<select>",
    options: {
        appendTo: "body",
        dropdown: true,
        label: null,
        name: "selectmenu",
        preIcon: "",
        postIcon: "",// "icon-caret-down",
        // callbacks
        open: null,
        focus: null,
        select: null,
        close: null,
        change: null
    },

    _create: function() {
        // make / set unique id
        var selectmenuId = this.element.uniqueId().attr( "id" )
            that = this;

        // array of button and menu id's
        this.ids = { id: selectmenuId, container: selectmenuId + "-container", button: selectmenuId + "-button", menu: selectmenuId + "-menu" };

        this.container = $("<div />", {
            id: this.ids.container,
            "class": "ui-select-container ui-" + this.options.name + "-container",
        }).insertAfter(this.element);

        this._drawButton();

        this._on( this.button, this._buttonEvents );
        this._hoverable( this.button );
        this._focusable( this.button );

        this._drawMenu();

        // document click closes menu
        this._on( document, {
            click: function( event ) {
                if ( this.isOpen && !$( event.target ).closest( "li.ui-state-disabled, li.ui-selectmenu-optgroup, #" + this.ids.button ).length ) {
                    this.close( event );
                }
            }
        });

        if ( this.options.disabled ) {
            this.disable();
        }
    },

    _drawButton: function() {
        var tabindex = this.element.attr( "tabindex" ),
            label = this.options.label ? this.options.label : this.label.text();

        // fix existing label
        this.label = $( "label[for='" + this.ids.id + "']" ).attr( "for", this.ids.button ).hide();
        // catch click event of the label
        this._on( this.label, {
            "click":  function( event ) {
                this.button.focus();
                event.preventDefault();
            }
        });

        // hide original select tag
        this.element.hide();

        // create button
        this.button = $( "<a />", {
            "class": "ui-button ui-selectmenu-button ui-widget ui-state-default",
            href: "#" + this.ids.id,
            tabindex: ( tabindex ? tabindex : this.options.disabled ? -1 : 0 ),
            id: this.ids.button,
            title: this.label.text(),
            role: "combobox",
            "aria-expanded": false,
            "aria-autocomplete": "list",
            "aria-owns": this.ids.menu,
            "aria-haspopup": true
        });

        this.buttonText = $( "<span />", {
                "class": "ui-selectmenu-text" ,
                //html: this.element.find( "option:selected" ).text() || "&nbsp;"
                html: label
            })
            .appendTo( this.button );

        if(this.options.preIcon !== "") {
            this.button.append( $( "<i />", {
                "class": this.options.preIcon
            }));
        }

        if(this.options.postIcon !== "") {
            this.button.append( $( "<i />", {
                "class": this.options.postIcon
            }));
        }

        // insert button into container
        this.button.appendTo( this.container );
    },

    _drawMenu: function() {
        var menuInstance,
            that = this;

        // create menu portion, append to body
        this.menu = $( "<ul />", {
            "class": "ui-selectmenu-menu",
            "aria-hidden": true,
            "aria-labelledby": this.ids.button,
            id: this.ids.menu
        }).hide().appendTo( this.container );

    },

    refresh: function() {
        this.menu.empty();

        var item,
            options = this.element.find( "option" );
        if ( options.length ) {
            this._readOptions( options );
            this._renderMenu( this.menu, this.items );

            this.menuItems = this.menu.find( "li" ).not( ".ui-selectmenu-optgroup" );

            // select current item
            item = this._getSelectedItem();
            this._setSelected( item.data( "ui-selectmenu-item" ) );

            // set disabled state
            this._setOption( "disabled", this._getCreateOptions().disabled );
        }
    },

    open: function( event ) {
        if ( !this.options.disabled ) {
            // make sure menu is refreshed on first init (needed at least for IE9)
            if ( this.isOpen === undefined ) {
                this.button.trigger( "focus" );
            }

            this.isOpen = true;
            this._toggleAttr();

            this.menu.zIndex( this.element.zIndex() + 1 ).show();

            this._trigger( "open", event );
        }
    },

    close: function( event ) {
        if ( this.isOpen ) {
            var id = this._getSelectedItem().attr( "id" );
            this.isOpen = false;
            this._toggleAttr();
            this.button.attr( "aria-activedescendant", id );
            this.menu.attr( "aria-activedescendant", id ).hide();
            this._trigger( "close", event );
        }
    },

    _renderMenu: function( ul, items ) {
        var that = this,
            currentOptgroup = "";

        $.each( items, function( index, item ) {
            if ( item.optgroup !== currentOptgroup ) {
                $( "<li />", {
                    "class": "ui-selectmenu-optgroup" + ( item.element.parent( "optgroup" ).attr( "disabled" ) ? " ui-state-disabled" : "" ),
                    html: item.optgroup
                }).appendTo( ul );
                currentOptgroup = item.optgroup;
            }
            that._renderItem( ul, item );
        });
    },

    _renderItem: function( ul, item ) {
        var li = $( "<li />", {
                id: this.ids.menu + item.index,
                html: item.label,
            }).data( "ui-selectmenu-item", item ),
            that = this;

        if ( item.disabled ) {
            li.addClass( "ui-state-disabled" );
        }

        li.on("click", function(event) {
            that._select(item, event);
        });

        return li.appendTo( ul );
    },

    _move: function( direction, event ) {
        if ( direction === "first" || direction === "last" ) {
            // set focus manually for first or last item
            //this.menu.menu( "focus", event, this.menuItems[ direction ]() );
        } else {
            // move to and focus next or prev item
            //this.menu.menu( direction, event );
        }
    },

    _getSelectedItem: function() {
        return this.menuItems.eq( this.element[ 0 ].selectedIndex );
    },

    _toggle: function( event ) {
        if ( this.isOpen ) {
            this.close( event );
        } else {
            this.open( event );
        }
    },

    _buttonEvents: {
        focus: function() {
            // init Menu on first focus
            this.refresh();
            // reset focus class as its removed by ui.widget._setOption
            this.button.addClass( "ui-state-focus" );
            this._off( this.button, "focus" );
        },
        click: function( event ) {
            this._toggle( event );
            event.preventDefault();
        },
        keydown: function( event ) {
            var prevDef = true;
            switch ( event.keyCode ) {
                case $.ui.keyCode.TAB:
                case $.ui.keyCode.ESCAPE:
                    if ( this.isOpen ) {
                        this.close( event );
                    }
                    prevDef = false;
                    break;
                case $.ui.keyCode.ENTER:
                    if ( this.isOpen ) {
                        //this.menu.menu( "select", event );
                    }
                    break;
                case $.ui.keyCode.UP:
                    if ( event.altKey ) {
                        this._toggle( event );
                    } else {
                        this._move( "previous", event );
                    }
                    break;
                case $.ui.keyCode.DOWN:
                    if ( event.altKey ) {
                        this._toggle( event );
                    } else {
                        this._move( "next", event );
                    }
                    break;
                case $.ui.keyCode.SPACE:
                    this._toggle( event );
                    break;
                case $.ui.keyCode.LEFT:
                    this._move( "previous", event );
                    break;
                case $.ui.keyCode.RIGHT:
                    this._move( "next", event );
                    break;
                case $.ui.keyCode.HOME:
                case $.ui.keyCode.PAGE_UP:
                    this._move( "first", event );
                    break;
                case $.ui.keyCode.END:
                case $.ui.keyCode.PAGE_DOWN:
                    this._move( "last", event );
                    break;
                default:
                    this.menu.trigger( event );
                    prevDef = false;
            }
            if ( prevDef ) {
                event.preventDefault();
            }
        }
    },

    select: function(item, event) {
        var oldIndex = this.element[0].selectedIndex;

        // change native select element
        this.element[0].selectedIndex = item.index;
        this._setSelected(item);

        if ( item.index !== oldIndex ) {
            this._trigger( "change", event, { item: item } );
        }

        this._toggle(event);
    },

    _select: function( item, event ) {
        var oldIndex = this.element[0].selectedIndex;

        if(this._trigger("select", event, { item: item })) {
            // change native select element
            this.element[0].selectedIndex = item.index;
            this._setSelected(item);

            if ( item.index !== oldIndex ) {
                this._trigger( "change", event, { item: item } );
            }

            this._toggle(event);
        }
    },

    _setSelected: function( item ) {
        // update button text
        //this.buttonText.html( item.label );
        // change ARIA attr
        this.menuItems.attr( "aria-selected", false );
        this._getSelectedItem().attr( "aria-selected", true );
        this.button.attr( "aria-labelledby", this.menuItems.eq( item.index ).attr( "id" ) );
    },

    _setOption: function( key, value ) {
        this._super( key, value );

        if ( key === "appendTo" ) {
            this.menu.appendTo( $( value || "body", this.element[ 0 ].ownerDocument )[ 0 ] );
        }
        if ( key === "disabled" ) {
            //this.menu.menu( "option", "disabled", value );
            if ( value ) {
                this.element.attr( "disabled", "disabled" );
                this.button.attr( "tabindex", -1 );
                this.close();
            } else {
                this.element.removeAttr( "disabled" );
                this.button.attr( "tabindex", 0 );
            }
        }
    },

    _toggleAttr: function(){
        this.menu.toggleClass( "ui-selectmenu-open", this.isOpen );
        this.menu.attr( "aria-hidden", !this.isOpen);
        this.button.attr( "aria-expanded", this.isOpen);
    },

    _getCreateOptions: function() {
        return { disabled: !!this.element.attr( "disabled" ) };
    },

    _readOptions: function( options ) {
        var data = [];
        $.each( options, function( index, item ) {
            var option = $( item ),
                optgroup = option.parent( "optgroup" );
            data.push({
                element: option,
                index: index,
                value: option.attr( "value" ),
                label: option.text() || "&nbsp;",
                optgroup: optgroup.attr( "label" ) || "",
                disabled: optgroup.attr( "disabled" ) || option.attr( "disabled" )
            });
        });
        this.items = data;
    },

    _destroy: function() {
        this.menu.remove();
        this.button.remove();
        this.element.show();
        this.element.removeUniqueId();
        this.label.attr( "for", this.ids.id );
    },

    widget: function() {
        return this.button;
    },

    menuWidget: function() {
        return this.menu;
    },
});

}( jQuery ));
