/*  PHD Virtual Time Slider
    Extends jQueryUI Slider widget
*/

(function($) {
    $.widget("phd.timeslider", $.ui.slider, {

        options: {
            animate: false,
            distance: 0,
            labels: ["Start", "End"],
            max: 100,
            min: 0,
            orientation: "horizontal",
            range: false,
            step: 1,
            value: 0,
            values: null
        },

        _create: function() {
            var i, handleCount,
                o = this.options,
                existingHandles = this.element.find( ".ui-slider-handle" ).addClass( "ui-state-default" ),
                handle = "<a class='ui-slider-handle ui-state-default' href='#'></a>",
                toggle = "<label><input type='checkbox' checked='checked' id=class='end-time-toggle'>End Time</label>",
                handles = [],
                numPages = 12;

            this._keySliding = false;
            this._mouseSliding = false;
            this._animateOff = true;
            this._handleIndex = null;
            this._detectOrientation();
            this._mouseInit();

            this.element
                .addClass( "ui-slider" + " ui-time-slider" +
                    " ui-slider-" + this.orientation + " ui-time-slider-" + this.orientation +
                    " ui-widget" +
                    ( o.disabled ? " ui-slider-disabled ui-disabled" : "" ) );

            this.range = $([]);

            if ( o.range ) {
                if ( o.range === true ) {
                    if ( !o.values ) {
                        o.values = [ this._valueMin(), this._valueMin() ];
                    }
                    if ( o.values.length && o.values.length !== 2 ) {
                        o.values = [ o.values[0], o.values[0] ];
                    }
                }

                this.range = $( "<div></div>" )
                    .appendTo( this.element )
                    .addClass( "ui-slider-range" +
                    ( ( o.range === "min" || o.range === "max" ) ? " ui-slider-range-" + o.range : "" ) );
            }

            handleCount = ( o.values && o.values.length ) || 1;

            for ( i = existingHandles.length; i < handleCount; i++ ) {
                handles.push( handle );
            }

            this.handles = existingHandles.add( $( handles.join( "" ) ).appendTo( this.element ) );

            this.handle = this.handles.eq( 0 );

            this.handles.add( this.range ).filter( "a" )
                .click(function( event ) {
                    event.preventDefault();
                })
                .mouseenter(function() {
                    if ( !o.disabled ) {
                        $( this ).addClass( "ui-state-hover" );
                    }
                })
                .mouseleave(function() {
                    $( this ).removeClass( "ui-state-hover" );
                })
                .focus(function() {
                    if ( !o.disabled ) {
                        $( ".ui-slider .ui-state-focus" ).removeClass( "ui-state-focus" );
                        $( this ).addClass( "ui-state-focus" );
                    } else {
                        $( this ).blur();
                    }
                })
                .blur(function() {
                    $( this ).removeClass( "ui-state-focus" );
                });

            this.handles.each(function( i ) {
                $( this ).data( "ui-slider-handle-index", i ).addClass( "handle-" + (i+1) );
                if(i === 1) {
                    $( this )
                    .append("<span class='handle-figure'></span>")
                    .append("<span class='handle-label'></span>")
                    .append("<span class='handle-value'></span>");
                } else {
                    $( this )
                    .append("<span class='handle-value'></span>")
                    .append("<span class='handle-label'></span>")
                    .append("<span class='handle-figure'></span>");
                }
            });
            this.handleLabels(o.labels);
            this._getTime();

            this._on( this.handles, {
                keydown: function( event ) {
                    var allowed, curVal, newVal, step,
                        index = $( event.target ).data( "ui-slider-handle-index" );

                    switch ( event.keyCode ) {
                        case $.ui.keyCode.HOME:
                        case $.ui.keyCode.END:
                        case $.ui.keyCode.PAGE_UP:
                        case $.ui.keyCode.PAGE_DOWN:
                        case $.ui.keyCode.UP:
                        case $.ui.keyCode.RIGHT:
                        case $.ui.keyCode.DOWN:
                        case $.ui.keyCode.LEFT:
                            event.preventDefault();
                            if ( !this._keySliding ) {
                                this._keySliding = true;
                                $( event.target ).addClass( "ui-state-active" );
                                allowed = this._start( event, index );
                                if ( allowed === false ) {
                                    return;
                                }
                            }
                            break;
                    }

                    step = this.options.step;
                    if ( this.options.values && this.options.values.length ) {
                        curVal = newVal = this.values( index );
                    } else {
                        curVal = newVal = this.value();
                    }

                    switch ( event.keyCode ) {
                        case $.ui.keyCode.HOME:
                            newVal = this._valueMin();
                            break;
                        case $.ui.keyCode.END:
                            newVal = this._valueMax();
                            break;
                        case $.ui.keyCode.PAGE_UP:
                            newVal = this._trimAlignValue( curVal + ( (this._valueMax() - this._valueMin()) / numPages ) );
                            break;
                        case $.ui.keyCode.PAGE_DOWN:
                            newVal = this._trimAlignValue( curVal - ( (this._valueMax() - this._valueMin()) / numPages ) );
                            break;
                        case $.ui.keyCode.UP:
                        case $.ui.keyCode.RIGHT:
                            if ( curVal === this._valueMax() ) {
                                return;
                            }
                            newVal = this._trimAlignValue( curVal + step );
                            break;
                        case $.ui.keyCode.DOWN:
                        case $.ui.keyCode.LEFT:
                            if ( curVal === this._valueMin() ) {
                                return;
                            }
                            newVal = this._trimAlignValue( curVal - step );
                            break;
                    }

                    this._slide( event, index, newVal );
                },
                keyup: function( event ) {
                    var index = $( event.target ).data( "ui-slider-handle-index" );

                    if ( this._keySliding ) {
                        this._keySliding = false;
                        this._stop( event, index );
                        this._change( event, index );
                        $( event.target ).removeClass( "ui-state-active" );
                    }
                }
            });

            if(o.range === true) {
                if(o.values[1] < o.values[0]) {
                    this.invert = true;
                    this.element.addClass("active-range").removeClass("inactive-range");
                    this.element.parent().addClass("active-range").removeClass("inactive-range");
                    this.range.addClass("inactive-range").removeClass("active-range");
                } else {
                    this.invert = false;
                    this.element.addClass("inactive-range").removeClass("active-range");
                    this.element.parent().addClass("inactive-range").removeClass("active-range");
                    this.range.addClass("active-range").removeClass("inactive-range");
                }
            } else {
                this.element.addClass("active-range").removeClass("inactive-range");
                this.element.parent().addClass("active-range").removeClass("inactive-range");
            }

            this._refreshValue();

            this._animateOff = false;
        },

        _destroy: function() {
            this.handles.remove();
            this.range.remove();

            this.element
                .removeClass( "ui-slider" +
                    " ui-slider-horizontal" +
                    " ui-slider-vertical" +
                    " ui-slider-disabled" +
                    " ui-widget" +
                    " ui-widget-content" +
                    " ui-corner-all" );

            this._mouseDestroy();
        },

        _toggleRange: function(checked) {
            this._setOption("range", checked);
        },

        _mouseCapture: function( event ) {
            var position, normValue, distance, closestHandle, index, allowed, offset, mouseOverHandle,
                that = this,
                o = this.options;

            if ( o.disabled ) {
                return false;
            }

            this.elementSize = {
                width: this.element.outerWidth(),
                height: this.element.outerHeight()
            };
            this.elementOffset = this.element.offset();

            position = { x: event.pageX, y: event.pageY };
            normValue = this._normValueFromMouse( position );
            distance = this._valueMax() - this._valueMin() + 1;
            this.handles.each(function( i ) {
                var thisDistance = Math.abs( normValue - that.values(i) );
                if ( distance > thisDistance ) {
                    distance = thisDistance;
                    closestHandle = $( this );
                    index = i;
                }
            });

            // workaround for bug #3736 (if both handles of a range are at 0,
            // the first is always used as the one with least distance,
            // and moving it is obviously prevented by preventing negative ranges)
            if( o.range === true && this.values(1) === o.min && this.values(2) === o.min ) { // JJ
                index += 1;
                closestHandle = $( this.handles[index] );
            }

            allowed = this._start( event, index );
            if ( allowed === false ) {
                return false;
            }
            this._mouseSliding = true;

            this._handleIndex = index;

            closestHandle
                .addClass( "ui-state-active" )
                .focus();

            offset = closestHandle.offset();
            mouseOverHandle = !$( event.target ).parents().andSelf().is( ".ui-slider-handle" );
            this._clickOffset = mouseOverHandle ? { left: 0, top: 0 } : {
                left: event.pageX - offset.left - ( closestHandle.width() / 2 ),
                top: event.pageY - offset.top -
                    ( closestHandle.height() / 2 ) -
                    ( parseInt( closestHandle.css("borderTopWidth"), 10 ) || 0 ) -
                    ( parseInt( closestHandle.css("borderBottomWidth"), 10 ) || 0) +
                    ( parseInt( closestHandle.css("marginTop"), 10 ) || 0)
            };

            if ( !this.handles.hasClass( "ui-state-hover" ) ) {
                this._slide( event, index, normValue );
            }
            this._animateOff = true;
            return true;
        },

        _slide: function( event, index, newVal ) {
            var otherVal,
                newValues,
                allowed;

            if ( this.options.values && this.options.values.length ) {
                otherVal = this.values( index ? 0 : 1 );

                if ( ( this.options.values.length === 2 && this.options.range === true ) &&
                        ( ( index === 0 && newVal > otherVal) || ( index === 1 && newVal < otherVal ) )
                    ) {
                    this.invert = true;
                    this.element.addClass("active-range").removeClass("inactive-range");
                    this.element.parent().addClass("active-range").removeClass("inactive-range");
                    this.range.addClass("inactive-range").removeClass("active-range");
                } else {
                    this.invert = false;
                    this.element.addClass("inactive-range").removeClass("active-range");
                    this.element.parent().addClass("inactive-range").removeClass("active-range");
                    this.range.addClass("active-range").removeClass("inactive-range");
                }

                if ( newVal !== this.values( index ) ) {
                    newValues = this.values();
                    newValues[ index ] = newVal;
                    // A slide can be canceled by returning false from the slide callback
                    allowed = this._trigger( "slide", event, {
                        handle: this.handles[ index ],
                        value: newVal,
                        values: newValues
                    } );
                    otherVal = this.values( index ? 0 : 1 );
                    if ( allowed !== false ) {
                        this.values( index, newVal, true );
                    }
                }
            } else {
                if ( newVal !== this.value() ) {
                    // A slide can be canceled by returning false from the slide callback
                    allowed = this._trigger( "slide", event, {
                        handle: this.handles[ index ],
                        value: newVal
                    } );
                    if ( allowed !== false ) {
                        this.value( newVal );
                    }
                }
            }
            this._getTime();
        },

        _refreshValue: function() {
            var lastValPercent, valPercent, value, valueMin, valueMax, k,
                oRange = this.options.range,
                o = this.options,
                that = this,
                animate = ( !this._animateOff ) ? o.animate : false,
                _set = {};

            if ( oRange && this.options.values && this.options.values.length ) {
                this.handles.each(function( i ) {
                    var hw = $(this).outerWidth();

                    k = parseInt(i);
                    if(that.invert) {
                        k = i ? i-1 : i + 1;
                    }
                    valPercent = ( that.values(k) - that._valueMin() ) / ( that._valueMax() - that._valueMin() ) * 100;
                    _set[ that.orientation === "horizontal" ? "left" : "bottom" ] = valPercent + "%";
                    _set[ "margin-left"] = "-" + hw/2 + "px";

                    $( that.handles[k] ).stop( 1, 1 )[ animate ? "animate" : "css" ]( _set, o.animate );

                    if ( that.options.range === true ) {
                        if ( that.orientation === "horizontal" ) {
                            if (( k === 1 && that.invert ) || ( k === 0 && !that.invert )) {
                                that.range.stop( 1, 1 )[ animate ? "animate" : "css" ]( { left: valPercent + "%" }, o.animate );
                            }
                            if (( k === 1 && !that.invert ) || ( k === 0 && that.invert )) {
                                that.range[ animate ? "animate" : "css" ]( { width: ( valPercent - lastValPercent ) + "%" }, { queue: false, duration: o.animate } );
                            }
                        } else {
                            if ( i === 0 ) {
                                that.range.stop( 1, 1 )[ animate ? "animate" : "css" ]( { bottom: ( valPercent ) + "%" }, o.animate );
                            }
                            if ( i === 1 ) {
                                that.range[ animate ? "animate" : "css" ]( { height: ( valPercent - lastValPercent ) + "%" }, { queue: false, duration: o.animate } );
                            }
                        }
                    }
                    lastValPercent = valPercent;
                });
            } else {
                value = this.value();
                valueMin = this._valueMin();
                valueMax = this._valueMax();
                valPercent = ( valueMax !== valueMin ) ?
                        ( value - valueMin ) / ( valueMax - valueMin ) * 100 :
                        0;

                var hw = $(this.handle).outerWidth();

                _set[ this.orientation === "horizontal" ? "left" : "bottom" ] = valPercent + "%";
                _set[ "margin-left"] = "-" + hw/2 + "px";
                this.handle.stop( 1, 1 )[ animate ? "animate" : "css" ]( _set, o.animate );

                if ( oRange === "min" && this.orientation === "horizontal" ) {
                    this.range.stop( 1, 1 )[ animate ? "animate" : "css" ]( { width: valPercent + "%" }, o.animate );
                }
                if ( oRange === "max" && this.orientation === "horizontal" ) {
                    this.range[ animate ? "animate" : "css" ]( { width: ( 100 - valPercent ) + "%" }, { queue: false, duration: o.animate } );
                }
                if ( oRange === "min" && this.orientation === "vertical" ) {
                    this.range.stop( 1, 1 )[ animate ? "animate" : "css" ]( { height: valPercent + "%" }, o.animate );
                }
                if ( oRange === "max" && this.orientation === "vertical" ) {
                    this.range[ animate ? "animate" : "css" ]( { height: ( 100 - valPercent ) + "%" }, { queue: false, duration: o.animate } );
                }
            }
        },

        _setOption: function( key, value ) {
            var i,
                valsLength = 0;

            if ( $.isArray( this.options.values ) ) {
                valsLength = this.options.values.length;
            }

            $.Widget.prototype._setOption.apply( this, arguments );

            switch ( key ) {
                case "disabled":
                    if ( value ) {
                        this.handles.filter( ".ui-state-focus" ).blur();
                        this.handles.removeClass( "ui-state-hover" );
                        this.handles.prop( "disabled", true );
                        this.element.addClass( "ui-disabled" );
                    } else {
                        this.handles.prop( "disabled", false );
                        this.element.removeClass( "ui-disabled" );
                    }
                    break;
                case "orientation":
                    this._detectOrientation();
                    this.element
                        .removeClass( "ui-slider-horizontal ui-slider-vertical" )
                        .addClass( "ui-slider-" + this.orientation );
                    this._refreshValue();
                    break;
                case "value":
                    this._animateOff = true;
                    this._refreshValue();
                    this._change( null, 0 );
                    this._animateOff = false;
                    break;
                case "values":
                    this._animateOff = true;
                    this._refreshValue();
                    for ( i = 0; i < valsLength; i += 1 ) {
                        this._change( null, i );
                    }
                    this._animateOff = false;
                    break;
                case "min":
                case "max":
                    this._animateOff = true;
                    this._refreshValue();
                    this._animateOff = false;
                    break;
                case "range":
                    this._animateOff = true;
                    this._destroy();
                    this.options.range = value;
                    if(value === false) {
                        this.value = this.values[0];
                        this.values = [];
                        this.options.values = [];
                    } else {
                        this.values = [this.value, this.options.min];
                    }
                    this._create();
                    console.log(this.options);
                    this._animateOff = false;

            }
        },

        getHandles: function() {
            return this.handles;
        },

        handleLabels: function(labels) {
            this.handles.each(function(i) {
                $(this).find(".handle-label").text(labels[i]);
            });
        },

        handleValues: function(values) {
            this.handles.each(function(i) {
                $(this).find(".handle-value").text(values[i]);
            });
        },

        _getTime: function() {
            var that = this;
            this.handles.each(function(i) {
                var minutes, hours,
                time = null,
                value = that.options.range === true ? that.values()[i] : that.value();

                hours = Math.floor(value / 60);
                minutes = value - (hours * 60);

                if (hours == 24) {
                    hours = 12;
                    time = "AM";
                } else if (hours >= 12) {
                    hours -= 12;
                    time = "PM";
                } else {
                    time = "AM";
                }

                if (hours === 0) {
                    hours = 12;
                }

                minutes = minutes < 10 ? "0" + minutes : minutes;

                $(this).find(".handle-value").text(hours + ":" + minutes + " " + time);
            });
        }
    
    });
})(jQuery);