(function($) {

    function isValidDate(value, element) {
        var MONTH_INDEX = 1;
        var DAY_INDEX = 2;
        var YEAR_INDEX = 3;

        var ret = false;
        var result;
        try {
            switch (DATE_FORMAT) {
                case 'dd.mm.yy':
                    // dd.mm.yyyy
                    DAY_INDEX = 1;
                    MONTH_INDEX = 2;
                    result = /^(\d{1,2})[\.\-](\d{1,2})[\.\-](\d{4})$/i.exec(value);
                    break;
                case 'dd/mm/yy':
                    // dd/mm/yyyy
                    DAY_INDEX = 1;
                    MONTH_INDEX = 2;
                    result = /^(\d{1,2})[\/\-](\d{1,2})[\/\-](\d{4})$/i.exec(value);
                    break;
                case 'yy-mm-dd':
                    // yyyy-mm-dd
                    YEAR_INDEX = 1;
                    MONTH_INDEX = 2;
                    DAY_INDEX = 3;
                    result = /^(\d{4})[\-\-](\d{1,2})[\-\-](\d{1,2})$/i.exec(value);
                    break;
                default:
                    // mm/dd/yyyy
                    result = /^(\d{1,2})[\/\-](\d{1,2})[\/\-](\d{4})$/i.exec(value);
                    break;
            }
        } catch (e) {
            // mm/dd/yyyy
            result = /^(\d{1,2})[\/\-](\d{1,2})[\/\-](\d{4})$/i.exec(value);
        }
        if (result && result.length == 4) {
            result[MONTH_INDEX] = parseInt(result[MONTH_INDEX]) - 1
            var dt = new Date(result[YEAR_INDEX], result[MONTH_INDEX], result[DAY_INDEX]);
            var month = dt.getMonth();
            if (!isNaN(month)) {
                var day = dt.getDate();
                var year = dt.getFullYear();
                if(result[MONTH_INDEX] == month && result[DAY_INDEX] == day && result[YEAR_INDEX] == year) {
                    // I return the Date object if it is valid.
                    // So isValidDate() can also be used to get back a date object.
                    ret = dt;
                }
            }
        }
        return ret;
    };

    window.isValidDate = isValidDate;

    $.validator.prototype.subset = function($container) {
        var validator = this,
            valid = true;
        $container.find(':input:visible').each(function() {
            if (!validator.element(this))
                valid = false;
        });
        return valid;
    };

    $.validator.addMethod("dateFmt", isValidDate, "Please enter a valid date in the format " + DATE_DISPLAY_FORMAT);

    $.validator.addMethod("dateFmtNotRequired", function(value, element) {
        value = $.trim(value);
        return (value ? isValidDate(value) : true);
    }, "Please enter a valid date in the format " + DATE_DISPLAY_FORMAT);

    $.validator.addMethod("validate-start-date", function(value, element) {
        var start = value, 
            end = $("#id_end_date");

        start = isValidDate(start);
        end = isValidDate(end.val());
        if (start && end) {
            return (end >= start);
        } else {
            return true;
        }
    }, ERR_START_DATE_GT_END_DATE);

    $.validator.addMethod("validate-treeview", function(value, element) {
            return $("input[name=inventory]").length > 0;
        },
        ERR_INVENTORY
    );

    $.validator.addMethod("validate-checklist", function(value, element) {
            return $(element).prevAll(".checklist").find(":checked").length > 0;
        },
        ERR_WEEKLY_RUN_DAY
    );

    $.validator.addMethod("validate-checklist-full", function(value, element) {
            return $(element).prevAll(".checklist").find(":checked").length > 0;
        },
        gettext("Select at least one day for the full backup to run")
    );

    $.validator.addMethod("time12h", function(value, element) {
            return this.optional(element) || /^((0?[1-9]|1[012])(:[0-5]\d){1,2}(\ ?[AP]M))$/i.test(value);
        }, 
        gettext("Please enter a valid time")
    );

    $.validator.addMethod("validate-mac-address", function(value, element) {
            var macregex = /^([a-zA-Z0-9]{2}\:){5}[a-zA-Z0-9]{2}$/;
            return value.test(macregex);
        },
        gettext("Invalid Mac Address")
    );

    $.validator.setDefaults({
        errorLabelContainer: "#form-error-list",
        errorContainer: "#form-errors",
        wrapper: "li",
        showErrors: function(errorMap, errorList) {
            $("#form-errors").find(".summary")
                .html(gettext("Correct the following errors:"));
            this.defaultShowErrors();
        },
        highlight: function(element, errorClass, validClass) {
            var $el = $(element),
                $container = $el.closest(".backup-catalog, .control-group, .control-row, .modal-form-box");
            $container.addClass(errorClass).removeClass(validClass);
        },
        unhighlight: function(element, errorClass, validClass) {
            var $el = $(element),
                $container = $el.closest(".backup-catalog, .control-group, .control-row, .modal-form-box");
            $container.addClass(validClass).removeClass(errorClass);
        }
    });

})(jQuery);